#!/bin/bash

expectedArgs=1
badArgsCode=1

if [ $# -lt $expectedArgs ]
then
	echo "Usage: deploy.sh <version>"
	echo "   Version can be major.minor.revision or major.minor.revision-suffix with a dedicated suffix"
	exit $badArgsCode
fi

version=$1
shortVersion=${version%%-*}

echo "Short version is $shortVersion"

file=CMakeLists.txt
tmpFile=tmp.txt

# Replace the short and long versions.
sed "s/set(LONG_NOMAD_VERSION [0-9]*.[0-9]*.[0-9]*[-a-zA-Z]*/set(LONG_NOMAD_VERSION $version/g" $file > $tmpFile
mv $tmpFile $file

sed "s/set(SHORT_NOMAD_VERSION [0-9]*.[0-9]*.[0-9]*/set(SHORT_NOMAD_VERSION $shortVersion/g" $file > $tmpFile
mv $tmpFile $file

# Build to replace nomad.version.
./build.sh

# Commit and push.
git add CMakeLists.txt
git commit -m "Version $version"
git push

# Tag the project with the version.
git tag -a $version -m "Version $version"
git push origin "$version"


