#!/bin/bash

initProperties() {
	
	export PATH=$PATH:/bin:/usr/bin
	
	if [ -z "$NOMAD_SETTINGS_CONFIG" ]
	then	
		settingsFile="$HOME/.nomad/Settings.properties"
	else
		settingsFile="$NOMAD_SETTINGS_CONFIG"
	fi

	if [ -f $settingsFile ];
	then
		echo "File $settingsFile exists."

		# Source the properties.
		. $settingsFile

		if [ ! "$nomadHome" = "" ]
		then
			NOMAD_HOME=$nomadHome
		fi
		if [ ! "$nomadNotifications" = "" ]
		then
			NOMAD_NOTIFICATIONS=$nomadNotifications
		fi
		if [ ! "$nomadUserNotifications" = "" ]
		then
			NOMAD_USER_NOTIFICATIONS=$nomadUserNotifications
		fi
		if [ ! "$nomadMail" = "" ]
		then
			NOMAD_MAIL=$nomadMail
		fi
	else
		echo "File $settingsFile does not exist."
	fi

	# Assign the properties.
	if [ "$NOMAD_HOME" = "" ]
	then
		NOMAD_HOME=$HOME/nomad/NomadServer
	fi
	if [ "$NOMAD_NOTIFICATIONS" = "" ]
	then
		NOMAD_NOTIFICATIONS=1
	fi
	if [ "$NOMAD_USER_NOTIFICATIONS" = "" ]
	then
		NOMAD_USER_NOTIFICATIONS=1
	fi
	if [ "$NOMAD_MAIL" = "" ]
	then
		NOMAD_MAIL="nomad_l@ill.fr"
	fi

	echo "NOMAD_HOME=$NOMAD_HOME"
	echo "NOMAD_NOTIFICATIONS=$NOMAD_NOTIFICATIONS"
	echo "NOMAD_USER_NOTIFICATIONS=$NOMAD_USER_NOTIFICATIONS"
}

dumpCore() {
	
	TEMP_DUMPCOREFILE="$HOME/log/tmpCore.txt"
	TEMP_DUMPCOREFILTEREDFILE="$HOME/log/tmpCoreFiltered.txt"
	
	gdb $NOMAD_HOME/bin/nomad -c $1 -x $NOMAD_HOME/gdbdump -batch > $TEMP_DUMPCOREFILE
	
	delimiter="Core was generated"
	dump="false"
	
	# iterate the lines of the file
	while read line
	do
		if [ "$dump" = "true" ]
		then
			echo $line >> $TEMP_DUMPCOREFILTEREDFILE
		fi
	
		coreLine=${line:0:18}
			
		if [ "$coreLine" = "$delimiter" ]
		then
			dump="true"
		fi
		
	done < $TEMP_DUMPCOREFILE
}

sendmail() {
	
	TEMP_FILE="$HOME/log/tmp.txt"
	core=`ls -t /home/cameo/core.* | head -n1`
	date >> $TEMP_FILE
		
	# testing date of the core file
	coreDate=`stat -c %Y $core`
	nowDate=`date +%s`
	
	# core file is valid only if the date of its last modification is close to now
	delay=`expr $nowDate - $coreDate`
	
	# core file is valid only if the date of its last modification is close to now
	# delay must not exceed 30s, otherwise do not send any mail
	if [ $delay -lt 30 ]
	then
		echo "Core : $core" >> $TEMP_FILE
		
		dumpCore $core
		cat $TEMP_DUMPCOREFILTEREDFILE >> $TEMP_FILE
		
		rm $TEMP_DUMPCOREFILE
		rm $TEMP_DUMPCOREFILTEREDFILE
		
		# possible states STARTING, RUNNING, STOPPING, KILLING
		
		if [ "$2" = "STARTING" ]
		then
			state="init"
		elif [ "$2" = "RUNNING" ]
		then
			state="exec"
		else
			state="exit"
		fi
				
		mail -s "[$HOSTNAME][server][$state] $1" -r nomadmail@ill.fr $NOMAD_MAIL < $TEMP_FILE
				
		# send to local contact mailing list
		MAILING_LIST_FILE="$HOME/.nomad/mailingList.txt"
		
		if [ -e $MAILING_LIST_FILE ] && [ ! "$NOMAD_NOTIFICATIONS" = "0" ]
		then
			if [ "$state" = "init" ] || [ "$state" = "exec" ]
			then
				addresses=`cat $MAILING_LIST_FILE`
				TEMP_FILE2="$HOME/log/tmp2.txt"
				echo " " >> $TEMP_FILE2
				
				if [ ! "$NOMAD_USER_NOTIFICATIONS" = "0" ]
				then
					# send mail to adresses
					mail -s "[$HOSTNAME] Nomad server crash" -r nomadmail@ill.fr $addresses < $TEMP_FILE2
				fi
				
				rm $TEMP_FILE2
			fi
		fi
	else
		echo "Core delay not ok: $coreDate $nowDate $delay" >> $TEMP_FILE
		echo "End of check" >> $TEMP_FILE
	fi
	
	rm $TEMP_FILE
}

initProperties

# error code is $2
sendmail "crash $2" "$3"

