/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectApplication.h"
#include <common/base/Exception.h>
#include <cameo/api/cameo.h>

using namespace std;
using namespace common;

DirectApplication * application = 0;

void shutdownHandler() {

	if (application->isStarted()) {
		cout << endl;
		application->quit(std::vector<std::string>(), false);

	} else if (application->isStopping()) {

		cout << endl << "Application forced to quit" << endl;

		application->forceQuit();
		exit(EXIT_FAILURE);

	} else {
		exit(EXIT_FAILURE);
	}
}

int main(int argc, char* argv[]) {

	cameo::This::init(argc, argv);

	cameo::This::handleStop([] {
		shutdownHandler();
	});

	application = new DirectApplication();

	try {
		application->init(argc, argv);
		application->run();
		application->reset();

	} catch (const Exception & e) {
		cout << "Error : " << e.printStack() << endl;

		return EXIT_FAILURE;
	}

	return application->getReturnCode();
}
