/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "common/base/ServerProperties.h"
#include "ics/Module/ModuleManager.h"
#include "ics/DataProvider/Property/PropertyInfos.h"
#include <iostream>
#include <cstdlib>
#include <sstream>

using namespace std;
using namespace common;

int main(int argc, char* argv[]) {

	ServerProperties::initInstance("");
	ModuleManager::initInstance();

	try {
		ModuleManager::getInstance()->loadModulesFromSequence(ServerProperties::getInstance()->getNomadHomePath() + "/mod");
	} catch (const Exception & e) {
		cout << "Unable to find the modules path." << endl;
	}

	// select the module to test
	string moduleName;

	// test the number of arguments
	if (argc > 1) {
		moduleName = argv[1];
	} else {
		cout << "Enter a module name:";
		getline(cin, moduleName);
	}

	cout << "Testing " << moduleName << endl;

	// test existence of module
	bool success = ModuleManager::getInstance()->testModule(moduleName);

	if (success) {
		cout << "Success." << endl;
	} else {
		cout << "Failure." << endl;
	}

	ModuleManager::resetInstance();

	return EXIT_SUCCESS;
}
