/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */
#include <string>
#include <iostream>
#include <DirectApplication.h>
#include <cameo/api/cameo.h>
#include "common/base/ServerProperties.h"
#include <dataprovider/transportlayer/MultiThreadCommunication.h>
#include <command/transportlayer/MultiThreadProcessorCommunication.h>
#include <command/transportlayer/dataprovider/PropertyAccessor.h>
#include <command/base/ServantManager.h>
#include <dataprovider/transportlayer/dataprovider/ServantAccessorDirectImpl.h>
#include "ics/Module/ModuleManager.h"
#include "ics/InstrumentAbstraction/ExperimentController.h"
#include "ics/InstrumentAbstraction/ExperimentControllerCreator.h"
#include "ics/PersistenceServices/Configuration/ExperimentControllerConfig.h"
#include "ics/InstrumentManager/InstrumentManager.h"
#include "ics/InstrumentManager/HardwareManager.h"
#include <common/utils/PropertyDescriptor.h>
#include "ics/Utilities/StringUtilities.h"

using namespace std;
using namespace common;
using namespace dataprovider;

string _logpath;

bool _ok = false;
DirectApplication* _application = NULL;

struct property_info {
	ExperimentController* controller;
	string controller_type;
	string name;
	string alias;
	bool setpoint;
	bool logged;
};

/*
 * startApplication
 */
int startApplication() {
	try {
		char const* args[] = { "-ics", "test" };
		_application->init(2, (char **) args);
		_application->run();
		return EXIT_SUCCESS;
	} catch (const Exception & e) {
		cout << "Error : " << e.printStack() << endl;
		return EXIT_FAILURE;
	}
}

/*
 * check
 */
void check(map<string, vector<property_info> >& properties_map) {
	map<string, vector<property_info> >::iterator iter1 = properties_map.begin();
	while (iter1 != properties_map.end()) {
		ostringstream propstream;
		string type1 = iter1->first;
		vector<property_info> pvect1 = iter1->second;
		vector<property_info>::iterator piter1 = pvect1.begin();
		while (piter1 != pvect1.end()) {
			property_info pinfo1 = *piter1;
			// Check setpoiint
			if ((pinfo1.setpoint == true) && (pinfo1.alias.empty() == true)) {
				propstream << pinfo1.name << " : setpoint property alias is empty" << endl;
			}
			// check size
			if ((pinfo1.setpoint == true) && (pinfo1.alias.size() > 10)) {
				propstream << pinfo1.name << " : has size " << pinfo1.alias.size() << " greater than 10" << endl;
			}
//			if (pinfo1.alias.empty() == true) {
//				propstream << pinfo1.name << " : property alias is empty " << endl;
//			}
//			else {
//				map<string, vector<property_info> >::iterator iter2 = iter1;
//				++iter2;
//				while (iter2 != properties_map.end()) {
//					string type2 = iter2->first;
//					vector<property_info> pvect2 = iter2->second;
//					vector<property_info>::iterator piter2 = pvect2.begin();
//					// Check duplicate name
//					while (piter2 != pvect2.end()) {
//						property_info pinfo2 = *piter2;
//						if (StringUtilities::cmp(pinfo1.alias, pinfo2.alias,pinfo1.alias.size()) == true) {
//							// check inherance
//	//						if (typeid(pinfo1.controller) != typeid(pinfo2.controller)) {
//								propstream << pinfo1.name << " : has dupplicate name in controller type " << pinfo2.controller_type << " with property " << pinfo2.name << endl;
//	//						}
//						}
//						++piter2;
//					}
//					++iter2;
//				}
//			}
			++piter1;
		}
		if (propstream.str().empty() == false) {
			ostringstream outputfilename;
			outputfilename << _logpath << "/" << type1 << "_propertyaliascheck.log";
			ofstream outputfile(outputfilename.str().c_str());
			if (outputfile.is_open() == true) {
				outputfile << propstream.str();
				outputfile.close();
			}
		}
		++iter1;
	}
}
//
//}
//int check(map<string, vector<common::PropertyDescriptor> >& properties_map) {
//
//	map<string, vector<common::PropertyDescriptor> >::iterator iter1 = properties_map.begin();
//	while(iter1 != properties_map.end()) {
//		string type1 = iter1->first;
//		vector<common::PropertyDescriptor>& pvect1 = iter1->second;
//		map<string, vector<common::PropertyDescriptor> >::iterator iter2 = iter1;
//		while(iter2 != properties_map.end()) {
//			string type2 = iter2->first;
//			vector<common::PropertyDescriptor>& pvect2 = iter2->second;
//			vector<common::PropertyDescriptor>::iterator piter1 = pvect1.begin();
//			while(piter1 != pvect1.end()) {
//				vector<common::PropertyDescriptor>::iterator piter2 = pvect2.begin();
//				common::PropertyDescriptor& pdesc1 = *piter1;
//				if (pdesc1.)
//				while(piter2 != pvect1.end()) {
//					common::PropertyDescriptor& pdesc2 = *piter2;
//					++piter2;
//				}
//				++piter1;
//			}
//			++iter2;
//		}
//		++iter1;
//	}
//	return EXIT_SUCCESS;
//}

int main(int argc, char* argv[]) {
	ostringstream outputfilepath;
	if (argc == 1) {
		outputfilepath << getenv("HOME") << "/serverlog";
	}
	else if (argc == 2) {
		outputfilepath << argv[1];
	}
	_logpath = outputfilepath.str();

	// Cameo init
//	char const * args[] = { "tcp://localhost:7000:propertyaliascheck" };
	cameo::This::init("propertyaliascheck", "tcp://localhost:7000");

	// Launch thread for Direct Application
	_application = new DirectApplication();
	std::thread td(&startApplication);
	while (_application->isStarted() == false) {
		usleep(10000);
	}

	// Start some instance
	ModuleManager::initInstance();

	// Load Modules from mod file
	try {
		ModuleManager::getInstance()->loadModulesFromSequence(ServerProperties::getInstance()->getNomadHomePath() + "/mod");
	} catch (const Exception & e) {
		cout << "Unable to find modules path" << endl;
		return EXIT_FAILURE;
	}

	map<string, vector<property_info> > properties_map;

	/*
	 *  Create all controllers
	 */
	vector<string> types = ModuleManager::getInstance()->getAllControllerTypes();
	vector<string>::iterator iter = types.begin();
	map<uint32, string> controllerIDs;
	uint32 id = 0;
	while (iter != types.end()) {
		string type = *iter;
		ostringstream name;
		name << "test-" << id++;
		uint32 controllerID = MultiThreadCommunication::getInstance()->getServantAccessor()->createController(0, name.str(),
				type, "hidden", false, common::HashMap<std::string, std::pair<std::string , std::string>>());
		controllerIDs[controllerID] = name.str();
		ostringstream propstream;
		try {
			InstrumentManager::getInstance()->createController(type, name.str(), "hidden", "", false,
					std::map<std::string,  std::pair<std::string , std::string>>());
			try {
				ExperimentController* cont = InstrumentManager::getInstance()->getExperimentController(name.str());
				command::ServantManager::getInstance()->createController(type, name.str(), controllerID);
				MultiThreadCommunication::getInstance()->getServantAccessor()->setServantInitialised(0, controllerID, true);
				Database::PropertyMap propertyMap = Database::getInstance(0)->getControllerProperties(controllerID);
				vector<property_info> pinfo;
				for (Database::PropertyMap::const_iterator it = propertyMap.begin(); it != propertyMap.end(); it++) {
					string pname = it->first;
					Database::PropertyItem item = it->second;
					bool isDynamic = (item.itemType == Database::PropertyItem::DYNAMIC);
					if (isDynamic == false) {
						property_info info;
						info.controller = cont;
						info.controller_type = type;
						info.name = pname;
						bool setpoint = false;
						Database::PropertyInfoItem infoItem;
						try {
							infoItem = Database::getInstance(0)->getSetpointPropertyInfoItem(type, pname);
							setpoint = true;
						} catch (const Database::NoPropertyInfo& e) {
							try {
								infoItem = Database::getInstance(0)->getNoSetpointPropertyInfoItem(type, pname);
							} catch (const Database::NoPropertyInfo& e) {
								// do nothing
							}
						}
						info.setpoint = setpoint;
						info.alias = infoItem.aliasPropertyName;
						pinfo.push_back(info);
					}
				}
				if (pinfo.empty() == false) {
					properties_map[type] = pinfo;
				}
			} catch (InstrumentManager::InstrumentManagerError const & e) {
				cerr << "ControllerNotFound" << endl;
			}
		} catch (InstrumentManager::InstrumentManagerError const & e) {
			cerr << "ControllerCreationException" << endl;
		}
		++iter;
	}

//  For Christina
//	/*
//	 *  Create all drivers
//	 */
//	types = ModuleManager::getInstance()->getAllDriverTypes();
//	iter = types.begin();
//	map<uint32, string> driverIDs;
//	id = 0;
//	while (iter != types.end()) {
//		string type = *iter;
//		ostringstream name;
//		name << type << "-" << id++;
//		uint32 driverID = MultiThreadCommunication::getInstance()->getServantAccessor()->createDriver(0,
//				name.str(), type, "perfect", "", 0, "", 0);
//		driverIDs[driverID] = name.str();
//		try {
//			HardwareManager::getInstance()->createDriver("", type, name.str(), "perfect", id);
//			try {
//				DeviceDriver* driver = HardwareManager::getInstance()->getDriver(name.str());
//				command::ServantManager::getInstance()->createDriver(type, name.str(), driverID);
//				MultiThreadCommunication::getInstance()->getServantAccessor()->setServantInitialised(0,
//						driverID, true);
//				cout << driver->getProperties() << endl;
//			} catch (HardwareManager::ControllerNotFound const & e) {
//				cerr << "ControllerNotFound" << endl;
//			}
//		} catch (HardwareManager::DriverCreationException const & e) {
//			cerr << "DriverCreationException" << endl;
//		}
//		++iter;
//	}

	/*
	 * Checking all alias
	 */
	check(properties_map);

//	/*
//	 *  Remove all drivers
//	 */
//	map<uint32, string>::iterator itermap = driverIDs.begin();
//	while (itermap != driverIDs.end()) {
//		uint32 driverID = itermap->first;
//		string name = itermap->second;
//		cout << "delete driver " << name << " - " << endl;
//		MultiThreadCommunication::getInstance()->getServantAccessor()->setServantInitialised(0, driverID,
//				false);
//		command::ServantManager::getInstance()->removeDriver(driverID);
//		HardwareManager::getInstance()->removeDriver(name);
//		MultiThreadCommunication::getInstance()->getServantAccessor()->removeServant(0, driverID);
//		++itermap;
//	}
	/*
	 *  Remove all controllers
	 */
	map<uint32, string>::iterator itermap = controllerIDs.begin();
	while (itermap != controllerIDs.end()) {
		uint32 controllerID = itermap->first;
		string name = itermap->second;
//		cout << "delete controller " << name << " - " << controllerID << endl;
		MultiThreadCommunication::getInstance()->getServantAccessor()->setServantInitialised(0, controllerID, false);
		command::ServantManager::getInstance()->removeController(controllerID);
		InstrumentManager::getInstance()->removeController(name); // Cause segmentation fault
		MultiThreadCommunication::getInstance()->getServantAccessor()->removeServant(0, controllerID);
		++itermap;
	}

	// Quit application
	_application->quit(std::vector<std::string>(), false);
	_application->reset();

	return EXIT_SUCCESS;
}
