/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CommandProcessorMain.h"
#include <command/application/CommandProcessor.h>
#include <common/Console.h>
#include <signal.h>
#include <iostream>

using namespace std;
using namespace common;
using namespace command;

CommandProcessor* commandProcessor = 0;

void shutdownHandler(int sig) {
	
	// only quiting the ProcessorManager application
	if (commandProcessor->getConsole() != 0) {
		commandProcessor->shutdown(false);
	}	
}

int main(int argc, char* argv[]) {
	
	signal(SIGINT, shutdownHandler);

	try {
		// Create the CommandProcessor (in its ProcessorManager form)
		commandProcessor = new CommandProcessor();
		commandProcessor->init(argc, argv);

		// Blocks the application until a shutdown.
		commandProcessor->run();
		commandProcessor->reset();
		
	} catch (const Exception & e) {
		cerr << e.printStack() << endl;
		
		return -1;
	}

	if (commandProcessor->onError()) {
		return -1;
	}

	return 0;
}