/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_DIRECTCOMMANDPROCESSOR_H_
#define COMMAND_DIRECTCOMMANDPROCESSOR_H_

#include "ProcessorManagerInterface.h"
#include <common/Application.h>
#include <common/base/Exception.h>
#include <thread>
#include <string> 

class DirectApplication;

namespace common {
	
	class Console;
}

namespace command {

/**
 * Implements the Application interface.
 */
class DirectCommandProcessor : public common::Application, public ProcessorManagerInterface {

public :
	/**
	 * Constructor.
	 */
	DirectCommandProcessor(DirectApplication * application);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DirectCommandProcessor();

	/**
	 * Initialises the principal agents of the Command Processor : Communication
	 * Manager, Processor, Servant Manager.
	 * throw ForkCreationException
	 */
	virtual void init(int argc, char* argv[]);
	
	/**
	 * Runs the command processor.
	 */
	virtual void run();
	
	/**
	 * Resets and cleans the command processor.
	 */
	virtual void reset();

	/**
	 * Sets the manager started.
	 */
	virtual void setStarted();

	/**
	 * Sets the state on error.
	 */
	virtual void setOnError();

	/**
	 * Returns true if the application is on error.
	 */
	bool onError() const;
	
	/**
	 * Shuts down the processor manager and all other processes.
	 */
	virtual void shutdown(bool isRemote);
	
	/**
	 * Shows the state of the processes.
	 */
	void show();
	
	/**
	 * Shows the current line of the processor.
	 */
	void where(unsigned long processorID);
	
	/**
	 * Unloads the modules that can be unloaded.
	 */
	void unloadUnusedModules();

	/**
	 * Executes a command coming from the Console object.
	 */
	virtual void execute(const std::string& commandName, const std::vector<std::string>& args);

	/**
 	 * Quits the application.
 	 */
	virtual void quit(const std::vector<std::string>& args, bool isRemote);	

private:
	static void loop();

	DirectApplication * _application;
	bool _onError;

	std::thread* _thread;

	static const std::string ICS;
	static const std::string SIMULATION;
	static const std::string SIMULATION3D;
	static const std::string DEBUG_ON;
};

}

#endif /*COMMAND_DIRECTCOMMANDPROCESSOR_H_*/
