/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ProcessorThread.h"
#include <command/processor/Processor.h>
#include <command/base/ServantManager.h>
#include <command/transportlayer/MultiThreadProcessorCommunication.h>
#include <command/processor/ThreadProcessorInfo.h>
#include <common/output/OutputStream.h>
#include <signal.h>
#include <iostream>
#include <fstream>
#include <functional>

using namespace std;
using namespace common;

namespace command {

ProcessorThread::ProcessorThread(unsigned long processorID) :
	_processorID(processorID),
	_thread(nullptr) {
}

ProcessorThread::~ProcessorThread() {
	delete _thread;
}

void ProcessorThread::start() {
	_thread = new std::thread(std::bind(&ProcessorThread::loop, _processorID));
}

void ProcessorThread::loop(unsigned long processorID) {

	ScopedThreadProcessorInfo processorInfo(processorID);

	// initialise the processor
	Processor::initInstance(processorID, MultiThreadProcessorCommunication::getInstance()->getProcessorManagerProxy(), MultiThreadProcessorCommunication::getInstance()->getServantAccessor(), MultiThreadProcessorCommunication::getInstance()->getCommandAccessor());

	// create proxy
	MultiThreadProcessorCommunication::getInstance()->createProcessorProxy(processorID);

	// run the Processor - blocking.
	Processor::getInstance()->run();

	// reset the instance
	Processor::resetInstance();
}

void ProcessorThread::join() {

	_thread->join();
}

unsigned long ProcessorThread::getProcessorID() const {
	return _processorID;
}

}
