/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_DATAHOLDER_H_
#define COMMAND_DATAHOLDER_H_

#include <common/base/HashMap.h>

namespace command {

class BaseCommand;

/**
 * Provides the base class for Controller, Driver and Service. Is used to store all Properties,
 * Commands, Property References, Property Handlers so that a synchronisation with the Data Provider
 * can be made. Methods and members are prefixed with an underscore to avoid name clash with N members.
 */
class DataHolder {

public:
	/**
	 * Servant type enum.
	 */
	typedef enum {DRIVER, CONTROLLER} Type;

	/**
	 * Constructor.
	 */
	DataHolder();

	/**
	 * Virtual destructor.
	 */
	virtual ~DataHolder();

	/**
	 * Gets the entity type (DRIVER, CONTROLLER or PROTOCOL).
	 */
	virtual Type _getServantType() const = 0;

	/**
	 * Gets the type. Reimplemented in Servant.
	 */
	virtual std::string const & _getType() const = 0;

	/**
	 * Adds a command to the command map.
	 */
	void _addCommand(BaseCommand* command);

protected:
	const common::HashMap<std::string, BaseCommand*>& _commands() const;

private:
	common::HashMap<std::string, BaseCommand*> __commands;
};

}

#endif /*COMMAND_DATAHOLDER_H_*/
