/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Servant.h"
#include <command/transportlayer/dataprovider/CommandAccessor.h>
#include <command/log/LogHandler.h>
#include <common/base/BaseTypeToString.h>
#include <common/base/ServerProperties.h>
#include <common/base/Convert.h>
#include <common/output/OutputStream.h>
#include <iostream>

using namespace std;
using namespace common;

namespace command {

Servant::NoSuchCommandException::NoSuchCommandException(unsigned long ID) :
	TraceableException("command::Servant::NoSuchCommandException", string("unable to find command with ID <") + to<string>(ID) + ">\n") {
}

Servant::Servant(unsigned long ID, std::string const & name) :
	__ID(ID),
	__name(name),
	__commandAccessor(nullptr) {
}

Servant::~Servant() {
}

unsigned long Servant::_getID() const {

	return __ID;
}

std::string Servant::_getName() const {

	return __name;
}

void Servant::_initAccessors(CommandAccessor* commandAccessor) {

	__commandAccessor = commandAccessor;
}

void Servant::_synchronise(CommandAccessor* commandAccessor) {

	_initAccessors(commandAccessor);

	// Only commands are synchronised in the nomad server
	// Command synchronisation
	const vector<CommandDescriptor> & commandDescriptors = _getCommandDescriptors();
	const HashMap<string, BaseCommand*>& commands = DataHolder::_commands();

	// verify that all data provider commands are declared
	vector<CommandDescriptor>::const_iterator cd;
	for (cd = commandDescriptors.begin(); cd != commandDescriptors.end(); cd++) {
		const CommandDescriptor& descriptor = *cd;

		if (!commands.find(descriptor.getName())) {
			// error checked later

		} else {

			// found the command
			BaseCommand* command = commands.get(descriptor.getName());
			command->init(descriptor.getID(), commandAccessor);

			// add in the command map
			__commands.insert(descriptor.getID(), command);
		}
	}
}

BaseCommand* Servant::_getCommand(unsigned long commandID) const {

	try {
		return __commands.get(commandID);

	} catch (Exception& e) {
		throw NoSuchCommandException(commandID);
	}

}

CommandAccessor* Servant::_commandAccessor() const {

	return __commandAccessor;
}

std::ostream& operator<< (std::ostream& os, Servant const & servant) {

	return os << servant._getName() << " (" << servant._getType() << ")";
}

}
