/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_SERVANT_H_
#define COMMAND_SERVANT_H_

#include "DataHolder.h"
#include <common/base/CommonTypes.h>
#include <common/base/HashMap.h>
#include <command/base/method/BaseCommand.h>
#include <command/exception/TraceableException.h>
#include <list>
#include <set>
#include <string>
#include <common/utils/CommandDescriptor.h>

namespace command {

class PropertyAccessor;
class CommandAccessor;
class ServantAccessor;

class Method;

/**
 * Base class for the servant hierarchy.
 */
class Servant : virtual public DataHolder {

public:
	/**
	 * Exception for no such command.
	 */
	class NoSuchCommandException : public TraceableException {

	public:
		/**
		 * Constructor.
		 */
		NoSuchCommandException(unsigned long ID);
	};

	/**
	 * Constructor with the ID and the name of the servant.
	 */
	Servant(unsigned long ID, std::string const & name);

	/**
	 * Virtual destructor.
	 */
	virtual ~Servant();

	/**
	 * Gets the ID.
	 */
	unsigned long _getID() const;

	/**
	 * Gets the name.
	 */
	std::string _getName() const;

	/**
	 * Synchronises the servant with the database.
	 */
	virtual void _synchronise(CommandAccessor* commandAccessor);

	/**
	 * throw NoSuchCommandException
	 */
	BaseCommand* _getCommand(unsigned long commandID) const;

protected:
	virtual std::vector<common::CommandDescriptor> _getCommandDescriptors() const = 0;

	/**
	 * Returns the command accessor.
	 */
	CommandAccessor* _commandAccessor() const;

	void _initAccessors(CommandAccessor* commandAccessor);

	common::HashMap<unsigned long, BaseCommand*> __commands;

private:
	unsigned long __ID;
	std::string __name;

	CommandAccessor* __commandAccessor;
};

std::ostream& operator<< (std::ostream& os, Servant const & servant);

}

#endif /*COMMAND_SERVANT_H_*/
