/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_DRIVER_BASE_H_
#define COMMAND_DRIVER_BASE_H_

#include <command/base/Servant.h>
#include <common/base/Date.h>
#include <common/base/CommonTypes.h>

namespace command {
namespace driver {

/**
 * Base class for drivers.
 */
class Base : public Servant {

public:
	/**
	 * Constructor with ID and name.
	 */
	Base(unsigned long ID, std::string const & name);

	/**
	 * Virtual destructor.
	 */
	virtual ~Base();

	/**
	 * Gets the type.
	 */
	virtual Type _getServantType() const;

	/**
	 * throw SynchronisationException
	 */
	virtual void _synchronise(CommandAccessor* commandAccessor);

protected:
	/**
	 * Gets the command descriptors for the controller.
	 */
	virtual std::vector<common::CommandDescriptor> _getCommandDescriptors() const;
};

}
}

#endif /*COMMAND_DRIVER_BASE_H_*/
