/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "BaseCommand.h"
#include <command/exception/NoSuchCommand.h>
#include <command/transportlayer/dataprovider/CommandAccessor.h>
#include <command/base/Servant.h>
#include <common/base/Convert.h>
#include <common/base/Compare.h>
#include <common/output/OutputStream.h>
#include <iostream>

using namespace std;
using namespace common;

namespace command {

BaseCommand::BaseCommand(const std::string& name, Servant* servant) :
	Method(name, servant),
	_exceededTimeoutMs(0.0),
	_ID(0),
	_commandAccessor(0) {
}

BaseCommand::~BaseCommand() {
}

const BaseCommand & BaseCommand::getReference() const {
	return *this;
}

BaseCommand & BaseCommand::getReference() {
	return *this;
}

void BaseCommand::init(unsigned long ID, CommandAccessor* commandAccessor) {

	_ID = ID;
	_commandAccessor = commandAccessor;
}

void BaseCommand::setState(common::CommandState state) {

	try {
		_commandAccessor->setCommandState(getServant()->_getID(), _ID, state);

	} catch (Exception&) {
		throw NoSuchCommand();
	}
}

common::SetCommandActiveResult BaseCommand::setActive(const std::string& servantType) {

	try {
		return _commandAccessor->setCommandActive(getServant()->_getServantType(), getServant()->_getID(), _ID, servantType);

	} catch (Exception&) {
		throw NoSuchCommand();
	}
}

unsigned long BaseCommand::getID() const {

	return _ID;
}

common::CommandState BaseCommand::getState() const {

	try {
		return _commandAccessor->getCommandState(_ID);

	} catch (Exception&) {
		throw NoSuchCommand();
	}
}

double BaseCommand::getExceededTimeoutMs() const {

	return _exceededTimeoutMs;
}

double BaseCommand::getPreRunCalculatedDurationMs() const {
	return -1.0;
}

void BaseCommand::setProgressionPeriod(unsigned long periodMs) {
}

unsigned long BaseCommand::getProgressionPeriod() const {
	return 0;
}

void BaseCommand::calculateProgression() {
}

double BaseCommand::getElapsedTimeMs() const {

	try {
		return _commandAccessor->getCommandElapsedTimeMs(_ID);

	} catch (Exception&) {
		throw NoSuchCommand();
	}
}

double BaseCommand::_elapsedTimeMs() const {

	try {
		return _commandAccessor->getCommandElapsedTimeMs(_ID);

	} catch (Exception&) {
		throw NoSuchCommand();
	}
}

double BaseCommand::getCurrentProgression() const {

	try {
		return _commandAccessor->getProgression(_ID);

	} catch (Exception&) {
		throw NoSuchCommand();
	}
}

double BaseCommand::_currentProgression() const {

	try {
		return _commandAccessor->getProgression(_ID);

	} catch (Exception&) {
		throw NoSuchCommand();
	}
}

void BaseCommand::_resetProgression() {
	_commandAccessor->setProgression(_ID, 0.0);
}

}
