/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_METHOD_H_
#define COMMAND_METHOD_H_

#include <command/base/Operation.h>
#include <string>

namespace command {

class Servant;

/**
 * Class defining a general Method. Derived into Command and Function. 
 */
class Method : public Operation {

public:
	/**
	 * Constructor.
	 */
	Method(const std::string& name, Servant* servant);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~Method();
	
	/**
	 * Sets the name.
	 */
	void setName(const std::string& name);
	
	/**
	 * Returns the name of the command.
	 */
	const std::string& getName() const;
	
	/**
	 * Sets the servant.
	 */
	void setServant(Servant* servant);
	
	/**
	 * Gets the servant.
	 */
	Servant* getServant() const;

private:
	std::string _name;
	Servant* _servant;
	
};

}

#endif /*COMMAND_METHOD_H_*/