/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "TraceableException.h"
#include <common/output/OutputStream.h>
#include <string>

using namespace std;
using namespace common;

namespace command {

TraceableException::TraceableException(std::string const & type, std::string const & text) :
	Exception(type, text),
	_previousException(0) {
}

TraceableException::TraceableException() :
	Exception("command::TraceableException", ""),
	_previousException(0) {
}

TraceableException::TraceableException(TraceableException const & exception) :
	Exception(exception),
	_previousException(0) {

	if (exception._previousException != 0) {
		_previousException = new TraceableException(*(exception._previousException));
	}
}

void TraceableException::operator=(TraceableException const & exception) {
	_exceptionStack = exception._exceptionStack;
	_previousException = 0;

	if (exception._previousException != 0) {
		_previousException = new TraceableException(*(exception._previousException));
	}
}

TraceableException::~TraceableException() {

	delete _previousException;
}

std::string TraceableException::printStack() const {
	return toString();
}

std::string TraceableException::printCallStack() const {

	string result;

	return result;
}

BaseThrower * TraceableException::createThrower() const {
	return 0;
}

std::string TraceableException::toString() const {

	string result;

	result += printCurrentException() + "\n";

	// history
	if (_previousException != 0) {
		result += string("following previous error : ") + _previousException->toString();
	}

	return result;
}

}
