/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_TRACEABLEEXCEPTION_H_
#define COMMAND_TRACEABLEEXCEPTION_H_

#include <common/base/Exception.h>
#include <vector>
#include "Thrower.h"

namespace command {

/**
 * Class defining a traceable exception. It inherits the common::Exception class
 * and adds a call stack. The idea is to be able to trace the exception and know from
 * where it has been thrown in the N code.
 */
class TraceableException : public common::Exception {

public:
	/**
	 * Constructor.
	 */
	TraceableException();

	/**
	 * Constructor with parameters.
	 */
	TraceableException(std::string const & type, std::string const & text);

	/**
	 * Copy constructor.
	 */
	TraceableException(TraceableException const & exception);

	/**
	 * Virtual destructor.
	 */
	virtual ~TraceableException();

	/**
	 * Prints the stack.
	 */
	virtual std::string printStack() const;

	/**
	 * Prints the call stack.
	 */
	virtual std::string printCallStack() const;

	/**
	 * Operator=.
	 */
	void operator=(TraceableException const & exception);

	/**
	 * Creates a new thrower.
	 */
	virtual BaseThrower * createThrower() const;

	/**
	 * Returns the stringified content of the exception.
	 */
	std::string toString() const;

protected:
	TraceableException* _previousException;

	// Friend functions.
	template<typename ExceptionType> friend ExceptionType linkTraceable(ExceptionType const & e, TraceableException* previousException);
};

template<typename ExceptionType>
ExceptionType linkTraceable(ExceptionType const & e, TraceableException* previousException) {

	ExceptionType result(e);
	result._previousException = previousException;

	return result;
}


}

#endif /*COMMAND_TRACEABLEEXCEPTION_H_*/
