/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_BASELOG_H_
#define COMMAND_BASELOG_H_

#include <string>
#include <common/base/CommonTypes.h>

namespace command {

/**
 * Class defining a Log entry.
 */
class BaseLog {

public:
	/**
	 * Constructor.
	 */
	BaseLog(common::LogType type, common::LogLevel level, const std::string& message);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~BaseLog();

	/**
	 * Return the log level
	 */
	common::LogLevel getLevel() const;

	/**
	 * Return the log type
	 */
	common::LogType getType() const;

	/**
	 * Return the log message
	 */
	virtual std::string getMessage() const;

private:
	common::LogType _type;
	common::LogLevel _level;
	std::string _message;
};

}

#endif /*COMMAND_BASELOG_H_*/
