/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATABASELOG_H_
#define DATABASELOG_H_

#include "BaseLog.h"
#include <list>
#include "common/base/LogElement.h"

namespace command {

class DatabaseLog : public BaseLog {

public:
	DatabaseLog(common::LogType type, common::LogLevel level, std::list<common::LogElement> const & lineElements, std::string const & imageName);
	virtual ~DatabaseLog();

	/**
	 * Sets the command ID
	 */
	void setCommandID(unsigned long commandID);

	/**
	 * Sets the command ID
	 */
	void setServantID(unsigned long servantID);

	const std::list<common::LogElement> & getLineElements() const;
	const std::string & getImageName() const;

	/**
	 * Returns the command ID
	 */
	unsigned long getCommandID() const;

	/**
	 * Returns the servant ID
	 */
	unsigned long getServantID() const;

private:
	unsigned long _commandID;
	unsigned long _servantID;
	std::list<common::LogElement> _lineElements;
	std::string _imageName;

};
}

#endif /* DATABASELOG_H_ */
