/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_MESSAGE_H_
#define COMMAND_MESSAGE_H_

#include <string>

namespace command {

/**
 * Class defining a message.
 */
class Message {

public:
	/**
	 * Constructor.
	 */
	Message();

	/**
	 * Virtual destructor.
	 */
	virtual ~Message();

	/**
	 * List of message type.
	 */
	typedef enum {
		INIT_SERVANTS, EXECUTE_COMMAND, EXECUTE_ENABLE_DISABLE_COMMAND, REGISTER_PROCESSOR, UNREGISTER_PROCESSOR, REGISTER_PROCESSOR_CREATOR,
		CREATE_PROCESSOR,
		COMMAND_SUCCESS, COMMAND_FAILURE, PROPERTY_SUCCESS, PROPERTY_FAILURE,
		SHUTDOWN, FINALISE_SHUTDOWN,
		TERMINATE,
		COMMAND, ENABLE_DISABLE_COMMAND, DELETE_PROCESSOR, TERMINATE_PROCESSOR, ABORT_PROCESSOR_CREATION,
		STOP_COMMAND, PAUSE_COMMAND, RESTART_COMMAND,
		STOP_SETTER, PAUSE_SETTER, RESTART_SETTER,
		INIT_DRIVERS_AND_PROTOCOLS, INIT_TERMINATED,
		REGISTER_HANDLER,
		EXECUTE_DURATION_REQUEST,
		DURATION_REQUEST,
		DURATION_REQUEST_SUCCESS,
		DURATION_REQUEST_FAILURE,
		SET_VALUE, ICS_SET_VALUE,
		REQUEST_DEBUG, BREAK_POINT, INTERRUPT, RUN, STEP_INTO, STEP_OVER, REQUEST_VARIABLE_VALUE,
		WRITE_LOG, WRITE_SIMPLE_LOG, DATAREADY_LOG
	} Type;

	/**
	 * Gets the type.
	 */
	virtual Type getType() const = 0;

	/**
	 * Gets the type string.
	 */
	std::string getTypeString() const;

};

}

#endif /*COMMAND_MESSAGE_H_*/
