/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "TerminateMessage.h"

namespace command {

TerminateMessage::TerminateMessage(int requestID, bool success, const std::string& message, long value, common::TerminationError error) :
	Message(),
	_requestID(requestID),
	_success(success),
	_error(error),
	_message(message),
	_value(value) {
}

TerminateMessage::~TerminateMessage() {
}

Message::Type TerminateMessage::getType() const {
	return Message::TERMINATE;
}

int TerminateMessage::getRequestID() const {
	return _requestID;
}

bool TerminateMessage::isSuccessful() const {
	return _success;
}

common::TerminationError TerminateMessage::getError() const {
	return _error;
}

const std::string& TerminateMessage::getMessage() const {
	return _message;
}

long TerminateMessage::getValue() const {
	return _value;
}

}