/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_TERMINATEMESSAGE_H_
#define COMMAND_TERMINATEMESSAGE_H_

#include <command/message/Message.h>
#include <common/base/Queue.h>
#include <common/base/CommonTypes.h>

namespace command {

/**
 * Class defining a terminate message.
 */
class TerminateMessage : public Message {

public:
	/**
	 * Constructor.
	 */
	TerminateMessage(int requestID, bool success, const std::string& message, long value, common::TerminationError error = common::NO_ERROR);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~TerminateMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the request ID.
	 */
	int getRequestID() const;
	
	/**
	 * Returns true if is successful.
	 */
	bool isSuccessful() const;
	
	/**
	 * Gets the error.
	 */
	common::TerminationError getError() const;
	
	/**
	 * Gets the message.
	 */
	const std::string& getMessage() const;
	
	/**
	 * Gets the value.
	 */
	long getValue() const;
	
private:
	int _requestID;
	bool _success;
	common::TerminationError _error;
	std::string _message;
	long _value;
};

}

#endif /*COMMAND_TERMINATEMESSAGE_H_*/