/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "WriteLogMessage.h"

namespace command {

WriteLogMessage::WriteLogMessage(common::LogType logType, common::LogLevel logLevel, std::list<common::LogElement> const & lineElements, std::string const & imageName, unsigned long commandID, unsigned long servantID, bool writeLogFile, common::Queue<LogHandler::Result>* waitForReplyQueue) :
	Message(),
	_logType(logType),
	_logLevel(logLevel),
	_commandID(commandID),
	_servantID(servantID),
	_writeLogFile(writeLogFile),
	_lineElements(lineElements),
	_imageName(imageName),
	_waitForReplyQueue(waitForReplyQueue){
}

WriteLogMessage::~WriteLogMessage() {
}

Message::Type WriteLogMessage::getType() const {
	return Message::WRITE_LOG;
}

common::LogLevel WriteLogMessage::getLogLevel() const {
	return _logLevel;
}

common::LogType WriteLogMessage::getLogType() const {
	return _logType;
}

unsigned long WriteLogMessage::getCommandID() const {
	return _commandID;
}

unsigned long WriteLogMessage::getServantID() const {
	return _servantID;
}

bool WriteLogMessage::getWriteLogFile() const {
	return _writeLogFile;
}

const std::list<common::LogElement>& WriteLogMessage::getLineElements() const {
	return _lineElements;
}

const std::string& WriteLogMessage::getImageName() const {
	return _imageName;
}

common::Queue<LogHandler::Result>* WriteLogMessage::getWaitForReplyQueue() const {
	return _waitForReplyQueue;
}

}
