/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_WRITELOGMESSAGE_H_
#define COMMAND_WRITELOGMESSAGE_H_

#include <command/message/Message.h>
#include <command/log/LogHandler.h>
#include <common/base/Queue.h>
#include <common/base/CommonTypes.h>
#include <list>
#include "common/base/LogElement.h"

namespace command {

/**
 * Class defining a write log message.
 */
class WriteLogMessage : public Message {

public:
	/**
	 * Constructor for new controller log messages
	 */
	WriteLogMessage(common::LogType logType, common::LogLevel logLevel, std::list<common::LogElement> const & lineElements, std::string const & imageName, unsigned long commandID, unsigned long servantID, bool writeLogFile, common::Queue<LogHandler::Result>* waitForReplyQueue);

	/**
	 * Virtual destructor.
	 */
	virtual ~WriteLogMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Return the log level
	 */
	common::LogLevel getLogLevel() const;

	/**
	 * Return the log type
	 */
	common::LogType getLogType() const;

	/**
	 * Returns the command ID
	 */
	unsigned long getCommandID() const;
	
	/**
	 * Returns the servant ID
	 */
	unsigned long getServantID() const;
	
	/**
	 * Returns true if writes the log file.
	 */
	bool getWriteLogFile() const;

	/**
	 * Get properties field for database log messages
	 */
	const std::list<common::LogElement>& getLineElements() const;

	/**
	 * Get imageName field for database log messages
	 */
	const std::string& getImageName() const;

	/**
	 * Gets the queue.
	 */
	common::Queue<LogHandler::Result>* getWaitForReplyQueue() const;

private:
	common::LogType _logType;
	common::LogLevel _logLevel;
	unsigned long _commandID;
	unsigned long _servantID;
	bool _writeLogFile;
	
	std::list<common::LogElement> _lineElements;
	std::string _imageName;

	common::Queue<LogHandler::Result>* _waitForReplyQueue;
};

}

#endif /*COMMAND_WRITELOGMESSAGE_H_*/
