/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_BASESETVALUEMESSAGE_H_
#define COMMAND_BASESETVALUEMESSAGE_H_

#include <command/message/Message.h>
#include <common/base/PropertyType.h>

namespace command {

/**
 * Class defining a base set value message to define an interface for typed SetValueMessage used
 * by the processor message queue.
 */
class BaseSetValueMessage : public Message {

public:
	/**
	 * Constructor.
	 */
	BaseSetValueMessage(unsigned long databaseID, unsigned long propertyID, unsigned long propertySetterID);
	
	/**
	 * Constructor.
	 */
	BaseSetValueMessage(unsigned long databaseID, unsigned long propertyID, unsigned long propertySetterID, long index);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~BaseSetValueMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the Database ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Gets the original Property ID (in case of dynamic properties).
	 */
	unsigned long getPropertyID() const;
	
	/**
	 * Gets the Property setter ID.
	 */
	unsigned long getPropertySetterID() const;

	/**
	 * Gets the index of the dynamic property. 
	 */
	long getIndex() const;

private:
	unsigned long _databaseID;
	unsigned long _propertyID;
	unsigned long _propertySetterID;
	long _index;
};

}

#endif /*COMMAND_BASESETVALUEMESSAGE_H_*/