/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_COMMANDMESSAGE_H_
#define COMMAND_COMMANDMESSAGE_H_

#include <command/message/Message.h>

namespace command {

class BaseCommand;

/**
 * Class defining a command message used by the processor message queue.
 */
class CommandMessage : public Message {

public:
	/**
	 * Constructor.
	 */
	CommandMessage(unsigned long databaseID, BaseCommand* command, bool skipIfWait, unsigned long commandExecutionID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CommandMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the Command.
	 */
	BaseCommand* getCommand() const;
	
	/**
	 * Gets the Database ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Returns true if skipping if waiting.
	 */
	bool skipIfWait() const;
	
	/**
	 * Gets the Command execution ID.
	 */
	unsigned long getCommandExecutionID() const;

private:
	BaseCommand* _command;
	unsigned long _databaseID;
	bool _skipIfWait;
	unsigned long _commandExecutionID;

};

}

#endif /*COMMAND_COMMANDMESSAGE_H_*/