/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_ICSSETVALUEMESSAGE_H_
#define COMMAND_ICSSETVALUEMESSAGE_H_

#include <command/message/processor/BaseSetValueMessage.h>
#include <common/base/PropertyType.h>
#include <boost/any.hpp>

namespace command {

class Servant;

/**
 * Class defining a ics set value message used by the processor message queue.
 */
class IcsSetValueMessage : public BaseSetValueMessage {

public:
	/**
	 * Constructor.
	 */
	IcsSetValueMessage(unsigned long databaseID, unsigned long propertyID, Servant* servant, unsigned long propertySetterID, const boost::any& value);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~IcsSetValueMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the Property.
	 */
	Servant* getServant() const;

	/**
	 * Gets the value.
	 */
	const boost::any& getValue() const;

private:
	Servant* _servant;
	boost::any _value;
};

}

#endif /*COMMAND_ICSSETVALUEMESSAGE_H_*/
