/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_INITSERVANTSMESSAGE_H_
#define COMMAND_INITSERVANTSMESSAGE_H_

#include <command/message/Message.h>
#include <common/base/Queue.h>

namespace command {

/**
 * Class defining a init servant message used by the processor message queue.
 */
class InitServantsMessage : public Message {

public:
	/**
	 * Constructor.
	 */
	InitServantsMessage(unsigned long databaseID, bool verify, bool driversAndProtocolsOnly, common::Queue<bool>* queue);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~InitServantsMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Returns true if verification has to be done.
	 */
	bool verify() const;
	
	/**
	 * Returns true if the drivers and the protocols are the only servants to initialise.
	 */
	bool driversAndProtocolsOnly() const;
	
	/**
	 * Gets the queue.
	 */
	common::Queue<bool>* getQueue() const;

private:
	unsigned long _databaseID;
	bool _verify;
	bool _driversAndProtocolsOnly;
	common::Queue<bool>* _queue;
};

} 

#endif /*COMMAND_INITSERVANTSMESSAGE_H_*/