/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_BASEEXECUTESETVALUEMESSAGE_H_
#define COMMAND_BASEEXECUTESETVALUEMESSAGE_H_

#include <command/message/Message.h>
#include <common/base/PropertyType.h>

namespace command {

class CompletionHandler;

/**
 * Class defining a base execute set value message used by the processor manager.
 */
class BaseExecuteSetValueMessage : public Message {

public:
	/**
	 * Constructor.
	 */
	BaseExecuteSetValueMessage(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, CompletionHandler* completionHandler);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~BaseExecuteSetValueMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the request ID.
	 */
	unsigned long getRequestID() const;

	/**
	 * Gets the servant ID.
	 */
	unsigned long getServantID() const;
	
	/**
	 * Gets the property ID.
	 */
	unsigned long getPropertyID() const;
	
	/**
	 * Gets the property ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Gets the Completion Handler.
	 */
	CompletionHandler* getCompletionHandler() const;
		
private:
	unsigned long _requestID;
	unsigned long _servantID;
	unsigned long _propertyID;
	unsigned long _databaseID;
	CompletionHandler* _completionHandler;
};

}

#endif /*COMMAND_BASEEXECUTESETVALUEMESSAGE_H_*/