/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_DURATIONREQUESTFAILUREMESSAGE_H_
#define COMMAND_DURATIONREQUESTFAILUREMESSAGE_H_

#include "ResponseMessage.h"

#include <string>

namespace command {

/**
 * Class defining a duration request failure message used by the processor manager.
 */
class DurationRequestFailureMessage : public ResponseMessage {
	
public:
	/**
	 * Constructor.
	 */
	DurationRequestFailureMessage(unsigned long processorID, unsigned long commandID, unsigned long durationRequestExecutionID, const std::string& message);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DurationRequestFailureMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the Command ID.
	 */
	unsigned long getCommandID() const;
	
	/**
	 * Gets the Duration request execution ID.
	 */
	unsigned long getDurationRequestExecutionID() const;
	
	/**
	 * Gets the message.
	 */
	const std::string& getMessage() const;
	
private:
	unsigned long _commandID;	
	unsigned long _durationRequestExecutionID;
	std::string _message;	
};

}

#endif /*COMMAND_DURATIONREQUESTFAILUREMESSAGE_H_*/