/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_DURATIONREQUESTSUCCESSMESSAGE_H_
#define COMMAND_DURATIONREQUESTSUCCESSMESSAGE_H_

#include "ResponseMessage.h"

namespace command {

/**
 * Class defining a duration request success message used by the processor manager.
 */
class DurationRequestSuccessMessage : public ResponseMessage {
	
public:
	/**
	 * Constructor.
	 */
	DurationRequestSuccessMessage(unsigned long processorID, unsigned long commandID, unsigned long durationRequestExecutionID, double duration);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DurationRequestSuccessMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the Command ID.
	 */
	unsigned long getCommandID() const;
	
	/**
	 * Gets the Duration request execution ID.
	 */
	unsigned long getDurationRequestExecutionID() const;
	
	/**
	 * Gets the duration.
	 */
	double getDuration() const;
	
private:
	unsigned long _commandID;	
	unsigned long _durationRequestExecutionID;
	double _duration;
	
};

}

#endif /*COMMAND_DURATIONREQUESTSUCCESSMESSAGE_H_*/