/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_EXECUTECOMMANDMESSAGE_H_
#define COMMAND_EXECUTECOMMANDMESSAGE_H_

#include <command/message/Message.h>

namespace command {

class CompletionHandler;

/**
 * Class defining a execute command message used by the processor manager.
 */
class ExecuteCommandMessage : public Message {

public:
	/**
	 * Constructor.
	 */
	ExecuteCommandMessage(unsigned long requestID, unsigned long databaseID, unsigned long commandID, bool skipIfWait, CompletionHandler* completionHandler);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ExecuteCommandMessage();

	/**
	 * Gets the request ID.
	 */
	unsigned long getRequestID() const;

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the Command ID.
	 */
	unsigned long getCommandID() const;
	
	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;

	/**
	 * Returns true if skipping if waiting.
	 */
	bool skipIfWait() const;

	/**
	 * Gets the Completion Handler.
	 */
	CompletionHandler* getCompletionHandler() const;
	

private:
	unsigned long _requestID;
	unsigned long _commandID;
	unsigned long _databaseID;
	bool _skipIfWait;
	CompletionHandler* _completionHandler;

};

}

#endif /*COMMAND_EXECUTECOMMANDMESSAGE_H_*/