/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_EXECUTESETVALUEMESSAGE_H_
#define COMMAND_EXECUTESETVALUEMESSAGE_H_

#include "BaseExecuteSetValueMessage.h"
#include <command/message/Message.h>
#include <common/base/PropertyType.h>

namespace command {

/**
 * Class defining an execute set value message used by the processor manager.
 * Must be a template class to return a typed value.
 */
template<typename ValueType>
class ExecuteSetValueMessage : public BaseExecuteSetValueMessage {

public:
	/**
	 * Constructor.
	 */
	ExecuteSetValueMessage(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, const ValueType& value, CompletionHandler* completionHandler);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ExecuteSetValueMessage();

	/**
	 * Gets the value.
	 */
	const ValueType& getValue() const;
		
private:
	ValueType _value;
};

}

#include "ExecuteSetValueMessage.inl"

#endif /*COMMAND_EXECUTESETVALUEMESSAGE_H_*/