/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_REGISTERPROCESSORMESSAGE_H_
#define COMMAND_REGISTERPROCESSORMESSAGE_H_

#include <command/message/Message.h>

namespace command {

class ProcessorProxy;

/**
 * Class defining a register processor message used by the processor manager.
 */
class RegisterProcessorMessage : public Message {

public:
	/**
	 * Constructor.
	 */
	RegisterProcessorMessage(ProcessorProxy* processorProxy, int PID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~RegisterProcessorMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the Processor Proxy.
	 */
	ProcessorProxy* getProcessorProxy() const;
	
	/**
	 * Gets the PID of the process associated with the Processor.
	 */
	int getPID() const;
	
private:
	ProcessorProxy* _processorProxy;
	int _PID;

};

}

#endif /*COMMAND_REGISTERPROCESSORMESSAGE_H_*/