/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_BASESETTERITEM_H_
#define COMMAND_BASESETTERITEM_H_

#include "OperationItem.h"

namespace command {

class CompletionHandler;

/**
 * Class defining a base setter Item used by the processor manager to store information
 * relative to the setter execution. 
 */
class BaseSetterItem : public OperationItem {
	
public:
	/**
	 * Constructor.
	 */
	BaseSetterItem(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long setterID, CompletionHandler* handler);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~BaseSetterItem();
	
	/**
	 * Gets the servant ID.
	 */
	unsigned long getServantID() const;
	
	/**
	 * Gets the property ID.
	 */
	unsigned long getPropertyID() const;
	
	/**
	 * Gets the setter ID.
	 */
	unsigned long getSetterID() const;
	
	/**
	 * Gets the Completion Handler.
	 */
	CompletionHandler* getHandler() const;

	/**
	 * Gets the operation type.
	 */
	virtual Type getType() const;
	
private:
	unsigned long _servantID;
	unsigned long _propertyID;
	unsigned long _setterID;
	CompletionHandler* _handler;
};

}

#endif /*COMMAND_BASESETTERITEM_H_*/