/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CommandItem.h"
#include <command/transportlayer/core/CompletionHandler.h>

namespace command {

CommandItem::CommandItem(unsigned long requestID, unsigned long databaseID, unsigned long commandID, bool skipIfWait, unsigned long commandExecutionID, CompletionHandler* handler) :
	OperationItem(requestID, databaseID),
	_commandID(commandID),
	_skipIfWait(skipIfWait),
	_commandExecutionID(commandExecutionID),
	_handler(handler) {
}

CommandItem::~CommandItem() {
	delete _handler;
}

unsigned long CommandItem::getCommandID() const {
	return _commandID;
}

bool CommandItem::skipIfWait() const {
	return _skipIfWait;
}

unsigned long CommandItem::getCommandExecutionID() const {
	return _commandExecutionID;
}

CompletionHandler* CommandItem::getHandler() const {
	return _handler;
}

OperationItem::Type CommandItem::getType() const {
	return OperationItem::COMMAND;
}

}