/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_COMMANDITEM_H_
#define COMMAND_COMMANDITEM_H_

#include "OperationItem.h"

namespace command {

class CompletionHandler;

/**
 * Class defining a command item used by the processor manager to store information
 * relative to the command execution. 
 */
class CommandItem : public OperationItem {

public:
	/**
	 * Constructor with an still created CompletionHandler object.
	 */
	CommandItem(unsigned long requestID, unsigned long databaseID, unsigned long commandID, bool skipIfWait, unsigned long commandExecutionID, CompletionHandler* handler);
	
	/**
	 * Virtual destructor. Due to asynchronous constraints, the CompletionHandler is
	 * deleted.
	 */
	virtual ~CommandItem();

	/**
	 * Gets the Command ID.
	 */
	unsigned long getCommandID() const;

	/**
	 * Returns true if skipping if waiting.
	 */
	bool skipIfWait() const;

	/**
	 * Gets the Command execution ID.
	 */
	unsigned long getCommandExecutionID() const;

	/**
	 * Gets the Completion Handler.
	 */
	CompletionHandler* getHandler() const;
	
	/**
	 * Gets the operation type.
	 */
	virtual Type getType() const;

private:
	unsigned long _commandID;
	bool _skipIfWait;
	unsigned long _commandExecutionID;
	CompletionHandler* _handler;

};

}


#endif /*COMMAND_COMMANDITEM_H_*/