/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_DURATIONREQUESTITEM_H_
#define COMMAND_DURATIONREQUESTITEM_H_

#include "OperationItem.h"

namespace command {

class CompletionHandler;

/**
 * Class defining a duration request item used by the processor manager to store information
 * relative to the command execution. 
 */
class DurationRequestItem : public OperationItem {

public:
	/**
	 * Constructor with an still created CompletionHandler object.
	 */
	DurationRequestItem(unsigned long requestID, unsigned long databaseID, unsigned long commandID, unsigned long durationRequestExecutionID, CompletionHandler* handler);
	
	/**
	 * Virtual destructor. Due to asynchronous constraints, the CompletionHandler is
	 * deleted.
	 */
	virtual ~DurationRequestItem();

	/**
	 * Gets the Command ID.
	 */
	unsigned long getCommandID() const;

	/**
	 * Gets the Duration request execution ID.
	 */
	unsigned long getDurationRequestExecutionID() const;

	/**
	 * Gets the Completion Handler.
	 */
	CompletionHandler* getHandler() const;
	
	/**
	 * Gets the operation type.
	 */
	virtual Type getType() const;

private:
	unsigned long _commandID;
	unsigned long _durationRequestExecutionID;
	CompletionHandler* _handler;

};

}


#endif /*COMMAND_DURATIONREQUESTITEM_H_*/