/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_OPERATIONITEM_H_
#define COMMAND_OPERATIONITEM_H_

namespace command {

/**
 * Class defining an operation item. Derived into BaseSetterItem and CommandItem.
 */
class OperationItem {
	
public:
	/**
	 * Type of operation item. 
	 */
	typedef enum {COMMAND, ENABLE_DISABLE_COMMAND, PROPERTY, DURATION, INIT_DRIVERS_AND_PROTOCOLS} Type;

	/**
	 * Constructor.
	 */
	OperationItem(unsigned long requestID, unsigned long databaseID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~OperationItem();

	/**
	 * Gets the request ID.
	 */
	unsigned long getRequestID() const;

	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;

	/**
	 * Gets the operation type.
	 */
	virtual Type getType() const = 0;

private:
	unsigned long _requestID;
	unsigned long _databaseID;
};

}

#endif /*COMMAND_OPERATIONITEM_H_*/