/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

namespace command {

template<typename ValueType>
struct IcsValueTypeConverter {
	typedef ValueType result;
};

template<>
struct IcsValueTypeConverter<std::string> {
	typedef std::string result;
};

template<typename ValueType>
void Processor::setValue(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, const ValueType& value) {

	try {
		Servant* servant = ServantManager::getInstance()->getServant(servantID);

		typename IcsValueTypeConverter<ValueType>::result cValue = value;
		_messageQueue->put(new IcsSetValueMessage(databaseID, propertyID, servant, propertySetterID, cValue));

	} catch (common::Exception& e) {
		common::out << "Set value failed" << common::eol;
		// failure
		_processorManagerProxy->propertyFailure(propertyID, propertySetterID, e.printStack());
		return;
	}
}

}
