/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ProcessorItem.h"
#include "CommandItem.h"
#include <command/transportlayer/command/ProcessorProxy.h>
#include <common/output/OutputStream.h>

using namespace common;

namespace command {

ProcessorItem::ProcessorItem(unsigned long ID) :
	_ID(ID),
	_PID(0),
	_state(INIT),
	_isActive(false),
	_proxy(0),
	_operationItem(0),
	_lastInactiveDate(Date()) {
}

ProcessorItem::ProcessorItem(const ProcessorItem& processorItem) :
	_ID(processorItem._ID),
	_PID(processorItem._PID),
	_state(processorItem._state),
	_isActive(processorItem._isActive),
	_proxy(0),
	_operationItem(0),
	_lastInactiveDate(processorItem._lastInactiveDate) {
}

ProcessorItem::~ProcessorItem() {

	// delete the proxy
	delete _proxy;
}

unsigned long ProcessorItem::getID() const {
	return _ID;
}

unsigned long  ProcessorItem::getPID() const {
	return _PID;
}

bool ProcessorItem::isInitialising() const {
	return _state == INIT;
}

bool ProcessorItem::isReady() const {
	return _state == READY;
}

bool ProcessorItem::onError() const {
	return _state == ON_ERROR;
}

bool ProcessorItem::isShuttingDown() const {
	return _state == SHUTTING_DOWN;
}

bool ProcessorItem::isActive() const {
	return _isActive;
}

ProcessorProxy* ProcessorItem::getProxy() const {
	return _proxy;
}

OperationItem* ProcessorItem::getOperationItem() const {
	return _operationItem;
}

const common::Date& ProcessorItem::getLastOperationDate() const {
	return _lastOperationDate;
}

const common::Date& ProcessorItem::getLastInactiveDate() const {
	return _lastInactiveDate;
}

void ProcessorItem::setPID(unsigned long  PID)  {
	_PID = PID;		
}

void ProcessorItem::setReady() {
	_state = READY;
}

void ProcessorItem::setOnError() {
	_state = ON_ERROR;
}

void ProcessorItem::setShuttingDown() {
	_state = SHUTTING_DOWN;
}

void ProcessorItem::setActive(bool isActive) {
	_isActive = isActive;
	if (!isActive) {
		_lastInactiveDate = common::Date();
	}
}

void ProcessorItem::setProxy(ProcessorProxy* proxy) {
	_proxy = proxy;
}

void ProcessorItem::setOperationItem(OperationItem* operationItem) {
	_operationItem = operationItem;
}

void ProcessorItem::updateLastOperationDate() {
	_lastOperationDate = common::Date();
}	
	
}