/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SetterItem.h"
#include "ProcessorItem.h"
#include <command/transportlayer/command/ProcessorProxy.h>
#include <common/output/OutputStream.h>

namespace command {

template<typename ValueType>
void ProcessorManagerSetValueMessageHelper<ValueType>::apply(ExecuteSetValueMessage<ValueType>* message) {
	
	ProcessorManager* instance = ProcessorManager::_instance;
	
	std::scoped_lock lock(instance->_processorsMutex);
	
	BaseSetterItem* setterItem = new SetterItem<ValueType>(message->getRequestID(), message->getDatabaseID(), message->getServantID(), message->getPropertyID(), message->getValue(), instance->_currentSetterExecutionID++, message->getCompletionHandler());
	instance->_pendingOperations.push_back(setterItem);
	
	// Determine if we have an inactive Processor
	common::HashMap<unsigned long, ProcessorItem*>::iterator p;
	for (p = instance->_processorMap.begin(); p != instance->_processorMap.end(); p++) {
		ProcessorItem* processorItem = p->second;

		// verify that the processor proxy has been set
		if (processorItem->getProxy() != 0) {
			if (processorItem->isReady()) {
				if (processorItem->getProxy()->isAlive()) {
					if (!processorItem->isActive()) {
						break;
					}
				} else {
					common::out << "Processor (pid = " << processorItem->getPID() << ") not responding" << common::eol;
				}
			}
		}
	}

	if (p != instance->_processorMap.end()) {
		instance->executePendingOperation(p->second);			
	} else {
		instance->createProcessorItem("to execute a setter");
	}
}

}