/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ThreadProcessorInfo.h"
#include <common/output/OutputStream.h>

using namespace common;

namespace command {

ThreadProcessorInfo * ThreadProcessorInfo::_instance = new ThreadProcessorInfo();

ThreadProcessorInfo::BadProcessorID::BadProcessorID() :
	Exception("command::ThreadProcessorInfo::BadProcessorID", "uninitialised processor ID") {
}

ThreadProcessorInfo * ThreadProcessorInfo::getInstance() {
	return _instance;
}

void ThreadProcessorInfo::setProcessorID(unsigned long processorID) {

	if (processorID <= 0) {
		throw BadProcessorID();
	}

	_processorIDs.set(ProcessorID(processorID));
}

unsigned long ThreadProcessorInfo::getProcessorID() {
	return _processorIDs.get().ID;
}

void ThreadProcessorInfo::release() {
	_processorIDs.release();
}

ScopedThreadProcessorInfo::ScopedThreadProcessorInfo(unsigned long processorID) {
	ThreadProcessorInfo::getInstance()->setProcessorID(processorID);
}

ScopedThreadProcessorInfo::~ScopedThreadProcessorInfo() {
	ThreadProcessorInfo::getInstance()->release();
}

}
