/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "MultiThreadProcessorCommunication.h"
#include <command/transportlayer/command/ProcessorProxyDirectImpl.h>
#include <command/transportlayer/command/LogCompletionHandlerDirectImpl.h>
#include <command/transportlayer/command/DirectProcessorManagerProxy.h>
#include <command/transportlayer/dataprovider/DirectPropertyAccessor.h>
#include <command/transportlayer/dataprovider/DirectCommandAccessor.h>
#include <command/transportlayer/dataprovider/DirectServantAccessor.h>
#include <command/transportlayer/dataprovider/DirectLogAccessor.h>
#include <command/transportlayer/dataprovider/DirectServerLogAccessor.h>
#include <command/transportlayer/dataprovider/DirectDataChangePublisher.h>
#include <command/transportlayer/MultiThreadProcessorManagerCommunication.h>
#include <dataprovider/transportlayer/MultiThreadCommunication.h>
#include <common/base/ServerProperties.h>
#include <common/output/OutputStream.h>
#include <iostream>

using namespace common;
using namespace std;

namespace command {

MultiThreadProcessorCommunication * MultiThreadProcessorCommunication::_instance = 0;

MultiThreadProcessorCommunication::MultiThreadProcessorCommunication() :
	Communication() {
}

MultiThreadProcessorCommunication::~MultiThreadProcessorCommunication() {
}

void MultiThreadProcessorCommunication::initInstance(int argc, char* argv[], const std::string& serverID) {

	if (_instance == 0) {
		_instance = new MultiThreadProcessorCommunication();
		_instance->init(argc, argv, serverID);
	}
}

MultiThreadProcessorCommunication * MultiThreadProcessorCommunication::getInstance() {

	return _instance;
}

void MultiThreadProcessorCommunication::resetInstance() {

	if (_instance != 0) {
		_instance->reset();
		delete _instance;
	}
}


void MultiThreadProcessorCommunication::init(int argc, char* argv[], const std::string& serverID) {
		
	try {
		// Create objects
		_dataChangePublisher = new DirectDataChangePublisher(dataprovider::MultiThreadCommunication::getInstance()->getDataChangePublisher());
		_processorManagerProxy = new DirectProcessorManagerProxy(MultiThreadProcessorManagerCommunication::getInstance()->getProcessorManagerProxy());
		_propertyAccessor = new DirectPropertyAccessor(dataprovider::MultiThreadCommunication::getInstance()->getPropertyAccessor());
		_commandAccessor = new DirectCommandAccessor(dataprovider::MultiThreadCommunication::getInstance()->getCommandAccessor(), _dataChangePublisher);
		_servantAccessor = new DirectServantAccessor(dataprovider::MultiThreadCommunication::getInstance()->getServantAccessor());
		_logAccessor = new DirectLogAccessor(dataprovider::MultiThreadCommunication::getInstance()->getLogAccessor());

		_logCompletionHandler = new LogCompletionHandlerDirectImpl();

		out << "Processor communications ready" << eol;
		
	} catch (Exception const & e) {
		throw link(InitFailureException("cannot initialise"), e);
	}
	
}

void MultiThreadProcessorCommunication::reset() {

	delete _propertyAccessor;
	delete _commandAccessor;
	delete _servantAccessor;
	
	delete _logAccessor;
	
	delete _dataChangePublisher;
}

ProcessorManagerProxy * MultiThreadProcessorCommunication::getProcessorManagerProxy() {
	return _processorManagerProxy;	
}

PropertyAccessor * MultiThreadProcessorCommunication::getPropertyAccessor() {
	return _propertyAccessor;
}

CommandAccessor * MultiThreadProcessorCommunication::getCommandAccessor() {
	return _commandAccessor;
}

ServantAccessor * MultiThreadProcessorCommunication::getServantAccessor() {
	return _servantAccessor;
}

LogAccessor * MultiThreadProcessorCommunication::getLogAccessor() {
	return _logAccessor;
}

void MultiThreadProcessorCommunication::createProcessorProxy(unsigned long processorID) {

	ProcessorProxyDirectImpl* processorProxy = new ProcessorProxyDirectImpl();
	processorProxy->setProcessorID(processorID);
	_processorManagerProxy->registerProcessor(processorProxy, processorID);
}

}
