/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_MULTITHREADPROCESSORCOMMUNICATION_H_
#define COMMAND_MULTITHREADPROCESSORCOMMUNICATION_H_

#include "Communication.h"
#include <command/transportlayer/command/ProcessorProxyImpl.h>

namespace common {

	class ServerLogAccessor;
}

namespace command {

class CommunicationState;
class ProcessorManagerProxy;
class PropertyAccessor;
class CommandAccessor;
class ServantAccessor;
class DriverCompletionHandlerImpl;
class DriverCompletionHandlerDirectImpl;
class LogAccessor;
class LogCompletionHandlerDirectImpl;
class LogCompletionHandler;
class ProcessorProxyDirectImpl;
class ProcessorProxyImpl;
class DataChangePublisher;

/**
 * Communication for the processor state.
 */
class MultiThreadProcessorCommunication : public Communication {

public:
	/**
	 * Initialises the instance.
	 * throw InitFailureException
	 */
	static void initInstance(int argc, char* argv[], const std::string& serverID);

	/**
	 * Gets the instance.
	 */
	static MultiThreadProcessorCommunication * getInstance();

	/**
	 * Resets the instance.
	 * throw DestructionFailureException
	 */
	static void resetInstance();

	/**
	 * Gets the ProcessorManagerProxy.
	 */
	ProcessorManagerProxy * getProcessorManagerProxy();

	/**
	 * Gets the PropertyAccessor.
	 */
	PropertyAccessor * getPropertyAccessor();

	/**
	 * Gets the CommandAccessor.
	 */
	CommandAccessor * getCommandAccessor();

	/**
	 * Gets the ServantAccessor.
	 */
	ServantAccessor* getServantAccessor();

	/**
	 * Gets the LogAccessor.
	 */
	LogAccessor * getLogAccessor();

	/**
	 * Creates the ProcessorProxy.
	 */
	void createProcessorProxy(unsigned long processorID);

private:
	/**
	 * Constructor.
	 */
	MultiThreadProcessorCommunication();

	/**
	 * Virtual destructor.
	 */
	virtual ~MultiThreadProcessorCommunication();

	/**
	 * Initialises the object.
	 * throw InitFailureException
	 */
	void init(int argc, char* argv[], const std::string& serverID);

	/**
	 * Resets the communications.
	 */
	void reset();

	// client objects
	ProcessorManagerProxy * _processorManagerProxy;
	PropertyAccessor * _propertyAccessor;
	CommandAccessor * _commandAccessor;
	ServantAccessor * _servantAccessor;

	// log accessors
	LogAccessor * _logAccessor;
	LogCompletionHandlerDirectImpl * _logCompletionHandler;

	// server objects
	DataChangePublisher* _dataChangePublisher;

	static MultiThreadProcessorCommunication * _instance;
};

}

#endif /*COMMAND_MULTITHREADPROCESSORCOMMUNICATION_H_*/
