/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "MultiThreadProcessorManagerCommunication.h"
#include <command/transportlayer/command/CommandLauncherDirectImpl.h>
#include <command/transportlayer/command/ProcessorManagerProxyDirectImpl.h>
#include <command/transportlayer/command/ProcessorManagerMessengerDirect.h>
#include <command/transportlayer/dataprovider/DirectServerLogAccessor.h>
#include <common/output/OutputStream.h>
#include <common/base/ServerProperties.h>
#include <dataprovider/transportlayer/MultiThreadCommunication.h>
#include <iostream>

using namespace std;
using namespace common;

namespace command {

MultiThreadProcessorManagerCommunication * MultiThreadProcessorManagerCommunication::_instance = 0;

MultiThreadProcessorManagerCommunication::MultiThreadProcessorManagerCommunication() :
	Communication() {
}

MultiThreadProcessorManagerCommunication::~MultiThreadProcessorManagerCommunication() {
}

void MultiThreadProcessorManagerCommunication::initInstance(int argc, char* argv[], bool remote, const std::string& serverID) {

	if (_instance == 0) {
		_instance = new MultiThreadProcessorManagerCommunication();
		_instance->init(argc, argv, remote, serverID);
	}
}

MultiThreadProcessorManagerCommunication * MultiThreadProcessorManagerCommunication::getInstance() {

	return _instance;
}

void MultiThreadProcessorManagerCommunication::resetInstance() {

	if (_instance != 0) {
		_instance->reset(false);
		delete _instance;
	}
}

void MultiThreadProcessorManagerCommunication::init(int argc, char* argv[], bool remote, const std::string& serverID) {
	try {
		// Create the Command Launcher
		_commandLauncher = new CommandLauncherDirectImpl();

		// Create the Processor Manager Proxy
		_processorManagerProxy = new ProcessorManagerProxyDirectImpl();

		_processorManagerMessenger = new ProcessorManagerMessengerDirect();
		_processorManagerMessenger->init();

		out << "Processor manager communications ready" << eol;

	} catch (Exception const & e) {
		throw link(InitFailureException("cannot initialise"), e);
	}
}

void MultiThreadProcessorManagerCommunication::run() {

	// Create a thread and listen to the server socket.
	_processorManagerMessenger->run();
}

void MultiThreadProcessorManagerCommunication::stop() {
}

void MultiThreadProcessorManagerCommunication::reset(bool remoteShutdown) {

	_processorManagerMessenger->reset();
	delete _processorManagerMessenger;

	// Delete the objects
	delete _commandLauncher;
	delete _processorManagerProxy;
}

ProcessorManagerMessenger* MultiThreadProcessorManagerCommunication::getProcessorManagerMessenger() const {
	return _processorManagerMessenger;
}

ProcessorManagerProxyDirectImpl* MultiThreadProcessorManagerCommunication::getProcessorManagerProxy() const {
	return _processorManagerProxy;
}

CommandLauncherDirectImpl* MultiThreadProcessorManagerCommunication::getCommandLauncher() const {
	return _commandLauncher;
}

}
