/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_MULTITHREADPROCESSORMANAGERCOMMUNICATION_H_
#define COMMAND_MULTITHREADPROCESSORMANAGERCOMMUNICATION_H_

#include "Communication.h"


namespace common {

class ServerLogAccessor;

}

namespace command {

class CommandLauncherDirectImpl;
class ProcessorManagerProxyDirectImpl;
class ProcessorManagerMessenger;
class ProcessorManagerMessengerDirect;

/**
 * Communication for the processor manager state.
 */
class MultiThreadProcessorManagerCommunication : public Communication {

public:
	/**
	 * Initialises the instance.
	 * throw InitFailureException
	 */
	static void initInstance(int argc, char* argv[], bool remote, const std::string& serverID);

	/**
	 * Gets the instance.
	 */
	static MultiThreadProcessorManagerCommunication * getInstance();

	/**
	 * Resets the instance.
	 * throw DestructionFailureException
	 */
	static void resetInstance();

	/**
	 * Starts the communications.
	 */
	void run();

	/**
	 * Stops the communication.
	 */
	void stop();

	/**
	 * Gets the Processor Manager Messenger.
	 */
	ProcessorManagerMessenger* getProcessorManagerMessenger() const;

	/**
	 * Gets the Processor Manager Proxy.
	 */
	ProcessorManagerProxyDirectImpl* getProcessorManagerProxy() const;

	/**
	 * Gets the command launcher.
	 */
	CommandLauncherDirectImpl* getCommandLauncher() const;

private:
	/**
	 * Constructor.
	 */
	MultiThreadProcessorManagerCommunication();

	/**
	 * Virtual destructor.
	 */
	virtual ~MultiThreadProcessorManagerCommunication();

	/**
	 * Initialises the object.
	 * throw InitFailureException
	 */
	void init(int argc, char* argv[], bool remote, const std::string& serverID);

	/**
	 * Resets the communications.
	 */
	void reset(bool remoteShutdown);

	CommandLauncherDirectImpl* _commandLauncher;
	ProcessorManagerProxyDirectImpl* _processorManagerProxy;

	ProcessorManagerMessengerDirect* _processorManagerMessenger;

	static MultiThreadProcessorManagerCommunication * _instance;
};

}

#endif /*COMMAND_MULTITHREADPROCESSORMANAGERCOMMUNICATION_H_*/
