/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CommandLauncherDirectImpl.h"
#include <command/processor/ProcessorManager.h>
#include <command/transportlayer/core/DirectCompletionHandler.h>
#include <common/base/PropertyType.h>

using namespace common;
using namespace std;

namespace command {

CommandLauncherDirectImpl::CommandLauncherDirectImpl() {
}

CommandLauncherDirectImpl::~CommandLauncherDirectImpl() {
}

void CommandLauncherDirectImpl::execute(unsigned long requestID, unsigned long databaseID, unsigned long commandID, bool skipIfWait, core::CompletionHandlerDirectImpl* completionHandler) {

	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->executeCommand(requestID, databaseID, commandID, skipIfWait, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::stop(unsigned long databaseID, unsigned long commandID) {

	ProcessorManager::getInstance()->stopCommand(databaseID, commandID);
}

void CommandLauncherDirectImpl::pause(unsigned long databaseID, unsigned long commandID) {

	ProcessorManager::getInstance()->pauseCommand(databaseID, commandID);
}

void CommandLauncherDirectImpl::restart(unsigned long databaseID, unsigned long commandID) {

	ProcessorManager::getInstance()->restartCommand(databaseID, commandID);
}

void CommandLauncherDirectImpl::stopSetter(unsigned long databaseID, unsigned long propertyID) {

	ProcessorManager::getInstance()->stopSetter(databaseID, propertyID);
}

void CommandLauncherDirectImpl::pauseSetter(unsigned long databaseID, unsigned long propertyID) {

	ProcessorManager::getInstance()->pauseSetter(databaseID, propertyID);
}

void CommandLauncherDirectImpl::restartSetter(unsigned long databaseID, unsigned long propertyID) {

	ProcessorManager::getInstance()->restartSetter(databaseID, propertyID);
}

void CommandLauncherDirectImpl::initDriversAndProtocols() {

	ProcessorManager::getInstance()->initDriversAndProtocols();
}

void CommandLauncherDirectImpl::executeDurationRequest(unsigned long requestID, unsigned long databaseID, unsigned long commandID, core::CompletionHandlerDirectImpl* completionHandler) {

	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->executeDurationRequest(requestID, databaseID, commandID, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setInt8Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int8 value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<int8>(requestID, databaseID, servantID, propertyID, value, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setUint16Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint16 value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<uint16>(requestID, databaseID, servantID, propertyID, value, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setUint32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint32 value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<uint32>(requestID, databaseID, servantID, propertyID, value, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setUint64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint64 value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<uint64>(requestID, databaseID, servantID, propertyID, value, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setInt16Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int16 value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<int16>(requestID, databaseID, servantID, propertyID, value, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setInt32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int32 value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<int32>(requestID, databaseID, servantID, propertyID, value, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setInt64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int64 value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<int64>(requestID, databaseID, servantID, propertyID, value, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setFloat32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, float32 value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<float32>(requestID, databaseID, servantID, propertyID, value, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setFloat64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, float64 value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<float64>(requestID, databaseID, servantID, propertyID, value, new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setStringValue(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, const std::string& value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<string>(requestID, databaseID, servantID, propertyID, string(value), new DirectCompletionHandler(completionHandler));
}

void CommandLauncherDirectImpl::setBooleanValue(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, bool value, core::CompletionHandlerDirectImpl* completionHandler) {
	// Creating the DirectCompletionHandler and passing it to the Processor
	ProcessorManager::getInstance()->setValue<bool>(requestID, databaseID, servantID, propertyID, value, new DirectCompletionHandler(completionHandler));
}

}
