/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_COMMANDLAUNCHERDIRECTIMPL_H_
#define COMMAND_COMMANDLAUNCHERDIRECTIMPL_H_

#include <common/base/BaseTypes.h>
#include <string>

namespace core {

class CompletionHandlerDirectImpl;

}

namespace command {

/**
 * Class defining the direct implementation of the command launcher. This provides a an interface
 * the processor, accepting commands to be executed. A completion handler is
 * passed allowing the Core to be notified of command completion.
 */
class CommandLauncherDirectImpl {
	
public:
	/**
	 * Constructor.
	 */
	CommandLauncherDirectImpl();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CommandLauncherDirectImpl();
	
	/**
	 * Forwards command request to the Processor along with a new CompletionHandler.
	 */
	virtual void execute(unsigned long requestID, unsigned long databaseID, unsigned long commandID, bool skipIfWait, core::CompletionHandlerDirectImpl* completionHandler);

	/**
	 * Forwards stop command request to the Processor. 
	 */
	virtual void stop(unsigned long databaseID, unsigned long commandID);
	
	/**
	 * Forwards pause command request to the Processor. 
	 */
	virtual void pause(unsigned long databaseID, unsigned long commandID);
	
	/**
	 * Forwards restart command request to the Processor. 
	 */
	virtual void restart(unsigned long databaseID, unsigned long commandID);
	
	/**
	 * Forwards stop setter reqeust to the Processor. 
	 */
	virtual void stopSetter(unsigned long databaseID, unsigned long propertyID);
	
	/**
	 * Forwards pause setter reqeust to the Processor. 
	 */
	virtual void pauseSetter(unsigned long databaseID, unsigned long propertyID);
	
	/**
	 * Forwards restart setter reqeust to the Processor. 
	 */
	virtual void restartSetter(unsigned long databaseID, unsigned long propertyID);
	
	/**
	 * initialises the drivers and the protocols.
	 */
	virtual void initDriversAndProtocols();
	
	/**
	 * Forwards duration request to the Processor along with a new CompletionHandler.
	 */
	virtual void executeDurationRequest(unsigned long requestID, unsigned long databaseID, unsigned long commandID, core::CompletionHandlerDirectImpl* completionHandler);

	/**
	 * Sets the int8 value.
	 */
	virtual void setInt8Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int8 value, core::CompletionHandlerDirectImpl* completionHandler);
		
	/**
	 * Sets the int16 value.
	 */
	virtual void setUint16Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint16 value, core::CompletionHandlerDirectImpl* completionHandler);

	/**
	 * Sets the int32 value.
	 */
	virtual void setUint32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint32 value, core::CompletionHandlerDirectImpl* completionHandler);

	/**
	 * Sets the int64 value.
	 */
	virtual void setUint64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint64 value, core::CompletionHandlerDirectImpl* completionHandler);

	/**
	 * Sets the int16 value.
	 */
	virtual void setInt16Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int16 value, core::CompletionHandlerDirectImpl* completionHandler);
		
	/**
	 * Sets the int32 value.
	 */
	virtual void setInt32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int32 value, core::CompletionHandlerDirectImpl* completionHandler);
		
	/**
	 * Sets the int64 value.
	 */
	virtual void setInt64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int64 value, core::CompletionHandlerDirectImpl* completionHandler);
		
	/**
	 * Sets the float32 value.
	 */
	virtual void setFloat32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, float32 value, core::CompletionHandlerDirectImpl* completionHandler);
		
	/**
	 * Sets the float64 value.
	 */
	virtual void setFloat64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, float64 value, core::CompletionHandlerDirectImpl* completionHandler);
		
	/**
	 * Sets the string value.
	 */
	virtual void setStringValue(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, const std::string& value, core::CompletionHandlerDirectImpl* completionHandler);
		
	/**
	 * Sets the boolean value.
	 */
	virtual void setBooleanValue(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, bool value, core::CompletionHandlerDirectImpl* completionHandler);
	
};

}

#endif /*COMMAND_COMMANDLAUNCHERDIRECTIMPL_H_*/
