/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectProcessorManagerProxy.h"
#include "ProcessorProxyDirectImpl.h"
#include "ProcessorManagerProxyDirectImpl.h"
#include <command/processor/Processor.h>
#include <iostream>

using namespace std;
using namespace common;

namespace command {

DirectProcessorManagerProxy::DirectProcessorManagerProxy(ProcessorManagerProxyDirectImpl* proxy) :
	_proxy(proxy) {
}

DirectProcessorManagerProxy::~DirectProcessorManagerProxy() {
}

	
void DirectProcessorManagerProxy::registerProcessor(ProcessorProxyImpl* processorProxy, unsigned long PID) {
	
	_proxy->registerProcessor(dynamic_cast<ProcessorProxyDirectImpl*>(processorProxy), PID);
}

void DirectProcessorManagerProxy::commandSuccess(unsigned long commandID, unsigned long commandExecutionID) {
	
	_proxy->commandSuccess(Processor::getInstance()->getProcessorID(), commandID, commandExecutionID);
}

void DirectProcessorManagerProxy::commandFailure(unsigned long commandID, unsigned long commandExecutionID, const std::string& message) {
	
	_proxy->commandFailure(Processor::getInstance()->getProcessorID(), commandID, commandExecutionID, message);
}

void DirectProcessorManagerProxy::propertySuccess(unsigned long propertyID, unsigned long propertySetterID) {
	
	_proxy->propertySuccess(Processor::getInstance()->getProcessorID(), propertyID, propertySetterID);
} 

void DirectProcessorManagerProxy::propertyFailure(unsigned long propertyID, unsigned long propertySetterID, const std::string& message) {
	
	_proxy->propertyFailure(Processor::getInstance()->getProcessorID(), propertyID, propertySetterID, message);
}

void DirectProcessorManagerProxy::durationSuccess(unsigned long commandID, unsigned long durationRequestExecutionID, double duration) {
	
	_proxy->durationRequestSuccess(Processor::getInstance()->getProcessorID(), commandID, durationRequestExecutionID, duration);
}
	
void DirectProcessorManagerProxy::durationFailure(unsigned long commandID, unsigned long durationRequestExecutionID, const std::string& message) {

	_proxy->durationRequestFailure(Processor::getInstance()->getProcessorID(), commandID, durationRequestExecutionID, message);
}

void DirectProcessorManagerProxy::initTerminated() {
	
	_proxy->initTerminated(Processor::getInstance()->getProcessorID());
}

}