/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectProcessorProxy.h"
#include <common/base/Exception.h>
#include <command/transportlayer/command/ProcessorProxyDirectImpl.h>
#include <iostream>

using namespace common;
using namespace std;

namespace command {

DirectProcessorProxy::DirectProcessorProxy(command::ProcessorProxyDirectImpl* proxy) :
	_proxy(proxy){
}

DirectProcessorProxy::~DirectProcessorProxy() {
	// delete the implementation!
	delete _proxy;
}

void DirectProcessorProxy::setProcessorID(unsigned long processorID) {
	_proxy->setProcessorID(processorID);
}

unsigned long DirectProcessorProxy::getProcessorID() {
	return _proxy->getProcessorID();
}

bool DirectProcessorProxy::isAlive() {
	try {
		return _proxy->isAlive();
	} catch (Exception& e) {
		return false;
	}
}

bool DirectProcessorProxy::canTerminate() {
	return _proxy->canTerminate();
}
	
void DirectProcessorProxy::terminate() {
	_proxy->terminate();
}

bool DirectProcessorProxy::initServants(unsigned long databaseID, bool verify) {
	return _proxy->initServants(databaseID, verify);
}

void DirectProcessorProxy::initDriversAndProtocols() {
	_proxy->initDriversAndProtocols();
}

std::string DirectProcessorProxy::getCurrentStack() {
	return _proxy->getCurrentStack();
}

void DirectProcessorProxy::execute(unsigned long databaseID, unsigned long commandID, bool skipIfWait, unsigned long commandExecutionID) {
	_proxy->execute(databaseID, commandID, skipIfWait, commandExecutionID);
}

void DirectProcessorProxy::stop(unsigned long databaseID, unsigned long commandOrPropertyID) {
	_proxy->stop(databaseID, commandOrPropertyID);
}

void DirectProcessorProxy::pause(unsigned long databaseID, unsigned long commandOrPropertyID) {
	_proxy->pause(databaseID, commandOrPropertyID);
}

void DirectProcessorProxy::restart(unsigned long databaseID, unsigned long commandOrPropertyID) {
	_proxy->restart(databaseID, commandOrPropertyID);
}

void DirectProcessorProxy::executeDurationRequest(unsigned long databaseID, unsigned long commandID, unsigned long commandExecutionID) {
	_proxy->executeDurationRequest(databaseID, commandID, commandExecutionID);
}

void DirectProcessorProxy::shutdown() {
	_proxy->shutdown();
}

void DirectProcessorProxy::setInt8Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int8 value) const {
	_proxy->setInt8Value(databaseID, servantID, propertyID, propertySetterID, value);
}

void DirectProcessorProxy::setUint16Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, uint16 value) const {
	_proxy->setUint16Value(databaseID, servantID, propertyID, propertySetterID, value);
}

void DirectProcessorProxy::setUint32Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, uint32 value) const {
	_proxy->setUint32Value(databaseID, servantID, propertyID, propertySetterID, value);
}

void DirectProcessorProxy::setUint64Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, uint64 value) const {
	_proxy->setUint64Value(databaseID, servantID, propertyID, propertySetterID, value);
}
		
void DirectProcessorProxy::setInt16Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int16 value) const {
	_proxy->setInt16Value(databaseID, servantID, propertyID, propertySetterID, value);
}
	
void DirectProcessorProxy::setInt32Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int32 value) const {
	_proxy->setInt32Value(databaseID, servantID, propertyID, propertySetterID, value);
}
		
void DirectProcessorProxy::setInt64Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int64 value) const {
	_proxy->setInt64Value(databaseID, servantID, propertyID, propertySetterID, value);
}
		
void DirectProcessorProxy::setFloat32Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, float32 value) const {
	_proxy->setFloat32Value(databaseID, servantID, propertyID, propertySetterID, value);
}
	
void DirectProcessorProxy::setFloat64Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, float64 value) const {
	_proxy->setFloat64Value(databaseID, servantID, propertyID, propertySetterID, value);
}
	
void DirectProcessorProxy::setStringValue(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, const std::string& value) const {
	_proxy->setStringValue(databaseID, servantID, propertyID, propertySetterID, value);
}
	
void DirectProcessorProxy::setBooleanValue(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, bool value) const {
	_proxy->setBooleanValue(databaseID, servantID, propertyID, propertySetterID, value);
}

}
