/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_DIRECTPROCESSORPROXY_H_
#define COMMAND_DIRECTPROCESSORPROXY_H_

#include "ProcessorProxy.h"

namespace command {

class ProcessorProxyDirectImpl;

/**
 * Class defining a direct processor proxy.
 */
class DirectProcessorProxy : public ProcessorProxy {
	
public:
	/**
	 * Constructor.
	 */
	DirectProcessorProxy(ProcessorProxyDirectImpl* proxy);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DirectProcessorProxy();
	
	/**
	 * Sets the Processor ID.
	 */
	virtual void setProcessorID(unsigned long processorID);
	
	/**
	 * Gets the Processor ID.
	 */
	virtual unsigned long getProcessorID();
	
	/**
	 * Returns true if is alive.
	 */
	virtual bool isAlive();
	
	/**
	 * Returns true if can terminate.
	 */
	virtual bool canTerminate();
	
	/**
	 * Terminates the processor.
	 */
	virtual void terminate();
	
	/**
	 * Returns true if the verification successes.
	 */
	virtual bool initServants(unsigned long databaseID, bool verify);
	
	/**
	 * Initialises the drivers and protocols.
	 */
	virtual void initDriversAndProtocols();
	
	/**
	 * Gets the current stack.
	 */
	virtual std::string getCurrentStack();
	
	/**
	 * Executes the Command.
	 */
	virtual void execute(unsigned long databaseID, unsigned long commandID, bool skipIfWait, unsigned long commandExecutionID);
	
	/**
	 * Stops the Command.
	 */
	virtual void stop(unsigned long databaseID, unsigned long commandOrPropertyID);

	/**
	 * Pauses the Command.
	 */
	virtual void pause(unsigned long databaseID, unsigned long commandOrPropertyID);

	/**
	 * Restarts the Command.
	 */
	virtual void restart(unsigned long databaseID, unsigned long commandOrPropertyID);
	
	/**
	 * Executes the Duration request.
	 */
	virtual void executeDurationRequest(unsigned long databaseID, unsigned long commandID, unsigned long durationRequestExecutionID);
	
	/**
	 * Shut downs the Processor.
	 */
	virtual void shutdown();
	
	/**
	 * Sets the int8 value for the property of ID propertyID.
	 */
	virtual void setInt8Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int8 value) const;

	/**
	 * Sets the int16 value for the property of ID propertyID.
	 */
	virtual void setUint16Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, uint16 value) const;

	/**
	 * Sets the int32 value for the property of ID propertyID.
	 */
	virtual void setUint32Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, uint32 value) const;

	/**
	 * Sets the int64 value for the property of ID propertyID.
	 */
	virtual void setUint64Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, uint64 value) const;
		
	/**
	 * Sets the int16 value for the property of ID propertyID.
	 */
	virtual void setInt16Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int16 value) const;
		
	/**
	 * Sets the int32 value for the property of ID propertyID.
	 */
	virtual void setInt32Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int32 value) const;
		
	/**
	 * Sets the int64 value for the property of ID propertyID.
	 */
	virtual void setInt64Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int64 value) const;
		
	/**
	 * Sets the float32 value for the property of ID propertyID.
	 */
	virtual void setFloat32Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, float32 value) const;
	
	/**
	 * Sets the float64 value for the property of ID propertyID.
	 */
	virtual void setFloat64Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, float64 value) const;
	
	/**
	 * Sets the string value for the property of ID propertyID.
	 */
	virtual void setStringValue(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, const std::string& value) const;
	
	/**
	 * Sets the bool value for the property of ID propertyID.
	 */
	virtual void setBooleanValue(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, bool value) const;

private:
	ProcessorProxyDirectImpl* _proxy;
};

}

#endif /*COMMAND_DIRECTPROCESSORPROXY_H_*/
