/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_LOGCOMPLETIONHANDLERDIRECTIMPL_H_
#define COMMAND_LOGCOMPLETIONHANDLERDIRECTIMPL_H_

#include <string>
#include <mutex>

namespace command {

/**
 * Class defining the direct implementation of log completion handler.
 */
class LogCompletionHandlerDirectImpl {
	
public:
	/**
	 * Constructor.
	 */
	LogCompletionHandlerDirectImpl();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~LogCompletionHandlerDirectImpl();

	/**
	 * Gets the ID.
	 */
	const std::string& getID() const;
	
	/**
	 * Terminates the request.
	 */
	void terminate(unsigned long requestID, bool success, const std::string& message);

private:
	static std::string getNextID();
	static unsigned long _counter;
	static std::mutex _counterMutex;

	std::string _ID;
	
};

}

#endif /*COMMAND_LOGCOMPLETIONHANDLERIMPL_H_*/
