/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_MESSENGER_H_
#define COMMAND_MESSENGER_H_

#include <string>

namespace command {

/**
 * Class defining an abstract messenger.
 */
class Messenger {
	
public:
	/**
	 * Type of the message.  
	 */
	typedef enum {STOP_SERVER_SOCKET, PROCESSOR_CREATION_REQUEST, PROCESSOR_TERMINATION_REQUEST, PROCESSOR_CREATION_FAILURE, PING, SHUTDOWN} Type;

	/**
	 * Virtual destructor.
	 */
	virtual ~Messenger();
	
	/**
	 * Initialises the communications.
	 */
	virtual void init() = 0;
	
	/**
	 * Runs the communications.
	 */
	virtual void run() = 0;
	
	/**
	 * Resets the communications.
	 */
	virtual void reset() = 0;

	/**
	 * Returns the type string.
	 */
	static std::string getTypeString(Type value);
};


}

#endif /*COMMAND_MESSENGER_H_*/