/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_PROCESSORMANAGERMESSENGERDIRECT_H_
#define COMMAND_PROCESSORMANAGERMESSENGERDIRECT_H_

#include "ProcessorManagerMessenger.h"
#include <command/message/Message.h>
#include <common/base/Exception.h>
#include <common/base/Queue.h>
#include <common/base/HashMap.h>
#include <thread>

namespace command {

class ProcessorThread;
class CreateProcessorMessage;
class TerminateProcessorMessage;
class ShutdownMessage;

/**
 * Class defining a direct processor manager messenger.
 */
class ProcessorManagerMessengerDirect : public ProcessorManagerMessenger {
	
public:
	/**
	 * Constructor.
	 */
	ProcessorManagerMessengerDirect();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ProcessorManagerMessengerDirect();
	
	/**
	 * Initialises the communications.
	 */
	void init();
	
	/**
	 * Runs the communications.
	 */
	void run();
	
	/**
	 * Sends a Create request.
	 */
	virtual void sendCreate(unsigned long processorID);
	
	/**
	 * Sends a Terminate request.
	 */
	virtual void sendTerminate(unsigned long PID);
	
	/**
	 * Sends a request to shutdown the Processor Creator.
	 */
	virtual void shutdown();
	
	/**
	 * Resets the communication.
	 */
	virtual void reset();
	
private:
	void processMessage(Message* message);
	void processMessage(CreateProcessorMessage* message);
	void processMessage(TerminateProcessorMessage* message);
	void processMessage(ShutdownMessage* message);

	static void loop(ProcessorManagerMessengerDirect* instance);

	common::HashMap<unsigned long, ProcessorThread*> _processorThreadMap;
	common::Queue<Message>* _messageQueue;
	std::thread* _thread;
	bool _running;
};

}

#endif /*COMMAND_PROCESSORMANAGERMESSENGERDIRECT_H_*/
