/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_PROCESSORMANAGERPROXY_H_
#define COMMAND_PROCESSORMANAGERPROXY_H_

#include <string>

namespace command {

class ProcessorProxyImpl;

/**
 * Class defining a processor manager proxy interface.
 */
class ProcessorManagerProxy {
	
public:
	/**
	 * Constructor.
	 */
	ProcessorManagerProxy();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ProcessorManagerProxy();
	
	/**
	 * Registers a new Processor.
	 */
	virtual void registerProcessor(ProcessorProxyImpl* processorProxy, unsigned long PID) = 0;

	/**
	 * Notifies that the Command successed.
	 */
	virtual void commandSuccess(unsigned long commandID, unsigned long commandExecutionID) = 0;
	
	/**
	 * Notifies that the Command failed.
	 */
	virtual void commandFailure(unsigned long commandID, unsigned long commandExecutionID, const std::string& message) = 0;
	
	/**
	 * Notifies that the Property setter successed.
	 */
	virtual void propertySuccess(unsigned long propertyID, unsigned long propertySetterID) = 0;
	
	/**
	 * Notifies that the Property setter failed.
	 */
	virtual void propertyFailure(unsigned long propertyID, unsigned long propertySetterID, const std::string& message) = 0;

	/**
	 * Notifies that the Duration request successed.
	 */
	virtual void durationSuccess(unsigned long commandID, unsigned long durationExecutionID, double duration) = 0;
	
	/**
	 * Notifies that the Duration request failed.
	 */
	virtual void durationFailure(unsigned long commandID, unsigned long durationExecutionID, const std::string& message) = 0;
	
	/**
	 * Notifies that the initialisation has terminated.
	 */
	virtual void initTerminated() = 0;
};

}

#endif /*COMMAND_PROCESSORMANAGERPROXY_H_*/