/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_PROCESSORMANAGERPROXYDIRECTIMPL_H_
#define COMMAND_PROCESSORMANAGERPROXYDIRECTIMPL_H_

#include <string>

namespace command {

class ProcessorProxyDirectImpl;

/**
 * Class defining a direct processor manager proxy implementation.
 */
class ProcessorManagerProxyDirectImpl {
	
public:
	/**
	 * Constructor.
	 */
	ProcessorManagerProxyDirectImpl();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ProcessorManagerProxyDirectImpl();
	
	/**
	 * Registers a new Processor.
	 */
	virtual void registerProcessor(ProcessorProxyDirectImpl* processorProxy, unsigned long PID);
	
	/**
	 * Notifies that the Command successed.
	 */
	virtual void commandSuccess(unsigned long processorID, unsigned long commandID, unsigned long commandExecutionID);
	
	/**
	 * Notifies that the Command failed.
	 */
	virtual void commandFailure(unsigned long processorID, unsigned long commandID, unsigned long commandExecutionID, const std::string& message);
	
	/**
	 * Notifies that the Property setter successed.
	 */
	virtual void propertySuccess(unsigned long processorID, unsigned long propertyID, unsigned long propertySetterID);
	
	/**
	 * Notifies that the Property setter failed.
	 */
	virtual void propertyFailure(unsigned long processorID, unsigned long propertyID, unsigned long propertySetterID, const std::string& message);
	
	/**
	 * Notifies that the Duration request successed.
	 */
	virtual void durationRequestSuccess(unsigned long processorID, unsigned long commandID, unsigned long durationRequestExecutionID, double duration);
	
	/**
	 * Notifies that the Duration request failed.
	 */
	virtual void durationRequestFailure(unsigned long processorID, unsigned long commandID, unsigned long durationRequestExecutionID, const std::string& message);
	
	/**
	 * Notifies that the initialisation has terminated.
	 */
	virtual void initTerminated(unsigned long processorID);
};

}

#endif /*COMMAND_PROCESSORMANAGERPROXYIMPL_H_*/