/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ProcessorProxyDirectImpl.h"
#include <command/processor/Processor.h>
#include <iostream>

using namespace common;
using namespace std;

namespace command {

ProcessorProxyDirectImpl::ProcessorProxyDirectImpl() :
	_processorID(0)	{
}

ProcessorProxyDirectImpl::~ProcessorProxyDirectImpl() {
}

void ProcessorProxyDirectImpl::setProcessorID(unsigned long processorID) {
	_processorID = processorID;
}

unsigned long ProcessorProxyDirectImpl::getProcessorID() {
	return _processorID;
}

bool ProcessorProxyDirectImpl::isAlive() {
	return true;
}

bool ProcessorProxyDirectImpl::canTerminate() {
	return (Processor::getInstance(_processorID) == 0);
}

void ProcessorProxyDirectImpl::terminate() {
}

bool ProcessorProxyDirectImpl::initServants(unsigned long databaseID, bool verify) {
	return Processor::getInstance(_processorID)->initServants(databaseID, verify);
}

void ProcessorProxyDirectImpl::initDriversAndProtocols() {
	Processor::getInstance(_processorID)->initDriversAndProtocols();
}

std::string ProcessorProxyDirectImpl::getCurrentStack() {
	return Processor::getInstance(_processorID)->getCurrentStack();
}

void ProcessorProxyDirectImpl::execute(unsigned long databaseID, unsigned long commandID, bool skipIfWait, unsigned long commandExecutionID) {
	Processor::getInstance(_processorID)->execute(databaseID, commandID, skipIfWait, commandExecutionID);
}

void ProcessorProxyDirectImpl::stop(unsigned long databaseID, unsigned long commandOrPropertyID) {
	Processor::getInstance(_processorID)->stop(databaseID, commandOrPropertyID);
}

void ProcessorProxyDirectImpl::pause(unsigned long databaseID, unsigned long commandOrPropertyID) {
	Processor::getInstance(_processorID)->pause(databaseID, commandOrPropertyID);
}

void ProcessorProxyDirectImpl::restart(unsigned long databaseID, unsigned long commandOrPropertyID) {
	Processor::getInstance(_processorID)->restart(databaseID, commandOrPropertyID);
}

void ProcessorProxyDirectImpl::executeDurationRequest(unsigned long databaseID, unsigned long commandID, unsigned long commandExecutionID) {
	Processor::getInstance(_processorID)->executeDurationRequest(databaseID, commandID, commandExecutionID);
}

void ProcessorProxyDirectImpl::shutdown() {
	Processor::getInstance(_processorID)->shutdown();
}

void ProcessorProxyDirectImpl::setInt8Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int8 value) {
	Processor::getInstance(_processorID)->setValue<int8>(databaseID, servantID, propertyID, propertySetterID, value);
}

void ProcessorProxyDirectImpl::setUint16Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, uint16 value) {
	Processor::getInstance(_processorID)->setValue<uint16>(databaseID, servantID, propertyID, propertySetterID, value);
}

void ProcessorProxyDirectImpl::setUint32Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, uint32 value) {
	Processor::getInstance(_processorID)->setValue<uint32>(databaseID, servantID, propertyID, propertySetterID, value);
}

void ProcessorProxyDirectImpl::setUint64Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, uint64 value) {
	Processor::getInstance(_processorID)->setValue<uint64>(databaseID, servantID, propertyID, propertySetterID, value);
}

void ProcessorProxyDirectImpl::setInt16Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int16 value) {
	Processor::getInstance(_processorID)->setValue<int16>(databaseID, servantID, propertyID, propertySetterID, value);
}

void ProcessorProxyDirectImpl::setInt32Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int32 value) {
	Processor::getInstance(_processorID)->setValue<int32>(databaseID, servantID, propertyID, propertySetterID, value);
}

void ProcessorProxyDirectImpl::setInt64Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, int64 value) {
	Processor::getInstance(_processorID)->setValue<int64>(databaseID, servantID, propertyID, propertySetterID, value);
}

void ProcessorProxyDirectImpl::setFloat32Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, float32 value) {
	Processor::getInstance(_processorID)->setValue<float32>(databaseID, servantID, propertyID, propertySetterID, value);
}

void ProcessorProxyDirectImpl::setFloat64Value(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, float64 value) {
	Processor::getInstance(_processorID)->setValue<float64>(databaseID, servantID, propertyID, propertySetterID, value);
}

void ProcessorProxyDirectImpl::setStringValue(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, const std::string& value) {
	Processor::getInstance(_processorID)->setValue<string>(databaseID, servantID, propertyID, propertySetterID, string(value));
}

void ProcessorProxyDirectImpl::setBooleanValue(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, unsigned long propertySetterID, bool value) {
	Processor::getInstance(_processorID)->setValue<bool>(databaseID, servantID, propertyID, propertySetterID, value);
}

void ProcessorProxyDirectImpl::setThreadProcessorID(unsigned long processorID) {
	_processorID = processorID;
}

}
