/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CommandStateChangeSubscriberDirectImpl.h"
#include <command/transportlayer/dataprovider/DirectDataChangePublisher.h>
#include <command/transportlayer/dataprovider/DatabaseIDKeeper.h>
#include <common/output/OutputStream.h>

using namespace common;
using namespace std;

namespace command {

unsigned long CommandStateChangeSubscriberDirectImpl::_counter = 0;
std::mutex CommandStateChangeSubscriberDirectImpl::_counterMutex;

CommandStateChangeSubscriberDirectImpl::CommandStateChangeSubscriberDirectImpl(DataChangePublisher* dataChangePublisher, Servant::Type servantType, unsigned long servantID, unsigned long commandID) :
	_ID(0),
	_servantType(servantType),
	_servantID(servantID),
	_commandID(commandID),
	_terminated(false),
	_dataChangePublisher(dataChangePublisher) {
}

CommandStateChangeSubscriberDirectImpl::~CommandStateChangeSubscriberDirectImpl() {
}

void CommandStateChangeSubscriberDirectImpl::setID(unsigned long subscriberID) {
	_ID = subscriberID;
}

unsigned long CommandStateChangeSubscriberDirectImpl::getID() {
	return _ID;
}

void CommandStateChangeSubscriberDirectImpl::subscribe() {
	_ID = static_cast<DirectDataChangePublisher*>(_dataChangePublisher)->subscribe(this);
}

void CommandStateChangeSubscriberDirectImpl::unsubscribe() {
	_dataChangePublisher->unsubscribe(_ID);
}

void CommandStateChangeSubscriberDirectImpl::propertyChanged(unsigned long databaseID, unsigned long propertyID) {
}

void CommandStateChangeSubscriberDirectImpl::commandStateChanged(unsigned long databaseID, unsigned long servantID, unsigned long commandID, common::CommandState commandState) {

	// receives all the command state changes. could be optimised!
	if (DatabaseIDKeeper::getID() == databaseID && commandState == common::INACTIVE) {

		//out << "commandStateChanged of (" << _servantID << " - " << _commandID << ") receives (" << servantID << " - " << commandID << ") inactive" << eol;

		if ((_servantType == DataHolder::CONTROLLER && commandID == _commandID)
			|| (_servantType == DataHolder::DRIVER && servantID == _servantID)) {
			_terminated = true;

			//out << "commandStateChanged of (" << _servantID << " - " << _commandID << ") receives (" << servantID << " - " << commandID << ") terminated" << eol;
		}
	}
}

void CommandStateChangeSubscriberDirectImpl::commandInterruptChanged(unsigned long databaseID, unsigned long servantID, unsigned long commandID, bool value) {
}

void CommandStateChangeSubscriberDirectImpl::commandProgressionChanged(unsigned long databaseID, unsigned long dataID, double progression) {
}

void CommandStateChangeSubscriberDirectImpl::configurationChanged(unsigned long databaseID, unsigned long servantID, common::ConfigurationState enable) {
}

void CommandStateChangeSubscriberDirectImpl::conditionStateChanged(unsigned long conditionId, bool on, common::ConditionState state) {
}

void CommandStateChangeSubscriberDirectImpl::conditionActionsStateChanged(unsigned long conditionId, common::ConditionState state) {
}

bool CommandStateChangeSubscriberDirectImpl::hasTerminated() {
	return _terminated;
}

}
