/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDSTATECHANGESUBSCRIBERDIRECTIMPL_H_
#define CORE_COMMANDSTATECHANGESUBSCRIBERDIRECTIMPL_H_

#include <command/base/Servant.h>
#include <common/utils/DataChangeSubscriberDirect.h>
#include <string>

namespace command {

class DataChangePublisher;

/**
 * Direct server implementation for the commands of the data change subscriber class.
 */
class CommandStateChangeSubscriberDirectImpl : public common::DataChangeSubscriberDirect {

public:
	/**
	 * Constructor.
	 */
	CommandStateChangeSubscriberDirectImpl(DataChangePublisher* dataChangePublisher, Servant::Type servantType, unsigned long servantID, unsigned long commandID);

	/**
	 * Virtual destructor.
	 */
	virtual ~CommandStateChangeSubscriberDirectImpl();

	/**
	 * Subscribes to publisher.
	 */
	void subscribe();

	/**
	 * Unsubscribes from publisher.
	 */
	void unsubscribe();

	/**
	 * Sets the ID of the subscriber.
	 */
	virtual void setID(unsigned long subscriberID);

	/**
	 * Returns the ID of the subscriber.
	 */
	virtual unsigned long getID();

	/**
	 * Notifies the subscriber that a property has changed.
	 */
	virtual void propertyChanged(unsigned long databaseID, unsigned long propertyID);

	/**
	 * Notifies the subscriber that a command state has changed.
	 */
	virtual void commandStateChanged(unsigned long databaseID, unsigned long servantID, unsigned long commandID, common::CommandState state);

	/**
	 * Notifies the subscriber that a command interrupt has changed.
	 */
	virtual void commandInterruptChanged(unsigned long databaseID, unsigned long servantID, unsigned long commandID, bool value);

	/**
	 * Notifies the subscriber that a command progression has changed.
	 */
	virtual void commandProgressionChanged(unsigned long databaseID, unsigned long commandID, double progression);

	/**
	 * Notifies the subscriber that the configuration has changed.
	 */
	virtual void configurationChanged(unsigned long databaseID, unsigned long servantID, common::ConfigurationState enable);

	/**
	 * Notifies the condition state.
	 */
	virtual void conditionStateChanged(unsigned long conditionId, bool on, common::ConditionState state);

	/**
	 * Notifies the condition state.
	 */
	virtual void conditionActionsStateChanged(unsigned long conditionId, common::ConditionState state);

	/**
	 * Returns true if has terminated.
	 */
	bool hasTerminated();

private:
	static unsigned long _counter;
	static std::mutex _counterMutex;

	unsigned long _ID;
	Servant::Type _servantType;
	unsigned long _servantID;
	unsigned long _commandID;
	bool _terminated;
	DataChangePublisher* _dataChangePublisher;
};

}

#endif /*CORE_COMMANDSTATECHANGESUBSCRIBERDIRECTIMPL_H_*/
