/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DIRECTCOMPLETIONHANDLER_H_
#define DIRECTCOMPLETIONHANDLER_H_

#include "CompletionHandler.h"

namespace core {

class CompletionHandlerDirectImpl;

}

namespace command {

/**
 * Wrapper class for the completion handler, implemented in the Core, and implements
 * the completion handler interface. This is used to notify the Core of command terminations. 
 */
class DirectCompletionHandler : public CompletionHandler {
	
public:

	/**
	 * Constructor. Takes a direct CompletionHandler as argument.
	 */
	DirectCompletionHandler(core::CompletionHandlerDirectImpl* proxy);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DirectCompletionHandler();
	
	/**
	 * Notifies the Core of command successfully being executed.
	 */
	virtual void commandSuccess(unsigned long requestID, unsigned long commandID);

	/**
	 * Notifies the Core that a problem occured during the execution and that the
	 * command therefore failed.
	 */
	virtual void commandFailure(unsigned long requestID, unsigned long commandID, const std::string& message);
	
	/**
	 * Notifies the Core of property successfully being set.
	 */
	virtual void propertySuccess(unsigned long requestID, unsigned long propertyID);
	
	/**
	 * Notifies the Core that a problem occured during the set and that the
	 * property therefore failed.
	 */
	virtual void propertyFailure(unsigned long requestID, unsigned long propertyID, const std::string& message);

	/**
	 * Notifies the Core of Duration request successfully being executed.
	 */
	virtual void durationRequestSuccess(unsigned long requestID, unsigned long commandID, double duration);
	
	/**
	 * Notifies the Core that a problem occured during the execution and that the
	 * command therefore failed.
	 */
	virtual void durationRequestFailure(unsigned long requestID, unsigned long commandID, const std::string& message);
	
private:
	core::CompletionHandlerDirectImpl* _proxy;
};

}

#endif /*DIRECTCOMPLETIONHANDLER_H_*/