/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_DIRECTLOGACCESSOR_H_
#define COMMAND_DIRECTLOGACCESSOR_H_

#include "LogAccessor.h"

namespace dataprovider {

	class LogAccessorDirectImpl;
}

namespace command {

/**
 * Direct implementation class of the log accessor.
 */
class DirectLogAccessor : public LogAccessor {
	
public:
	/**
	 * Constructor
	 */
	DirectLogAccessor(dataprovider::LogAccessorDirectImpl* proxy);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DirectLogAccessor();
	
	/**
	 * Write controller log messages
	 */
	virtual void write(common::LogType type, common::LogLevel level, const std::list<common::LogElement>& lineElements, const std::string & imageName, unsigned long commandID, unsigned long servantID, bool writeLogFile, unsigned long requestID);


	/**
	 * Write "only text" log from Command line and both ascii and nexus writer.
	 */
	virtual void write(common::LogType type, common::LogLevel level, const std::string & message, bool writeLogFile);

	/**
	 * Write image data ready log message
	 */
	virtual void imageDataReady(const std::string & image);

private:
	dataprovider::LogAccessorDirectImpl* _proxy;
};

}

#endif /*COMMAND_DIRECTLOGACCESSOR_H_*/
