/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectPropertyAccessor.h"
#include "DatabaseIDKeeper.h"
#include <dataprovider/transportlayer/dataprovider/PropertyAccessorDirectImpl.h>
#include <iostream>

using namespace dataprovider;
using namespace std;
using namespace common;

namespace command {

DirectPropertyAccessor::DirectPropertyAccessor(dataprovider::PropertyAccessorDirectImpl* proxy)
: _proxy(proxy) {
}

DirectPropertyAccessor::~DirectPropertyAccessor() {
}

vector<common::PropertyDescriptor> DirectPropertyAccessor::getControllerProperties(unsigned long controllerID) const {

	try {
		return _proxy->getControllerProperties(DatabaseIDKeeper::getID(), controllerID);

	} catch (PropertyListCreationException& e) {
		throw;
	} catch (PropertyAccessorDirectImpl::NoSuchServantException) {
		throw NoSuchServantException(controllerID);
	}

}

std::vector<common::PropertyDescriptor> DirectPropertyAccessor::getDriverProperties(unsigned long driverID) const {

	try {
		return _proxy->getDriverProperties(DatabaseIDKeeper::getID(), driverID);

	} catch (PropertyListCreationException& e) {
		throw;
	} catch (PropertyAccessorDirectImpl::NoSuchServantException) {
		throw NoSuchServantException(driverID);
	}

}

unsigned long DirectPropertyAccessor::getPropertyID(unsigned long servantID, const std::string& propertyName) const {

	try {
		return _proxy->getPropertyID(DatabaseIDKeeper::getID(), servantID, propertyName);

	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyName);

	}
}

unsigned long DirectPropertyAccessor::addPropertyToServant(unsigned long servantID, const std::string& propertyName, const std::string& propertyType, bool spy, bool saveBox, bool & added) const {

	try {
		return _proxy->addPropertyToServant(DatabaseIDKeeper::getID(), servantID, propertyName, propertyType, spy, saveBox, added);

	} catch (PropertyAccessorDirectImpl::NoSuchServantException & e) {
		throw NoSuchServantException(servantID);

	}
}

unsigned long DirectPropertyAccessor::addDynamicPropertyToServant(unsigned long servantID, const std::string& propertyName, const std::string& propertyType, bool spy) const {

	try {
		return _proxy->addDynamicPropertyToServant(DatabaseIDKeeper::getID(), servantID, propertyName, propertyType, spy);

	} catch (PropertyAccessorDirectImpl::NoSuchServantException & e) {
		throw NoSuchServantException(servantID);

	}
}

void DirectPropertyAccessor::removePropertyFromServant(unsigned long servantID, unsigned long propertyID) const {

	try {
		_proxy->removePropertyFromServant(DatabaseIDKeeper::getID(), servantID, propertyID);

	} catch (PropertyAccessorDirectImpl::NoSuchServantException & e) {
		throw NoSuchServantException(servantID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException& e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setPropertySettings(unsigned long propertyID, const std::string& propertyFormat, bool save, bool spy) const {

	try {
		_proxy->setPropertySettings(DatabaseIDKeeper::getID(), propertyID, propertyFormat, save, spy);

	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::resizeDynamicProperty(unsigned long propertyID, long size) const {

	try {
		_proxy->resizeDynamicProperty(DatabaseIDKeeper::getID(), propertyID, size);

	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

long DirectPropertyAccessor::getDynamicPropertySize(unsigned long propertyID) const {

	try {
		return _proxy->getDynamicPropertySize(DatabaseIDKeeper::getID(), propertyID);

	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

unsigned long DirectPropertyAccessor::getDynamicPropertyIDAtIndex(unsigned long propertyID, long index, bool resize) const {

	try {
		return _proxy->getDynamicPropertyIDAtIndex(DatabaseIDKeeper::getID(), propertyID, index, resize);

	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::IndexOutOfBoundsException & e) {
		throw IndexOutOfBoundsException(propertyID);
	}
}

long DirectPropertyAccessor::getIndexOfDynamicPropertyID(unsigned long parentPropertyID, unsigned long propertyID) const {
	return _proxy->getIndexOfDynamicPropertyID(DatabaseIDKeeper::getID(), parentPropertyID, propertyID);
}

int8 DirectPropertyAccessor::getInt8Value(unsigned long propertyID) const {

	try {
		return _proxy->getInt8Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

int16 DirectPropertyAccessor::getInt16Value(unsigned long propertyID) const {

	try {
		return _proxy->getInt16Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

uint16 DirectPropertyAccessor::getUint16Value(unsigned long propertyID) const {

	try {
		return _proxy->getUint16Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

int32 DirectPropertyAccessor::getInt32Value(unsigned long propertyID) const {

	try {
		return _proxy->getInt32Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

uint32 DirectPropertyAccessor::getUint32Value(unsigned long propertyID) const {

	try {
		return _proxy->getUint32Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

int64 DirectPropertyAccessor::getInt64Value(unsigned long propertyID) const {

	try {
		return _proxy->getInt64Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

uint64 DirectPropertyAccessor::getUint64Value(unsigned long propertyID) const {

	try {
		return _proxy->getUint64Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

float32 DirectPropertyAccessor::getFloat32Value(unsigned long propertyID) const {

	try {
		return _proxy->getFloat32Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

float64 DirectPropertyAccessor::getFloat64Value(unsigned long propertyID) const {

	try {
		return _proxy->getFloat64Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

std::string DirectPropertyAccessor::getStringValue(unsigned long propertyID) const {

	try {
		return _proxy->getStringValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

std::string DirectPropertyAccessor::getBinaryStringValue(unsigned long propertyID) const {

	try {
		vector<int8> value = _proxy->getBinaryStringValue(DatabaseIDKeeper::getID(), propertyID);

		std::string result;
		result.resize(value.size());

		for (int i = 0; i < value.size(); ++i) {
			result[i] = value[i];
		}

		return result;

	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

bool DirectPropertyAccessor::getBooleanValue(unsigned long propertyID) const {

	try {
		return _proxy->getBooleanValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsBooleanValue(unsigned long propertyID, bool value) const {
	try {
		return _proxy->setIcsBooleanValue(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsBooleanValueAndFormattedValue(unsigned long propertyID, bool value, const std::string& formattedValue) const {
	try {
		return _proxy->setIcsBooleanValueAndFormattedValue(DatabaseIDKeeper::getID(), propertyID, value, formattedValue);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::getIcsBooleanValue(unsigned long propertyID) const {
	try {
		return _proxy->getIcsBooleanValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsInt16Value(unsigned long propertyID, int16 value) const {
	try {
		return _proxy->setIcsInt16Value(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsInt16ValueAndFormattedValue(unsigned long propertyID, int16 value, const std::string& formattedValue) const {
	try {
		return _proxy->setIcsInt16ValueAndFormattedValue(DatabaseIDKeeper::getID(), propertyID, value, formattedValue);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int16 DirectPropertyAccessor::getIcsInt16Value(unsigned long propertyID) const {
	try {
		return _proxy->getIcsInt16Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsUint16Value(unsigned long propertyID, uint16 value) const {
	try {
		return _proxy->setIcsUint16Value(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsUint16ValueAndFormattedValue(unsigned long propertyID, uint16 value, const std::string& formattedValue) const {
	try {
		return _proxy->setIcsUint16ValueAndFormattedValue(DatabaseIDKeeper::getID(), propertyID, value, formattedValue);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

uint16 DirectPropertyAccessor::getIcsUint16Value(unsigned long propertyID) const {
	try {
		return _proxy->getIcsUint16Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsInt32Value(unsigned long propertyID, int32 value) const {
	try {
		return _proxy->setIcsInt32Value(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsInt32ValueAndFormattedValue(unsigned long propertyID, int32 value, const std::string& formattedValue) const {
	try {
		return _proxy->setIcsInt32ValueAndFormattedValue(DatabaseIDKeeper::getID(), propertyID, value, formattedValue);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int32 DirectPropertyAccessor::getIcsInt32Value(unsigned long propertyID) const {
	try {
		return _proxy->getIcsInt32Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsUint32Value(unsigned long propertyID, uint32 value) const {
	try {
		return _proxy->setIcsUint32Value(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsUint32ValueAndFormattedValue(unsigned long propertyID, uint32 value, const std::string& formattedValue) const {
	try {
		return _proxy->setIcsUint32ValueAndFormattedValue(DatabaseIDKeeper::getID(), propertyID, value, formattedValue);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

uint32 DirectPropertyAccessor::getIcsUint32Value(unsigned long propertyID) const {
	try {
		return _proxy->getIcsUint32Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsInt64Value(unsigned long propertyID, int64 value) const {
	try {
		return _proxy->setIcsInt64Value(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsInt64ValueAndFormattedValue(unsigned long propertyID, int64 value, const std::string& formattedValue) const {
	try {
		return _proxy->setIcsInt64ValueAndFormattedValue(DatabaseIDKeeper::getID(), propertyID, value, formattedValue);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int64 DirectPropertyAccessor::getIcsInt64Value(unsigned long propertyID) const {
	try {
		return _proxy->getIcsInt64Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsUint64Value(unsigned long propertyID, uint64 value) const {
	try {
		return _proxy->setIcsUint64Value(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsUint64ValueAndFormattedValue(unsigned long propertyID, uint64 value, const std::string& formattedValue) const {
	try {
		return _proxy->setIcsUint64ValueAndFormattedValue(DatabaseIDKeeper::getID(), propertyID, value, formattedValue);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

uint64 DirectPropertyAccessor::getIcsUint64Value(unsigned long propertyID) const {
	try {
		return _proxy->getIcsUint64Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsFloat32Value(unsigned long propertyID, float32 value) const {
	try {
		return _proxy->setIcsFloat32Value(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsFloat32ValueAndFormattedValue(unsigned long propertyID, float32 value, const std::string& formattedValue) const {
	try {
		return _proxy->setIcsFloat32ValueAndFormattedValue(DatabaseIDKeeper::getID(), propertyID, value, formattedValue);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

float32 DirectPropertyAccessor::getIcsFloat32Value(unsigned long propertyID) const {
	try {
		return _proxy->getIcsFloat32Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsFloat64Value(unsigned long propertyID, float64 value) const {
	try {
		return _proxy->setIcsFloat64Value(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsFloat64ValueAndFormattedValue(unsigned long propertyID, float64 value, const std::string& formattedValue) const {
	try {
		return _proxy->setIcsFloat64ValueAndFormattedValue(DatabaseIDKeeper::getID(), propertyID, value, formattedValue);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

float64 DirectPropertyAccessor::getIcsFloat64Value(unsigned long propertyID) const {
	try {
		return _proxy->getIcsFloat64Value(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsStringValue(unsigned long propertyID, const std::string& value) const {
	try {
		return _proxy->setIcsStringValue(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::setIcsStringValueAndFormattedValue(unsigned long propertyID, const std::string& value, const std::string& formattedValue) const {
	try {
		return _proxy->setIcsStringValueAndFormattedValue(DatabaseIDKeeper::getID(), propertyID, value, formattedValue);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

std::string DirectPropertyAccessor::getIcsStringValue(unsigned long propertyID) const {
	try {
		return _proxy->getIcsStringValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsInt16ArrayValue(unsigned long propertyID, int16* value) const {
	try {
		_proxy->setIcsInt16ArrayValue(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int16* DirectPropertyAccessor::getIcsInt16ArrayValue(unsigned long propertyID) const {
	try {
		return _proxy->getIcsInt16ArrayValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsUint16ArrayValue(unsigned long propertyID, uint16* value) const {
	try {
		_proxy->setIcsUint16ArrayValue(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

uint16* DirectPropertyAccessor::getIcsUint16ArrayValue(unsigned long propertyID) const {
	try {
		return _proxy->getIcsUint16ArrayValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsInt32ArrayValue(unsigned long propertyID, int32* value) const {
	try {
		_proxy->setIcsInt32ArrayValue(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int32* DirectPropertyAccessor::getIcsInt32ArrayValue(unsigned long propertyID) const {
	try {
		return _proxy->getIcsInt32ArrayValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsUint32ArrayValue(unsigned long propertyID, uint32* value) const {
	try {
		_proxy->setIcsUint32ArrayValue(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

uint32* DirectPropertyAccessor::getIcsUint32ArrayValue(unsigned long propertyID) const {
	try {
		return _proxy->getIcsUint32ArrayValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsInt64ArrayValue(unsigned long propertyID, int64* value) const {
	try {
		_proxy->setIcsInt64ArrayValue(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int64* DirectPropertyAccessor::getIcsInt64ArrayValue(unsigned long propertyID) const {
	try {
		return _proxy->getIcsInt64ArrayValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsUint64ArrayValue(unsigned long propertyID, uint64* value) const {
	try {
		_proxy->setIcsUint64ArrayValue(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

uint64* DirectPropertyAccessor::getIcsUint64ArrayValue(unsigned long propertyID) const {
	try {
		return _proxy->getIcsUint64ArrayValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsFloat32ArrayValue(unsigned long propertyID, float32* value) const {
	try {
		_proxy->setIcsFloat32ArrayValue(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

float32* DirectPropertyAccessor::getIcsFloat32ArrayValue(unsigned long propertyID) const {
	try {
		return _proxy->getIcsFloat32ArrayValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsFloat64ArrayValue(unsigned long propertyID, float64* value) const {
	try {
		_proxy->setIcsFloat64ArrayValue(DatabaseIDKeeper::getID(), propertyID, value);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

float64* DirectPropertyAccessor::getIcsFloat64ArrayValue(unsigned long propertyID) const {
	try {
		return _proxy->getIcsFloat64ArrayValue(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsInt16ArraySize(unsigned long propertyID, int32 size) const {
	try {
		_proxy->setIcsInt16ArraySize(DatabaseIDKeeper::getID(), propertyID, size);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int32 DirectPropertyAccessor::getIcsInt16ArraySize(unsigned long propertyID) const {
	try {
		return _proxy->getIcsInt16ArraySize(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsUint16ArraySize(unsigned long propertyID, int32 size) const {
	try {
		_proxy->setIcsUint16ArraySize(DatabaseIDKeeper::getID(), propertyID, size);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int32 DirectPropertyAccessor::getIcsUint16ArraySize(unsigned long propertyID) const {
	try {
		return _proxy->getIcsUint16ArraySize(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsInt32ArraySize(unsigned long propertyID, int32 size) const {
	try {
		_proxy->setIcsInt32ArraySize(DatabaseIDKeeper::getID(), propertyID, size);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int32 DirectPropertyAccessor::getIcsInt32ArraySize(unsigned long propertyID) const {
	try {
		return _proxy->getIcsInt32ArraySize(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsUint32ArraySize(unsigned long propertyID, int32 size) const {
	try {
		_proxy->setIcsUint32ArraySize(DatabaseIDKeeper::getID(), propertyID, size);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int32 DirectPropertyAccessor::getIcsUint32ArraySize(unsigned long propertyID) const {
	try {
		return _proxy->getIcsUint32ArraySize(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsInt64ArraySize(unsigned long propertyID, int32 size) const {
	try {
		_proxy->setIcsInt64ArraySize(DatabaseIDKeeper::getID(), propertyID, size);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int32 DirectPropertyAccessor::getIcsInt64ArraySize(unsigned long propertyID) const {
	try {
		return _proxy->getIcsInt64ArraySize(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsUint64ArraySize(unsigned long propertyID, int32 size) const {
	try {
		_proxy->setIcsUint64ArraySize(DatabaseIDKeeper::getID(), propertyID, size);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int32 DirectPropertyAccessor::getIcsUint64ArraySize(unsigned long propertyID) const {
	try {
		return _proxy->getIcsUint64ArraySize(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsFloat32ArraySize(unsigned long propertyID, int32 size) const {
	try {
		_proxy->setIcsFloat32ArraySize(DatabaseIDKeeper::getID(), propertyID, size);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int32 DirectPropertyAccessor::getIcsFloat32ArraySize(unsigned long propertyID) const {
	try {
		return _proxy->getIcsFloat32ArraySize(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::setIcsFloat64ArraySize(unsigned long propertyID, int32 size) const {
	try {
		_proxy->setIcsFloat64ArraySize(DatabaseIDKeeper::getID(), propertyID, size);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

int32 DirectPropertyAccessor::getIcsFloat64ArraySize(unsigned long propertyID) const {
	try {
		return _proxy->getIcsFloat64ArraySize(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::updateIcsProperty(unsigned long propertyID) const {
	try {
		return _proxy->updateIcsProperty(DatabaseIDKeeper::getID(), propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

void DirectPropertyAccessor::importArrayData(unsigned long propertyID, const std::string& fileName, int8 type) {

}

bool DirectPropertyAccessor::isScannableProperty(const std::string & controllerType, const std::string & wantedPropertyName) const {
	return _proxy->isScannableProperty(DatabaseIDKeeper::getID(), controllerType, wantedPropertyName);
}

PropertyAccessor::PropertyInfoItem DirectPropertyAccessor::getPropertyInfoItem(const std::string & controllerType, const std::string & wantedPropertyName) const {

	PropertyAccessor::PropertyInfoItem result;
	try {
		dataprovider::Database::PropertyInfoItem item = _proxy->getPropertyInfoItem(DatabaseIDKeeper::getID(), controllerType, wantedPropertyName);
		result.actualPropertyName = item.actualPropertyName;
		result.setpointPropertyName = wantedPropertyName;
		result.offsetPropertyName = item.offsetPropertyName;
		result.shortPropertyName = item.aliasPropertyName;
		result.isScan = item.isScan;
		result.isSpy = item.isSpy;
		result.isDynamic = item.isDynamic;

	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(wantedPropertyName);
	}

	return result;
}

std::vector<PropertyAccessor::PropertyInfoItem> DirectPropertyAccessor::getPropertyInfoItems(const std::string & controllerType) const {

	vector<PropertyAccessor::PropertyInfoItem> result;

	vector<Database::PropertyInfoItem> items = _proxy->getPropertyInfoItems(DatabaseIDKeeper::getID(), controllerType);

	for (auto i : items) {
		PropertyAccessor::PropertyInfoItem item;
		item.actualPropertyName = i.actualPropertyName;
		item.setpointPropertyName = i.setpointPropertyName;
		item.offsetPropertyName = i.offsetPropertyName;
		item.shortPropertyName = i.aliasPropertyName;
		item.isScan = i.isScan;
		item.isSpy = i.isSpy;
		item.isDynamic = i.isDynamic;

		result.push_back(item);
	}

	return result;
}

void DirectPropertyAccessor::addPropertyInfoItem(const std::string & controllerType, const std::string & wantedPropertyName, const std::string & actualPropertyName, const std::string & offsetPropertyName, const std::string & shortPropertyName, bool isScan, bool isSpy, bool isDynamic) const {
	_proxy->addPropertyInfoItem(DatabaseIDKeeper::getID(), controllerType, wantedPropertyName, actualPropertyName, offsetPropertyName, shortPropertyName, isScan, isSpy, isDynamic);
}

int32 DirectPropertyAccessor::getNumberOfScannableProperties(const std::string & controllerType) const {
	return _proxy->getNumberOfScannableProperties(DatabaseIDKeeper::getID(), controllerType);
}

void DirectPropertyAccessor::setConstantPropertyUnit(unsigned long propertyID, const std::string & value) const {
	_proxy->setConstantPropertyUnit(DatabaseIDKeeper::getID(), propertyID, value);
}

void DirectPropertyAccessor::setPropertyUnit(unsigned long propertyID, unsigned long unitPropertyID) const {
	_proxy->setPropertyUnit(DatabaseIDKeeper::getID(), propertyID, unitPropertyID);
}

void DirectPropertyAccessor::setEnumeratedValues(unsigned long propertyID, unsigned long valuesPropertyID) const {
	_proxy->setEnumeratedValues(DatabaseIDKeeper::getID(), propertyID, valuesPropertyID);
}

void DirectPropertyAccessor::setEnumeratedLabels(unsigned long propertyID, unsigned long labelsPropertyID) const {
	_proxy->setEnumeratedLabels(DatabaseIDKeeper::getID(), propertyID, labelsPropertyID);
}

void DirectPropertyAccessor::addServantType(unsigned long servantID, const std::string& type, bool atFront) const {
	_proxy->addServantType(DatabaseIDKeeper::getID(), servantID, type, atFront);
}

void DirectPropertyAccessor::setServantInDebug(unsigned long servantID) const {
	_proxy->setServantInDebug(DatabaseIDKeeper::getID(), servantID);
}

}
