/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectServantAccessor.h"
#include "DatabaseIDKeeper.h"
#include <dataprovider/transportlayer/dataprovider/ServantAccessorDirectImpl.h>
#include <iostream>
#include <common/utils/ChildServantDescriptor.h>
#include <common/utils/ServantDescriptor.h>

using namespace std;
using namespace common;
using namespace dataprovider;

namespace command {

DirectServantAccessor::DirectServantAccessor(dataprovider::ServantAccessorDirectImpl* proxy) :
	_proxy(proxy) {
}

DirectServantAccessor::~DirectServantAccessor() {
}

std::vector<common::ServantDescriptor> DirectServantAccessor::getControllers() const {
	return _proxy->getControllers(DatabaseIDKeeper::getID());
}

std::vector<common::ServantDescriptor> DirectServantAccessor::getDrivers() const {
	return _proxy->getDrivers(DatabaseIDKeeper::getID());
}

std::vector<common::ChildServantDescriptor> DirectServantAccessor::getControllerChildServants(unsigned long controllerID) const {
	return _proxy->getChildServants(DatabaseIDKeeper::getID(), controllerID);
}

std::vector<common::ChildServantDescriptor> DirectServantAccessor::getDriverChildServants(unsigned long driverID) const {
	return _proxy->getChildServants(DatabaseIDKeeper::getID(), driverID);
}

bool DirectServantAccessor::servantExists(unsigned long servantID) const {
	return _proxy->servantExists(DatabaseIDKeeper::getID(), servantID);
}

ServantAccessor::ServantType DirectServantAccessor::getServantType(unsigned long servantID) const {

	try {
		return (ServantAccessor::ServantType)_proxy->getServantType(DatabaseIDKeeper::getID(), servantID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

std::string DirectServantAccessor::getControllerType(unsigned long controllerID) const {

	try {
		return _proxy->getControllerType(DatabaseIDKeeper::getID(), controllerID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(controllerID);
	}
}

common::DriverMode DirectServantAccessor::getMode(unsigned long driverID) const {

	try {
		return _proxy->getMode(DatabaseIDKeeper::getID(), driverID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(driverID);
	}
}

unsigned long DirectServantAccessor::getDriverParentID(unsigned long driverID) const {

	try {
		return _proxy->getDriverParentID(DatabaseIDKeeper::getID(), driverID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(driverID);
	}
}

unsigned long DirectServantAccessor::getProtocolIDForDriver(unsigned long driverID) const {

	try {
		return _proxy->getProtocolIDForDriver(DatabaseIDKeeper::getID(), driverID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(driverID);
	}
}

unsigned long DirectServantAccessor::getEnabledOrUniqueServantID(unsigned long parentID, const std::string& nServantName, long childIndex) const {

	try {
		return _proxy->getEnabledOrUniqueServantID(DatabaseIDKeeper::getID(), parentID, nServantName, childIndex);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID, nServantName, childIndex);
	} catch (ServantAccessorDirectImpl::ServantDisabledException& e) {
		throw ServantDisabledException(parentID, nServantName, childIndex);
	} catch (ServantAccessorDirectImpl::IndexOutOfBoundsException& e) {
		throw IndexOutOfBoundsException(e.getSize());
	}
}

std::vector<unsigned long> DirectServantAccessor::getServantIDs(unsigned long parentID, const std::string& nServantName, long childIndex) const {

	try {
		return _proxy->getServantIDs(DatabaseIDKeeper::getID(), parentID, nServantName, childIndex);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID, nServantName, childIndex);
	}
}

std::string DirectServantAccessor::getServantName(unsigned long servantID) {

	try {
		return _proxy->getServantName(DatabaseIDKeeper::getID(), servantID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

void DirectServantAccessor::addServant(unsigned long parentID, const std::string& nServantName, bool isList) const {

	try {
		_proxy->addServant(DatabaseIDKeeper::getID(), parentID, nServantName, isList);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID, nServantName, -1);
	}
}

long DirectServantAccessor::getServantListSize(unsigned long parentID, const std::string& nServantName) const {

	try {
		return _proxy->getServantListSize(DatabaseIDKeeper::getID(), parentID, nServantName);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID, nServantName, -1);
	}
}

void DirectServantAccessor::resizeServantList(unsigned long parentID, const std::string& nServantName, long size) const {
	try {
		_proxy->resizeServantList(DatabaseIDKeeper::getID(), parentID, nServantName, size);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID, nServantName, -1);
	}
}

void DirectServantAccessor::assignServant(unsigned long parentID, const std::string& nServantName, long childIndex, unsigned long newParentID, const std::string& newNServantName, long newChildIndex) {

	try {
		_proxy->assignServantFromParent(DatabaseIDKeeper::getID(), parentID, nServantName, childIndex, newParentID, newNServantName, newChildIndex);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID);

	} catch (ServantAccessorDirectImpl::ServantAssignmentException& e) {
		throw ServantAssignmentException(string("unable to assign servant '") + nServantName + "' of " + what(parentID) + (childIndex != -1 ? " at index " + to<string>(childIndex) : ""));
	}
}

void DirectServantAccessor::assignServant(unsigned long parentID, const std::string& nServantName, long childIndex, unsigned long newServantID) {
	try {
		_proxy->assignServant(DatabaseIDKeeper::getID(), parentID, nServantName, childIndex, newServantID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID);

	} catch (ServantAccessorDirectImpl::ServantAssignmentException& e) {
		throw ServantAssignmentException(string("unable to assign servant '") + nServantName + "' of " + what(parentID) + (childIndex != -1 ? " at index " + to<string>(childIndex) : ""));

	} catch (...) {
		out << "Exception" << eol;
		throw;
	}
}

void DirectServantAccessor::assignServant(unsigned long servantID, bool newServant, bool localServant) {

	try {
		_proxy->assignServantWithID(DatabaseIDKeeper::getID(), servantID, newServant, localServant);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

unsigned long DirectServantAccessor::removeServantFromParent(unsigned long parentID, const std::string& nName, long index) {

	try {
		return _proxy->removeServantFromParent(DatabaseIDKeeper::getID(), parentID, nName, index);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID);

	} catch (ServantAccessorDirectImpl::NoSuchChildException& e) {
		throw NoSuchServantException(parentID, nName, index);

	} catch (ServantAccessorDirectImpl::NoSuchServantChildNameException& e) {
		throw NoSuchServantChildNameException(nName);
	}
}

void DirectServantAccessor::removeServant(unsigned long servantID) {

	try {
		_proxy->removeServant(DatabaseIDKeeper::getID(), servantID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

void DirectServantAccessor::setServantInitialised(unsigned long servantID, bool initialised) const {

	try {
		_proxy->setServantInitialised(DatabaseIDKeeper::getID(), servantID, initialised);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

bool DirectServantAccessor::isServantInitialised(unsigned long servantID) const {

	try {
		return _proxy->isServantInitialised(DatabaseIDKeeper::getID(), servantID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

bool DirectServantAccessor::isServantInitialised(unsigned long parentID, const std::string& nName, long childIndex) const {

	try {
		return _proxy->isServantInitialisedFromParent(DatabaseIDKeeper::getID(), parentID, nName, childIndex);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID, nName, childIndex);
	}
}

void DirectServantAccessor::resetElapsedSimulationTime(unsigned long driverID) const {

	try {
		_proxy->resetElapsedSimulationTime(DatabaseIDKeeper::getID(), driverID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(driverID);
	}
}

double DirectServantAccessor::getElapsedSimulationTimeMs(unsigned long driverID) const {

	try {
		return _proxy->getElapsedSimulationTimeMs(DatabaseIDKeeper::getID(), driverID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(driverID);
	}
}

void DirectServantAccessor::enableServant(unsigned long servantID, bool all) {

	try {
		_proxy->enableServant(DatabaseIDKeeper::getID(), servantID, all);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	} catch (ServantAccessorDirectImpl::ServantEnableException& e) {
		throw ServantEnableException(string("cannot enable ") + what(servantID));
	}
}

void DirectServantAccessor::enableServant(unsigned long parentID, const std::string& nName, long childIndex) {

	try {
		_proxy->enableServantFromParent(DatabaseIDKeeper::getID(), parentID, nName, childIndex);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID, nName, childIndex);
	} catch (ServantAccessorDirectImpl::ServantEnableException& e) {
		throw ServantEnableException(string("cannot enable servant '") + nName + "' of " + what(parentID) + (childIndex != -1 ? " at index " + to<string>(childIndex) : ""));
	}
}

void DirectServantAccessor::disableServant(unsigned long servantID, bool all) {

	try {
		_proxy->disableServant(DatabaseIDKeeper::getID(), servantID, all);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

void DirectServantAccessor::disableServant(unsigned long parentID, const std::string& nName, long childIndex) {

	try {
		_proxy->disableServantFromParent(DatabaseIDKeeper::getID(), parentID, nName, childIndex);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID, nName, childIndex);
	}
}

bool DirectServantAccessor::isServantEnabled(unsigned long servantID) {

	try {
		return _proxy->isServantEnabled(DatabaseIDKeeper::getID(), servantID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

bool DirectServantAccessor::isServantEnabled(unsigned long parentID, const std::string& nName, long childIndex) {

	try {
		return _proxy->isServantEnabledFromParent(DatabaseIDKeeper::getID(), parentID, nName, childIndex);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(parentID, nName, childIndex);
	}
}

std::vector<unsigned long> DirectServantAccessor::configureServant(unsigned long servantID, const std::string& configurationName) {

	try {
		return _proxy->configureServant(DatabaseIDKeeper::getID(), servantID, configurationName);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);

	} catch (ServantAccessorDirectImpl::CannotConfigureServantException& e) {
		throw CannotConfigureServantException(servantID);
	}
}

std::string DirectServantAccessor::what(unsigned long elementID) {
	return _proxy->what(DatabaseIDKeeper::getID(), elementID);
}

void DirectServantAccessor::setClientType(unsigned long servantID, const std::string& clientType) {
	_proxy->setClientType(DatabaseIDKeeper::getID(), servantID, clientType);
}

bool DirectServantAccessor::isServantPausable(unsigned long servantID) {
	return _proxy->isServantPausable(DatabaseIDKeeper::getID(), servantID);
}

unsigned long DirectServantAccessor::getServantID(const std::string& name) {
	return _proxy->getServantID(DatabaseIDKeeper::getID(), name);
}

void DirectServantAccessor::setServantVisible(unsigned long servantID, bool value) {
	_proxy->setServantVisible(DatabaseIDKeeper::getID(), servantID, value);
}

void DirectServantAccessor::setServantCommand(unsigned long servantID, bool value) {
	_proxy->setServantCommand(DatabaseIDKeeper::getID(), servantID, value);
}

void DirectServantAccessor::setServantFavourite(unsigned long servantID, bool value) {
	_proxy->setServantFavourite(DatabaseIDKeeper::getID(), servantID, value);
}

void DirectServantAccessor::enableConfigurationChanged() {
	_proxy->enableConfigurationChanged(DatabaseIDKeeper::getID());
}

void DirectServantAccessor::disableConfigurationChanged() {
	_proxy->disableConfigurationChanged(DatabaseIDKeeper::getID());
}

void DirectServantAccessor::configurationChanged() {
	_proxy->configurationChanged(DatabaseIDKeeper::getID());
}

void DirectServantAccessor::setServantFamily(unsigned long servantID, const std::string& familyName, const std::string& subfamilyName) {
	_proxy->setServantFamily(DatabaseIDKeeper::getID(), servantID, familyName, subfamilyName);
}

std::string DirectServantAccessor::getServantFamily(unsigned long servantID) {
	return _proxy->getServantFamily(DatabaseIDKeeper::getID(), servantID);
}

std::string DirectServantAccessor::getServantSubfamily(unsigned long servantID) {
	return _proxy->getServantSubfamily(DatabaseIDKeeper::getID(), servantID);
}

bool DirectServantAccessor::isServantFavourite(unsigned long servantID) {
	return _proxy->isServantFavourite(DatabaseIDKeeper::getID(), servantID);
}

}
