/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_LOGACCESSOR_H_
#define COMMAND_LOGACCESSOR_H_

#include <command/exception/TraceableException.h>
#include <common/base/CommonTypes.h>
#include <list>
#include "common/base/LogElement.h"

namespace command {


/**
 * Class defining the log accessor interface.
 */
class LogAccessor {
	
public:
	/**
	 * Constructor
	 */
	LogAccessor();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~LogAccessor();

	virtual void write(common::LogType type, common::LogLevel level, const std::list<common::LogElement>& lineElements, const std::string & imageName, unsigned long commandID, unsigned long servantID, bool writeLogFile, unsigned long requestID) = 0;

	virtual void write(common::LogType type, common::LogLevel level, const std::string & message, bool writeLogFile) = 0;

	virtual void imageDataReady(const std::string & image) = 0;

};

}

#endif /*COMMAND_LOGACCESSOR_H_*/
