/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PropertyAccessor.h"
#include <common/base/Convert.h>

using namespace common;
using namespace std;

namespace command {
	
PropertyAccessor::PropertyListCreationException::PropertyListCreationException(const string& type, unsigned long ID) :
	TraceableException("command::PropertyAccessor::PropertyListCreationException", string("server failed to create list of " + type + " properties for ID " + common::to<string>(ID))) {
}

PropertyAccessor::NoSuchServantException::NoSuchServantException(unsigned long servantID)
: TraceableException("command::PropertyAccessor::NoSuchServantException", string("unable to find servant <") + to<string>(servantID) + ">") {
}

PropertyAccessor::NoSuchPropertyException::NoSuchPropertyException(unsigned long propertyID)
: TraceableException("command::PropertyAccessor::NoSuchPropertyException", string("unable to find property <") + to<string>(propertyID) + ">") {
}

PropertyAccessor::NoSuchPropertyException::NoSuchPropertyException(const std::string& propertyName)
: TraceableException("command::PropertyAccessor::NoSuchPropertyException", string("unable to find property '") + propertyName + "'") {
}

PropertyAccessor::BadPropertyTypeException::BadPropertyTypeException(unsigned long propertyID)
: TraceableException("command::PropertyAccessor::BadPropertyTypeException", string("requesting bad type for property <") + to<std::string>(propertyID) + ">") {
}

PropertyAccessor::IndexOutOfBoundsException::IndexOutOfBoundsException(unsigned long propertyID)
: TraceableException("command::PropertyAccessor::IndexOutOfBoundsException", string("requesting bad index for array property <") + to<std::string>(propertyID) + ">") {
}

PropertyAccessor::FileNotFoundException::FileNotFoundException(const std::string& fileName)
: TraceableException("command::PropertyAccessor::FileNotFoundException", string("File not found : ") + fileName) {
}

PropertyAccessor::NotResizableException::NotResizableException(unsigned long propertyID)
: TraceableException("command::PropertyAccessor::NotResizableException", string("array property <") + to<std::string>(propertyID) + "> is not resizable") {
}

PropertyAccessor::NullPropertyReferenceException::NullPropertyReferenceException(unsigned long propertyID)
: TraceableException("command::PropertyAccessor::NullPropertyReferenceException", string("property <") + to<std::string>(propertyID) + "> is a null reference") {
}

PropertyAccessor::NotConfiguredException::NotConfiguredException(unsigned long propertyID)
: TraceableException("command::PropertyAccessor::NotConfiguredException", string("property <") + to<std::string>(propertyID) + "> is not configured") {
}

PropertyAccessor::PropertyAccessor() {
}
	
PropertyAccessor::~PropertyAccessor() {
}
	
}