/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ServantAccessor.h"
#include <common/base/Convert.h>

using namespace command;
using namespace common;
using namespace std;

ServantAccessor::ServantsListCreationException::ServantsListCreationException(const string& type) :
	TraceableException("command::ServantAccessor::ServantsListCreationException", string("server failed to create list of " + type + " servants")) {
}

ServantAccessor::ChildServantsListCreationException::ChildServantsListCreationException(const string& type) :
	TraceableException("command::ServantAccessor::ChildServantsListCreationException", string("server failed to create list of " + type + " child servants")) {
}

ServantAccessor::NoSuchServantException::NoSuchServantException(unsigned long servantID) :
	TraceableException("command::ServantAccessor::NoSuchServantException", string("server failed to find servant with ID <" + to<string>(servantID) + ">")) {
}

ServantAccessor::NoSuchServantException::NoSuchServantException(unsigned long parentID, const std::string& nServantName, long childIndex) :
	TraceableException("command::ServantAccessor::NoSuchServantException", string("server failed to find servant '") + nServantName + "' with parent ID <" + to<string>(parentID) + ">" + (childIndex != -1 ? (" at index " + to<string>(childIndex)) : "")) {
}

ServantAccessor::ServantAssignmentException::ServantAssignmentException(const std::string& message) :
	TraceableException("command::ServantAccessor::ServantAssignmentException", message) {
}

ServantAccessor::IndexOutOfBoundsException::IndexOutOfBoundsException(long size) :
	TraceableException("command::ServantAccessor::IndexOutOfBoundsException", string("index must be between 0 and ") + to<string>(size)),
	_size(size) {
}

ServantAccessor::CannotConfigureServantException::CannotConfigureServantException(unsigned long servantID) :
	TraceableException("command::ServantAccessor::CannotConfigureServantException", string("server failed to configure servant with ID <" + to<string>(servantID) + ">")) {
}

long ServantAccessor::IndexOutOfBoundsException::getSize() const {
	return _size;
}

ServantAccessor::ServantDisabledException::ServantDisabledException(unsigned long parentID, const std::string nServantName, long childIndex) :
	TraceableException("command::ServantAccessor::ServantDisabledException", string("server found servant '") + nServantName + "' disabled with parent ID <" + to<string>(parentID) + ">" + (childIndex != -1 ? (" at index " + to<string>(childIndex)) : "")) {
}

ServantAccessor::ServantEnableException::ServantEnableException(const std::string& message) :
	TraceableException("command::ServantAccessor::ServantEnableException", message) {
}

ServantAccessor::NoSuchServantChildNameException::NoSuchServantChildNameException(const std::string& nName) :
	TraceableException("command::ServantAccessor::NoSuchServantChildNameException", string("server failed to find servant child with name '" + nName + "'")) {
}

ServantAccessor::ServantAccessor() {
}

ServantAccessor::~ServantAccessor() {
}