/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_APPLICATION_H_
#define COMMON_APPLICATION_H_

#include <iostream>
#include <vector>
#include <signal.h>

namespace common {

/**
 * Interface for the three applications: Core, CommandProcessor and DataProvider. 
 */
class Application {

public:
	/**
	 * Constructor.
	 */
	Application();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~Application();

	/**
	 * Initialises the application.
	 */
	virtual void init(int argc, char* argv[]) = 0;
	
	/**
	 * Runs the application.
	 */
	virtual void run() = 0;
	
	/**
	 * Resets the application.
	 */
	virtual void reset() = 0;

	/**
	 * Executes the command.
	 */
	virtual void execute(const std::string& commandName, const std::vector<std::string>& args) = 0;
	
	/**
	 * Quits the application.
	 */
	virtual void quit(const std::vector<std::string>& args, bool isRemote) = 0;
	
	/**
	 * Const string to represent the config path.
	 */
	const static std::string CONFIG_PATH;
	
	/**
	 * Attributes.
	 */
	const static std::string CLEAN;
	const static std::string STDOUT;
	const static std::string REMOTE;
	
	/**
	 * Sets the signal handlers.
	 */
	static void setSignalHandlers(sighandler_t handler);
	
	/**
	 * Resets the signal handlers.
	 */
	static void resetSignalHandlers();
};

}

#endif /*COMMON_APPLICATION_H_*/