/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ConsoleCommandLauncher.h"
#include "Application.h"
#include <common/base/Exception.h>
#include <common/base/StringUtilities.h>

using namespace std;

namespace common {

const std::string ConsoleCommandLauncher::QUIT_COMMAND = "quit";

bool ConsoleCommandLauncher::launchCommand(const std::string& commandLine, Application* application, bool isRemote) {

	bool running = true;
	if (application != 0 && commandLine != "") {

		list<string> tokens = StringUtilities::split(commandLine);

		if (tokens.size() > 0) {
			string command = tokens.front();
			tokens.pop_front();

			vector<string> arguments;

			for (list<string>::const_iterator t = tokens.begin(); t != tokens.end(); t++) {
				arguments.push_back(*t);
			}

			// Reading the command
			try {
				// Test the special "quit" command.
				if (command == QUIT_COMMAND) {
					application->quit(arguments, isRemote);
					running = false;
				} else {
					application->execute(command, arguments);
				}
			} catch (Exception& e) {
				cerr << e.printStack() << endl;
			}

		}
	}

	return running;
}

}
