/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "InputConsole.h"
#include "Application.h"
#include "ConsoleCommandLauncher.h"
#include <common/base/Exception.h>

#include <boost/bind.hpp>
#include <iostream>
#include <cstdio>
#include <sys/epoll.h>

using namespace std;
using namespace boost;

namespace common {

InputConsole::InputConsole(Application * application) :
	_thread(0),
	_running(false),
	_application(application) { 
}

InputConsole::~InputConsole() {
	
	delete _thread;
}

void InputConsole::run() {
	
	_thread = new std::thread(bind(&loop, this));
}

void InputConsole::reset() {
	
	_running = false;
	
	if (_thread) {
		_thread->join();
	}
}

void InputConsole::quit() {

	_running = false;
}

void InputConsole::loop(InputConsole * console) {

	// Interpreting the commands
	console->_running = true;

	char commandLine[COMMAND_LENGTH_MAX];
	int fd = epoll_create(5);

	struct epoll_event event;

	event.events = EPOLLIN;
	event.data.fd = 0;

	epoll_ctl(fd, EPOLL_CTL_ADD, 0, &event);

	// Loop.
	while (console->_running) {

		// Polling stdin by 100ms.
		int res = 0;
		while (res == 0) {
			res = epoll_wait(fd, &event, 1, 100);

			if (!console->_running) {
				return;
			}
		}

		// A line was written.
		size_t size = read(0, commandLine, sizeof(commandLine));

		if (size < COMMAND_LENGTH_MAX) {
			// Overwrite the \n character with the \0 end of string character.
			commandLine[size - 1] = '\0';
			console->_running = ConsoleCommandLauncher::launchCommand(commandLine, console->_application, false);
		} else {
			cerr << "Command cannot exceed " << COMMAND_LENGTH_MAX << " characters!" << endl;
		}
	}
}

}
