/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_INPUTCONSOLE_H_
#define COMMON_INPUTCONSOLE_H_

#include "Console.h"
#include <thread>

namespace common {

class Application;

/**
 * Console class to add to an application to provide a text console. Implemented with a 
 * thread. Forward the command to the Application::execute() method. A special command "quit"
 * terminates the Console.
 */
class InputConsole : public Console {
	
public:
	/**
	 * Constructor with the Application argument.
	 */
	InputConsole(Application * application);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~InputConsole();

	/**
	 * Runs the console.
	 */
	virtual void run();
	
	/**
	 * Waits for the end of the thread.
	 */
	virtual void reset();

	/**
	 * Quits the console.
	 */
	virtual void quit();

private:
	std::thread * _thread;
	bool _running;
	Application * _application;
	
	static void loop(InputConsole * console);
	
	static const int COMMAND_LENGTH_MAX = 200;
};	

}

#endif /*COMMON_INPUTCONSOLE_H_*/
