/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_NOINPUTCONSOLE_H_
#define COMMON_NOINPUTCONSOLE_H_

#include <boost/thread/thread.hpp>
#include <boost/thread/recursive_mutex.hpp>
#include "Console.h"

namespace common {

class Application;

/**
 * Class for console with no input.
 */
class NoInputConsole : public Console {
	
public:
	/**
	 * Constructor.
	 */
	NoInputConsole(Application * application);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~NoInputConsole();

	/**
	 * Runs the console.
	 */
	virtual void run();
	
	/**
	 * Waits for the end of the thread.
	 */
	virtual void reset();

	/**
	 * Quits the console.
	 */
	virtual void quit();
};	

}

#endif /*COMMON_NOINPUTCONSOLE_H_*/