/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*
 *  derived from MultiGaussPeakFinder 
 * F.Cecillon 08-07
 *
 */

#include "CenterOfMassPeakFinder.h"

using namespace blitz;

////////////////////////////////////////////////////////////////////////////
//// constructors
////////////////////////////////////////////////////////////////////////////

CenterOfMassPeakFinder::CenterOfMassPeakFinder(const blitz::Array<double, 1>& y, const blitz::Array<double, 1>& x)  :
	m_vecX(x), m_vecY(y), m_position(0.0), m_amplitude(0.0) {

	calculate();
}

CenterOfMassPeakFinder::CenterOfMassPeakFinder(const blitz::Array<double, 1>& y)  :
	m_vecY(y), m_position(0.0), m_amplitude(0.0) {

	m_vecX.resize(y.size());
	firstIndex i;
	m_vecX = i;

	calculate();
}

////////////////////////////////////////////////////////////////////////////
//// this is the method to find the center of mass
////////////////////////////////////////////////////////////////////////////

double CenterOfMassPeakFinder::getPosition() const {
	return m_position;
}

double CenterOfMassPeakFinder::getAmplitude() const {
	return m_amplitude;
}

void CenterOfMassPeakFinder::calculate() {
	if (m_vecX.size() != m_vecY.size()) {
		throw invalid_argument("CenterOfMassPeakFinder::findPeaks: x size must equal y size!");
	}

	try {
		VerticalMoments moments(m_vecX, m_vecY);
		m_position = moments.mmean(); // get the center of mass of the data (vertical mean)
		m_amplitude = max(m_vecY);

	} catch (const logic_error& error) {
		cerr << error.what() << endl;
		throw domain_error("CenterOfMassPeakFinder::findPeaks: error in peaks calculation");
	}
}
