/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef BZ_BLITZ_H
 #include <blitz/blitz.h>
#endif

#include <blitz/array.h>  

namespace blitz {

template<typename T>
double determinant(const Array<T, 2>& B) 
{
  // this function provides the determinant of matrices for 
  // sizes up to 3;
  int lbFirstDim    = B.lbound(firstDim);
  int lbSecondDim   = B.lbound(secondDim);

  int sizeFirstDim  = B.ubound(firstDim) - lbFirstDim + 1;
  int sizeSecondDim = B.ubound(secondDim) - lbSecondDim + 1;

  // preconditions
  if (sizeFirstDim != sizeSecondDim)
    throw std::invalid_argument("determinant: matrix must be square!");

  if (sizeFirstDim > 3)
    throw std::invalid_argument("determinant: matrix must be 3x3!");

  int i0 = lbFirstDim,  i1 = lbFirstDim  + 1, i2 = lbFirstDim  + 2;
  int j0 = lbSecondDim, j1 = lbSecondDim + 1, j2 = lbSecondDim + 2;

  if (sizeFirstDim == 3 && sizeSecondDim == 3)
  {
    return - B(i0, j2)*B(i1, j1)*B(i2, j0) + B(i0, j1)*B(i1, j2)*B(i2, j0)
           + B(i0, j2)*B(i1, j0)*B(i2, j1) - B(i0, j0)*B(i1, j2)*B(i2, j1)
           - B(i0, j1)*B(i1, j0)*B(i2, j2) + B(i0, j0)*B(i1, j1)*B(i2, j2);
  }
  else
  {
    if (sizeFirstDim == 2 && sizeSecondDim == 2)
      return -(B(i0, j1)*B(i1, j0)) + B(i0, j0)*B(i1, j1);
    else
      if (sizeFirstDim == 1 && sizeSecondDim == 1)
        return B(i0, j0);
  }
  
  return 0;
}

}