/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef BZ_BLITZ_H
 #include <blitz/blitz.h>
#endif

#include <blitz/array.h>  
#include <blitz/array/convolve.h>

namespace blitz {

template<typename T>
Array<T,1> differentiate(const Array<T,1>& data,unsigned int filterLength)
{
int n=data.size();
// make a differentiation of the data by a square filter of length filterLength
// make the convolution kernel
if(filterLength<3) std::cout<<" Problem the filter length should be greater than 3"<<std::endl;
Array<double,1> kernel(filterLength*2); 

for(unsigned int i=0;i<2*filterLength;i++) kernel(i)=(i<filterLength)?1.0:-1.0;
kernel/=filterLength*2;

Array<double,1> result=convolve(data,kernel);

std::cout<<"convolution size="<<result.size()<<" "<<data.size()<<" "<<kernel.size()<<" "<<result(Range(filterLength,n-1+filterLength)).size()<<" "<<filterLength<< std::endl;

return result(Range(filterLength+1,n-filterLength));
}

}