/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DataReadWrite.h"

#include <vector>
#include <fstream>
#include <sstream>
#include <stdio.h>
#include <string.h>

using namespace std;
using namespace blitz;

blitz::Array<double, 1> readDataFile(const std::string& fileName)
{
  char str[20];
  ifstream file(fileName.c_str(), ios::in);
  
  if (!file.is_open())
  {
    cerr << "Error: bad file!" << endl;
    return Array<double, 1>(0);
  }

  // Getting the number of elements
  vector<double> array;
  while (!file.eof())
  {
    file.getline(str, 2000);
    if (strcmp(str, "#end") && strcmp(str, "#begin"))
    {
      istringstream s(str);
      double value;
      s >> value;
      array.push_back(value);
    }
  }
  file.close();
  
  cout << "Size = " << array.size() << endl;
  Array<double, 1> A(array.size());
  
  for (unsigned int i = 0; i < array.size(); ++i)  
    A(i) = array[i];
    
  return A;
}

void writeDataFile(const std::string& fileName, const blitz::Array<double, 1>& array)
{
  ofstream file(fileName.c_str(), ios::out);
  
  for (int i = array.lbound(firstDim); i <= array.ubound(firstDim); ++i)
    file << array(i) << endl;
   
  file.close();
}
