/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <blitz/array.h>
#include <random/uniform.h>
#include <random/normal.h>
#include <iostream>
#include <cmath>
#include <list>
#include <sstream>
#include <cstdlib>
#include <limits>

#include "common/analysislibrary/MultiGaussPeakFinder.h"
#include "DataReadWrite.h"

using namespace std;
using namespace blitz;

int main(int argc, char *argv[]) {

	if (argc < 6) {
		cerr << "usage: gaussgen <samples> <min> <max> <noise> <output> [n]" << endl;
		return 0;
	}

	int samples;
	double min, max, noise;

	istringstream ssamples(argv[1]);
	ssamples >> samples;

	istringstream smin(argv[2]);
	smin >> min;

	istringstream smax(argv[3]);
	smax >> max;

	istringstream sn(argv[4]);
	sn >> noise;

	bool negative = false;
	if (argc > 6 && string(argv[6]) == "n") {
		negative = true;
	}

	Array<double, 1> data(samples);

	double coef = 100;
	double a0, a1, a2, a3;
	a1 = 0;
	a2 = 1;
	a3 = 0;

	if (negative) {
		a0 = -1;
		data = coef;
	} else {
		a0 = 1;
		data = 0;
	}

	ranlib::Uniform<double> uniform;

	double x = min;
	double delta = (max - min) / (samples - 1);

	for (int i = 0; i < samples; ++i) {
		x = min + delta * i;
		data(i) += (a0 * exp(0.5 * (-pow((x - a1) / a2, 2))) + a3 + noise * (0.5 - uniform.random())) * coef;
	}

	writeDataFile(argv[5], data);

	return 1;
}
