/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef BZ_ARRAY_GAUSSJORDAN_H
#define BZ_ARRAY_GAUSSJORDAN_H

#ifndef BZ_ARRAY_H
 #error <blitz/array/myMath.h> must be included after <blitz/array.h>
#endif

namespace blitz {

/*! \file inverseGJ.h 
\brief Compute the inverse of matrices up to 6x6
*/

/// This is a general implementation of the Gauss-Jordan algorithm for matrices inversion.
/// It should not be called directly but rather through the template inverse. 
/// In this template the matrix inversion is explicitly written for matrices un to 3x3, this
/// to optimize the speed of calculation.
/// @param B is the square matrix to invert.
/// Precondition: square matrix.

template<typename T>
Array<T, 2> inverseGJ(const Array<T, 2>& B);

}

#include "GaussJordan.inl"

#endif // BZ_ARRAY_GAUSSJORDAN_H