/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cassert>
#include <cmath>

#ifndef BZ_BLITZ_H
 #include <blitz/blitz.h>
#endif

#include <blitz/array.h> 
#include<math.h>

namespace blitz {

template<typename T>
Array<T,2> inverseGJ(const Array<T,2>& B)
{
  // this function provides the inverse matrix 

  // it the size is not correct, then the determinant is 0.
  // same way, if the determiant is 0, the matrix cannot be inversed.

  // first some dimensions
  int ubFirstDim  = B.ubound(firstDim);
  int lbFirstDim  = B.lbound(firstDim);
  int n           = ubFirstDim - lbFirstDim + 1;
  
  // precondition
  assert(B.ubound(secondDim) - B.lbound(secondDim) + 1 == n);

  Array<T,2> a(B); /// creates the result array from B values (copy because B is const)

  //////////////////////////////////////////////////////////////////////////////
  int i, icol, irow, j, k, l, ll;
  double big, dum, pivinv;

  //int n=a.nrows(); TBR
  //int m=b.ncols(); TBR
  // Vec_INT indxc(n),indxr(n),ipiv(n); TBR
  Array<int, 1> indxc(n), indxr(n), ipiv(n);

  // for (j=0;j<n;j++) ipiv[j]=0; TBR
  ipiv = 0;
  for (i = 0; i < n; ++i) 
  {
    big = 0.0;
    for (j = 0; j < n; ++j)
      if (ipiv(j) != 1)
        for (k = 0; k < n; ++k) 
        {
          if (ipiv(k) == 0) 
          {
            double absa = (a(j, k) > 0) ? a(j, k) : -a(j, k);
            if (absa >= big) 
            {
              big = absa;
              irow = j;
              icol = k;
            }
          }
        }
	++(ipiv(icol));
	
        if (irow != icol) 
        {
          for (l = 0; l < n; ++l) 
          {// swapping
            double tmp = a(irow, l);
            a(irow, l) = a(icol, l);
            a(icol, l) = tmp;			
          }
          //for (l=0;l<n;l++) SWAP(a(irow,l),a(icol,l));
          // for (l=0;l<m;l++) SWAP(b[irow][l],b[icol][l]); TBR
	}
	
        indxr(i) = irow;
	indxc(i) = icol;
	
        // precondition: invertible matrix!
        assert(::fabs(a(icol, icol)) > threshold);
	
        pivinv = 1.0/a(icol, icol);
	a(icol, icol) = 1.0;
	
        for (l = 0; l < n; ++l) a(icol, l) *= pivinv;
	//for (l=0;l<m;l++) b(icol,l) *= pivinv;
	
        for (ll = 0; ll < n; ++ll)
          if (ll != icol) 
          {
            dum=a(ll, icol);
            a(ll, icol)=0.0;
            for (l = 0; l < n; ++l) 
              a(ll, l) -= a(icol, l)*dum;
              // for (l=0;l<m;l++) b[ll][l] -= b[icol][l]*dum; TBR
          }
  }
  for (l = n - 1; l >= 0; --l) 
  {
    if (indxr(l) != indxc(l))
      for (k = 0; k < n; ++k) 
      { // swapping 
        double tmp = a(k, indxr(l));
        a(k, indxr(l)) = a(k, indxc(l));
        a(k, indxc(l)) = tmp;
      }
  // SWAP(a(k,indxr(l)),a(k,indxc(l)));
  }

  return a;
}

}