/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "LinearRegression.h"
//#include "Threshold.h"
//#include "Inverse.h"

#include <string>

using namespace std;

//LinearRegression::LinearRegression()
//{
//}

LinearRegression::LinearRegression(const double * taby ,const double * tabx ,const int NumberOfPoints) 
: m_coef(777.0,777.0)
{
  // Precondition.
  if (NumberOfPoints < 0)
    throw invalid_argument("LinearRegression:: Number of points must be greater than 2 !");

  double SumXiYi = 0;
  double SumXi = 0;
  double SumYi = 0;
  double SumXi2 = 0;
  
  for (int ki = 0; ki<NumberOfPoints;ki++) {
	  SumXiYi = 	SumXiYi + taby[ki]*tabx[ki];
	  SumXi   =   SumXi + tabx[ki];
	  SumYi   =   SumYi + taby[ki];
	  SumXi2  =  SumXi2 + tabx[ki]*tabx[ki];
  }
  
  double n = NumberOfPoints ;
  double det = SumXi2 - (SumXi*SumXi)/n;
  if (det == 0) throw invalid_argument("LinearRegression:: Determinant  is null check your data !");
  
  m_coef.second = (SumXiYi  - (SumXi*SumYi)/n ) / (det);
  
  m_coef.first = (SumYi - (m_coef.second*SumXi) ) / n;
}

std::pair<double, double> LinearRegression::getCoef() const 
{
   if (fabs(m_coef.second - 777.00)<0.001){
	   string error("LinearRegression::coef: cannot access ");
	   throw invalid_argument(error);
   }

	return m_coef ;
}