/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef BZ_BLITZ_H
 #include <blitz/blitz.h>
#endif

#include <blitz/array.h> 
#include <math.h>  

namespace blitz {

/////////////////////////////////////////////////////////////////////////////////
// general moments
/////////////////////////////////////////////////////////////////////////////////

template<typename T>
Array<double,2>  moments(const Array<T,1>& y)
{// call the general moments with x being the indexes of y
int lb=y.lbound(firstDim),ub=y.ubound(firstDim);
Array<T,1> x(ub-lb+1);
firstIndex i;
x=i+lb;
return moments(x,y);
}

template<typename T>
Array<double,2>  moments(const Array<T,1>& x, const Array<T,1>& y)
{ // this function provides the moments for 1D arrays
// the moments are returned as
//	0 - The sum
//	1 - The mean
//	2 - The variance
//	3 - The rms (sqrt(variance))
// this array of seven values is returned in the first dimension for vertical moments
// in the second dimension for horizontal moments	 
  
 
// both arays must have the same dimension 
// SOME ERRORS SHOULD BE THROWN HERE, HOW ?
if(x.size() !=y.size()) std::cout<<"ARRAY do no have the same dimension"<<std::endl;
if(x.size() ==0 ) std::cout<<"ARRAY should have at least one sample "<<std::endl;


// compute the standard error weight , if the value of y is equal to 0,
// then the weigth is equal to 0
 
// compute the sums
double n=(double)y.size();
double sY=sum(y);
double sY2=sum(pow2(y));
double sXY=sum(x*y);
double sX2Y=sum(x*x*y);

// compute the moments
double horizontalMean=sY/n;
double horizontalVariance=sY2/n-horizontalMean*horizontalMean;
double horizontalRMS=std::sqrt(horizontalVariance);
double verticalMean=sXY/sY;
double verticalVariance=sX2Y/sY-verticalMean*verticalMean;
double verticalRMS=std::sqrt(verticalVariance);

// returning the results

Array<double,2> result(2,4);

result = sY,horizontalMean,horizontalVariance,horizontalRMS,
		 sXY,verticalMean,verticalVariance,verticalRMS;

return result;
}

/////////////////////////////////////////////////////////////////////////////////
// vertical moments
/////////////////////////////////////////////////////////////////////////////////

template<typename T>
Array<double,1> verticalMoments(const Array<T,1>& y)
{// call the general moments with x being the indexes of y
int lb=y.lbound(firstDim),ub=y.ubound(firstDim);
Array<T,1> x(ub-lb+1);
firstIndex i;
x=i+lb;
return verticalMoments(x,y);
}

template<typename T>
Array<double,1>  verticalMoments(const Array<T,1>& x, const Array<T,1>& y)
{ // this function provides the moments for 1D arrays
// the moments are returned as
//	0 - The sum
//	1 - The mean
//	2 - The variance
//	3 - The rms (sqrt(variance))
// this array of seven values is returned in the first dimension for vertical moments
// in the second dimension for horizontal moments	 
  
 
// both arays must have the same dimension 
// SOME ERRORS SHOULD BE THROWN HERE, HOW ?
if(x.size() !=y.size()) std::cout<<"ARRAY do no have the same dimension"<<std::endl;
if(x.size() ==0 ) std::cout<<"ARRAY should have at least one sample "<<std::endl;


// compute the standard error weight , if the value of y is equal to 0,
// then the weigth is equal to 0
 
// compute the sums
double sY=sum(y);
double sXY=sum(x*y);
double sX2Y=sum(x*x*y);

// compute the moments
double verticalMean=sXY/sY;
double verticalVariance=sX2Y/sY-verticalMean*verticalMean;
double verticalRMS=std::sqrt(verticalVariance);

// returning the results

Array<double,1> result(4);

result = sXY,verticalMean,verticalVariance,verticalRMS;

return result;
}

/////////////////////////////////////////////////////////////////////////////////
// general moments
/////////////////////////////////////////////////////////////////////////////////

template<typename T>
Array<double,1>  horizontalMoments(const Array<T,1>& y)
{// call the general moments with x being the indexes of y
int lb=y.lbound(firstDim),ub=y.ubound(firstDim);
Array<T,1> x(ub-lb+1);
firstIndex i;
x=i+lb;
return horizontalMoments(x,y);
}

template<typename T>
Array<double,1>  horizontalMoments(const Array<T,1>& x, const Array<T,1>& y)
{ // this function provides the moments for 1D arrays
// the moments are returned as
//	0 - The sum
//	1 - The mean
//	2 - The variance
//	3 - The rms (sqrt(variance))
// this array of seven values is returned in the first dimension for vertical moments
// in the second dimension for horizontal moments	 
  
 
// both arays must have the same dimension 
// SOME ERRORS SHOULD BE THROWN HERE, HOW ?
if(x.size() !=y.size()) std::cout<<"ARRAY do no have the same dimension"<<std::endl;
if(x.size() ==0 ) std::cout<<"ARRAY should have at least one sample "<<std::endl;


// compute the standard error weight , if the value of y is equal to 0,
// then the weigth is equal to 0
 
// compute the sums
double n=(double)y.size();
double sY=sum(y);
double sY2=sum(pow2(y));


// compute the moments
double horizontalMean=sY/n;
double horizontalVariance=sY2/n-horizontalMean*horizontalMean;
double horizontalRMS=std::sqrt(horizontalVariance);

// returning the results

Array<double,1> result(4);

result = sY,horizontalMean,horizontalVariance,horizontalRMS;

return result;
}


}