/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef POLYNOMIALPEAKFINDER_H
#define POLYNOMIALPEAKFINDER_H

#include <blitz/array.h>   
#include <stdexcept>
#include <fstream>
#include "Polynomial.h"

class PolynomialPeakFinder {

public:
	/**
	 * Constructor.
	 * @order the order of the fitted polynomial.
	 * @param y the values of the curve to fit.
	 * @param x the abscissa of the curve to fit. This array should have the same dimension than y.
	 */
	PolynomialPeakFinder(int order, const blitz::Array<double, 1>& y, const blitz::Array<double, 1>& x) ;

	/**
	 * Same constructor as above, but without abscissa values.
	 */
	PolynomialPeakFinder(int order, const blitz::Array<double, 1>& y) ;

	/**
	 * Destructor.
	 */
	~PolynomialPeakFinder();

	/**
	 * Returns the maximum peak position.
	 */
	double getMaximumPeakPosition() const ;

	/**
	 * Returns the fitted polynomial.
	 */
	Polynomial * getPolynomial() const;

	/**
	 * Gets the quality.
	 */
	double getQuality() const;

private:
	void calculateQuality();

	blitz::Array<double, 1> _y;
	blitz::Array<double, 1> _x;

	Polynomial * _fittedPolynomial;
	double _quality;
};

#endif