/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef TOOLS_H
#define TOOLS_H

#include <blitz/array.h>
#include <stdexcept>

namespace blitz {

/** \file matrixTools.h 
 * Compute some common matrix operations.
 */

/**
 * Identity matrix.
 */
template<typename T>
Array<T, 2> Identity(int n);

/**
 * Norm 1 for matrices.
 */
template<typename T>
T norm1(const Array<T, 2>& A);

/**
 * Matrix - matrix product.
 */
template<typename T>
Array<T, 2> product(const Array<T, 2>& A, const Array<T, 2>& B)
 ;

/**
 * Matrix - vector product.
 */
template<typename T>
Array<T, 1> product(const Array<T, 2>& A, const Array<T, 1>& X)
 ;

}

#include "Tools.inl"

#endif