/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cassert>

#ifndef BZ_BLITZ_H
 #include <blitz/blitz.h>
#endif

#include <blitz/array.h>  

namespace blitz {

template<typename T>
Array<T, 2> Identity(int n)
{
  Array<double, 2> I(n, n);
  
  firstIndex i;
  secondIndex j;
  
  I = (i == j);
  
  return I;
}

template<typename T>
T norm1(const Array<T, 2>& A)
{
  Array<T, 1> Max(A.rows());
  
  firstIndex i;
  secondIndex j;
  
  Max = sum(abs(A(j, i)), j);

  Array<T, 1> r(1);
  r = max(Max);
  
  return r(0);
}

template<typename T>
Array<T, 2> product(const Array<T, 2>& A, const Array<T, 2>& B) 
{
  if (A.columns() != B.rows())
    throw std::invalid_argument("product: A columns must equal B rows!");
 
  Array<T, 2> R(A.rows(), B.columns());

  firstIndex i;
  secondIndex j;
  thirdIndex k;
  
  R = sum(A(i, k) * B(k, j), k);
  
  return R;
}

template<typename T>
Array<T, 1> product(const Array<T, 2>& A, const Array<T, 1>& X) 
{
  if (A.columns() != X.size())
    throw std::invalid_argument("product: A columns must equal X size!");
 
  Array<T, 1> R(A.rows());

  firstIndex i;
  secondIndex j;
  
  R = sum(A(i, j) * X(j), j);
  
  return R;
}

}