/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_ARRAY_H_
#define COMMON_ARRAY_H_

#include <common/base/Exception.h>
#include <common/base/BaseArray.h>

namespace common {

/**
 * Base template class for the array classes.
 */
template<typename ValueType>
class Array : public BaseArray {
	
public:
	/**
	 * Default Constructor.
	 */
	Array();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~Array();
	
	/**
	 * Resizes the array.
	 */
	virtual void resize(long size) = 0;
	
	/**
	 * Gets the size.
	 */
	virtual long getSize() const = 0;
	
	/**
	 * Gets the value at index.
	 */
	virtual const ValueType& operator[](long index) const = 0;
	
	/**
	 * Gets the value at index.
	 */
	virtual ValueType& operator[](long index) = 0;
	
	/**
	 * Gets the size. For generated code only.
	 */
	long _size() const;
	
	/**
	 * Sets the array resizable.
	 */
	virtual void setResizable(bool resizable) = 0;
	
	/**
	 * Returns true if is resizable.
	 */
	virtual bool isResizable() const = 0;
	
	/**
	 * Copies the array.
	 */
	virtual Array<ValueType> * copy() const = 0;
};

}

#include "Array.inl"

#endif /*COMMON_ARRAY_H_*/
