/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "BaseArray.h"
#include <common/base/PrimitiveConvert.h>

namespace common {

BaseArray::BadSizeException::BadSizeException(long size) 
	: Exception("common::BaseArray::BadSizeException", PrimitiveConvert<std::string, long>::convert(size) + "!") {
}
	
BaseArray::NotResizableException::NotResizableException(long wantedSize, long fixedSize)
	: Exception("common::BaseArray::NotResizableException", std::string("the array with fixed size '") + PrimitiveConvert<std::string, long>::convert(fixedSize)
																	+ "' is not resizable (requested size '" + PrimitiveConvert<std::string, long>::convert(wantedSize) + "')") {
}	

BaseArray::NotResizableException::NotResizableException()
	: Exception("common::BaseArray::NotResizableException", std::string("the array is not resizable")) {
}	

BaseArray::IndexOutOfBoundsException::IndexOutOfBoundsException(long index, long size) 
	: Exception("common::BaseArray::IndexOutOfBoundsException", "requesting index '" + PrimitiveConvert<std::string, long>::convert(index) + "' that must be between 0 and " + PrimitiveConvert<std::string, long>::convert(size)) {
}

BaseArray::BaseArray() {
}

BaseArray::~BaseArray() {
}

}