/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_BASEARRAY_H_
#define COMMON_BASEARRAY_H_

#include <common/base/Exception.h>

namespace common {

/**
 * Base class for the array classes.
 */ 
class BaseArray {

public:
	/**
	 * Bad size exception.
	 */
	class BadSizeException : public common::Exception {
	
	public:
		/**
		 * Constructor.
		 */
		BadSizeException(long size);
	};
	
	/**
	 * Not resizable exception.
	 */
	class NotResizableException : public common::Exception {
	
	public:
		/**
		 * Constructor with parameters.
		 */
		NotResizableException(long wantedSize, long fixedSize);
		
		/**
		 * Constructor.
		 */
		NotResizableException();
	};
	
	/**
	 * Index out of bounds exception.
	 */
	class IndexOutOfBoundsException : public common::Exception {
	
	public:
		/**
		 * Constructor.
		 */
		IndexOutOfBoundsException(long index, long size);
	};
	
	/**
	 * Constructor.
	 */
	BaseArray();
	
	/**
	 * virtual destructor.
	 */
	virtual ~BaseArray();
};

}

#endif /*COMMON_BASEARRAY_H_*/