/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <common/output/OutputStream.h>

namespace common {

template<typename ValueType>
ClonedArray<ValueType>::ClonedArray() :
	_fixedSize(-1), _realSize(0) {
}

template<typename ValueType>
ClonedArray<ValueType>::ClonedArray(long fixedSize) :
	_fixedSize(fixedSize), _realSize(0) {

	if (fixedSize > 0) {
		_realSize = fixedSize;
	}
}

template<typename ValueType>
ClonedArray<ValueType>::ClonedArray(const ClonedArray<ValueType>& array) :
	_fixedSize(array._fixedSize),
	_realSize(array._realSize) {
}

template<typename ValueType>
ClonedArray<ValueType>::ClonedArray(const RealArray<ValueType>& array) :
	_fixedSize(array._fixedSize),
	_realSize(array._realSize) {
}

template<typename ValueType>
ClonedArray<ValueType>::~ClonedArray() {
}

template<typename ValueType>
ClonedArray<ValueType>& ClonedArray<ValueType>::operator= (const ClonedArray<ValueType>& array) {
	_fixedSize = array._fixedSize;
	_realSize = array._realSize;

	return *this;
}
template<typename ValueType>
void ClonedArray<ValueType>::resize(long size) {

	if (_realSize != size) {

		if (size < 0) {
			throw BaseArray::BadSizeException(size);
		}

		if (_fixedSize == -1) {
			_realSize = size;

		} else {
			throw BaseArray::NotResizableException(size, _fixedSize);
		}
	}
}

template<typename ValueType>
long ClonedArray<ValueType>::getSize() const {
	return _realSize;
}

template<typename ValueType>
const ValueType& ClonedArray<ValueType>::operator[](long index) const {

	if (index < 0 || index >= _realSize) {
		throw BaseArray::IndexOutOfBoundsException(index, _realSize);
	}

	static ValueType value;

	return value;
}

template<typename ValueType>
ValueType& ClonedArray<ValueType>::operator[](long index) {

	if (index < 0 || index >= _realSize) {
		throw BaseArray::IndexOutOfBoundsException(index, _realSize);
	}

	static ValueType value;

	return value;
}

template<typename ValueType>
void ClonedArray<ValueType>::setResizable(bool resizable) {
	if (resizable) {
		_fixedSize = -1;
	} else {
		_fixedSize = _realSize;
	}
}

template<typename ValueType>
bool ClonedArray<ValueType>::isResizable() const {
	return (_fixedSize == -1);
}

template<typename ValueType>
long ClonedArray<ValueType>::_size() const {
	return getSize();
}


template<typename ValueType>
Array<ValueType> * ClonedArray<ValueType>::copy() const {
	return new RealArray<ValueType>();
}

}
