/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_COMMONTYPES_H_
#define COMMON_COMMONTYPES_H_

namespace common {

	/**
	 * Driver mode.
	 */
	typedef enum {REAL_MODE, SIMULATED_MODE, PERFECT_MODE} DriverMode;
	
	/**
	 * Set command active result.
	 */
	typedef enum {EXECUTE, RECURSIVE, WAIT, SKIP} SetCommandActiveResult;
	
	/**
	 * Termination error.
	 */
	typedef enum {NO_ERROR, NULL_PROPERTY_REFERENCE, NO_SUCH_PROPERTY, BAD_PARAMETERS, UNKNOWN_ERROR} TerminationError;
	
	/**
	 * Output type.
	 */
	typedef enum {NORMAL, ERROR, DEBUGGING, STARTUP, SHUTDOWN} OutputType;

	/**
	 * Configuration state.
	 */
	typedef enum {ENABLED, DISABLED, TREE, PROPERTY, CONTROLLER_ADDED, CONTROLLER_REMOVED} ConfigurationState;

	/**
	 * Log type.
	 */
	typedef enum {START, END, ACQ_START, ACQ_END, SINGLE} LogType;

	/**
	 * Log level.
	 */
	typedef enum {LOG_INFO, LOG_WARNING, LOG_ERROR, LOG_DEBUG, LOG_SYNC} LogLevel;

	/**
	 * Server type.
	 */
	typedef enum {ICS, NOMAD} ServerType;

	/**
	 * Condition type.
	 */
	typedef enum {CONDITION_ALARM, CONDITION_WARNING, CONDITION_WATCHDOG, CONDITION_COMMAND_ZONE_TERMINATION} ConditionType;

	/**
	 * Condition state.
	 */
	typedef enum {CONDITION_ACTIVE, CONDITION_INACTIVE, CONDITION_ON_ACTIVATION_DELAY} ConditionState;

	/**
	 * Command state.
	 */
	typedef enum {INACTIVE, PENDING, STARTING, ACTIVE, PAUSED, PAUSING, STOPPED, STOPPING, RESUMING, UNKNOWN} CommandState;

	/**
	 * Expression status.
	 */
	typedef enum {VALID_EXPRESSION, INVALID_EXPRESSION} ExpressionStatus;

	/**
	 * Expression error.
	 */
	typedef enum {NO_EXPRESSION_ERROR = 0, DISABLED_CONTROLLER_ERROR, ENUMERATED_VALUE_ERROR, UNDEFINED_VARIABLE_ERROR, PARSER_ERROR} ExpressionError;

	/**
	 * Token state.
	 */
	enum class TokenState {FREE, OWNED};

	/**
	 * Remote control state.
	 */
	enum class RemoteControlState {DISABLED, ENABLED};

	/**
	 * Token authorization status.
	 */
	enum class TokenAuthorizationStatus {OK, TOKEN_NOT_OWNED, CLIENT_DISCONNECTED};

	/**
	 * Critical state.
	 */
	enum class CriticalState {DISABLED, ENABLED};

	/**
	 * File type.
	 */
	enum class FileType {SCRIPT_FILE, SETTINGS_FILE, PAL_FILE, XBU_FILE, ATTACHED_FILE};
}

#endif /*COMMON_COMMONTYPES_H_*/
