/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_COMPARE_H_
#define COMMON_COMPARE_H_

namespace common {

/**
 * Class utility to compare the values of numbers. Forwards calls to Equals class.
 */	
struct Compare {
	/**
	 * Returns true if the values are equals.
	 */
	template<typename ValueType0, typename ValueType1>
	static bool equals(const ValueType0& v0, const ValueType1& v1);

	/**
	 * Returns true if the values are not equals.
	 */
	template<typename ValueType0, typename ValueType1>
	static bool notEquals(const ValueType0& v0, const ValueType1& v1);
	
	/**
	 * Returns true if the first value is greater than the second value.
	 */
	template<typename ValueType0, typename ValueType1>
	static bool greaterThan(const ValueType0& v0, const ValueType1& v1);
	
	/**
	 * Returns true if the first value is greater than or equals the second value.
	 */
	template<typename ValueType0, typename ValueType1>
	static bool greaterThanOrEquals(const ValueType0& v0, const ValueType1& v1);
	
	/**
	 * Returns true if the first value is lower than the second value.
	 */
	template<typename ValueType0, typename ValueType1>
	static bool lowerThan(const ValueType0& v0, const ValueType1& v1);
	
	/**
	 * Returns true if the first value is lower than or equals the second value.
	 */
	template<typename ValueType0, typename ValueType1>
	static bool lowerThanOrEquals(const ValueType0& v0, const ValueType1& v1);
};

}

#include "Compare.inl"

#endif /*COMMAND_COMPARE_H_*/