/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_CONVERT_H_
#define COMMON_CONVERT_H_

#include <common/base/Exception.h>
#include <string>
#include <sstream>

#include <iostream>

namespace common {

/**
 * Conversion exception.
 */
class ConversionException : public Exception {
	
public:
	/**
 	 * Constructor.
 	 */
	ConversionException(const std::string& message);
};

template<typename ToType, typename FromType>
struct ConvertHelper;

/**
 * Primary template for the to template class. Forwards to the ConvertHelper template class.
 */
template<typename ToType, typename FromType>
ToType to(FromType const & value, std::string const & format = "") {
	return ConvertHelper<ToType, FromType>::convert(value, format);
}

template<typename ToType, typename FromType>
struct ConvertHelper {
	static ToType convert(FromType const & value, std::string const & format);
};


std::string convertToHexString(unsigned long value);
std::string convertSignal(int signal);
std::string convertStatus(int status);
std::string convertStringToXML(std::string const & value);
std::string convertXMLStringToAscii(std::string const & value);
std::string convertStringToJSON(std::string const & value);

}

#include "Convert.inl"

#endif /*COMMON_CONVERT_H_*/
