/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Duration.h"
#include "Date.h"

using namespace boost::posix_time;

namespace common {

Duration::Duration() {
}

Duration::Duration(const Date& d0, const Date& d1) {
	
	_duration = d1._time - d0._time;
}

Duration::Duration(double time, TimeUnit unit) {
	
	if (unit == MILLISECONDS) {
		_duration = microseconds((long)(time * 1000));
	} else if (unit == SECONDS) {
		_duration = microseconds((long)(time * 1000000));
	}
}

Duration::~Duration() {
}

void Duration::operator+= (const Duration& duration) {
	_duration += duration._duration;
}

void Duration::operator-= (const Duration& duration) {
	_duration -= duration._duration;
}

std::string Duration::toString() const {
	return to_iso_string(_duration);
}

Duration operator- (const Duration& d0, const Duration& d1) {
	Duration result = d0;
	result -= d1;
	
	return result;
}

Duration operator+ (const Duration& d0, const Duration& d1) {
	Duration result = d0;
	result += d1;
	
	return result;
}

Duration operator- (const Date& d1, const Date& d0) {
	return Duration(d0, d1);
}

double Duration::getSeconds() const {
	return ((double)_duration.total_nanoseconds())/1000000000.0;	
}

double Duration::getMilliseconds() const {
	return ((double)_duration.total_nanoseconds())/1000000.0;	
}

const boost::posix_time::time_duration& Duration::getPosixTime() const {
	return _duration;
}

std::ostream& operator<<(std::ostream &o, const Duration& duration) {
	o << duration.toString();
	
	return o;
}

}
