/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Encrypt.h"
#include <iostream>
#include <random>
#include <algorithm>

using namespace std;

namespace common {

std::string idToShortURL(int64 n) {
	// Map to store 62 possible characters
	char map[] = "abcdefghijklmnopqrstuvwxyzABCDEF"
			"GHIJKLMNOPQRSTUVWXYZ0123456789";

	string shorturl;

	// Convert given integer id to a base 62 number
	while (n) {
		// use above map to store actual character
		// in short url
		shorturl.push_back(map[n % 62]);
		n = n / 62;
	}

	// Reverse shortURL to complete base conversion
	reverse(shorturl.begin(), shorturl.end());

	return shorturl;
}

// Function to get integer ID back from a short url
int64 shortURLtoID(const std::string& shortURL) {
	int64 id = 0; // initialize result

	// A simple base conversion logic
	for (int i = 0; i < shortURL.length(); i++) {
		if ('a' <= shortURL[i] && shortURL[i] <= 'z')
			id = id * 62 + shortURL[i] - 'a';
		if ('A' <= shortURL[i] && shortURL[i] <= 'Z')
			id = id * 62 + shortURL[i] - 'A' + 26;
		if ('0' <= shortURL[i] && shortURL[i] <= '9')
			id = id * 62 + shortURL[i] - '0' + 52;
	}
	return id;
}

std::string encrypt(int64 value) {

	constexpr int N = 15;

	// Map to store 62 possible characters
	char map[] = "abcdefghijklmnopqrstuvwxyzABCDEF"
			"GHIJKLMNOPQRSTUVWXYZ0123456789";

	// Add
	std::default_random_engine generator { static_cast<long unsigned int>(time(0)) };
	std::uniform_int_distribution<int> distribution(0, 61);

	// Add noise.
	string result(N, ' ');

	for (int i = 0; i < N; ++i) {
		result[i] = map[distribution(generator)];
	}

	result += idToShortURL(value);

	return result;
}

int64 decrypt(const std::string& value) {

	constexpr int N = 15;

	string realData = value.substr(N);

	return shortURLtoID(realData);
}

}
