/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Exception.h"

#include <iostream>

using namespace std;

namespace common {

Exception::Exception(std::string const & type, std::string const & text) {
	element e;
	e.type = type;
	e.text = text;

	_exceptionStack.push_back(e);
}

Exception::~Exception() {
}

void Exception::setChild(Exception & exception) {

	// copying the stack
	for (list<element>::const_iterator e = exception._exceptionStack.begin(); e != exception._exceptionStack.end(); ++e) {

		_exceptionStack.push_back(*e);
	}
}

void Exception::setChild(Exception const & exception) {

	// copying the stack
	for (list<element>::const_iterator e = exception._exceptionStack.begin(); e != exception._exceptionStack.end(); ++e) {

		_exceptionStack.push_back(*e);
	}
}

std::string Exception::printStack() const {

	string result;

	string offset("");

	for (list<element>::const_iterator e = _exceptionStack.begin(); e != _exceptionStack.end(); ++e) {

		result += offset + e->type + " : " + e->text + "\n";
		offset += "  ";
	}

	return result;
}

std::string Exception::printCurrentException() const {

	element const & e = _exceptionStack.front();
	string result = e.type + " : " + e.text;

	return result;
}

std::string const & Exception::getMessage() const {
	return _exceptionStack.front().text;
}

}