/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_FILEUTILITIES_
#define COMMON_FILEUTILITIES_

#include <boost/filesystem/fstream.hpp>
#include <common/base/Exception.h>
#include <string>
#include <vector>
#include <utility>

namespace common {

/**
 * Class to help with file tasks.
 */
class FileUtilities {
	
public:
	/**
	 * Exception class for unaccessible file.
	 */
	class NoSuchDirectoryException : public Exception {
	
	public:
		/**
		 * Constructor.
		 */
		NoSuchDirectoryException(std::string const & fileName);
	};

	/**
	 * Exception class for unaccessible file.
	 */
	class NotRegularFile : public Exception {

	public:
		/**
		 * Constructor.
		 */
		NotRegularFile(std::string const & fileName);
	};

	/**
	 * Gets the list of filenames in the path name with the extension.
	 */
	static std::vector<std::string> getFilesWithExtension(const std::string& pathName, const std::string& extension);

	/**
	 * Gets the list of file paths in the path name with the extension. Explores all the directory hierarchy.
	 */
	static std::vector<boost::filesystem::path> getFilePathsWithExtension(const std::string& pathName, const std::string& extension, bool recursive = true);
	
	/**
	 * Returns the stringified path. For boost portability.
	 */
	static std::string nameToString(std::string const & name);

	/**
	 * Returns the stringified path. For boost portability.
	 */
	static std::string nameToString(boost::filesystem::path const & name);

	/*!
	 * \brief compose file from path and name (use for XBU, ub matrix file, pal file...)
	 */
	static std::string getFileName(const boost::filesystem::path& path1, const std::string& name, const std::string& defaultname,
			const std::string& extension) ;

	/*!
	 * \brief compose file from path and name (use for XBU, ub matrix file, pal file...)
	 */
	static std::string getDirectoryName(const boost::filesystem::path& path1, const std::string& name) ;

	/*!
	 * \brief get file size
	 */
	static time_t getFileLastTime(const std::string& name) ;
};

}

#endif /*COMMON_FILEUTILITIES_*/
