/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

namespace common {

template<typename Key, typename Data, typename HashFun>
HashMap<Key, Data, HashFun>::NoSuchKeyException::NoSuchKeyException(Key const & key)
: BaseNoSuchKeyException("common::HashMap<Key, Data>::NoSuchKeyException", to<std::string>(key)) {
};

template<typename Key, typename Data, typename HashFun>
void HashMap<Key, Data, HashFun>::erase(Key const & key) {

	typename ImplementationType::iterator it = _impl.find(key);
	if (it == _impl.end()) {
		throw NoSuchKeyException(key);
	}

	_impl.erase(it);
}

template<typename Key, typename Data, typename HashFun>
Data & HashMap<Key, Data, HashFun>::get(Key const & key) {

	typename ImplementationType::iterator it = _impl.find(key);
	if (it == _impl.end()) {
		throw NoSuchKeyException(key);
	}

	return it->second;
}

template<typename Key, typename Data, typename HashFun>
Data const & HashMap<Key, Data, HashFun>::get(Key const & key) const {

	typename ImplementationType::const_iterator it = _impl.find(key);
	if (it == _impl.end()) {
		throw NoSuchKeyException(key);
	}

	return it->second;
}

template<typename Key, typename Data, typename HashFun>
unsigned long HashMap<Key, Data, HashFun>::getSize() const {

	return _impl.size();
}

template<typename Key, typename Data, typename HashFun>
std::vector<std::pair<Key, Data> > HashMap<Key, Data, HashFun>::getData() const {

	std::vector<std::pair<Key, Data> > result;

	for (typename ImplementationType::const_iterator it = _impl.begin(); it != _impl.end(); it++) {
		result.push_back(*it);
	}

	return result;
}

template<typename Key, typename Data, typename HashFun>
void HashMap<Key, Data, HashFun>::clear() {

	_impl.clear();
}

}