/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_HASH_H_
#define COMMON_HASH_H_

#ifdef __GNUC__
	#if __GNUC__ == 3 && __GNUC_MINOR__ <= 3
		#include <ext/stl_hash_fun.h>
	#elif __GNUC__ == 4 && __GNUC_MINOR__ <= 2
		#include <ext/hash_fun.h>
	#else
		#include <backward/hash_fun.h>
	#endif
#endif

namespace common {

/**
 * Template hash class for the HashMap. Default implementation with the gnu.
 */
template<typename Type>
struct Hash {

	std::size_t operator()(Type const & value) const {
		return h(value);
	}

	__gnu_cxx::hash<Type> h;
};

/**
 * Template hash class specialization for the HashMap.
 */
template<typename Type>
struct Hash<Type*> {

	std::size_t operator()(Type* const & value) const {
		return h(reinterpret_cast<int>(value));
	}

	__gnu_cxx::hash<int> h;
};


}

/**
 * Defining the hash function for string by reusing char * version.
 */
namespace __gnu_cxx {

template<>
struct hash<std::string> {

	std::size_t operator()(std::string const & s) const {
		return __stl_hash_string(s.c_str());
	}
};

}


#endif