/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "LogElement.h"

namespace common {

const std::string LogElement::ACTUAL = "actual";
const std::string LogElement::SETPOINT = "setpoint";
const std::string LogElement::OFFSET = "offset";

LogElement::LogElement() :
	m_propertyLog(false),
	m_hidden(false) {
}

LogElement::LogElement(bool propertyLog, bool hidden, std::string const & name, std::string const & type, std::string const& qualifier, std::string const & unit, std::string const & value) :
	m_name(name),
	m_type(type),
	m_qualifier(qualifier),
	m_value(value),
	m_unit(unit),
	m_propertyLog(propertyLog),
	m_hidden(hidden) {
}

const std::string& LogElement::getName() const {
	return m_name;
}

const std::string& LogElement::getType() const {
	return m_type;
}

const std::string& LogElement::getQualifier() const {
	return m_qualifier;
}

const std::string& LogElement::getUnit() const {
	return m_unit;
}

const std::string& LogElement::getValue() const {
	return m_value;
}

bool LogElement::isPropertyLog() const {
	return m_propertyLog;
}

bool LogElement::isHidden() const {
	return m_hidden;
}

}
