/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PrimitiveConvert.h"
#include <boost/format.hpp>
#include <string>
#include <sstream>
#include <iostream>

using namespace std;

namespace common {

string PrimitiveConvert<string, char>::convert(char const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % (short)value).str();
	}
	
	return (boost::format(format) % (short)value).str();
}

string PrimitiveConvert<string, signed char>::convert(signed char const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % (short)value).str();
	}

	return (boost::format(format) % (short)value).str();
}

string PrimitiveConvert<string, short>::convert(short const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % value).str();
	}
	
	return (boost::format(format) % value).str();
}

string PrimitiveConvert<string, long>::convert(long const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % value).str();
	}
	
	return (boost::format(format) % value).str();
}

string PrimitiveConvert<string, unsigned long>::convert(unsigned long const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % value).str();
	}
	
	return (boost::format(format) % value).str();
}

string PrimitiveConvert<string, int>::convert(int const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % value).str();
	}
	
	return (boost::format(format) % value).str();
}

string PrimitiveConvert<string, unsigned int>::convert(unsigned int const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % value).str();
	}
	
	return (boost::format(format) % value).str();
}

string PrimitiveConvert<string, unsigned short>::convert(unsigned short const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % value).str();
	}
	
	return (boost::format(format) % value).str();
}

string PrimitiveConvert<string, long long>::convert(long long const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % value).str();
	}
	
	return (boost::format(format) % value).str();
}

string PrimitiveConvert<string, bool>::convert(bool const & value, std::string const & format) {

	if (value)
		return "true";
	
	return "false";	
}

string PrimitiveConvert<string, float32>::convert(float32 const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % value).str();
	}
	
	return (boost::format(format) % value).str();
}

string PrimitiveConvert<string, float64>::convert(float64 const & value, std::string const & format) {

	if (format == "") {
		return (boost::format("%s") % value).str();
	}
	
	return (boost::format(format) % value).str();
}

string PrimitiveConvert<string, string>::convert(string const & value, std::string const & format) {

	return value;
}

char PrimitiveConvert<char, string>::convert(string const & value, std::string const & format) {

	// short, because char is special due to its character representation
	short r;
	
	istringstream iss(value);
	if ((value.size() > 1 && value.substr(0, 2) == "0x") || (format == "%x") || (format == "%X")) {
		iss >> hex >> r;
	} else {
		iss >> r;
	}
	
	return r;	
}

short PrimitiveConvert<short, string>::convert(string const & value, std::string const & format) {
	
	short r;
	
	istringstream iss(value);
	if ((value.size() > 1 && value.substr(0, 2) == "0x") || (format == "%x") || (format == "%X")) {
		iss >> hex >> r;
	} else {
		iss >> r;
	}
	
	return r;	
}

unsigned short PrimitiveConvert<unsigned short, string>::convert(string const & value, std::string const & format) {

	unsigned short r;

	istringstream iss(value);
	if ((value.size() > 1 && value.substr(0, 2) == "0x") || (format == "%x") || (format == "%X")) {
		iss >> hex >> r;
	} else {
		iss >> r;
	}

	return r;
}

long PrimitiveConvert<long, string>::convert(string const & value, std::string const & format) {
	
	long r;
	
	istringstream iss(value);
	if ((value.size() > 1 && value.substr(0, 2) == "0x") || (format == "%x") || (format == "%X")) {
		iss >> hex >> r;
	} else {
		iss >> r;
	}
				
	return r;	
}

unsigned long PrimitiveConvert<unsigned long, string>::convert(string const & value, std::string const & format) {
	
	istringstream iss(value);
	unsigned long r;
	if ((value.size() > 1 && value.substr(0, 2) == "0x") || (format == "%x") || (format == "%X")) {
		iss >> hex >> r;
	} else {
		iss >> r;
	}
	
	return r;	
}

int PrimitiveConvert<int, string>::convert(string const & value, std::string const & format) {
	
	istringstream iss(value);
	int r;
	if ((value.size() > 1 && value.substr(0, 2) == "0x") || (format == "%x") || (format == "%X")) {
		iss >> hex >> r;
	} else {
		iss >> r;
	}
	
	return r;	
}

unsigned int PrimitiveConvert<unsigned int, string>::convert(string const & value, std::string const & format) {
	
	istringstream iss(value);
	unsigned int r;
	if ((value.size() > 1 && value.substr(0, 2) == "0x") || (format == "%x") || (format == "%X")) {
		iss >> hex >> r;
	} else {
		iss >> r;
	}
	
	return r;	
}

long long PrimitiveConvert<long long, string>::convert(string const & value, std::string const & format) {
	
	istringstream iss(value);
	long long r;
	if ((value.size() > 1 && value.substr(0, 2) == "0x") || (format == "%x") || (format == "%X")) {
		iss >> hex >> r;
	} else {
		iss >> r;
	}
	
	return r;	
}

bool PrimitiveConvert<bool, string>::convert(string const & value, std::string const & format) {
	
	return (value == "true");
}

float32 PrimitiveConvert<float32, string>::convert(string const & value, std::string const & format) {
	
	istringstream iss(value);
	float32 r;
	iss >> r;
	
	return r;	
}

float64 PrimitiveConvert<float64, string>::convert(string const & value, std::string const & format) {
	
	istringstream iss(value);
	float64 r;
	iss >> r;
	
	return r;	
}

}
