/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_PROPERTYTYPE_H_
#define COMMON_PROPERTYTYPE_H_

#include <string>

namespace common {

template<typename Type>
struct PropertyTypeHelper;

/**
 * Class defined for convenience to offer a list of the avalaible types and their conversion to string.
 */
class PropertyType {

public:
	/**
	 * List of types.
	 */
	typedef enum {INT8 = 0, UINT16, INT16, UINT32, INT32, UINT64, INT64, FLOAT32, FLOAT64, STRING, BOOLEAN,
		INT8_ARRAY, INT16_ARRAY, INT32_ARRAY, INT64_ARRAY, FLOAT32_ARRAY, FLOAT64_ARRAY,
		STRING_ARRAY, BOOLEAN_ARRAY,
		ICS_INT32_ARRAY, ICS_FLOAT64_ARRAY, UINT16_ARRAY, UINT32_ARRAY, UINT64_ARRAY, ICS_INT16_ARRAY, ICS_UINT16_ARRAY, ICS_UINT32_ARRAY, ICS_INT64_ARRAY, ICS_UINT64_ARRAY, ICS_FLOAT32_ARRAY} Type;

	/**
	 * Gets the enum type from the static type.
	 */
	template<typename StaticType>
	static Type getType();

	/**
	 * Gets the enum array type from the static type.
	 */
	template<typename StaticType>
	static Type getArrayType();
};

}

#include "PropertyType.inl"

#endif /*COMMON_PROPERTYTYPE_H_*/
