/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_PROXYARRAY_H_
#define COMMON_PROXYARRAY_H_

#include "Array.h"

namespace common {

/**
 * Base template class for the proxy array classes.
 */
template<typename ValueType>
class ProxyArray : public Array<ValueType> {
	
public:
	/**
	 * Default Constructor.
	 */
	ProxyArray();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ProxyArray();
	
	/**
	 * Resets the data.
	 */
	void setData(ValueType* data);

	/**
	 * Resizes the array.
	 */
	virtual void resize(long size);
	
	/**
	 * Gets the size.
	 */
	virtual long getSize() const;
	
	/**
	 * Gets the value at index.
	 */
	virtual const ValueType& operator[](long index) const;
	
	/**
	 * Gets the value at index.
	 */
	virtual ValueType& operator[](long index);
	
	/**
	 * Sets the array resizable.
	 */
	virtual void setResizable(bool resizable);
	
	/**
	 * Returns true if is resizable.
	 */
	virtual bool isResizable() const;
	
	/**
	 * Copies the array.
	 */
	virtual Array<ValueType> * copy() const;

private:
	ValueType* _data;
	long _size;
};

}

#include "ProxyArray.inl"

#endif /*COMMON_PROXYARRAY_H_*/
