/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Real.h"
#include "ServerProperties.h"
#include <common/output/OutputStream.h>
#include <cmath>
#include <iostream>

using namespace std;

namespace common {

const std::string Real::FLOAT_TOLERANCE = "floatTolerance";
const std::string Real::DOUBLE_TOLERANCE = "doubleTolerance";

float Real::_floatTolerance = (float) 1E-5;
double Real::_doubleTolerance = (double) 1E-7;

void Real::init() {
	
	try {
		_floatTolerance = ServerProperties::getInstance()->get<float>(FLOAT_TOLERANCE);
	} catch (Exception& e) {
		out << "Setting default value " << _floatTolerance << " to property 'floatTolerance'" << eol;
	}
	
	try {
		_doubleTolerance = ServerProperties::getInstance()->get<double>(DOUBLE_TOLERANCE);
	} catch (Exception& e) {
		out << "Setting default value " << _doubleTolerance << " to property 'doubleTolerance'" << eol;
	}
}

bool Real::equals(float a, float b) {
	// no tolerance test
	return (a == b);
}

bool Real::equals(double a, double b) {
	// no tolerance test	
	return (a == b);
}

}
