/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_REAL_H_
#define COMMON_REAL_H_

#include <string>

namespace common {

/**
 * Helper class to test equality between float values with a threshold. The threshold is defined
 * in the server configuration file. 
 */
class Real {
	
public:
	/**
	 * Initialises the threshold for double and float values. If not called, then the default
	 * thesholds are used.
	 */
	static void init();
	
	/**
	 * Returns true if a equals b.
	 */
	static bool equals(float a, float b);
	
	/**
	 * Returns true if a equals b.
	 */
	static bool equals(double a, double b);

private:
	static const std::string FLOAT_TOLERANCE;
	static const std::string DOUBLE_TOLERANCE;
	
	static float _floatTolerance;
	static double _doubleTolerance;
};

}

#endif /*COMMON_REAL_H_*/