/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_STRINGUTILITIES_H_
#define COMMON_STRINGUTILITIES_H_

#include "Convert.h"

namespace common {

/**
 * Helper class to achieve split operations on strings.
 */
class StringUtilities {

public:
	/**
 	 * Returns the list of token from the split of the input string with separator.
 	 */
	static std::list<std::string> split(const std::string& input, const std::string& separator = " ");

	/**
 	 * Returns the list of token from the split of the input string with separator.
 	 */
	static std::list<StringHelper::SplitItem> splitWithSeparator(const std::string& input, const std::string& separator = " ");

	/**
 	 * Loads the input into a list of nodes.
 	 */
	template<typename Type>
	static Type load(const std::string& input) {

		StringHelper::Node* node = StringHelper::tokenise(input);

		Type result;

		try {
			result = StringHelper::Loader<Type>::load(node);
			StringHelper::release(node);

		} catch (Exception&) {
			StringHelper::release(node);
			throw;
		}

		return result;
	}

	/**
	 * Replaces any occurrence of previousWord with replacementWord in inputString.
	 */
	static std::string replaceText(const std::string& inputString, const std::string& previousWord, const std::string& replacementWord);

	/**
	 * Strips the whitespaces of inputString.
	 */
	static std::string stripWhitespaces(const std::string& inputString);

};

inline std::list<std::string> StringUtilities::split(const std::string& input, const std::string& separator) {

	return StringHelper::split(input, separator);
}

inline std::list<StringHelper::SplitItem> StringUtilities::splitWithSeparator(const std::string& input, const std::string& separator) {

	return StringHelper::splitWithSeparator(input, separator);
}

}

#endif /*COMMON_STRINGUTILITIES_H_*/
