/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_TICKER_H_
#define COMMON_TICKER_H_

#include <string>
#include "Date.h"
#include "Duration.h"

namespace common {

/**
 * Class to help in the profiling of methods. Produces a time between
 * 'ticks' to determine the efficiency of the code at different places.
 */
class Ticker {
	
public:
	/**
	 * Constructor.
	 */
	Ticker(int N = 1);
	
	/**
	 * Destructor.
	 */
	~Ticker();
	
	/**
	 * Restarts the ticker.
	 */
	void restart();
	
	/**
	 * Sets a tick, meaning that if there has been N ticks since the last print of message
	 * then the message is printed with the average time since the last message was printed.
	 */
	void tick(const std::string& message);

private:
	Date _date;
	Duration _duration;
	int _N;
	int _counter;
};

}

#endif /*COMMON_TICKER_H_*/