/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "TimeCondition.h"
#include <iostream>

using namespace std;

namespace common {
	
TimeCondition::TimeCondition() :
	_notifyDone(false) {
}

TimeCondition::~TimeCondition() {
}
	
bool TimeCondition::wait(long milliseconds) {

	std::unique_lock<std::mutex> lock(_conditionMutex);
	
	if (!_notifyDone) {
		// If there is no timeout, then the condition was awakened by a notify.
		return (_condition.wait_for(lock, std::chrono::milliseconds(milliseconds)) == std::cv_status::no_timeout);
	}
	
	// Sure that notify occurred.
	return true;
}

bool TimeCondition::wait(const Date& date) {
	
	Date currentDate;
	double milliseconds = (date - currentDate).getMilliseconds();

	return wait(static_cast<long>(milliseconds));
}

void TimeCondition::notify() {

	std::unique_lock<std::mutex> lock(_conditionMutex);

	_notifyDone = true;	
	_condition.notify_one();
}

}
