/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "BaseHttpConnection.h"
#include <iostream>
#include <istream>
#include <ostream>

using namespace std;

namespace common {

ConnectionException::ConnectionException(std::string const & message) :
	Exception("common::ConnectionException", message) {
}

RequestException::RequestException(std::string const & message, uint32 statusCode) :
	Exception("common::RequestException", message),
	m_statusCode(statusCode) {
}
	
uint32 RequestException::getStatusCode() const {
	return m_statusCode;
}

BaseHttpConnection::BaseHttpConnection():
	m_methodType("GET") {
}

BaseHttpConnection::~BaseHttpConnection() {
}

void BaseHttpConnection::setTimeout(const Duration& timeout) {
	m_timeout = timeout.getPosixTime();
}

void BaseHttpConnection::openUrl(const std::string& url, const std::string& port) {
	m_url = url;
	m_port = port;
}

void BaseHttpConnection::setMethod(const std::string& type) {
	m_methodType = type;
}

void BaseHttpConnection::setPath(const std::string& path) {
	m_path = path;
}

void BaseHttpConnection::setProperty(const std::string& propertyName, const std::string& value) {
	m_properties[propertyName] = value;
}

void BaseHttpConnection::setData(const std::string& data) {
	m_data = data;
}

}
