/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_RESTJSONHTTPCONNECTION_H_
#define COMMON_RESTJSONHTTPCONNECTION_H_

#include "HttpConnection.h"
#include <boost/property_tree/ptree.hpp>

namespace common {

/**
 * Class managing REST JSON http connections.
 */
class RestJsonHttpConnection : public HttpConnection {
	
public:
	/**
	 * Constructor.
	 */
	RestJsonHttpConnection();
	
	/**
	 * Destructor. Closes the connection.
	 */
	virtual ~RestJsonHttpConnection();

	/**
	 * Sets the data with string data.
	 */
	void setData(const std::string& data);

	/**
	 * Sets the data with property_tree data.
	 * Uses the serializer of boost property_tree that sets the " for every field.
	 * Use a regex can solve the problem:
	 * boost::regex exp("\"(null|true|false|[0-9]+(\\.[0-9]+)?)\"");
	 * std::string rv = boost::regex_replace(serializedData, exp, "$1");
	 */
	void setData(const boost::property_tree::ptree& data);

	/**
	 * Sends the message. Waits until response is returned by the client.
	 * Returns the JSON result of parsing.
	 * throws RequestException.
	 */
	boost::property_tree::ptree send(bool verbose = false);
};

}

#endif
