/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RestJsonHttpSyncConnection.h"
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>
#include <boost/lexical_cast.hpp>
#include <sstream>

using namespace boost;
using namespace std;

namespace common {

RestJsonHttpSyncConnection::RestJsonHttpSyncConnection() {
	
	// set the method
	setMethod("POST");

	// set the properties
	setProperty("Content-Type", "application/json");
	setProperty("Accept", "application/json");
}

RestJsonHttpSyncConnection::~RestJsonHttpSyncConnection() {
	
}

void RestJsonHttpSyncConnection::setData(const std::string& data) {

	HttpSyncConnection::setData(data);

	// set the property content-length
	setProperty("Content-Length", lexical_cast<string>(data.length()));
}

void RestJsonHttpSyncConnection::setData(const boost::property_tree::ptree& data) {

	// transform property tree into string
	stringstream os;

	// encoding without space and newlines (pretty = false)
	write_json(os, data, false);

	string dataString(os.str());

	HttpSyncConnection::setData(dataString);

	// set the property content-length
	setProperty("Content-Length", lexical_cast<string>(dataString.length()));
}

boost::property_tree::ptree RestJsonHttpSyncConnection::send(bool verbose) {

	string response = HttpSyncConnection::send(verbose);

	// parse the result
	property_tree::ptree result;

	if (!response.empty()) {
		stringstream is(response);

		read_json(is, result);
	}

	return result;
}

}
