/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_WEBSOCKETCONNECTION_H_
#define COMMON_WEBSOCKETCONNECTION_H_

#include <string>
#include <common/base/Exception.h>
#include <common/base/Duration.h>

namespace common {

/**
 * Exception class for connection errors.
 */
class WebSocketConnectionException: public common::Exception {

public:
	/**
	 * Constructor.
	 */
	WebSocketConnectionException(std::string const & message);
};

/**
 * Class managing websocket connections.
 */
class WebSocketConnection {

public:
	/**
	 * Constructor.
	 */
	WebSocketConnection();
	
	/**
	 * Destructor. Closes the connection.
	 */
	virtual ~WebSocketConnection();

	/**
	 * Opens the connection with url, port, and text .
	 */
	void openUrl(const std::string& url, const std::string& port);

	/**
	 * Sets the path.
	 * For instance with http://www.example.com:8080/resources/image
	 * url = www.example.com, port = 8080, path = /resources/image
	 */
	void setPath(const std::string& path);

	/**
	 * Sets the timeout.
	 */
	void setTimeout(const Duration& timeout);

	/**
	 * Sends the message. Waits until response is returned by the client.
	 * If verbose is true, request and reponse headers are printed.
	 * throws RequestException.
	 */
	std::string send(const std::string& text, bool verbose = false);

private:

	std::string m_url;
	std::string m_port;
	std::string m_path;
	boost::posix_time::time_duration m_timeout;

};

}

#endif
