/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Debug.h"
#include "Color.h"
#include <thread>
#include <sstream>
#include <iostream>

namespace common {

std::atomic_bool Debug::m_enabled = false;
thread_local std::string Debug::m_space;

bool Debug::isEnabled() {
	return m_enabled;
}

void Debug::enable() {
	m_enabled = true;
}

void Debug::disable() {
	m_enabled = false;
}

std::string Debug::space() {
	return m_space;
}

std::string Debug::prefix() {

	if (m_enabled) {
		std::string prefix = m_space;
		std::ostringstream ss;
		ss << std::this_thread::get_id() << m_space << " ";

		return ss.str();
	}

	return "";
}

Debug::ScopedCall::ScopedCall(const std::string& servantInfo, const std::string& property, const boost::any& value) {

	if (m_enabled) {
		std::string prefix = m_space;
		if (!m_space.empty()) {
			prefix += "\u21b3";
		}
		std::ostringstream ss;
		ss << std::this_thread::get_id() << " " << prefix << servantInfo << " calls refresher for " << property;

		std::cout << ss.str() << std::endl;

		m_space += "    ";
	}
}

Debug::ScopedCall::ScopedCall(const std::string& servantInfo, const std::string& property, const std::string& servant) {

	if (m_enabled) {
		std::string prefix = m_space;
		if (!m_space.empty()) {
			prefix += "\u21b3";
		}
		std::ostringstream ss;
		ss << std::this_thread::get_id() << " " << prefix << servantInfo << " calls updater for " << property << " of " << servant;

		std::cout << ss.str() << std::endl;

		m_space += "    ";
	}
}

Debug::ScopedCall::~ScopedCall() {

	if (m_enabled) {
		if (m_space.length() >= 4) {
			m_space = m_space.substr(0, m_space.length() - 4);
		}
	}
}

}
