/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_DEBUG_H_
#define COMMON_DEBUG_H_

#include <atomic>
#include <string>
#include <boost/any.hpp>

namespace common {

/**
 * Class providing some debug information for refresher and updater calls.
 */
class Debug	{

public:
	/**
	 * Returns true if is enabled.
	 */
	static bool isEnabled();

	/**
	 * Enables the debug mode.
	 */
	static void enable();

	/**
	 * Disables the debug mode.
	 */
	static void disable();

	/**
	 * Returns the space.
	 */
	static std::string space();

	/**
	 * Returns the debug prefix concatenating the thread id and the space.
	 */
	static std::string prefix();

	/**
	 * Class providing a scoped call to increase and decrease space.
	 */
	class ScopedCall {

	public:
		/**
		 * Constructor for a refresher call.
		 */
		ScopedCall(const std::string& servantInfo, const std::string& property, const boost::any& value);

		/**
		 * Constructor for an updater call.
		 */
		ScopedCall(const std::string& servantInfo, const std::string& property, const std::string& servant);

		/**
		 * Destructor.
		 */
		~ScopedCall();
	};

private:
	static std::atomic_bool m_enabled;
	static thread_local std::string m_space;
};

}

#endif
