/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_ENDOFLINE_H_
#define COMMON_ENDOFLINE_H_

#include <string>

namespace common {

/**
 * Class used to define an end of line for the output stream.
 */
class EndOfLine {

public:
	/**
	 * Constructor.
	 */
	EndOfLine();
	
	/**
	 * Constructor with parameters.
	 */
	EndOfLine(std::string const & file, int line);
	
	/**
	 * Gets the file path.
	 */
	std::string const & getFile() const;
	
	/**
	 * Gets the file path.
	 */
	std::string const & getFileName() const;
	
	/**
	 * Gets the line.
	 */
	int getLine() const;

private:
	std::string _file;
	std::string _filename;
	int _line;
};
  
static const EndOfLine endOfLine = EndOfLine();

#define eol EndOfLine {__FILE__, __LINE__}

}

#endif /*COMMON_ENDOFLINE_H_*/
