/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <common/base/Convert.h>
#include <iostream>

namespace common {

template<int OutType>
OutputStream<OutType>::OutputStream(bool stdout) :
	_serverLogAccessor(0),
	_stdout(stdout) {
}

template<int OutType>
OutputStream<OutType>::~OutputStream() {
}

template<int OutType>
void OutputStream<OutType>::setServerLogAccessor(ServerLogAccessor* serverLogAccessor) {
	_serverLogAccessor = serverLogAccessor;
}

template<int OutType>
void OutputStream<OutType>::setStdOut(bool stdout) {
	_stdout = stdout;
}

template<int OutType>
void OutputStream<OutType>::setProcessName(std::string const & name) {
	_processName = name;
}

template<int OutType, typename ValueType>
struct OutputStreamHelper {
	
	static OutputStream<OutType> & apply(OutputStream<OutType> & os, ValueType const & value) {
		
		std::scoped_lock lock(os._lineMutex);
		
		os._lineMap[pthread_self()].value.append(to<std::string>(value));
		
		return os;
	}
};

template<>
struct OutputStreamHelper<0, EndOfLine> {
	
	static OutputStream<0> & apply(OutputStream<0> & out, EndOfLine const & e);
};

template<>
struct OutputStreamHelper<1, EndOfLine> {
	
	static OutputStream<1> & apply(OutputStream<1> & out, EndOfLine const & e);
};

template<>
struct OutputStreamHelper<2, EndOfLine> {
	
	static OutputStream<2> & apply(OutputStream<2> & out, EndOfLine const & e);
};

template<>
struct OutputStreamHelper<3, EndOfLine> {
	
	static OutputStream<3> & apply(OutputStream<3> & out, EndOfLine const & e);
};

template<>
struct OutputStreamHelper<4, EndOfLine> {

	static OutputStream<4> & apply(OutputStream<4> & out, EndOfLine const & e);
};

template<>
struct OutputStreamHelper<0, NoLog> {
	
	static OutputStream<0> & apply(OutputStream<0> & out, NoLog const &);
};

template<>
struct OutputStreamHelper<1, NoLog> {
	
	static OutputStream<1> & apply(OutputStream<1> & out, NoLog const &);
};

template<>
struct OutputStreamHelper<2, NoLog> {
	
	static OutputStream<2> & apply(OutputStream<2> & out, NoLog const &);
};

template<>
struct OutputStreamHelper<3, NoLog> {
	
	static OutputStream<3> & apply(OutputStream<3> & out, NoLog const &);
};

template<>
struct OutputStreamHelper<4, NoLog> {

	static OutputStream<4> & apply(OutputStream<4> & out, NoLog const &);
};

template<>
struct OutputStreamHelper<0, Startup> {
	
	static OutputStream<0> & apply(OutputStream<0> & out, Startup const &);
};

template<>
struct OutputStreamHelper<1, Startup> {
	
	static OutputStream<1> & apply(OutputStream<1> & out, Startup const &);
};

template<>
struct OutputStreamHelper<2, Startup> {
	
	static OutputStream<2> & apply(OutputStream<2> & out, Startup const &);
};

template<>
struct OutputStreamHelper<3, Startup> {
	
	static OutputStream<3> & apply(OutputStream<3> & out, Startup const &);
};

template<>
struct OutputStreamHelper<4, Startup> {

	static OutputStream<4> & apply(OutputStream<4> & out, Startup const &);
};

template<>
struct OutputStreamHelper<0, Shutdown> {
	
	static OutputStream<0> & apply(OutputStream<0> & out, Shutdown const &);
};

template<>
struct OutputStreamHelper<1, Shutdown> {
	
	static OutputStream<1> & apply(OutputStream<1> & out, Shutdown const &);
};

template<>
struct OutputStreamHelper<2, Shutdown> {
	
	static OutputStream<2> & apply(OutputStream<2> & out, Shutdown const &);
};

template<>
struct OutputStreamHelper<3, Shutdown> {
	
	static OutputStream<3> & apply(OutputStream<3> & out, Shutdown const &);
};

template<>
struct OutputStreamHelper<4, Shutdown> {

	static OutputStream<4> & apply(OutputStream<4> & out, Shutdown const &);
};

template<>
struct OutputStreamHelper<0, char *> {
	
	static OutputStream<0> & apply(OutputStream<0> & out, char * value);
};

template<>
struct OutputStreamHelper<1, char *> {
	
	static OutputStream<1> & apply(OutputStream<1> & out, char * value);
};

template<>
struct OutputStreamHelper<2, char *> {
	
	static OutputStream<2> & apply(OutputStream<2> & out, char * value);
};

template<>
struct OutputStreamHelper<3, char *> {
	
	static OutputStream<3> & apply(OutputStream<3> & out, char * value);
};

template<>
struct OutputStreamHelper<4, char *> {

	static OutputStream<4> & apply(OutputStream<4> & out, char * value);
};

template<>
struct OutputStreamHelper<0, const char *> {
	
	static OutputStream<0> & apply(OutputStream<0> & out, const char * value);
};

template<>
struct OutputStreamHelper<1, const char *> {
	
	static OutputStream<1> & apply(OutputStream<1> & out, const char * value);
};

template<>
struct OutputStreamHelper<2, const char *> {
	
	static OutputStream<2> & apply(OutputStream<2> & out, const char * value);
};

template<>
struct OutputStreamHelper<3, const char *> {
	
	static OutputStream<3> & apply(OutputStream<3> & out, const char * value);
};

template<>
struct OutputStreamHelper<4, const char *> {

	static OutputStream<4> & apply(OutputStream<4> & out, const char * value);
};

}
