/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "GenericParser.h"
#include <common/base/Convert.h>
#include <iostream>

using namespace std;

namespace common {

GenericParser::ParseException::ParseException(const std::string& fileName, unsigned int line, const std::string& message) :
	Exception("common::GenericParser::ParseException", string("Error parsing XML file ") + fileName + ((line == 0) ? "" : " at line " + to<string>(line)) + ": " + message) {
}

GenericParser::UnknownTagException::UnknownTagException(const std::string& tag, const std::string& fileName) :
	Exception("common::GenericParser::UnknownTagException", string("Unknown tag '") + tag + "' in XML file " + fileName) {
}

GenericParser::GenericParser() : _attributes(nullptr) {
}

GenericParser::~GenericParser() {
}

void GenericParser::init() {
}

void GenericParser::parseXMLFile(const string& fileName) {

	_fileName = fileName;

	pugi::xml_parse_result result = _doc.load_file(_fileName.c_str());
	if (!result) {
		throw ParseException(fileName, (uint32) result.offset, printError(_fileName, (uint32) result.offset, result.description()));
	}
	else {
		parse(_doc);
	}
}


void GenericParser::parseXMLString(const std::string& content) {

	_fileName = "string";

	istringstream sbuf(content);
	pugi::xml_parse_result result = _doc.load(sbuf);
	if (!result) {
		throw ParseException(content, (uint32) result.offset, result.description());
	}
}

void GenericParser::parse(const pugi::xml_node node) {
	string nodename = node.name();
	_iter = node;
	if (nodename.empty() ==false) {
		startElement(node.name());
	}
	for (auto child : node.children()) {
		parse(child);
	}
	if (nodename.empty() ==false) {
		endElement(node.name());
	}
}

string GenericParser::getAttribute(const std::string& attributeName) {
	string buf;

	pugi::xml_attribute attr = _iter->attribute(attributeName.c_str());
	if (attr) {
		buf = attr.value();
	}
	return buf;
}

void GenericParser::startElement(const std::string& qName) {
	// do nothing by default
}

void GenericParser::endElement(const std::string& qName) {
	// do nothing by default
}
}
