/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_GENERICPARSER_H_
#define COMMON_GENERICPARSER_H_

#include <common/base/Exception.h>
#include <string>

#include <pugixml.hpp>
#include "PersistenceServices/XMLErrors.h"

namespace common {

class GenericParser: public XMLErrors {

public:
	
	/**
	 * Exception for parse error.
	 */
	class ParseException : public common::Exception {
	
	public:
		/**
		 * Constructor.
		 */
		ParseException(const std::string& fileName, unsigned int line, const std::string& message);
	};
	
	/**
	 * Exception for unknown tag exception.
	 */
	class UnknownTagException  : public common::Exception {
	
	public:
		/**
		 * Constructor.
		 */
		UnknownTagException(const std::string& tag, const std::string& fileName);
	};
	
	/**
	 * Constructor.
	 */
	GenericParser();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~GenericParser();

	/**
	 * Initialises the Xerces parser.
	 * throw InitFailureException
	 */
	void init();

	/**
	 * Parses the file of path fileName.
	 * throw ParseException
	 */
	void parseXMLFile(const std::string& fileName);

	/**
	 * Parses the string.
	 * throw ParseException
	 */
	void parseXMLString(const std::string& content);

protected:
	/**
	 * Gets the string attribute from its name attributeName.
	 */
	std::string getAttribute(const std::string& attributeName);


	/**
	 * Overloaded method to add the protected startElement callback.
	 */
	virtual void startElement(const std::string& qname);
	
	/**
	 * Overloaded method to add the protected endElement callback.
	 */
	virtual void endElement(const std::string& qname);

	std::string _fileName;

private:

	void parse(const pugi::xml_node node);

	pugi::xml_document _doc;
	pugi::xml_node_iterator _iter;
	const pugi::xml_attribute* _attributes;
};

}

#endif /*COMMON_GENERICPARSER_H_*/
