/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ChildServantDescriptor.h"
#include <common/base/Convert.h>

using namespace std;

namespace common {

ChildServantDescriptor::ChildServantDescriptor(const std::string& n_name, bool isList, const std::vector<long>& indexArray, const std::vector<std::string>& nameArray, long size) :
	_n_name(n_name),
	_isList(isList),
	_size(size) {
	
	vector<long>::const_iterator i = indexArray.begin();
	vector<string>::const_iterator n = nameArray.begin();
	
	while (i != indexArray.end()) {
		_indexNameArray.push_back(pair<long, string>(*i, *n));
		
		i++;
		n++;
	}
}

ChildServantDescriptor::~ChildServantDescriptor() {
}

const string& ChildServantDescriptor::getNName() const {
	return _n_name;
}

long ChildServantDescriptor::getSize() const {
	return _size;
}

std::string ChildServantDescriptor::toString() const {
	return string("Servant ") + _n_name;
}

bool ChildServantDescriptor::isList() const {
	return _isList;
}

const std::vector<std::pair<long, std::string> >& ChildServantDescriptor::getIndexNameArray() const {
	
	return _indexNameArray;
}		

}
