/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_CHILDSERVANTDESCRIPTOR_H_
#define COMMON_CHILDSERVANTDESCRIPTOR_H_

#include <string>
#include <vector>
#include <utility>

namespace common {

/**
 * Class used to locally hold details of child servants stored in the data provider. 
 */
class ChildServantDescriptor	{

public:

	/**
	 * Constructor. Takes the DataProvider ID, the name and servant type.
	 */
	ChildServantDescriptor(const std::string& n_name, bool isList, const std::vector<long>& indexArray, const std::vector<std::string>& nameArray, long size);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ChildServantDescriptor();

	/**
	 * Returns the N name of the Servant.
	 */
	const std::string& getNName() const;

	/**
	 * Returns true if the child is part of a list.
	 */
	bool isList() const;
	
	/**
	 * Returns the index of the child if it is part of a list
	 */
	const std::vector<std::pair<long, std::string> >& getIndexNameArray() const;
	
	/**
	 * Returns the size.
	 */
	long getSize() const;
	
	/**
	 * Returns the Servant in stringified form.
	 */
	std::string toString() const;

private:
	std::string _n_name;	
	bool _isList;
	long _size;
	std::vector<std::pair<long, std::string> > _indexNameArray;		

};

} // namespace common

#endif /*COMMON_CHILDSERVANTDESCRIPTOR_H_*/