/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_COMMANDDESCRIPTOR_H_
#define COMMON_COMMANDDESCRIPTOR_H_

#include <string>
#include <vector>

namespace common {

/**
 * Class used to locally hold details of commands stored in the data provider. 
 */
class CommandDescriptor	{

public:

	/**
	 * Constructor. Takes DataProvider ID and command name.
	 */
	CommandDescriptor(unsigned long ID, const std::string& name);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CommandDescriptor();

	/**
	 * Returns the name of the command.
	 */
	const std::string& getName() const;
	
	/**
	 * Returns the DataProvider ID of the command.
	 */
	unsigned long getID() const;
	
	/**
	 * Returns the Command in stringified form.
	 */
	virtual std::string toString() const;
	
private:
	unsigned long _ID;
	std::string _name;
};

} // namespace common

#endif /*COMMON_COMMANDDESCRIPTOR_H_*/