/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_DATACHANGESUBSCRIBERDIRECT_H_
#define CORE_DATACHANGESUBSCRIBERDIRECT_H_

#include <common/base/CommonTypes.h>
#include <string>

namespace common {

/**
 * Direct server interface of the data change subscriber class.
 */
class DataChangeSubscriberDirect {

public:
	/**
	 * Sets the ID of the subscriber.
	 */
	virtual void setID(unsigned long subscriberID) = 0;

	/**
	 * Returns the ID of the subscriber.
	 */
	virtual unsigned long getID() = 0;

	/**
	 * Notifies the subscriber that a property has changed.
	 */
	virtual void propertyChanged(unsigned long databaseID, unsigned long propertyID) = 0;

	/**
	 * Notifies the subscriber that a command state has changed.
	 */
	virtual void commandStateChanged(unsigned long databaseID, unsigned long servantID, unsigned long commandID, CommandState state) = 0;

	/**
	 * Notifies the subscriber that a command interrupt has changed.
	 */
	virtual void commandInterruptChanged(unsigned long databaseID, unsigned long servantID, unsigned long commandID, bool value) = 0;

	/**
	 * Notifies the subscriber that a command progression has changed.
	 */
	virtual void commandProgressionChanged(unsigned long databaseID, unsigned long commandID, double progression) = 0;

	/**
	 * Notifies the subscriber that the configuration has changed.
	 */
	virtual void configurationChanged(unsigned long databaseID, unsigned long servantID, common::ConfigurationState enable) = 0;

	/**
	 * Notifies the condition state.
	 */
	virtual void conditionStateChanged(unsigned long conditionId, bool on, common::ConditionState state) = 0;

	/**
	 * Notifies the condition state.
	 */
	virtual void conditionActionsStateChanged(unsigned long conditionId, common::ConditionState state) = 0;
};

}

#endif /*CORE_DATACHANGESUBSCRIBERDIRECT_H_*/
