/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PropertyDescriptor.h"
#include <common/base/Convert.h>

using namespace std;

namespace common {

PropertyDescriptor::PropertyDescriptor(const vector<unsigned long>& IDs, const string& name, const string& type, bool isDynamic, bool isSpy) :
	_IDs(IDs),
	_name(name),
	_type(type),
	_isDynamic(isDynamic),
	_isSpy(isSpy) {
}

PropertyDescriptor::~PropertyDescriptor() {
}

const string& PropertyDescriptor::getName() const {
	return _name;
}

const string& PropertyDescriptor::getType() const {
	return _type;
}

const vector<unsigned long>& PropertyDescriptor::getIDs() const {
	return _IDs;
}

bool PropertyDescriptor::isDynamic() const {
	return _isDynamic;
}

bool PropertyDescriptor::isSpy() const {
	return _isSpy;
}

string PropertyDescriptor::toString() const {
	if (_isDynamic) {
		string idString = "[";
		if (_IDs.size() > 0) {
			for (size_t i = 0; i < _IDs.size() - 1; i++) {
				idString = idString + to<string>(_IDs[i]) + ", ";
			}
			idString = idString + to<string>(_IDs[_IDs.size() - 1]);
		}
		idString = idString + "]";
		
		return string("Array property ") + getName() + ", type = " + getType() + ", IDs = " + idString;
	}
	return string("Elementary property ") + getName() + ", type = " + getType() + ", ID = " + to<string>(_IDs[0]);
}

}
