/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_PROPERTYDESCRIPTOR_H_
#define COMMON_PROPERTYDESCRIPTOR_H_

#include <string>
#include <vector>

namespace common {

/**
 * Class used to locally hold details of Properties stored in the Data Provider. 
 */
class PropertyDescriptor	{

public:
	/**
	 * Constructor. Takes vector of data provider IDs (in case of a DynamicProperty),
	 * property name, property type and boolean indicating whether it is dynamic.
	 */
	PropertyDescriptor(const std::vector<unsigned long>& IDs, const std::string& name, const std::string& type, bool isDynamic, bool isSpy);
	
	/**
 	 * Virtual destructor.
 	 */
 	virtual ~PropertyDescriptor();

	/**
	 * Returns the Property name.
	 */
	const std::string& getName() const;
	
	/**
	 * Returns the Property type.
	 */
	const std::string& getType() const;

	/**
	 * Returns the Property in stringified form.
	 */
	virtual std::string toString() const;
	
	/**
	 * Returns the IDs relevant to the Property.
	 */
	const std::vector<unsigned long>& getIDs() const;
	
	/**
	 * Returns true if the Property is dynamic.
	 */
	bool isDynamic() const;

	/**
	 * Returns true if the Property is spy.
	 */
	bool isSpy() const;

private:
	std::vector<unsigned long> _IDs;
	std::string _name;
	std::string _type;
	bool _isDynamic;
	bool _isSpy;
};

} // namespace common

#endif /*COMMON_PROPERTYDESCRIPTOR_H_*/
