/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMON_SERVANTDESCRIPTOR_H_
#define COMMON_SERVANTDESCRIPTOR_H_

#include <string>

namespace common {

/**
 * Class used to locally hold details of servants stored in the data provider. 
 */
class ServantDescriptor	{

public:

	/**
	 * Constructor. Takes the DataProvider ID, the name and servant type.
	 */
	ServantDescriptor(unsigned long ID, const std::string& name, const std::string& type, bool isRoot);
	
	/**
	 * Destructor.
	 */
	virtual ~ServantDescriptor();

	/**
	 * Returns the DataProvider ID of the Servant.
	 */
	unsigned long getID() const;
	
	/**
	 * Returns the name of the Servant.
	 */
	const std::string& getName() const;
	
	/**
	 * Returns the type of the Servant.
	 */
	const std::string& getType() const;

	/**
	 * Returns true if is root.
	 */
	bool isRoot() const;

	/**
	 * Returns the Servant in stringified form.
	 */
	std::string toString() const;

private:
	unsigned long _ID;
	std::string _name;
	std::string _type;
	bool _isRoot;

};

} // namespace command

#endif /*COMMAND_SERVANTDESCRIPTOR_H_*/