/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_DIRECTCORE_H_
#define DATAPROVIDER_DIRECTCORE_H_

#include <common/Application.h>
#include <string>

namespace core {

/**
 * Core application.
 */
class DirectCore : public common::Application {

public :
	/**
	 * Constructor.
	 */
	DirectCore();

	/**
	 * Virtual destructor.
	 */
	virtual ~DirectCore();

	/**
	 * Initialises the application. Initialises the Database and the CommunicationManager
	 * object.
	 */
	virtual void init(int argc, char* argv[]);

	/**
	 * Runs the application.
	 */
	virtual void run();

	/**
	 * Resets the application.
	 */
	virtual void reset();

	/**
	 * Stops the main loop.
	 */
	void shutdown(bool isRemote);

	/**
	 * Returns true if is on error.
	 */
	bool onError() const;

	/**
	 * Shows the current commands.
	 */
	void show();

	/**
	 * Shows the command zone of ID.
	 */
	void commandZone(unsigned long ID);

	/**
	 * Executes the command called commandName.
	 */
	virtual void execute(const std::string& commandName, const std::vector<std::string>& args);

	/**
 	 * Quits the application.
 	 */
	virtual void quit(const std::vector<std::string>& args, bool isRemote);

	/**
	 * Replaces the property names with alias in condition file.
	 */
	void replacePropertyNamesWithAlias();

private:
	void set(unsigned long propertyID, const std::string& type, const std::string& value);

	bool _onError;
	std::string _serverId;

	static const std::string ICS;
	static const std::string SIMULATION;
	static const std::string SIMULATION3D;
	static const std::string VISA;

	void test(unsigned long ID, long timeMs);
};


} // namespace core

#endif /*DATAPROVIDER_DIRECTCORE_H_*/
