/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_BASEVALUEWRAPPER_H_
#define CORE_BASEVALUEWRAPPER_H_

#include <common/base/Exception.h>
#include <string>
#include <set>

namespace core {

/**
 * Class defining the interface for a value wrapper for the BooleanExpression class.
 */
class BaseValueWrapper {

public:
	/**
	 * Exception when the enumerated value does not exist.
	 */
	class NoSuchEnumeratedValueError : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		NoSuchEnumeratedValueError(const std::string& message);
	};

	/**
	 * Exception when the controller is disabled.
	 */
	class DisabledControllerError : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		DisabledControllerError(const std::string& message);
	};

	/**
	 * Exception when the variable is undefined.
	 */
	class UndefinedVariableError : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		UndefinedVariableError(const std::string& message);
	};

	/**
	 * Constructor.
	 */
	BaseValueWrapper();

	/**
	 * Virtual destructor.
	 */
	virtual ~BaseValueWrapper();

	/**
	 * Sets the database ID.
	 */
	void setDatabaseID(unsigned long databaseID);

	/**
	 * Gets the property IDs of the properties it depends on.
	 */
	virtual std::set<unsigned long> getPropertyIDs() = 0;

	/**
	 * Gets the address of the value.
	 */
	double & getValue();

	/**
	 * Updates the value.
	 */
	virtual void updateValue() = 0;

protected:
	double _value;
	unsigned long _databaseID;
};

}

#endif
