/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_ENUMERATEDVALUEWRAPPER_H_
#define CORE_ENUMERATEDVALUEWRAPPER_H_

#include "BaseValueWrapper.h"
#include <string>

namespace core {

/**
 * Class defining a wrapper for the enumerated values.
 */
template<typename Type>
class EnumeratedValueWrapper : public BaseValueWrapper {

	using BaseValueWrapper::NoSuchEnumeratedValueError;

public:
	/**
	 * Constructor.
	 */
	EnumeratedValueWrapper(unsigned long propertyID, const std::string& label);

	/**
	 * Virtual destructor.
	 */
	virtual ~EnumeratedValueWrapper();

	/**
	 * Gets the property IDs of the properties it depends on.
	 */
	virtual std::set<unsigned long> getPropertyIDs();

	/**
	 * Updates the value.
	 */
	virtual void updateValue();

private:
	unsigned long _propertyID;
	std::string _label;
};

}

#include "EnumeratedValueWrapper.inl"

#endif

