/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "EnumeratedValueWrapper.h"
#include "core/dataaccess/DataAccess.h"
#include "core/base/property/DataAccessHelper.h"

namespace core {

template<typename Type>
EnumeratedValueWrapper<Type>::EnumeratedValueWrapper(unsigned long propertyID, const std::string& label) :
	_propertyID(propertyID),
	_label(label) {
}

template<typename Type>
EnumeratedValueWrapper<Type>::~EnumeratedValueWrapper() {
}

template<typename Type>
std::set<unsigned long> EnumeratedValueWrapper<Type>::getPropertyIDs() {
	std::set<unsigned long> result;
	
	//result.insert(_property->getPropertyID());
	
	return result;
}

template<typename Type>
void EnumeratedValueWrapper<Type>::updateValue() {
	
	// Get the IDs of the enumerated labels and values.
	unsigned long enumeratedLabelsID = DataAccess::getInstance()->getEnumeratedLabels(_databaseID, _propertyID);
	unsigned long enumeratedValuesID = DataAccess::getInstance()->getEnumeratedValues(_databaseID, _propertyID);

	// Get the dynamic property size.
	long enumeratedLabelsSize = DataAccess::getInstance()->getDynamicPropertySize(_databaseID, enumeratedLabelsID);

	// Search for the label.
	for (int i = 0; i < enumeratedLabelsSize; ++i) {

		unsigned long labelPropertyID = DataAccess::getInstance()->getDynamicPropertyIDAtIndex(_databaseID, enumeratedLabelsID, i, false);
		std::string label = DataAccess::getInstance()->getStringValue(_databaseID, labelPropertyID);

		// Check the label.
		if (label == _label) {
			unsigned long valuePropertyID = DataAccess::getInstance()->getDynamicPropertyIDAtIndex(_databaseID, enumeratedValuesID, i, false);
			_value = DataAccessHelper<Type>::getValue(_databaseID, valuePropertyID);
			
			return;
		}
	}
	
	// The constant is not found, an exception is thrown.
	throw BaseValueWrapper::NoSuchEnumeratedValueError(std::string("No enumerated value '") + _label + "'");
}

}

