/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_MATHEXPRESSIONRENAMER_H_
#define CORE_MATHEXPRESSIONRENAMER_H_

#include "MathExpression.h"

namespace core {

/**
 * Class defining a renamer for math expressions by replacing property names with their alias.
 */
class MathExpressionRenamer : public MathExpression {

public:
	/**
	 * Constructor.
	 */
	MathExpressionRenamer(const std::string& expression);

	/**
	 * Virtual destructor.
	 */
	virtual ~MathExpressionRenamer();

	/**
	 * Executes the renaming and return the new expression.
	 */
	std::string execute();

//private:
	std::string getNewVariableName(const std::string& variableName, const VariableInfos& variableInfos);

	/**
	 * Callback for mu::Parser.
	 */
	static double * addValue(const char * valueName, void * pUserData);

	struct NamePair {
		std::string name;
		std::string newName;
		double value;
	};

	static bool compareNamePairs(NamePair * p1, NamePair * p2);

	std::vector<NamePair *> m_namePairs;
};

}

#endif
