/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_PROPERTYWRAPPER_H_
#define CORE_PROPERTYWRAPPER_H_

#include <core/base/property/Property.h>
#include "BasePropertyWrapper.h"

namespace core {

/**
 * Class defining a property wrapper for the MathExpression class.
 */
template<typename Type>
class PropertyWrapper : public BasePropertyWrapper {

public:
	/**
	 * Constructor.
	 */
	PropertyWrapper(unsigned long propertyID);

	/**
	 * Virtual destructor.
	 */
	virtual ~PropertyWrapper();

	/**
	 * Gets the property IDs of the properties it depends on.
	 */
	virtual std::set<unsigned long> getPropertyIDs();

	/**
	 * Updates the value.
	 */
	virtual void updateValue();

private:
	std::unique_ptr<Property<Type>> _property;
	unsigned long _servantID;
};

}

#include "PropertyWrapper.inl"

#endif

