/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <core/base/property/Property.h>
#include "BasePropertyWrapper.h"

namespace core {

template<typename Type>
PropertyWrapper<Type>::PropertyWrapper(unsigned long propertyID) :
	_property(new Property<Type>(propertyID)) {

	// Get the servant ID.
	_servantID = DataAccess::getInstance()->getServantIDFromPropertyID(0, propertyID);
}

template<typename Type>
PropertyWrapper<Type>::~PropertyWrapper() {
}

template<typename Type>
std::set<unsigned long> PropertyWrapper<Type>::getPropertyIDs() {
	std::set<unsigned long> result;
	result.insert(_property->getPropertyID());
	return result;
}

template<typename Type>
void PropertyWrapper<Type>::updateValue() {

	// Check if the servant is enabled only outside the init phase.
	if (!DataAccess::getInstance()->isServantEnabledAndInitialised(_databaseID, _servantID)) {

		// Get the servant name.
		std::string servantName = DataAccess::getInstance()->getServantName(_databaseID, _servantID);

		// Throw the exception.
		throw DisabledControllerError("Controller '" + servantName + "' is disabled");
	}

	// Get the server value from the database ID.
	_property->getServerValue(_databaseID);
	_value = static_cast<double>(_property->getValue());
}

}
