/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "UserVariableWrapper.h"
#include <core/variables/UserVariable.h>

using namespace std;

namespace core {

UserVariableWrapper::UserVariableWrapper(const std::string& name, UserVariable* variable) :
	_name(name), _variable(variable) {
}

UserVariableWrapper::~UserVariableWrapper() {
}

std::string UserVariableWrapper::getName() const {
	return _name;
}

std::set<unsigned long> UserVariableWrapper::getPropertyIDs() {

	if (_variable != nullptr) {
		return _variable->getPropertyIDs();
	}

	return set<unsigned long>();
}

void UserVariableWrapper::updateValue() {

	// Throw an exception if the variable does not exist.
	if (_variable == nullptr) {
		throw UndefinedVariableError("Variable '" + _name + "' is undefined");
	}

	_variable->updateValue();
	_value = _variable->getValue();
}

}
