/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_VARIABLEWRAPPER_H_
#define CORE_VARIABLEWRAPPER_H_

#include "BaseValueWrapper.h"

namespace core {

/**
 * Class defining the variable wrapper for the MathExpression class.
 */
class VariableWrapper : public BaseValueWrapper {

public:
	/**
	 * Constructor.
	 */
	VariableWrapper();

	/**
	 * Virtual destructor.
	 */
	virtual ~VariableWrapper();

	/**
	 * Returns true if is initialised.
	 */
	bool isInitialised() const;

	/**
	 * Gets the property IDs of the properties it depends on.
	 */
	virtual std::set<unsigned long> getPropertyIDs();

	/**
	 * Sets the value.
	 */
	void setValue(double value);

	/**
	 * Updates the value.
	 */
	virtual void updateValue();

private:
	bool _initialised;
};

}

#endif /*CORE_VARIABLEWRAPPER_H_*/
