/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_BASEMATHPROPERTY_H_
#define CORE_BASEMATHPROPERTY_H_

#include "BaseProperty.h"
#include <string>

namespace core {

/**
 * Base class for the math property.
 */
class BaseMathProperty : public BaseProperty {

public:
	/**
	 * Constructor.
	 */
	BaseMathProperty(unsigned long propertyID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~BaseMathProperty();

	/**
	 * Gets the type.
	 */
	virtual std::string getType() const = 0;

	/**
	 * Returns true if is expression.
	 */
	bool isExpression() const;

	/**
	 * Sets the expression.
	 */
	void setExpression(const std::string& expression);
	
	/**
	 * Gets the expression.
	 */
	const std::string& getExpression() const;
	
	/**
	 * Initialises the property.
	 */
	virtual void init(unsigned long databaseID) = 0;
	
private:
	bool _isExpression;
	std::string _expression;	
};

}

#endif/*CORE_BASEMATHPROPERTY_H_*/
