/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_BASEPROPERTY_H_
#define CORE_BASEPROPERTY_H_

#include <common/base/Exception.h>
#include <string>

namespace core {

/**
 * Base class for the property.
 */
class BaseProperty {

public:
	/**
	 * Execution error for the set value.
	 */
	class SetValueFailure : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		SetValueFailure(const std::string& message);
	};

	/**
	 * Constructor.
	 */
	BaseProperty(unsigned long propertyID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~BaseProperty();
	
	/**
	 * Gets the property ID.
	 */
	unsigned long getPropertyID() const;
	
	/**
	 * Sets the server value.
	 */
	virtual void setServerValue(unsigned long databaseID) = 0;
	
	/**
	 * Gets the server value.
	 */
	virtual void getServerValue(unsigned long databaseID) = 0;
	
	/**
	 * Gets the value.
	 */
	virtual std::string getStringValue() = 0;

	/**
	 * Gets the formatted value. We cannot get the future value formatted here.
	 */
	virtual std::string getFormattedValue() = 0;

protected:
	unsigned long propertyID() const;
	
private:
	unsigned long _propertyID;
};

}

#endif/*CORE_BASEPROPERTY_H_*/
