/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_MATHPROPERTY_H_
#define CORE_MATHPROPERTY_H_

#include "BaseMathProperty.h"
#include <mutex>

namespace core {

/**
 * Class template defining a math property for the command zone.
 * A mutex has been added to protect against concurrent access of the _value member from:
 * - server which retrieves the value from data provider
 * - client which reads the value
 */
template<typename ValueType>
class MathProperty : public BaseMathProperty {

public:
	/**
	 * Constructor.
	 */
	MathProperty(unsigned long propertyID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~MathProperty();

	/**
	 * Gets the type.
	 */
	virtual std::string getType() const;
	
	/**
	 * Sets the server value.
	 */
	virtual void setServerValue(unsigned long databaseID);
	
	/**
	 * Gets the server value.
	 */
	virtual void getServerValue(unsigned long databaseID);

	/**
	 * Initialises the property.
	 */
	virtual void init(unsigned long databaseID);
 	
 	/**
	 * Sets the value.
	 */
	void setValue(ValueType value);
	
	/**
	 * Gets the value.
	 */
	ValueType getValue();
	
	/**
	 * Gets the formatted value.
	 */
	virtual std::string getFormattedValue();

	/**
	 * Gets the stringified value.
	 */
	virtual std::string getStringValue();
	
private:
	std::mutex _mutex;
	ValueType _value;
	std::string _formattedValue;
};

}

#include "MathProperty.inl" 

#endif/*CORE_MATHPROPERTY_H_*/
