/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DataAccessHelper.h"
#include <core/scheduler/Sender.h>
#include <core/dataaccess/DataAccess.h>
#include <boost/lexical_cast.hpp>

namespace core {

template<typename ValueType>
Property<ValueType>::Property(unsigned long propertyID) :
	BaseProperty(propertyID) {
}

template<typename ValueType>
Property<ValueType>::~Property() {
}
	
template<typename ValueType>
void Property<ValueType>::setServerValue(unsigned long databaseID) {

	ValueType valueCopy;
	{
		std::scoped_lock lock(_mutex);
		valueCopy = _value;
	}

	// blocking call to Sender
	bool success = Sender::setValueOrReset(databaseID, propertyID(), valueCopy, true);
	if (!success) {
		std::string message = std::string("failed to set value ") + common::to<std::string>(valueCopy) + " to " + DataAccess::getInstance()->what(databaseID, propertyID());
		throw BaseProperty::SetValueFailure(message);
	}
}

template<typename ValueType>
void Property<ValueType>::getServerValue(unsigned long databaseID) {

	std::scoped_lock lock(_mutex);
	_value = DataAccessHelper<ValueType>::getValue(databaseID, propertyID());
	_formattedValue = DataAccess::getInstance()->getFormattedValue(databaseID, propertyID());
}

template<typename ValueType>
void Property<ValueType>::setValue(ValueType value) {

	std::scoped_lock lock(_mutex);
	_value = value;
}

template<typename ValueType>
ValueType Property<ValueType>::getValue() {

	std::scoped_lock lock(_mutex);
	return _value;
}

template<typename ValueType>
std::string Property<ValueType>::getStringValue() {

	std::scoped_lock lock(_mutex);
	return boost::lexical_cast<std::string>(_value);
}

template<typename ValueType>
std::string Property<ValueType>::getFormattedValue() {

	std::scoped_lock lock(_mutex);
	return _formattedValue;
}

}
