/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SERVANTCLONER_H_
#define CORE_SERVANTCLONER_H_

#include <string>
#include <map>
#include <common/base/BaseTypes.h>

namespace dataprovider {

class ServantAccessorDirectImpl;

}

namespace core {

/**
 * Singleton class providing an access to the servant cloner.
 */
class ServantCloner {

public:
	/**
	 * Gets the instance.
	 */
	static ServantCloner* getInstance();

	/**
	 * Initialises the instance.
	 */
	static void initInstance(dataprovider::ServantAccessorDirectImpl* proxy);

	/**
	 * Clones the controller.
	 */
	unsigned long cloneController(unsigned long controllerID);

	unsigned long cloneOrGetController(unsigned long controllerID);

	bool removeCloneController(unsigned long cloneID);

	void removeCloneControllers();

private:
	ServantCloner(dataprovider::ServantAccessorDirectImpl* proxy);
	~ServantCloner();

	dataprovider::ServantAccessorDirectImpl* _proxy;

	static ServantCloner* _instance;

	std::map<unsigned long, unsigned long> m_SaveCloneIDs;
	std::map<unsigned long, unsigned long> m_SaveCloneIDsCounter;
};

}

#endif
