/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CheckExpressionCommandBox.h"
#include <memory>

using namespace std;

namespace core {

CheckExpressionCommandBox::~CheckExpressionCommandBox() {
}

common::ExpressionStatus CheckExpressionCommandBox::checkStaticExpression(const std::string & expression, common::ExpressionError& error, std::string& message) {

	try {
		unique_ptr<MathExpression> mathExpression(new MathExpression(expression));

		mathExpression->init();
		mathExpression->checkStaticValue();

		return common::VALID_EXPRESSION;
	}
	catch (const MathExpression::ParserError & e) {
		error = common::PARSER_ERROR;
		message = e.getMessage();
	}
	catch (const MathExpression::UndefinedVariableError & e) {
		error = common::UNDEFINED_VARIABLE_ERROR;
		message = e.getMessage();
	}
	catch (const MathExpression::DisabledControllerError & e) {
		error = common::DISABLED_CONTROLLER_ERROR;
		message = e.getMessage();
	}
	catch (const MathExpression::NoSuchEnumeratedValueError & e) {
		error = common::ENUMERATED_VALUE_ERROR;
		message = e.getMessage();
	}

	return common::INVALID_EXPRESSION;
}

}
