/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDBOXFACTORY_H_
#define CORE_COMMANDBOXFACTORY_H_

#include <string>
#include <common/base/Exception.h>
#include "AtomicCommandBox.h"
#include "ScanCommandBox.h"
#include "ForLoopCommandBox.h"
#include <core/commandzone/control/ControlCommandBox.h>
#include "generic/PropertyGenericCommandBox.h"

namespace core {

/**
 * Command box factory.
 */
class CommandBoxFactory {

public:
	/**
	 * No such command exception.
	 */
	class NoSuchCommandException : public common::Exception {
	
	public:
		/**
		 * Constructor.
		 */
		NoSuchCommandException(unsigned long commandID);
	};
	
	/**
	 * Gets the instance.
	 */
	static CommandBoxFactory* getInstance();

	/**
	 * Creates an atomic command box.
	 */
	AtomicCommandBox* createAtomicCommandBox(const std::string& controllerName, bool isSettings);
	
	/**
	 * Creates a scan command box.
	 */
	ScanCommandBox* createScanCommandBox(const std::string& scanName, const std::string& fullpath);
	
	/**
	 * Creates a for loop command box.
	 */
	ForLoopCommandBox* createForLoopCommandBox(const std::string& variableName);

	/**
	 * Creates an atomic command box.
	 */
	CommandBox* createGenericCommandBox(PropertyGenericCommandBox::BoxType type);

	/**
	 * Creates a control command box.
	 */
	ControlCommandBox* createControlCommandBox(ControlCommandBox::BoxType type);

	/**
	 * Clones an atomic command box.
	 */
	AtomicCommandBox* cloneAtomicCommandBox(const CommandBox* commandBox);

	/**
	 * Clones a scan command box.
	 */
	ScanCommandBox* cloneScanCommandBox(const CommandBox* commandBox);

	/**
	 * Clones a for loop command box.
	 */
	ForLoopCommandBox* cloneForLoopCommandBox(const CommandBox* commandBox);

	/**
	 * Clones a generic command box.
	 */
	CommandBox* cloneGenericCommandBox(const CommandBox* commandBox);

	/**
	 * Clones a control command box.
	 */
	ControlCommandBox* cloneControlCommandBox(const CommandBox* commandBox);

private:
	CommandBoxFactory();
	~CommandBoxFactory();

	static CommandBoxFactory* _instance;	

};

}


#endif /*CORE_COMMANDBOXFACTORY_H_*/
