/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDBOXPARENT_H_
#define CORE_COMMANDBOXPARENT_H_

#include <vector>
#include <string>

namespace core {

class CommandBox;

/**
 * Class defining a command box parent.
 */
class CommandBoxParent {

public:
	/**
	 * Constructor.
	 */
	CommandBoxParent();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CommandBoxParent();

	/**
	 * Notifies the container the execution has begun.
	 */
	virtual void onExecutionBegin() = 0;
	
	/**
	 * Notifies the container that the execution has ended.
	 */
	virtual void onExecutionEnd() = 0;
	
	/**
	 * Notifies the container that the execution has been paused.
	 */
	virtual void onExecutionPause() = 0;

	/**
	 * Notifies the container that one if its command boxes has finished.
	 */
	virtual void onCommandBoxFinished(CommandBox* commandBox) = 0;

	/**
	 * Notifies the container that the execution progress has changed.
	 */
	virtual void onContainerProgressChanged(double progression) = 0;

	/**
	 * Adds the variable names to the list.
	 */
	virtual void addVariableNamesToList(std::vector<std::string>& variableNames) = 0;

	/**
	 * Returns true if there is no command to execute anymore.
	 * Only ForLoopCommandBox is able to return true.
	 */
	virtual bool hasEnded() = 0;
};

}

#endif /*CORE_COMMANDBOXPARENT_H_*/
