/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDSEQUENCER_H_
#define CORE_COMMANDSEQUENCER_H_

#include <common/base/Exception.h>
#include <thread>

namespace common {

template<typename ItemType>
class Queue;

}

namespace core {

class CommandBox;
class Message;
class Shutdown;
class ExecuteCommandBox;

/**
 * Class defining a command sequencer to sequentialise the execution of the command boxes.
 */
class CommandSequencer {

public:
	/**
	 * Gets the instance.
	 */
	static CommandSequencer* getInstance();

	/**
	 * Initialises the instance.
	 */
	static void initInstance();

	/**
	 * Resets the instance.
	 */
	static void resetInstance();
	
	/**
	 * Posts a new command box for both property setting and command execution.
	 */
	void executeCommandBox(CommandBox* commandBox);
	
	/**
	 * Posts a new command box for initialising execution.
	 */
	void initialiseCommandBoxExecution(CommandBox* commandBox); 
	
	/**
	 * Posts a new command box for launching the command.
	 */
	void launchCommandBox(CommandBox* commandBox); 
	
private:
	CommandSequencer();
	~CommandSequencer();
	void init();
	void reset();
	
	void processMessage(Message* message);
	void processMessage(Shutdown* message);
	void processMessage(ExecuteCommandBox* message);
	
	static void loop();
		
	static CommandSequencer* _instance;
	bool _running;
	std::thread* _thread;
	common::Queue<Message>* _messageQueue;
};

}

#endif /*CORE_COMMANDSEQUENCER_H_*/
