/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_ESTIMATEDTMEMANAGER_H_
#define CORE_ESTIMATEDTMEMANAGER_H_

#include <cameo/api/cameo.h>
#include <thread>
#include <atomic>

namespace common {
	template<typename ValueType>
	class Queue;
}

namespace core {

class Message;
class Shutdown;
class CommandZoneEvent;
class CommandZoneEventPublisher;
class ModifiableCommandBoxContainer;

/**
 * Class defining a command zone event server.
 */
class EstimatedTimeManager {

public:

	/**
	 * Compute estimated time error.
	 */
	class ComputeEstimatedTimeError {
	};

	/**
	 * Gets the instance.
	 */
	static EstimatedTimeManager* getInstance();

	/**
	 * Initialises the instance.
	 */
	static void initInstance(CommandZoneEventPublisher* commandZoneEventPublisher);

	/**
	 * Resets the instance.
	 */
	static void resetInstance();

	/**
	 * Notifies that the command zone compute a new estimated time.
	 */
	void newEstimatedTime();

	void getEstimatedTime();

	void setActivationState(bool state);

	void cloneCommandZone();

private:
	EstimatedTimeManager(CommandZoneEventPublisher* commandZoneEventPublisher);
	virtual ~EstimatedTimeManager();

	void init();
	void reset();

	static void run();

	void processMessage(Message* message);
	void processMessage(Shutdown* message);
	void processMessage(CommandZoneEvent* commandZoneEvent);

	void computeEstimatedTime();

	static EstimatedTimeManager* _instance;

	CommandZoneEventPublisher* _commandZoneEventPublisher;
	common::Queue<Message>* _queue;
	std::thread* _thread;
	static bool _running;
	bool _activated;
	bool _useEstimatedTime;
	std::atomic_bool _estimatedRunning;
	double _estimatedTime;
	std::unique_ptr<cameo::coms::Requester> _requester;
	std::unique_ptr<core::ModifiableCommandBoxContainer> _cloneContainer;

	static const std::string USE_ESTIMATED_TIME;

};


} // namespace core

#endif /*CORE_COMMANDZONEEVENTSERVER_H_*/
