/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_MODIFIABLECOMMANDBOXCONTAINER_H_
#define CORE_MODIFIABLECOMMANDBOXCONTAINER_H_

#include "CommandBoxContainer.h"
#include <common/base/Exception.h>

namespace core {

/**
 * Class defining a modifiable command box container.
 */
class ModifiableCommandBoxContainer : public CommandBoxContainer {

public:
	/**
	 * Command box not found exception.
	 */
	class CommandBoxNotFound : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		CommandBoxNotFound(unsigned long commandBoxID);
	};

	/**
	 * Constructor.
	 */
	ModifiableCommandBoxContainer(CommandBoxParent* parent);

	/**
	 * Virtual destructor.
	 */
	virtual ~ModifiableCommandBoxContainer();

	/**
	 * clone.
	 */
	ModifiableCommandBoxContainer* clone(CommandBoxParent* parent);

	/**
	 * Gets the first command box. Returns 0 if the container is empty.
	 */
	CommandBox* getFirstCommandBox();

	/**
	 * Adds a new command box at the beginning.
	 * Returns the ID of the created command box.
	 * throw CommandBoxNotFound
	 */
	unsigned long addCommandBoxAtBeginning(CommandBox* commandBox);

	/**
	 * Adds a new command box above or below another one.
	 * Returns the ID of the created command box.
	 * throw CommandBoxNotFound
	 */
	unsigned long addCommandBox(CommandBox* commandBox, unsigned long commandBoxToShiftID, bool above);

	/**
	 * throw CommandBoxNotFound
	 */
	void moveCommandBoxToEnd(unsigned long commandBoxID);

	/**
	 * throw CommandBoxNotFound
	 */
	void moveCommandBox(unsigned long commandBoxToMoveID, unsigned long commandBoxToShiftID, bool above);

	/**
	 * Deletes the command box.
	 */
	void deleteCommandBox(unsigned long commandBoxID);

	/**
	 * Deletes the finished command boxes.
	 */
	std::vector<unsigned long> deleteFinishedCommandBoxes();

	/**
	 * Deletes the finished command boxes.
	 */
	std::vector<unsigned long> deleteFinishedCommandBoxes(int commandBoxesToDeleteCounter, int& deletedCommandBoxesCounter);

};

}

#endif /*CORE_MODIFIABLECOMMANDBOXCONTAINER_H_*/
