/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ParallelGroup.h"
#include "CommandSequencer.h"
#include "CommandBox.h"
#include <iostream>

using namespace std;

namespace core {

ParallelGroup::ParallelGroup() :
	_lastCommandBox(0),
	_stopped(false),
	_isCompleted(false){
}

ParallelGroup::~ParallelGroup() {
}

void ParallelGroup::addCommandBox(CommandBox* commandBox) {
	_commandBoxes[commandBox].started = false;
	_commandBoxes[commandBox].terminated = false;
	_lastCommandBox = commandBox;
}

CommandBox* ParallelGroup::getLastCommandBox() {
	return _lastCommandBox;
}

void ParallelGroup::reset() {
	_commandBoxes.clear();
	_lastCommandBox = 0;
	_isCompleted = false;
}
	
void ParallelGroup::execute() {

	// write all properties first
	for (map<CommandBox*, CommandBoxStatus>::iterator b = _commandBoxes.begin(); b != _commandBoxes.end(); b++) {
		if (!b->second.started) {
			CommandSequencer::getInstance()->initialiseCommandBoxExecution(b->first);
		}	
	}	

	// execute commands after
	for (map<CommandBox*, CommandBoxStatus>::iterator b = _commandBoxes.begin(); b != _commandBoxes.end(); b++) {
		if (!b->second.started) {
			b->second.started = true;
			CommandSequencer::getInstance()->launchCommandBox(b->first);
		}	
	}	
}

void ParallelGroup::onCommandBoxFinished(CommandBox* commandBox) {
	_commandBoxes[commandBox].terminated = true;
	
	if (allCommandsFinished()) {
		_isCompleted = true;
	}
}
	
bool ParallelGroup::isCompleted() const {
	return _isCompleted;
}

bool ParallelGroup::allCommandsFinished() const {
		
	for (map<CommandBox*, CommandBoxStatus>::const_iterator b = _commandBoxes.begin(); b != _commandBoxes.end(); b++) {
		if (!b->second.terminated) {
			return false;
		}
	}
	
	return true;
}

double ParallelGroup::calculateMinimumProgression() {
	
	double minimumProgression = 100.0;
	for (map<CommandBox*, CommandBoxStatus>::const_iterator b = _commandBoxes.begin(); b != _commandBoxes.end(); b++) {
		CommandBox* commandBox = b->first;
		if (commandBox->getProgression() < minimumProgression) {
			minimumProgression = commandBox->getProgression();
		}
	}

	return minimumProgression;
}

}
