/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "MathParserCommandPropertyState.h"
#include "CommandZoneEventServer.h"
#include <core/transportlayer/dataprovider/PropertyAccessor.h>
#include <core/dataaccess/DataNotificationRouter.h>

namespace core {

template<typename ValueType>
struct PropertyCommandBoxHelper {

	static void setMathParserPropertyValue(MathParserCommandPropertyState* propertyState, unsigned long propertyID, ValueType value);
	static ValueType getMathParserPropertyValue(MathParserCommandPropertyState* propertyState, unsigned long propertyID);
};

template<typename ValueType>
void PropertyCommandBoxHelper<ValueType>::setMathParserPropertyValue(MathParserCommandPropertyState* propertyState, unsigned long propertyID, ValueType value) {

	// do nothing, may never happen!
	std::cout << "setMathParserPropertyValue<ValueType>::setMathParserPropertyValue" << std::endl;
}

template<typename ValueType>
ValueType PropertyCommandBoxHelper<ValueType>::getMathParserPropertyValue(MathParserCommandPropertyState* propertyState, unsigned long propertyID) {

	// do nothing, may never happen!
	return ValueType();
}

template<>
struct PropertyCommandBoxHelper<std::string> {

	static void setMathParserPropertyValue(MathParserCommandPropertyState* propertyState, unsigned long propertyID, std::string value);
	static std::string getMathParserPropertyValue(MathParserCommandPropertyState* propertyState, unsigned long propertyID);
};

template<typename ValueType>
void PropertyCommandBox::setPropertyValue(unsigned long propertyID, ValueType value) {

	try {
		if (!_propertyState->hasProperty(propertyID)) {
			// subscribe to property changes
			DataNotificationRouter::getInstance()->addPropertyChangeSubscriber(propertyID, this);
		}

		PropertyCommandBoxHelper<ValueType>::setMathParserPropertyValue(dynamic_cast<MathParserCommandPropertyState*>(_propertyState.get()), propertyID, value);

		CommandZoneEventServer::getInstance()->commandBoxPropertyChanged(getID(), propertyID);

	} catch (PropertyAccessor::NullPropertyReferenceException& e) {
		throw link(NullPropertyReferenceException(), e);
	} catch (PropertyAccessor::NoSuchPropertyException& e) {
		throw link(NoSuchPropertyException(), e);
	} catch (PropertyAccessor::BadPropertyTypeException& e) {
		throw link(BadPropertyType(), e);
	} catch (PropertyState::BadPropertyType& e) {
		throw link(BadPropertyType(), e);
	}
}

template<typename ValueType>
ValueType PropertyCommandBox::getPropertyValue(unsigned long propertyID) {

	try {
		if (!_propertyState->hasProperty(propertyID)) {
			// subscribe to property changes
			DataNotificationRouter::getInstance()->addPropertyChangeSubscriber(propertyID, this);
		}

		return PropertyCommandBoxHelper<ValueType>::getMathParserPropertyValue(dynamic_cast<MathParserCommandPropertyState*>(_propertyState.get()), propertyID);

	} catch (PropertyAccessor::NullPropertyReferenceException& e) {
		throw link(NullPropertyReferenceException(), e);
	} catch (PropertyAccessor::NoSuchPropertyException& e) {
		throw link(NoSuchPropertyException(), e);
	} catch (PropertyAccessor::BadPropertyTypeException& e) {
		throw link(BadPropertyType(), e);
	} catch (PropertyState::BadPropertyType& e) {
		throw link(BadPropertyType(), e);
	}
}

}
