/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SaveBoxPropertyManager.h"

#include <common/output/OutputStream.h>
#include <common/base/ServerProperties.h>
#include <core/transportlayer/MultiThreadCommunication.h>
#include <core/transportlayer/dataprovider/StateAccessor.h>
#include <iostream>

using namespace std;
using namespace common;

namespace core {

SaveBoxPropertyManager::NoSuchPropertyException::NoSuchPropertyException(unsigned long propertyID)
: Exception("core::SaveBoxPropertyManager::NoSuchPropertyException", string("unable to find property <") + to<string>(propertyID) + ">") {
}

SaveBoxPropertyManager* SaveBoxPropertyManager::m_instance = nullptr;


SaveBoxPropertyManager::SaveBoxPropertyManager() {
}

SaveBoxPropertyManager::~SaveBoxPropertyManager() {
}

SaveBoxPropertyManager* SaveBoxPropertyManager::getInstance() {
	return m_instance;
}

void SaveBoxPropertyManager::initInstance() {

	if (m_instance == nullptr) {
		m_instance = new SaveBoxPropertyManager();
		m_instance->init();
	}
}

void SaveBoxPropertyManager::resetInstance() {

	delete m_instance;
	m_instance = nullptr;
}

void SaveBoxPropertyManager::init() {
}


string SaveBoxPropertyManager::getPropertySavedInBoxValue(unsigned long propertyID) {
	if (_lastValues.find(propertyID)) {
		return _lastValues.get(propertyID);
	}
	throw NoSuchPropertyException(propertyID);
}

void SaveBoxPropertyManager::setPropertySavedInBoxValue(unsigned long propertyID, std::string value) {
	_lastValues.insert(propertyID, value);
}


}
