/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SAVEBOXPROPERTYMANAGER_H_
#define CORE_SAVEBOXPROPERTYMANAGER_H_

#include <common/base/CommonTypes.h>
#include <common/base/HashMap.h>
#include <string>

namespace core {

class SaveBoxPropertyManager {

public:
	/**
	 * Returns the instance.
	 */
	static SaveBoxPropertyManager* getInstance();

	/**
	 * Initialises the instance.
	 */
	static void initInstance();

	/**
	 * resets the instance.
	 */
	static void resetInstance();


	std::string getPropertySavedInBoxValue(unsigned long propertyID);
	void setPropertySavedInBoxValue(unsigned long propertyID, std::string value);

	/**
	 * No such Property exception.
	 */
	class NoSuchPropertyException : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		NoSuchPropertyException(unsigned long propertyID);
	};


private:
	common::HashMap<unsigned long, std::string> _lastValues;


	SaveBoxPropertyManager();
	~SaveBoxPropertyManager();

	void init();

	static SaveBoxPropertyManager* m_instance;

};

}

#endif
