/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_BREAKCOMMANDBOX_H_
#define CORE_BREAKCOMMANDBOX_H_

#include "ControlCommandBox.h"

namespace core {

class ForLoopCommandBox;

/**
 * Class defining an if command box.
 */
class BreakCommandBox : public ControlCommandBox {

public:
	/**
	 * Constructor.
	 */
	BreakCommandBox();

	/**
	 * Virtual destructor.
	 */
	virtual ~BreakCommandBox();

	/**
	 * Constructor.
	 */
	BreakCommandBox* clone() const;

	/**
	 * Returns true if the command box can run.
	 */
	virtual bool canRun() const;

	/**
	 * Sets the database ID to be used during the execution.
	 */
	virtual void setExecutionDatabaseID(unsigned long databaseID);

	/**
	 * Execute the command box.
	 */
	virtual void execute();

	/**
	 * Initialises execution.
	 */
	virtual void initialiseExecution();

	/**
	 * Launches the command associated with the command box.
	 */
	virtual void launchCommand();

	/**
	 * Stops the command box.
	 */
	virtual void stop();

	/**
	 * Stops the command box at end.
	 */
	virtual void stopAtEnd();

	/**
	 * Pauses the command box.
	 */
	virtual void pause();

	/**
	 * Pauses the command box at end.
	 */
	virtual void pauseAtEnd();

	/**
	 * Restarts the command box.
	 */
	virtual void restart();

	/**
	 * Resets the command box.
	 */
	virtual void reset();

	/**
	 * Adds the variable names to the list.
	 */
	virtual void addVariableNamesToList(std::vector<std::string>& variableNames);

	/**
	 * Needed for for loops. sets the given variable name with the value.
	 * Iterates through command containers if necessary.
	 */
	virtual void setVariable(const std::string& variableName, double value);

	/**
	 * Changes the variable name and updates any expressions that use it.
	 */
	virtual void changeVariableName(const std::string& formerName, const std::string& newName);

	/**
	 * Sets inside for loop.
	 */
	virtual void setInsideForLoop();

	/**
	 * Prints the content of the box.
	 */
	virtual std::string toXMLString(const std::string& space = std::string()) const;

	/**
	 * Prints the content of the box.
	 */
	virtual std::string toString(const std::string& space = std::string()) const;
	
private:
	std::string getName();
	ForLoopCommandBox* getForLoopParent();
};

}

#endif /*CORE_BREAKCOMMANDBOX_H_*/
