/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PrintCommandBox.h"
#include <core/log/LogHandler.h>
#include <core/log/SimpleLog.h>
#include <common/base/Convert.h>

using namespace std;
using namespace common;

namespace core {

PrintCommandBox::PrintCommandBox(bool forEstimatedTime) :
	PropertyGenericCommandBox(GenericCommandBox::PRINT, forEstimatedTime) {
}

PrintCommandBox::~PrintCommandBox() {
}

PrintCommandBox* PrintCommandBox::clone() const {
	PrintCommandBox* pcb = new PrintCommandBox(true);
	pcb->PropertyGenericCommandBox::copy(this);
	return pcb;
}

void PrintCommandBox::executeCommand() {

	// write log
	LogHandler::getInstance()->write(SimpleLog(common::LOG_INFO, "Print           --> " + _propertyName));
}

std::string PrintCommandBox::toXMLString(const std::string& space) const {

	return space + "<print id=\"" + to<string>(getID()) + "\" logId=\"" + to<string>(logID()) + "\" controllerId=\"" + to<string>(_controllerID) + "\"/>";
}

std::string PrintCommandBox::toString(const std::string& space) const {

	return space + "print " + _controllerName;
}

}
