/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SetZeroCommandBox.h"
#include <core/dataaccess/DataAccess.h>
#include <core/scheduler/Sender.h>
#include <core/scheduler/Scheduler.h>
#include <core/log/LogHandler.h>
#include <core/log/SimpleLog.h>
#include <common/base/Convert.h>

using namespace std;
using namespace common;

namespace core {

const std::string SetZeroCommandBox::REFERENCE_POSITION_PROPERTY = "reference_position";
const std::string SetZeroCommandBox::WRITE_PARAMS_COMMAND = "writeparams";

SetZeroCommandBox::SetZeroCommandBox(bool forEstimatedTime) :
	PropertyGenericCommandBox(GenericCommandBox::SETZERO, forEstimatedTime) {

	setPropertyName(REFERENCE_POSITION_PROPERTY);
}

SetZeroCommandBox::~SetZeroCommandBox() {
}

SetZeroCommandBox* SetZeroCommandBox::clone() const {
	SetZeroCommandBox* szcb = new SetZeroCommandBox(true);
	szcb->PropertyGenericCommandBox::copy(this);
	return szcb;
}

void SetZeroCommandBox::executeCommand() {

	// Getting the property ID
	unsigned long controllerID = DataAccess::getInstance()->getServantID(_controllerName);

	try {
		unsigned long commandID = DataAccess::getInstance()->getCommandID(getExecutionDatabaseID(), controllerID, WRITE_PARAMS_COMMAND);

		if (Scheduler::getInstance()->canRun(getExecutionDatabaseID(), commandID)) {
			RequestReply reply = Sender::newCommand(getExecutionDatabaseID(), commandID, false, true);
			if (reply.getValue() == RequestReply::REJECT) {
				common::err << DataAccess::getInstance()->what(getExecutionDatabaseID(), commandID) << " rejected : " << reply.getReason() << common::eol;
			}
		}

	} catch (const Exception& e) {
		// do nothing
	}

	// write log
	LogHandler::getInstance()->write(SimpleLog(common::LOG_INFO, "Set zero        --> " + _controllerName + " " + _value));
}


std::string SetZeroCommandBox::toXMLString(const std::string& space) const {

	return space + "<set_zero id=\"" + to<string>(getID()) + "\" logId=\"" + to<string>(logID()) + "\" controllerId=\"" + to<string>(_controllerID) + "\"/>";
}

std::string SetZeroCommandBox::toString(const std::string& space) const {

	return space + "setzero " + _controllerName + " = " + _value;
}


}
