/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "VariableCommandBox.h"
#include "../CommandSequencer.h"
#include "../CommandBoxContainer.h"
#include "../CommandZoneEventServer.h"
#include "../CommandZone.h"
#include <core/variables/UserVariableManager.h>
#include <core/variables/UserVariable.h>
#include <core/log/LogHandler.h>
#include <core/log/SimpleLog.h>
#include <core/log/CommandZoneContentLog.h>
#include <common/base/Convert.h>

using namespace std;
using namespace common;

namespace core {

VariableCommandBox::VariableCommandBox(bool forEstimatedTime) :
	CommandBox(forEstimatedTime),
	GenericCommandBox(GenericCommandBox::VARIABLE) {
}

VariableCommandBox::~VariableCommandBox() {
}

VariableCommandBox* VariableCommandBox::clone() const {
	VariableCommandBox* vcb = new VariableCommandBox(true);
	vcb->CommandBox::copy(this);
	vcb->GenericCommandBox::copy(this);
	return vcb;
}

CommandBox::CommandBoxType VariableCommandBox::getType() const {
	return CommandBox::GENERIC;
}

void VariableCommandBox::setVariableName(const std::string& name) {
	m_variableName = name;
}

std::string VariableCommandBox::getVariableName() const {
	return m_variableName;
}

void VariableCommandBox::setExpression(const std::string& expression) {
	m_expression = expression;
}

std::string VariableCommandBox::getExpression() const {
	return m_expression;
}


bool VariableCommandBox::canRun() const {
	return true;
}

void VariableCommandBox::setExecutionDatabaseID(unsigned long databaseID) {

}

void VariableCommandBox::execute() {

	initialiseExecution();
	launchCommand();
}

void VariableCommandBox::initialiseExecution() {

	// Sets the content of the command zone to the data provider
	LogHandler::getInstance()->write(CommandZoneContentLog(toXMLString()));
}

void VariableCommandBox::launchCommand() {

	setRunning();

	executeCommand();

	setFinished();
}

void VariableCommandBox::stop() {

}

void VariableCommandBox::stopAtEnd() {

}

void VariableCommandBox::pause() {

}

void VariableCommandBox::pauseAtEnd() {

}

void VariableCommandBox::restart() {

}

void VariableCommandBox::reset() {
	setConstruction();
}

void VariableCommandBox::addVariableNamesToList(std::vector<std::string>& variableNames) {
	parent()->addVariableNamesToList(variableNames);
}

void VariableCommandBox::setVariable(const std::string& variableName, double value) {

}

void VariableCommandBox::changeVariableName(const std::string& formerName, const std::string& newName) {

}

bool VariableCommandBox::referencesUserVariable(const std::string& variableName) {
	return false;
}

std::set<std::string> VariableCommandBox::getReferencedUserVariables() {
	return set<string>();
}

void VariableCommandBox::setInsideForLoop() {

}

void VariableCommandBox::executeCommand() {

	if (m_variableName == "") {
		LogHandler::getInstance()->write(SimpleLog(common::LOG_ERROR, "Set variable error: cannot set empty variable to " + m_expression));
		return;
	}

	try {
		core::UserVariableManager::getInstance()->editUserVariable(m_variableName, m_expression);

		// Write log.
		LogHandler::getInstance()->write(SimpleLog(common::LOG_INFO, "Set variable " + m_variableName + " := " + m_expression));
	}
	catch (core::UserVariableManager::NoSuchUserVariableException const & e) {
//		LogHandler::getInstance()->write(SimpleLog(common::LOG_INFO, "Set variable    --> " + m_variableName + " " + m_expression));
	}
	catch (core::UserVariableManager::InvalidExpressionException const & e) {
//		responseMessage.set_error(common::Error::INVALID_EXPRESSION);
//		responseMessage.set_message(e.getMessage());
	}
	catch (core::UserVariableManager::DisabledUserVariableException const & e) {
//		responseMessage.set_error(common::Error::DISABLED_VARIABLE);
//		responseMessage.set_message(e.getMessage());
	}

}

std::string VariableCommandBox::toXMLString(const std::string& space) const {

	return space + "<set_variable id=\"" + to<string>(getID()) + "\" logId=\"" + to<string>(logID()) + "\" variableName=\"" + m_variableName + "\"/>";
}

std::string VariableCommandBox::toString(const std::string& space) const {

	return space + "set variable " + m_variableName + " = " + m_expression;
}


}
