/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_ALARM_H_
#define CORE_ALARM_H_

#include "Condition.h"
#include <common/base/TimeCondition.h>
#include <thread>

namespace core {

/**
 * Class defining an error.
 */
class Alarm : public Condition {

public:
	/**
	 * Constructor.
	 */
	Alarm(const std::string& name);

	/**
	 * Destructor.
	 */
	virtual ~Alarm();

	/**
	 * Sets on or off.
	 */
	virtual void setOn(bool value);

	/**
	 * Sets the activation delay.
	 */
	void setActivationDelay(int delay);

	/**
	 * Gets the activation delay.
	 */
	int getActivationDelay();

	/**
	 * Sets the command zone.
	 */
	void setStopCommandZone(bool value);

	/**
	 * Returns true if stops the command zone.
	 */
	bool isStopCommandZone();

	/**
	 * Checks the error.
	 */
	virtual void check();

	/**
	 * Gets the notification subject.
	 */
	virtual std::string getNotificationSubject();

	/**
	 * Gets the string type.
	 */
	virtual std::string getStringType();

	/**
	 * Returns the stringified content of the condition.
	 */
	virtual std::string toString(const std::string& space);

	/**
	 * Returns the xml stringified content of the condition.
	 */
	virtual std::string toXMLString(const std::string& space);

	/**
	 * Converts the condition to an element.
	 */
	virtual ConditionElement * toConditionElement();

private:
	void delay();

	int _activationDelay;
	bool _stopCommandZone;
	common::TimeCondition * _delayCondition;
	std::thread * _delayThread;
};

}

#endif /* CORE_ALARM_H_ */
