/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDZONETERMINATION_H_
#define CORE_COMMANDZONETERMINATION_H_

#include "Warning.h"
#include <core/transportlayer/core/commandzone/CommandZoneEventSubscriber.h>

namespace core {

class CommandZoneExecutionEvent;

/**
 * Class defining a command zone termination warning.
 */
class CommandZoneTermination : public Warning, public CommandZoneEventSubscriber {

public:
	/**
	 * Constructor.
	 */
	CommandZoneTermination();

	/**
	 * Destructor.
	 */
	virtual ~CommandZoneTermination();

	/**
	 * Returns true if is removable.
	 */
	virtual bool isRemovable() const;

	/**
	 * Sets on or off.
	 */
	virtual void setOn(bool value);

	/**
	 * Checks the error.
	 */
	virtual void check();

	/**
	 * Gets the notification subject.
	 */
	virtual std::string getNotificationSubject();

	/**
	 * Returns the ID of the subscriber.
	 */
	virtual unsigned long getID();

	/**
	 * Notifies the subscriber
	 */
	virtual void notify(CommandZoneEvent* commandZoneEvent);

	/**
	 * Returns the stringified content of the condition.
	 */
	virtual std::string toString(const std::string& space);

	/**
	 * Returns the xml stringified content of the condition.
	 */
	virtual std::string toXMLString(const std::string& space);

	/**
	 * Converts the condition to an element.
	 */
	virtual ConditionElement * toConditionElement();

	/**
	 * Name of the condition.
	 */
	static const std::string NAME;

private:
	void notify(CommandZoneExecutionEvent* commandZoneEvent);

	unsigned long _ID;
};

}

#endif /* CORE_COMMANDZONETERMINATION_H_ */
