/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_WARNING_H_
#define CORE_WARNING_H_

#include "Condition.h"

namespace core {

/**
 * Class defining a warning.
 */
class Warning : public Condition {

public:
	/**
	 * Constructor.
	 */
	Warning(const std::string& name);

	/**
	 * Constructor with type.
	 */
	Warning(common::ConditionType type, const std::string& name);

	/**
	 * Destructor.
	 */
	virtual ~Warning();

	/**
	 * Sets on or off.
	 */
	virtual void setOn(bool value);

	/**
	 * Checks the error.
	 */
	virtual void check();

	/**
	 * Gets the notification subject.
	 */
	virtual std::string getNotificationSubject();

	/**
	 * Gets the string type.
	 */
	virtual std::string getStringType();

	/**
	 * Returns the stringified content of the condition.
	 */
	virtual std::string toString(const std::string& space);

	/**
	 * Returns the xml stringified content of the condition.
	 */
	virtual std::string toXMLString(const std::string& space);

	/**
	 * Converts the condition to an element.
	 */
	virtual ConditionElement * toConditionElement();
};

}

#endif /* CORE_WARNING_H_ */
