/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_WATCHDOG_H_
#define CORE_WATCHDOG_H_

#include "Condition.h"
#include <thread>

namespace core {

/**
 * Class defining a watchdog.
 */
class Watchdog : public Condition {

public:
	/**
	 * Constructor.
	 */
	Watchdog(const std::string& name);

	/**
	 * Destructor.
	 */
	virtual ~Watchdog();

	/**
	 * Sets on or off.
	 */
	virtual void setOn(bool value);

	/**
	 * Sets the actions.
	 */
	void setActions(const std::string& actions);

	/**
	 * Gets the actions.
	 */
	const std::string& getActions();

	/**
	 * Checks the error.
	 */
	virtual void check();

	/**
	 * Gets the notification subject.
	 */
	virtual std::string getNotificationSubject();

	/**
	 * Gets the string type.
	 */
	virtual std::string getStringType();

	/**
	 * Returns the stringified content of the condition.
	 */
	virtual std::string toString(const std::string& space);

	/**
	 * Returns the xml stringified content of the condition.
	 */
	virtual std::string toXMLString(const std::string& space);

	/**
	 * Converts the condition to an element.
	 */
	virtual ConditionElement * toConditionElement();

protected:
	void setActionsState(common::ConditionState state);
	common::ConditionState getActionsState();

private:
	void executeActions();

	std::string _actions;
	common::ConditionState _actionsState;
	std::thread * _actionsThread;

	static const std::string WATCHDOG_COMMANDLINE;
};

}

#endif /* CORE_WATCHDOG_H_ */
