/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_BASEDATACHANGESUBSCRIBER_H_
#define CORE_BASEDATACHANGESUBSCRIBER_H_

#include <common/base/Queue.h>

namespace core {

/**
 * Base class for the data change subscriber. 
 */
class BaseDataChangeSubscriber {

public :
	/**
	 * Constructor.
	 */
	BaseDataChangeSubscriber();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~BaseDataChangeSubscriber();
	
	/**
	 * Sets the database ID.
	 */
	void setDatabaseID(unsigned long databaseID);
	
	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Waits for the completion of the subscription.
	 */
	void waitForSubscriptionCompletion();
	
	/**
	 * Sets for the subscription completed.
	 */
	void setSubscriptionCompleted();

private :
	unsigned long _databaseID;
	common::Queue<bool>* _waitQueue;
};


} 

#endif /*CORE_BASEDATACHANGESUBSCRIBER_H_*/