/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDTERMINATEDSUBSCRIBER_H_
#define CORE_COMMANDTERMINATEDSUBSCRIBER_H_

#include "DataChangeSubscriber.h"
#include <mutex>

namespace core {

/**
 * Class defining a command terminated subscriber.
 */
class CommandTerminatedSubscriber : public DataChangeSubscriber {

public :
	/**
	 * Constructor.
	 */
	CommandTerminatedSubscriber();

	/**
	 * Virtual destructor.
	 */
	virtual ~CommandTerminatedSubscriber();

	/**
	 * Sets the command active.
	 */
	void setCommandActive(unsigned long requestID);

	/**
	 * Gets the subscriber type.
	 */
	virtual SubscriberType getSubscriberType() const;

	/**
	 * Notifies that the command has terminated with a request ID.
	 */
	void commandTerminated(unsigned long databaseID, unsigned long commandID, unsigned long requestID);

	/**
	 * Notifies that the command has terminated.
	 */
	virtual void commandTerminated(unsigned long databaseID, unsigned long commandID, unsigned long requestID, bool sameRequest) = 0;

private:
	unsigned long _requestID;
	std::recursive_mutex _mutex;
};

}

#endif /*CORE_COMMANDTERMINATEDSUBSCRIBER_H_*/
