/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DataAccess.h"
#include <dataprovider/transportlayer/dataprovider/ServantAccessorDirectImpl.h>
#include <dataprovider/transportlayer/dataprovider/CommandAccessorDirectImpl.h>
#include <dataprovider/transportlayer/dataprovider/PropertyAccessorDirectImpl.h>
#include <common/output/OutputStream.h>
#include <iostream>

using namespace std;
using namespace common;
using namespace dataprovider;

namespace core {

DataAccess* DataAccess::_instance = 0;

DataAccess::DataAccess() {
}

DataAccess::~DataAccess() {
}

DataAccess* DataAccess::getInstance() {
	return _instance;
}

void DataAccess::initInstance(dataprovider::ServantAccessorDirectImpl* servantAccessor, dataprovider::PropertyAccessorDirectImpl* propertyAccessor, dataprovider::CommandAccessorDirectImpl* commandAccessor) {

	if (_instance == 0) {
		_instance = new DataAccess();
		_instance->init(servantAccessor, propertyAccessor, commandAccessor);
	}
}

void DataAccess::init(dataprovider::ServantAccessorDirectImpl* servantAccessor, dataprovider::PropertyAccessorDirectImpl* propertyAccessor, dataprovider::CommandAccessorDirectImpl* commandAccessor) {
	_servantAccessor = servantAccessor;
	_propertyAccessor = propertyAccessor;
	_commandAccessor = commandAccessor;
}

DataAccess::ServantType DataAccess::getServantType(unsigned long databaseID, const std::string& servantName) const {
	if (_servantAccessor->getServantType(databaseID, getServantID(servantName)) == ServantAccessorDirectImpl::CONTROLLER) {
		return CONTROLLER;
	} else {
		return DRIVER;
	}
}

std::string DataAccess::getControllerType(unsigned long databaseID, unsigned long controllerID) const {
	return _servantAccessor->getControllerType(databaseID, controllerID);
}

std::string DataAccess::getDriverType(unsigned long databaseID, unsigned long driverID) const {
	return _servantAccessor->getDriverType(databaseID, driverID);
}

unsigned long DataAccess::getServantID(const std::string& servantName) const {
	return _servantAccessor->getServantID(0, servantName);
}

unsigned long DataAccess::getPropertyID(unsigned long databaseID, unsigned long servantID, const std::string& propertyName) const {
	return _propertyAccessor->getPropertyID(databaseID, servantID, propertyName);
}

long DataAccess::getDynamicPropertySize(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getDynamicPropertySize(databaseID, propertyID);
}

unsigned long DataAccess::getDynamicPropertyIDAtIndex(unsigned long databaseID, unsigned long propertyID, long index, bool resize) const {
	return _propertyAccessor->getDynamicPropertyIDAtIndex(databaseID, propertyID, index, resize);
}

unsigned long DataAccess::getCommandID(unsigned long databaseID, unsigned long servantID, const std::string& commandName) const {
	return _commandAccessor->getCommandID(databaseID, servantID, commandName);
}

unsigned long DataAccess::getSisterCommandID(unsigned long databaseID, unsigned long commandID, const std::string& commandName) const {
	return _commandAccessor->getSisterCommandID(databaseID, commandID, commandName);
}

bool DataAccess::isCommandFromIcsServant(unsigned long databaseID, unsigned long commandID) const {
	return _commandAccessor->isCommandFromIcsServant(databaseID, commandID);
}

unsigned long DataAccess::getServantIDFromPropertyID(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getServantIDFromPropertyID(databaseID, propertyID);
}

bool DataAccess::isServantEnabledAndInitialised(unsigned long databaseID, unsigned long servantID) const {
	return _servantAccessor->isServantEnabledAndInitialised(databaseID, servantID);
}

unsigned long DataAccess::getServantIDFromCommandID(unsigned long databaseID, unsigned long commandID) const {
	return _commandAccessor->getServantIDFromCommandID(databaseID, commandID);
}

std::string DataAccess::getServantName(unsigned long databaseID, unsigned long servantID) const {
	return _servantAccessor->getServantName(databaseID, servantID);
}

std::string DataAccess::getCommandName(unsigned long databaseID, unsigned long commandID) const {
	return _commandAccessor->getCommandName(databaseID, commandID);
}

double DataAccess::getCommandElapsedTimeMs(unsigned long databaseID, unsigned long commandID) const {
	return _commandAccessor->getCommandElapsedTimeMs(databaseID, commandID);
}

std::string DataAccess::getPropertyType(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getPropertyType(databaseID, propertyID);
}

std::string DataAccess::getPropertyName(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getPropertyName(databaseID, propertyID);
}

std::pair<std::string, long> DataAccess::getCompletePropertyName(unsigned long databaseID, unsigned long servantID, unsigned long propertyID) const {
	return _propertyAccessor->getCompletePropertyName(databaseID, servantID, propertyID);
}

DataAccess::PropertyInfo DataAccess::getPropertyInfo(unsigned long databaseID, const std::string& controllerType, unsigned long propertyID) {

	DataAccess::PropertyInfo info;

	string propertyName = _propertyAccessor->getPropertyName(databaseID, propertyID);
	info.aliasPropertyName = propertyName;

	vector<Database::PropertyInfoItem> items = _propertyAccessor->getAllPropertyInfoItems(databaseID, controllerType);

	for (auto item : items) {

		if (item.setpointPropertyName == propertyName) {
			info.isSetpoint = true;
			info.isDynamic = item.isDynamic;
			info.aliasPropertyName = item.aliasPropertyName;
		}
		else if (item.offsetPropertyName == propertyName) {
			info.isOffset = true;
			info.isDynamic = item.isDynamic;
			info.aliasPropertyName = item.aliasPropertyName;
		}
		else if (item.actualPropertyName == propertyName) {
			info.isDynamic = item.isDynamic;
			info.aliasPropertyName = item.aliasPropertyName;
		}
	}

	return info;
}

int8 DataAccess::getInt8Value(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getInt8Value(databaseID, propertyID);
}

uint16 DataAccess::getUint16Value(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getUint16Value(databaseID, propertyID);
}

uint32 DataAccess::getUint32Value(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getUint32Value(databaseID, propertyID);
}

uint64 DataAccess::getUint64Value(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getUint64Value(databaseID, propertyID);
}

int16 DataAccess::getInt16Value(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getInt16Value(databaseID, propertyID);
}

int32 DataAccess::getInt32Value(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getInt32Value(databaseID, propertyID);
}

int64 DataAccess::getInt64Value(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getInt64Value(databaseID, propertyID);
}

float32 DataAccess::getFloat32Value(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getFloat32Value(databaseID, propertyID);
}

float64 DataAccess::getFloat64Value(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getFloat64Value(databaseID, propertyID);
}

std::string DataAccess::getStringValue(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getStringValue(databaseID, propertyID);
}

bool DataAccess::getBooleanValue(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getBooleanValue(databaseID, propertyID);
}

std::string DataAccess::getFormattedValue(unsigned long databaseID, unsigned long propertyID) const {
	return _propertyAccessor->getFormattedValue(databaseID, propertyID);
}

void DataAccess::setCommandState(unsigned long databaseID, unsigned long servantID, unsigned long commandID, CommandState state) {
	_commandAccessor->setCommandState(databaseID, servantID, commandID, state);
}

common::CommandState DataAccess::getCommandState(unsigned long databaseID, unsigned long commandID) const {
	return _commandAccessor->getCommandState(databaseID, commandID);
}

void DataAccess::setCommandFromCommandZone(unsigned long databaseID, unsigned long commandID, bool fromCommandZone) {

	_commandAccessor->setCommandFromCommandZone(databaseID, commandID, fromCommandZone);
}

bool DataAccess::commandExists(unsigned long commandID) {

	return _commandAccessor->commandExists(0, commandID);
}

bool DataAccess::propertyExists(unsigned long propertyID) {

	return _propertyAccessor->propertyExists(0, propertyID);
}

bool DataAccess::servantExists(unsigned long servantID) {

	return _servantAccessor->servantExists(0, servantID);
}

void DataAccess::setCommandLogID(unsigned long databaseID, unsigned long commandID, const std::string& logID) {

	_commandAccessor->setCommandLogID(databaseID, commandID, logID);
}

std::string DataAccess::what(unsigned long databaseID, unsigned long elementID) {

	return _servantAccessor->what(databaseID, elementID);
}

bool DataAccess::isServantPausable(unsigned long databaseID, unsigned long commandID) {
	unsigned long servantID = _commandAccessor->getServantIDFromCommandID(databaseID, commandID);

	return _servantAccessor->isServantPausable(databaseID, servantID);
}

std::vector<DataAccess::PropertyInfoItem> DataAccess::getPropertyInfoItems(unsigned long databaseID, const std::string& controllerType) {

	vector<PropertyInfoItem> result;

	vector<Database::PropertyInfoItem> items = _propertyAccessor->getPropertyInfoItems(databaseID, controllerType);

	for (auto i : items) {
		PropertyInfoItem item;
		item.actualPropertyName = i.actualPropertyName;
		item.setpointPropertyName = i.setpointPropertyName;
		item.offsetPropertyName = i.offsetPropertyName;
		item.aliasPropertyName = i.aliasPropertyName;
		item.isScan = i.isScan;
		item.isSpy = i.isSpy;
		item.isDynamic = i.isDynamic;

		result.push_back(item);
	}

	return result;
}

std::vector<DataAccess::PropertyInfoItem> DataAccess::getAllPropertyInfoItems(unsigned long databaseID, const std::string& controllerType) {

	vector<PropertyInfoItem> result;

	vector<Database::PropertyInfoItem> items = _propertyAccessor->getAllPropertyInfoItems(databaseID, controllerType);

	for (auto i : items) {
		PropertyInfoItem item;
		item.actualPropertyName = i.actualPropertyName;
		item.setpointPropertyName = i.setpointPropertyName;
		item.offsetPropertyName = i.offsetPropertyName;
		item.aliasPropertyName = i.aliasPropertyName;
		item.isScan = i.isScan;
		item.isSpy = i.isSpy;
		item.isDynamic = i.isDynamic;

		result.push_back(item);
	}

	return result;
}

bool DataAccess::isPropertySavedInCommandBox(unsigned long databaseID, unsigned long propertyID) {
	return _propertyAccessor->isPropertySavedInCommandBox(databaseID, propertyID);
}

std::vector<unsigned long> DataAccess::getPropertiesSavedInBox(unsigned long databaseID) {
	return _propertyAccessor->getPropertiesSavedInBox(databaseID);
}

unsigned long DataAccess::getEnumeratedValues(unsigned long databaseID, unsigned long propertyID) {
	return _propertyAccessor->getEnumeratedValues(databaseID, propertyID);
}

unsigned long DataAccess::getEnumeratedLabels(unsigned long databaseID, unsigned long propertyID) {
	return _propertyAccessor->getEnumeratedLabels(databaseID, propertyID);
}

unsigned long DataAccess::getUnitPropertyID(unsigned long databaseID, unsigned long propertyID) {
	return _propertyAccessor->getUnitPropertyID(databaseID, propertyID);
}

bool DataAccess::isServantInDebug(unsigned long databaseID, unsigned long servantID) {
	return _servantAccessor->isServantInDebug(databaseID, servantID);
}

}
