/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PropertySetterTerminatedSubscriber.h"

namespace core {

PropertySetterTerminatedSubscriber::PropertySetterTerminatedSubscriber() :
	_requestID(0) {
}

PropertySetterTerminatedSubscriber::~PropertySetterTerminatedSubscriber() {
}

void PropertySetterTerminatedSubscriber::setSetterActive(unsigned long requestID) {
	
	// the requestID is set by the Scheduler
	// the requestID is also accessed via the DataNotificationRouter  
	std::scoped_lock lock(_mutex);
	
	_requestID = requestID;
}

DataChangeSubscriber::SubscriberType PropertySetterTerminatedSubscriber::getSubscriberType() const {
	return PROPERTY_SETTER_TERMINATED;
}

void PropertySetterTerminatedSubscriber::propertySetterTerminated(unsigned long databaseID, unsigned long propertyID, unsigned long requestID, bool successful) {
	
	// the requestID is set by the Scheduler
	// the requestID is also accessed via the DataNotificationRouter  
	std::scoped_lock lock(_mutex);

	if (_requestID == requestID) {
		propertySetterTerminated(databaseID, propertyID, successful);
	}
	
	_requestID = 0;
}

}
