/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_LOG_H_
#define CORE_LOG_H_

#include <common/base/CommonTypes.h>
#include <string>

namespace core {

/**
 * Class defining a log entry.
 */
class Log {

public:
	/**
	 * Constructor.
	 */
	Log(common::LogType type, common::LogLevel level, const std::string& message);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~Log();

	/**
	 * Return the log level
	 */
	common::LogLevel getLevel() const;

	/**
	 * Return the log type
	 */
	common::LogType getType() const;

	/**
	 * Return the log message
	 */
	std::string getMessage() const;

	const static std::string NAME;
	const static std::string TYPE;
	const static std::string VALUE;

private:
	common::LogType _type;
	common::LogLevel _level;
	std::string _message;
	
};


}

#endif /*CORE_LOG_H_*/
