/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "LogHandler.h"
#include "CommandLog.h"
#include "SimpleLog.h"
#include "CommandZoneContentLog.h"
#include "CommandZoneEventLog.h"
#include "VariableChangeLog.h"
#include "ForLoopStartLog.h"
#include "ForLoopEndLog.h"
#include "IfEvaluationLog.h"
#include "BreakEvaluationLog.h"
#include "UserScanStartLog.h"
#include <core/transportlayer/dataprovider/LogAccessor.h>
#include <common/output/OutputStream.h>
#include "Utilities/Simulated.h"

using namespace std;
using namespace common;

namespace core {

LogHandler* LogHandler::_instance = 0;

LogHandler::LogHandler() :
	_logAccessor(0) {
}

LogHandler::~LogHandler() {

}

void LogHandler::initInstance(LogAccessor* logAccessor) {
	if (_instance == 0) {
		_instance = new LogHandler();
		_instance->init(logAccessor);
	}
}

void LogHandler::resetInstance() {

	if (_instance != 0) {
		_instance->reset();
	}

	delete _instance;
	_instance = 0;
}

LogHandler* LogHandler::getInstance() {
	return _instance;
}


void LogHandler::init(LogAccessor* logAccessor) {
	_logAccessor = logAccessor;
}

void LogHandler::reset() {
}

void LogHandler::write(const Log& log) {

	if (dynamic_cast<const CommandLog*>(&log) != 0) {
		write(dynamic_cast<const CommandLog*>(&log));

	} else if (dynamic_cast<const SimpleLog*>(&log) != 0) {
		write(dynamic_cast<const SimpleLog*>(&log));

	} else if (dynamic_cast<const CommandZoneContentLog*>(&log) != 0) {
		write(dynamic_cast<const CommandZoneContentLog*>(&log));

	} else if (dynamic_cast<const CommandZoneEventLog*>(&log) != 0) {
		write(dynamic_cast<const CommandZoneEventLog*>(&log));

	} else if (dynamic_cast<const VariableChangeLog*>(&log) != 0) {
		write(dynamic_cast<const VariableChangeLog*>(&log));

	} else if (dynamic_cast<const ForLoopStartLog*>(&log) != 0) {
		write(dynamic_cast<const ForLoopStartLog*>(&log));

	} else if (dynamic_cast<const ForLoopEndLog*> (&log) != 0) {
		write(dynamic_cast<const ForLoopEndLog*> (&log));

	} else if (dynamic_cast<const IfEvaluationLog*>(&log) != 0) {
		write(dynamic_cast<const IfEvaluationLog*>(&log));

	} else if (dynamic_cast<const BreakEvaluationLog*>(&log) != 0) {
		write(dynamic_cast<const BreakEvaluationLog*>(&log));

	} else if (dynamic_cast<const UserScanStartLog*>(&log) != 0) {
		write(dynamic_cast<const UserScanStartLog*>(&log));

	}

}

void LogHandler::write(const CommandLog* log) {
	_logAccessor->writeCommandLog(log->getType(), log->getLevel(), log->getMessage(), log->getLogID());
}

void LogHandler::write(const SimpleLog* log) {
	_logAccessor->writeSimpleLog(log->getLevel(), log->getMessage(), !Simulated::activated && !Simulated::visa && !Simulated::for3d);
}

void LogHandler::write(const CommandZoneContentLog* log) {
	_logAccessor->writeCommandZoneContentLog(log->getMessage());
}

void LogHandler::write(const CommandZoneEventLog* log) {
	_logAccessor->writeCommandZoneEventLog(log->getMessage());
}

void LogHandler::write(const VariableChangeLog* log) {
	_logAccessor->writeVariableChangeLog(log->getLevel(), log->getName(), log->getType(), log->getValue(), log->getMessage(), log->getMessageParentId());
}

void LogHandler::write(const ForLoopStartLog* log) {
	_logAccessor->writeForLoopStartLog(log->getLevel(), log->getVariablesField(), log->getMessage(), log->getMessageId());
}

void LogHandler::write(const ForLoopEndLog* log) {
	_logAccessor->writeForLoopEndLog(log->getLevel(), log->getMessage(), log->getMessageId());
}

void LogHandler::write(const IfEvaluationLog* log) {
	_logAccessor->writeIfEvaluationLog(log->getLevel(), log->getName(), log->getType(), log->getValue(), log->getMessage());
}

void LogHandler::write(const BreakEvaluationLog* log) {
	_logAccessor->writeBreakEvaluationLog(log->getLevel(), log->getName(), log->getType(), log->getValue(), log->getMessage(), log->getMessageParentId());
}

void LogHandler::write(const UserScanStartLog* log) {
	_logAccessor->writeUserScanStartLog(log->getLevel(), log->getName(), log->getMessage());
}

}
