/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_LOGHANDLER_H_
#define CORE_LOGHANDLER_H_

#include <string>

namespace core {

class Log;
class CommandLog;
class SimpleLog;
class CommandZoneContentLog;
class CommandZoneEventLog;
class VariableChangeLog;
class ForLoopStartLog;
class ForLoopEndLog;
class IfEvaluationLog;
class BreakEvaluationLog;
class UserScanStartLog;
class LogAccessor;

/**
 * Class defining a log handler singleton.
 */
class LogHandler {

public:
	/**
	 * Gets the instance.
	 */
	static LogHandler* getInstance();
	
	/**
	 * Initialises the instance.
	 */
	static void initInstance(LogAccessor* logAccessor);
	
	/**
	 * Resets the instance.
	 */
	static void resetInstance();
	
	/**
	 * Writes the log message.
	 */
	void write(const Log& log);

private:
	LogHandler();
	virtual ~LogHandler();

	void init(LogAccessor* logAccessor);
	void reset();

	void write(const CommandLog* log);
	void write(const SimpleLog* log);
	void write(const CommandZoneContentLog* log);
	void write(const CommandZoneEventLog* log);
	void write(const VariableChangeLog* log);
	void write(const ForLoopStartLog* log);
	void write(const ForLoopEndLog* log);
	void write(const IfEvaluationLog* log);
	void write(const BreakEvaluationLog* log);
	void write(const UserScanStartLog* log);

	static LogHandler* _instance;

	LogAccessor* _logAccessor;

};

}

#endif /*CORE_LOGHANDLER_H_*/
