/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_MESSAGE_H_
#define CORE_MESSAGE_H_

#include <string>

namespace core {

/**
 * Base class for the Scheduler message queue.
 */
class Message {

public:
	/**
	 * Constructor.
	 */
	Message();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~Message();

	/**
	 * Type of the message.
	 */
	typedef enum {
		NEW_COMMAND, NEW_ENABLE_DISABLE_COMMAND, STOP_COMMAND, PAUSE_COMMAND, RESTART_COMMAND, 
		STOP_SETTER, PAUSE_SETTER, RESTART_SETTER,
		INIT_DRIVERS_AND_PROTOCOLS, 
		COMMAND_TERMINATED, SET_VALUE, PROPERTY_SETTER_TERMINATED, 
		DURATION_REQUEST, DURATION_RESULT,
		EXECUTE_COMMAND_BOX, COMMAND_ZONE_EVENT,
		SHUTDOWN} Type;
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const = 0;	

	/**
	 * Sets the request ID;
	 */
	void setRequestID(unsigned long requestID);
	
	/**
	 * Gets the request ID;
	 */
	unsigned long getRequestID() const;

	/**
	 * Gets the type string.
	 */
	std::string getTypeString() const;

private:
	unsigned long _requestID;

};

}


#endif /*CORE_MESSAGE_H_*/
