/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDBOXESDELETEDEVENT_H_
#define CORE_COMMANDBOXESDELETEDEVENT_H_

#include "CommandZoneEvent.h"
#include <vector>

namespace core {

/**
 * Class defining a command boxes deleted event.
 */
class CommandBoxesDeletedEvent : public CommandZoneEvent {
	
public:
	/**
	 * Constructor.
	 */
	CommandBoxesDeletedEvent(unsigned long commandZoneID, const std::vector<unsigned long>& commandBoxIDs);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CommandBoxesDeletedEvent();

	/**
	 * Gets the event type.
	 */
	virtual EventType getEventType() const;	
	
	/**
	 * Returns the command Zone ID.
	 */
	unsigned long getCommandZoneID() const;

	/**
	 * Returns the command box IDs.
	 */
	const std::vector<unsigned long>& getCommandBoxIDs() const;
	
private:
	unsigned long _commandZoneID;
	std::vector<unsigned long> _commandBoxIDs;
};

}

#endif
