/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDZONEEVENT_H_
#define CORE_COMMANDZONEEVENT_H_

#include <core/message/Message.h>

namespace core {

/**
 * Class defining a command zone event.
 */
class CommandZoneEvent : public Message {

public:

	/**
	 * Event type.
	 */
	typedef enum {
		COMMAND_BOX_EXECUTION,
		COMMAND_BOX_PROGRESSION,
		COMMAND_BOX_RESET,
		COMMAND_BOX_PROPERTY,
		COMMAND_ZONE_EXECUTION,
		COMMAND_ZONE_PROGRESSION,
		COMMAND_ZONE_RESET,
		COMMAND_ZONE_ERROR,
		FORLOOP_CURRENT_VALUE_CHANGED,
		COMMAND_BOX_EXPRESSION_CHANGED,
		COMMAND_BOX_EXPRESSION_VALUE_CHANGED,
		COMMAND_BOXES_DELETED,
		COMMAND_ZONE_SYNC,
		NEW_ESTIMATED_TIME
	} EventType;

	/**
	 * Constructor.
	 */
	CommandZoneEvent();

	/**
	 * Virtual destructor.
	 */
	virtual ~CommandZoneEvent();

	/**
	 * Gets the type (Message::COMMAND_ZONE_EVENT).
	 */
	virtual Type getType() const;

	/**
	 * Gets the event type.
	 */
	virtual EventType getEventType() const = 0;

private:

};

}

#endif /*CORE_COMMANDZONEEVENT_H_*/
