/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDZONEEXECUTIONEVENT_H_
#define CORE_COMMANDZONEEXECUTIONEVENT_H_

#include "CommandZoneEvent.h"

namespace core {

/**
 * Class defining a command zone execution event.
 */
class CommandZoneExecutionEvent : public CommandZoneEvent {
	
public:
	typedef enum {STARTED, PAUSED, STOPPING_AT_END, PAUSING_AT_END, FINISHED} ExecutionState;

	/**
	 * Constructor.
	 */
	CommandZoneExecutionEvent(unsigned long commandZoneID, ExecutionState executionState);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CommandZoneExecutionEvent();

	/**
	 * Gets the event type.
	 */
	virtual EventType getEventType() const;	
	
	/**
	 * Returns the command Zone ID.
	 */
	unsigned long getCommandZoneID() const;
	
	/**
	 * Returns the execution state.
	 */
	ExecutionState getExecutionState() const;
	
	
private:
	unsigned long _commandZoneID;
	ExecutionState _executionState;
};

}

#endif /*CORE_COMMANDZONEEXECUTIONEVENT_H_*/
