/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDZONESYNCEVENT_H_
#define CORE_COMMANDZONESYNCEVENT_H_

#include "CommandZoneEvent.h"

namespace core {

/**
 * Class defining a command zone sync event.
 */
class CommandZoneSyncEvent : public CommandZoneEvent {
	
public:
	/**
	 * Constructor.
	 */
	CommandZoneSyncEvent(const std::string& serializedMessageType, const std::string& serializedMessageContent, const std::string& serializedResponse);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CommandZoneSyncEvent();

	/**
	 * Gets the event type.
	 */
	virtual EventType getEventType() const;	
	
	/**
	 * Returns the message type.
	 */
	const std::string& getSerializedMessageType() const;

	/**
	 * Gets the message content.
	 */
	const std::string& getSerializedMessageContent() const;
	
	/**
	 * Gets the message response.
	 */
	const std::string& getSerializedResponse() const;

private:
	std::string _serializedMessageType;
	std::string _serializedMessageContent;
	std::string _serializedResponse;
};

}

#endif

