/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_EXECUTECOMMANDBOX_H_
#define CORE_EXECUTECOMMANDBOX_H_

#include <core/message/Message.h>

namespace core {

class CommandBox;

/**
 * Class defining an execute command box.
 */
class ExecuteCommandBox : public Message {
	
public:

	/**
	 * Action type.
	 */
	typedef enum {INIT, COMMAND, ALL} ActionType;

	/**
	 * Constructor with the command box.
	 */
	ExecuteCommandBox(CommandBox* commandBox, ActionType type);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ExecuteCommandBox();

	/**
	 * Gets the command ID.
	 */
	CommandBox* getCommandBox();

	/**
	 * Gets the type (Message::EXECUTE_COMMAND_BOX).
	 */
	virtual Type getType() const;	

	ActionType getActionType() const;
	
private:
	CommandBox* _commandBox;
	ActionType _type;	
};

}

#endif /*CORE_EXECUTECOMMANDBOX_H_*/