/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_FORLOOPCOMMANDBOXCURRENTVALUEEVENT_H_
#define CORE_FORLOOPCOMMANDBOXCURRENTVALUEEVENT_H_

#include "CommandZoneEvent.h"

namespace core {

/**
 * Class defining a for loop command box current value event.
 */
class ForLoopCommandBoxCurrentValueEvent : public CommandZoneEvent {
	
public:
	/**
	 * Constructor.
	 */
	ForLoopCommandBoxCurrentValueEvent(unsigned long commandBoxID, long index, double currentValue);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ForLoopCommandBoxCurrentValueEvent();

	/**
	 * Gets the event type.
	 */
	virtual EventType getEventType() const;	
	
	/**
	 * Returns the command box ID.
	 */
	unsigned long getCommandBoxID() const;
	
	/**
	 * Returns the line index.
	 */
	long getIndex() const;
	
	/**
	 * Returns the current value.
	 */
	double getCurrentValue() const;
	
	
private:
	unsigned long _commandBoxID;
	long _index;
	double _currentValue;
};

}

#endif /*CORE_FORLOOPCOMMANDBOXCURRENTVALUEEVENT_H_*/