/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_NEWESTIMATEDTIMEEVENT_H_
#define CORE_NEWESTIMATEDTIMEEVENT_H_

#include "CommandZoneEvent.h"
#include <common/base/CommonTypes.h>

namespace core {

/**
 * Class defining a new file created event.
 */
class NewEstimatedTimeMessage : public CommandZoneEvent {
	
public:
	/**
	 * Constructor.
	 */
	NewEstimatedTimeMessage(unsigned long commandZoneID, double value, bool docompute);

	/**
	 * Gets the event type.
	 */
	virtual EventType getEventType() const;
	
	/**
	 * Virtual destructor.
	 */
	virtual ~NewEstimatedTimeMessage();

	/**
	 * Returns the command Zone ID.
	 */
	unsigned long getCommandZoneID() const;

	/**
	 * Returns the estimated time value.
	 */
	double getValue() const;

	/**
	 * Returns the estimated time value.
	 */
	bool getDoCompute() const;

private:
	unsigned long _commandZoneID;
	double _value;
	bool _docompute;
};

}

#endif
