/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_CONFIGURATIONCHANGEDMESSAGE_H_
#define CORE_CONFIGURATIONCHANGEDMESSAGE_H_

#include "DataNotificationMessage.h"
#include <common/base/CommonTypes.h>

namespace core {

/**
 * Class defining a configuration changed message.
 */
class ConfigurationChangedMessage : public DataNotificationMessage {

public :	
	/**
	 * Constructor.
	 */
	ConfigurationChangedMessage(unsigned long databaseID, unsigned long servantID, common::ConfigurationState enable);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ConfigurationChangedMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the servant ID.
	 */
	unsigned long getServantID() const;

	/**
	 * Gets the enable state.
	 */
	common::ConfigurationState getState() const;

private :
	unsigned long _servantID;
	common::ConfigurationState _state;
};

}

#endif /*CORE_CONFIGURATIONCHANGEDMESSAGE_H_*/
