/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_DATANOTIFICATIONMESSAGE_H_
#define CORE_DATANOTIFICATIONMESSAGE_H_

#include <string>

namespace core {

/**
 * Class defining a data notification message.
 */
class DataNotificationMessage {

public :
	/**
	 * Constructor.
	 */
	DataNotificationMessage(unsigned long databaseID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DataNotificationMessage();
	
	/**
	 * Type of message.
	 */
	typedef enum {PROPERTY_CHANGED, COMMAND_STATE_CHANGED, COMMAND_PROGRESS_CHANGED, COMMAND_TERMINATED, PROPERTY_SETTER_TERMINATED, CONFIGURATION_CHANGED, SUBSCRIPTION, SHUTDOWN} Type;

	/**
	 * Gets the type.
	 */
	virtual Type getType() const = 0;

	/**
	 * Gets the databade ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Gets the stringified type.
	 */
	std::string getTypeString() const;
		
private :
	unsigned long _databaseID;
};

} 

#endif /*CORE_DATANOTIFICATIONMESSAGE_H_*/
