/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_PROPERTYSETTERTERMINATEDMESSAGE_H_
#define CORE_PROPERTYSETTERTERMINATEDMESSAGE_H_

#include "DataNotificationMessage.h"

namespace core {

/**
 * Class defining a property setter terminated message.
 */
class PropertySetterTerminatedMessage : public DataNotificationMessage {

public :
	/**
	 * Constructor.
	 */
	PropertySetterTerminatedMessage(unsigned long databaseID, unsigned long propertyID, unsigned long requestID, bool successful);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~PropertySetterTerminatedMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the property ID.
	 */
	unsigned long getPropertyID() const;
	
	/**
	 * Gets the request ID.
	 */
	unsigned long getRequestID() const;
	
	/**
	 * Returns true if successful.
	 */
	bool isSuccessful() const;
	
private :
	unsigned long _propertyID;
	unsigned long _requestID;
	bool _isSuccessful;
};


} // namespace dataprovider

#endif /*CORE_PROPERTYSETTERTERMINATEDMESSAGE_H_*/