/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SubscriptionMessage.h"
#include <core/dataaccess/DataChangeSubscriber.h>

namespace core {


SubscriptionMessage::SubscriptionMessage(unsigned long dataID, DataChangeSubscriber* subscriber, bool isSubscriber) :
	DataNotificationMessage(0),
	_dataID(dataID),
	_subscriber(subscriber),
	_isSubscriber(isSubscriber),
	_isBulk(false) {
}

SubscriptionMessage::SubscriptionMessage(std::set<unsigned long> dataIDs, DataChangeSubscriber* subscriber, bool isSubscriber) :
	DataNotificationMessage(0),
	_dataIDs(dataIDs),
	_subscriber(subscriber),
	_isSubscriber(isSubscriber),
	_isBulk(true) {
}

SubscriptionMessage::~SubscriptionMessage() {
}

SubscriptionMessage::Type SubscriptionMessage::getType() const {
	return DataNotificationMessage::SUBSCRIPTION;
}

unsigned long SubscriptionMessage::getDataID() const {
	return _dataID;
}

const std::set<unsigned long>& SubscriptionMessage::getDataIDs() const {
	return _dataIDs;
}

DataChangeSubscriber* SubscriptionMessage::getSubscriber() const {
	return _subscriber;
}

bool SubscriptionMessage::isSubscriber() const {
	return _isSubscriber;
}

bool SubscriptionMessage::isBulk() const {
	return _isBulk;
}

}