/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SUBSCRIPTIONMESSAGE_H_
#define CORE_SUBSCRIPTIONMESSAGE_H_

#include "DataNotificationMessage.h"
#include <set>

namespace core {

class DataChangeSubscriber;

/**
 * Class defining a subscription message.
 */
class SubscriptionMessage : public DataNotificationMessage {

public :
	/**
	 * Constructor.
	 */
	SubscriptionMessage(unsigned long dataID, DataChangeSubscriber* subscriber, bool isSubscriber);
	
	/**
	 * Constructor.
	 */
	SubscriptionMessage(std::set<unsigned long> dataIDs, DataChangeSubscriber* subscriber, bool isSubscriber);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~SubscriptionMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the data ID.
	 */
	unsigned long getDataID() const;
	
	/**
	 * Gets the data IDs.
	 */
	const std::set<unsigned long>& getDataIDs() const;
	
	/**
	 * Gets the subscriber.
	 */
	DataChangeSubscriber* getSubscriber() const;
	
	/**
	 * Returns true if is a subscriber.
	 */
	bool isSubscriber() const; 

	/**
	 * Returns true if is bulk.
	 */
	bool isBulk() const; 

private :
	unsigned long _dataID;
	std::set<unsigned long> _dataIDs;
	DataChangeSubscriber* _subscriber;
	bool _isSubscriber;
	bool _isBulk;

};


} // namespace dataprovider

#endif /*CORE_SUBSCRIPTIONMESSAGE_H_*/