/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_BASESETVALUECOMMAND_H_
#define CORE_BASESETVALUECOMMAND_H_

#include <common/base/Queue.h>
#include <core/scheduler/RequestReply.h>
#include "Operation.h"

namespace core {

class PropertySetterTerminatedSubscriber;

/**
 * Class defining a set value message for the Scheduler message queue. 
 */
class BaseSetValue : public Operation {
	
public:
	/**
	 * Constructor with the servantID, propertyID.
	 */
	BaseSetValue(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, PropertySetterTerminatedSubscriber* subscriber, common::Queue<RequestReply>* waitForReplyQueue);	
	/**
	 * Virtual destructor.
	 */
	virtual ~BaseSetValue();

	/**
	 * Clones the Operation.
	 */
	virtual Operation* clone() const;

	/**
	 * Gets the property ID.
	 */
	unsigned long getPropertyID() const;

	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Gets the subscriber.
	 */
	PropertySetterTerminatedSubscriber* getSubscriber() const;
		
	/**
	 * Gets the type (Message::NEW_COMMAND).
	 */
	virtual Type getType() const;	
	
	/**
	 * Gets the "wait for reply" queue
	 */
	common::Queue<RequestReply>* getWaitForReplyQueue() const;
	
private:
	unsigned long _propertyID;
	unsigned long _databaseID;
	PropertySetterTerminatedSubscriber* _subscriber;
	common::Queue<RequestReply>* _waitForReplyQueue;
};

}

#endif /*CORE_BASESETVALUECOMMAND_H_*/