/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_DURATIONREQUEST_H_
#define CORE_DURATIONREQUEST_H_

#include <common/base/Queue.h>
#include "Operation.h"

namespace core {

/**
 * Class defining a duration request message for the Scheduler message queue. 
 */
class DurationRequest : public Operation {
	
public:
	/**
	 * Constructor with the ID.
	 */
	DurationRequest(unsigned long databaseID, unsigned long servantID, unsigned long commandID, common::Queue<double>* waitForReplyQueue);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DurationRequest();

	/**
	 * Clones the Operation.
	 */
	virtual Operation* clone() const;

	/**
	 * Gets the ID.
	 */
	unsigned long getCommandID() const;

	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Gets the type (Message::DURATION_REQUEST).
	 */
	virtual Type getType() const;	
	
	/**
	 * Gets the "wait for reply" queue
	 */
	common::Queue<double>* getWaitForReplyQueue() const;
	
private:
	unsigned long _commandID;
	unsigned long _databaseID;
	common::Queue<double>* _waitForReplyQueue;
	
};

}

#endif /*CORE_DURATIONREQUEST_H_*/