/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DurationResult.h"

namespace core {

DurationResult::DurationResult(unsigned long requestID, unsigned long commandID, double result, const std::string& message, bool successful) :
	Message(),
	_commandID(commandID),
	_result(result),
	_message(message),
	_successful(successful) {
	
	setRequestID(requestID);
}

DurationResult::~DurationResult() {
}

unsigned long DurationResult::getCommandID() const {
	
	return _commandID;
}

Message::Type DurationResult::getType() const {
	
	return Message::DURATION_RESULT;
}

double DurationResult::getResult() const {

	return _result;
}

bool DurationResult::isSuccessful() const {
	
	return _successful;
}

const std::string& DurationResult::getMessage() const {
	
	return _message;
}

}