/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_DURATIONRESULT_H_
#define CORE_DURATIONRESULT_H_

#include <core/message/Message.h>
#include <string>

namespace core {

/**
 * Class defining a duration result message for the Scheduler message queue. 
 */
class DurationResult : public Message {
	
public:
	/**
	 * Constructor.
	 */
	DurationResult(unsigned long requestID, unsigned long commandID, double result, const std::string& message, bool successful);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DurationResult();

	/**
	 * Gets the command ID.
	 */
	unsigned long getCommandID() const;
	
	/**
	 * Gets the type of the message (Message::COMMAND_TERMINATED).
	 */
	virtual Type getType() const;	

	/**
	 * Returns the result.
	 */
	double getResult() const;
	
	/**
	 * Returns true if the command has terminated successfully.
	 */
	bool isSuccessful() const;
	
	/**
	 * Gets the string message.
	 */
	const std::string& getMessage() const;
	
private:
	unsigned long _commandID;
	double _result;
	std::string _message;
	bool _successful;

};

}

#endif /*CORE_DURATIONRESULT_H_*/