/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "NewCommand.h"
#include <core/dataaccess/CommandTerminatedSubscriber.h>

namespace core {

NewCommand::NewCommand(unsigned long databaseID, unsigned long servantID, unsigned long commandID, bool fromCommandZone, bool skipIfWait, CommandTerminatedSubscriber* subscriber, common::Queue<RequestReply>* waitForReplyQueue) :
	Operation(servantID),
	_commandID(commandID),
	_databaseID(databaseID),
	_fromCommandZone(fromCommandZone),
	_skipIfWait(skipIfWait),
	_subscriber(subscriber),
	_waitForReplyQueue(waitForReplyQueue) {
}

NewCommand::~NewCommand() {
}

Operation* NewCommand::clone() const {
	
	return new NewCommand(*this);
}

unsigned long NewCommand::getCommandID() const {
	
	return _commandID;
}

unsigned long NewCommand::getDatabaseID() const {
	
	return _databaseID;
}

bool NewCommand::fromCommandZone() const {

	return _fromCommandZone; 
}

bool NewCommand::skipIfWait() const {
	
	return _skipIfWait;
}

CommandTerminatedSubscriber* NewCommand::getSubscriber() const {

	return _subscriber;
}

Message::Type NewCommand::getType() const {
	
	return Message::NEW_COMMAND;
}

common::Queue<RequestReply>* NewCommand::getWaitForReplyQueue() const {
	
	return _waitForReplyQueue;
}

}