/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_NEWCOMMAND_H_
#define CORE_NEWCOMMAND_H_

#include <common/base/Queue.h>
#include <core/scheduler/RequestReply.h>
#include "Operation.h"

namespace core {

class CommandTerminatedSubscriber;

/**
 * Class defining a new command message for the Scheduler message queue. 
 */
class NewCommand : public Operation {
	
public:
	/**
	 * Constructor with the ID.
	 */
	NewCommand(unsigned long databaseID, unsigned long servantID, unsigned long commandID, bool fromCommandZone, bool skipIfWait, CommandTerminatedSubscriber* subscriber, common::Queue<RequestReply>* waitForReplyQueue);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~NewCommand();

	/**
	 * Clones the Operation.
	 */
	virtual Operation* clone() const;

	/**
	 * Gets the command ID.
	 */
	unsigned long getCommandID() const;

	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Returns true if the command was started from the command zone.
	 */
	bool fromCommandZone() const;
	
	/**
	 * Returns true if skipping if waiting.
	 */
	bool skipIfWait() const;
	
	/**
	 * Gets the subscriber.
	 */
	CommandTerminatedSubscriber* getSubscriber() const;
	
	/**
	 * Gets the type (Message::NEW_COMMAND).
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the "wait for reply" queue
	 */
	common::Queue<RequestReply>* getWaitForReplyQueue() const;
		
private:
	unsigned long _commandID;
	unsigned long _databaseID;
	bool _fromCommandZone;
	bool _skipIfWait;
	CommandTerminatedSubscriber* _subscriber;
	common::Queue<RequestReply>* _waitForReplyQueue;
};

}

#endif /*CORE_NEWCOMMAND_H_*/