/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_NEWENABLEDISABLECOMMAND_H_
#define CORE_NEWENABLEDISABLECOMMAND_H_

#include <common/base/Queue.h>
#include <core/scheduler/RequestReply.h>
#include "Operation.h"

namespace core {

/**
 * Class defining a new enable disable command message for the Scheduler message queue. 
 */
class NewEnableDisableCommand : public Operation {
	
public:
	/**
	 * Constructor with the ID.
	 */
	NewEnableDisableCommand(unsigned long databaseID, unsigned long commandID, bool value, common::Queue<RequestReply>* waitForReplyQueue);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~NewEnableDisableCommand();

	/**
	 * Clones the Operation.
	 */
	virtual Operation* clone() const;

	/**
	 * Gets the command ID.
	 */
	unsigned long getCommandID() const;

	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Returns true if the command is enable, false if disable.
	 */
	bool getValue() const;
	
	/**
	 * Gets the type (Message::NEW_ENABLE_DISABLE_COMMAND).
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the "wait for reply" queue
	 */
	common::Queue<RequestReply>* getWaitForReplyQueue() const;
		
private:
	unsigned long _commandID;
	unsigned long _databaseID;
	bool _value;
	common::Queue<RequestReply>* _waitForReplyQueue;
};

}

#endif /*CORE_NEWENABLEDISABLECOMMAND_H_*/