/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_PAUSECOMMAND_H_
#define CORE_PAUSECOMMAND_H_

#include <core/message/Message.h>

namespace core {

/**
 * Class defining a pause command message for the Scheduler message queue. 
 */
class PauseCommand : public Message {
	
public:
	/**
	 * Constructor with the ID.
	 */
	PauseCommand(unsigned long databaseID, unsigned long commandID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~PauseCommand();

	/**
	 * Gets the ID.
	 */
	unsigned long getCommandID() const;

	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Gets the type (Message::PAUSE_COMMAND).
	 */
	virtual Type getType() const;	
	
private:
	unsigned long _commandID;
	unsigned long _databaseID;
	
};

}

#endif /*CORE_PAUSECOMMAND_H_*/