/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SETVALUECOMMAND_H_
#define CORE_SETVALUECOMMAND_H_

#include "BaseSetValue.h"

namespace core {

/**
 * Template class defining a typed set value message for the Scheduler message queue. 
 */
template<typename ValueType>
class SetValue : public BaseSetValue {
	
public:
	/**
	 * Constructor with the servantID, propertyID and value.
	 */
	SetValue(unsigned long databaseID, unsigned long servantID, unsigned long propertyID, const ValueType& value, PropertySetterTerminatedSubscriber* subscriber, common::Queue<RequestReply>* waitForReplyQueue);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~SetValue();
	
	/**
	 * Clones the Operation.
	 */
	virtual Operation* clone() const;

	/**
	 * Gets the value.
	 */
	const ValueType& getValue() const;
	
private:
	ValueType _value;
	
};

}

#include "SetValue.inl"

#endif /*CORE_SETVALUECOMMAND_H_*/