/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ConditionElement.h"
#include <common/output/OutputStream.h>
#include <iostream>

using namespace common;

namespace core {

ConditionElement::ConditionElement(common::ConditionType type, const std::string& name, int activationDelay, bool on, bool stopCommandZone) :
	_type(type),
	_name(name),
	_activationDelay(activationDelay),
	_on(on),
	_stopCommandZone(stopCommandZone) {
}

ConditionElement::~ConditionElement() {
}

const std::string& ConditionElement::getName() const {
	return _name;
}

common::ConditionType ConditionElement::getType() const {
	return _type;
}

int ConditionElement::getActivationDelay() const {
	return _activationDelay;
}

bool ConditionElement::isOn() const {
	return _on;
}

bool ConditionElement::isStopCommandZone() const {
	return _stopCommandZone;
}

void ConditionElement::setExpression(const std::string& value) {
	_expression = value;
}

const std::string& ConditionElement::getExpression() const {
	return _expression;
}

void ConditionElement::setMessage(const std::string& value) {
	_message = value;
}

const std::string& ConditionElement::getMessage() const {
	return _message;
}

void ConditionElement::setMailRecipient(const std::string& recipient) {
	_mailRecipient = recipient;
}

const std::string& ConditionElement::getMailRecipient() const {
	return _mailRecipient;
}

void ConditionElement::setSmsNumber(const std::string& number) {
	_smsNumber = number;
}

const std::string& ConditionElement::getSmsNumber() const {
	return _smsNumber;
}

void ConditionElement::setActions(const std::string& actions) {
	_actions = actions;
}

const std::string& ConditionElement::getActions() const {
	return _actions;
}

}
