/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_CONDITIONELEMENT_H_
#define CORE_CONDITIONELEMENT_H_

#include <common/base/CommonTypes.h>
#include <string>
#include <vector>

namespace core {

/**
 * Class defining an alarm element.
 */
class ConditionElement {

public:
	/**
	 * Constructor.
	 */
	explicit ConditionElement(common::ConditionType type, const std::string& name, int activationDelay, bool on, bool stopCommandZone);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ConditionElement();

	/**
	 * Gets the name.
	 */
	const std::string& getName() const;

	/**
	 * Returns the level of the alarm.
	 */
	common::ConditionType getType() const;

	/**
	 * Gets the activation delay.
	 */
	int getActivationDelay() const;

	/**
	 * Returns true if is on.
	 */
	bool isOn() const;

	/**
	 * Returns true if stops the command zone (only for Alarm).
	 */
	bool isStopCommandZone() const;

	/**
	 * Sets the expression of the alarm.
	 */
	void setExpression(const std::string& value);

	/**
	 * Returns the expression of the alarm.
	 */
	const std::string& getExpression() const;

	/**
	 * Sets the message of the alarm.
	 */
	void setMessage(const std::string& value);
	
	/**
	 * Returns the message of the alarm.
	 */
	const std::string& getMessage() const;
	
	/**
	 * Sets the mail recipient.
	 */
	void setMailRecipient(const std::string& recipient);

	/**
	 * Gets the mail recipient.
	 */
	const std::string& getMailRecipient() const;

	/**
	 * Adds a sms destination.
	 */
	void setSmsNumber(const std::string& number);

	/**
	 * Gets the sms number.
	 */
	const std::string& getSmsNumber() const;

	/**
	 * Sets the actions.
	 */
	void setActions(const std::string& actions);

	/**
	 * Gets the actions.
	 */
	const std::string& getActions() const;

private:
	common::ConditionType _type;
	std::string _name;
	int _activationDelay;
	bool _on;
	bool _stopCommandZone;
	std::string _expression;
	std::string _message;
	std::string _mailRecipient;
	std::string _smsNumber;
	std::string _actions;
};

}

#endif /*CORE_CONDITIONELEMENT_H_*/
