/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_CONDITIONSPARSER_H_
#define CORE_CONDITIONSPARSER_H_

#include <vector>

#include <common/parser/GenericParser.h>

namespace core {

class ConditionElement;

/**
 * Class defining an XML parser for the user alarms.
 */
class ConditionsParser : public common::GenericParser {

public:
	/**
	 * Constructor.
	 */
	ConditionsParser();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~ConditionsParser();

	/**
	 * Gets the alarms.
	 */
	const std::vector<ConditionElement*>& getConditionElements() const;
	
protected:
	virtual void startElement(const std::string& qName);
	virtual void endElement(const std::string& qName);
	
private:
	const static std::string CONDITIONS_TAG;
	const static std::string ALARM_TAG;
	const static std::string WARNING_TAG;
	const static std::string WATCHDOG_TAG;
	const static std::string COMMAND_ZONE_TERMINATION_TAG;
	const static std::string MAIL_TAG;
	const static std::string SMS_TAG;

	const static std::string EXPRESSION_TAG;
	const static std::string MESSAGE_TAG;
	const static std::string ACTIONS_TAG;

	const static std::string NAME_ATTRIBUTE;
	const static std::string ON_ATTRIBUTE;
	const static std::string ACTIVATION_DELAY_ATTRIBUTE;
	const static std::string STOP_COMMAND_ZONE_ATTRIBUTE;
	const static std::string RECIPIENT_ATTRIBUTE;
	const static std::string NUMBER_ATTRIBUTE;
	const static std::string VALUE_ATTRIBUTE;
	const static std::string CONTENT_ATTRIBUTE;

	std::vector<ConditionElement*> _conditions;
	ConditionElement* _currentCondition;
};

}

#endif /*CORE_CONDITIONSPARSER_H_*/
