/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDEXCLUSIONELEMENT_H_
#define CORE_COMMANDEXCLUSIONELEMENT_H_

#include <string>

namespace core {

/**
 * Class defining a command exclusion element.
 */
class CommandExclusionElement {

public:
	/**
	 * Constructor.
	 */
	CommandExclusionElement(const std::string& servantName, const std::string& commandName, bool toBeQueued);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CommandExclusionElement();

	/*
	 * Returns the servant name.
	 */
	const std::string& getServantName() const;	
	
	/*
	 * Returns the command name.
	 */
	const std::string& getCommandName() const;	

	/*
	 * Returns true if to be queued.
	 */
	bool toBeQueued() const;	
	
private:
	std::string _servantName;
	std::string _commandName;
	bool _toBeQueued;

};

}

#endif /*CORE_COMMANDEXCLUSIONELEMENT_H_*/