/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SchedulerRulesParser.h"
#include "ParallelExclusionElement.h"
#include "CommandExclusionElement.h"

#include <iostream>

using namespace std;
using namespace common;

namespace core {

const std::string SchedulerRulesParser::SCHEDULER_RULES_TAG = "scheduler_rules";
const std::string SchedulerRulesParser::PARALLEL_EXCLUSION_TAG = "parallel_exclusion";
const std::string SchedulerRulesParser::NOT_PARALLELISABLE_TAG = "not_parallelisable";
const std::string SchedulerRulesParser::COMMAND_TAG = "command";

const std::string SchedulerRulesParser::NAME_ATTRIBUTE = "name";
const std::string SchedulerRulesParser::SERVANT_ATTRIBUTE = "servant";
const std::string SchedulerRulesParser::QUEUE_ATTRIBUTE = "queue";

using namespace std;

SchedulerRulesParser::SchedulerRulesParser() :
	common::GenericParser(),
	_currentExclusion(0) {
}

SchedulerRulesParser::~SchedulerRulesParser() {
	vector<ParallelExclusionElement*>::iterator it;
	for (it = _exclusions.begin(); it < _exclusions.end(); it++) {
		delete (*it);
	}

	vector<CommandExclusionElement*>::iterator cit;
	for (cit = _notParallelisables.begin(); cit < _notParallelisables.end(); cit++) {
		delete (*cit);
	}
	
}


void SchedulerRulesParser::startElement(const std::string& qName) {
	
	if (qName == SCHEDULER_RULES_TAG) {
		// do nothing
		
	} else if (qName == PARALLEL_EXCLUSION_TAG) {
		_currentExclusion = new ParallelExclusionElement();
		_exclusions.push_back(_currentExclusion);

	} else if (qName == NOT_PARALLELISABLE_TAG) {
		_notParallelisables.push_back(new CommandExclusionElement(getAttribute(SERVANT_ATTRIBUTE), getAttribute(NAME_ATTRIBUTE), getAttribute(QUEUE_ATTRIBUTE) == "true"));

	} else if (qName == COMMAND_TAG) {
		CommandExclusionElement* command = new CommandExclusionElement(getAttribute(SERVANT_ATTRIBUTE), getAttribute(NAME_ATTRIBUTE), getAttribute(QUEUE_ATTRIBUTE) == "true");
		_currentExclusion->addCommand(command);
		

	} else {
		throw UnknownTagException(qName, _fileName);
	}
	
}


std::vector<ParallelExclusionElement*>& SchedulerRulesParser::getExclusions() {
	return _exclusions;
}

std::vector<CommandExclusionElement*>& SchedulerRulesParser::getNotParallelisables() {
	return _notParallelisables;
}



}
