/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SCHEDULERRULESPARSER_H_
#define CORE_SCHEDULERRULESPARSER_H_

#include <vector>

#include <common/parser/GenericParser.h>

namespace core {

class ParallelExclusionElement;
class CommandExclusionElement;

/**
 * Class defining an XML parser for the scheduler rules.
 */
class SchedulerRulesParser : public common::GenericParser {

public:
	/**
	 * Constructor.
	 */
	SchedulerRulesParser();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~SchedulerRulesParser();

	/**
	 * Returns the parallel exclusion elements.
	 */
	std::vector<ParallelExclusionElement*>& getExclusions();

	/**
	 * Returns the not parallelisable elements.
	 */
	std::vector<CommandExclusionElement*>& getNotParallelisables();
	
	
protected:
	virtual void startElement(const std::string& qName);
	
private:
	const static std::string SCHEDULER_RULES_TAG;
	const static std::string PARALLEL_EXCLUSION_TAG;
	const static std::string NOT_PARALLELISABLE_TAG;
	const static std::string COMMAND_TAG;
	
	const static std::string NAME_ATTRIBUTE;
	const static std::string SERVANT_ATTRIBUTE;
	const static std::string QUEUE_ATTRIBUTE;
	 

	std::vector<ParallelExclusionElement*> _exclusions;
	std::vector<CommandExclusionElement*> _notParallelisables;
	ParallelExclusionElement* _currentExclusion;

};

}

#endif /*CORE_SCHEDULERRULESPARSER_H_*/
