/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SpyElement.h"
#include "SpyCommandOverride.h"

#include <iostream>

namespace core {

SpyElement::SpyElement(const std::string& driverName, const std::string& commandName, float periodMs) :
	_driverName(driverName),
	_commandName(commandName),
	_periodMs(periodMs) {
}

SpyElement::~SpyElement() {
	std::vector<SpyCommandOverride*>::iterator it;
	for (it = _commandOverrides.begin(); it < _commandOverrides.end(); it++) {
		delete (*it);
	}
}

const std::string& SpyElement::getDriverName() const {
	return _driverName;
}

const std::string& SpyElement::getCommandName() const {
	return _commandName;
}

float SpyElement::getPeriodMs() const {
	return _periodMs;
}

void SpyElement::addCommandOverride(SpyCommandOverride* commandOverride) {
	_commandOverrides.push_back(commandOverride);
}

const std::vector<SpyCommandOverride*>& SpyElement::getCommandOverrides() const {
	return _commandOverrides;
}

}