/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SPYELEMENT_H_
#define CORE_SPYELEMENT_H_

#include <string>
#include <vector>

namespace core {

class SpyCommandOverride;

/**
 * Class defining a spy element.
 */
class SpyElement {

public:
	/**
	 * Constructor.
	 */
	SpyElement(const std::string& driverName, const std::string& commandName, float periodMs);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~SpyElement();

	/**
	 * Returns the name of the driver.
	 */
	const std::string& getDriverName() const;
	
	/**
	 * Returns the name of the command.
	 */
	const std::string& getCommandName() const;
	
	/**
	 * Returns the period of the driver.
	 */
	float getPeriodMs() const;
	
	/**
	 * Adds a new spy command override
	 */
	void addCommandOverride(SpyCommandOverride* commandOverride);
	
	/**
	 * Returns the spy command overrides
	 */
	const std::vector<SpyCommandOverride*>& getCommandOverrides() const;
	
private:
	std::string _driverName;
	std::string _commandName;
	float _periodMs;
	
	std::vector<SpyCommandOverride*> _commandOverrides;
};

}

#endif /*CORE_ALARMELEMENT_H_*/