/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SpyParser.h"
#include "SpyElement.h"
#include "SpyCommandOverride.h"
#include <common/base/Convert.h>

#include <iostream>

using namespace std;
using namespace common;

namespace core {

const std::string SpyParser::DRIVERS_TAG = "drivers";
const std::string SpyParser::DRIVER_TAG = "driver";
const std::string SpyParser::COMMAND_OVERRIDE_TAG = "command_override";
const std::string SpyParser::NAME_ATTRIBUTE = "name";
const std::string SpyParser::COMMAND_ATTRIBUTE = "command";
const std::string SpyParser::PERIODMS_ATTRIBUTE = "periodMs";
const std::string SpyParser::CONTROLLER_ATTRIBUTE = "controller";
	
using namespace std;

SpyParser::SpyParser() :
	common::GenericParser() {
}

SpyParser::~SpyParser() {

	vector<SpyElement*>::iterator it;
	for (it = _spyElements.begin(); it < _spyElements.end(); it++) {
		delete (*it);
	}
}

void SpyParser::startElement(const std::string& qName) {
	
	if (qName == DRIVERS_TAG) {
		// do nothing
		
	} else if (qName == DRIVER_TAG) {
		_currentSpyElement = new SpyElement(getAttribute(NAME_ATTRIBUTE), getAttribute(COMMAND_ATTRIBUTE), to<float>(getAttribute(PERIODMS_ATTRIBUTE)));
		_spyElements.push_back(_currentSpyElement);
		
	} else if (qName == COMMAND_OVERRIDE_TAG) {
		_currentSpyElement->addCommandOverride(new SpyCommandOverride(getAttribute(CONTROLLER_ATTRIBUTE), getAttribute(COMMAND_ATTRIBUTE)));
	} else {
		throw UnknownTagException(qName, _fileName);
	}
	
}

const std::vector<SpyElement*>& SpyParser::getSpyElements() const {
	return _spyElements;
}



}
