/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SPYPARSER_H_
#define CORE_SPYPARSER_H_

#include <vector>

#include <common/parser/GenericParser.h>

namespace core {

class SpyElement;

/**
 * Class defining an XML parser for the spy.
 */
class SpyParser : public common::GenericParser {

public:
	/**
	 * Constructor.
	 */
	SpyParser();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~SpyParser();

	/**
	 * Gets the spy elements.
	 */
	const std::vector<SpyElement*>& getSpyElements() const;
		
protected:
	virtual void startElement(const std::string& qName);
	
private:
	const static std::string DRIVERS_TAG;
	const static std::string DRIVER_TAG;
	const static std::string COMMAND_OVERRIDE_TAG;
	
	const static std::string NAME_ATTRIBUTE;
	const static std::string COMMAND_ATTRIBUTE;
	const static std::string PERIODMS_ATTRIBUTE;
	const static std::string CONTROLLER_ATTRIBUTE;

	std::vector<SpyElement*> _spyElements;
	SpyElement* _currentSpyElement;
};

}

#endif /*CORE_USERALARMSPARSER_H_*/
