/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SurveyParser.h"
#include <common/base/Convert.h>
#include <ics/InstrumentManager/InstrumentManager.h>
#include <ics/InstrumentAbstraction/ExperimentController.h>
#include <iostream>

using namespace std;

namespace core {

const std::string SurveyParser::SURVEY_TAG = "survey";
const std::string SurveyParser::PROPERTY_TAG = "property";
const std::string SurveyParser::SERVANT_NAME_ATTRIBUTE = "servant_name";
const std::string SurveyParser::PROPERTY_NAME_ATTRIBUTE = "property_name";
const std::string SurveyParser::INDEX_ATTRIBUTE = "index";
const std::string SurveyParser::ALIAS_ATTRIBUTE = "alias";

using namespace std;

SurveyParser::SurveyParser() :
	common::GenericParser() {
}

SurveyParser::~SurveyParser() {
}

void SurveyParser::startElement(const std::string& qName) {

	if (qName == SURVEY_TAG) {
		// do nothing

	} else if (qName == PROPERTY_TAG) {
		SurveyElement surveyElement;
		surveyElement.servantName = getAttribute(SERVANT_NAME_ATTRIBUTE);
		surveyElement.propertyName = getAttribute(PROPERTY_NAME_ATTRIBUTE);

		if (getAttribute(INDEX_ATTRIBUTE) != "") {
			surveyElement.index = common::to<long>(getAttribute(INDEX_ATTRIBUTE));
		} else {
			surveyElement.index = -1;
		}

		// adding alias from ICS controllers
		if (InstrumentManager::getInstance() != 0) {
			try {
				surveyElement.alias = InstrumentManager::getInstance()->getExperimentController(surveyElement.servantName)->getPropertyAlias(surveyElement.propertyName);

			} catch (...) {
				// Does nothing
			}
		} else {
			surveyElement.alias = getAttribute(ALIAS_ATTRIBUTE);
		}

		_surveyElements.push_back(surveyElement);

	} else {
		throw UnknownTagException(qName, _fileName);
	}

}

const std::vector<SurveyParser::SurveyElement>& SurveyParser::getSurveyElements() const {
	return _surveyElements;
}

}
