/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SURVEYPARSER_H_
#define CORE_SURVEYPARSER_H_

#include <vector>

#include <common/parser/GenericParser.h>

namespace core {

/**
 * Class defining an XML parser for the survey.
 */
class SurveyParser : public common::GenericParser {

public:
	/**
	 * Constructor.
	 */
	SurveyParser();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~SurveyParser();

	/**
	 * Class defining a survey element.
	 */
	struct SurveyElement {
		std::string servantName;
		std::string propertyName;
		long index;
		std::string alias;
	};
	
	/**
	 * Gets the survey elements.
	 */
	const std::vector<SurveyElement>& getSurveyElements() const;
		
protected:
	virtual void startElement(const std::string& qName);
	
private:
	const static std::string SURVEY_TAG;
	const static std::string PROPERTY_TAG;
	
	const static std::string SERVANT_NAME_ATTRIBUTE;
	const static std::string PROPERTY_NAME_ATTRIBUTE;
	const static std::string INDEX_ATTRIBUTE;
	const static std::string ALIAS_ATTRIBUTE;

	std::vector<SurveyElement> _surveyElements;
};

}

#endif /*CORE_SURVEYPARSER_H_*/
