/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "VariablesParser.h"
#include "VariableElement.h"
#include <core/conditions/CommandZoneTermination.h>
#include <common/output/OutputStream.h>
#include <common/base/Convert.h>
#include <iostream>


using namespace std;
using namespace common;

namespace core {

const std::string VariablesParser::VARIABLES_TAG = "variables";
const std::string VariablesParser::VARIABLE_TAG = "variable";
const std::string VariablesParser::EXPRESSION_TAG = "expression";

const std::string VariablesParser::NAME_ATTRIBUTE = "name";
const std::string VariablesParser::VALUE_ATTRIBUTE = "value";
const std::string VariablesParser::CONTENT_ATTRIBUTE = "content";

using namespace std;

VariablesParser::VariablesParser() :
	common::GenericParser(),
	_currentVariable(0) {
}

VariablesParser::~VariablesParser() {
	vector<VariableElement*>::iterator it;
	for (it = _variables.begin(); it < _variables.end(); it++) {
		delete (*it);
	}
	_variables.clear();
}


void VariablesParser::startElement(const std::string& qName) {

	if (qName == VARIABLES_TAG) {
		// do nothing

	} else if (qName == VARIABLE_TAG) {
		_currentVariable = new VariableElement(getAttribute(NAME_ATTRIBUTE));

	} else if (qName == EXPRESSION_TAG) {
		_currentVariable->setExpression(getAttribute(VALUE_ATTRIBUTE));
		
	} else {
		throw UnknownTagException(qName, _fileName);
	}
	
}

void VariablesParser::endElement(const std::string& qName) {

	if (qName == VARIABLES_TAG) {
		// do nothing
		
	} else if (qName == VARIABLE_TAG) {
		// verify that we have a condition and an action
		_variables.push_back(_currentVariable);

	} else if (qName == EXPRESSION_TAG) {
		// do nothing

	} else {
		throw UnknownTagException(qName, _fileName);
	}
}

const std::vector<VariableElement*>& VariablesParser::getVariableElements() const {
	return _variables;
}

}
