/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_VARIABLESPARSER_H_
#define CORE_VARIABLESPARSER_H_

#include <vector>
#include <common/parser/GenericParser.h>

namespace core {

class VariableElement;

/**
 * Class defining an XML parser for the user variables.
 */
class VariablesParser : public common::GenericParser {

public:
	/**
	 * Constructor.
	 */
	VariablesParser();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~VariablesParser();

	/**
	 * Gets the alarms.
	 */
	const std::vector<VariableElement*>& getVariableElements() const;
	
protected:
	virtual void startElement(const std::string& qName);
	virtual void endElement(const std::string& qName);
	
private:
	const static std::string VARIABLES_TAG;
	const static std::string VARIABLE_TAG;

	const static std::string EXPRESSION_TAG;

	const static std::string NAME_ATTRIBUTE;
	const static std::string VALUE_ATTRIBUTE;
	const static std::string CONTENT_ATTRIBUTE;

	std::vector<VariableElement*> _variables;
	VariableElement* _currentVariable;
};

}

#endif /*CORE_VARIABLESPARSER_H_*/
