/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_RESOURCEMANAGER_H_
#define CORE_RESOURCEMANAGER_H_

#include <common/base/CommonTypes.h>
#include <string>

namespace core {

class StateAccessor;

/**
 * Class defining the singleton resource manager.
 */
class ResourceManager {

public:
	/**
	 * Returns the instance.
	 */	
	static ResourceManager* getInstance();
	
	/**
	 * Initialises the instance.
	 */
	static void initInstance();

	/**
	 * resets the instance.
	 */
	static void resetInstance();

	/**
	 * Notifies that a new file has been created.
	 */
	void notifyNewFileCreated(common::FileType fileType, const std::string& fileName);

private:
	ResourceManager();
	~ResourceManager();

	void init();

	static ResourceManager* m_instance;

	StateAccessor* m_stateAccessor;
};

}

#endif
