/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_REQUESTREPLY_H_
#define CORE_REQUESTREPLY_H_

#include <string>

namespace core {

/**
 * Class defining a request reply.
 */
class RequestReply { 

public:
	/**
	 * Type of request reply.
	 */
	typedef enum {EXECUTE, QUEUE, REJECT, FAIL} Value;
	
	/**
	 * Constructor.
	 */
	RequestReply(Value value, std::string const & reason = "");
	
	/**
	 * Virtual destructor.
	 */
	virtual ~RequestReply();
	
	/**
	 * Gets the value.
	 */
	Value getValue() const;
	
	/**
	 * Gets the reason.
	 */
	std::string const & getReason() const;
	
private:
	Value _value;
	std::string _reason;
};	

}

#endif /*CORE_REQUESTREPLY_H_*/