/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SCHEDULERARBITRATOR_H_
#define CORE_SCHEDULERARBITRATOR_H_

#include "Scheduler.h"
#include <common/base/HashMap.h>
#include <string>
#include <vector>
#include <list>

namespace core {

class Operation;

/**
 * Class defining a scheduler arbitrator that manages the arbitration of execution of commands given
 * a set of rules.
 */
class SchedulerArbitrator {

public:
	/**
	 * Constructor.
	 */
	SchedulerArbitrator();

	/**
	 * Virtual destructor.
	 */
	virtual ~SchedulerArbitrator();

	/**
	 * Loads the rules.
	 */
	void loadRules(const std::string& configPath);

	/**
	 * Print the rules.
	 */
	void printRules() const;

	/**
	 * Requests the execution of a command.
	 */
	RequestReply requestExecution(unsigned long databaseID, unsigned long commandID, const std::list<Operation*>& activeOperations, bool rejectSameCommand = true) const;

	/**
	 * Requests the execution of a enable/disable command.
	 */
	RequestReply requestEnableDisableExecution(unsigned long databaseID, unsigned long commandID, const std::list<Operation*>& activeOperations) const;

	/**
	 * Requests the execution of a command duration.
	 */
	RequestReply requestDuration(unsigned long databaseID, unsigned long commandID, const std::list<Operation*>& activeOperations) const;

	/**
	 * Requests the execution of a property setter.
	 */
	RequestReply requestPropertySetter(unsigned long databaseID, unsigned long propertyID, const std::list<Operation*>& activeOperations) const;

private:
	const static std::string SCHEDULER_RULES_PATH;

	common::HashMap<unsigned long, std::vector<std::pair<unsigned long, bool> > > _rules;
	common::HashMap<unsigned long, bool> _notParallelisableCommandIDs;
};

}

#endif /*CORE_SCHEDULERARBITRATOR_H_*/