/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SENDERCOMMANDSTATECHANGESUBSCRIBER_H_
#define CORE_SENDERCOMMANDSTATECHANGESUBSCRIBER_H_

#include "core/dataaccess/CommandStateChangeSubscriber.h"
#include <common/base/Queue.h>

namespace core {

/**
 * Class defining a command state change subscriber for Sender.
 */
class SenderCommandStateChangeSubscriber : public CommandStateChangeSubscriber {

public :
	/**
	 * Constructor.
	 */
	SenderCommandStateChangeSubscriber(unsigned long databaseID, unsigned long commandID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~SenderCommandStateChangeSubscriber();
	
	/**
	 * Notifies that the command state has changed.
	 */
	virtual void commandStateChanged(unsigned long databaseID, unsigned long commandID, common::CommandState state);

	/**
	 * Waits for the termination of the command.
	 */
	void waitForTermination();

private:
	unsigned long _databaseID;
	unsigned long _commandID;
	common::Queue<bool>* _waitForTerminationQueue;
};

} 

#endif /*CORE_COMMANDSTATECHANGESUBSCRIBER_H_*/
