/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SenderPropertySetterTerminatedSubscriber.h"
#include <common/output/OutputStream.h>
#include <iostream>

using namespace common;
using namespace std;

namespace core {

SenderPropertySetterTerminatedSubscriber::SenderPropertySetterTerminatedSubscriber(unsigned long databaseID, unsigned long propertyID) :
	_propertyID(propertyID),
	_waitForTerminationQueue(new Queue<bool>(1)) {
	
	setDatabaseID(databaseID);
}

SenderPropertySetterTerminatedSubscriber::~SenderPropertySetterTerminatedSubscriber() {
	delete _waitForTerminationQueue;
}

void SenderPropertySetterTerminatedSubscriber::propertySetterTerminated(unsigned long databaseID, unsigned long propertyID, bool successful) {
	
	if (propertyID == _propertyID) {
		_waitForTerminationQueue->put(new bool(successful));
	}	
}

bool SenderPropertySetterTerminatedSubscriber::waitForTermination() {
	bool* successPointer = _waitForTerminationQueue->get();
	bool success = *successPointer;
	delete successPointer;
	
	return success;
}

}