/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SENDERPROPERTYSETTERTERMINATEDSUBSCRIBER_H_
#define CORE_SENDERPROPERTYSETTERTERMINATEDSUBSCRIBER_H_

#include <core/dataaccess/PropertySetterTerminatedSubscriber.h>
#include <common/base/Queue.h>

namespace core {

/**
 * Class defining a sender property setter terminated subscriber.
 */
class SenderPropertySetterTerminatedSubscriber : public PropertySetterTerminatedSubscriber {

public :
	/**
	 * Constructor.
	 */
	SenderPropertySetterTerminatedSubscriber(unsigned long databaseID, unsigned long propertyID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~SenderPropertySetterTerminatedSubscriber();
	
	/**
	 * Notifies that the property setter has terminated.
	 */
	virtual void propertySetterTerminated(unsigned long databaseID, unsigned long propertyID, bool successful);
	
	/**
	 * Waits for the termination of the command.
	 */
	bool waitForTermination();
	
private:
	unsigned long _propertyID;
	common::Queue<bool>* _waitForTerminationQueue;
};

} 

#endif /*CORE_SENDERPROPERTYSETTERTERMINATEDSUBSCRIBER_H_*/