/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */
#include <common/connection/RestJsonHttpSyncConnection.h>
#include <common/base/Exception.h>
#include <common/base/ServerProperties.h>
#include <boost/property_tree/json_parser.hpp>
#include <rapidjson/writer.h>
#include <rapidjson/stringbuffer.h>
#include <rapidjson/prettywriter.h>
#include "FluentdSerdonWriter.h"
#include <common/base/ServerProperties.h>

using namespace std;
using namespace common;
using namespace boost;

using namespace rapidjson;

namespace core {


const std::string FluentdSerdonWriter::SRC = "src";
const std::string FluentdSerdonWriter::DST = "dst";

using namespace common;

FluentdSerdonWriter::SerdonWriterException::SerdonWriterException(const std::string& message) :
		common::Exception("acquisition::FluentdSerdonWriter::SerdonWriterException", message) {
}

FluentdSerdonWriter::FluentdSerdonWriter() :
		m_showMessages(false) {

	m_nomadSendData = ServerProperties::getInstance()->getNomadSenddata();
	to_lower(m_nomadSendData);

	try {
		m_useNewSerdonTransfer = ServerProperties::getInstance()->getString("useNewSerdonTransfer");
	}
	catch (...) {
		m_useNewSerdonTransfer = false;
	}
	to_lower(m_useNewSerdonTransfer);
}

FluentdSerdonWriter::~FluentdSerdonWriter() {

}

void FluentdSerdonWriter::write(const std::string& src, const std::string& dst) {

//	if ((m_nomadSendData == "-1") || (m_nomadSendData == "2") || (m_nomadSendData == "-2") {
//		return;
//	}

	//cout << "m_useNewSerdonTransfer = " << m_useNewSerdonTransfer << endl;
	if (m_useNewSerdonTransfer != "true") {
		return;
	}

	m_properties.clear();

	rapidjson:: StringBuffer s;
	PrettyWriter<StringBuffer> writer(s);

//	Start new message
	writer.StartObject();

	try {

		writer.Key(SRC.c_str());
		writer.String(src.c_str());

		writer.Key(DST.c_str());
		if (m_nomadSendData == "-1" || m_nomadSendData == "0") {
			writer.String(dst.c_str());
		} else if ((m_nomadSendData == "1")) {
			writer.String(dst.c_str());
		}

		writer.EndObject();

		// only show messages in case it was activated
		if (m_showMessages) {
			cout << "Fluentd log message: " << s.GetString() << endl << endl;
		}

		// Trying to detect bad JSON messages (e.g. driver errors with bad characters) using rapidjson::Document.Parse(string).HasParseError()
		// does not provide errors.

		RestJsonHttpSyncConnection con;

		con.openUrl("localhost", "8888");

		con.setPath(string("/datatransfert"));

		con.setData(s.GetString());
		boost::property_tree::ptree response = con.send(false);

	} catch (const Exception& e) {
		// log to server
		cerr << e.printStack() << endl;
		slog << e.printStack() << eol;
	}
}

void FluentdSerdonWriter::showJSONMessages(bool value) {
	m_showMessages = value;
}

}
