/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef FLUENTDSERDONWRITER_H_
#define FLUENTDSERDONWRITER_H_

#include <common/base/Exception.h>
#include <common/base/BaseTypes.h>
#include <string>
#include <boost/property_tree/ptree.hpp>

namespace core {

class FluentdSerdonWriter {

public:
	/**
	 * DataWriter exception.
	 */
	class SerdonWriterException : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		SerdonWriterException(const std::string& message);
	};


	FluentdSerdonWriter();
	~FluentdSerdonWriter();
	void write(const std::string& src, const std::string& dst);

	void showJSONMessages(bool value);

private:

	std::string m_nomadSendData;
	std::string m_useNewSerdonTransfer;

	boost::property_tree::ptree m_properties;

	bool m_showMessages;

	const static std::string SRC;
	const static std::string DST;

};

}

#endif /* FLUENTDSERDONWRITER_H_ */
