/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ServantInfo.h"

#include <ics/InstrumentAbstraction/ExperimentControllerCreator.h>
#include <ics/DeviceContainer/DeviceDriverCreator.h>
#include <ics/Module/ModuleManager.h>
#include <common/base/ServerProperties.h>
#include <common/output/OutputStream.h>
#include <iostream>
#include <fstream>

using namespace std;
using namespace common;

namespace core {

ServantInfo* ServantInfo::_instance = 0;

ServantInfo::ServantInfo() {
}

ServantInfo::~ServantInfo() {
}

ServantInfo* ServantInfo::getInstance() {
	return _instance;
}

void ServantInfo::initInstance() {

	if (_instance == 0) {
		_instance = new ServantInfo();
		_instance->init();
	}
}

void ServantInfo::init() {
}

void ServantInfo::retrieveTypeClassMap() {

	string outFileName = ServerProperties::getInstance()->getHomePath() + "/ControllerTypeClassMap.properties";

	ofstream outFile;
	outFile.open(outFileName.c_str());

	map<string, string> typeClassMap = ExperimentControllerCreator::getInstance()->retrieveAllTypeClasses();

	for (map<string, string>::const_iterator t = typeClassMap.begin(); t != typeClassMap.end(); ++t) {
		outFile << t->first << "=" << t->second << endl;
	}

	outFile.close();

	cout << "Wrote " << outFileName << endl;


	map<string, DeviceDriverCreator::TypeInfo> typeClassNameMap = DeviceDriverCreator::getInstance()->retrieveAllTypeClasses();

	outFileName = ServerProperties::getInstance()->getHomePath() + "/DriverTypeClassMap.properties";

	outFile.open(outFileName.c_str());

	for (map<string, DeviceDriverCreator::TypeInfo>::const_iterator t = typeClassNameMap.begin(); t != typeClassNameMap.end(); ++t) {
		outFile << t->first << "=" << t->second.cppClass << endl;
	}

	outFile.close();

	cout << "Wrote " << outFileName << endl;


	outFileName = ServerProperties::getInstance()->getHomePath() + "/DriverTypeClassNameMap.properties";

	outFile.open(outFileName.c_str());

	for (map<string, DeviceDriverCreator::TypeInfo>::const_iterator t = typeClassNameMap.begin(); t != typeClassNameMap.end(); ++t) {
		outFile << t->first << "=" << t->second.cppClassName << endl;
	}

	outFile.close();

	cout << "Wrote " << outFileName << endl;


	outFileName = ServerProperties::getInstance()->getHomePath() + "/DriverTypeCategoriesMap.properties";

	outFile.open(outFileName.c_str());

	for (map<string, DeviceDriverCreator::TypeInfo>::const_iterator t = typeClassNameMap.begin(); t != typeClassNameMap.end(); ++t) {
		outFile << t->first << "=" << t->second.categories << endl;
	}

	outFile.close();

	cout << "Wrote " << outFileName << endl;


	outFileName = ServerProperties::getInstance()->getHomePath() + "/ModuleTypeMap.properties";

	outFile.open(outFileName.c_str());

	typeClassMap = ModuleManager::getInstance()->getAllTypeModules();

	for (map<string, string>::const_iterator t = typeClassMap.begin(); t != typeClassMap.end(); ++t) {
		outFile << t->first << "=" << t->second << endl;
	}

	outFile.close();

	cout << "Wrote " << outFileName << endl;
}

void ServantInfo::retrieveControllerTypeList() {

	string outFileName = ServerProperties::getInstance()->getHomePath() + "/ControllerList.properties";

	ofstream outFile;
	outFile.open(outFileName.c_str());

	// server types
	map<string, string> typeClassMap = ExperimentControllerCreator::getInstance()->retrieveAllTypeClasses();

	outFile << "# server" << endl;
	for (map<string, string>::const_iterator t = typeClassMap.begin(); t != typeClassMap.end(); ++t) {
		outFile << t->first << endl;
	}

	// module types
	map<string, vector<string> > controllerTypeMap = ModuleManager::getInstance()->getControllerTypesByModule();

	for (map<string, vector<string> >::const_iterator m = controllerTypeMap.begin(); m != controllerTypeMap.end(); ++m) {

		const vector<string>& controllerTypes = m->second;
		if (!controllerTypes.empty()) {

			outFile << endl << "# module " << m->first << endl;
			for (vector<string>::const_iterator t = controllerTypes.begin(); t != controllerTypes.end(); ++t) {
				outFile << *t << endl;
			}
		}
	}

	outFile.close();

	cout << "Wrote " << outFileName << endl;
}

void ServantInfo::displayTypeStats() {

	int serverControllerCounter = 0;
	int serverDriverCounter = 0;
	int moduleCounter = 0;
	int moduleControllerCounter = 0;
	int moduleDriverCounter = 0;

	map<string, string> typeClassMap = ExperimentControllerCreator::getInstance()->retrieveAllTypeClasses();
	serverControllerCounter = typeClassMap.size();

	//map<string, DeviceDriverCreator::TypeInfo> typeClassNameMap = DeviceDriverCreator::getInstance()->retrieveAllTypeClasses();
	//serverDriverCounter = typeClassNameMap.size();

	moduleCounter = ModuleManager::getInstance()->getLoadedModules().size();
	moduleControllerCounter = ModuleManager::getInstance()->getAllControllerTypes().size();
	moduleDriverCounter = ModuleManager::getInstance()->getAllDriverTypes().size();

	cout << "#controllers (server) = " << serverControllerCounter << endl;
	cout << "#drivers     (server) = " << serverDriverCounter << endl;
	cout << "#modules              = " << moduleCounter << endl;
	cout << "#controllers (module) = " << moduleControllerCounter << endl;
	cout << "#drivers     (module) = " << moduleDriverCounter << endl;
}

}
