/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SERVANTINFO_H_
#define CORE_SERVANTINFO_H_

#include <string>
#include <common/base/BaseTypes.h>

namespace core {

/**
 * Singleton class providing an access to the servant info.
 */
class ServantInfo {

public:
	/**
	 * Gets the instance.
	 */
	static ServantInfo* getInstance();

	/**
	 * Initialises the instance.
	 */
	static void initInstance();

	/**
	 * Retrieves the type class map.
	 */
	void retrieveTypeClassMap();

	/**
	 * Retrieves the type list.
	 */
	void retrieveControllerTypeList();

	/**
	 * Displays some type stats.
	 */
	void displayTypeStats();

private:
	ServantInfo();
	~ServantInfo();
	void init();

	static ServantInfo* _instance;
};

}

#endif
