/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef FLUENTDSURVEYWRITER_H_
#define FLUENTDSURVEYWRITER_H_

#include <common/base/Exception.h>
#include <common/base/BaseTypes.h>
#include <string>
#include <boost/regex.hpp>
#include <boost/property_tree/ptree.hpp>

#include <rapidjson/writer.h>
#include <rapidjson/stringbuffer.h>
#include <rapidjson/prettywriter.h>

namespace common {

class Date;

}


namespace core {

class SurveyElement;
class PropertySurveyElement;
class NumorSurveyElement;


class FluentdSurveyWriter {

public:
	/**
	 * Survey exception.
	 */
	class SurveyException : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		SurveyException(const std::string& message);
	};


	FluentdSurveyWriter();
	~FluentdSurveyWriter();
	void write(SurveyElement* element);

	void showJSONMessages(bool value);

private:

	void init();

	void createValuesMessageUsingRapidJson(PropertySurveyElement* element, rapidjson::PrettyWriter<rapidjson::StringBuffer> &writer);
	void createValuesMessageUsingRapidJson(NumorSurveyElement* element, rapidjson::PrettyWriter<rapidjson::StringBuffer> &writer);

	std::string m_nomadSendData;

	//Properties IDs
	unsigned long m_propertyIdOfProposalId;
	bool m_initDone;

	boost::property_tree::ptree m_properties;

	bool m_showMessages;

	const static std::string VERSION_STR;
	const static int32 VERSION_VALUE;

	// Histo
	const static std::string CTYPE;
	const static std::string CNAME;
	const static std::string PNAME;
	const static std::string ALIAS;
	const static std::string UNIT;
	const static std::string DAY;
	const static std::string PROPOSALID;
	const static std::string NVALUES;
	const static std::string VALUES;
	const static std::string VAL;
	const static std::string TIMESTAMP;

	// Numor
	const static std::string NNUMORS;
	const static std::string NUMORS;
	const static std::string NUMOR;
	const static std::string START;
	const static std::string END;

	// Proposal

};

}

#endif /* FLUENTDSURVEYWRITER_H_ */
