/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PropertySurveyElement.h"
#include "Survey.h"
#include <core/dataaccess/DataAccess.h>
#include <core/dataaccess/DataNotificationRouter.h>
#include <iostream>

using namespace std;
using namespace common;

namespace core {

PropertySurveyElement::PropertySurveyElement(unsigned long servantID, unsigned long propertyID, long index, const std::string& servantName, const std::string& propertyName, const std::string& alias) :
	m_servantID(servantID),
	m_propertyID(propertyID),
	m_index(index),
	m_servantName(servantName),
	m_propertyName(propertyName),
	m_alias(alias),
	m_propertyType(DataAccess::getInstance()->getPropertyType(0, m_propertyID)),
	m_lastDate(Date()),
	m_lastValue("") {
	
	try {
		m_servantType = DataAccess::getInstance()->getControllerType(0, m_servantID);
	}
	catch (const Exception&) {
		m_servantType = DataAccess::getInstance()->getDriverType(0, m_servantID);
	}

	try {
		m_propertyUnit = DataAccess::getInstance()->getStringValue(0, DataAccess::getInstance()->getUnitPropertyID(0, m_propertyID));
	}
	catch (...) {
	}

	// Get the start command ID.
	m_startCommandID = DataAccess::getInstance()->getCommandID(0, m_servantID, "start");

	// Subscriber.
	DataNotificationRouter::getInstance()->addCommandStateChangeSubscriber(m_startCommandID, this);

}

PropertySurveyElement::~PropertySurveyElement() {

	DataNotificationRouter::getInstance()->removeCommandStateChangeSubscriber(m_startCommandID, this);
}

void PropertySurveyElement::commandStateChanged(unsigned long databaseID, unsigned long commandID, common::CommandState state) {
	if (commandID == m_startCommandID) {
		m_active = (state == common::ACTIVE);
	}
}

unsigned long PropertySurveyElement::getServantID() const {
	return m_servantID;
}

std::string PropertySurveyElement::getServantName() const {
	return m_servantName;
}

std::string PropertySurveyElement::getServantType() const {
	return m_servantType;
}

unsigned long PropertySurveyElement::getPropertyID() const {
	return m_propertyID;
}

long PropertySurveyElement::getIndex() const {
	return m_index;
}

std::string PropertySurveyElement::getPropertyName() const {
	return m_propertyName;
}

std::string PropertySurveyElement::getAlias() const {
	return m_alias;
}

std::string PropertySurveyElement::getPropertyType() const {
	return m_propertyType;
}

void PropertySurveyElement::setActiveWriteTime(int seconds) {
	m_activeWriteTime = seconds;
}

int PropertySurveyElement::getActiveWriteTime() const {
	return m_activeWriteTime;
}

void PropertySurveyElement::setLastDate(const common::Date& lastDate) {
	m_lastDate = lastDate;
}

common::Date PropertySurveyElement::getLastDate() const {
	return m_lastDate;
}

void PropertySurveyElement::setLastValue(const std::string& lastValue) {
	m_lastValue = lastValue;
}

const std::string& PropertySurveyElement::getLastValue() const {
	return m_lastValue;
}

const std::string& PropertySurveyElement::getPropertyUnit() const {
	return m_propertyUnit;
}

bool PropertySurveyElement::isActive() const {
	return m_active;
}

bool PropertySurveyElement::isFirst() const {
	return m_first;
}

void PropertySurveyElement::setFirst(bool isFirst) {
	m_first = isFirst;
}


}
