/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SURVEYWRITER_H_
#define CORE_SURVEYWRITER_H_

#include <string>
#include <mutex>

namespace common {

class Date;

}

namespace core {

class SurveyElement;
class PropertySurveyElement;
class NumorSurveyElement;

/**
 * Class defining a survey writer.
 */
class SurveyWriter {

public:
	/**
	 * Gets the instance.
	 */
	static SurveyWriter* getInstance();
	
	/**
	 * Writes the element.
	 * Returns the formatted XML line.
	 */
	std::string write(SurveyElement* element, bool useLast);
	
	/**
	 * Returns true if is a new file.
	 */
	bool isNewFile();

private:
	SurveyWriter();
	~SurveyWriter();
	std::string createSurveyHeader(const common::Date& date) const;
	std::string format(SurveyElement* element, bool useLast) const;
	std::string format(PropertySurveyElement* element, bool useLast) const;
	std::string format(NumorSurveyElement* element, bool useLast) const;
	std::string formatData(const std::string& time, const std::string& ctype, const std::string& cname, const std::string& pname, const std::string& alias, long index, const std::string& pvalue) const;
	
	std::recursive_mutex m_mutex;

	static SurveyWriter* m_instance;
	
	const static std::string SURVEY_PATH;
};

}

#endif /* CORE_SURVEYWRITER_H_ */
