/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectCommandLauncher.h"
#include <command/transportlayer/command/CommandLauncherDirectImpl.h>
#include <core/transportlayer/core/CompletionHandlerDirectImpl.h>

using namespace common;
using namespace command;

namespace core {

DirectCommandLauncher::DirectCommandLauncher(command::CommandLauncherDirectImpl* proxy, CompletionHandlerDirectImpl* completionHandler) :
	_proxy(proxy),
	_completionHandler(completionHandler) {
}

DirectCommandLauncher::~DirectCommandLauncher() {
}

void DirectCommandLauncher::execute(unsigned long requestID, unsigned long databaseID, unsigned long commandID, bool skipIfWait) const {
	_proxy->execute(requestID, databaseID, commandID, skipIfWait, _completionHandler);
}

void DirectCommandLauncher::stop(unsigned long databaseID, unsigned long commandID) const {
	_proxy->stop(databaseID, commandID);
}

void DirectCommandLauncher::pause(unsigned long databaseID, unsigned long commandID) const {
	_proxy->pause(databaseID, commandID);
}
	
void DirectCommandLauncher::restart(unsigned long databaseID, unsigned long commandID) const {
	_proxy->restart(databaseID, commandID);
}

void DirectCommandLauncher::stopSetter(unsigned long databaseID, unsigned long propertyID) const {
	_proxy->stopSetter(databaseID, propertyID);
}

void DirectCommandLauncher::pauseSetter(unsigned long databaseID, unsigned long propertyID) const {
	_proxy->pauseSetter(databaseID, propertyID);
}

void DirectCommandLauncher::restartSetter(unsigned long databaseID, unsigned long propertyID) const {
	_proxy->restartSetter(databaseID, propertyID);
}

void DirectCommandLauncher::initDriversAndProtocols() const {
	_proxy->initDriversAndProtocols();
}

void DirectCommandLauncher::executeDurationRequest(unsigned long requestID, unsigned long databaseID, unsigned long commandID) const {
	_proxy->executeDurationRequest(requestID, databaseID, commandID, _completionHandler);
}

void DirectCommandLauncher::setInt8Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int8 value) const {
	_proxy->setInt8Value(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}
		
void DirectCommandLauncher::setUint16Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint16 value) const {
	_proxy->setUint16Value(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}

void DirectCommandLauncher::setUint32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint32 value) const {
	_proxy->setUint32Value(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}

void DirectCommandLauncher::setUint64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint64 value) const {
	_proxy->setUint64Value(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}

void DirectCommandLauncher::setInt16Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int16 value) const {
	_proxy->setInt16Value(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}
		
void DirectCommandLauncher::setInt32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int32 value) const {
	_proxy->setInt32Value(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}
		
void DirectCommandLauncher::setInt64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int64 value) const {
	_proxy->setInt64Value(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}
		
void DirectCommandLauncher::setFloat32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, float32 value) const {
	_proxy->setFloat32Value(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}
	
void DirectCommandLauncher::setFloat64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, float64 value) const {
	_proxy->setFloat64Value(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}
	
void DirectCommandLauncher::setStringValue(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, const std::string& value) const {
	
	_proxy->setStringValue(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}
	
void DirectCommandLauncher::setBooleanValue(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, bool value) const {
	_proxy->setBooleanValue(requestID, databaseID, servantID, propertyID, value, _completionHandler);
}

}
