/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_DIRECTPROCESSORPROXY_H_
#define CORE_DIRECTPROCESSORPROXY_H_

#include "CommandLauncher.h"

namespace command {

class CommandLauncherDirectImpl;

}

namespace core {

class CompletionHandlerDirectImpl;

/**
 * Direct implementation of the command launcher class.
 */
class DirectCommandLauncher : public CommandLauncher {
	
public:
	/**
	 * Constructor.
	 */
	DirectCommandLauncher(command::CommandLauncherDirectImpl* proxy, CompletionHandlerDirectImpl* completionHandler);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DirectCommandLauncher();
	
	/**
	 * Executes the command with ID commandID.
	 */
	virtual void execute(unsigned long requestID, unsigned long databaseID, unsigned long commandID, bool skipIfWait) const;
	
	/**
	 * Stops the command with ID commandID.
	 */
	virtual void stop(unsigned long databaseID, unsigned long commandID) const;
	
	/**
	 * Pauses the command with ID commandID.
	 */
	virtual void pause(unsigned long databaseID, unsigned long commandID) const;
	
	/**
	 * Restarts the command with ID commandID.
	 */
	virtual void restart(unsigned long databaseID, unsigned long commandID) const;
	
	/**
	 * Stops the setter with ID propertyID.
	 */
	virtual void stopSetter(unsigned long databaseID, unsigned long propertyID) const;
	
	/**
	 * Pauses the setter with ID propertyID.
	 */
	virtual void pauseSetter(unsigned long databaseID, unsigned long propertyID) const;
	
	/**
	 * Restarts the setter with ID propertyID.
	 */
	virtual void restartSetter(unsigned long databaseID, unsigned long propertyID) const;
	
	/**
	 * Initialises the drivers and the protocols
	 */
	virtual void initDriversAndProtocols() const;
	
	/**
	 * Executes the duration request.
	 */
	virtual void executeDurationRequest(unsigned long requestID, unsigned long databaseID, unsigned long commandID) const;
	
	/**
	 * Sets the int8 value for the property of ID propertyID.
	 */
	virtual void setInt8Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int8 value) const;

	/**
	 * Sets the int16 value for the property of ID propertyID.
	 */
	virtual void setUint16Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint16 value) const;

	/**
	 * Sets the int32 value for the property of ID propertyID.
	 */
	virtual void setUint32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint32 value) const;

	/**
	 * Sets the int64 value for the property of ID propertyID.
	 */
	virtual void setUint64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, uint64 value) const;

	/**
	 * Sets the int16 value for the property of ID propertyID.
	 */
	virtual void setInt16Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int16 value) const;
		
	/**
	 * Sets the int32 value for the property of ID propertyID.
	 */
	virtual void setInt32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int32 value) const;
		
	/**
	 * Sets the int64 value for the property of ID propertyID.
	 */
	virtual void setInt64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, int64 value) const;
		
	/**
	 * Sets the float32 value for the property of ID propertyID.
	 */
	virtual void setFloat32Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, float32 value) const;
	
	/**
	 * Sets the float64 value for the property of ID propertyID.
	 */
	virtual void setFloat64Value(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, float64 value) const;
	
	/**
	 * Sets the string value for the property of ID propertyID.
	 */
	virtual void setStringValue(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, const std::string& value) const;
	
	/**
	 * Sets the bool value for the property of ID propertyID.
	 */
	virtual void setBooleanValue(unsigned long requestID, unsigned long databaseID, unsigned long servantID, unsigned long propertyID, bool value) const;
	
private:
	command::CommandLauncherDirectImpl* _proxy;
	CompletionHandlerDirectImpl* _completionHandler;
};

}

#endif /*CORE_DIRECTPROCESSORPROXY_H_*/
