/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DataChangeSubscriberDirectImpl.h"
#include <core/transportlayer/dataprovider/DirectDataChangePublisher.h>
#include <core/dataaccess/DataNotificationRouter.h>
#include <common/output/OutputStream.h>

using namespace common;
using namespace std;

namespace core {

DataChangeSubscriberDirectImpl::DataChangeSubscriberDirectImpl(DataChangePublisher* dataChangePublisher) :
	_ID(0),
	_dataChangePublisher(dataChangePublisher) {
}

DataChangeSubscriberDirectImpl::~DataChangeSubscriberDirectImpl() {
}

void DataChangeSubscriberDirectImpl::setID(unsigned long subscriberID) {
	_ID = subscriberID;
}

unsigned long DataChangeSubscriberDirectImpl::getID() {
	return _ID;
}

void DataChangeSubscriberDirectImpl::subscribe() {
	_ID = static_cast<DirectDataChangePublisher*>(_dataChangePublisher)->subscribe(this);
}

void DataChangeSubscriberDirectImpl::unsubscribe() {
	_dataChangePublisher->unsubscribe(_ID);
}

void DataChangeSubscriberDirectImpl::propertyChanged(unsigned long databaseID, unsigned long propertyID) {
	DataNotificationRouter::getInstance()->propertyChanged(databaseID, propertyID);
}

void DataChangeSubscriberDirectImpl::commandStateChanged(unsigned long databaseID, unsigned long servantID, unsigned long commandID, CommandState commandState) {
	DataNotificationRouter::getInstance()->commandStateChanged(databaseID, commandID, commandState);
}

void DataChangeSubscriberDirectImpl::commandInterruptChanged(unsigned long databaseID, unsigned long servantID, unsigned long commandID, bool value) {
	// does nothing.
}

void DataChangeSubscriberDirectImpl::commandProgressionChanged(unsigned long databaseID, unsigned long dataID, double progression) {
	DataNotificationRouter::getInstance()->commandProgressChanged(databaseID, dataID, progression);
}

void DataChangeSubscriberDirectImpl::configurationChanged(unsigned long databaseID, unsigned long servantID, common::ConfigurationState enable) {
	DataNotificationRouter::getInstance()->configurationChanged(databaseID, servantID, enable);
}

void DataChangeSubscriberDirectImpl::conditionStateChanged(unsigned long conditionId, bool on, common::ConditionState state) {
	// ...
}

void DataChangeSubscriberDirectImpl::conditionActionsStateChanged(unsigned long conditionId, common::ConditionState state) {
	// ...
}

}
