/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef ANALYSISRESPONDER_H
#define ANALYSISRESPONDER_H

#include <cameo/api/cameo.h>
#include <thread>

namespace analysis {
	class DataToFitMessage;
}

namespace core {

/**
 * Server implementation of the analysis responder class.
 */
class AnalysisResponder {

public:
	/**
	 * Constructor.
	 */
	AnalysisResponder();

	/**
	 * Virtual destructor.
	 */
	virtual ~AnalysisResponder();

	/**
	 * Initialises the responder by starting the thread waiting for the requests.
	 */
	void init();

	/**
	 * Stops the responder and waits for the termination of the thread.
	 */
	void stop();

private:
	void loop();
	void initProcessFunctions();

	void processGetFit(const std::string & message, std::string & response);

	std::unique_ptr<std::thread> m_thread;
	std::unique_ptr<cameo::coms::basic::Responder> m_responder;

	typedef void (AnalysisResponder:: *ProcessFunctionType)(const std::string &, std::string &);
	std::map<int, ProcessFunctionType> m_processFunctions;

	void simpleGaussianFit(const analysis::DataToFitMessage& data, std::string & response);
	void multiGaussianFit(const analysis::DataToFitMessage& data, std::string & response);
	void centerOfMassFit(const analysis::DataToFitMessage& data, std::string & response);
};

}
#endif // ANALYSISRESPONDER_H
