/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMPLETIONHANDLERDIRECTIMPL_H_
#define CORE_COMPLETIONHANDLERDIRECTIMPL_H_

#include <string>

namespace core {

/**
 * Direct server implementation of the completion handler class.
 */
class CompletionHandlerDirectImpl {
	
public:
	/**
	 * Constructor.
	 */
	CompletionHandlerDirectImpl();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CompletionHandlerDirectImpl();
	
	/**
	 * The command with ID commandID terminates successfully. Posts a message to the
	 * scheduler message queue.
	 */
	virtual void commandSuccess(unsigned long requestID, unsigned long commandID);
	
	/**
	 * The command with ID commandID fails with message message. Posts a message to the
	 * scheduler message queue.
	 */
	virtual void commandFailure(unsigned long requestID, unsigned long commandID, const std::string& message);
	
	/**
	 * The Property with ID propertyID is set successfully. Posts a message to the
	 * scheduler message queue.
	 */
	virtual void propertySuccess(unsigned long requestID, unsigned long propertyID);
	
	/**
	 * The Property with ID propertyID is not set successfully. Posts a message to the
	 * scheduler message queue.
	 */
	virtual void propertyFailure(unsigned long requestID, unsigned long propertyID, const std::string& message);
	
	/**
	 * The Duration request with command ID commandID terminates successfully. Posts a message to the
	 * scheduler message queue.
	 */
	virtual void durationRequestSuccess(unsigned long requestID, unsigned long commandID, double duration);
	
	/**
	 * The Duration request with command ID commandID fails with message message. Posts a message to the
	 * scheduler message queue.
	 */
	virtual void durationRequestFailure(unsigned long requestID, unsigned long commandID, const std::string& message);
	
};

}

#endif /*CORE_COMPLETIONHANDLERDIRECTIMPL_H_*/