/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_DATABASERESPONDER_H_
#define CORE_DATABASERESPONDER_H_

#include <common/base/BaseTypes.h>
#include <cameo/api/cameo.h>
#include <thread>

namespace dataprovider {

class PropertyAccessorDirectImpl;
class CommandAccessorDirectImpl;

}

namespace core {

/**
 * Server implementation of the database responder class.
 */
class DatabaseResponder {

public:
	/**
	 * Constructor.
	 */
	DatabaseResponder(dataprovider::PropertyAccessorDirectImpl* propertyAccessor, dataprovider::CommandAccessorDirectImpl* commandAccessor);

	/**
	 * Virtual destructor.
	 */
	virtual ~DatabaseResponder();

	/**
	 * Initialises the responder by starting the thread waiting for the requests.
	 */
	void init();

	/**
	 * Stops the responder and waits for the termination of the thread.
	 */
	void stop();

private:
	void loop();
	void initProcessFunctions();

	bool processGetUint16Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetUint32Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetUint64Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetInt16Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetInt32Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetInt64Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetFloat32Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetFloat64Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetStringValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetBooleanValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processSetUint16Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetUint32Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetUint64Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetInt16Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetInt32Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetInt64Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetFloat32Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetFloat64Value(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetStringValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetBooleanValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetArraySize(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetUint16Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetUint32Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetUint64Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetInt16Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetInt32Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetInt64Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetFloat32Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetFloat64Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processSetInt16Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetInt32Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetInt64Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetUint16Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetUint32Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetUint64Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetFloat32Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetFloat64Array(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetCommandState(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetCommandProgression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processExecuteCommand(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processStopCommand(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetWholeScannableProperties(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetFormattedValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetPropertyIDRequest(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetWholeDynamicProperties(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetProperyTypeRequest(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetDynamicPropertyIdsRequest(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetServantIDRequest(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	dataprovider::PropertyAccessorDirectImpl* m_propertyAccessor;
	dataprovider::CommandAccessorDirectImpl* m_commandAccessor;

	std::unique_ptr<std::thread> m_thread;
	std::unique_ptr<cameo::coms::basic::Responder> m_responder;

	typedef bool (DatabaseResponder:: *ProcessFunctionType)(cameo::coms::basic::Request *, const std::string &, std::string &);

	struct ProcessFunction {
		ProcessFunctionType functionPtr;
		bool activity;

		ProcessFunction()
			: functionPtr(nullptr), activity(true) {}

		ProcessFunction(ProcessFunctionType functionPtrValue, bool activityValue)
			: functionPtr(functionPtrValue), activity(activityValue) {}
	};

	std::map<int, ProcessFunction> m_processFunctions;
};

}

#endif

