/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SERVANTMANAGERRESPONDER_H_
#define CORE_SERVANTMANAGERRESPONDER_H_

#include <common/base/BaseTypes.h>
#include <cameo/api/cameo.h>
#include <thread>

namespace dataprovider {

class ServantAccessorDirectImpl;

}

namespace core {

/**
 * Server implementation of the servant manager responder class.
 */
class ServantManagerResponder {

public:
	/**
	 * Constructor.
	 */
	ServantManagerResponder(dataprovider::ServantAccessorDirectImpl* servantAccessor);

	/**
	 * Virtual destructor.
	 */
	virtual ~ServantManagerResponder();

	/**
	 * Initialises the responder by starting the thread waiting for the requests.
	 */
	void init();

	/**
	 * Stops the responder and waits for the termination of the thread.
	 */
	void stop();

private:
	void loop();
	void initProcessFunctions();

	bool processGetWholeConfiguration(const std::string & message, std::string & response);
	bool processGetWholeConfigurationUpdate(const std::string & message, std::string & response);

	bool processGetControllerConfiguration(const std::string & message, std::string & response);
	bool processGetServantSpyProperties(const std::string & message, std::string & response);
	bool processGetServantDynamicProperties(const std::string & message, std::string & response);
	bool processGetServantSpyDynamicProperties(const std::string & message, std::string & response);

	bool processCloneController(const std::string & message, std::string & response);
	bool processRemoveController(const std::string & message, std::string & response);

	bool processGetDriverChannel(const std::string & message, std::string & response);
	bool processReconnectDriver(const std::string & message, std::string & response);
	bool processRemoveDriver(const std::string & message, std::string & response);

	bool processPauseSpies(const std::string & message, std::string & response);

	bool processGetPropertiesForGeneric(const std::string & message, std::string & response);
	bool processGetWebSpyProperties(const std::string & message, std::string & response);
	bool processGetIcsExperimentControllersConfiguration(const std::string & message, std::string & response);
	bool processGetIcsTestControllersConfiguration(const std::string & message, std::string & response);

	bool processGetResourceFilesByExtension(const std::string & message, std::string & response);
	bool processReadBinaryResourceFile(const std::string & message, std::string & response);
	bool processReadResourceFile(const std::string & message, std::string & response);
	bool processWriteResourceFile(const std::string & message, std::string & response);
	bool processGetResourceFileLastTime(const std::string & message, std::string & response);

	bool processGetServantPropertiesValue(const std::string & message, std::string & response);
	bool processGetServantCommandsState(const std::string & message, std::string & response);

	bool processGetFamilyImageKeys(const std::string & message, std::string & response);
	bool processGetControllerIconKeys(const std::string & message, std::string & response);

	bool processUserLog(const std::string & message, std::string & response);

	bool processGetControllerTypesInheritingType(const std::string & message, std::string & response);

	bool processHasRunningCommands(const std::string & message, std::string & response);

	bool processGetSpyPeriod(const std::string & message, std::string & response);

	bool processGetPropertiesSavedInBox(const std::string & message, std::string & response);

	bool processGetPropertySavedInBoxValue(const std::string & message, std::string & response);
	bool processSetPropertySavedInBoxValue(const std::string & message, std::string & response);


	dataprovider::ServantAccessorDirectImpl* m_servantAccessor;

	std::unique_ptr<std::thread> m_thread;
	std::unique_ptr<cameo::coms::basic::Responder> m_responder;

	typedef bool (ServantManagerResponder:: *ProcessFunctionType)(const std::string &, std::string &);

	struct ProcessFunction {
		ProcessFunctionType functionPtr;
		bool activity;

		ProcessFunction()
			: functionPtr(nullptr), activity(true) {}

		ProcessFunction(ProcessFunctionType functionPtrValue, bool activityValue)
			: functionPtr(functionPtrValue), activity(activityValue) {}
	};

	std::map<int, ProcessFunction> m_processFunctions;
};

}

#endif

