/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_SESSIONRESPONDER_H_
#define CORE_SESSIONRESPONDER_H_

#include <core/session/SessionManager.h>
#include <thread>
#include <memory>
#include <vector>

namespace core {

/**
 * Server implementation of the session responder class.
 */
class SessionResponder {

public:
	/**
	 * Constructor.
	 */
	SessionResponder();

	/**
	 * Virtual destructor.
	 */
	virtual ~SessionResponder();

	/**
	 * Initialises the responder by starting the thread waiting for the requests.
	 */
	void init();

	/**
	 * Stops the responder and waits for the termination of the thread.
	 */
	void stop();

private:
	void loop();
	void initProcessFunctions();

	void processLogin(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	void processLoginRefused(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	void processEnableRemoteControlRequest(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	void processDisableRemoteControlRequest(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	void processGetTokenRequest(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	void processReleaseTokenRequest(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	void processSync(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	void processChatMessage(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	void processCrashMailRequest(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	void processConnectedUsersRequest(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	typedef void (SessionResponder:: *ProcessFunctionType)(cameo::coms::basic::Request *, const std::string &, std::string &);

	std::unique_ptr<std::thread> m_thread;
	std::unique_ptr<cameo::coms::basic::Responder> m_responder;
	std::map<int, ProcessFunctionType> m_processFunctions;
};

}

#endif

