/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_VARIABLEMANAGERRESPONDER_H_
#define CORE_VARIABLEMANAGERRESPONDER_H_

#include <common/base/BaseTypes.h>
#include <cameo/api/cameo.h>
#include <thread>

namespace core {

/**
 * Server implementation of the variable manager responder class.
 */
class UserVariableManagerResponder {

public:
	/**
	 * Constructor.
	 */
	UserVariableManagerResponder();

	/**
	 * Virtual destructor.
	 */
	virtual ~UserVariableManagerResponder();

	/**
	 * Initialises the responder by starting the thread waiting for the requests.
	 */
	void init();

	/**
	 * Stops the responder and waits for the termination of the thread.
	 */
	void stop();

private:
	void loop();
	void initProcessFunctions();

	bool processVerifyExpression(const std::string & message, std::string & response);
	bool processAddVariable(const std::string & message, std::string & response);
	bool processEditVariable(const std::string & message, std::string & response);
	bool processRemoveVariable(const std::string & message, std::string & response);
	bool processGetVariableValue(const std::string & message, std::string & response);
	bool processGetVariablesXMLContent(const std::string & message, std::string & response);
	bool processGetUsableVariableListValue(const std::string & message, std::string & response);

	std::unique_ptr<std::thread> m_thread;
	std::unique_ptr<cameo::coms::basic::Responder> m_responder;

	typedef bool (UserVariableManagerResponder:: *ProcessFunctionType)(const std::string &, std::string &);

	struct ProcessFunction {
		ProcessFunctionType functionPtr;
		bool activity;

		ProcessFunction()
			: functionPtr(nullptr), activity(true) {}

		ProcessFunction(ProcessFunctionType functionPtrValue, bool activityValue)
			: functionPtr(functionPtrValue), activity(activityValue) {}
	};

	std::map<int, ProcessFunction> m_processFunctions;
};

}

#endif

