/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDZONEEVENTPUBLISHER_H_
#define CORE_COMMANDZONEEVENTPUBLISHER_H_

#include <cameo/api/cameo.h>
#include <map>
#include <mutex>

namespace core {

class CommandZoneEventSubscriber;
class CommandZoneEvent;
class CommandBoxExecutionEvent;
class CommandBoxProgressionEvent;
class CommandBoxResetEvent;
class CommandBoxPropertyEvent;
class CommandZoneExecutionEvent;
class CommandZoneProgressionEvent;
class CommandZoneResetEvent;
class CommandZoneErrorEvent;
class ForLoopCommandBoxCurrentValueEvent;
class CommandBoxExpressionChangedEvent;
class CommandBoxExpressionValueChangedEvent;
class CommandBoxesDeletedEvent;
class CommandZoneSyncEvent;

/**
 * Class defining the command zone event publisher.
 */
class CommandZoneEventPublisher {
	
public:
	/**
	 * Constructor.
	 */
	CommandZoneEventPublisher();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CommandZoneEventPublisher();
	
	/**
	 * Adds a subscriber.
	 */
	unsigned long subscribeDirect(CommandZoneEventSubscriber* subscriber);
	
	/**
	 * Removes a subscriber.
	 */
	void unsubscribeDirect(unsigned long subscriberID);

	/**
	 * Notifies a command zone event.
	 */
	void notify(CommandZoneEvent* commandZoneEvent);

private:
	void notify(CommandBoxExecutionEvent* commandZoneEvent);
	void notify(CommandBoxProgressionEvent* commandZoneEvent);
	void notify(CommandBoxResetEvent* commandZoneEvent);
	void notify(CommandBoxPropertyEvent* commandZoneEvent);
	void notify(CommandZoneExecutionEvent* commandZoneEvent);
	void notify(CommandZoneProgressionEvent* commandZoneEvent);
	void notify(CommandZoneResetEvent* commandZoneEvent);
	void notify(CommandZoneErrorEvent* commandZoneEvent);
	void notify(ForLoopCommandBoxCurrentValueEvent* commandZoneEvent);
	void notify(CommandBoxExpressionChangedEvent* commandZoneEvent);
	void notify(CommandBoxExpressionValueChangedEvent* commandZoneEvent);
	void notify(CommandBoxesDeletedEvent* commandZoneEvent);
	void notify(CommandZoneSyncEvent* commandZoneEvent);

	static unsigned long _counter;
	std::map<unsigned long, CommandZoneEventSubscriber*> _subscribers;
			
	std::recursive_mutex _mutex;
	
	std::unique_ptr<cameo::coms::Publisher> m_publisher;
};

}

#endif
