/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_COMMANDZONERESPONDER_H_
#define CORE_COMMANDZONERESPONDER_H_

#include <common/base/BaseTypes.h>
#include <cameo/api/cameo.h>
#include <thread>

namespace core {

class CommandBox;
class PropertyCommandBox;
class CheckExpressionCommandBox;
class AtomicCommandBox;
class GenericCommandBox;
class PropertyGenericCommandBox;
class VariableCommandBox;
class IfCommandBox;
class ScanCommandBox;
class ForLoopCommandBox;
class ControlCommandBox;
class BreakCommandBox;

/**
 * Server implementation of the command box responder class.
 */
class CommandZoneResponder {

public:
	/**
	 * Constructor.
	 */
	CommandZoneResponder();

	/**
	 * Virtual destructor.
	 */
	virtual ~CommandZoneResponder();

	/**
	 * Initialises the responder by starting the thread waiting for the requests.
	 */
	void init();

	/**
	 * Stops the responder and waits for the termination of the thread.
	 */
	void stop();

private:
	void loop();
	void initProcessFunctions();

	/**
	 * Returns the command box associated with the commandBoxID.
	 */
	CommandBox* getCommandBox(unsigned long commandBoxID);
	PropertyCommandBox* getPropertyCommandBox(unsigned long commandBoxID);
	CheckExpressionCommandBox* getCheckExpressionCommandBox(unsigned long commandBoxID);
	AtomicCommandBox* getAtomicCommandBox(unsigned long commandBoxID);
	GenericCommandBox* getGenericCommandBox(unsigned long commandBoxID);
	PropertyGenericCommandBox* getPropertyGenericCommandBox(unsigned long commandBoxID);
	VariableCommandBox* getVariableCommandBox(unsigned long commandBoxID);

	IfCommandBox* getIfCommandBox(unsigned long commandBoxID);
	ScanCommandBox* getScanCommandBox(unsigned long commandBoxID);
	ForLoopCommandBox* getForLoopCommandBox(unsigned long commandBoxID);
	ControlCommandBox* getControlCommandBox(unsigned long commandBoxID);
	BreakCommandBox* getBreakCommandBox(unsigned long commandBoxID);

	bool processToggleBackground(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processToggleParallel(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processIsBackground(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processIsParallel(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetExecutionState(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetModifiable(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processIsModifiable(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processStopCommandBox(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetCommandBoxProgression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetCommandID(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processIsSettings(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetDatabaseID(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetSettingsFileName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetStringValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetFormattedValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processSetSettingsFileName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processSetControllerName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetPropertyName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processSetUserVariableName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetUserVariableExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetUserVariableName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetUserVariableExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetControllerName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetPropertyName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetBoxType(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetCommandList(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processAddNewAtomicCommandBoxAtEnd(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAddNewAtomicCommandBox(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAddNewScanCommandBoxAtEnd(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAddNewScanCommandBox(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAddNewForLoopCommandBoxAtEnd(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAddNewForLoopCommandBox(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAddNewGenericCommandBoxAtEnd(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAddNewGenericCommandBox(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAddNewControlCommandBoxAtEnd(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAddNewControlCommandBox(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processMoveCommandBox(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processMoveCommandBoxToEnd(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processDeleteCommandBox(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processCheckExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processCheckStaticExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetScanName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetNumberOfLines(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetVariableName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetType(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetStepType(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetValues(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetStartValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetEndValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetStepSize(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetVariableName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetStartValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetEndValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetStepSize(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetCurrentValue(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetType(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetStepType(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetValues(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAddForLoopLine(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processRemoveForLoopLine(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetScanCommandBoxUserVariableNames(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processCreate(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processReset(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processExecute(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processStop(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processStopAtEnd(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processPause(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processPauseAtEnd(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processRestart(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processIsRunning(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processIsPaused(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processIsPausingAtEnd(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processIsStoppingAtEnd(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetCommandZoneExecutionState(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetProgression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetConflictingCommands(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetEstimatedTime(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processPrint(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processCheckPropertyExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processCheckStaticPropertyExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSetPropertyExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetPropertyExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetVariableList(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processSaveFile(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processSaveBinaryFile(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetFileContent(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetBinaryFileContent(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processGetFileContentAbsolutePath(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetTree(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processDeleteFile(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processCreateDirectory(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processAppendToFile(cameo::coms::basic::Request * request, const std::string & message, std::string & response);

	bool processCloneForEstimatedTime(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processComputeEstimatedTimeState(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processFileExists(cameo::coms::basic::Request * request, const std::string & message, std::string & response);


	bool processGetLastControllerName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processGetLastSetPropertyName(cameo::coms::basic::Request * request, const std::string & message, std::string & response);
	bool processInitPropertyExpression(cameo::coms::basic::Request * request, const std::string & message, std::string & response);


	std::unique_ptr<std::thread> m_thread;
	std::unique_ptr<cameo::coms::basic::Responder> m_responder;

	typedef bool (CommandZoneResponder:: *ProcessFunctionType)(cameo::coms::basic::Request *, const std::string &, std::string &);

	struct ProcessFunction {
		ProcessFunctionType functionPtr;
		bool notification;
		bool activity;

		ProcessFunction()
			: functionPtr(nullptr), notification(true), activity(false) {}

		ProcessFunction(ProcessFunctionType functionPtrValue, bool notificationValue, bool activityValue)
			: functionPtr(functionPtrValue), notification(notificationValue), activity(activityValue) {}
	};

	std::map<int, ProcessFunction> m_processFunctions;
};

}

#endif
