/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectCommandAccessor.h"
#include <command/transportlayer/common/CommandStateChangeSubscriberDirectImpl.h>
#include <dataprovider/transportlayer/dataprovider/CommandAccessorDirectImpl.h>
#include <common/output/OutputStream.h>
#include <iostream>
#include <vector>

using namespace std;
using namespace common;
using namespace dataprovider;

namespace core {

DirectCommandAccessor::DirectCommandAccessor(dataprovider::CommandAccessorDirectImpl* proxy)
: _proxy(proxy) {
}

DirectCommandAccessor::~DirectCommandAccessor() {
}

unsigned long DirectCommandAccessor::getControllerCommandID(unsigned long databaseID, unsigned long controllerID, const std::string& commandName) {

	bool found = false;
	unsigned long foundID = 0;

	vector<CommandDescriptor> commands;
	try {
		commands = _proxy->getControllerCommands(databaseID, controllerID);

	} catch (CommandAccessorDirectImpl::NoSuchServantException) {
		throw NoSuchServantException(controllerID);
	}

	// iterate over the list and create a new ServantDescriptor for each item
	vector<CommandDescriptor>::const_iterator listIterator = commands.begin();
	while (listIterator != commands.end()) {

		common::CommandDescriptor descriptor = *listIterator;

		if (descriptor.getName() == commandName) {
			found = true;
			foundID = descriptor.getID();
		}

	}

	if (!found) {
		throw NoSuchCommandException(controllerID, commandName);
	}

	return foundID;
}

unsigned long DirectCommandAccessor::getDriverCommandID(unsigned long databaseID, unsigned long driverID, const std::string& commandName) {

	bool found = false;
	unsigned long foundID = 0;

	vector<CommandDescriptor> commands;
	try {
		commands = _proxy->getDriverCommands(databaseID, driverID);

	} catch (CommandAccessorDirectImpl::NoSuchServantException) {
		throw NoSuchServantException(driverID);
	}

	// iterate over the list and create a new ServantDescriptor for each item
	vector<CommandDescriptor>::const_iterator listIterator = commands.begin();
	while (listIterator != commands.end()) {

		common::CommandDescriptor descriptor = *listIterator;

		if (descriptor.getName() == commandName) {
			found = true;
			foundID = descriptor.getID();
		}

	}

	if (!found) {
		throw NoSuchCommandException(driverID, commandName);
	}

	return foundID;
}


std::string DirectCommandAccessor::getCommandName(unsigned long databaseID, unsigned long commandID) {

	try {
		return _proxy->getCommandName(databaseID, commandID);

	} catch (CommandAccessorDirectImpl::NoSuchCommandException& e) {
		// problem occured on the server side
		throw NoSuchCommandException(commandID);
	}
}

void DirectCommandAccessor::setCommandState(unsigned long databaseID, unsigned long servantID, unsigned long commandID, common::CommandState state) {

	try {
		 _proxy->setCommandState(databaseID, servantID, commandID, state);

	} catch (CommandAccessorDirectImpl::NoSuchCommandException& e) {
		// problem occured on the server side
		throw NoSuchCommandException(commandID);
	}
}

common::CommandState DirectCommandAccessor::getCommandState(unsigned long databaseID, unsigned long commandID) const {

	try {
		return _proxy->getCommandState(databaseID, commandID);

	} catch (CommandAccessorDirectImpl::NoSuchCommandException& e) {
		// problem occured on the server side
		throw NoSuchCommandException(commandID);
	}
}

void DirectCommandAccessor::setCommandFromCommandZone(unsigned long databaseID, unsigned long commandID, bool fromCommandZone) {

	try {
		_proxy->setCommandFromCommandZone(databaseID, commandID, fromCommandZone);

	} catch (CommandAccessorDirectImpl::NoSuchCommandException& e) {
		// problem occured on the server side
		throw NoSuchCommandException(commandID);
	}
}

unsigned long DirectCommandAccessor::getServantIDFromCommandID(unsigned long databaseID, unsigned long commandID) const {

	try {
		return _proxy->getServantIDFromCommandID(databaseID, commandID);

	} catch (CommandAccessorDirectImpl::NoSuchCommandException& e) {
		// problem occured on the server side
		throw NoSuchCommandException(commandID);
	}

}


void DirectCommandAccessor::setCommandLogID(unsigned long databaseID, unsigned long commandID, const std::string& logID) {

	try {
		_proxy->setCommandLogID(databaseID, commandID, logID);

	} catch (CommandAccessorDirectImpl::NoSuchCommandException& e) {
		// problem occured on the server side
		throw NoSuchCommandException(commandID);
	}
}

bool DirectCommandAccessor::commandExists(unsigned long databaseID, unsigned long commandID) const {

	return _proxy->commandExists(databaseID, commandID);
}

unsigned long DirectCommandAccessor::getSisterCommandID(unsigned long databaseID, unsigned long commandID, const std::string& commandName) const {

	try {
		return _proxy->getSisterCommandID(databaseID, commandID, commandName);

	} catch (CommandAccessorDirectImpl::NoSuchCommandException& e) {
		// problem occured on the server side
		throw NoSuchCommandException(commandID);
	}
}

bool DirectCommandAccessor::isCommandFromIcsServant(unsigned long databaseID, unsigned long commandID) const {

	try {
		return _proxy->isCommandFromIcsServant(databaseID, commandID);

	} catch (CommandAccessorDirectImpl::NoSuchCommandException& e) {
		// problem occured on the server side
		throw NoSuchCommandException(commandID);
	}
}

}
