/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_DIRECTCOMMANDACCESSOR_H_
#define CORE_DIRECTCOMMANDACCESSOR_H_

#include "CommandAccessor.h"

namespace dataprovider {

class CommandAccessorDirectImpl;

}

namespace core {

/**
 * Class defining a direct Command Accessor.
 */
class DirectCommandAccessor : public CommandAccessor {

public:

	/**
	 * Constructor.
	 */
	DirectCommandAccessor(dataprovider::CommandAccessorDirectImpl* proxy);

	/**
	 * Virtual destructor.
	 */
	virtual ~DirectCommandAccessor();

	/**
	 * Returns the ID of the command name of given controller.
	 * throw NoSuchServantException, NoSuchCommandException
	 */
	virtual unsigned long getControllerCommandID(unsigned long databaseID, unsigned long controllerID, const std::string& propertyName);

	/**
	 * Returns the ID of the command name of given driver.
	 * throw NoSuchServantException, NoSuchCommandException
	 */
	virtual unsigned long getDriverCommandID(unsigned long databaseID, unsigned long driverID, const std::string& propertyName);

	/**
	 * Returns the command name associated with the command ID.
	 * throw NoSuchCommandException
	 */
	virtual std::string getCommandName(unsigned long databaseID, unsigned long commandID);

	/**
	 * Sets the state of the command
	 * throw NoSuchCommandException
	 */
	virtual void setCommandState(unsigned long databaseID, unsigned long servantID, unsigned long commandID, common::CommandState state);

	/**
	 * Sets the command from the command zone.
	 * throw NoSuchCommandException
	 */
	virtual void setCommandFromCommandZone(unsigned long databaseID, unsigned long commandID, bool fromCommandZone);

	/**
	 * Returns the state of the command
	 * throw NoSuchCommandException
	 */
	virtual common::CommandState getCommandState(unsigned long databaseID, unsigned long commandID) const;

	/**
	 * Returns the ID of the servant that contains the command ID.
	 * throw NoSuchCommandException
	 */
	virtual unsigned long getServantIDFromCommandID(unsigned long databaseID, unsigned long commandID) const;

	/**
	 * Sets the log ID of the command.
	 * throw NoSuchCommandException
	 */
	virtual void setCommandLogID(unsigned long databaseID, unsigned long commandID, const std::string& logID);

	/**
	 * Returns true if is a command.
	 */
	virtual bool commandExists(unsigned long databaseID, unsigned long commandID) const;

	/**
	 * Gets the sister command ID.
	 * throw NoSuchCommandException
	 */
	virtual unsigned long getSisterCommandID(unsigned long databaseID, unsigned long commandID, const std::string& commandName) const;

	/**
	 * Returns true if the command belongs to a ics servant
	 * throw NoSuchCommandException
	 */
	virtual bool isCommandFromIcsServant(unsigned long databaseID, unsigned long commandID) const;

private:
	dataprovider::CommandAccessorDirectImpl* _proxy;
};

}

#endif /*CORE_DIRECTCOMMANDACCESSOR_H_*/
