/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_DIRECTLOGACCESSOR_H_
#define CORE_DIRECTLOGACCESSOR_H_

#include "LogAccessor.h"

namespace dataprovider {

	class LogAccessorDirectImpl;
}

namespace core {

/**
 * Direct implementation class of the log accessor.
 */
class DirectLogAccessor : public LogAccessor {
	
public:
	/**
	 * Constructor
	 */
	DirectLogAccessor(dataprovider::LogAccessorDirectImpl* proxy);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DirectLogAccessor();

	/**
	 * Write the Command log entry
	 */
	virtual void writeCommandLog(common::LogType type, common::LogLevel level, const std::string& logMessage, const std::string& logID);

	/**
	 * Write the Simple log entry
	 */
	virtual void writeSimpleLog(common::LogLevel level, const std::string& logMessage, bool writeLogFile);
		
	/**
	 * Write the Command Zone Content log entry
	 */
	virtual void writeCommandZoneContentLog(const std::string& content);

	/**
	 * Write the Command Zone Event log entry
	 */
	virtual void writeCommandZoneEventLog(const std::string& content);

	/**
	 * Write a Variable Change log entry
	 */
	virtual void writeVariableChangeLog(common::LogLevel level, const std::string& variableName, const std::string& variableType, const std::string& variableValue, const std::string& message, const unsigned long messageParentId);

	/**
	 * Write a For Loop Start log entry
	 */
	virtual void writeForLoopStartLog(common::LogLevel level, const std::string& variablesField, const std::string& message, const unsigned long messageId);

	/**
	 * Write a For Loop End log entry
	 */
	virtual void writeForLoopEndLog(common::LogLevel level, const std::string& message, const unsigned long messageId);

	/**
	 * Write an If log entry
	 */
	virtual void writeIfEvaluationLog(common::LogLevel level, const std::string& name, const std::string& type, const std::string& value, const std::string& message);

	/**
	 * Write an Break Evaluation log entry
	 */
	virtual void writeBreakEvaluationLog(common::LogLevel level, const std::string& name, const std::string& type, const std::string& value, const std::string& message, const unsigned long messageParentId);

	/**
	 * Write an User's Scan Start log entry
	 */
	virtual void writeUserScanStartLog(common::LogLevel level, const std::string& name, const std::string& message);


private:
	dataprovider::LogAccessorDirectImpl* _proxy;
};

}

#endif /*CORE_DIRECTLOGACCESSOR_H_*/
