/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectPropertyAccessor.h"
#include <dataprovider/transportlayer/dataprovider/PropertyAccessorDirectImpl.h>
#include <common/output/OutputStream.h>
#include <vector>

using namespace dataprovider;
using namespace std;
using namespace common;

namespace core {

DirectPropertyAccessor::DirectPropertyAccessor(dataprovider::PropertyAccessorDirectImpl* proxy)
: _proxy(proxy) {
}

DirectPropertyAccessor::~DirectPropertyAccessor() {
}

unsigned long DirectPropertyAccessor::getPropertyID(unsigned long databaseID, unsigned long servantID, const std::string& propertyName) {

	try {
		return _proxy->getPropertyID(databaseID, servantID, propertyName);

	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException) {
		throw NoSuchPropertyException();
	}
}

	
unsigned long DirectPropertyAccessor::getDynamicPropertyIDAtIndex(unsigned long databaseID, unsigned long propertyID, long index, bool resize) const {
	
	try {
		return _proxy->getDynamicPropertyIDAtIndex(databaseID, propertyID, index, resize);
				
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::IndexOutOfBoundsException & e) {
		throw IndexOutOfBoundsException(propertyID);
	}
}

unsigned long DirectPropertyAccessor::getServantIDFromPropertyID(unsigned long databaseID, unsigned long propertyID) const {

	try {
		return _proxy->getServantIDFromPropertyID(databaseID, propertyID);

	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

std::string DirectPropertyAccessor::getPropertyType(unsigned long databaseID, unsigned long propertyID) {

	try {
		return _proxy->getPropertyType(databaseID, propertyID);

	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

bool DirectPropertyAccessor::propertyExists(unsigned long databaseID, unsigned long propertyID) const {
	
	return _proxy->propertyExists(databaseID, propertyID);
}

int8 DirectPropertyAccessor::getInt8Value(unsigned long databaseID, unsigned long propertyID) const {

	try {
		return _proxy->getInt8Value(databaseID, propertyID);
		
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}
	
int16 DirectPropertyAccessor::getInt16Value(unsigned long databaseID, unsigned long propertyID) const {

	try {
		return _proxy->getInt16Value(databaseID, propertyID);
		
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

int32 DirectPropertyAccessor::getInt32Value(unsigned long databaseID, unsigned long propertyID) const {

	try {
		return _proxy->getInt32Value(databaseID, propertyID);
		
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

int64 DirectPropertyAccessor::getInt64Value(unsigned long databaseID, unsigned long propertyID) const {

	try {
		return _proxy->getInt64Value(databaseID, propertyID);
	
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

float32 DirectPropertyAccessor::getFloat32Value(unsigned long databaseID, unsigned long propertyID) const {
	
	try {
		return _proxy->getFloat32Value(databaseID, propertyID);
	
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

float64 DirectPropertyAccessor::getFloat64Value(unsigned long databaseID, unsigned long propertyID) const {
	
	try {
		return _proxy->getFloat64Value(databaseID, propertyID);
	
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

std::string DirectPropertyAccessor::getStringValue(unsigned long databaseID, unsigned long propertyID) const {

	try {
		return _proxy->getStringValue(databaseID, propertyID);
	
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}	
}

bool DirectPropertyAccessor::getBooleanValue(unsigned long databaseID, unsigned long propertyID) const {

	try {
		return _proxy->getBooleanValue(databaseID, propertyID);
	
	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	} catch (PropertyAccessorDirectImpl::BadPropertyTypeException & e) {
		throw BadPropertyTypeException(propertyID);
	}
}

std::string DirectPropertyAccessor::getFormattedValue(unsigned long databaseID, unsigned long propertyID) const {

	try {
		return _proxy->getFormattedValue(databaseID, propertyID);

	} catch (PropertyAccessorDirectImpl::NullPropertyReferenceException & e) {
		throw NullPropertyReferenceException(propertyID);
	} catch (PropertyAccessorDirectImpl::NoSuchPropertyException & e) {
		throw NoSuchPropertyException(propertyID);
	}
}

}
