/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_DIRECTPROPERTYACCESSOR_H_
#define CORE_DIRECTPROPERTYACCESSOR_H_

#include "PropertyAccessor.h"

namespace dataprovider {

class PropertyAccessorDirectImpl;

}

namespace core {
	
/**
 * Class defining a direct property accessor.
 */
class DirectPropertyAccessor : public PropertyAccessor {
	
public:
	/**
	 * Constructor.
	 */
	DirectPropertyAccessor(dataprovider::PropertyAccessorDirectImpl* proxy);

	/**
	 * Virtual destructor.
	 */
	virtual ~DirectPropertyAccessor();

	/**
	 * Returns the ID of the property name of given servant.
	 * throw NoSuchServantException, NoSuchPropertyException
	 */
	virtual unsigned long getPropertyID(unsigned long databaseID, unsigned long servantID, const std::string& propertyName);
	
	/**
	 * Gets the dynamic property size.
	 * throw NoSuchPropertyException, IndexOutOfBoundsException
	 */
	virtual unsigned long getDynamicPropertyIDAtIndex(unsigned long databaseID, unsigned long propertyID, long index, bool resize) const;

	/**
	 * Returns the ID of the servant that contains the property ID.
	 */
	virtual unsigned long getServantIDFromPropertyID(unsigned long databaseID, unsigned long propertyID) const;
	
	/**
	 * Returns the type of property.
	 * throw NoSuchPropertyException
	 */
	virtual std::string getPropertyType(unsigned long databaseID, unsigned long propertyID);
	
	/**
	 * Returns true if is a property.
	 */
	virtual bool propertyExists(unsigned long databaseID, unsigned long propertyID) const;
		
	/**
	 * Gets the int8 value for the property of ID propertyID.
	 * throw NoSuchPropertyException, BadPropertyTypeException
	 */
	virtual int8 getInt8Value(unsigned long databaseID, unsigned long propertyID) const;
	
	/**
	 * Gets the int16 value for the property of ID propertyID.
	 * throw NoSuchPropertyException, BadPropertyTypeException
	 */
	virtual int16 getInt16Value(unsigned long databaseID, unsigned long propertyID) const;

	/**
	 * Gets the int32 value for the property of ID propertyID.
	 * throw NoSuchPropertyException, BadPropertyTypeException
	 */
	virtual int32 getInt32Value(unsigned long databaseID, unsigned long propertyID) const;

	/**
	 * Gets the int64 value for the property of ID propertyID.
	 * throw NoSuchPropertyException, BadPropertyTypeException
	 */
	virtual int64 getInt64Value(unsigned long databaseID, unsigned long propertyID) const;

	/**
	 * Gets the float32 value for the property of ID propertyID.
	 * throw NoSuchPropertyException, BadPropertyTypeException
	 */
	virtual float32 getFloat32Value(unsigned long databaseID, unsigned long propertyID) const;

	/**
	 * Gets the float64 value for the property of ID propertyID.
	 * throw NoSuchPropertyException, BadPropertyTypeException
	 */
	virtual float64 getFloat64Value(unsigned long databaseID, unsigned long propertyID) const;

	/**
	 * Gets the string value for the property of ID propertyID.
	 * throw NoSuchPropertyException, BadPropertyTypeException
	 */
	virtual std::string getStringValue(unsigned long databaseID, unsigned long propertyID) const;

	/**
	 * Gets the boolean value for the property of ID propertyID.
	 * throw NoSuchPropertyException, BadPropertyTypeException
	 */
	virtual bool getBooleanValue(unsigned long databaseID, unsigned long propertyID) const;
	
	/**
	 * Gets the formatted value for the property of ID propertyID.
	 * throw NoSuchPropertyException
	 */
	virtual std::string getFormattedValue(unsigned long databaseID, unsigned long propertyID) const;

private:
	dataprovider::PropertyAccessorDirectImpl* _proxy;
};
	
}

#endif /*CORE_DIRECTPROPERTYACCESSOR_H_*/
