/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectServantAccessor.h"
#include <dataprovider/transportlayer/dataprovider/ServantAccessorDirectImpl.h>
#include <iostream>
#include <common/utils/ChildServantDescriptor.h>
#include <common/utils/ServantDescriptor.h>

using namespace std;
using namespace common;
using namespace dataprovider;

namespace core {

DirectServantAccessor::DirectServantAccessor(dataprovider::ServantAccessorDirectImpl* proxy) :
	_proxy(proxy) {
}

DirectServantAccessor::~DirectServantAccessor() {
}

void DirectServantAccessor::init () {

	vector<common::ServantDescriptor> controllers = _proxy->getControllers(0);

	// iterate over the list
	vector<common::ServantDescriptor>::const_iterator controllerListIterator = controllers.begin();
	while (controllerListIterator != controllers.end()) {
		_IDs.insert(controllerListIterator->getName(), controllerListIterator->getID());
	}

	vector<common::ServantDescriptor> drivers = _proxy->getDrivers(0);

	// iterate over the list
	vector<common::ServantDescriptor>::const_iterator driverListIterator = drivers.begin();
	while (driverListIterator != drivers.end()) {
		_IDs.insert(driverListIterator->getName(), driverListIterator->getID());
	}
}

unsigned long DirectServantAccessor::getServantID(const std::string& servantName) const {

	try {
		unsigned long servantID = _proxy->getServantID(0, servantName);

		return servantID;

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantName);
	}
}

ServantAccessor::ServantType DirectServantAccessor::getServantType(unsigned long databaseID, unsigned long servantID) const {

	try {
		return (ServantAccessor::ServantType)_proxy->getServantType(databaseID, servantID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

std::string DirectServantAccessor::getServantName(unsigned long databaseID, unsigned long servantID) {
	try {
		return _proxy->getServantName(databaseID, servantID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

ServantAccessor::ServantType DirectServantAccessor::getServantType(unsigned long databaseID, const std::string& servantName) const {

	return getServantType(databaseID, getServantID(servantName));
}

std::string DirectServantAccessor::getControllerType(unsigned long databaseID, unsigned long controllerID) const {
	try {
		return _proxy->getControllerType(databaseID, controllerID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(controllerID);
	}
}

std::string DirectServantAccessor::getDriverType(unsigned long databaseID, unsigned long driverID) const {
	try {
		return _proxy->getDriverType(databaseID, driverID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(driverID);
	}
}

bool DirectServantAccessor::isServantEnabledAndInitialised(unsigned long databaseID, unsigned long servantID) const {

	try {
		return _proxy->isServantEnabled(databaseID, servantID)
			&& _proxy->isServantInitialised(databaseID, servantID);

	} catch (ServantAccessorDirectImpl::NoSuchServantException& e) {
		throw NoSuchServantException(servantID);
	}
}

std::string DirectServantAccessor::what(unsigned long databaseID, unsigned long elementID) const {

	return _proxy->what(databaseID, elementID);
}

bool DirectServantAccessor::servantExists(unsigned long databaseID, unsigned long servantID) const {

	return _proxy->servantExists(databaseID, servantID);
}

bool DirectServantAccessor::isServantPausable(unsigned long databaseID, unsigned long servantID) const {

	return _proxy->isServantPausable(databaseID, servantID);
}

long DirectServantAccessor::getDriverChannel(unsigned long databaseID, unsigned long servantID) const {

	return _proxy->getDriverChannel(databaseID, servantID);
}

}
