/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CORE_DIRECTSERVANTACCESSOR_H_
#define CORE_DIRECTSERVANTACCESSOR_H_

#include <string>

#include "ServantAccessor.h"
#include <common/base/HashMap.h>

namespace dataprovider {

class ServantAccessorDirectImpl;

}

namespace core {

/**
 * Class defining a direct servant accessor.
 */
class DirectServantAccessor : public ServantAccessor {

public:
	/**
 	 * Constructor.
 	 */
	DirectServantAccessor(dataprovider::ServantAccessorDirectImpl* proxy);

	/**
 	 * Virtual destructor.
	 */
	virtual ~DirectServantAccessor();

	/**
	 * Initialises.
	 */
	virtual void init();

	/**
	 * Returns the ID corresponding to the servant name.
	 * throw NoSuchServantException
	 */
	virtual unsigned long getServantID(const std::string& servantName) const;

	/**
	 * Returns the servant name associated with the servant ID.
	 */
	virtual std::string getServantName(unsigned long databaseID, unsigned long servantID);

	/**
	 * Gets the servant type.
	 * throw NoSuchServantException
	 */
	virtual ServantType getServantType(unsigned long databaseID, unsigned long servantID) const;

	/**
	 * Gets the servant type.
	 * throw NoSuchServantException
	 */
	virtual ServantType getServantType(unsigned long databaseID, const std::string& servantName) const;

	/**
	 * Gets the controller type.
	 * throw NoSuchServantException
	 */
	virtual std::string getControllerType(unsigned long databaseID, unsigned long controllerID) const;

	/**
	 * Gets the driver type.
	 * throw NoSuchServantException
	 */
	virtual std::string getDriverType(unsigned long databaseID, unsigned long driverID) const;

	/**
	 * Returns true if the servant is enabled.
	 */
	virtual bool isServantEnabledAndInitialised(unsigned long databaseID, unsigned long servantID) const;

	/**
	 * Gets the description of the element.
	 */
	virtual std::string what(unsigned long databaseID, unsigned long elementID) const;

	/**
	 * Returns true if is a servant.
	 */
	virtual bool servantExists(unsigned long databaseID, unsigned long servantID) const;

	/**
	 * Returns true if servant is pausable.
	 */
	virtual bool isServantPausable(unsigned long databaseID, unsigned long servantID) const;

	/**
	 * Returns the driver channel of a driver for a given ID.
	 * throw NoSuchServantException
	 */
	virtual long getDriverChannel(unsigned long databaseID, unsigned long servantID) const;

private:
	dataprovider::ServantAccessorDirectImpl* _proxy;

	common::HashMap<std::string, unsigned long> _IDs;

};

}

#endif /*CORE_DIRECTSERVANTACCESSOR_H_*/