/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectStateAccessor.h"
#include "../../../dataprovider/transportlayer/dataprovider/StateAccessorDirectImpl.h"

namespace core {

DirectStateAccessor::DirectStateAccessor(dataprovider::StateAccessorDirectImpl* proxy) :
	_proxy(proxy) {
}
	
DirectStateAccessor::~DirectStateAccessor() {
	
}

void DirectStateAccessor::notifyConditionState(unsigned long conditionId, bool on, common::ConditionState state) {
	_proxy->notifyConditionState(conditionId, on, state);
}

void DirectStateAccessor::notifyActionsState(unsigned long conditionId, common::ConditionState state) {
	_proxy->notifyActionsState(conditionId, state);
}

void DirectStateAccessor::notifyUserVariableValueChanged(unsigned long databaseId, const std::string& name, double value) {
	_proxy->notifyUserVariableValueChanged(databaseId, name, value);
}

void DirectStateAccessor::notifyUserVariableStateChanged(unsigned long databaseId, const std::string& name, bool enabled) {
	_proxy->notifyUserVariableStateChanged(databaseId, name, enabled);
}

void DirectStateAccessor::notifyTokenStateChanged(common::CriticalState criticalState, common::RemoteControlState remoteControlState, common::TokenState state, uint32 ownerClientID, const std::string& ownerEndpoint, const std::string& user) {
	_proxy->notifyTokenStateChanged(criticalState, remoteControlState, state, ownerClientID, ownerEndpoint, user);
}

void DirectStateAccessor::notifyChatMessage(uint32 clientID, int64 date, const std::string& sender, const std::string& message, const std::string& fileName) {
	_proxy->notifyChatMessage(clientID, date, sender, message, fileName);
}

void DirectStateAccessor::notifyConnectedUsersChanged() {
	_proxy->notifyConnectedUsersChanged();
}

void DirectStateAccessor::notifyNewFileCreated(common::FileType fileType, const std::string& fileName) {
	_proxy->notifyNewFileCreated(fileType, fileName);
}

}

