/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_DIRECTSTATEACCESSOR_H_
#define COMMAND_DIRECTSTATEACCESSOR_H_

#include <common/base/CommonTypes.h>
#include <string>
#include "StateAccessor.h"

namespace dataprovider {
	class StateAccessorDirectImpl;
}

namespace core {

/**
 * Abstract class defining the state accessor.
 */
class DirectStateAccessor : public StateAccessor {
	
public:
	/**
	 * Constructor
	 */
	DirectStateAccessor(dataprovider::StateAccessorDirectImpl* proxy);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DirectStateAccessor();

	/**
	 * Notifies the condition state.
	 */
	virtual void notifyConditionState(unsigned long conditionId, bool on, common::ConditionState state);

	/**
	 * Notifies the watchdog actions state.
	 */
	virtual void notifyActionsState(unsigned long conditionId, common::ConditionState state);

	/**
	 * Notifies the user variable value changed.
	 */
	virtual void notifyUserVariableValueChanged(unsigned long databaseId, const std::string& name, double value);

	/**
	 * Notifies the user variable state changed.
	 */
	virtual void notifyUserVariableStateChanged(unsigned long databaseId, const std::string& name, bool enabled);

	/**
	 * Notifies a token state change.
	 */
	virtual void notifyTokenStateChanged(common::CriticalState criticalState, common::RemoteControlState remoteControlState, common::TokenState state, uint32 ownerClientID, const std::string& ownerEndpoint, const std::string& user);

	/**
	 * Notifies a chat message.
	 */
	virtual void notifyChatMessage(uint32 clientID, int64 date, const std::string& sender, const std::string& message, const std::string& fileName);

	/**
	 * Notifies change in the connected users.
	 */
	virtual void notifyConnectedUsersChanged();

	/**
	 * Notifies that a new file has been created.
	 */
	virtual void notifyNewFileCreated(common::FileType fileType, const std::string& fileName);

private:
	dataprovider::StateAccessorDirectImpl* _proxy;
};

}

#endif
