/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_LOGACCESSOR_H_
#define COMMAND_LOGACCESSOR_H_

#include <string>
#include <core/log/Log.h>

namespace core {

/**
 * Abstract class defining the log accessor.
 */
class LogAccessor {
	
public:
	/**
	 * Constructor
	 */
	LogAccessor();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~LogAccessor();

	/**
	 * Write the Command log entry
	 */
	virtual void writeCommandLog(common::LogType type, common::LogLevel level, const std::string& logMessage, const std::string& logID) = 0;

	/**
	 * Write the Simple log entry
	 */
	virtual void writeSimpleLog(common::LogLevel level, const std::string& logMessage, bool writeLogFile) = 0;
		
	/**
	 * Write the Command Zone Content log entry
	 */
	virtual void writeCommandZoneContentLog(const std::string& content) = 0;

	/**
	 * Write the Command Zone Event log entry
	 */
	virtual void writeCommandZoneEventLog(const std::string& content) = 0;

	/**
	 * Write a Variable Change log entry
	 */
	virtual void writeVariableChangeLog(common::LogLevel level, const std::string& variableName, const std::string& variableType, const std::string& variableValue, const std::string& message, const unsigned long messageParentId) = 0;

	/**
	 * Write a For Loop Start log entry
	 */
	virtual void writeForLoopStartLog(common::LogLevel level, const std::string& variablesField, const std::string& message, const unsigned long messageId) = 0;

	/**
	 * Write a For Loop End log entry
	 */
	virtual void writeForLoopEndLog(common::LogLevel level, const std::string& message, const unsigned long messageId) = 0;

	/**
	 * Write an If Evaluation log entry
	 */
	virtual void writeIfEvaluationLog(common::LogLevel level, const std::string& name, const std::string& type, const std::string& value, const std::string& message) = 0;

	/**
	 * Write a Break Evaluation log entry
	 */
	virtual void writeBreakEvaluationLog(common::LogLevel level, const std::string& name, const std::string& type, const std::string& value, const std::string& message, const unsigned long messageParentId) = 0;

	/**
	 * Write an User's Scan Start log entry
	 */
	virtual void writeUserScanStartLog(common::LogLevel level, const std::string& name, const std::string& message) = 0;


};

}

#endif /*COMMAND_LOGACCESSOR_H_*/
