/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PropertyAccessor.h"
#include "common/base/Convert.h"

using namespace common;
using namespace std;

namespace core {
	
PropertyAccessor::PropertyListCreationException::PropertyListCreationException(unsigned long ID) :
	common::Exception("core::PropertyAccessor::PropertyListCreationException", string("server failed to create list of properties for servant with ID ") + common::to<string>(ID)) {
}
	
PropertyAccessor::NoSuchServantException::NoSuchServantException(long servantID)
: common::Exception("core::PropertyAccessor::NoSuchServantException", string("unable to find servant <") + to<string>(servantID) + ">") {
}

PropertyAccessor::NoSuchPropertyException::NoSuchPropertyException()
: common::Exception("core::PropertyAccessor::NoSuchPropertyException", string("unable to find property")) {
}

PropertyAccessor::NoSuchPropertyException::NoSuchPropertyException(long propertyID)
: common::Exception("core::PropertyAccessor::NoSuchPropertyException", string("unable to find property <") + to<string>(propertyID) + ">") {
}

PropertyAccessor::NoSuchPropertyException::NoSuchPropertyException(const std::string& propertyName)
: common::Exception("core::PropertyAccessor::NoSuchPropertyException", string("unable to find property \"") + propertyName + "\"") {
}

PropertyAccessor::IndexOutOfBoundsException::IndexOutOfBoundsException(unsigned long propertyID)
: common::Exception("core::PropertyAccessor::IndexOutOfBoundsException", string("requesting bad index for array property <") + to<std::string>(propertyID) + ">") {
}

PropertyAccessor::BadPropertyTypeException::BadPropertyTypeException(long propertyID)
: common::Exception("core::PropertyAccessor::BadPropertyTypeException", string("requesting bad type for property <") + to<std::string>(propertyID) + ">") {
}

PropertyAccessor::NullPropertyReferenceException::NullPropertyReferenceException(long propertyID)
: common::Exception("core::PropertyAccessor::NullPropertyReferenceException", string("null reference for property <") + to<std::string>(propertyID) + ">") {
}

PropertyAccessor::PropertyAccessor() {
}
	
PropertyAccessor::~PropertyAccessor() {
}
	
}