/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_STATEACCESSOR_H_
#define COMMAND_STATEACCESSOR_H_

#include <common/base/CommonTypes.h>
#include <common/base/BaseTypes.h>
#include <string>

namespace core {

/**
 * Abstract class defining an accessor for the notifications of conditions, variables, token.
 */
class StateAccessor {
	
public:
	/**
	 * Constructor
	 */
	StateAccessor();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~StateAccessor();

	/**
	 * Notifies the condition state.
	 */
	virtual void notifyConditionState(unsigned long conditionId, bool on, common::ConditionState) = 0;

	/**
	 * Notifies the watchdog actions state.
	 */
	virtual void notifyActionsState(unsigned long conditionId, common::ConditionState) = 0;

	/**
	 * Notifies the user variable value changed.
	 */
	virtual void notifyUserVariableValueChanged(unsigned long databaseId, const std::string& name, double value) = 0;

	/**
	 * Notifies the user variable state changed.
	 */
	virtual void notifyUserVariableStateChanged(unsigned long databaseId, const std::string& name, bool enabled) = 0;

	/**
	 * Notifies a token state change.
	 */
	virtual void notifyTokenStateChanged(common::CriticalState criticalState, common::RemoteControlState remoteControlState, common::TokenState state, uint32 ownerClientID, const std::string& ownerEndpoint, const std::string& user) = 0;

	/**
	 * Notifies a chat message.
	 */
	virtual void notifyChatMessage(uint32 clientID, int64 date, const std::string& sender, const std::string& message, const std::string& fileName) = 0;

	/**
	 * Notifies change in the connected users.
	 */
	virtual void notifyConnectedUsersChanged() = 0;

	/**
	 * Notifies that a new file has been created.
	 */
	virtual void notifyNewFileCreated(common::FileType fileType, const std::string& fileName) = 0;
};

}

#endif

