/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_DIRECTDATAPROVIDER_H_
#define DATAPROVIDER_DIRECTDATAPROVIDER_H_

#include <common/Application.h>
#include <common/base/HashMap.h>
#include <string>

namespace dataprovider {

class Lock;

/**
 * Dataprovider application.
 */
class DirectDataProvider : public common::Application {

public :
	/**
	 * Constructor.
	 */
	DirectDataProvider();

	/**
	 * Virtual destructor.
	 */
	virtual ~DirectDataProvider();

	/**
	 * Initialises the application. Initialises the Database and the CommunicationManager
	 * object.
	 */
	virtual void init(int argc, char* argv[]);

	/**
	 * Runs the application.
	 */
	virtual void run();

	/**
	 * Resets the application.
	 */
	virtual void reset();

	/**
	 * Executes the command called commandName.
	 */
	virtual void execute(const std::string& commandName, const std::vector<std::string>& args);

	/**
 	 * Quits the application.
 	 */
	virtual void quit(const std::vector<std::string>& args, bool isRemote);

	/**
	 * Shutdowns the application.
	 */
	void shutdown(bool isRemote);

private :
	void show();

	bool _onError;
	common::HashMap<unsigned long, Lock*> _lockMap;

	static const std::string ICS;
	static const std::string SIMULATION;
	static const std::string SIMULATION3D;
};


} // namespace dataprovider

#endif /*DATAPROVIDER_DIRECTDATAPROVIDER_H_*/
