/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

namespace dataprovider {

template<typename Key, typename Data, typename HashFun>
LockableHashMap<Key, Data, HashFun>::NoSuchKeyException::NoSuchKeyException(Key const & key)
: BaseNoSuchKeyException("dataprovider::LockableHashMap<Key, Data>::NoSuchKeyException", common::to<std::string>(key)) {
};

template<typename Key, typename Data, typename HashFun>
void LockableHashMap<Key, Data, HashFun>::erase(Key const & key) {

	ScopedLock lock(_lockManager->lockMaps(common::eol));

	typename ImplementationType::iterator it = _impl.find(key);
	if (it == _impl.end()) {
		throw NoSuchKeyException(key);
	}

	_impl.erase(it);
}

template<typename Key, typename Data, typename HashFun>
Data & LockableHashMap<Key, Data, HashFun>::get(Key const & key) {

	ScopedLock lock(_lockManager->lockMaps(common::eol));

	typename ImplementationType::iterator it = _impl.find(key);
	if (it == _impl.end()) {
		throw NoSuchKeyException(key);
	}

	return it->second;
}

template<typename Key, typename Data, typename HashFun>
unsigned long LockableHashMap<Key, Data, HashFun>::getSize() {

	ScopedLock lock(_lockManager->lockMaps(common::eol));

	return _impl.size();
}

template<typename Key, typename Data, typename HashFun>
std::vector<Key> LockableHashMap<Key, Data, HashFun>::getKeySet() {

	ScopedLock lock(_lockManager->lockMaps(common::eol));

	std::vector<Key> result(_impl.size());

	int i = 0;
	for (typename ImplementationType::const_iterator it = _impl.begin(); it != _impl.end(); it++) {
		result[i] = it->first;
		i++;
	}

	return result;
}

template<typename Key, typename Data, typename HashFun>
void LockableHashMap<Key, Data, HashFun>::clear() {

	ScopedLock lock(_lockManager->lockMaps(common::eol));

	_impl.clear();
}

}