/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "IndexGenerator.h"

using namespace std;

namespace dataprovider {

std::map<unsigned long, IndexGenerator *> IndexGenerator::_instances;
std::mutex IndexGenerator::_mutex;

IndexGenerator::IndexGenerator(unsigned long ID) 
: _ID(ID), _currentID(1) {
}

IndexGenerator::~IndexGenerator() {
}

void IndexGenerator::initInstance() {
	
	_instances[0] = new IndexGenerator(0);
}

IndexGenerator* IndexGenerator::getInstance(unsigned long ID) {
	
	std::scoped_lock lock(_mutex);
	
	return _instances[ID];
}

void IndexGenerator::resetInstance(unsigned long ID) {

	std::scoped_lock lock(_mutex);

	if (_instances[ID] != 0) {
		delete _instances[ID];
		_instances[ID] = 0;
	}
}
	
void IndexGenerator::resetInstances() {

	std::scoped_lock lock(_mutex);

	unsigned long index;

	for (map<unsigned long, IndexGenerator *>::iterator g = _instances.begin(); g != _instances.end(); g++) {
		delete g->second;
	}
	_instances.clear();
}

unsigned long IndexGenerator::getNextIndex() {
	
	std::scoped_lock lock(_mutex);
	
	return _currentID++;
}

unsigned long IndexGenerator::getCurrentIndex() {
	
	std::scoped_lock lock(_mutex);
	
	return _currentID;
}

void IndexGenerator::clone(unsigned long ID) {
	
	std::scoped_lock lock(_mutex);
	
	IndexGenerator* instance = new IndexGenerator(ID);
	instance->_currentID = _currentID;

	_instances[ID] = instance;
}

}
