/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_INDEXGENERATOR_H_
#define DATAPROVIDER_INDEXGENERATOR_H_

#include <map>
#include <mutex>

namespace dataprovider {
	
/**
 * Class generating unique index.
 */
class IndexGenerator {

public:
	/**
	 * Initialises the instance.
	 */
	static void initInstance();
	
	/**
	 * Gets the instance.
	 */
	static IndexGenerator* getInstance(unsigned long ID = 0);
	
	/**
	 * Resets the instance.
	 */
	static void resetInstance(unsigned long ID);
	
	/**
	 * Resets the instances.
	 */
	static void resetInstances();

	/**
	 * Gets the next unique index.
	 */
	unsigned long getNextIndex();
	
	/**
	 * Gets the current index.
	 */
	unsigned long getCurrentIndex();
	
	/**
	 * Clones the IndexGenerator. Keeps the current ID.
	 */
	void clone(unsigned long ID);

private:
	IndexGenerator(unsigned long ID);
	~IndexGenerator();
	
	unsigned long _ID;
	unsigned long _currentID;
	static std::map<unsigned long, IndexGenerator *> _instances;
	static std::mutex _mutex;
};

}

#endif /*DATAPROVIDER_INDEXGENERATOR_H_*/
