/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "BaseDynamicProperty.h"
#include "DataProperty.h"

using namespace std;

namespace dataprovider {

BaseDynamicProperty::BaseDynamicProperty(std::string const & name, bool generated, bool saveValue, std::string const & format) :
	BaseProperty(name, generated, saveValue, format) {
}

BaseDynamicProperty::BaseDynamicProperty(BaseDynamicProperty const & content) :
	BaseProperty(content),
	_propertyIDs(content._propertyIDs) {
}

BaseDynamicProperty::~BaseDynamicProperty() {
}

void BaseDynamicProperty::addProperty(DataProperty * property) {

	if (property != 0) {
		_propertyIDs.push_back(property->getID());
		property->setParentID(getParentID());
		
	} else {
		// null property
		_propertyIDs.push_back(0);
	}
}

std::vector<unsigned long> const & BaseDynamicProperty::getPropertyIDs() const {
	
	return _propertyIDs;
}

long BaseDynamicProperty::getSize() const {
	
	return (long)_propertyIDs.size();
}

}
