/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_BASEDYNAMICPROPERTY_H_
#define DATAPROVIDER_BASEDYNAMICPROPERTY_H_

#include "BaseProperty.h"
#include <common/base/Exception.h>
#include <vector>
#include <list>

namespace dataprovider {

/**
 * Forward declarations.
 */
class DataProperty;

/**
 * Class defining a property list.
 */
class BaseDynamicProperty : public BaseProperty {
	
public:
	/**
	 * Constructor.
	 */
	BaseDynamicProperty(std::string const & name, bool generated, bool saveValue, std::string const & format);
	
	/**
	 * Copy constructor.
	 */
	BaseDynamicProperty(BaseDynamicProperty const & content);
	
	/**
	 * Virtual destructor. Does not delete the BasePropertyContent objects.
	 */
	virtual ~BaseDynamicProperty();
	
	/**
	 * Creates a Property.
	 */
	virtual DataProperty * createProperty(bool generated, bool saveValue) const = 0;
	
	/**
	 * Resizes the container. Returns the list of property IDs removed.
	 */
	virtual std::vector<unsigned long> downsize(long size) = 0;
	
	/**
	 * Adds a property.
	 */
	void addProperty(DataProperty * property);
	
	/**
	 * Gets the content list.
	 */
	std::vector<unsigned long> const & getPropertyIDs() const;
	
	/**
	 * Gets the size.
	 */
	long getSize() const;
	
protected:
	std::vector<unsigned long> _propertyIDs;
};
	
}

#endif /*DATAPROVIDER_BASEDYNAMICPROPERTY_H_*/