/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_BASEELEMENTARYPROPERTY_H_
#define DATAPROVIDER_BASEELEMENTARYPROPERTY_H_

#include "DataProperty.h"

namespace dataprovider {

class Lock;
	
/**
 * Class defining a base elementary property by opposition to property array.
 */
class BaseElementaryProperty : public DataProperty {
	
public:
	/**
	 * Constructor.
	 */
	BaseElementaryProperty(std::string const & name, bool generated, bool saveValue, std::string const & format);
	
	/**
	 * Virtual destructor. Does not delete the BasePropertyContent object.
	 */
	virtual ~BaseElementaryProperty();
	
	/**
	 * Gets the size of data.
	 */
	virtual unsigned short getSizeOfData() const = 0;

	/**
	 * Sets the formatted value.
	 */
	void setFormattedValue(const std::string& value);

	/**
	 * Gets the formatted value.
	 */
	std::string const & getFormattedValue() const;

protected:
	std::string _formattedValue;
};

}

#endif /*DATAPROVIDER_BASEELEMENTARYPROPERTY_H_*/
