/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_BASEICSARRAYPROPERTY_H_
#define DATAPROVIDER_BASEICSARRAYPROPERTY_H_

#include "DataProperty.h"
#include <vector>
#include <string>

namespace dataprovider {
	
/**
 * Class defining a base ics array property.
 */
class BaseIcsArrayProperty : public DataProperty {
	
public:
	/**
	 * Constructor.
	 */
	BaseIcsArrayProperty(std::string const & name, std::string const & type, bool saveValue, const std::string & format);
	
	/**
	 * Copy constructor.
	 */
	BaseIcsArrayProperty(const BaseIcsArrayProperty& content);
	
	/**
	 * Destructor.
	 */
	virtual ~BaseIcsArrayProperty();
	
	/**
	 * Gets the stringified type.
	 */
	virtual std::string getType() const;

	/**
	 * Sets the value of the property by converting from a string value.
	 */
	virtual void setValueFromString(const std::vector<std::string>& stringValue) = 0;

	/**
	 * Sets the value of the property by converting from a string value.
	 */
	virtual bool setValueFromString(const std::string& stringValue, const std::string& format);

private:
	std::string _type;
};

}

#endif /*DATAPROVIDER_BASEICSARRAYPROPERTY_H_*/