/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "BaseProperty.h"
#include "Command.h"
#include <dataprovider/database/Database.h>
#include <common/output/OutputStream.h>

using namespace std;
using namespace common;

namespace dataprovider {

BaseProperty::BaseProperty(std::string const & name, bool generated, bool saveValue, std::string const & format) :
	_name(name),
	_generated(generated),
	_saveValue(saveValue),
	_spy(false),
	_saveBox(false),
	_format(format),
	_parentID(0),
	_command(new Command(name)),
	_enumeratedValuesPropertyID(0),
	_enumeratedLabelsPropertyID(0) {
}

BaseProperty::BaseProperty(const BaseProperty& property) :
	DataElement(property),
	_name(property._name),
	_generated(property._generated),
	_saveValue(property._saveValue),
	_spy(property._spy),
	_saveBox(property._saveBox),
	_format(property._format),
	_parentID(property._parentID),
	_command(new Command(*property._command)),
	_enumeratedValuesPropertyID(property._enumeratedValuesPropertyID),
	_enumeratedLabelsPropertyID(property._enumeratedLabelsPropertyID) {
}

BaseProperty::~BaseProperty() {
	delete _command;
}

void BaseProperty::setID(unsigned long ID) {
	DataElement::setID(ID);
	_command->setID(ID);
}

void BaseProperty::setDatabaseID(unsigned long databaseID) {
	DataElement::setDatabaseID(databaseID);
	_command->setDatabaseID(databaseID);
}

void BaseProperty::setFormat(const std::string& format) {
	_format = format;
}

void BaseProperty::setSaveValue(bool value) {
	_saveValue = value;
}

void BaseProperty::setSpy(bool value) {
	_spy = value;
}

void BaseProperty::setSaveBox(bool value) {
	_saveBox = value;
}

Command* BaseProperty::getCommand() const {
	return _command;
}

std::string const & BaseProperty::getName() const {
	return _name;
}

bool BaseProperty::isGenerated() const {
	return _generated;
}

bool BaseProperty::saveValue() const {
	return _saveValue;
}

bool BaseProperty::isSpy() const {
	return _spy;
}

bool BaseProperty::isSaveBox() const {
	return _saveBox;
}

std::string const & BaseProperty::getFormat() const {
	return _format;
}

void BaseProperty::setParentID(unsigned long parentID) {
	_parentID = parentID;
	_command->setServantID(parentID);
}

unsigned long BaseProperty::getParentID() const {
	return _parentID;
}

std::string BaseProperty::getParentString() const {
	return Database::getInstance()->what(_parentID);
}

void BaseProperty::setEnumeratedValuesPropertyID(unsigned long propertyID) {
	_enumeratedValuesPropertyID = propertyID;
}

unsigned long BaseProperty::getEnumeratedValuesPropertyID() const {
	return _enumeratedValuesPropertyID;
}

void BaseProperty::setEnumeratedLabelsPropertyID(unsigned long propertyID) {
	_enumeratedLabelsPropertyID = propertyID;
}

unsigned long BaseProperty::getEnumeratedLabelsPropertyID() const {
	return _enumeratedLabelsPropertyID;
}

}
