/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_BASEPROPERTY_H_
#define DATAPROVIDER_BASEPROPERTY_H_

#include "DataElement.h"
#include <string>

namespace dataprovider {
	
class PropertyHolder;
class Command;
	
/**
 * Abstract class defining a property for servant. It is inherited by ElementaryProperty
 * and PropertyArray.
 */
class BaseProperty : public DataElement {
	
public:
	/**
	 * Constructor.
	 */
	BaseProperty(std::string const & name, bool generated, bool saveValue, std::string const & format);
	
	/**
	 * Copy constructor.
	 */
	BaseProperty(const BaseProperty& property);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~BaseProperty();
	
	/**
	 * Returns the type.
	 */
	virtual std::string getType() const = 0;
	
	/**
	 * Sets the ID.
	 */
	virtual void setID(unsigned long ID);

	/**
	 * Sets the database ID.
	 */
	virtual void setDatabaseID(unsigned long databaseID);
	
	/**
	 * Sets the format.
	 */
	void setFormat(const std::string& format);

	/**
	 * Sets the save value.
	 */
	void setSaveValue(bool value);

	/**
	 * Sets the spy.
	 */
	void setSpy(bool value);

	/**
	 * Sets the save box.
	 */
	void setSaveBox(bool value);

	/**
	 * Returns the command for the setter.
	 */
	Command* getCommand() const;
	
	/**
	 * Gets the name.
	 */
	std::string const & getName() const; 

	/**
	 * Returns true if is generated.
	 */
	bool isGenerated() const;

	/**
	 * Returns true if saves the value.
	 */
	bool saveValue() const;

	/**
	 * Returns true if is spy.
	 */
	bool isSpy() const;

	/**
	 * Returns true if is save box.
	 */
	bool isSaveBox() const;

	/**
	 * Gets the format.
	 */
	std::string const & getFormat() const;

	/**
	 * Sets the parent.
	 */
	void setParentID(unsigned long parentID);

	/**
	 * Gets the parent ID.
	 */
	unsigned long getParentID() const;

	/**
	 * Returns the string describing the parent.
	 */
	std::string getParentString() const;

	/**
	 * Sets the enumerated values property ID.
	 */
	void setEnumeratedValuesPropertyID(unsigned long propertyID);

	/**
	 * Gets the enumerated values property ID.
	 */
	unsigned long getEnumeratedValuesPropertyID() const;

	/**
	 * Sets the enumerated labels property ID.
	 */
	void setEnumeratedLabelsPropertyID(unsigned long propertyID);

	/**
	 * Gets the enumerated labels property ID.
	 */
	unsigned long getEnumeratedLabelsPropertyID() const;

private:
	std::string _name;
	bool _generated;
	bool _saveValue;
	bool _spy;
	bool _saveBox;
	std::string _format;
	unsigned long _parentID;
	Command* _command;
	unsigned long _enumeratedValuesPropertyID;
	unsigned long _enumeratedLabelsPropertyID;
};

}

#endif /*DATAPROVIDER_BASEPROPERTY_H_*/
