/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Controller.h"
#include "Command.h"
#include "BaseProperty.h"
#include <common/output/OutputStream.h>

using namespace std;
using namespace common;

namespace dataprovider {

Controller::Controller(std::string const & controllerType, std::string const & name, bool isCommand, bool isVisible, bool isPausable, bool isFavourite, bool generated) :
	Servant(name, controllerType, isCommand, isVisible, isPausable, isFavourite),
	_controllerType(controllerType),
	_generated(generated),
	_referenceCounter(0) {
}

Controller::Controller(Controller const & content) :
	Servant(content),
	_controllerType(content._controllerType),
	_generated(false),
	_referenceCounter(0) {
}

Controller::~Controller() {
}

bool Controller::isGenerated() const {

	return _generated;
}

Servant::Type Controller::getServantType() const {
	
	return CONTROLLER;
}

std::string const & Controller::getControllerType() const {
	
	return _controllerType;
}

DataElement * Controller::clone() const {
	
	return new Controller(*this);
}

int Controller::getReferenceCounter() const {
	
	return _referenceCounter;
}
	
void Controller::incrementReferenceCounter() {

	_referenceCounter++;
}
	
void Controller::decrementReferenceCounter() {
	
	_referenceCounter--;
}

std::string Controller::toString() const {
	
	return string("controller '") + getName() + "' (" + getControllerType() + ")";
}

}
