/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_CONTROLLER_H_
#define DATAPROVIDER_CONTROLLER_H_

#include "Servant.h"

namespace dataprovider {

/**
 * Class defining a controller.
 */
class Controller : public Servant {
	
public:
	/**
	 * Constructor.
	 */
	Controller(std::string const & controllerType, std::string const & name, bool isCommand, bool isVisible, bool isPausable, bool isFavourite, bool generated = false);
	
	/**
	 * Copy constructor.
	 */
	Controller(Controller const & content);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~Controller();

	/**
	 * Returns true if is generated.
	 */
	bool isGenerated() const;
	
	/**
	 * Gets the servant type.
	 */
	virtual Type getServantType() const;
	
	/**
	 * Gets the controller type.
	 */
	virtual std::string const & getControllerType() const;

	/**
	 * Clones the element by keeping the references to the sub-elements.
	 */
	virtual DataElement * clone() const;
	
	/**
	 * Returns the reference counter.
	 */
	int getReferenceCounter() const;
	
	/**
	 * Increments the reference counter.
	 */
	void incrementReferenceCounter();
	
	/**
	 * Decrements the reference counter.
	 */
	void decrementReferenceCounter();
	
	/**
	 * Prints the servant.
	 */
	virtual std::string toString() const;

private:
	std::string _controllerType;
	bool _generated;
	int _referenceCounter;
};
	
}

#endif /*DATAPROVIDER_CONTROLLER_H_*/