/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_DATAELEMENT_H_
#define DATAPROVIDER_DATAELEMENT_H_

#include <common/base/HashMap.h>

namespace dataprovider {

/**
 * Base class for the Data Provider package.
 */	
class DataElement {

public:
	/**
	 * Constructor.
	 */
	DataElement();

	/**
	 * Virtual destructor.
	 */
	virtual ~DataElement();

	/**
	 * Sets the ID.
	 */
	virtual void setID(unsigned long ID);

	/**
	 * Sets the database ID.
	 */
	virtual void setDatabaseID(unsigned long databaseID);
	
	/**
	 * Clones the element by keeping the references to the sub-elements.
	 */
	virtual DataElement * clone() const = 0;
	
	/**
	 * Gets the ID.
	 */
	unsigned long getID() const;
	
	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;
	
	/**
	 * Prints the servant.
	 */
	virtual std::string toString() const = 0;

private:
	unsigned long _ID;
	unsigned long _databaseID;
};

}

#endif /*DATAPROVIDER_DATAELEMENT_H_*/