/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_DATAPROPERTY_H_
#define DATAPROVIDER_DATAPROPERTY_H_

#include "BaseProperty.h"

namespace dataprovider {
	
/**
 * Abstract class defining a property containing data. It is inherited by ElementaryProperty
 * and ArrayProperty.
 */
class DataProperty : public BaseProperty {
	
public:
	/**
	 * Constructor.
	 */
	DataProperty(std::string const & name, bool generated, bool saveValue, const std::string & format);
	
	/**
	 * Copy constructor.
	 */
	DataProperty(const DataProperty& content);  
	
	/**
	 * Destructor.
	 */
	virtual ~DataProperty();

	/**
	 * Gets the stringified type.
	 */
	virtual std::string getType() const = 0;

	/**
	 * Gets the stringified value. Sets the boolean value to true if has an overflow.
	 */
	virtual std::string getStringValue(bool& overflow) const = 0;
	
	/**
	 * Sets the value of the property by converting from a string value.
	 */
	virtual bool setValueFromString(const std::string& stringValue, const std::string& format) = 0;
};

}

#endif /*DATAPROVIDER_DATAPROPERTY_H_*/
