/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Driver.h"
#include "Command.h"
#include "BaseProperty.h"

using namespace std;
using namespace common;

namespace dataprovider {

Driver::MultipleParentException::MultipleParentException(std::string const & name)
: Exception("dataprovider::Driver::MultipleParentException", string("multiple parents for driver '") + name + "'") {
}

Driver::Driver(std::string const & driverType, std::string const & name, bool isCommand, bool isVisible, bool isPausable)
: Servant(name, driverType, isCommand, isVisible, isPausable, false),
	_driverType(driverType),
	_fatherInitialised(false),
	_fatherID(0),
	_channel(0),
	_mode(common::REAL_MODE),
	_state(common::INACTIVE),
	_runningProcessorID(0),
	_runningThreadID(0) {
}

Driver::Driver(Driver const & content) :
	Servant(content),
	_driverType(content._driverType),
	_fatherInitialised(false),
	_fatherID(content._fatherID),
	_channel(0),
	_mode(content._mode),
	_state(content._state),
	_runningProcessorID(content._runningProcessorID),
	_runningThreadID(content._runningThreadID) {
}

Driver::~Driver() {
}

Servant::Type Driver::getServantType() const {

	return DRIVER;
}

std::string const & Driver::getDriverType() const {

	return _driverType;
}

void Driver::setFatherInfo(unsigned long fatherID, std::string const & childName, long channel) {

	if (!_fatherInitialised) {
		_fatherID = fatherID;
		_fatherChildName = childName;
		_channel = channel;
		_fatherInitialised = true;
	} else {
		throw MultipleParentException(getName());
	}
}

unsigned long Driver::getFatherID() {
	return _fatherID;
}

long Driver::getChannel() {
	return _channel;
}

bool Driver::isRoot() const {
	return _fatherID == 0;
}

void Driver::setMode(common::DriverMode mode) {
	_mode = mode;
}

common::DriverMode Driver::getMode() const {
	return _mode;
}

void Driver::setState(common::CommandState state) {
	_state = state;
}

void Driver::setRunningProcessorID(unsigned long processorID) {
	_runningProcessorID = processorID;
}

void Driver::setRunningThreadID(unsigned long threadID) {
	_runningThreadID = threadID;
}

common::CommandState Driver::getState() const {
	return _state;
}

unsigned long Driver::getRunningProcessorID() const {

	return _runningProcessorID;
}

unsigned long Driver::getRunningThreadID() const {

	return _runningThreadID;
}

void Driver::resetElapsedSimulationTime() {

	_beginSimulationDate = Date();
}

double Driver::getElapsedSimulationTimeMs() const {

	return (Date() - _beginSimulationDate).getMilliseconds();
}

DataElement * Driver::clone() const {

	return new Driver(*this);
}

std::string Driver::toString() const {

	return string("driver '") + getName() + "' (" + getDriverType() + ")";
}

}
