/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_DYNAMICPROPERTY_H_
#define DATAPROVIDER_DYNAMICPROPERTY_H_

#include "BaseDynamicProperty.h"
#include <common/base/Exception.h>
#include <vector>

namespace dataprovider {

/**
 * Class defining a dynamic property.
 */
template<typename ValueType>
class DynamicProperty : public BaseDynamicProperty {
	
public:
	/**
	 * Constructor.
	 */
	DynamicProperty(std::string const & name, bool generated, bool saveValue);
	
	/**
	 * Constructor with format.
	 */
	DynamicProperty(std::string const & name, bool generated, bool saveValue, std::string const & format);
	
	/**
	 * Copy constructor.
	 */
	DynamicProperty(DynamicProperty const & content);
	
	/**
	 * Virtual destructor. Does not delete the BasePropertyContent objects.
	 */
	virtual ~DynamicProperty();
	
	/**
	 * Clones the element by keeping the references to the sub-elements.
	 */
	virtual DataElement * clone() const;
	
	/**
	 * Creates a Property.
	 */
	virtual DataProperty * createProperty(bool generated, bool saveValue) const;
	
	/**
	 * Resizes the container. Returns the list of property IDs removed.
	 */
	virtual std::vector<unsigned long> downsize(long size);

	/**
	 * Returns the type.
	 */
	virtual std::string getType() const;
	
	/**
	 * Prints the element.
	 */
	virtual std::string toString() const;
};
	
}

#include "DynamicProperty.inl" 

#endif /*DATAPROVIDER_DYNAMICPROPERTY_H_*/
