/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_ELEMENTARYPROPERTY_H_
#define DATAPROVIDER_ELEMENTARYPROPERTY_H_

#include "BaseElementaryProperty.h"

namespace dataprovider {

/**
 * Class defining an elementary property.
 */
template<typename ValueType>
class ElementaryProperty : public BaseElementaryProperty {
	
public:
	/**
	 * Constructor.
	 */
	ElementaryProperty(std::string const & name, bool generated, bool saveValue);

	/**
	 * Constructor with format.
	 */
	ElementaryProperty(std::string const & name, bool generated, bool saveValue, const std::string & format);
	
	/**
	 * Copy constructor.
	 */
	ElementaryProperty(const ElementaryProperty& property);
	
	/**
	 * Virtual destructor. Does not delete the BasePropertyContent object.
	 */
	virtual ~ElementaryProperty();

	/**
	 * Clones the element by keeping the references to the sub-elements.
	 */
	virtual DataElement * clone() const;
	
	/**
	 * Gets the size of data. 
	 */
	virtual unsigned short getSizeOfData() const;
	
	/**
	 * Sets the ID.
	 */
	virtual void setID(unsigned long ID);
	
	/**
	 * Gets the stringified type.
	 */
	virtual std::string getType() const;

	/**
	 * Gets the stringified value.
	 */
	virtual std::string getStringValue(bool& overflow) const;
	
	/**
	 * Sets the value of the property by converting from a string value.
	 */
	virtual bool setValueFromString(const std::string& stringValue, const std::string& format);

	/**
	 * Sets the value.
	 */
	void setValue(ValueType const & value);
	
	/**
	 * Gets the value.
	 */
	ValueType const & getValue() const;
	
	/**
	 * Gets the value.
	 */
	ValueType & getValue();

	/**
	 * Prints the element.
	 */
	virtual std::string toString() const;

private:
	ValueType _value;
};

}

#include "ElementaryProperty.inl"

#endif /*DATAPROVIDER_BASEELEMENTARYPROPERTY_H_*/
