/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <common/base/TypeToString.h>
#include <common/base/Equals.h>
#include <iostream>
#include "PropertyHolder.h"

namespace dataprovider {

template <typename ValueType>
ElementaryProperty<ValueType>::ElementaryProperty(std::string const & name, bool generated, bool saveValue) :
	BaseElementaryProperty(name, generated, saveValue, ""),
	_value(ValueType()) {
}
	
template <typename ValueType>
ElementaryProperty<ValueType>::ElementaryProperty(std::string const & name, bool generated, bool saveValue, std::string const & format) :
	BaseElementaryProperty(name, generated, saveValue, format),
	_value(ValueType()) {
}

template <typename ValueType>
ElementaryProperty<ValueType>::ElementaryProperty(const ElementaryProperty& property) :
	BaseElementaryProperty(property), 
	_value(property._value) {
}

template <typename ValueType>
ElementaryProperty<ValueType>::~ElementaryProperty() {
}

template<typename ValueType>
DataElement * ElementaryProperty<ValueType>::clone() const {
	
	return new ElementaryProperty<ValueType>(*this);
}

template<typename ValueType>
unsigned short ElementaryProperty<ValueType>::getSizeOfData() const {
	return sizeof(ValueType);
}

template<typename ValueType>
void ElementaryProperty<ValueType>::setID(unsigned long ID) {
	DataElement::setID(ID);
}

template<typename ValueType>
std::string ElementaryProperty<ValueType>::getType() const {
	
	return common::TypeToString<ValueType>::convert();
}

template<typename ValueType>
bool ElementaryProperty<ValueType>::setValueFromString(const std::string& stringValue, const std::string& format) {

	ValueType oldValue = _value;
	_value = common::to<ValueType>(stringValue, format);
	
	return !common::Equals<ValueType>::compare(oldValue, _value);
}

template<typename ValueType>
std::string ElementaryProperty<ValueType>::getStringValue(bool& overflow) const {

	overflow = false;
	
	return common::to<std::string>(_value, getFormat());
}

template<typename ValueType>
void ElementaryProperty<ValueType>::setValue(ValueType const & value) {
	
	_value = value;
}

template<typename ValueType>	
ValueType const & ElementaryProperty<ValueType>::getValue() const {
	
	return _value;
}

template<typename ValueType>
ValueType & ElementaryProperty<ValueType>::getValue() {
	
	return _value;
}

template<typename ValueType>
std::string ElementaryProperty<ValueType>::toString() const {
	
	return std::string("property '") + getName() + "' (elementary <" + getType() + ">) of " + getParentString();
}

}
