/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_ICSARRAYPROPERTY_H_
#define DATAPROVIDER_ICSARRAYPROPERTY_H_

#include "ArrayTransferProperty.h"
#include "BaseIcsArrayProperty.h"
#include <common/base/ProxyArray.h>

namespace dataprovider {
	
/**
 * Class defining a ics array property.
 */
template<typename ValueType>
class IcsArrayProperty : public BaseIcsArrayProperty, public ArrayTransferProperty<ValueType> {
	
public:
	/**
	 * Constructor.
	 */
	IcsArrayProperty(std::string const & name, std::string const & type, bool saveValue, const std::string & format);
	
	/**
	 * Copy constructor.
	 */
	IcsArrayProperty(const IcsArrayProperty<ValueType>& content);
	
	/**
	 * Destructor.
	 */
	virtual ~IcsArrayProperty();

	/**
	 * Clones the element by keeping the references to the sub-elements.
	 */
	virtual DataElement * clone() const;

	/**
	 * Prints the element.
	 */
	virtual std::string toString() const;

	/**
	 * Gets the stringified value. Sets the boolean value to true if has an overflow.
	 */
	virtual std::string getStringValue(bool& overflow) const;
	
	/**
	 * Sets the value of the property by converting from a string value.
	 */
	virtual void setValueFromString(const std::vector<std::string>& stringValue);

	/**
	 * Sets the pointer value.
	 */
	void setPointerValue(ValueType* value);

	/**
	 * Gets the pointer value.
	 */
	ValueType* getPointerValue() const;

	/**
	 * Sets the size value.
	 */
	void setSizeValue(long size);

	/**
	 * Gets the value.
	 */
	long getSizeValue() const;

	/**
	 * Gets the value.
	 */
	virtual common::Array<ValueType> const & getValue() const;

	/**
	 * Gets the value.
	 */
	virtual common::Array<ValueType> & getValue();

private:
	common::ProxyArray<ValueType> _array;
	ValueType* _value;
	long _size;
};

}

#include "IcsArrayProperty.inl"

#endif /*DATAPROVIDER_ICSARRAYPROPERTY_H_*/