/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <common/base/Convert.h>

namespace dataprovider {

template<typename ValueType>
IcsArrayProperty<ValueType>::IcsArrayProperty(std::string const & name, std::string const & type, bool saveValue, const std::string & format) :
	BaseIcsArrayProperty(name, type, saveValue, format),
	_value(0),
	_size(0) {
}

template<typename ValueType>
IcsArrayProperty<ValueType>::IcsArrayProperty(const IcsArrayProperty<ValueType>& content) :
	BaseIcsArrayProperty(content),
	_value(0),
	_size(0) {
}

template<typename ValueType>
IcsArrayProperty<ValueType>::~IcsArrayProperty() {
}

template<typename ValueType>
DataElement * IcsArrayProperty<ValueType>::clone() const {
	return 0;
}

template<typename ValueType>
std::string IcsArrayProperty<ValueType>::toString() const {
	return std::string("property '") + getName() + "' (ics array <" + getType() + ">) of " + getParentString();
}

template<typename ValueType>
void IcsArrayProperty<ValueType>::setValueFromString(const std::vector<std::string>& stringValue) {
	_size = stringValue.size();
	_value = new ValueType[_size];

	int i = 0;
	for (std::vector<std::string>::const_iterator v = stringValue.begin(); v != stringValue.end(); v++) {
		_value[i] = common::to<ValueType>(*v);
		i++;
	}

	_array.setData(_value);
	_array.resize(_size);
}

template<typename ValueType>
std::string IcsArrayProperty<ValueType>::getStringValue(bool& overflow) const {

	std::string result = "[";

	if (_size > 1) {

		result += common::to<std::string>(_value[0]) + ", ";


		if (_size > 20) {

			for (long i = 1; i < 19; i++) {
				result += common::to<std::string>(_value[i]) + ", ";
			}

			result += "..., ";

		} else {
			for (long i = 1; i < _size - 1; i++) {
				result += common::to<std::string>(_value[i]) + ", ";
			}
		}
	}

	if (_size > 0) {
		result += common::to<std::string>(_value[_size - 1]);
	}

	result += "]";

	return result;
}

template<typename ValueType>
void IcsArrayProperty<ValueType>::setPointerValue(ValueType* value) {
	_value = value;
	_array.setData(_value);

}

template<typename ValueType>
ValueType* IcsArrayProperty<ValueType>::getPointerValue() const {
	return _value;
}

template<typename ValueType>
void IcsArrayProperty<ValueType>::setSizeValue(long size) {
	_size = size;
	_array.resize(_size);
}

template<typename ValueType>
long IcsArrayProperty<ValueType>::getSizeValue() const {
	return _size;
}

template<typename ValueType>
common::Array<ValueType> const & IcsArrayProperty<ValueType>::getValue() const {
	return _array;
}

template<typename ValueType>
common::Array<ValueType> & IcsArrayProperty<ValueType>::getValue() {
	return _array;
}

}
