/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PropertyHolder.h"
#include "BaseProperty.h"

using namespace std;
using namespace common;

namespace dataprovider {
	
PropertyHolder::PropertyHolder() :
	DataElement(),
 	_initialised(false) {
}

PropertyHolder::PropertyHolder(PropertyHolder const & content) :
	DataElement(content),
	_properties(content._properties),
 	_initialised(content._initialised) {
}
	
PropertyHolder::~PropertyHolder() {

	// No need to iterate the properties to delete them.
}

void PropertyHolder::addProperty(const std::string& propertyName, BaseProperty* property) {
	
	_properties.insert(propertyName, property->getID());
	property->setParentID(getID());
}

void PropertyHolder::removeProperty(const std::string& propertyName) {

	_properties.erase(propertyName);
}

unsigned long PropertyHolder::getPropertyID(const std::string& propertyName) const {
	
	if (_properties.find(propertyName)) {
		return _properties.get(propertyName);
	}
	return 0;
}

common::HashMap<std::string, unsigned long> const & PropertyHolder::getProperties() const {
	
	return _properties;
}

void PropertyHolder::setInitialised(bool initialised) {
	
	_initialised = initialised;
}
	
bool PropertyHolder::isInitialised() const {
	
	return _initialised;
}

}