/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_PROPERTYHOLDER_H_
#define DATAPROVIDER_PROPERTYHOLDER_H_

#include "DataElement.h"

#include <common/base/HashMap.h>
#include <string>
#include <set>

namespace dataprovider {

class BaseProperty;

/**
 * Class defining the property holder element in the Data Provider database.
 */
class PropertyHolder : public DataElement {

public:
	/**
	 * Constructor.
	 */
	PropertyHolder();
	
	/**
	 * Copy constructor.
	 */
	PropertyHolder(PropertyHolder const & content);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~PropertyHolder();
	
	/**
	 * Adds a Property.
	 */
	void addProperty(const std::string& propertyName, BaseProperty* property);

	/**
	 * Removes a Property.
	 */
	void removeProperty(const std::string& propertyName);

	/**
	 * Gets the Property ID.
	 */
	unsigned long getPropertyID(const std::string& propertyName) const;
	
	/**
	 * Gets the list of property IDs.
	 */
	common::HashMap<std::string, unsigned long> const & getProperties() const;
	
	/**
	 * Sets initialised.
	 */
	virtual void setInitialised(bool initialised);
	
	/**
	 * Returns true if it is initialised.
	 */
	bool isInitialised() const;

private:
	common::HashMap<std::string, unsigned long> _properties;
	bool _initialised;
};

}

#endif /*DATAPROVIDER_PROPERTYHOLDER_H_*/