/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef SERVANTICONANDFAMILYMAPS_H_
#define SERVANTICONANDFAMILYMAPS_H_

#include <map>
#include <string>

namespace dataprovider {

class ServantIconAndFamilyMaps {

public:
	static ServantIconAndFamilyMaps* getInstance();

	void readClientPluginFiles();
	void readFamilyImageKeys(const std::string& XMLFileName);

	std::string getControllerIconKey(const std::string& controllerType);
	std::string getFamilyImageKey(const std::string& familyName);
	const std::map<std::string, std::string>& getFamilyImageKeys() const;
	const std::map<std::string, std::string>& getControllerIconKeys() const;

	const static std::string TAG_XMLATTR;
	const static std::string TAG_CONTROLLER_CONFIG_PLUGIN;
	const static std::string TAG_FAMILY_CONFIG_PLUGIN;
	const static std::string TAG_FAMILY_IMAGES;
	const static std::string TAG_FAMILY;
	const static std::string TAG_IMAGE;

	const static std::string TYPE_ATTRIBUTE;
	const static std::string KEY_ATTRIBUTE;
	const static std::string NAME_ATTRIBUTE;
	const static std::string FAMILY_ATTRIBUTE;

private:
	ServantIconAndFamilyMaps();
	virtual ~ServantIconAndFamilyMaps();

	void readServantPluginFiles();
	void readFamilyPluginFiles();
	void readServantXMLFile(const std::string& XMLFileName);
	void readFamilyImagesXMLFile(const std::string& XMLFileName);
	void readFamilyPluginXMLFile(const std::string& XMLFileName);

	std::map<std::string, std::string> m_controllerIcons; //First string = controllerType, second string = controllerIconKey
	std::map<std::string, std::string> m_familyImages;

	static ServantIconAndFamilyMaps* m_instance;

};

}

#endif
