/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ControllerLog.h"
#include <dataprovider/database/Database.h>

namespace dataprovider {

ControllerLog::ControllerLog(common::LogType type, common::LogLevel level, const std::list<common::LogElement>& lineElements, const std::string & imageName, unsigned long commandID, unsigned long servantID) :
		Log(type, level, getOutputMessage(servantID, lineElements)),
		_commandID(commandID),
		_servantID(servantID),
		_lineElements(lineElements),
		_imageName(imageName) {

}

ControllerLog::~ControllerLog() {
}


unsigned long ControllerLog::getCommandID() const {
	return _commandID;
}

unsigned long ControllerLog::getServantID() const {
	return _servantID;
}

const std::list<common::LogElement>& ControllerLog::getLineElements() const {
	return _lineElements;
}

const std::string& ControllerLog::getImageName() const {
	return _imageName;
}


std::string ControllerLog::getOutputMessage(unsigned long servantID, std::list<common::LogElement> lineElements) {
	std::string outputMessage;
	//Add name to the output message, if servantID != 0
	if (servantID != 0) {
		std::string name = Database::getInstance()->getServantName(servantID);
		//Verify the name field is 15 characters long, if not truncate it!
		name.resize(15, ' ');
		outputMessage += name;

	} else {
		std::cerr << "Error related to controller log message: servant ID is 0!" << std::endl;
	}

	for (std::list<common::LogElement>::const_iterator it = lineElements.begin(); it != lineElements.end(); it++) {
		common::LogElement logElement = *it;

		if (!logElement.isHidden()) {
			outputMessage += logElement.getValue() + logElement.getUnit();
		}
	}
	return outputMessage;
}


}

