/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DirectLogPublisher.h"
#include <common/output/OutputStream.h>
#include <iostream>

using namespace AmqpClient;
using namespace std;

namespace dataprovider {

const std::string EXCHANGE_NAME = "weasel";

DirectLogPublisher::DirectLogPublisher(const std::string& host) :
	m_host(host),
	m_connected(false) {
}

DirectLogPublisher::~DirectLogPublisher() {
}

void DirectLogPublisher::init() {

	// Try to connect if not already connected.
	if (!m_connected) {
		try {
			m_channel = Channel::Create(m_host, 5672, "nomad", "nomad");
			m_channel->DeclareExchange(EXCHANGE_NAME, Channel::EXCHANGE_TYPE_FANOUT, true, false);

			// The channel is now connected.
			m_connected = true;

		} catch (const std::exception& e) {
			common::err << "Cannot connect to the RabbitMQ server : " << e.what() << common::eol;
		}
	}
}

void DirectLogPublisher::publish(const std::string& jsonMessage) {

	// Initialise the connection if not connected.
	init();

	// Publish the message if connected.
	if (m_connected) {

		// Create the message.
		BasicMessage::ptr_t message = BasicMessage::Create(jsonMessage);

		try {
			if (!m_channel->BasicPublish(EXCHANGE_NAME, "", message, false, false, 1000)) {
				common::err << "Timeout while publishing to the RabbitMQ server" << common::eol;
			}

		} catch (const std::exception& e) {
			common::err << "Cannot publish the log message to the RabbitMQ server : " << e.what() << common::eol;

			// The channel is now disconnected.
			m_connected = false;
		}
	}
}

}

