/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_SERVERLOGHANDLER_H_
#define DATAPROVIDER_SERVERLOGHANDLER_H_

#include <common/base/CommonTypes.h>
#include <thread>

namespace common {

template<typename ItemType>
class Queue;

}

namespace dataprovider {

class Message;
class WriteServerLogMessage;
class ShutdownMessage;

/**
 * Class defining a server log handler which processes server log messages coming from the different application components.
 * The writeLog method is not blocking.
 */
class ServerLogHandler {

public:
	/**
	 * Gets the instance.
	 */
	static ServerLogHandler* getInstance();
	
	/**
	 * Initialises the instance.
	 */
	static void initInstance();
	
	/**
	 * Resets the instance.
	 */
	static void resetInstance();
	
	/**
	 * Writes the log line.
	 */
	void writeLog(const std::string& processName, unsigned long threadID, common::OutputType type, const std::string& line);
		
private:
	ServerLogHandler();
	~ServerLogHandler();
	
	void init();
	void reset();
	
	void processMessage(Message* message);
	void processMessage(WriteServerLogMessage* message);
	void processMessage(ShutdownMessage* message);
	
	static void loop();

	static ServerLogHandler * _instance;
	
	bool _running;
	common::Queue<Message>* _messageQueue;
	std::thread* _thread;
	
	const static std::string SERVER_LOG_PATH;
};

}

#endif
