/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_TEXTLOGWRITER_H_
#define DATAPROVIDER_TEXTLOGWRITER_H_

#include <string>
#include "LogWriter.h"
#include "Log.h"

namespace common {

class Date;

}

namespace dataprovider {

class ControllerLog;
class CommandLog;
class SimpleLog;
class CommandZoneContentLog;
class CommandZoneEventLog;
class CommandZoneParser;
class CommandZoneElement;

/**
 * Class defining a text log writer.
 */
class TextLogWriter : public LogWriter {

public:
	/**
	 * Constructor.
	 */
	TextLogWriter();

	/**
	 * Sets the output name.
	 */
	void setOutputName(std::string const &name);

	/**
	 * Virtual destructor.
	 */
	virtual ~TextLogWriter();

	/**
	 * Writes the log entry.
	 */
	virtual void write(Log* log, int n = 0);

private:
	std::string format(Log* log);
	std::string format(ControllerLog* log);
	std::string format(CommandLog* log);
	std::string format(SimpleLog* log);
	std::string format(CommandZoneContentLog* log);
	std::string format(CommandZoneEventLog* log);
	std::string formatOtherTypeOfMessages(Log* log);

	std::string createLevelString(common::LogLevel level);

	bool isASimpleTextLogMessage(Log* log);

	static std::string formatColumn(const std::string& value, const std::string& format = DATA_FORMAT);
	std::string formatCommandZoneContent(CommandZoneElement* content);

	const static std::string LOG_PATH;
	const static std::string DATE_FORMAT;
	const static std::string LEVEL_FORMAT;
	const static std::string DATA_FORMAT;

	std::string _outputName;
	CommandZoneParser* _parser;
};


}

#endif /*DATAPROVIDER_TEXTLOGWRITER_H_*/
