/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_XMLLOGWRITER_H_
#define DATAPROVIDER_XMLLOGWRITER_H_

#include <string>
#include "LogWriter.h"
#include "Log.h"

namespace common {

class Date;

}

namespace dataprovider {

class ControllerLog;
class CommandLog;
class SimpleLog;
class CommandZoneContentLog;
class CommandZoneEventLog;
class CommandZoneParser;
class CommandZoneElement;
class AtomicElement;
class GenericElement;
class ScanElement;
class ForLoopElement;
class CommandZone;
class Start;
class IfElement;
class BreakElement;

/**
 * Class defining an XML log writer.
 */
class XMLLogWriter : public LogWriter {

public:
	/**
	 * Constructor.
	 */
	XMLLogWriter();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~XMLLogWriter();

	/**
	 * Writes the log entry.
	 */
	virtual void write(Log* log, int n = 0);

private:
	std::string format(Log* log, int n);
	std::string format(ControllerLog* log, int n);
	std::string format(CommandLog* log);
	std::string format(SimpleLog* log);
	std::string format(CommandZoneContentLog* log);
	std::string format(CommandZoneEventLog* log);
	std::string formatOtherTypeOfMessages(Log* log);

	std::string createLogHeader(const common::Date& date);
	std::string createNumorStart(const std::string& time, const std::string& numor);
	std::string createNumorEnd(const std::string& time);
	std::string createLevelString(common::LogLevel level);
	
	std::string getServantName(unsigned long servantID) const;
	std::string getServantType(unsigned long servantID) const;
	
	bool isASimpleTextLogMessage(Log* log);

	std::string formatCommandZoneContent(CommandZoneElement* content, const std::string& indent = "\t");
	std::string formatCommandZoneContent(AtomicElement* element, const std::string& indent);
	std::string formatCommandZoneContent(GenericElement* element, const std::string& indent);
	std::string formatCommandZoneContent(ScanElement* element, const std::string& indent);
	std::string formatCommandZoneContent(ForLoopElement* element, const std::string& indent);
	std::string formatCommandZoneContent(CommandZone* element, const std::string& indent);
	std::string formatCommandZoneContent(Start* element, const std::string& indent);
	std::string formatCommandZoneContent(IfElement* element, const std::string& indent);
	std::string formatCommandZoneContent(BreakElement* element, const std::string& indent);
	
	const static std::string LOG_PATH;
	
	CommandZoneParser* _parser;
};


}

#endif /*DATAPROVIDER_XMLLOGWRITER_H_*/
