/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_WRITELOGMESSAGE_H_
#define DATAPROVIDER_WRITELOGMESSAGE_H_

#include <dataprovider/message/Message.h>

namespace dataprovider {

class LogCompletionHandler;
class Log;


/**
 * Class defining a write log Message.
 */
class WriteLogMessage : public Message {

public:
	/**
	 * Constructor.
	 */
	WriteLogMessage(Log* log, unsigned long requestID, LogCompletionHandler* logCompletionHandler);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~WriteLogMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the log.
	 */
	Log* getLog() const;
	
	/**
	 * Gets the request ID.
	 */
	unsigned long getRequestID() const;
	
	/**
	 * Gets the LogCompletionHandler.
	 */
	LogCompletionHandler* getLogCompletionHandler() const;

private:
	Log* _log;
	unsigned long _requestID;
	LogCompletionHandler* _logCompletionHandler;
};

}

#endif /*DATAPROVIDER_WRITELOGMESSAGE_H_*/