/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_WRITESERVERLOGMESSAGE_H_
#define DATAPROVIDER_WRITESERVERLOGMESSAGE_H_

#include <dataprovider/message/Message.h>
#include <common/base/CommonTypes.h>

namespace dataprovider {

/**
 * Class defining a write server log Message.
 */
class WriteServerLogMessage : public Message {

public:
	/**
	 * Constructor.
	 */
	WriteServerLogMessage(const std::string& processName, unsigned long threadID, common::OutputType type, const std::string& line);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~WriteServerLogMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the process name.
	 */
	const std::string& getProcessName() const;
	
	/**
	 * Returns the thread ID.
	 */
	unsigned long getThreadID() const;
	
	/**
	 * Returns the log type.
	 */
	common::OutputType getLogType() const;
	
	/**
	 * Returns the line.
	 */
	const std::string& getLine() const;
	
private:
	std::string _processName;
	unsigned long _threadID;
	common::OutputType _logType;
	std::string _line;
};

}

#endif /*DATAPROVIDER_WRITESERVERLOGMESSAGE_H_*/