/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_CHATMESSAGE_H_
#define DATAPROVIDER_CHATMESSAGE_H_

#include "NotificationMessage.h"
#include <common/base/CommonTypes.h>
#include <common/base/BaseTypes.h>

namespace dataprovider {

/**
 * Class defining a chat message.
 */
class ChatMessage : public NotificationMessage {

public:
	/**
	 * Constructor.
	 */
	ChatMessage(uint32 clientID, int64 date, const std::string& sender, const std::string& message, const std::string& fileName);

	/**
	 * Virtual destructor.
	 */
	virtual ~ChatMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the client ID.
	 */
	uint32 getClientID() const;
	
	/**
	 * Gets the date.
	 */
	int64 getDate() const;

	/**
	 * Gets the sender.
	 */
	std::string getSender() const;

	/**
	 * Gets the message.
	 */
	std::string getMessage() const;

	/**
	 * Gets the fileName.
	 */
	std::string getFileName() const;

private:
	uint32 m_clientID;
	int64 m_date;
	std::string m_sender;
	std::string m_message;
	std::string m_fileName;

};


}

#endif
