/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_COMMANDINTERRUPTCHANGEDMESSAGE_H_
#define DATAPROVIDER_COMMANDINTERRUPTCHANGEDMESSAGE_H_

#include "NotificationMessage.h"
#include <dataprovider/dataelement/Command.h>

namespace dataprovider {

/**
 * Class defining a command state changed message.
 */
class CommandInterruptChangedMessage : public NotificationMessage {

public:
	/**
	 * Constructor.
	 */
	CommandInterruptChangedMessage(unsigned long databaseID, unsigned long servantID, unsigned long commandID, bool value);

	/**
	 * Virtual destructor.
	 */
	virtual ~CommandInterruptChangedMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the servant ID.
	 */
	unsigned long getServantID() const;

	/**
	 * Gets the command ID.
	 */
	unsigned long getCommandID() const;

	/**
	 * Gets the command interrupt.
	 */
	bool getValue() const;

private:
	unsigned long _servantID;
	unsigned long _commandID;
	bool _value;
};

} // namespace dataprovider

#endif /*DATAPROVIDER_COMMANDINTERRUPTCHANGEDMESSAGE_H_*/