/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_CONDITIONSTATECHANGEDMESSAGE_H_
#define DATAPROVIDER_CONDITIONSTATECHANGEDMESSAGE_H_

#include "NotificationMessage.h"
#include <common/base/CommonTypes.h>

namespace dataprovider {

/**
 * Class defining a property changed message.
 */
class ConditionStateChangedMessage : public NotificationMessage {

public:
	/**
	 * Constructor.
	 */
	ConditionStateChangedMessage(unsigned long conditionId, bool on, common::ConditionState state);

	/**
	 * Virtual destructor.
	 */
	virtual ~ConditionStateChangedMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the id.
	 */
	unsigned long getConditionId() const;

	/**
	 * Returns true if is on.
	 */
	bool isOn() const;

	/**
	 * Returns the state.
	 */
	common::ConditionState getState() const;

private:
	unsigned long _conditionId;
	bool _on;
	common::ConditionState _state;
};


} // namespace dataprovider

#endif /*DATAPROVIDER_CONDITIONSTATECHANGEDMESSAGE_H_*/
