/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_LOGMESSAGE_H_
#define DATAPROVIDER_LOGMESSAGE_H_

#include <dataprovider/log/Log.h>
#include "NotificationMessage.h"

namespace dataprovider {

/**
 * Class defining a log message.
 */
class LogMessage : public NotificationMessage {

public:
	/**
	 * Constructor.
	 */
	LogMessage(common::LogType type, common::LogLevel level, const std::string& message, unsigned long commandID, unsigned long servantID);

	/**
	 * Virtual destructor.
	 */
	virtual ~LogMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the log type.
	 */
	common::LogType getLogType() const;

	/**
	 * Gets the log level.
	 */
	common::LogLevel getLogLevel() const;
	
	/**
	 * Gets the message.
	 */
	const std::string& getMessage() const;

	/**
	 * Gets the command ID.
	 */
	unsigned long getCommandID() const;

	/**
	 * Gets the servant ID.
	 */
	unsigned long getServantID() const;

private:
	common::LogType _logType;
	common::LogLevel _logLevel;
	std::string _message;
	unsigned long _commandID;
	unsigned long _servantID;
};


} // namespace dataprovider

#endif
