/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_NEWFILECREATEDEVENT_H_
#define DATAPROVIDER_NEWFILECREATEDEVENT_H_

#include "NotificationMessage.h"
#include <common/base/CommonTypes.h>

namespace dataprovider {

/**
 * Class defining a new file created event.
 */
class NewFileCreatedMessage : public NotificationMessage {
	
public:
	/**
	 * Constructor.
	 */
	NewFileCreatedMessage(common::FileType fileType, const std::string& fileName);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~NewFileCreatedMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Returns the file type.
	 */
	common::FileType getFileType() const;
	
	/**
	 * Returns the file name.
	 */
	const std::string& getFileName() const;
	
private:
	common::FileType _fileType;
	std::string _fileName;
};

}

#endif
