/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_NOTIFICATIONMESSAGE_H_
#define DATAPROVIDER_NOTIFICATIONMESSAGE_H_

#include <string>

namespace dataprovider {

/**
 * Base class defining a notification message.
 */
class NotificationMessage {

public:
	typedef enum {
		PROPERTY_CHANGED,
		COMMAND_STATE_CHANGED,
		COMMAND_INTERRUPT_CHANGED,
		COMMAND_PROGRESSION_CHANGED,
		CONFIGURATION_CHANGED,
		ICS_PROPERTY_CHANGED,
		LOG,
		LOG_XML,
		SURVEY_XML,
		LOG_DATAREADY,
		ENABLE_CONFIGURATION_CHANGED,
		CONDITION_STATE_CHANGED,
		CONDITION_ACTIONS_STATE_CHANGED,
		USER_VARIABLE_VALUE_CHANGED,
		USER_VARIABLE_STATE_CHANGED,
		NEW_FILE,
		TOKEN_STATE_CHANGED,
		CHAT_MESSAGE,
		CONNECTED_USERS_CHANGED,
		NEW_FILE_CREATED} Type;

	/**
	 * Constructor.
	 */
	NotificationMessage(unsigned long databaseID);

	/**
	 * Virtual destructor.
	 */
	virtual ~NotificationMessage();

	/**
	 * Gets the type.
	 */
	virtual Type getType() const = 0;

	/**
	 * Gets the database ID.
	 */
	unsigned long getDatabaseID() const;

	/**
	 * Gets the type string.
	 */
	std::string getTypeString() const;

private :
	unsigned long _databaseID;

};

}

#endif /*DATAPROVIDER_DATANOTIFICATIONMESSAGE_H_*/
