/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "TokenStateChangedMessage.h"

namespace dataprovider {


TokenStateChangedMessage::TokenStateChangedMessage(common::CriticalState criticalState, common::RemoteControlState remoteControlState, common::TokenState state, uint32 ownerClientID, const std::string& ownerEndpoint, const std::string& user) :
	NotificationMessage(0),
	m_criticalState(criticalState),
	m_remoteControlState(remoteControlState),
	m_state(state),
	m_ownerClientID(ownerClientID),
	m_ownerEndpoint(ownerEndpoint),
	m_user(user) {
}

TokenStateChangedMessage::~TokenStateChangedMessage() {
}

NotificationMessage::Type TokenStateChangedMessage::getType() const {
	return NotificationMessage::TOKEN_STATE_CHANGED;
}

common::CriticalState TokenStateChangedMessage::getCriticalState() const {
	return m_criticalState;
}

common::RemoteControlState TokenStateChangedMessage::getRemoteControlState() const {
	return m_remoteControlState;
}

common::TokenState TokenStateChangedMessage::getState() const {
	return m_state;
}

uint32 TokenStateChangedMessage::getOwnerClientID() const {
	return m_ownerClientID;
}

const std::string& TokenStateChangedMessage::getOwnerEndpoint() const {
	return m_ownerEndpoint;
}

const std::string& TokenStateChangedMessage::getUser() const {
	return m_user;
}

}
