/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_TOKENSTATECHANGEDMESSAGE_H_
#define DATAPROVIDER_TOKENSTATECHANGEDMESSAGE_H_

#include "NotificationMessage.h"
#include <common/base/CommonTypes.h>
#include <common/base/BaseTypes.h>

namespace dataprovider {

/**
 * Class defining a token state changed message.
 */
class TokenStateChangedMessage : public NotificationMessage {

public:
	/**
	 * Constructor.
	 */
	TokenStateChangedMessage(common::CriticalState criticalState, common::RemoteControlState remoteControlState, common::TokenState state, uint32 ownerClientID, const std::string& ownerEndpoint, const std::string& user);

	/**
	 * Virtual destructor.
	 */
	virtual ~TokenStateChangedMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;

	/**
	 * Gets the critical state.
	 */
	common::CriticalState getCriticalState() const;
	
	/**
	 * Gets the remote control state.
	 */
	common::RemoteControlState getRemoteControlState() const;

	/**
	 * Gets the property ID.
	 */
	common::TokenState getState() const;

	/**
	 * Gets the owner client ID.
	 */
	uint32 getOwnerClientID() const;

	/**
	 * Gets the owner endpoint.
	 */
	const std::string& getOwnerEndpoint() const;

	/**
	 * Gets the user.
	 */
	const std::string& getUser() const;

private:
	common::CriticalState m_criticalState;
	common::RemoteControlState m_remoteControlState;
	common::TokenState m_state;
	uint32 m_ownerClientID;
	std::string m_ownerEndpoint;
	std::string m_user;
};


}

#endif
