/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_USERVARIABLESTATECHANGEDMESSAGE_H_
#define DATAPROVIDER_USERVARIABLESTATECHANGEDMESSAGE_H_

#include "NotificationMessage.h"
#include <common/base/CommonTypes.h>

namespace dataprovider {

/**
 * Class defining a user variable state changed message.
 */
class UserVariableStateChangedMessage : public NotificationMessage {

public:
	/**
	 * Constructor.
	 */
	UserVariableStateChangedMessage(unsigned long databaseId, const std::string& name, bool enabled);

	/**
	 * Virtual destructor.
	 */
	virtual ~UserVariableStateChangedMessage();
	
	/**
	 * Gets the type.
	 */
	virtual Type getType() const;
	
	/**
	 * Gets the name.
	 */
	const std::string& getName() const;

	/**
	 * Gets the value.
	 */
	bool isEnabled() const;

private:
	std::string _name;
	bool _isEnabled;
};


}

#endif

