/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "LogPublisher.h"
//#include <common/protobuf/generated/NotificationMessages.pb.h>
#include <NotificationMessages.pb.h>
#include <common/output/OutputStream.h>
#include <vector>
#include <iostream>

using namespace std;
using namespace common;
using namespace cameo;

namespace dataprovider {

LogPublisher::LogPublisher(const std::string& serverId) {

	string logPublisherName = "log_publisher";

	// Create the publisher.
	m_publisher = coms::Publisher::create(logPublisherName);
	m_publisher->init();

	cout << "Created log publisher " << logPublisherName << endl;
}

LogPublisher::~LogPublisher() {

	cout << "Deleted log publisher" << endl;
}

void LogPublisher::notifyLog(common::LogType type, common::LogLevel level, const std::string& message, unsigned long commandID, unsigned long servantID) {

	//out << "notifying log " << (int)level << " " << commandID << " " << message << eol;

	// Create the type.
	notification::Message messageType;
	messageType.set_type(notification::Message::Log);

	// Create and send the message.
	notification::Log logMessage;

	if (type == common::START) {
		logMessage.set_type(notification::Log::START);
	}
	else if (type == common::END) {
		logMessage.set_type(notification::Log::END);
	}
	else if (type == common::ACQ_START) {
		logMessage.set_type(notification::Log::ACQ_START);
	}
	else if (type == common::ACQ_END) {
		logMessage.set_type(notification::Log::ACQ_END);
	}
	else if (type == common::SINGLE) {
		logMessage.set_type(notification::Log::SINGLE);
	}

	if (level == common::LOG_INFO) {
		logMessage.set_level(notification::Log::INFO);
	}
	else if (level == common::LOG_WARNING) {
		logMessage.set_level(notification::Log::WARNING);
	}
	else if (level == common::LOG_ERROR) {
		logMessage.set_level(notification::Log::ERROR);
	}
	else if (level == common::LOG_DEBUG) {
		logMessage.set_level(notification::Log::DEBUG);
	}
	else if (level == common::LOG_SYNC) {
		logMessage.set_level(notification::Log::SYNC);
	}

	logMessage.set_message(message);
	logMessage.set_commandid(commandID);
	logMessage.set_servantid(servantID);

	// Publish the message
	m_publisher->sendTwoParts(messageType.SerializeAsString(), logMessage.SerializeAsString());
}

void LogPublisher::notifyLogXML(const std::string& message) {

	// Create the type.
	notification::Message messageType;
	messageType.set_type(notification::Message::LogXML);

	// Create and send the message.
	notification::MessageXML logMessage;

	logMessage.set_message(message);

	// Publish the message
	m_publisher->sendTwoParts(messageType.SerializeAsString(), logMessage.SerializeAsString());
}

void LogPublisher::notifyLogImageDataReady(const std::string& image) {

	// Create the type.
	notification::Message messageType;
	messageType.set_type(notification::Message::ImageDataReady);

	// Publish the message
	m_publisher->sendTwoParts(messageType.SerializeAsString(), image);
}

}
