/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_LOGPUBLISHER_H_
#define DATAPROVIDER_LOGPUBLISHER_H_

#include <dataprovider/dataelement/Command.h>
#include <common/base/CommonTypes.h>
#include <cameo/api/cameo.h>

namespace dataprovider {

/**
 * Class defining a log publisher which delivers notifications.
 */
class LogPublisher {

public:
	/**
	 * Constructor.
	 */
	LogPublisher(const std::string& serverId);

	/**
	 * Virtual destructor.
	 */
	virtual ~LogPublisher();

	/**
	 * Notifies the subscribers of a log.
	 */
	void notifyLog(common::LogType type, common::LogLevel level, const std::string& message, unsigned long commandID, unsigned long servantID);

	/**
	 * Notifies the subscribers of a xml log.
	 */
	void notifyLogXML(const std::string& message);

	/**
	 * Notifies the subscribers of a log which has an image associated.
	 */
	void notifyLogImageDataReady(const std::string& image);

private:
	std::unique_ptr<cameo::coms::Publisher> m_publisher;
};

}

#endif /*DATAPROVIDER_LOGPUBLISHER_H_*/
