/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_COMMANDZONEPARSER_H_
#define DATAPROVIDER_COMMANDZONEPARSER_H_

#include <common/parser/GenericParser.h>
#include <vector>
#include <list>

namespace dataprovider {

class CommandZoneElement;

/**
 * Class defining an XML parser for the command zone xml content.
 */
class CommandZoneParser : public common::GenericParser {

public:
	/**
	 * Constructor.
	 */
	CommandZoneParser();

	/**
	 * Virtual destructor.
	 */
	virtual ~CommandZoneParser();

	/**
	 * Gets the content.
	 */
	CommandZoneElement* getContent() const;

protected:
	const static std::string ZONE_TAG;
	const static std::string START_TAG;

	const static std::string SCAN_TAG;
	const static std::string ATOMIC_TAG;
	const static std::string FOR_LOOP_TAG;
	const static std::string FOR_LOOP_LINE_TAG;

	const static std::string FIX_TAG;
	const static std::string CLEAR_TAG;
	const static std::string SET_TAG;
	const static std::string SET_ZERO_TAG;

	const static std::string IF_TAG;
	const static std::string THEN_TAG;
	const static std::string ELSE_TAG;
	const static std::string BREAK_TAG;

	const static std::string NAME_ATTRIBUTE;
	const static std::string ID_ATTRIBUTE;
	const static std::string LOG_ID_ATTRIBUTE;
	const static std::string COMMAND_ID_ATTRIBUTE;
	const static std::string CONTROLLER_ID_ATTRIBUTE;
	const static std::string IS_SETTINGS_ATTRIBUTE;
	const static std::string SETTINGS_FILE_NAME_ATTRIBUTE;

	const static std::string VARIABLE_NAME_ATTRIBUTE;
	const static std::string TYPE_ATTRIBUTE;
	const static std::string START_VALUE_ATTRIBUTE;
	const static std::string END_VALUE_ATTRIBUTE;
	const static std::string STEP_ATTRIBUTE;
	const static std::string VALUES_ATTRIBUTE;
	const static std::string CONDITION_ATTRIBUTE;

	virtual void startElement(const std::string& qName);
	virtual void endElement(const std::string& qName);

private:
	std::list<CommandZoneElement*> _currentElementStack;
	CommandZoneElement* _root;

};

}

#endif /*DATAPROVIDER_COMMANDZONEPARSER_H_*/
