/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_ATOMICELEMENT_H_
#define DATAPROVIDER_ATOMICELEMENT_H_

#include "CommandZoneElement.h"

namespace dataprovider {

/**
 * Class defining an atomic element for the command zone parser.
 */
class AtomicElement : public CommandZoneElement {

public:
	/**
	 * Constructor.
	 */
	AtomicElement(unsigned long ID, const std::string& logID, unsigned long commandID, bool isSettings, const std::string& settingsFileName);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~AtomicElement();
	
	/**
	 * Gets the command ID.
	 */
	unsigned long getCommandID() const;
	
	/**
	 * Returns true if the element is a settings element.
	 */
	bool isSettings() const;
	
	/**
	 * Returns the settings file name.
	 */
	std::string getSettingsFileName() const;

private:
	unsigned long _commandID;
	bool _isSettings;
	std::string _settingsFileName;
};

}

#endif /*DATAPROVIDER_ATOMICELEMENT_H_*/
