/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_COMMANDZONEELEMENT_H_
#define DATAPROVIDER_COMMANDZONEELEMENT_H_

#include <vector>
#include <string>

namespace dataprovider {

/**
 * Class defining a command zone element for the command zone parser.
 */
class CommandZoneElement {

public:
	/**
	 * Constructor.
	 */
	CommandZoneElement(unsigned long ID, const std::string& logID);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~CommandZoneElement();
	
	/**
	 * Gets the ID.
	 */
	unsigned long getID() const;
	
	/**
	 * Gets the log ID.
	 */
	const std::string& getLogID() const;
	
	/**
	 * Adds the command zone element child.
	 */
	void addChild(CommandZoneElement* child);
	
	/**
	 * Gets the children.
	 */
	const std::vector<CommandZoneElement*>& getChildren() const;
	
private:
	unsigned long _ID;
	std::string _logID;

	std::vector<CommandZoneElement*> _children;
};

}

#endif /*DATAPROVIDER_COMMANDZONEELEMENT_H_*/