/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_FORLOOPLINEELEMENT_H_
#define DATAPROVIDER_FORLOOPLINEELEMENT_H_

#include "CommandZoneElement.h"
#include <string>

namespace dataprovider {

/**
 * Class defining a for loop line element for the command zone parser.
 */
class ForLoopLineElement : public CommandZoneElement {

public:
	/**
	 * Constructor.
	 */
	ForLoopLineElement(const std::string& variableName, const std::string& type, const std::string& startValue, const std::string& endValue, const std::string& step, const std::string& values);

	/**
	 * Virtual destructor.
	 */
	virtual ~ForLoopLineElement();

	/**
	 * Gets the variable name.
	 */
	const std::string& getVariableName() const;

	/**
	 * Gets the type.
	 */
	const std::string& getType() const;

	/**
	 * Gets the start value.
	 */
	const std::string& getStartValue() const;

	/**
	 * Gets the end value.
	 */
	const std::string& getEndValue() const;

	/**
	 * Gets the step value.
	 */
	const std::string& getStep() const;

	/**
	 * Gets the values.
	 */
	const std::string& getValues() const;

private:
	std::string _variableName;
	std::string _type;
	std::string _startValue;
	std::string _endValue;
	std::string _step;
	std::string _values;
};

}

#endif /*DATAPROVIDER_FORLOOPLINEELEMENT_H_*/