/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ControllerElement.h"
#include <iostream>

using namespace std;

namespace dataprovider {

ControllerElement::ControllerElement(const std::string& name, const std::string& type, bool isCommand, bool isVisible, bool isEnabled, bool isPausable) :
	ServantElement(name, type, isEnabled, isPausable),
	_isCommand(isCommand),
	_isVisible(isVisible) {

}

ControllerElement::~ControllerElement() {
}

void ControllerElement::addServantList(const std::string& n_name) {

	if (!_servants.find(n_name)) {
		
		pair<bool, vector<ControllerChild> > item;
		item.first = true;
		
		for (vector<ControllerChild>::iterator c = item.second.begin(); c != item.second.end(); c++) {
			c->isNull = true;
		}
		
		_servants.insert(n_name, item);
	}
}

void ControllerElement::addServantToList(const std::string& n_name, bool isController, const std::string& name, long position) {

	vector<ControllerChild> & servants = _servants[n_name].second;
	
	if (position >= (long)servants.size()) {
		servants.resize(position + 1);
	}
	
	servants[position].isNull = false;
	servants[position].isController = isController;
	servants[position].name = name;

}
	
void ControllerElement::addServant(const std::string& n_name, bool isController, const std::string& name) {

	if (!_servants.find(n_name)) {
		
		pair<bool, vector<ControllerChild> > item;
		item.first = false;
		
		_servants.insert(n_name, item);
	} else {
		// exception
		throw ChildNameException(getName(), n_name, name);
	}
	
	vector<ControllerChild> & servants = _servants[n_name].second;
	servants.resize(1);
	servants[0].isNull = false;
	servants[0].isController = isController;
	servants[0].name = name;
}

const common::HashMap<std::string, std::pair<bool, std::vector<ControllerElement::ControllerChild> > >& ControllerElement::getServants() const {
	return _servants;
}

bool ControllerElement::isCommand() const {
	return _isCommand;
}

bool ControllerElement::isVisible() const {
	return _isVisible;
}

void ControllerElement::setFamily(const std::string& family) {
	_family = family;
}

const std::string& ControllerElement::getFamily() const {
	return _family;
}

}
