/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_CONTROLLERELEMENT_H_
#define DATAPROVIDER_CONTROLLERELEMENT_H_

#include "ServantElement.h"
#include <common/base/HashMap.h>

namespace dataprovider {

/**
 * Class defining a Controller element for the ConfigurationParser. 
 */
class ControllerElement : public ServantElement {

public:
	/**
	 * Constructor with the name and the type.
	 */
	ControllerElement(const std::string& name, const std::string& type, bool isCommand, bool isVisible, bool isEnabled, bool isPausable);
	
	/**
	 * Virtual destructor.
	 */	
	virtual ~ControllerElement();

	/**
	 * Adds a servant list.
	 */
	void addServantList(const std::string& n_name);

	/**
	 * Adds a servant.
	 */
	void addServantToList(const std::string& n_name, bool isController, const std::string& name, long position);
	
	/**
	 * Adds a servant to the list.
	 */
	void addServant(const std::string& n_name, bool isController, const std::string& name);

	/**
	 * Controller child
	 */
	struct ControllerChild {
		bool isNull;
		bool isController;
		std::string name;
	};
	
	/**
	 * Gets the servants.
	 */
	const common::HashMap<std::string, std::pair<bool, std::vector<ControllerChild> > >& getServants() const;

	/**
	 * returns true if is a command.
	 */
	bool isCommand() const;
	
	/**
	 * returns true if is visible.
	 */
	bool isVisible() const;
	
	/**
	 * Sets the family
	 */
	void setFamily(const std::string& family);
	
	/**
	 * Returns the family.
	 */
	const std::string& getFamily() const;

private:
	common::HashMap<std::string, std::pair<bool, std::vector<ControllerChild> > > _servants;
	bool _isCommand;
	bool _isVisible;
	std::string _family;
};

}

#endif /*DATAPROVIDER_CONTROLLERELEMENT_H_*/
