/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DriverElement.h"
#include <iostream>

using namespace std;

namespace dataprovider {

DriverElement::DriverElement(const std::string& name, const std::string& type, bool isEnabled, bool isPausable) :
	ServantElement(name, type, isEnabled, isPausable) {
}

DriverElement::~DriverElement() {
}

void DriverElement::addChildList(std::string const & name) {
	
	if (!_children.find(name)) {
		
		pair<bool, vector<list<DriverElement*> > > item;
		item.first = true;
		_children.insert(name, item);
	}
}

void DriverElement::addChildToList(std::string const & name, DriverElement* driver, long channel) {
	
	vector<list<DriverElement*> > & drivers = _children[name].second;

	if (channel >= (long)drivers.size()) {
		drivers.resize(channel + 1);
	}	

	drivers[channel].push_back(driver);
}

void DriverElement::addChild(std::string const & name, DriverElement* driver) {
	
	if (!_children.find(name)) {
		
		pair<bool, vector<list<DriverElement*> > > item;
		item.first = false;
		
		_children.insert(name, item);
	}
	
	vector<list<DriverElement*> > & drivers = _children[name].second;
	drivers.resize(1);
	drivers[0].push_back(driver);
}
	
common::HashMap<std::string, std::pair<bool, std::vector<std::list<DriverElement*> > > > const & DriverElement::getChildren() const {
	return _children;
}

void DriverElement::setMode(common::DriverMode mode) {
	_mode = mode;
}

common::DriverMode DriverElement::getMode() const {
	return _mode;
}

}
