/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_DRIVERELEMENT_H_
#define DATAPROVIDER_DRIVERELEMENT_H_

#include "ServantElement.h"
#include <common/base/HashMap.h>
#include <common/base/CommonTypes.h>
#include <vector>
#include <list>

namespace dataprovider {

class ProtocolElement;

/**
 * Class defining a Driver element for the ConfigurationParser. 
 */
class DriverElement : public ServantElement {

public:
	/**
	 * Constructor with the name and the type.
	 */
	DriverElement(const std::string& name, const std::string& type, bool isEnabled, bool isPausable);
	
	/**
	 * Virtual destructor.
	 */	
	virtual ~DriverElement();

	/**
	 * Adds a child driver name list.
	 */
	void addChildList(std::string const & name);

	/**
	 * Adds a child driver name to the list at channel channel.
	 */
	void addChildToList(std::string const & name, DriverElement* driver, long channel);
	
	/**
	 * Adds a child driver name.
	 */
	void addChild(std::string const & name, DriverElement* driver);
	
	/**
	 * Gets the children.
	 */
	common::HashMap<std::string, std::pair<bool, std::vector<std::list<DriverElement*> > > > const & getChildren() const;
	
	/**
	 * Sets the simulated state.
	 */
	void setMode(common::DriverMode mode);

	/**
	 * Returns true if is simulated.
	 */
	common::DriverMode getMode() const;
	
private:
	common::HashMap<std::string, std::pair<bool, std::vector<std::list<DriverElement*> > > > _children;
	common::DriverMode _mode;
};

}

#endif /*DATAPROVIDER_DRIVERELEMENT_H_*/
