/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DynamicPropertyElement.h"
#include "PropertyContentElement.h"
#include <iostream>

using namespace std;

namespace dataprovider {

DynamicPropertyElement::DynamicPropertyElement(const std::string& type, const std::string& name, bool saveValue, std::string const & format) :
	PropertyElement(name),
	_saveValue(saveValue),
	_format(format) {
	
	if (type.size() >= 2) {
		_isArray = (type.substr(type.size() - 2) == "[]");
		if (_isArray) {
			_type = type.substr(0, type.size() - 2);
		} else {
			_type = type;
		}	
	}
}

DynamicPropertyElement::~DynamicPropertyElement() {
	
	for (vector<PropertyContentElement*>::const_iterator c = _content.begin(); c != _content.end(); c++) {
		delete *c;
	}
}

const std::string& DynamicPropertyElement::getType() const {
	return _type;
}

bool DynamicPropertyElement::isArray() const {
	return _isArray;
}

bool DynamicPropertyElement::saveValue() const {
	return _saveValue;		
}

std::string const & DynamicPropertyElement::getFormat() const {
	
	return _format;
}

void DynamicPropertyElement::addContent(const std::string& contentString, bool saveValue) {

	// add only non-virtual arrays to list properties
	_content.push_back(new PropertyContentElement(_type, contentString, _isArray, saveValue, _format));
}

const std::vector<PropertyContentElement*>& DynamicPropertyElement::getContent() const {
	return _content;
}

}
