/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_DYNAMICPROPERTYELEMENT_H_
#define DATAPROVIDER_DYNAMICPROPERTYELEMENT_H_

#include "PropertyElement.h"
#include <dataprovider/dataelement/DataElement.h>
#include <string>
#include <vector>

namespace dataprovider {

class PropertyContentElement;

/**
 * Class defining a Property vector element for the ConfigurationParser.
 */
class DynamicPropertyElement : public PropertyElement {

public:
	/**
	 * Constructor.
	 */
	DynamicPropertyElement(const std::string& type, const std::string& name, bool saveValue, std::string const & format);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DynamicPropertyElement();
	
	/**
	 * Gets the type.
	 */
	const std::string& getType() const;
	
	/**
	 * Return true if contains arrays.
	 */
	bool isArray() const;

	/**
	 * Returns true if saves the value.
	 */
	bool saveValue() const;
	
	/**
	 * Gets the format.
	 */
	std::string const & getFormat() const;
	
	/**
	 * Add a new PropertyContentElement content.
	 */
	void addContent(const std::string& contentString, bool saveValue);
	
	/**
	 * Gets the content.
	 */
	const std::vector<PropertyContentElement*>& getContent() const;

private:
	std::string _type;
	std::vector<PropertyContentElement*> _content;
	bool _isArray;
	bool _saveValue;
	std::string _format;
};

}

#endif /*DATAPROVIDER_DYNAMICPROPERTYELEMENT_H_*/