/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_PROPERTYCONTENTELEMENT_H_
#define DATAPROVIDER_PROPERTYCONTENTELEMENT_H_

#include <dataprovider/dataelement/DataElement.h>
#include <string>
#include <vector>

namespace dataprovider {

/**
 * Class defining a Property content element for the ConfigurationParser.
 */
class PropertyContentElement {

public:
	/**
	 * Constructor for arrays with type and value.
	 */
	PropertyContentElement(const std::string& type, const std::string& value, bool isArray, bool saveValue, std::string const & format);
	
	/**
	 * Virtual destructor.
	 */
	virtual ~PropertyContentElement();

	/**
	 * Gets the type.
	 */
	const std::string& getType() const;
	
	/**
	 * Returns true if is an array.
	 */
	bool isArray() const;

	/**
	 * Returns true if saves the value.
	 */
	bool saveValue() const;

	/**
	 * Gets the format.
	 */
	const std::string & getFormat() const;

	/**
	 * Gets the value.
	 */
	const std::string& getValue() const;

private:
	std::string _type;
	bool _isArray;
	std::string _value;
	bool _saveValue;
	std::string _format;
};

}

#endif /*DATAPROVIDER_PROPERTYCONTENTELEMENT_H_*/
