/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ServantElement.h"
#include "PropertyElement.h"
#include "CommandElement.h"
#include <iostream>

using namespace std;
using namespace common;

namespace dataprovider {

ServantElement::ChildPositionException::ChildPositionException() :
	Exception("dataprovider::ServantElement::ChildPositionException", string("Error in position of child")) {
}

ServantElement::ChildNameException::ChildNameException(const std::string parentName, const std::string nName, const std::string childName) :
	Exception("dataprovider::ServantElement::ChildNameException", string("Error in name of child for ") + parentName + " with child nName " + nName + " with servant name " + childName) {
}

ServantElement::ServantElement(const std::string& name, const std::string& type, bool isEnabled, bool isPausable) :
	_serverType(NOMAD),
	_name(name),
	_type(type),
	_isEnabled(isEnabled),
	_isPausable(isPausable) {

}

ServantElement::~ServantElement() {

	for (vector<PropertyElement*>::iterator p = _properties.begin(); p != _properties.end(); p++) {
		delete (*p);
	}
	
	for (vector<CommandElement*>::iterator c = _commands.begin(); c != _commands.end(); c++) {
		delete (*c);
	}
}

void ServantElement::setServerType(common::ServerType serverType) {
	_serverType = serverType;
}

common::ServerType ServantElement::getServerType() const {
	return _serverType;
}

const std::string& ServantElement::getName() const {
	return _name;
}

const std::string& ServantElement::getType() const {
	return _type;
}

void ServantElement::addProperty(PropertyElement* property) {
	_properties.push_back(property);
}

const std::vector<PropertyElement*>& ServantElement::getProperties() const {
	return _properties;
}

void ServantElement::addCommand(CommandElement* command) {
	_commands.push_back(command);
}

const std::vector<CommandElement*>& ServantElement::getCommands() const {
	return _commands;
}

bool ServantElement::isEnabled() const {
	return _isEnabled;
}

bool ServantElement::isPausable() const {
	return _isPausable;
}

}