/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_DRIVERCOMPLETIONHANDLER_H_
#define DATAPROVIDER_DRIVERCOMPLETIONHANDLER_H_

#include <string>

#include <common/base/Exception.h>

namespace dataprovider {

/**
 * Abstract class defining the driver completion handler.
 */
class DriverCompletionHandler {
	
public:
	/**
	 * Constructor
	 */
	DriverCompletionHandler();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~DriverCompletionHandler();
		
	/**
	 * Notifies a success.
	 */
	virtual void success(unsigned long completionID, bool propertyChanged) = 0;
		
	/**
	 * Notifies a failure.
	 */
	virtual void failure(unsigned long completionID, bool timeout, int timeoutMs, const std::string& message) = 0;
	
};

}

#endif /*DATAPROVIDER_DRIVERCOMPLETIONHANDLER_H_*/