/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CommandAccessorDirectImpl.h"
#include <dataprovider/transportlayer/common/DirectDataChangeSubscriber.h>
#include <common/output/OutputStream.h>
#include <iostream>

using namespace std;
using namespace common;
using namespace boost;

namespace dataprovider {

CommandAccessorDirectImpl::NoSuchServantException::NoSuchServantException()
	: common::Exception("common::dataprovider::NoSuchServantException", "No such servant") {}

CommandAccessorDirectImpl::NoSuchCommandException::NoSuchCommandException()
	: common::Exception("common::dataprovider::NoSuchCommandException", "No such command") {}

CommandAccessorDirectImpl::CommandAccessorDirectImpl() {
}

CommandAccessorDirectImpl::~CommandAccessorDirectImpl() {
}

std::vector<common::CommandDescriptor> CommandAccessorDirectImpl::getControllerCommands(unsigned long databaseID, unsigned long controllerID) {
	try {
		return createCommandList(Database::getInstance(databaseID)->getControllerCommands(controllerID));
	} catch (Database::NoSuchServantException& e) {
		throw NoSuchServantException();
	}
}

std::vector<common::CommandDescriptor> CommandAccessorDirectImpl::getDriverCommands(unsigned long databaseID, unsigned long driverID) {
	try {
		return createCommandList(Database::getInstance(databaseID)->getDriverCommands(driverID));
	} catch (Database::NoSuchServantException& e) {
		throw NoSuchServantException();
	}
}

unsigned long CommandAccessorDirectImpl::getServantID(unsigned long databaseID, unsigned long commandID) {

	try {
		return Database::getInstance(databaseID)->getServantID(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}

}

unsigned long CommandAccessorDirectImpl::getCommandID(unsigned long databaseID, unsigned long servantID, const std::string& commandName) {

	try {
		return Database::getInstance(databaseID)->getCommandID(servantID, commandName);

	} catch (Database::NoSuchCommandException& e) {
		cout << "NoSuchCommandException" << endl;
		throw NoSuchCommandException();
	}
}

std::string CommandAccessorDirectImpl::getCommandName(unsigned long databaseID, unsigned long commandID) {

	try {
		return Database::getInstance(databaseID)->getCommandName(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}


std::vector<common::CommandDescriptor> CommandAccessorDirectImpl::createCommandList(const Database::CommandMap& commandMap) {

	vector<CommandDescriptor> result;

	// fill up the list iterator with CommandDescriptors
	for (Database::CommandMap::const_iterator it = commandMap.begin(); it != commandMap.end(); it++) {
		// it->first = name, it->second = id
		result.push_back(CommandDescriptor(it->second, it->first));
	}

	return result;

}

void CommandAccessorDirectImpl::setCommandState(unsigned long databaseID, unsigned long servantID, unsigned long commandID, common::CommandState state) {

	try {
		Database::getInstance(databaseID)->setCommandState(servantID, commandID, state);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

CommandAccessorDirectImpl::ActiveResult CommandAccessorDirectImpl::setCommandActive(unsigned long databaseID, unsigned long servantID, unsigned long commandID, const std::string& servantType, unsigned long processorID, unsigned long threadID, common::DataChangeSubscriberDirect* subscriber) {

	try {
		return (ActiveResult)Database::getInstance(databaseID)->setCommandActive(servantID, commandID, servantType, processorID, threadID, new DirectDataChangeSubscriber(subscriber));

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

common::CommandState CommandAccessorDirectImpl::getCommandState(unsigned long databaseID, unsigned long commandID) {

	try {
		return Database::getInstance(databaseID)->getCommandState(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

double CommandAccessorDirectImpl::getCommandElapsedTimeMs(unsigned long databaseID, unsigned long commandID) {

	try {
		return Database::getInstance(databaseID)->getCommandElapsedTimeMs(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}

}

unsigned long CommandAccessorDirectImpl::addCommandToServant(unsigned long databaseID, unsigned long servantID, const std::string& commandName) {

	try {
		return Database::getInstance(databaseID)->addCommandToServant(servantID, commandName);

	} catch (Database::NoSuchServantException& e) {
		throw NoSuchServantException();
	}
}

unsigned long CommandAccessorDirectImpl::getServantIDFromCommandID(unsigned long databaseID, unsigned long commandID) {

	try {
		return Database::getInstance(databaseID)->getServantIDFromCommandID(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

unsigned long CommandAccessorDirectImpl::getSisterCommandID(unsigned long databaseID, unsigned long commandID, const std::string& commandName) {

	try {
		unsigned long servantID = Database::getInstance(databaseID)->getServantID(commandID);

		return Database::getInstance(databaseID)->getCommandID(servantID, commandName);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

bool CommandAccessorDirectImpl::isCommandFromIcsServant(unsigned long databaseID, unsigned long commandID) {

	try {
		unsigned long servantID = Database::getInstance(databaseID)->getServantID(commandID);

		return Database::getInstance(databaseID)->isIcsServant(servantID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

void CommandAccessorDirectImpl::setProgression(unsigned long databaseID, unsigned long commandID, double progression) {

	try {
		Database::getInstance(databaseID)->setProgression(commandID, progression);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}

}

void CommandAccessorDirectImpl::setUnknownProgression(unsigned long databaseID, unsigned long commandID) {

	try {
		Database::getInstance(databaseID)->setUnknownProgression(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

double CommandAccessorDirectImpl::getProgression(unsigned long databaseID, unsigned long commandID) {

	try {
		return Database::getInstance(databaseID)->getProgression(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

double CommandAccessorDirectImpl::getCommandRemainingTimeMs(unsigned long databaseID, unsigned long commandID) {

	try {
		return Database::getInstance(databaseID)->getCommandRemainingTimeMs(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

double CommandAccessorDirectImpl::getCommandDurationMs(unsigned long databaseID, unsigned long commandID) {

	try {
		return Database::getInstance(databaseID)->getCommandDurationMs(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

int64 CommandAccessorDirectImpl::getCommandStartDate(unsigned long databaseID, unsigned long commandID) {

	try {
		return Database::getInstance(databaseID)->getCommandStartDate(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

void CommandAccessorDirectImpl::setCommandPreRunCalculatedDurationMs(unsigned long databaseID, unsigned long commandID, double duration) {

	try {
		Database::getInstance(databaseID)->setCommandPreRunCalculatedDurationMs(commandID, duration);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

double CommandAccessorDirectImpl::getCommandPreRunCalculatedDurationMs(unsigned long databaseID, unsigned long commandID) {

	try {
		return Database::getInstance(databaseID)->getCommandPreRunCalculatedDurationMs(commandID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

void CommandAccessorDirectImpl::setCommandLogID(unsigned long databaseID, unsigned long commandID, const std::string& logID) {

	try {
		Database::getInstance(databaseID)->setCommandLogID(commandID, logID);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

void CommandAccessorDirectImpl::setCommandFromCommandZone(unsigned long databaseID, unsigned long commandID, bool fromCommandZone) {

	try {
		Database::getInstance(databaseID)->setCommandFromCommandZone(commandID, fromCommandZone);

	} catch (Database::NoSuchCommandException& e) {
		throw NoSuchCommandException();
	}
}

bool CommandAccessorDirectImpl::commandExists(unsigned long databaseID, unsigned long commandID) {

	return Database::getInstance(databaseID)->isCommand(commandID);
}

std::string CommandAccessorDirectImpl::what(unsigned long databaseID, unsigned long elementID) {

	return Database::getInstance(databaseID)->what(elementID, false);
}

void CommandAccessorDirectImpl::reportBug(int64 date) {

	Database::getInstance()->reportBug(date);
}

}
