/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_LOGACCESSORDIRECTIMPL_H_
#define DATAPROVIDER_LOGACCESSORDIRECTIMPL_H_

#include <dataprovider/log/Log.h>
#include <list>
#include "common/base/LogElement.h"

namespace command {
	class LogCompletionHandlerDirectImpl;
}

namespace dataprovider {

class LogAccessorDirectImpl {
	
public:
	LogAccessorDirectImpl();
	virtual ~LogAccessorDirectImpl();

	void writeCommandLog(common::LogType type, common::LogLevel level, const std::string& logMessage, const std::string& logID);
	void writeSimpleLog(common::LogLevel level, const std::string& logMessage, bool writeLogFile);
	void writeCommandZoneContentLog(const std::string& content);
	void writeCommandZoneEventLog(const std::string& content);

	/**
	 * New Log message types
	 */
	void writeVariableChangeLog(common::LogLevel level, const std::string& variableName, const std::string& variableType, const std::string& variableValue, const std::string& message, const unsigned long messageParentId);
	void writeForLoopStartLog(common::LogLevel level, const std::string& variableName, const std::string& message, unsigned long messageId);
	void writeForLoopEndLog(common::LogLevel level, const std::string& message, unsigned long messageId);
	void writeIfEvaluationLog(common::LogLevel level, const std::string& name, const std::string& type, const std::string& value, const std::string& message);
	void writeBreakEvaluationLog(common::LogLevel level, const std::string& name, const std::string& type, const std::string& value, const std::string& message, const unsigned long messageParentId);
	void writeUserScanStartLog(common::LogLevel level, const std::string& name, const std::string& message);

	void writeNLog(common::LogType type, common::LogLevel level, const std::list<common::LogElement>& lineElements, const std::string & imageName,unsigned long commandID, unsigned long servantID, bool writeLogFile, unsigned long requestID, command::LogCompletionHandlerDirectImpl* completionHandler);
	void imageDataReadyLog(const std::string& image);

	void writeServerLog(const std::string& processName, unsigned long threadID, bool error, const std::string& line);
};

}

#endif /*DATAPROVIDER_LOGACCESSORIMPL_H_*/
