/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PropertyAccessorDirectImpl.h"
#include <common/base/PropertyType.h>
#include <iostream>

using namespace std;
using namespace common;
using namespace boost;

namespace dataprovider {

PropertyAccessorDirectImpl::PropertyAccessorDirectImpl() {
}

PropertyAccessorDirectImpl::~PropertyAccessorDirectImpl() {
}

std::vector<common::PropertyDescriptor> PropertyAccessorDirectImpl::getControllerProperties(unsigned long databaseID, unsigned long controllerID) {

	try {
		return createPropertyList(Database::getInstance(databaseID)->getControllerProperties(controllerID));
	} catch (Database::NoSuchServantException& e) {
		throw NoSuchServantException();
	}
}

std::vector<common::PropertyDescriptor> PropertyAccessorDirectImpl::getDriverProperties(unsigned long databaseID, unsigned long driverID) {

	try {
		return createPropertyList(Database::getInstance(databaseID)->getDriverProperties(driverID));
	} catch (Database::NoSuchServantException& e) {
		throw NoSuchServantException();
	}
}

unsigned long PropertyAccessorDirectImpl::addPropertyToServant(unsigned long databaseID, unsigned long servantID, const std::string& propertyName, const std::string& propertyType, bool spy, bool saveBox, bool & added) {
	try {
		return Database::getInstance(databaseID)->addPropertyToServant(servantID, propertyName, propertyType, spy, saveBox, added);

	} catch (Database::NoSuchServantException& e) {
		throw NoSuchServantException();
	}
}

unsigned long PropertyAccessorDirectImpl::addDynamicPropertyToServant(unsigned long databaseID, unsigned long servantID, const std::string& propertyName, const std::string& propertyType, bool spy) {
	try {
		return Database::getInstance(databaseID)->addDynamicPropertyToServant(servantID, propertyName, propertyType, spy);

	} catch (Database::NoSuchServantException& e) {
		throw NoSuchServantException();
	}
}

void PropertyAccessorDirectImpl::removePropertyFromServant(unsigned long databaseID, unsigned long servantID, unsigned long propertyID) {

	try {
		Database::getInstance(databaseID)->removePropertyFromServant(servantID, propertyID);

	} catch (Database::NoSuchServantException& e) {
		throw NoSuchServantException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setPropertySettings(unsigned long databaseID, unsigned long propertyID, const std::string& propertyFormat, bool save, bool spy) {

	try {
		Database::getInstance(databaseID)->setPropertySettings(propertyID, propertyFormat, save, spy);

	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::resizeDynamicProperty(unsigned long databaseID, unsigned long propertyID, int32 size) {

	try {
		Database::getInstance(databaseID)->resizeDynamicProperty(propertyID, size);

	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

long PropertyAccessorDirectImpl::getDynamicPropertySize(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getDynamicPropertySize(propertyID);

	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

unsigned long PropertyAccessorDirectImpl::getDynamicPropertyIDAtIndex(unsigned long databaseID, unsigned long propertyID, int32 index, bool resize) {

	try {
		return Database::getInstance(databaseID)->getDynamicPropertyIDAtIndex(propertyID, index, resize);

	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::IndexOutOfBoundsException & e) {
		throw IndexOutOfBoundsException();
	}
}

int32 PropertyAccessorDirectImpl::getIndexOfDynamicPropertyID(unsigned long databaseID, unsigned long parentPropertyID, unsigned long propertyID) {
	return Database::getInstance(databaseID)->getIndexOfDynamicPropertyID(parentPropertyID, propertyID);
}

std::string PropertyAccessorDirectImpl::getPropertyType(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getPropertyType(propertyID);

	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

std::string PropertyAccessorDirectImpl::getPropertyName(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getPropertyName(propertyID);

	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

std::pair<std::string, long> PropertyAccessorDirectImpl::getCompletePropertyName(unsigned long databaseID, unsigned long servantID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getCompletePropertyName(servantID, propertyID);

	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::propertyExists(unsigned long databaseID, unsigned long propertyID) {

	return Database::getInstance(databaseID)->isProperty(propertyID);
}

unsigned long PropertyAccessorDirectImpl::getServantIDFromPropertyID(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getServantIDFromPropertyID(propertyID);

	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

unsigned long PropertyAccessorDirectImpl::getPropertyID(unsigned long databaseID, unsigned long servantID, const std::string& propertyName) {

	try {
		return Database::getInstance(databaseID)->getPropertyID(servantID, propertyName);

	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int8 PropertyAccessorDirectImpl::getInt8Value(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<int8>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

int16 PropertyAccessorDirectImpl::getInt16Value(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<int16>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

uint16 PropertyAccessorDirectImpl::getUint16Value(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<uint16>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

int32 PropertyAccessorDirectImpl::getInt32Value(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<int32>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

uint32 PropertyAccessorDirectImpl::getUint32Value(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<uint32>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

int64 PropertyAccessorDirectImpl::getInt64Value(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<int64>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

uint64 PropertyAccessorDirectImpl::getUint64Value(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<uint64>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

float32 PropertyAccessorDirectImpl::getFloat32Value(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<float32>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

float64 PropertyAccessorDirectImpl::getFloat64Value(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<float64>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

std::string PropertyAccessorDirectImpl::getStringValue(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<string>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

std::vector<int8> PropertyAccessorDirectImpl::getBinaryStringValue(unsigned long databaseID, unsigned long propertyID) {

	try {
		vector<int8> result;

		// get the string value
		string value = Database::getInstance(databaseID)->getValue<string>(propertyID);

		for (unsigned long i = 0; i < value.size(); ++i) {
			result[i] = value[i];
		}

		return result;

	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

bool PropertyAccessorDirectImpl::getBooleanValue(unsigned long databaseID, unsigned long propertyID) {

	try {
		return Database::getInstance(databaseID)->getValue<bool>(propertyID);
	} catch (Database::NullPropertyReferenceException& e) {
		throw NullPropertyReferenceException();
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		err << e.printStack() << eol;
		throw BadPropertyTypeException();
	}
}

std::vector<common::PropertyDescriptor> PropertyAccessorDirectImpl::createPropertyList(const Database::PropertyMap& propertyMap) {

	vector<common::PropertyDescriptor> result;

	// fill up the list iterator with PropertyDescriptors
	for (Database::PropertyMap::const_iterator it = propertyMap.begin(); it != propertyMap.end(); it++) {
		result.push_back(common::PropertyDescriptor(it->second.IDs, it->first, it->second.propertyType, (it->second.itemType == Database::PropertyItem::DYNAMIC), it->second.isSpy));
	}

	return result;
}

bool PropertyAccessorDirectImpl::setIcsBooleanValue(unsigned long databaseID, unsigned long propertyID, bool value) {
	try {
		return Database::getInstance(databaseID)->setIcsValue<bool>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsBooleanValueAndFormattedValue(unsigned long databaseID, unsigned long propertyID, bool value, const std::string& formattedValue) {
	try {
		return Database::getInstance(databaseID)->setIcsValueAndFormattedValue<bool>(propertyID, value, formattedValue);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::getIcsBooleanValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getValue<bool>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsInt16Value(unsigned long databaseID, unsigned long propertyID, int16 value) {
	try {
		return Database::getInstance(databaseID)->setIcsValue<int16>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsInt16ValueAndFormattedValue(unsigned long databaseID, unsigned long propertyID, int16 value, const std::string& formattedValue) {
	try {
		return Database::getInstance(databaseID)->setIcsValueAndFormattedValue<int16>(propertyID, value, formattedValue);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int16 PropertyAccessorDirectImpl::getIcsInt16Value(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getValue<int16>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsUint16Value(unsigned long databaseID, unsigned long propertyID, uint16 value) {
	try {
		return Database::getInstance(databaseID)->setIcsValue<uint16>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsUint16ValueAndFormattedValue(unsigned long databaseID, unsigned long propertyID, uint16 value, const std::string& formattedValue) {
	try {
		return Database::getInstance(databaseID)->setIcsValueAndFormattedValue<uint16>(propertyID, value, formattedValue);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

uint16 PropertyAccessorDirectImpl::getIcsUint16Value(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getValue<uint16>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsInt32Value(unsigned long databaseID, unsigned long propertyID, int32 value) {
	try {
		return Database::getInstance(databaseID)->setIcsValue<int32>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsInt32ValueAndFormattedValue(unsigned long databaseID, unsigned long propertyID, int32 value, const std::string& formattedValue) {
	try {
		return Database::getInstance(databaseID)->setIcsValueAndFormattedValue<int32>(propertyID, value, formattedValue);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32 PropertyAccessorDirectImpl::getIcsInt32Value(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getValue<int32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsUint32Value(unsigned long databaseID, unsigned long propertyID, uint32 value) {
	try {
		return Database::getInstance(databaseID)->setIcsValue<uint32>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsUint32ValueAndFormattedValue(unsigned long databaseID, unsigned long propertyID, uint32 value, const std::string& formattedValue) {
	try {
		return Database::getInstance(databaseID)->setIcsValueAndFormattedValue<uint32>(propertyID, value, formattedValue);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

uint32 PropertyAccessorDirectImpl::getIcsUint32Value(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getValue<uint32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsInt64Value(unsigned long databaseID, unsigned long propertyID, int64 value) {
	try {
		return Database::getInstance(databaseID)->setIcsValue<int64>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsInt64ValueAndFormattedValue(unsigned long databaseID, unsigned long propertyID, int64 value, const std::string& formattedValue) {
	try {
		return Database::getInstance(databaseID)->setIcsValueAndFormattedValue<int64>(propertyID, value, formattedValue);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int64 PropertyAccessorDirectImpl::getIcsInt64Value(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getValue<int64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsUint64Value(unsigned long databaseID, unsigned long propertyID, uint64 value) {
	try {
		return Database::getInstance(databaseID)->setIcsValue<uint64>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsUint64ValueAndFormattedValue(unsigned long databaseID, unsigned long propertyID, uint64 value, const std::string& formattedValue) {
	try {
		return Database::getInstance(databaseID)->setIcsValueAndFormattedValue<uint64>(propertyID, value, formattedValue);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

uint64 PropertyAccessorDirectImpl::getIcsUint64Value(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getValue<uint64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsFloat32Value(unsigned long databaseID, unsigned long propertyID, float32 value) {
	try {
		return Database::getInstance(databaseID)->setIcsValue<float32>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsFloat32ValueAndFormattedValue(unsigned long databaseID, unsigned long propertyID, float32 value, const std::string& formattedValue) {
	try {
		return Database::getInstance(databaseID)->setIcsValueAndFormattedValue<float32>(propertyID, value, formattedValue);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

float32 PropertyAccessorDirectImpl::getIcsFloat32Value(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getValue<float32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsFloat64Value(unsigned long databaseID, unsigned long propertyID, float64 value) {
	try {
		return Database::getInstance(databaseID)->setIcsValue<float64>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsFloat64ValueAndFormattedValue(unsigned long databaseID, unsigned long propertyID, float64 value, const std::string& formattedValue) {
	try {
		return Database::getInstance(databaseID)->setIcsValueAndFormattedValue<float64>(propertyID, value, formattedValue);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

float64 PropertyAccessorDirectImpl::getIcsFloat64Value(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getValue<float64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsStringValue(unsigned long databaseID, unsigned long propertyID, const std::string& value) {
	try {
		return Database::getInstance(databaseID)->setIcsValue<string>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::setIcsStringValueAndFormattedValue(unsigned long databaseID, unsigned long propertyID, const std::string& value, const std::string& formattedValue) {
	try {
		return Database::getInstance(databaseID)->setIcsValueAndFormattedValue<string>(propertyID, value, formattedValue);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

std::string PropertyAccessorDirectImpl::getIcsStringValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getValue<string>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsInt16ArrayValue(unsigned long databaseID, unsigned long propertyID, int16* value) {
	try {
		Database::getInstance(databaseID)->setIcsArrayValue<int16>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int16* PropertyAccessorDirectImpl::getIcsInt16ArrayValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArrayValue<int16>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsUint16ArrayValue(unsigned long databaseID, unsigned long propertyID, uint16* value) {
	try {
		Database::getInstance(databaseID)->setIcsArrayValue<uint16>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

uint16* PropertyAccessorDirectImpl::getIcsUint16ArrayValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArrayValue<uint16>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsInt32ArrayValue(unsigned long databaseID, unsigned long propertyID, int32* value) {
	try {
		Database::getInstance(databaseID)->setIcsArrayValue<int32>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32* PropertyAccessorDirectImpl::getIcsInt32ArrayValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArrayValue<int32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsUint32ArrayValue(unsigned long databaseID, unsigned long propertyID, uint32* value) {
	try {
		Database::getInstance(databaseID)->setIcsArrayValue<uint32>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

uint32* PropertyAccessorDirectImpl::getIcsUint32ArrayValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArrayValue<uint32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsInt64ArrayValue(unsigned long databaseID, unsigned long propertyID, int64* value) {
	try {
		Database::getInstance(databaseID)->setIcsArrayValue<int64>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int64* PropertyAccessorDirectImpl::getIcsInt64ArrayValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArrayValue<int64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsUint64ArrayValue(unsigned long databaseID, unsigned long propertyID, uint64* value) {
	try {
		Database::getInstance(databaseID)->setIcsArrayValue<uint64>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

uint64* PropertyAccessorDirectImpl::getIcsUint64ArrayValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArrayValue<uint64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsFloat32ArrayValue(unsigned long databaseID, unsigned long propertyID, float32* value) {
	try {
		Database::getInstance(databaseID)->setIcsArrayValue<float32>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

float32* PropertyAccessorDirectImpl::getIcsFloat32ArrayValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArrayValue<float32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsFloat64ArrayValue(unsigned long databaseID, unsigned long propertyID, float64* value) {
	try {
		Database::getInstance(databaseID)->setIcsArrayValue<float64>(propertyID, value);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

float64* PropertyAccessorDirectImpl::getIcsFloat64ArrayValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArrayValue<float64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsInt16ArraySize(unsigned long databaseID, unsigned long propertyID, int32 size) {
	try {
		Database::getInstance(databaseID)->setIcsArraySize<int16>(propertyID, size);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32 PropertyAccessorDirectImpl::getIcsInt16ArraySize(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<int16>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsUint16ArraySize(unsigned long databaseID, unsigned long propertyID, int32 size) {
	try {
		Database::getInstance(databaseID)->setIcsArraySize<uint16>(propertyID, size);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32 PropertyAccessorDirectImpl::getIcsUint16ArraySize(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<uint16>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsInt32ArraySize(unsigned long databaseID, unsigned long propertyID, int32 size) {
	try {
		Database::getInstance(databaseID)->setIcsArraySize<int32>(propertyID, size);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32 PropertyAccessorDirectImpl::getIcsInt32ArraySize(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<int32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsUint32ArraySize(unsigned long databaseID, unsigned long propertyID, int32 size) {
	try {
		Database::getInstance(databaseID)->setIcsArraySize<uint32>(propertyID, size);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32 PropertyAccessorDirectImpl::getIcsUint32ArraySize(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<uint32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsInt64ArraySize(unsigned long databaseID, unsigned long propertyID, int32 size) {
	try {
		Database::getInstance(databaseID)->setIcsArraySize<int64>(propertyID, size);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32 PropertyAccessorDirectImpl::getIcsInt64ArraySize(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<int64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsUint64ArraySize(unsigned long databaseID, unsigned long propertyID, int32 size) {
	try {
		Database::getInstance(databaseID)->setIcsArraySize<uint64>(propertyID, size);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32 PropertyAccessorDirectImpl::getIcsUint64ArraySize(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<uint64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsFloat32ArraySize(unsigned long databaseID, unsigned long propertyID, int32 size) {
	try {
		Database::getInstance(databaseID)->setIcsArraySize<float32>(propertyID, size);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32 PropertyAccessorDirectImpl::getIcsFloat32ArraySize(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<float32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::setIcsFloat64ArraySize(unsigned long databaseID, unsigned long propertyID, int32 size) {
	try {
		Database::getInstance(databaseID)->setIcsArraySize<float64>(propertyID, size);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32 PropertyAccessorDirectImpl::getIcsFloat64ArraySize(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<float64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::getIcsInt16ArrayValue(unsigned long databaseID, unsigned long propertyID, std::vector<int16> & array) {
	try {
		Database::getInstance(databaseID)->getIcsArrayValue<int16>(propertyID, array);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::getIcsUint16ArrayValue(unsigned long databaseID, unsigned long propertyID, std::vector<uint16> & array) {
	try {
		Database::getInstance(databaseID)->getIcsArrayValue<uint16>(propertyID, array);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::getIcsInt32ArrayValue(unsigned long databaseID, unsigned long propertyID, std::vector<int32> & array) {
	try {
		Database::getInstance(databaseID)->getIcsArrayValue<int32>(propertyID, array);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::getIcsUint32ArrayValue(unsigned long databaseID, unsigned long propertyID, std::vector<uint32> & array) {
	try {
		Database::getInstance(databaseID)->getIcsArrayValue<uint32>(propertyID, array);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::getIcsInt64ArrayValue(unsigned long databaseID, unsigned long propertyID, std::vector<int64> & array) {
	try {
		Database::getInstance(databaseID)->getIcsArrayValue<int64>(propertyID, array);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::getIcsUint64ArrayValue(unsigned long databaseID, unsigned long propertyID, std::vector<uint64> & array) {
	try {
		Database::getInstance(databaseID)->getIcsArrayValue<uint64>(propertyID, array);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::getIcsFloat32ArrayValue(unsigned long databaseID, unsigned long propertyID, std::vector<float32> & array) {
	try {
		Database::getInstance(databaseID)->getIcsArrayValue<float32>(propertyID, array);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::getIcsFloat64ArrayValue(unsigned long databaseID, unsigned long propertyID, std::vector<float64> & array) {
	try {
		Database::getInstance(databaseID)->getIcsArrayValue<float64>(propertyID, array);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

int32 PropertyAccessorDirectImpl::getArraySize(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<float32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		// try with int32
		//throw NoSuchPropertyException();
	}
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<float64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		// try with int32
		//throw NoSuchPropertyException();
	}
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<int16>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
//		throw NoSuchPropertyException();
	}
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<int32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
//		throw NoSuchPropertyException();
	}
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<int64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
//		throw NoSuchPropertyException();
	}
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<uint16>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
//		throw NoSuchPropertyException();
	}
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<uint32>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
//		throw NoSuchPropertyException();
	}
	try {
		return Database::getInstance(databaseID)->getIcsArraySize<uint64>(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	} catch (Database::BadPropertyTypeException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::getEncodedInt16ArrayValue(unsigned long databaseID, unsigned long propertyID, std::string & encodedString) {

	Database::getInstance(databaseID)->getEncodedArrayValue<int16>(propertyID, encodedString);
}

void PropertyAccessorDirectImpl::getEncodedUint16ArrayValue(unsigned long databaseID, unsigned long propertyID, std::string & encodedString) {

	Database::getInstance(databaseID)->getEncodedArrayValue<uint16>(propertyID, encodedString);
}

void PropertyAccessorDirectImpl::getEncodedInt32ArrayValue(unsigned long databaseID, unsigned long propertyID, std::string & encodedString) {

	Database::getInstance(databaseID)->getEncodedArrayValue<int32>(propertyID, encodedString);
}

void PropertyAccessorDirectImpl::getEncodedUint32ArrayValue(unsigned long databaseID, unsigned long propertyID, std::string & encodedString) {

	Database::getInstance(databaseID)->getEncodedArrayValue<uint32>(propertyID, encodedString);
}

void PropertyAccessorDirectImpl::getEncodedInt64ArrayValue(unsigned long databaseID, unsigned long propertyID, std::string & encodedString) {

	Database::getInstance(databaseID)->getEncodedArrayValue<int64>(propertyID, encodedString);
}

void PropertyAccessorDirectImpl::getEncodedUint64ArrayValue(unsigned long databaseID, unsigned long propertyID, std::string & encodedString) {

	Database::getInstance(databaseID)->getEncodedArrayValue<uint64>(propertyID, encodedString);
}

void PropertyAccessorDirectImpl::getEncodedFloat32ArrayValue(unsigned long databaseID, unsigned long propertyID, std::string & encodedString) {

	Database::getInstance(databaseID)->getEncodedArrayValue<float32>(propertyID, encodedString);
}

void PropertyAccessorDirectImpl::getEncodedFloat64ArrayValue(unsigned long databaseID, unsigned long propertyID, std::string & encodedString) {

	Database::getInstance(databaseID)->getEncodedArrayValue<float64>(propertyID, encodedString);
}

std::string PropertyAccessorDirectImpl::getFormattedValue(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->getFormattedValue(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

void PropertyAccessorDirectImpl::updateIcsProperty(unsigned long databaseID, unsigned long propertyID) {
	try {
		return Database::getInstance(databaseID)->updateIcsProperty(propertyID);
	} catch (Database::NoSuchPropertyException& e) {
		throw NoSuchPropertyException();
	}
}

bool PropertyAccessorDirectImpl::isScannableProperty(unsigned long databaseID, const std::string & controllerType, const std::string & wantedPropertyName) const {
	return Database::getInstance(databaseID)->isScannableProperty(controllerType, wantedPropertyName);
}

Database::PropertyInfoItem PropertyAccessorDirectImpl::getPropertyInfoItem(unsigned long databaseID, const std::string & controllerType, const std::string & wantedPropertyName) const {
	try {
		return Database::getInstance(databaseID)->getSetpointPropertyInfoItem(controllerType, wantedPropertyName);
	} catch (Database::NoPropertyInfo& e) {
		throw NoSuchPropertyException();
	}
}

std::vector<Database::PropertyInfoItem> PropertyAccessorDirectImpl::getPropertyInfoItems(unsigned long databaseID, const std::string & controllerType) const {
	return Database::getInstance(databaseID)->getPropertyInfoItems(controllerType);
}

std::vector<Database::PropertyInfoItem> PropertyAccessorDirectImpl::getAllPropertyInfoItems(unsigned long databaseID, const std::string & controllerType) const {
	return Database::getInstance(databaseID)->getAllPropertyInfoItems(controllerType);
}

void PropertyAccessorDirectImpl::addPropertyInfoItem(unsigned long databaseID, const std::string & controllerType, const std::string & wantedPropertyName, const std::string & actualPropertyName, const std::string & offsetPropertyName, const std::string & shortPropertyName, bool isScan, bool isSpy, bool isDynamic) {
	Database::getInstance(databaseID)->addPropertyInfoItem(controllerType, wantedPropertyName, actualPropertyName, offsetPropertyName, shortPropertyName, isScan, isSpy, isDynamic);
}

int32 PropertyAccessorDirectImpl::getNumberOfScannableProperties(unsigned long databaseID, const std::string & controllerType) const {
	return Database::getInstance(databaseID)->getNumberOfScannableProperties(controllerType);
}

void PropertyAccessorDirectImpl::setConstantPropertyUnit(unsigned long databaseID, unsigned long propertyID, const std::string & value) const {
	Database::getInstance(databaseID)->setConstantPropertyUnit(propertyID, value);
}

void PropertyAccessorDirectImpl::setPropertyUnit(unsigned long databaseID, unsigned long propertyID, unsigned long unitPropertyID) const {
	Database::getInstance(databaseID)->setPropertyUnit(propertyID, unitPropertyID);
}

unsigned long PropertyAccessorDirectImpl::getUnitPropertyID(unsigned long databaseID, unsigned long propertyID) const {
	return Database::getInstance(databaseID)->getUnitPropertyID(propertyID);
}

void PropertyAccessorDirectImpl::setEnumeratedValues(unsigned long databaseID, unsigned long propertyID, unsigned long valuesPropertyID) const {
	Database::getInstance(databaseID)->setEnumeratedValuesProperty(propertyID, valuesPropertyID);
}

void PropertyAccessorDirectImpl::setEnumeratedLabels(unsigned long databaseID, unsigned long propertyID, unsigned long labelsPropertyID) const {
	Database::getInstance(databaseID)->setEnumeratedLabelsProperty(propertyID, labelsPropertyID);
}

unsigned long PropertyAccessorDirectImpl::getEnumeratedValues(unsigned long databaseID, unsigned long propertyID) const {
	return Database::getInstance(databaseID)->getEnumeratedValuesProperty(propertyID);
}

unsigned long PropertyAccessorDirectImpl::getEnumeratedLabels(unsigned long databaseID, unsigned long propertyID) const {
	return Database::getInstance(databaseID)->getEnumeratedLabelsProperty(propertyID);
}

bool PropertyAccessorDirectImpl::isPropertySavedInCommandBox(unsigned long databaseID, unsigned long propertyID) const {
	return Database::getInstance(databaseID)->isPropertySavedInCommandBox(propertyID);
}

std::vector<unsigned long> PropertyAccessorDirectImpl::getPropertiesSavedInBox(unsigned long databaseID) const {
	return Database::getInstance(databaseID)->getPropertiesSavedInBox();
}

void PropertyAccessorDirectImpl::addServantType(unsigned long databaseID, unsigned long servantID, const std::string& type, bool atFront) const {
	try {
		Database::getInstance(databaseID)->addServantType(servantID, type, atFront);

	} catch (Database::NoSuchServantException& e) {
		throw NoSuchServantException();
	}
}

std::string PropertyAccessorDirectImpl::getWholeScannableProperties(unsigned long databaseID) const {

	return Database::getInstance(databaseID)->getWholeScannableProperties();
}

void PropertyAccessorDirectImpl::setServantInDebug(unsigned long databaseID, unsigned long servantID) const {
	Database::getInstance(databaseID)->setServantInDebug(servantID);
}

}
