/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DATAPROVIDER_STATEACCESSORDIRECTIMPL_H_
#define DATAPROVIDER_STATEACCESSORDIRECTIMPL_H_

#include <common/base/CommonTypes.h>
#include <common/base/BaseTypes.h>
#include <string>

namespace dataprovider {

class StateAccessorDirectImpl {
	
public:
	/**
	 * Constructor.
	 */
	StateAccessorDirectImpl();

	/**
	 * Virtual destructor.
	 */
	virtual ~StateAccessorDirectImpl();

	/**
	 * Notifies the condition state.
	 */
	void notifyConditionState(unsigned long conditionId, bool on, common::ConditionState state);

	/**
	 * Notifies the watchdog actions state.
	 */
	void notifyActionsState(unsigned long conditionId, common::ConditionState state);

	/**
	 * Notifies the user variable value changed.
	 */
	void notifyUserVariableValueChanged(unsigned long databaseId, const std::string& name, double value);

	/**
	 * Notifies the user variable state changed.
	 */
	void notifyUserVariableStateChanged(unsigned long databaseId, const std::string& name, bool enabled);

	/**
	 * Notifies a token state change.
	 */
	void notifyTokenStateChanged(common::CriticalState criticalState, common::RemoteControlState remoteControlState, common::TokenState state, uint32 ownerClientID, const std::string& ownerEndpoint, const std::string& user);

	/**
	 * Notifies a chat message.
	 */
	void notifyChatMessage(uint32 clientID, int64 date, const std::string& sender, const std::string& message, const std::string& fileName);

	/**
	 * Notifies the connected users changed.
	 */
	void notifyConnectedUsersChanged();

	/**
	 * Notifies that a new file has been created.
	 */
	void notifyNewFileCreated(common::FileType fileType, const std::string& fileName);
};

}

#endif

