/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CLSTIMATEDTMEMANAGER_H_
#define CLSTIMATEDTMEMANAGER_H_

#include <thread>
#include <atomic>

namespace common {
	template<typename ValueType>
	class Queue;
}

namespace core {
class Message;
class Shutdown;
class CommandZoneEvent;
}

/**
 * Class defining a command zone event server.
 */
class CLEstimatedTimeManager {

public:

	/**
	 * Compute estimated time error.
	 */
	class ComputeEstimatedTimeError {
	};

	/**
	 * Gets the instance.
	 */
	static CLEstimatedTimeManager* getInstance();

	/**
	 * Initialises the instance.
	 */
	static void initInstance();

	/**
	 * Resets the instance.
	 */
	static void resetInstance();

	/**
	 * Notifies that the command zone compute a new estimated time.
	 */
	void newEstimatedTime();

	void getEstimatedTime();

//	void setActivationState(bool state);

private:
	CLEstimatedTimeManager();
	virtual ~CLEstimatedTimeManager();

	void init();
	void reset();

	static void run();

	void processMessage(core::Message* message);
	void processMessage(core::Shutdown* message);
	void processMessage(core::CommandZoneEvent* commandZoneEvent);

	static CLEstimatedTimeManager* _instance;

	common::Queue<core::Message>* _queue;
	std::thread* _thread;
	static bool _running;
	bool _activated;
	bool _useEstimatedTime;
	std::atomic_bool _estimatedRunning;

	static const std::string USE_ESTIMATED_TIME;

};

#endif /*CORE_COMMANDZONEEVENTSERVER_H_*/
