/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief CommandLine Manager (Singleton)
 * \author J. Locatelli
 * \date 12-01-2011
 */

#ifndef COMMANDLINECOMMAND_H
#define COMMANDLINECOMMAND_H

#include "InstrumentAbstraction/ExperimentController.h"
#include <string>


class CommandLineCommand {
public:
	CommandLineCommand() : _controller(nullptr), _stop(false), _start(true) {
	}
	CommandLineCommand(ExperimentController* ec, bool bg) : _controller(ec), _stop(false), _start(true) {
	}
	CommandLineCommand(ExperimentController* ec) : _controller(ec), _stop(false), _start(true) {
	}
	CommandLineCommand(const CommandLineCommand& command);

	void push_property(const std::string& prop, const std::string& pvalue) {
		_properties[prop] = pvalue;
	}

	template<typename Type> void push_property(Property<Type>& prop, const std::string& pvalue) {
		_baseproperties[&prop] = pvalue;
	}

	template<typename Type> void push_property(SimpleProperty<Type>& prop, const std::string& pvalue) {
		_simpleproperties[&prop] = pvalue;
	}

	void setController(ExperimentController* cont) {
		_controller = cont;
	}

	ExperimentController* getController() const {
		return _controller;
	}

	void stopBefore() {
		_stop = true;
	}

	bool doStopBefore() const {
		return _stop;
	}

	void notStart() {
		_start = false;
	}

	bool toStart() const {
		return _start;
	}

	void applyProperties() const;

	std::map<std::string, std::string> getPropertiesMap();

private:

	ExperimentController* _controller;
	std::map<std::string, std::string> _properties;
	std::map<BaseProperty*, std::string> _baseproperties;
	std::map<BaseProperty*, std::string> _simpleproperties;
	bool _stop;
	bool _start;
};

#endif // COMMANDLINECOMMAND_H
