/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief CommandLine Parser
 * \author J. Locatelli
 * \date 12-01-2011
 */

#ifndef COMMANDLINEPARSER_H
#define COMMANDLINEPARSER_H

#include <string>

#include "CommandSystem/CommandLine/Parsers/AbstractParser.h"

class BadParser;
class Items;

/*!
 * \class CommandLineParser
 * \brief CommandLine Parser
 *
 * \par
 * This class dispatches commands on the different types of parser.
 * it test in the first time the last parser used, if no result re-test with other parsers.
 */
class CommandLineParser {

public:

	/*!
	 * \brief Constructor
	 */
	CommandLineParser();

	/*!
	 * \brief Destructor
	 */
	~CommandLineParser();


	CommandLineParser* clone();

	void setParser(const std::string& parserType) ;

	/*!
	 * \brief Method called for executing a command
	 *
	 * \param[in] text the command to treat
	 */
	void executeCommand(const Items& items);

	/**
	 * Compute Estimated Time.
	 */
	void computeEstimatedTime(Items items, bool doprogression);

	void incNbError() {
		m_CurrentParser->incNbError();
	}

protected:

	/*!
	 * \brief Method called for parsing a command
	 *
	 * \param[in] items The items to parse
	 */
	void parse(Items items) ;

	void initParser();

	AbstractParser* m_CurrentParser; //! Current parser used or we call it also the last one

	std::string m_ParserType;
};

#endif //COMMANDLINEPARSER_H
