/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief Common Command Definition
 * \author J. Locatelli
 * \date 10-02-2011
 */

#ifndef COMMONCMDDEF_H
#define COMMONCMDDEF_H

#include "InstrumentAbstraction/ExperimentController.h"

/*!
 * \namespace Common Command Definition
 */
namespace CommonCmdDef {

/*
 * Parsers Name
 */
static const char GENERIC_PARSER[] = "generic";
static const char TAS_PARSER[] = "tas";
static const char D20D4_PARSER[] = "d20d4";
static const char DIFF_PARSER[] = "diff";
static const char D2B_PARSER[] = "d2b";

/*
 * Commandes
 */
static const char PARSER_CMD[] = "parser";
static const char XBU_CMD[] = "xbu";
static const char DO_CMD[] = "do";
static const char FORLOOP_CMD[] = "for";
static const char END_FORLOOP_CMD[] = "endfor";
static const char BREAK_CMD[] = "break";
static const char IF_CMD[] = "if";
static const char END_IF_CMD[] = "endif";
static const char COMMENT_CMD1[] = "!";
static const char COMMENT_CMD2[] = "#";
static const char SHOW_CMD[] = "show";
static const char WAIT_CMD[] = "wait";
static const char STOP_CMD[] = "stop";
static const char PAUSE_CMD[] = "pause";
static const char CONTINU_CMD[] = "continu";
static const char PWD_CMD[] = "pwd";
static const char CD_CMD[] = "cd";
static const char REFRESH_CMD[] = "refresh";

static const char BACKGROUND_CMG[] = "bg";

/*
 * Constantes
 */
static const char ACTUAL_NAME[] = "actual";
static const char WANTED_NAME[] = "wanted";
static const char USE_RELATIVE_NAME[] = "use_relative";
static const char RELATIVE_NAME[] = "relative";
static const char OFFSET_NAME[] = "offset";
static const char REFERENCE_NAME[] = "reference";
static const char MIN_NAME[] = "min";
static const char MAX_NAME[] = "max";
static const char POSITION[] = "_position";
static const char BACKLASH_NAME[] = "backlash";

static const char SAMPLE_SETTING_NAME[] = "sample_settings";
static const char SAMPLE_SETTING_SELECTOR_NAME[] = "sample_environment_selector";
static const char SAMPLE_SELECTOR_NAME[] = "selected_controller";
//static const char SAMPLE_TEMP_WANTED_NAME[] = "wanted_temperature_";
//static const char SAMPLE_TEMP_REGUL_NAME[] = "regulation_temperature_";
//static const char SAMPLE_TEMP_SAMPLE_NAME[] = "sample_temperature_";
//static const char SAMPLE_TEMP_KELVIN_NAME[] = "kelvin";
//static const char SAMPLE_TEMP_CELSIUS_NAME[] = "celsius";
//static const char SAMPLE_TEMP_UNIT_NAME[] = "temperature_unit";
//static const char SAMPLE_RAMP_MODE_NAME[] = "ramp_mode";
//static const char SAMPLE_RAMP_SPEED_NAME[] = "ramp_speed";
//static const char SAMPLE_NONE_RAMP_NAME[] = "none";
//static const char SAMPLE_SETPOINT_RAMP_NAME[] = "setpoint";
//static const char MAG_WANTED_NAME[] = "wanted_field";
//static const char MAG_ACTUAL_NAME[] = "actual_field";
//static const char MAG_MODE_NAME[] = "persistence_mode";
//static const char COLDVALVE_WANTED_NAME[] = "wanted_pressure";
static const char COLDVALVE_ACTUAL_NAME[] = "actual_pressure";

static const char TO_TYPE_NAME[] = "timeout_type";
static const char TO_NAME[] = "timeout";
static const char USE_TO_NAME[] = "use_timeout";
static const char STABILISATION_TYPE_NAME[] = "stabilisation_type";
static const char STABILISATION_NAME[] = "stabilisation";
static const char USE_STABILISATION_NAME[] = "use_stabilisation";
static const char FIXED_NAME[] = "fixed";

static const char WAIT_NAME[] = "wait";

static const char TAB[] = "     "; // 5 spaces
static const char CURSOR[] = " --> ";

// For loop
static const char VARIABLE[] = "$";
static const char TO[] = "to";
static const char SEPARATOR[] = ";";

// scan
static const char CONTROLLER_PROPERTY_NAME[] = "controller";
static const char WANTED_PROPERTY_NAME[] = "wanted_property";
static const char ACTUAL_PROPERTY_NAME[] = "actual_property";
static const char OFFSET_PROPERTY_NAME[] = "offset_property";
static const char MODE_PROPERTY_NAME[] = "mode";
static const char DELTA_TYPE_PROPERTY_NAME[] = "delta_type";
static const char REFERENCE_PROPERTY_NAME[] = "reference";
static const char DELTA_PROPERTY_NAME[] = "delta";
static const char N_STEPS_PROPERTY_NAME[] = "n_steps";
static const char START_PROPERTY_NAME[] = "start";
static const char END_PROPERTY_NAME[] = "end";

// Data param
static const char DATAPARAM_N_GROUPS_NAME[] = "n_groups";
static const char DATAPARAM_GROUP_SIZE_NAME[] = "group_size";
static const char DATAPARAM_PROPERTY_NAME[] = "name";
static const char DATAPARAM_VALUE_NAME[] = "value";

/*
 * Errors
 */

/*!
 * \class BadInput
 * \brief Exception class
 *
 * Bad input, syntax error when paring line
 */
class BadInput {
public:
	/*!
	 * \brief Constructor
	 */
	BadInput() {
		// Missed an input
		m_Input = "Missed input";
		noerrormsg = false;
	}
	/*!
	 * \brief Constructor
	 * \param[in] anInput The text contains the bad input
	 */
	BadInput(const std::string anInput, bool nomsg = false) {
		m_Input = anInput;
		noerrormsg = nomsg;
	}
	std::string m_Input; //! The input text of the error
	bool noerrormsg;
};

/*!
 * \class PropertyNotFound
 * \brief Exception class
 *
 * Property not found
 */
//class PropertyNotFound {
//public:
//	/*!
//	 * \brief Constructor
//	 * \param[in] controller The controller where we looks for property
//	 * \param[in] name The name of property
//	 */
//	PropertyNotFound(ExperimentController* controller, const std::string name) {
//		m_Controller = controller;
//		m_Name = name;
//	}
//	ExperimentController* m_Controller; //! The controller link
//	std::string m_Name; //! The property name
//};

/*!
 * \class ControllerNotFound
 * \brief Exception class
 *
 * Controller not found
 */
//class ControllerNotFound {
//public:
//	/*!
//	 * \brief Constructor
//	 * \param[in] name The name of the controller or controller type
//	 */
//	ControllerNotFound(const std::string name) {
//		m_Name = name;
//	}
//	std::string m_Name; //! The name of the controller or the controller type
//};

///*!
// * \class CannotSetValue
// * \brief Exception class
// *
// * Cannot change property value
// */
//class CannotSetValue {
//public:
//	/*!
//	 * \brief Constructor
//	 * \param[in] controller The controller where we looks for property
//	 * \param[in] name The name of property
//	 */
//	CannotSetValue(ExperimentController* controller, const std::string name) {
//		m_Controller = controller;
//		m_Name = name;
//	}
//	ExperimentController* m_Controller; //! The controller link
//	std::string m_Name; //! The property name
//};


}

#endif //COMMONCMDDEF_H
