/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief Abstract command
 * \author J. Locatelli
 * \date 13-01-2011
 */

#ifndef ABSTRACTCMD_H
#define ABSTRACTCMD_H

#include "CommandSystem/CommandLine/Parsers/AbstractParser.h"
#include "CommandSystem/CommandLine/CommonCmdDef.h"

#include <boost/any.hpp>
#include <boost/algorithm/string.hpp>

class CommandLineCommand;

/*!
 * \class AbstractCmd
 * \brief Abstract command
 *
 * \par
 * This class implement the interface of all command class.
 * There is no design need for doing that, but Parser class will be very long.
 */
class AbstractCmd {

public:

	/*!
	 * \brief Constructor
	 */
	AbstractCmd(AbstractParser* aParser);

	/*!
	 * \brief Destructor
	 */
	virtual ~AbstractCmd();

	/*!
	 * \brief Decode the command line
	 *
	 * \param[in] items list of item contains on the command line
	 * \throws bad_any_cast bad cast on boost any property
	 * \throws BadInput syntax error when paring line
	 * \throws PropertyNotFound property not found
	 * \throws ControllerNotFound controller not found
	 * \throws CannotSetValue could not set property value
	 */
	virtual void parse(Items& items, bool doestimatedprogression)  = 0;

protected:

	AbstractParser* m_Parser; //! Link to the called parser
	std::vector<CommandLineCommand> _controllersToStart;
};

#endif //ABSTRACTCMD_H
