/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief For Loop command
 * \author J. Locatelli
 * \date 15-02-2011
 */

#include "CommandSystem/CommandLine/Parsers/Common/IfBreakCmd.h"
#include "CommandSystem/CommandLine/Parsers/Common/IfLine.h"
#include "CommandSystem/CommandLine/Parsers/Common/ForLoopCmd.h"
#include "SpecificParser.h"

#include <boost/algorithm/string.hpp>
#include <fstream>
#include <iostream>
using namespace std;

/*
 * Constructor
 */
IfBreakCmd::IfBreakCmd(AbstractParser* aParser, ForLoopCmd* aParent) :
		AbstractCmd(aParser), m_Parent(aParent) {
}

/*
 * Destructor
 */
IfBreakCmd::~IfBreakCmd() {
}

/*
 * parse
 */
void IfBreakCmd::parse(Items& items, bool doestimatedprogression)  {
	if (m_Parser->isForEstimatedTime()) {
		return;
	}

	// Parse for if line
	++items;
	if (items.end() == true) {
		// Nothing found so error
		throw CommonCmdDef::BadInput("No if evaluation expression");
	}
	string command;
	while(items.end() == false) {
		command.append(*items);
		command.append(" ");
		++items;
	}
	IfLine IfLine(m_Parser, command);
	bool result = IfLine.evaluate();
	m_Parser->outputText("break ");
	m_Parser->outputText(command);
	m_Parser->outputText(" is ");
	if (result == true) {
		m_Parser->outputText("true");
		m_Parent->doBrake();
	}
	else {
		m_Parser->outputText("false");
	}
	m_Parser->outputEndl();
}
