/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief If break command
 * \author J. Locatelli
 * \date 36-03-2015
 */

#ifndef IFBREAKCMD_H
#define IFBREAKCMD_H

#include "CommandSystem/CommandLine/Parsers/Common/AbstractCmd.h"

class ForLoopCmd;

/*!
 * \class IfBreakCmd
 * \brief If break inside a for loop
 */
class IfBreakCmd: public AbstractCmd {

public:

	/*!
	 * \brief Constructor
	 */
	IfBreakCmd(AbstractParser* aParser, ForLoopCmd* aParent = NULL);

	/*!
	 * \brief Destructor
	 */
	virtual ~IfBreakCmd();

	/*!
	 * \brief Decode the command line
	 *
	 * \param[in] items list of item contains on the command line
	 * \throws bad_any_cast bad cast on boost any property
	 * \throws BadInput syntax error when paring line
	 * \throws PropertyNotFound property not found
	 * \throws ControllerNotFound controller not found
	 * \throws CannotSetValue could not set property value
	 */
	virtual void parse(Items& items, bool doestimatedprogression) ;

private:

	ForLoopCmd* m_Parent;
};

#endif // IFBREAKCMD_H
