/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief For Loop command
 * \author J. Locatelli
 * \date 15-02-2011
 */

#include "CommandSystem/CommandLine/Parsers/Common/IfCmd.h"
#include "CommandSystem/CommandLine/Parsers/Common/IfLine.h"
#include "SpecificParser.h"

#include <fstream>
#include <iostream>
using namespace std;

/*
 * Constructor
 */
IfCmd::IfCmd(AbstractParser* aParser, XBUInfo& xbuinfo) :
		AbstractCmd(aParser), m_XBUInfo(xbuinfo) {
	// moved $ into name chars
}

/*
 * Destructor
 */
IfCmd::~IfCmd() {
}

/*
 * parse
 */
void IfCmd::parse(Items& items, bool doestimatedprogression)  {
	// Parse for loop line
	// Parse for if line
	++items;
	if (items.end() == true) {
		// Nothing found so error
		throw CommonCmdDef::BadInput("No if evaluation expression");
	}
	string command;
	while(items.end() == false) {
		command.append(*items);
		command.append(" ");
		++items;
	}
	IfLine IfLine(m_Parser, command);

	// Increment current line
	m_XBUInfo.incCurrentLine();

	// File Commands until the endif
	uint32 i = m_XBUInfo.getCurrentLine();
	int32 ifloop = -1;
	vector<Items>::iterator iter = m_XBUInfo.getCommands().begin();
	while (iter != m_XBUInfo.getCommands().end()) {
		Items tmp = *iter;
		if (tmp.getSize() > 0) {
			if (StringUtilities::cmp(*tmp, CommonCmdDef::IF_CMD)) {
				++ifloop;
				if (ifloop > 0) {
					addCommand(tmp, i);
					++i;
				}
			} else if (StringUtilities::cmp(*tmp, CommonCmdDef::END_IF_CMD)) {
				if (ifloop == 0) {
					// end of this for loop
					break;
				} else {
					addCommand(tmp, i);
					++i;
					--ifloop;
				}
			} else {
				addCommand(tmp, i);
				++i;
			}
		}
		++iter;
	}

	m_Parser->outputText("if ");
	m_Parser->outputText(command);
	m_Parser->outputText(" is ");
	if ((IfLine.evaluate() == true) || (m_Parser->isForEstimatedTime() == true)) {
		m_Parser->outputText("true");
		m_Parser->outputEndl();

		m_Parser->setIfRunning(command, true);

		uint32 incline = 0;
		if (m_Parser->isForEstimatedTime() == true) {
			incline = m_Parser->getIfCurrentPoint(command);
		}
		/*
		 * Run commands
		 */
		// Execute the list of new commands
		vector<Items>::iterator iterItemsCommands = m_Commands.begin() + incline;
		vector<uint32>::iterator iterLineNumber = m_LineNumber.begin() + incline;
		while (iterItemsCommands != m_Commands.end()) {
			uint32 currentline = *iterLineNumber;
			m_Parser->setIfCurrentPoint(command, incline);
			// Normal command (no for loop)
			ostringstream buf1;
			buf1 << "XBU (If) " << m_XBUInfo.getXBUFile().filename().string() << " line " << currentline << " > "
					<< (*iterItemsCommands).getString();
			m_Parser->outputText(buf1.str());
			m_Parser->outputEndl();
			m_Parser->setSpyActualCommand((*iterItemsCommands).getString());
			vector<Items> buf;
			vector<Items>::iterator nextiterItemsCommands = iterItemsCommands;
			while (nextiterItemsCommands != m_Commands.end()) {
				buf.push_back(*nextiterItemsCommands);
				++nextiterItemsCommands;
			}
			XBUInfo info(m_XBUInfo.getXBUFile(), currentline, buf);
			SpecificParser cmd(m_Parser->isForEstimatedTime());
			bool found = cmd.decode(*iterItemsCommands, m_Parser, info, doestimatedprogression);
			if (found == false) {
				m_Parser->outputError(COMMAND_LINE_BAD_COMMAND, "");
			}
			// Stopped ?
			if (m_Parser->isStopped() == true) {
				break;
			}
			iterItemsCommands += info.getCurrentLine() - currentline + 1;
			iterLineNumber += info.getCurrentLine() - currentline + 1;
			++incline;
		}

		m_Parser->setIfRunning(command, false);
	}
	else {
		m_Parser->outputText("false");
		m_Parser->outputEndl();
	}
	m_XBUInfo.incCurrentLine((int32) m_Commands.size());
}

/*
 * addCommand
 */
void IfCmd::addCommand(const Items& items, uint32 n) {
	m_Commands.push_back(items);
	m_LineNumber.push_back(n);
}

