/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief For Loop command
 * \author J. Locatelli
 * \date 15-02-2011
 */

#ifndef IFCMD_H
#define IFCMD_H

#include "CommandSystem/CommandLine/Parsers/Common/AbstractCmd.h"
#include "XBUInfo.h"
#include <iostream>

/*!
 * \class IfCmd
 * \brief If Cmd
 *
 */
class IfCmd: public AbstractCmd {

public:

	/*!
	 * \brief Constructor
	 */
	IfCmd(AbstractParser* aParser, XBUInfo& xbuinfo);

	/*!
	 * \brief Destructor
	 */
	virtual ~IfCmd();

	/*!
	 * \brief Decode the command line
	 *
	 * \param[in] items list of item contains on the command line
	 * \throws bad_any_cast bad cast on boost any property
	 * \throws BadInput syntax error when paring line
	 * \throws PropertyNotFound property not found
	 * \throws ControllerNotFound controller not found
	 * \throws CannotSetValue could not set property value
	 */
	virtual void parse(Items& items, bool doestimatedprogression) ;

private:

	/*!
	 * \brief Add command to for loop execution list
	 *
	 * \param[in] items command items
	 * \param[in] n xbu line number
	 */
	void addCommand(const Items& items, uint32 n);

	XBUInfo& m_XBUInfo;

	std::vector<Items> m_Commands; //! Command list to be executed by the for loop
	std::vector<uint32> m_LineNumber; //! Commands XBU line number. Use for print info.
};

#endif // IFCMD_H
