/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief IfLine info class
 * \author J. Locatelli
 * \date 25-03-2015
 */

#ifndef IFLINE_H
#define IFLINE_H

#include <iostream>
#include <string>
#include <muParser.h>
#include "CommandSystem/CommandLine/Parsers/AbstractParser.h"
#include "CommandSystem/CommandLine/CommonCmdDef.h"

/*!
 * \class IfLine
 * \brief IfLine info class
 *
 * \par
 * This class contains information of if command.
 */
class IfLine {

public:

	/*!
	 * \brief constructor for STEP and NBPOINT loop
	 *
	 * \param[in] name The name of the variable ($x)
	 */
	IfLine(AbstractParser* aParser, const std::string& command) ;

	~IfLine();

	/*!
	 * \brief Getter of the current value
	 *
	 * \return The value
	 */
	bool evaluate() ;

	/*!
	 * \brief Operator <<+ overload
	 *
	 * \param[in/out] s The output stream where printing items
	 * \param[in] its The IfLine to print
	 * \return The output stream where the items was print
	 */
	friend std::ostream& operator <<(std::ostream& os, const IfLine& loop);

private:

	static float64 * addValue(const char * valueName, void * pUserData) ;

	AbstractParser* m_Parser;
	std::string m_Command; //! The variable name
	mu::Parser m_MathParser; //! Math parser used for calculating current value from a string expression
	std::vector<float64*> m_Values;

	class MuParserUserData {
	public:
		MuParserUserData(AbstractParser* aParser, std::vector<float64*>* aValues) {
			parser = aParser;
			values = aValues;
		}
		AbstractParser* parser;
		std::vector<float64*>* values;
	};

	MuParserUserData* m_Data;

	static float64 notOperator(float64 a);

	static float64 multiplyOperator(float64 a, float64 b);

	static float64 addOperator(float64 a, float64 b);

	static float64 divideOperator(float64 a, float64 b);

	static float64 lessOperator(float64 a, float64 b);

	static float64 lesserOperator(float64 a, float64 b);

	static float64 lesserOrEqualsOperator(float64 a, float64 b);

	static float64 greaterOperator(float64 a, float64 b);

	static float64 greaterOrEqualsOperator(float64 a, float64 b);

	static float64 equalsOperator(float64 a, float64 b);

	static float64 notEqualsOperator(float64 a, float64 b);

	static float64 modOperator(float64 a, float64 b);

};

#endif // IFLINE_H
