/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief Parser command
 * \author J. Locatelli
 * \date 09-02-2012
 */

#include "CommandSystem/CommandLine/Parsers/Common/ParserCmd.h"
#include "CommandSystem/CommandLine/CommonCmdDef.h"
#include "CommandSystem/CommandLine/CommandLineManager.h"

using namespace std;

/*
 * Constructor
 */
ParserCmd::ParserCmd(AbstractParser* aParser) :
	AbstractCmd(aParser) {
	// empty
}

/*
 * Destructor
 */
ParserCmd::~ParserCmd() {
	// empty
}

/*
 * parse
 */
void ParserCmd::parse(Items& items, bool doestimatedprogression)  {

	++items;
	if (items.end() == true) {
		// List parsers
		m_Parser->outputText(CommonCmdDef::TAB);
		m_Parser->outputText("Actual parser : ");
		m_Parser->outputText(CommandLineManager::getInstance()->getParserType());
		m_Parser->outputEndl();

		m_Parser->outputText(CommonCmdDef::TAB);
		m_Parser->outputText("Available parsers : ");
		m_Parser->outputText(CommonCmdDef::GENERIC_PARSER);
		m_Parser->outputText(", ");
		m_Parser->outputText(CommonCmdDef::TAS_PARSER);
		m_Parser->outputText(", ");
		m_Parser->outputText(CommonCmdDef::D20D4_PARSER);
		m_Parser->outputText(", ");
		m_Parser->outputText(CommonCmdDef::DIFF_PARSER);
		m_Parser->outputEndl();
	}
	else {
		// Set new parser
		try {
			CommandLineManager::getInstance()->setParserType(*items);
			m_Parser->outputText(CommonCmdDef::TAB);
			m_Parser->outputText("New parser : ");
			m_Parser->outputText(*items);
		} catch (BadParser&) {
			throw CommonCmdDef::BadInput("Unknown parser type " + *items);
		}
	}

	m_Parser->outputEndl();
}
