/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief Show command
 * \author J. Locatelli
 * \date 31-03-2015
 */

#include "CommandSystem/CommandLine/Parsers/Common/ShowCmd.h"

using namespace std;

/*
 * Constructor
 */
ShowCmd::ShowCmd(AbstractParser* aParser) :
	AbstractCmd(aParser) {
	// empty
}

/*
 * Destructor
 */
ShowCmd::~ShowCmd() {
	// empty
}

/*
 * parse
 */
void ShowCmd::parse(Items& items, bool doestimatedprogression)  {

	++items;
	if (items.end() == true) {
		throw CommonCmdDef::BadInput("Inputs are missing");
	}

	ExperimentController* controller = m_Parser->getControllerByName(*items);
	if (controller == NULL) {
		throw CommonCmdDef::BadInput("Controller not found");
	}

	vector<AnyProperty*> properties = controller->getAllProperties();
	vector<AnyProperty*>::iterator iter = properties.begin();
	while(iter != properties.end()) {
		AnyProperty* prop = *iter;
		string value = m_Parser->getPropertyValueAsString(controller, prop->getName());
		string name = prop->getName();
		string alias = prop->getAlias();
		ostringstream buf;
		buf << name;
		if (alias.empty() == false) {
			buf <<" (" << alias << ")";
		}
		buf << " " << value;
		m_Parser->outputText(buf.str());
		m_Parser->outputEndl();
		++iter;
	}
}
