/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief Abstract class describes the parser
 * \author J. Locatelli
 * \date 12-01-2010
 */

#include "SpecificParser.h"
#include "ForLoopCmd.h"
#include "IfBreakCmd.h"
#include "IfCmd.h"
#include "XBUInfo.h"
#include "CommandSystem/CommandLine/Parsers/AbstractParser.h"

/*
 * decode
 */
bool SpecificParser::decode(Items items, AbstractParser* aParser, XBUInfo& xbuinfo, bool doprogression, ForLoopCmd* aParent)
		 {
	bool found = true;
	/*
	 * First item is the command
	 */
	if (StringUtilities::cmp(*items, CommonCmdDef::FORLOOP_CMD) == true) {
		// Wait command
		ForLoopCmd cmd(aParser, xbuinfo, aParent);
		cmd.parse(items, doprogression);
	} else if ((StringUtilities::cmp(*items, CommonCmdDef::BREAK_CMD) == true) && (aParent!= NULL)) {
		// If break command
		IfBreakCmd cmd(aParser, aParent);
		cmd.parse(items, doprogression);
	} else if (StringUtilities::cmp(*items, CommonCmdDef::IF_CMD) == true) {
		// If break command
		IfCmd cmd(aParser, xbuinfo);
		cmd.parse(items, doprogression);
	} else {
		found = aParser->decode(items, doprogression);
	}
	return found;
}
