/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief Abstract class describes the parser
 * \author J. Locatelli
 * \date 12-01-2010
 */

#ifndef SPECIFICPARSER_H
#define SPECIFICPARSER_H

#include "CommandSystem/CommandLine/Items.h"
#include "CommandSystem/CommandLine/CommonCmdDef.h"

class XBUInfo;
class AbstractParser;
class ForLoopCmd;

/*!
 * \class SpecificParser
 * \brief Abstract class describes the parser
 *
 * \par
 * This class is the virtual class of all command line parsers.
 * It contains all common functionalities.
 */
class SpecificParser {

public:

	SpecificParser(bool forEstimatedTime) : _forEstimatedTime(forEstimatedTime) {

	}
	/*!
	 * \brief Decode the command line
	 *
	 * \param[in] items list of item contains on the command line
	 * \throws BadInput syntax error when paring line
	 */
	bool decode(Items items, AbstractParser* aParser, XBUInfo& xbuinfo, bool doprogression, ForLoopCmd* aParent = NULL) ;

private:

	bool _forEstimatedTime;

};

#endif //SPECIFICPARSER_H
