/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief Wait command
 * \author J. Locatelli
 * \date 10-02-2011
 */

#include "CommandSystem/CommandLine/Parsers/Common/WaitCmd.h"
#include "CommandSystem/CommandLine/CommonCmdDef.h"
#include "CommandNames.h"
#include "DataProvider/Units.h"

#include <boost/any.hpp>

using namespace std;
using namespace boost;

/*
 * Constructor
 */
WaitCmd::WaitCmd(AbstractParser* aParser) :
	AbstractCmd(aParser) {
	// empty
}

/*
 * Destructor
 */
WaitCmd::~WaitCmd() {
	// empty
}

/*
 * parse
 */
void WaitCmd::parse(Items& items, bool doestimatedprogression)  {

	// Get the controller wait
	ExperimentController* controller = m_Parser->getControllerByType(CommonCmdDef::WAIT_NAME);
	if (controller == NULL) {
		throw CommonCmdDef::BadInput("Controller wait is missing");
	}
	CommandLineCommand clc(controller);

	++items;
	if (items.end() == false) {
		// Test if type time is collate to time value
		string time, type;
		if (m_Parser->convertStringToTimeAndType(*items, time, type) == true) {
			// get time value
			clc.push_property("wanted_time", time);
			clc.push_property("time_type", type);
		}
		else {
			// get time value
			clc.push_property("wanted_time", *items);
			++items;
			if (items.end() == false) {
				// Get time type
				if (StringUtilities::cmp(*items, unit::HOUR, 1) == true) {
					clc.push_property("time_type", unit::HOUR);
				} else if (StringUtilities::cmp(*items, unit::MINUTE, 1) == true) {
					clc.push_property("time_type", unit::MINUTE);
				} else if (StringUtilities::cmp(*items, unit::SECOND, 1) == true) {
					clc.push_property("time_type", unit::SECOND);
				}
			}
		}
	}
	// Execute action on the controller wait
	_controllersToStart.push_back(clc);
	m_Parser->startControllers(COMMAND_START, _controllersToStart, m_Parser->isSynchronized(), doestimatedprogression);
}
