/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief XBU or DO command
 * \author J. Locatelli
 * \date 10-02-2011
 */

#ifndef XBUCMD_H
#define XBUCMD_H

#include "CommandSystem/CommandLine/Parsers/Common/AbstractCmd.h"

/*!
 * \class XBUCmd
 * \brief Xbu or do command
 *
 * \par
 * This class implement the xbu or do command. It's executing a xbu script file
 */
class XBUCmd: public AbstractCmd {

public:

	/*!
	 * \brief Constructor
	 */
	XBUCmd(AbstractParser* aParser);

	/*!
	 * \brief Destructor
	 */
	virtual ~XBUCmd();

	/*!
	 * \brief Decode the command line
	 *
	 * \param[in] items list of item contains on the command line
	 * \throws bad_any_cast bad cast on boost any property
	 * \throws BadInput syntax error when paring line
	 * \throws PropertyNotFound property not found
	 * \throws ControllerNotFound controller not found
	 * \throws CannotSetValue could not set property value
	 */
	virtual void parse(Items& items, bool doestimatedprogression) ;

private:

	uint32 getNumberLines(const std::string& filename);

};

#endif // XBUCMD_H
