/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief XBUInfo info class
 * \author J. Locatelli
 * \date 15-02-2011
 */

#include "CommandSystem/CommandLine/Parsers/Common/XBUInfo.h"
#include "CommandSystem/CommandLine/Items.h"

using namespace std;
using namespace boost::filesystem;

/*
 * Constructor
 */
XBUInfo::XBUInfo(boost::filesystem::path xbufile, uint32 currentline, std::vector<Items> commands) :
	m_XBUFile(xbufile), m_CurrentLine(currentline), m_Commands(commands) {
}

/*
 * copy Constructor
 */
XBUInfo::XBUInfo(const XBUInfo& f) {
	m_XBUFile = f.m_XBUFile;
	m_CurrentLine = f.m_CurrentLine;
	m_Commands = f.m_Commands;
}

/*
 * getXBUFile
 */
const path& XBUInfo::getXBUFile() const {
	return m_XBUFile;
}

/*
 * getCurrentLine
 */
uint32 XBUInfo::getCurrentLine() const {
	return m_CurrentLine;
}

/*
 * incCurrentLine
 */
void XBUInfo::incCurrentLine(uint32 inc) {
	m_CurrentLine += inc;
}

/*
 * getCommands
 */
std::vector<Items>& XBUInfo::getCommands() {
	return m_Commands;
}

/*
 * operator <<
 */
ostream& operator <<(ostream& os, const XBUInfo& info) {
	os << "XBUFIle " << info.m_XBUFile.string() << " at line " << info.m_CurrentLine;
	return os;
}
