/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief XBUInfo info class
 * \author J. Locatelli
 * \date 15-02-2011
 */

#ifndef XBUINFO_H
#define XBUINFO_H

#include <string>
#include <common/base/BaseTypes.h>
#include "boost/filesystem/operations.hpp"

class Items;

/*!
 * \class XBUInfo
 * \brief XBUInfo info class
 *
 * \par
 * This class contains information of For XBU files.
 */
class XBUInfo {

public:

	/*!
	 * \brief constructor for STEP and NBPOINT loop
	 *
	 * \param[in] xbufile xbu file path
	 * \param[in] currentline The current line in the xbu file
	 */
	XBUInfo(boost::filesystem::path xbufile, uint32 currentline, std::vector<Items> commands);

	/*!
	 * \brief copy constructor
	 *
	 * \param[in] f The reference object for copying
	 */
	XBUInfo(const XBUInfo& f);

	/*!
	 * \brief Getter of the xbu file
	 *
	 * \return The name
	 */
	const boost::filesystem::path& getXBUFile() const;

	/*!
	 * \brief Getter of the current line
	 *
	 * \return The number
	 */
	uint32 getCurrentLine() const;

	/*!
	 * \brief Set current value at the next value
	 */
	void incCurrentLine(uint32 inc = 1);

	/*!
	 * \brief Getter of the next commands
	 *
	 * \return The number
	 */
	std::vector<Items>& getCommands();

	/*!
	 * \brief Operator <<+ overload
	 *
	 * \param[in/out] s The output stream where printing items
	 * \param[in] its The XBUInfo to print
	 * \return The output stream where the items was print
	 */
	friend std::ostream& operator <<(std::ostream& os, const XBUInfo& loop);

private:

	boost::filesystem::path m_XBUFile;
	uint32 m_CurrentLine;
	std::vector<Items> m_Commands;

};

#endif // XBUINFO_H
