/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CommandSystem/ConfigurationManager.h"
#include "InstrumentManager/HardwareManager.h"
#include "InstrumentManager/InstrumentManager.h"
#include "ics/Scheduler/AgentPolicy/AgentPolicy.h"

ConfigurationManager* ConfigurationManager::_instance = new ConfigurationManager();

ConfigurationManager* ConfigurationManager::getInstance() {
	return _instance;
}

ConfigurationManager::ConfigurationManager() :
	_running(false), _thread(nullptr) {
}

ConfigurationManager::~ConfigurationManager() {
}

void ConfigurationManager::resetInstance(bool resetManagers) {

	_instance->terminate(resetManagers);
	delete _instance;
}

void ConfigurationManager::loadConfiguration() {

	HardwareManager::initInstance();
	InstrumentManager::initInstance();

	HardwareManager::getInstance()->loadTestControllers();
	InstrumentManager::getInstance()->loadExperimentControllers();

	HardwareManager::getInstance()->initTestControllers();

	InstrumentManager::getInstance()->configExperimentControllers();
	InstrumentManager::getInstance()->initExperimentControllers();

	HardwareManager::getInstance()->loadReports();
	InstrumentManager::getInstance()->loadReports();

	AgentPolicy::getInstance(); // Check agent policy XML file at the start
}

void ConfigurationManager::saveConfiguration() {
	InstrumentManager::getInstance()->saveExperimentControllers(false);
	HardwareManager::getInstance()->saveTestControllers(false);
}

void ConfigurationManager::backupConfiguration() {
	InstrumentManager::getInstance()->saveExperimentControllers(true);
	HardwareManager::getInstance()->saveTestControllers(true);
}

void ConfigurationManager::start() {
	_thread = new std::thread(&_loop);
}

void ConfigurationManager::terminate(bool resetManagers) {
	_running = false;

	if (_thread != nullptr) {
		_thread->join();
		delete _thread;
	}

	// Reset managers is set to false when the application is forced to shutdown
	if (resetManagers) {
		InstrumentManager::reset();
		HardwareManager::reset();
	}
}

void ConfigurationManager::_loop() {

	_instance->_running = true;
	while (_instance->_running) {
		int32 i = 0;
		while (i < CONFIGURATION_SAVE_PERIOD && _instance->_running) {
			// sleep for 1 second
			sleep(1);
			i++;
		}
		if (_instance->_running) {
			_instance->saveConfiguration();
		}
	}

	_instance->saveConfiguration();
}
