/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CONFIGURATIONMANAGER_H
#define CONFIGURATIONMANAGER_H

#include <common/base/BaseTypes.h>
#include <thread>

class ConfigurationManager {

public:
	/**
	 * Gets the instance.
	 */
	static ConfigurationManager* getInstance();

	/**
	 * Resets the instance. Terminates the loop.
	 */
	static void resetInstance(bool resetManagers = true);

	/**
	 * Loads the configuration (controllers and drivers).
	 */
	void loadConfiguration();

	/**
	 * Saves the configuration (controllers and drivers).
	 */
	void saveConfiguration();

	/**
	 * Backups the configuration (controllers and drivers).
	 */
	void backupConfiguration();

	/**
	 * Starts the thread.
	 */
	void start();

private:
	ConfigurationManager();
	~ConfigurationManager();

	static void _loop();
	void terminate(bool resetManagers);

	static ConfigurationManager* _instance;

	bool _running;
	std::thread* _thread;

	static const int32 CONFIGURATION_SAVE_PERIOD = 60;
};

#endif
