/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef SIMULATEDCONFIGURATIONMANAGER_H
#define SIMULATEDCONFIGURATIONMANAGER_H

#include <common/base/Exception.h>
#include <cameo/api/cameo.h>
#include <string>

class SimulatedConfigurationManager {

public:
	/**
	 * Exception for initialisation failure.
	 */
	class InitFailureException  : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		InitFailureException(const std::string& problem);
	};

	/**
	 * Gets the instance.
	 */
	static SimulatedConfigurationManager* getInstance();

	/**
	 * Initialises the connection.
	 */
	void init();

	/**
	 * Resets the instance. Terminates the loop.
	 */
	static void resetInstance();

	/**
	 * Loads the configuration (controllers and drivers).
	 */
	void loadConfiguration();

	bool refreshConfiguration();

private:
	SimulatedConfigurationManager();
	~SimulatedConfigurationManager();

	void loadExperimentControllers();
	void loadTestControllers();

	bool refreshExperimentControllers();
	bool refreshTestControllers();

	std::unique_ptr<cameo::coms::Requester> m_servantManagerRequester;

	static SimulatedConfigurationManager* m_instance;
};

#endif
