/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ChangeAspect.h"
#include <iostream>

using namespace std;

/**
 * 
 */
ChangeAspect::ChangeAspect(const ChangeAspectType& type,
		const string& controllerType, const string& controllerName) :
	m_type(type), m_ControllerType(controllerType), m_ControllerName(
			controllerName) {

	m_stringParam = "";
	m_longParam = 0;
	m_nextChangeAspect = 0;
}

/**
 * 
 */
ChangeAspect::ChangeAspect(const ChangeAspectType& type,
		const string& controllerType, const string& controllerName,
		const string& stringParam) :
	m_type(type), m_ControllerType(controllerType), m_ControllerName(
			controllerName), m_stringParam(stringParam) {

	m_longParam = 0;
	m_nextChangeAspect = 0;
}

/**
 * 
 */
ChangeAspect* ChangeAspect::clone() {
	ChangeAspect* result = 0;
	result = new ChangeAspect(this->getType(), this->getControllerType(),
			this->getControllerName());
	result->setStringParam(this->getStringParam());
	result->setLongParam(this->getLongParam());
	return result;
}

/**
 * 
 */
ChangeAspectType ChangeAspect::getType() const {
	return m_type;
}

/**
 * 
 */
const string& ChangeAspect::getControllerType() const {
	return m_ControllerType;
}

const string& ChangeAspect::getControllerName() const {
	return m_ControllerName;
}

void ChangeAspect::setControllerType(const string& type) {
	m_ControllerType = type;
}

void ChangeAspect::setControllerName(const string& name) {
	m_ControllerName = name;
}

/**
 * 
 */
const string& ChangeAspect::getStringParam() const {
	return m_stringParam;
}

/**
 * 
 */
void ChangeAspect::setStringParam(const string& stringParam) {
	m_stringParam = stringParam;
}

/**
 * 
 */
int32 ChangeAspect::getLongParam() const {
	return m_longParam;
}

/**
 * 
 */
void ChangeAspect::setLongParam(const int32& longParam) {
	m_longParam = longParam;
}

/**
 * 
 */
bool ChangeAspect::hasNext() const {
	bool result = false;
	if (m_nextChangeAspect != 0) {
		result = true;
	}
	return result;
}

/**
 * 
 */
ChangeAspect* ChangeAspect::getNext() const {
	return m_nextChangeAspect;
}

/**
 * 
 */
void ChangeAspect::setNext(ChangeAspect* next) {
	m_nextChangeAspect = next;
}

/**
 * 
 */
void ChangeAspect::link(ChangeAspect* changeAspectToLink) {
	ChangeAspect* lastItem = m_getLastItem();
	lastItem->setNext(changeAspectToLink);
}

/**
 * 
 */
bool ChangeAspect::operator==(const ChangeAspect& caToCompare) {
	bool result = false;

	// If adresses are different, compare members
	if ((getType() == caToCompare.m_type) && (getControllerType()
			== caToCompare.m_ControllerType) && (getControllerName()
			== caToCompare.m_ControllerName) && (getStringParam()
			== caToCompare.m_stringParam) && (getLongParam()
			== caToCompare.m_longParam)) {
		result = true;
	}

	return result;
}

/**
 * 
 */
ostream& operator<<(ostream&s, const ChangeAspect& ca) {
	s << "Type: " << ca.getType() << endl << "Controller Type: "
			<< ca.getControllerType() << endl << "Controller Name: "
			<< ca.getControllerName() << endl << "Param String: "
			<< ca.getStringParam() << endl << "Param long: "
			<< ca.getLongParam() << endl << "Has next: " << ca.hasNext()
			<< endl << endl;
	return s;
}

/**
 * 
 */
ChangeAspect::ChangeAspect(const ChangeAspect& ca) {
	//empty. To avoid its usage.
}

/**
 * 
 */
ChangeAspect* ChangeAspect::m_getLastItem() {
	ChangeAspect* current = this;
	while (current->hasNext()) {
		current = current->getNext();
	}
	return current;
}
