/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CHANGEASPECT_H
#define CHANGEASPECT_H

#include <string>

#include "ChangeAspectType.h"
#include <common/base/BaseTypes.h>

class ChangeAspect {

public:
	/**
	 * Constructors
	 */
	ChangeAspect(const ChangeAspectType& type, const std::string& controllerType, const std::string& controllerName);

	ChangeAspect(const ChangeAspectType& type, const std::string& controllerType, const std::string& controllerName,
			const std::string& stringParam);

	/**
	 * To clone a ChangeAspect
	 */
	ChangeAspect* clone();

	/**
	 * To get the ChangeAspect type (PROPERTY, ERROR or PROGRESS)
	 */
	ChangeAspectType getType() const;

	/**
	 * To get the ID of the ChangeAspect
	 * ID is the object that sent the ChangeAspect
	 */
	const std::string& getControllerType() const;
	const std::string& getControllerName() const;

	void setControllerType(const std::string& type);
	void setControllerName(const std::string& name);

	/**
	 * To get the std::string parameter of the ChangeAspect
	 */
	const std::string& getStringParam() const;

	/**
	 * To set the std::string parameter of the ChangeAspect
	 */
	void setStringParam(const std::string& stringParam);

	/**
	 * To set the std::string parameter of the change aspect
	 */
	int32 getLongParam() const;

	/**
	 * To set the int32 parameter of the ChangeAspect
	 */
	void setLongParam(const int32& longParam);

	/**
	 * To check if ChangeAspect is link to another one
	 */
	bool hasNext() const;

	/**
	 * To get the next ChangeAspect
	 */
	ChangeAspect* getNext() const;

	/**
	 * To set the next ChangeAspect
	 */
	void setNext(ChangeAspect* nextChangeAspect);

	/**
	 * To link a new ChangeAspect
	 */
	void link(ChangeAspect*);

	/**
	 * Equality operator
	 */
	bool operator==(const ChangeAspect& caToCompare);

	/**
	 * To put change in a stream
	 */
	friend std::ostream& operator<<(std::ostream&s, const ChangeAspect&);

	/**
	 * Constants for data file
	 */

	static const int32 UPDATE_RECORD = 0;
	static const int32 NEW_RECORD = 1;
	static const int32 APPEND_RECORD = 2;
	static const int32 LOAD_FILE = 4;
	/* New constante for scan mode */
	static const int32 NEW_RECORD_INC = 8;
	static const int32 UPDATE_RECORD_INC = 16;
	static const int32 APPEND_RECORD_INC = 32;
	static const int32 LOAD_FILE_INC = 64;

private:
	ChangeAspectType m_type;
	std::string m_ControllerType;
	std::string m_ControllerName;
	std::string m_stringParam;
	int32 m_longParam;
	ChangeAspect* m_nextChangeAspect;
	ChangeAspect(const ChangeAspect&);
	ChangeAspect* m_getLastItem();
};

#endif
