/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMANDMANAGER_H
#define COMMANDMANAGER_H

#include <string>
#include <vector>
#include <map>
#include <iostream>
#include <common/base/BaseTypes.h>
#include <common/base/CommonTypes.h>

namespace command {

class CommandAccessor;

}

class CommandManager {

public:
	CommandManager(const std::string& name);
	virtual ~CommandManager();

	CommandManager* clone(const std::string& name) const;

	void renameServant(const std::string& name);

	std::string getCommands() const;
	void addCommand(const std::string& commandName);
	void removeCommand(const std::string& commandName);
	bool commandExists(const std::string& commandName) const;
	void setCommandState(const std::string& commandName, common::CommandState state) const;
	common::CommandState getCommandState(const std::string& commandName) const;

	/**
	 * Changes the state of the running commands (i.e. not INACTIVE) except read, status and fromCommand.
	 */
	void setRunningCommandState(const std::string& fromCommand, common::CommandState state) const;

	/**
	 * Returns true if there is at least one running command (i.e. not INACTIVE) except read, status that have the state state.
	 */
	bool hasRunningCommandState(common::CommandState state) const;

	/**
	 * Returns true if there is no running command.
	 */
	bool hasOnlyInactiveCommands() const;

	std::vector<std::string> getCommandList() const;
	void setProgression(float64 value);

private:
	std::map<std::string, uint32> _commands;

	std::string _name;
	uint32 _servantID;

	command::CommandAccessor* _commandAccessor;
};

#endif
