/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DataProvider/Property/PropertyInfos.h"
#include <command/transportlayer/MultiThreadProcessorCommunication.h>
#include <command/transportlayer/dataprovider/ServantAccessor.h>
#include <command/transportlayer/dataprovider/PropertyAccessor.h>
#include <iostream>

using namespace std;

NoPropertyInfo::NoPropertyInfo(const std::string & controllerType, const std::string & wantedPropertyName) :
	runtime_error(std::string("The property ") + controllerType + " " + wantedPropertyName + " has no info") {
}

PropertyInfos * PropertyInfos::_instance = 0;

void PropertyInfos::initInstance() {
	_instance = new PropertyInfos();
}

PropertyInfos * PropertyInfos::getInstance() {
	return _instance;
}

PropertyInfos::PropertyInfos() {
	init();
}

bool PropertyInfos::isScannable(const std::string & controllerType, const std::string & wantedPropertyName) const {
	return _propertyAccessor->isScannableProperty(controllerType, wantedPropertyName);
}

PropertyInfos::Item PropertyInfos::getItem(const std::string & controllerType, const std::string & wantedPropertyName) const {

	PropertyInfos::Item result;

	try {
		command::PropertyAccessor::PropertyInfoItem item = _propertyAccessor->getPropertyInfoItem(controllerType, wantedPropertyName);
		result.actualPropertyName = item.actualPropertyName;
		result.offsetPropertyName = item.offsetPropertyName;
		result.shortPropertyName = item.shortPropertyName;
		result.isScan = item.isScan;
		result.isSpy = item.isSpy;
		result.isDynamic = item.isDynamic;

	} catch (command::PropertyAccessor::NoSuchPropertyException& e) {
		throw NoPropertyInfo(controllerType, wantedPropertyName);
	}

	return result;
}

void PropertyInfos::addItem(const std::string & controllerType, const std::string & wantedPropertyName, const std::string & actualPropertyName, const std::string & offsetPropertyName, const std::string & shortPropertyName, bool isScan, bool isSpy, bool isDynamic) {
	_propertyAccessor->addPropertyInfoItem(controllerType, wantedPropertyName, actualPropertyName, offsetPropertyName, shortPropertyName, isScan, isSpy, isDynamic);
}

int32 PropertyInfos::getNumberOfScannableProperties(const std::string & controllerType) const {
	return _propertyAccessor->getNumberOfScannableProperties(controllerType);
}

void PropertyInfos::init() {
	_propertyAccessor = command::MultiThreadProcessorCommunication::getInstance()->getPropertyAccessor();
}
