/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef ARRAYPROPERTY_T_H
#define ARRAYPROPERTY_T_H

#include "ValueProperty.h"

template<typename Type>
class ArrayProperty : public ValueProperty<Type *> {

public:
	typedef Type * TypePtr;

	using ValueProperty<TypePtr>::operator=;
	using ValueProperty<TypePtr>::m_servantProperty;

	ArrayProperty() {}

	template<typename ServantType>
	void init(ServantType * servant, int32 flags, const std::string& name, const std::string& alias = "") {
		ValueProperty<TypePtr>::init(servant, name, alias, flags);
	}

	template<typename ServantType, int aliasLength>
	void init(ServantType * servant, int32 flags, const std::string& name, const char(&aliasName)[aliasLength]) {

		std::string alias = aliasName;

		// Do not check for drivers.
		if (servant->getServantClass() != DRIVER_CLASS) {
			alias = checkString<EXECUTION_WARNING, 0, 10>(name, aliasName);
			if (formatAlias(alias)) {
				common::warn << "Alias for '" << name << "' renamed into '" << alias << "'" << common::eol;
			}
		}

		ValueProperty<TypePtr>::init(servant, name, alias, flags);

		// Do not set property info for drivers.
		if (servant->getServantClass() != DRIVER_CLASS) {
			bool spy = ((SPY & flags) == 1);
			this->addPropertyInfo(ServantType::TYPE, "", name, "", alias, false, spy);
		}
	}

	void operator=(const TypePtr& value);
	void set(const TypePtr& value);
	void update(const TypePtr& value);
	virtual void formatValue();

	void setSize(int32 size);
	int32 getSize();
	void sendEvent();

	void setColumnSize(int32 size);
	int32 getColumnSize();
};

template<typename Type>
void ArrayProperty<Type>::operator=(const typename ArrayProperty<Type>::TypePtr& value) {
	m_servantProperty->setValue(value);
}

template<typename Type>
void ArrayProperty<Type>::set(const typename ArrayProperty<Type>::TypePtr& value) {
	m_servantProperty->setValue(value);
}

template<typename Type>
void ArrayProperty<Type>::update(const typename ArrayProperty<Type>::TypePtr& value) {
	m_servantProperty->updateValue(value);
}

template<typename Type>
void ArrayProperty<Type>::formatValue() {
	// do nothing for arrays
}

template<typename Type>
void ArrayProperty<Type>::setSize(int32 size) {
	this->m_servantProperty->setSize(size);
}

template<typename Type>
int32 ArrayProperty<Type>::getSize() {
	return this->m_servantProperty->getSize();
}

template<typename Type>
void ArrayProperty<Type>::sendEvent() {
	this->m_servantProperty->sendArrayEvent();
}

template<typename Type>
void ArrayProperty<Type>::setColumnSize(int32 size) {
	this->m_servantProperty->setColumnSize(size);
}

template<typename Type>
int32 ArrayProperty<Type>::getColumnSize() {
	return this->m_servantProperty->getColumnSize();
}

#endif
