/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DYNAMICUPDATER_H
#define DYNAMICUPDATER_H

#include <boost/any.hpp>
#include <common/base/Function.h>
#include <common/base/BaseTypes.h>

class DynamicUpdater {

	friend class PropertyUpdater;

public:
	DynamicUpdater();
	~DynamicUpdater();

	void update(int32 index);

private:
	template<typename Class, typename Object>
	void setUpdater(void (Class::* f)(int32), Object * obj) {
		m_updater = new common::Function1<void, int32, Object>(std::bind(f, std::placeholders::_1, std::placeholders::_2), obj);
	}

	template<typename Class, typename Param1, typename Object, typename Arg1>
	void setUpdater(void (Class::* f)(int32, Param1), Object * obj, Arg1 arg1) {
		m_updater = new common::Function1<void, int32, Object>(std::bind(f, std::placeholders::_1, arg1, std::placeholders::_2), obj);
	}

	template<typename Class, typename Param1, typename Param2, typename Object, typename Arg1, typename Arg2>
	void setUpdater(void (Class::* f)(int32, Param1, Param2), Object * obj, Arg1 arg1, Arg2 arg2) {
		m_updater = new common::Function1<void, int32, Object>(std::bind(f, std::placeholders::_1, arg1, arg2, std::placeholders::_2), obj);
	}

	template<typename Class, typename Param1, typename Param2, typename Param3, typename Object, typename Arg1, typename Arg2, typename Arg3>
	void setUpdater(void (Class::* f)(int32, Param1, Param2, Param3), Object * obj, Arg1 arg1, Arg2 arg2, Arg3 arg3) {
		m_updater = new common::Function1<void, int32, Object>(std::bind(f, std::placeholders::_1, arg1, arg2, arg3, std::placeholders::_2), obj);
	}

	template<typename Class, typename Param1, typename Param2, typename Param3, typename Param4, typename Object, typename Arg1, typename Arg2, typename Arg3, typename Arg4>
	void setUpdater(void (Class::* f)(int32, Param1, Param2, Param3, Param4), Object * obj, Arg1 arg1, Arg2 arg2, Arg3 arg3, Arg4 arg4) {
		m_updater = new common::Function1<void, int32, Object>(std::bind(f, std::placeholders::_1, arg1, arg2, arg3, arg4, std::placeholders::_2), obj);
	}

	common::BaseFunction1<void, int32> * m_updater;
};

#endif
