/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Format.h"
#include "ValueProperty.h"

ConstantFormat::ConstantFormat(const std::string& value)
: m_value(value) {}

BaseFormat * ConstantFormat::clone() const {
	return new ConstantFormat(*this);
}

std::string ConstantFormat::getValue() const {
	return m_value;
}

PropertyFormat::PropertyFormat(const ValueProperty<std::string> & property)
: m_property(property) {}

BaseFormat * PropertyFormat::clone() const {
	return new PropertyFormat(*this);
}

std::string PropertyFormat::getValue() const {
	return m_property.get();
}

Format::Format()
: m_format(new ConstantFormat("%1%")) {
}

Format::Format(const Format & format) :
	m_format(format.m_format->clone()) {
}

Format::~Format() {
	delete m_format;
}

void Format::copy(const Format & format) {
	m_format = format.m_format->clone();
}

void Format::setFormat(const std::string & value) {

	delete m_format;
	m_format = new ConstantFormat(value);
}

void Format::setFormat(const ValueProperty<std::string> & property) {

	delete m_format;
	m_format = new PropertyFormat(property);
}

std::string Format::getFormat() const {
	return m_format->getValue();
}
