/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef FORMAT_H
#define FORMAT_H

#include <string>
#include <boost/format.hpp>

template<typename>
class ValueProperty;

class BaseFormat {

public:
	virtual ~BaseFormat() {}
	virtual BaseFormat * clone() const = 0;
	virtual std::string getValue() const = 0;
};

class ConstantFormat : public BaseFormat {

public:
	ConstantFormat(const std::string& value);

	virtual BaseFormat * clone() const;
	virtual std::string getValue() const;

private:
	std::string m_value;
};

class PropertyFormat : public BaseFormat {

public:
	PropertyFormat(const ValueProperty<std::string> & property);

	virtual BaseFormat * clone() const;
	virtual std::string getValue() const;

private:
	const ValueProperty<std::string> & m_property;
};

class Format {

public:
	Format();
	Format(const Format & format);
	~Format();

	void copy(const Format & format);
	void setFormat(const std::string & value);
	void setFormat(const ValueProperty<std::string> & property);

	std::string getFormat() const;

private:
	BaseFormat * m_format;
};

#endif
