/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef FORMATTER_H
#define FORMATTER_H

#include <common/base/Function.h>
#include <string>

template<typename Type>
class Formatter {

public:
	Formatter();
	Formatter(const Formatter<Type> & formatter);
	virtual ~Formatter();

	bool isFormatterDefined() const;

	void unsetFormatFunction();

	template<typename Function>
	void setFormatFunction(Function f) {
		m_formatter = new common::Function1<std::string, Type, void>(std::bind(f, std::placeholders::_1), 0);
	}

	template<typename Function, typename Object>
	void setFormatFunction(Function f, Object * obj) {
		m_formatter = new common::Function1<std::string, Type, Object>(std::bind(f, std::placeholders::_1, std::placeholders::_2), obj);
	}

	std::string format(Type value) const;

	template<typename ServantType>
	void copy(ServantType * servant, const Formatter<Type> & formatter) {

		if (formatter.m_formatter != 0) {
			m_formatter = formatter.m_formatter->clone();
			m_formatter->setObject(servant);
		}
	}

private:
	common::BaseFunction1<std::string, Type> * m_formatter;
};

template<typename Type>
Formatter<Type>::Formatter() :
	m_formatter(0) {
}

template<typename Type>
Formatter<Type>::Formatter(const Formatter<Type> & formatter) :
	m_formatter(0) {

	// copy the formatter
	if (formatter.m_formatter != 0) {
		m_formatter = formatter.m_formatter->clone();
	}
}

template<typename Type>
Formatter<Type>::~Formatter() {
	delete m_formatter;
}

template<typename Type>
bool Formatter<Type>::isFormatterDefined() const {
	return (m_formatter != 0);
}

template<typename Type>
void Formatter<Type>::unsetFormatFunction() {
	delete m_formatter;
	m_formatter = 0;
}

template<typename Type>
std::string Formatter<Type>::format(Type value) const {

	if (m_formatter != 0) {
		return m_formatter->execute(value);
	}

	return "";
}


#endif
