/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PropertyContainer.h"
#include "../PropertyManager.h"
#include "BaseProperty.h"
#include "BaseDynamicProperty.h"

using namespace std;

PropertyContainer::PropertyContainer() {
}

PropertyContainer::PropertyContainer(const PropertyContainer& container) {
	// do not copy the maps
}

void PropertyContainer::registerProperty(const std::string& propertyName, BaseProperty * property) {
	m_properties[propertyName] = property;
}

void PropertyContainer::registerDynamicProperty(const std::string& propertyName, BaseDynamicProperty * property) {
	m_dynamicProperties[propertyName] = property;
}

void PropertyContainer::registerDynamicProperty2(const std::string& propertyName, BaseDynamicProperty * property) {
	m_dynamicProperties2[propertyName] = property;
}

void PropertyContainer::unregisterProperty(const std::string& propertyName) {
	m_properties.erase(propertyName);
}

void PropertyContainer::unregisterDynamicProperty(const std::string& propertyName) {
	m_dynamicProperties.erase(propertyName);
}

void PropertyContainer::unregisterDynamicProperty2(const std::string& propertyName) {
	m_dynamicProperties2.erase(propertyName);
}

void PropertyContainer::formatValue(const std::string& propertyName) {

	PropertyManager::DynamicPropertyInfo info = PropertyManager::getDynamicPropertyInfo(propertyName);
	if (!info.isDynamic) {
		map<string, BaseProperty *>::iterator r = m_properties.find(propertyName);

		if (r != m_properties.end()) {
			r->second->formatValue();
		}

	} else {
		// test double dynamic property
		PropertyManager::DynamicPropertyInfo info2 = PropertyManager::getDynamicPropertyInfo(info.name);

		if (!info2.isDynamic) {
			// dynamic property 1
			map<string, BaseDynamicProperty *>::iterator r = m_dynamicProperties.find(info.name);

			if (r != m_dynamicProperties.end()) {
				r->second->formatValues();
			}

		} else {
			// dynamic property 2
			// warning, info2 is for index1 and info for index2
			map<string, BaseDynamicProperty *>::iterator r = m_dynamicProperties2.find(info2.name);

			if (r != m_dynamicProperties2.end()) {
				r->second->formatValues();
			}
		}
	}
}

void PropertyContainer::formatAllValues() {

	// iterate all the properties
	for (map<std::string, BaseProperty *>::const_iterator r = m_properties.begin(); r != m_properties.end(); ++r) {
		r->second->formatValue();
	}

	for (map<std::string, BaseDynamicProperty *>::const_iterator r = m_dynamicProperties.begin(); r != m_dynamicProperties.end(); ++r) {
		r->second->formatValues();
	}

	for (map<std::string, BaseDynamicProperty *>::const_iterator r = m_dynamicProperties2.begin(); r != m_dynamicProperties2.end(); ++r) {
		r->second->formatValues();
	}

}


