/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PropertyRefresher.h"
#include "../PropertyManager.h"
#include "BaseProperty.h"
#include "BaseDynamicProperty.h"
#include <common/output/Debug.h>

using namespace std;

PropertyRefresher::PropertyRefresher() :
	PropertyContainer() {
}

PropertyRefresher::PropertyRefresher(const PropertyRefresher& pr) :
	PropertyContainer(pr) {
}

void PropertyRefresher::refresh(const std::string& propertyName, const boost::any & value) {

	PropertyManager::DynamicPropertyInfo info = PropertyManager::getDynamicPropertyInfo(propertyName);
	if (!info.isDynamic) {
		map<string, BaseProperty *>::iterator r = m_properties.find(propertyName);

		if (r != m_properties.end()) {
			BaseRefresher * refresher = dynamic_cast<BaseRefresher *>(r->second);
			if (refresher != nullptr) {
				if (refresher->hasRefresher()) {
					common::Debug::ScopedCall call(getServantInfo(), propertyName, value);
					refresher->refresh(value);
				}
			} else {
				cerr << "Cannot call refresher on " << propertyName << endl;
			}
		}

	} else {
		// test double dynamic property
		PropertyManager::DynamicPropertyInfo info2 = PropertyManager::getDynamicPropertyInfo(info.name);

		if (!info2.isDynamic) {
			// dynamic property 1
			map<string, BaseDynamicProperty *>::iterator r = m_dynamicProperties.find(info.name);

			if (r != m_dynamicProperties.end()) {
				BaseDynamicRefresher * refresher = dynamic_cast<BaseDynamicRefresher *>(r->second);
				if (refresher != nullptr) {
					if (refresher->hasRefresher()) {
						common::Debug::ScopedCall call(getServantInfo(), propertyName, value);
						refresher->refresh(info.index, value);
					}
				} else {
					cerr << "Cannot call refresher on " << propertyName << endl;
				}
			}

		} else {
			// dynamic property 2
			// warning, info2 is for index1 and info for index2
			map<string, BaseDynamicProperty *>::iterator r = m_dynamicProperties2.find(info2.name);

			if (r != m_dynamicProperties2.end()) {
				BaseDynamicRefresher2 * refresher = dynamic_cast<BaseDynamicRefresher2 *>(r->second);
				if (refresher != nullptr) {
					if (refresher->hasRefresher()) {
						common::Debug::ScopedCall call(getServantInfo(), propertyName, value);
						refresher->refresh(info2.index, info.index, value);
					}
				} else {
					cerr << "Cannot call refresher on " << propertyName << endl;
				}
			}
		}
	}
}


