/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef PROPERTYREFRESHER_H
#define PROPERTYREFRESHER_H

#include "PropertyContainer.h"
#include "Refresher.h"
#include "DynamicRefresher.h"
#include "DynamicRefresher2.h"
#include <string>
#include <map>

class PropertyRefresher : public PropertyContainer {

	template<typename>
	friend class ValueProperty;

	template<typename>
	friend class ValueDynamicProperty;

	template<typename>
	friend class ValueDynamicProperty2;

public:
	PropertyRefresher();
	PropertyRefresher(const PropertyRefresher& pr);

	template<typename Type, typename Function, typename Object>
	void registerRefresher(Refresher<Type> & property, Function f, Object obj) {
		property.setRefresher(f, obj);
	}

	template<typename Type, typename Function, typename Object, typename Arg1>
	void registerRefresher(Refresher<Type> & property, Function f, Object obj, Arg1 arg1) {
		property.setRefresher(f, obj, arg1);
	}

	template<typename Type, typename Function, typename Object, typename Arg1, typename Arg2>
	void registerRefresher(Refresher<Type> & property, Function f, Object obj, Arg1 arg1, Arg2 arg2) {
		property.setRefresher(f, obj, arg1, arg2);
	}

	template<typename Type, typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3>
	void registerRefresher(Refresher<Type> & property, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3) {
		property.setRefresher(f, obj, arg1, arg2, arg3);
	}

	template<typename Type, typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3, typename Arg4>
	void registerRefresher(Refresher<Type> & property, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3, Arg4 arg4) {
		property.setRefresher(f, obj, arg1, arg2, arg3, arg4);
	}


	template<typename Type, typename Function, typename Object>
	void registerRefresher(DynamicRefresher<Type> & property, Function f, Object obj) {
		property.setRefresher(f, obj);
	}

	template<typename Type, typename Function, typename Object, typename Arg1>
	void registerRefresher(DynamicRefresher<Type> & property, Function f, Object obj, Arg1 arg1) {
		property.setRefresher(f, obj, arg1);
	}

	template<typename Type, typename Function, typename Object, typename Arg1, typename Arg2>
	void registerRefresher(DynamicRefresher<Type> & property, Function f, Object obj, Arg1 arg1, Arg2 arg2) {
		property.setRefresher(f, obj, arg1, arg2);
	}

	template<typename Type, typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3>
	void registerRefresher(DynamicRefresher<Type> & property, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3) {
		property.setRefresher(f, obj, arg1, arg2, arg3);
	}

	template<typename Type, typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3, typename Arg4>
	void registerRefresher(DynamicRefresher<Type> & property, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3, Arg4 arg4) {
		property.setRefresher(f, obj, arg1, arg2, arg3, arg4);
	}


	template<typename Type, typename Function, typename Object>
	void registerRefresher(DynamicRefresher2<Type> & property, Function f, Object obj) {
		property.setRefresher(f, obj);
	}

	template<typename Type, typename Function, typename Object, typename Arg1>
	void registerRefresher(DynamicRefresher2<Type> & property, Function f, Object obj, Arg1 arg1) {
		property.setRefresher(f, obj, arg1);
	}

	template<typename Type, typename Function, typename Object, typename Arg1, typename Arg2>
	void registerRefresher(DynamicRefresher2<Type> & property, Function f, Object obj, Arg1 arg1, Arg2 arg2) {
		property.setRefresher(f, obj, arg1, arg2);
	}

	template<typename Type, typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3>
	void registerRefresher(DynamicRefresher2<Type> & property, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3) {
		property.setRefresher(f, obj, arg1, arg2, arg3);
	}

	template<typename Type, typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3, typename Arg4>
	void registerRefresher(DynamicRefresher2<Type> & property, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3, Arg4 arg4) {
		property.setRefresher(f, obj, arg1, arg2, arg3, arg4);
	}

	void refresh(const std::string& propertyName, const boost::any & value);

	virtual std::string getServantInfo() const = 0;
};

#endif
