/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PropertyUpdater.h"
#include "../PropertyManager.h"
#include <common/output/Debug.h>
#include <iostream>

using namespace std;

PropertyUpdater::PropertyUpdater() {
}

PropertyUpdater::PropertyUpdater(const PropertyUpdater& u) {
	// Do not copy the maps.
}

PropertyUpdater::~PropertyUpdater() {

	for (map<pair<string, string>, Updater *>::iterator u = m_updaters.begin(); u != m_updaters.end(); ++u) {
		delete u->second;
	}

	for (map<pair<string, string>, DynamicUpdater *>::iterator u = m_dynamicUpdaters.begin(); u != m_dynamicUpdaters.end(); ++u) {
		delete u->second;
	}

	for (map<pair<string, string>, DynamicUpdater2 *>::iterator u = m_dynamicUpdaters2.begin(); u != m_dynamicUpdaters2.end(); ++u) {
		delete u->second;
	}
}

void PropertyUpdater::unregisterPropertyUpdaters(const std::string& servantName) {

	// Updaters.
	{
		map<pair<string, string>, Updater *>::iterator u = m_updaters.begin();
		while (u != m_updaters.end()) {
			if (u->first.first == servantName) {
				m_updaters.erase(u++);
//				cout << "Erased from updaters" << endl;
			} else {
			   ++u;
			}
		}
	}

	// Dynamic updaters.
	{
		map<pair<string, string>, DynamicUpdater *>::iterator u = m_dynamicUpdaters.begin();
		while (u != m_dynamicUpdaters.end()) {
			if (u->first.first == servantName) {
				m_dynamicUpdaters.erase(u++);
//				cout << "Erased from dynamic updaters" << endl;
			} else {
			   ++u;
			}
		}
	}

	// Dynamic updaters 2.
	{
		map<pair<string, string>, DynamicUpdater2 *>::iterator u = m_dynamicUpdaters2.begin();
		while (u != m_dynamicUpdaters2.end()) {
			if (u->first.first == servantName) {
				m_dynamicUpdaters2.erase(u++);
//				cout << "Erased from dynamic updaters 2" << endl;
			} else {
			   ++u;
			}
		}
	}

}

void PropertyUpdater::unregisterPropertyUpdater(const std::string& servantName, const std::string& propertyName) {

	// Search in updaters.
	{
		map<pair<string, string>, Updater *>::iterator u = m_updaters.find(pair<string, string>(servantName, propertyName));

		if (u != m_updaters.end()) {
			m_updaters.erase(u);
			//cout << "Erased from updaters" << endl;
			return;
		}
	}

	// Search in dynamic updaters, propertyName does not contain the id.
	{
		map<pair<string, string>, DynamicUpdater *>::iterator u = m_dynamicUpdaters.find(pair<string, string>(servantName, propertyName));

		if (u != m_dynamicUpdaters.end()) {
			m_dynamicUpdaters.erase(u);
			//cout << "Erased from dynamic updaters" << endl;
			return;
		}
	}

	// Search in double dynamic updaters, propertyName does not contain the ids.
	{
		map<pair<string, string>, DynamicUpdater2 *>::iterator u = m_dynamicUpdaters2.find(pair<string, string>(servantName, propertyName));

		if (u != m_dynamicUpdaters2.end()) {
			m_dynamicUpdaters2.erase(u);
			//cout << "Erased from dynamic updaters 2" << endl;
			return;
		}
	}
}

void PropertyUpdater::updateServantProperty(const std::string& servantName, const std::string& propertyName) {

	PropertyManager::DynamicPropertyInfo info = PropertyManager::getDynamicPropertyInfo(propertyName);
	if (!info.isDynamic) {
		map<pair<string, string>, Updater *>::iterator u = m_updaters.find(pair<string, string>(servantName, propertyName));

		if (u != m_updaters.end()) {
			common::Debug::ScopedCall call(getServantInfo(), propertyName, servantName);
			u->second->update();
		}
	} else {
		// Check double dynamic property.
		PropertyManager::DynamicPropertyInfo info2 = PropertyManager::getDynamicPropertyInfo(info.name);

		if (!info2.isDynamic) {
			// Dynamic property 1.
			map<pair<string, string>, DynamicUpdater *>::iterator u = m_dynamicUpdaters.find(pair<string, string>(servantName, info.name));

			if (u != m_dynamicUpdaters.end()) {
				common::Debug::ScopedCall call(getServantInfo(), propertyName, servantName);
				u->second->update(info.index);
			}

		} else {
			// Dynamic property 2.
			// Warning, info2 is for index1 and info for index2.
			map<pair<string, string>, DynamicUpdater2 *>::iterator u = m_dynamicUpdaters2.find(pair<string, string>(servantName, info2.name));

			if (u != m_dynamicUpdaters2.end()) {
				common::Debug::ScopedCall call(getServantInfo(), propertyName, servantName);
				u->second->update(info2.index, info.index);
			}

		}
	}
}

