/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef PROPERTYUPDATER_H
#define PROPERTYUPDATER_H

#include "Updater.h"
#include "DynamicUpdater.h"
#include "DynamicUpdater2.h"
#include <string>
#include <map>

class PropertyUpdater {

	friend class ExperimentController;

	template<typename>
	friend class ControllerPtr;

public:
	PropertyUpdater();
	PropertyUpdater(const PropertyUpdater& u);
	virtual ~PropertyUpdater();

	virtual std::string getServantInfo() const = 0;

	void updateServantProperty(const std::string& servantName, const std::string& propertyName);

private:
	void unregisterPropertyUpdaters(const std::string& servantName);
	void unregisterPropertyUpdater(const std::string& servantName, const std::string& propertyName);

	template<typename Function, typename Object>
	void registerPropertyUpdater(const std::string& servantName, const std::string& propertyName, Function f, Object obj) {

		Updater * updater = new Updater();
		updater->setUpdater(f, obj);

		m_updaters[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1>
	void registerPropertyUpdater(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1) {

		Updater * updater = new Updater();
		updater->setUpdater(f, obj, arg1);

		m_updaters[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1, typename Arg2>
	void registerPropertyUpdater(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1, Arg2 arg2) {

		Updater * updater = new Updater();
		updater->setUpdater(f, obj, arg1, arg2);

		m_updaters[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3>
	void registerPropertyUpdater(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3) {

		Updater * updater = new Updater();
		updater->setUpdater(f, obj, arg1, arg2, arg3);

		m_updaters[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3, typename Arg4>
	void registerPropertyUpdater(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3, Arg4 arg4) {

		Updater * updater = new Updater();
		updater->setUpdater(f, obj, arg1, arg2, arg3, arg4);

		m_updaters[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object>
	void registerDynamicPropertyUpdater(const std::string& servantName, const std::string& propertyName, Function f, Object obj) {

		DynamicUpdater * updater = new DynamicUpdater();
		updater->setUpdater(f, obj);

		m_dynamicUpdaters[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1>
	void registerDynamicPropertyUpdater(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1) {

		DynamicUpdater * updater = new DynamicUpdater();
		updater->setUpdater(f, obj, arg1);

		m_dynamicUpdaters[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1, typename Arg2>
	void registerDynamicPropertyUpdater(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1, Arg2 arg2) {

		DynamicUpdater * updater = new DynamicUpdater();
		updater->setUpdater(f, obj, arg1, arg2);

		m_dynamicUpdaters[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3>
	void registerDynamicPropertyUpdater(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3) {

		DynamicUpdater * updater = new DynamicUpdater();
		updater->setUpdater(f, obj, arg1, arg2, arg3);

		m_dynamicUpdaters[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3, typename Arg4>
	void registerDynamicPropertyUpdater(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3, Arg4 arg4) {

		DynamicUpdater * updater = new DynamicUpdater();
		updater->setUpdater(f, obj, arg1, arg2, arg3, arg4);

		m_dynamicUpdaters[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object>
	void registerDynamicPropertyUpdater2(const std::string& servantName, const std::string& propertyName, Function f, Object obj) {

		DynamicUpdater2 * updater = new DynamicUpdater2();
		updater->setUpdater(f, obj);

		m_dynamicUpdaters2[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1>
	void registerDynamicPropertyUpdater2(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1) {

		DynamicUpdater2 * updater = new DynamicUpdater2();
		updater->setUpdater(f, obj, arg1);

		m_dynamicUpdaters2[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1, typename Arg2>
	void registerDynamicPropertyUpdater2(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1, Arg2 arg2) {

		DynamicUpdater2 * updater = new DynamicUpdater2();
		updater->setUpdater(f, obj, arg1, arg2);

		m_dynamicUpdaters2[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3>
	void registerDynamicPropertyUpdater2(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3) {

		DynamicUpdater2 * updater = new DynamicUpdater2();
		updater->setUpdater(f, obj, arg1, arg2, arg3);

		m_dynamicUpdaters2[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	template<typename Function, typename Object, typename Arg1, typename Arg2, typename Arg3, typename Arg4>
	void registerDynamicPropertyUpdater2(const std::string& servantName, const std::string& propertyName, Function f, Object obj, Arg1 arg1, Arg2 arg2, Arg3 arg3, Arg4 arg4) {

		DynamicUpdater2 * updater = new DynamicUpdater2();
		updater->setUpdater(f, obj, arg1, arg2, arg3, arg4);

		m_dynamicUpdaters2[std::pair<std::string, std::string>(servantName, propertyName)] = updater;
	}

	std::map<std::pair<std::string, std::string>, Updater *> m_updaters;
	std::map<std::pair<std::string, std::string>, DynamicUpdater *> m_dynamicUpdaters;
	std::map<std::pair<std::string, std::string>, DynamicUpdater2 *> m_dynamicUpdaters2;
};

#endif
