/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "StatusProperty.h"

const std::string StatusProperty::IDLE_STR = "Idle";
const std::string StatusProperty::RUNNING_STR = "Running";
const std::string StatusProperty::WARNING_STR = "Warning";
const std::string StatusProperty::ERROR_STR = "Error";
const std::string StatusProperty::FIXED_STR = "Fixed";
const std::string StatusProperty::UNKNOWN_STR = "Unknown";


std::string StatusProperty::getServantName() {
	return m_servantProperty->getServantName();
}

void StatusProperty::set(const StatusProperty& property) {
	m_servantProperty->updateValue(boost::any_cast<int32>(property.m_property->getValue()));
	formatValue();
}

void StatusProperty::setIdle() {
	m_servantProperty->updateValue((int32) IDLE, IDLE_STR);
}
void StatusProperty::setRunning() {
	m_servantProperty->updateValue((int32) RUNNING, RUNNING_STR);
}
void StatusProperty::setWarning() {
	m_servantProperty->updateValue((int32) WARNING, WARNING_STR);
}
void StatusProperty::setError() {
	m_servantProperty->updateValue((int32) ERROR, ERROR_STR);
}
void StatusProperty::setFixed() {
	m_servantProperty->updateValue((int32) FIXED, FIXED_STR);
}
void StatusProperty::setUnknown() {
	m_servantProperty->updateValue((int32) UNKNOWN, UNKNOWN_STR);
}

bool StatusProperty::isIdle() const {
	return (boost::any_cast<int32>(m_property->getValue()) == IDLE);
}
bool StatusProperty::isRunning() const {
	return (boost::any_cast<int32>(m_property->getValue()) == RUNNING);
}
bool StatusProperty::isWarning() const {
	return (boost::any_cast<int32>(m_property->getValue()) == WARNING);
}
bool StatusProperty::isError() const {
	return (boost::any_cast<int32>(m_property->getValue()) == ERROR);
}
bool StatusProperty::isFixed() const {
	return (boost::any_cast<int32>(m_property->getValue()) == FIXED);
}
bool StatusProperty::isUnknown() const {
	return (boost::any_cast<int32>(m_property->getValue()) == UNKNOWN);
}

int32 StatusProperty::operator()() const {
	return boost::any_cast<int32>(m_property->getValue());
}

int32 StatusProperty::get() const {
	return boost::any_cast<int32>(m_property->getValue());
}

void StatusProperty::formatValue() {
	std::ostringstream s;
	toString(s);
	m_servantProperty->updateValue(get(), s.str());
}

common::LogElement StatusProperty::toPropertyLog(bool hidden) const {

	std::ostringstream s;
	toString(s);

	std::string name;
	if (getAlias() != "") {
		name = getAlias();
	} else {
		name = getName();
	}

	std::string qualifier = common::LogElement::ACTUAL;
	if (getQualifier() == BaseProperty::SETPOINT) {
		qualifier = common::LogElement::SETPOINT;
	} else if (getQualifier() == BaseProperty::OFFSET) {
		qualifier = common::LogElement::OFFSET;
	}

	return common::LogElement(true, hidden, name, PropertyTypeToString<int32>::value, qualifier, "", s.str());
}

std::ostream& operator<<(std::ostream& s, const StatusProperty& property) {
	return property.toString(s);
}

