/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef STRINGLENGTH_H
#define STRINGLENGTH_H

#include <string>
#include <iostream>
#include <common/output/OutputStream.h>

enum ERROR_TYPE {COMPILATION_ERROR, EXECUTION_WARNING};

/**
 * Class template to control at compile-time the size of strings.
 */
template<int Error, int Min, int Max, int Current>
struct StringLength;


template<int Min, int Max, int Current>
struct StringLength<COMPILATION_ERROR, Min, Max, Current> {
	static const std::string& value(const std::string& name, const std::string& s) {
		return StringLength<COMPILATION_ERROR, Min, Max, Current - 1>::value(name, s);
	}
};

template<int Min, int Max>
struct StringLength<COMPILATION_ERROR, Min, Max, Min> {
	static const std::string& value(const std::string& name, const std::string& s) {
		return s;
	}
};

template<int Min, int Max>
struct StringLength<COMPILATION_ERROR, Min, Max, Max>;

template<int Min, int Max>
struct StringLength<COMPILATION_ERROR, Min, Max, -1>;


template<int Min, int Max, int Current>
struct StringLength<EXECUTION_WARNING, Min, Max, Current> {
	static const std::string& value(const std::string& name, const std::string& s) {
		if (Current > Max - 1) {
			common::warn << "Alias '" << s << "' for '" << name << "' has size " << Current << " greater than " << Max - 1 << common::eol;
		} else if (Current < Min) {
			common::warn << "Alias '" << s << "' for '" << name << "' has size " << Current << " smaller than " << Min << common::eol;
		}

		return s;
	}
};


template<int Error, int Min, int Max, int Length>
std::string checkString(const std::string& name, const char(&p)[Length]) {
	return StringLength<Error, Min, Max + 1, Length - 1>::value(name, p);
}

#endif
