/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Unit.h"
#include "ValueProperty.h"

ConstantUnit::ConstantUnit(const std::string& value)
: m_value(value) {}

BaseUnit * ConstantUnit::clone() const {
	return new ConstantUnit(*this);
}

std::string ConstantUnit::getValue() const {
	return m_value;
}

PropertyUnit::PropertyUnit(const ValueProperty<std::string> & property)
: m_property(property) {}

BaseUnit * PropertyUnit::clone() const {
	return new PropertyUnit(*this);
}

std::string PropertyUnit::getValue() const {
	return m_property.get();
}

Unit::Unit()
: m_unit(new ConstantUnit(""))	{
}

Unit::Unit(const Unit & unit) :
	m_unit(unit.m_unit->clone()) {
}

Unit::~Unit() {
	delete m_unit;
}

void Unit::copy(const Unit & unit) {
	m_unit = unit.m_unit->clone();
}

void Unit::setUnit(const std::string & value) {

	delete m_unit;
	m_unit = new ConstantUnit(value);
}

void Unit::setUnit(const ValueProperty<std::string> & property) {

	delete m_unit;
	m_unit = new PropertyUnit(property);
}

std::string Unit::getUnit() const {
	return m_unit->getValue();
}
