/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef UNIT_H
#define UNIT_H

#include <string>

template<typename>
class ValueProperty;

class BaseUnit {

public:
	virtual ~BaseUnit(){}
	virtual BaseUnit * clone() const = 0;
	virtual std::string getValue() const = 0;
};

class ConstantUnit : public BaseUnit {

public:
	ConstantUnit(const std::string& value);

	virtual BaseUnit * clone() const;
	virtual std::string getValue() const;

private:
	std::string m_value;
};

class PropertyUnit : public BaseUnit {

public:
	PropertyUnit(const ValueProperty<std::string> & property);

	virtual BaseUnit * clone() const;
	virtual std::string getValue() const;

private:
	const ValueProperty<std::string> & m_property;
};

class Unit {

public:
	Unit();
	Unit(const Unit & unit);

	virtual ~Unit();

protected:
	void copy(const Unit & unit);
	void setUnit(const std::string & value);
	void setUnit(const ValueProperty<std::string> & property);

	virtual std::string getUnit() const;

private:
	BaseUnit * m_unit;
};

#endif
