/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef PUBLISHER_H
#define PUBLISHER_H

#include <common/base/BaseTypes.h>
#include <string>
#include <list>
#include <mutex>

class Subscriber;
class ChangeAspect;

class Publisher {

public:

	/**
	 * Constructor
	 */
	Publisher();

	/**
	 * Destructor
	 */
	virtual ~Publisher();

	/**
	 * To get the unique ID of the publisher
	 */
	virtual const std::string& getId();

	/**
	 * To register a new subscriber
	 */
	virtual void attach(Subscriber* subscriber);

	/**
	 * To unregister a subscriber
	 */
	virtual void detach(Subscriber* subscriber);

	/**
	 * To notify subscribers that a change occured in the publisher
	 */
	virtual void notify(ChangeAspect* changeAspect);

	/**
	 * To check if some subscribers have registerd to the publisher
	 */
	virtual bool isAttached() {
		return m_attached;
	}

protected:
	static std::recursive_mutex m_publisherMutex;

private:
	Publisher(const Publisher&);

	std::list<Subscriber*> m_subscribers;

	std::string m_publisherId;
	static std::string m_idPrefix;
	static int32 m_idIndex;
	bool m_attached;
};

inline const std::string& Publisher::getId() {
	return m_publisherId;
}

#endif
