/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DataProvider/Repeater.h"
#include "DataProvider/ChangeAspectType.h"
#include "DataProvider/ChangeAspect.h"
#include "DataProvider/ChangeAspectFactory.h"

Repeater::Repeater() :
	m_activeCounter(0) {

	m_repeaterId = Publisher::getId() + Subscriber::getId();
}

Repeater::~Repeater() {
}

void Repeater::notify(ChangeAspect* changeAspect) {

	if (changeAspect == NULL) {
		return;
	}

	Publisher::notify(changeAspect);
}

void Repeater::update(ChangeAspect* changeAspect) {

	/*
	 * Determine the type of Change Aspect and call the relevant
	 * method (subclassed to specific cases).
	 */
	ChangeAspect* currentChangeAspect = changeAspect;
	ChangeAspect* last = 0;

	if (true) {
		/*
		 * Lock the publisher during an update: problems occur when a subscriber
		 * does a detach but we are in the middle of calling the updateProperty/
		 * updateProgress methods.
		 */
		std::scoped_lock pubLock(m_publisherMutex);

		do {
			switch (currentChangeAspect->getType()) {
			case PROPERTY:
				updateServantProperty(currentChangeAspect);
				break;

			case PROGRESS:
				updateProgress(currentChangeAspect);
				break;

			case ERROR:
				updateError(currentChangeAspect);
				break;

			case LOG_CHANGE:
				Publisher::notify(currentChangeAspect);

			default:
				break;
			}
			last = currentChangeAspect;
			currentChangeAspect = currentChangeAspect->getNext();
		} while (last->hasNext());
	}
}

void Repeater::setActive() {
	m_activeCounter++;

	/*
	 * If we become active then perform some specific operations.
	 */
	if (m_activeCounter == 1) {
		activate();
	}
}

void Repeater::unsetActive() {
	m_activeCounter--;

	/*
	 * Something went wrong... just in case.
	 */
	if (m_activeCounter < 0)
		m_activeCounter = 0;

	/*
	 * if we are now deactivated then perform specific ops
	 */
	if (m_activeCounter == 0) {
		deactivate();
	}

}

bool Repeater::isActive() {
	if (m_activeCounter == 0) {
		return false;
	} else {
		return true;
	}
}

void Repeater::updateServantProperty(ChangeAspect* changeAspect) {
	std::string propertyName = changeAspect->getStringParam();
	std::string controllerName = changeAspect->getControllerName();
	PropertyUpdater::updateServantProperty(controllerName, propertyName);
	updateProperty(changeAspect);
}
