/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef REPEATER_H
#define REPEATER_H

#include "DataProvider/Publisher.h"
#include "DataProvider/Subscriber.h"
#include "DataProvider/Property/TypedProperty/PropertyUpdater.h"

class ChangeAspectFactory;
class ChangeAspect;

class Repeater: public Publisher, public Subscriber, public PropertyUpdater {
public:
	/**
	 * Constructor
	 */
	Repeater();

	/**
	 * Destructor
	 */
	virtual ~Repeater();

	/**
	 * Publisher::notify redefinition
	 */
	virtual void notify(ChangeAspect*);

	/**
	 * Subscriber::update() implementation;
	 */
	virtual void update(ChangeAspect*);

	virtual const std::string& getId();

	/**
	 * If we want to have the option of only forwarding ChangeAspects
	 * during a command (for example) we can use the "active" flag (or
	 * counter). It also allows us to place the Device Drivers onto
	 * the Observer thread for more rapid updates. The flag is not used
	 * in this class but a sub-classed repeater takes advantage of it.
	 */
	void setActive();
	void unsetActive();

protected:
	virtual void updateProperty(ChangeAspect* ca) {
	}
	virtual void updateProgress(ChangeAspect* ca) {
	}
	virtual void updateError(ChangeAspect* ca) {
	}

	virtual void activate() = 0;
	virtual void deactivate() = 0;

	bool isActive();

private:
	void updateServantProperty(ChangeAspect* changeAspect);

	std::string m_repeaterId;

	int32 m_activeCounter;
	std::recursive_mutex m_updateMutex;
};

inline const std::string& Repeater::getId() {
	return m_repeaterId;
}

#endif
