/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DataProvider/ServantManager.h"
#include <command/transportlayer/MultiThreadProcessorCommunication.h>
#include <command/transportlayer/dataprovider/ServantAccessor.h>

using namespace std;
using namespace command;

ServantManager::ServantManager(const std::string& name) :
	_name(name),
	_servantID(0),
	_servantAccessor(0) {

	if (MultiThreadProcessorCommunication::getInstance() != 0) {
		_servantID = (uint32) MultiThreadProcessorCommunication::getInstance()->getServantAccessor()->getServantID(name);
		_servantAccessor = MultiThreadProcessorCommunication::getInstance()->getServantAccessor();
	}
}

ServantManager::~ServantManager() {
}

ServantManager* ServantManager::clone(const std::string& name) const {

	ServantManager* manager = new ServantManager(name);

	return manager;
}

void ServantManager::renameServant(const std::string& name) {
	_name = name;
}

void ServantManager::enable() {

	if (_servantAccessor != 0) {
		_servantAccessor->enableServant(_servantID, false);
	}
}

void ServantManager::disable() {

	if (_servantAccessor != 0) {
		_servantAccessor->disableServant(_servantID, false);
	}
}

void ServantManager::setVisible(bool value) {

	if (_servantAccessor != 0) {
		_servantAccessor->setServantVisible(_servantID, value);
	}
}

void ServantManager::setFavourite(bool value) {

	if (_servantAccessor != 0) {
		_servantAccessor->setServantFavourite(_servantID, value);
	}
}

void ServantManager::setCommand(bool value) {

	if (_servantAccessor != 0) {
		_servantAccessor->setServantCommand(_servantID, value);
	}
}

void ServantManager::enableConfigurationChanged() {

	if (_servantAccessor != 0) {
		_servantAccessor->enableConfigurationChanged();
	}
}

void ServantManager::disableConfigurationChanged() {

	if (_servantAccessor != 0) {
		_servantAccessor->disableConfigurationChanged();
	}
}

void ServantManager::configurationChanged() {

	if (_servantAccessor != 0) {
		_servantAccessor->configurationChanged();
	}
}

void ServantManager::setFamily(const std::string& familyName, const std::string& subfamilyName) {

	if (_servantAccessor != 0) {
		_servantAccessor->setServantFamily(_servantID, familyName, subfamilyName);
	}
}

std::string ServantManager::getFamily() const {

	if (_servantAccessor != 0) {
		return _servantAccessor->getServantFamily(_servantID);
	}

	return "";
}

std::string ServantManager::getSubfamily() const {

	if (_servantAccessor != 0) {
		return _servantAccessor->getServantSubfamily(_servantID);
	}

	return "";
}

bool ServantManager::isFavourite() const {

	if (_servantAccessor != 0) {
		return _servantAccessor->isServantFavourite(_servantID);
	}

	return false;
}
