/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef SUBSCRIBER_H
#define SUBSCRIBER_H

#include <common/base/BaseTypes.h>

class ChangeAspect;

/**
 * This class is part of the publisher/subscriber pattern
 */

class Subscriber {

public:
	/**
	 * Destructor
	 */
	virtual ~Subscriber();

	/**
	 * To get the unique Id of Subscriber
	 */
	virtual const std::string& getId() const;

	/**
	 *
	 */
	bool operator==(const Subscriber&) const;

	/**
	 * Method used by the publishers to notify changes
	 */
	virtual void update(ChangeAspect* changeAspect) = 0;

protected:
	Subscriber();

private:
	std::string m_subscriberId;
	static std::string m_idPrefix;
	static int32 m_idIndex;

};

inline const std::string& Subscriber::getId() const {
	return m_subscriberId;
}

inline bool Subscriber::operator==(const Subscriber& s) const {
	bool result = false;
	//cout << "s.getId() " << s.getId() << " getId() " << getId() << endl;
	if (s.getId() == getId()) {
		result = true;
	}
	return result;
}

#endif
