/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DeviceDriverCounter.h"

DeviceDriverCounter::DeviceDriverCounter(DeviceDriver* deviceDriver, int32 period) :
	m_deviceDriver(deviceDriver), m_period(period), m_counter(0), m_storedCounter(0), m_active(true) {
}


DeviceDriver* DeviceDriverCounter::getDeviceDriver() const {
	return m_deviceDriver;
}

int32 DeviceDriverCounter::getPeriod() const {
	return m_period;
}

void DeviceDriverCounter::setPeriod(int32 period) {
	m_period = period;
}

int32 DeviceDriverCounter::getCounter() const {
	return m_counter;
}

void DeviceDriverCounter::incrementCounter() {
	if (m_active) {
		m_counter++;
	}
}

void DeviceDriverCounter::resetCounter() {
	m_counter = 0;
}

void DeviceDriverCounter::activate() {
	m_active = true;
	m_counter = m_storedCounter;
}

void DeviceDriverCounter::deactivate() {
	m_active = false;
	m_storedCounter = m_counter;
	m_counter = -1;
}
