/*
 * Nomad Instrument Control Software
 * Copyright 2011 Institut Laue-Langevin
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
*/
#include "DeviceContainer/DeviceDriverCreator.h"
#include "Module/ModuleManager.h"

// Trace include
#include "LoggingServices/Level.h"














































































































































































#include <string>
#include <iostream>
#include <fstream>

using namespace boost;
using namespace std;

#define NEW_DRIVER(N, D) if (aType==N) {if (create) {driver = new D(aType,aName,catlist,aRunningMode);} else {m_typeClassMap[aType].cppClass = #D;m_typeClassMap[aType].cppClassName = #N;}}
#define ADD_CATEGORY1(N, C) if (aType==N) {catlist.push_back(C);m_typeClassMap[aType].categories = std::string( #C );}
#define ADD_CATEGORY2(N, C1, C2) if (aType==N) {catlist.push_back(C1);catlist.push_back(C2);m_typeClassMap[aType].categories = std::string( #C1 ) + ", " + std::string( #C2 );}
#define ADD_CATEGORY3(N, C1, C2, C3) if (aType==N) {catlist.push_back(C1);catlist.push_back(C2);catlist.push_back(C3);m_typeClassMap[aType].categories = std::string( #C1 ) + ", " + std::string( #C2 ) + ", " + std::string( #C3 );}
#define ADD_CATEGORY4(N, C1, C2, C3, C4) if (aType==N) {catlist.push_back(C1);catlist.push_back(C2);catlist.push_back(C3);catlist.push_back(C4);m_typeClassMap[aType].categories = std::string( #C1 ) + ", " + std::string( #C2 ) + ", " + std::string( #C3 ) + ", " + std::string( #C4 );}
#define ADD_CATEGORY5(N, C1, C2, C3, C4, C5) if (aType==N) {catlist.push_back(C1);catlist.push_back(C2);catlist.push_back(C3);catlist.push_back(C4);catlist.push_back(C5);m_typeClassMap[aType].categories = std::string( #C1 ) + ", " + std::string( #C2 ) + ", " + std::string( #C3 ) + ", " + std::string( #C4 ) + ", " + std::string( #C5 );}

// dummy class
class DummyDriver : public DeviceDriver {
public:
	DummyDriver(const string& aType, const string& aName, const vector<string>& aCategory, const int32 aRunningMode) {}
	virtual void execute(const string& aCommand) {}
};

/*
 * Construtor
 */
DeviceDriverCreator::DeviceDriverCreator() {
}

/*
 * getInstance
 */
DeviceDriverCreator* DeviceDriverCreator::getInstance()
{
  if (m_instance == 0) {
    m_instance = new DeviceDriverCreator();
  }
  return m_instance;
}

/*
 * instance
 */
DeviceDriverCreator* DeviceDriverCreator::m_instance= 0;

void DeviceDriverCreator::reset()
{
  delete m_instance;
}

void DeviceDriverCreator::initFromModules() {

	vector<string> moduleDriverTypes = ModuleManager::getInstance()->getAllDriverTypes();

	for (vector<string>::const_iterator d = moduleDriverTypes.begin(); d != moduleDriverTypes.end(); ++d) {
		m_DriverTypes.push_back(*d);
	}
}

/*
 * getKnownIDs
 */
string DeviceDriverCreator::getKnownIDs() const
{
  ostringstream idList;

  

  for (uint32 i=0;i<m_DriverTypes.size();++i) {
    idList << m_DriverTypes[i] << endl;
  }

  return idList.str();
}

/*
 * verifyType
 */
bool DeviceDriverCreator::verifyType(const string& aDriverID) const
{
  bool found = false;

  

  for (uint32 i=0;i<m_DriverTypes.size();++i) {
    if (m_DriverTypes[i] == aDriverID) {
      found = true;
    }
  }

  return found;
}

/*
 * createDeviceDriver
 */
DeviceDriver* DeviceDriverCreator::create(const string& aType,
					  const string& aName,
					  const uint32 aRunningMode,
					  bool create)
{
  DeviceDriver* driver=0;
  vector<string> catlist;

  


  if ((aRunningMode != NORMAL_MODE_DEVICE_CONTAINER) &&
      (aRunningMode != FUNTIONALTEST_MODE_DEVICE_CONTAINER) &&
      (aRunningMode != UNITARYTEST_MODE_DEVICE_CONTAINER)) {
    // Error bad mode
  }
  else
  {
	  // calling the module manager
	  driver = ModuleManager::getInstance()->makeDriver(aType, aName);
	  driver->setRunningMode(aRunningMode);

	  if (driver == 0) {
		  // error bad device type
		  throw NotOk(DEVICEDRIVER_TYPE_ERROR);
	  }
  }

  return driver;
}

/*
 * getDeviceDriverList
 */
const vector<string>& DeviceDriverCreator::getDeviceDriverList() const
{
	return m_DriverTypes;

}

std::map<std::string, DeviceDriverCreator::TypeInfo> DeviceDriverCreator::retrieveAllTypeClasses() {

	for (vector<string>::const_iterator t = m_DriverTypes.begin(); t != m_DriverTypes.end(); t++) {
		try {
			create(*t, "driver", 0, false);

		} catch (...) {
		}
	}

	return m_typeClassMap;

}
