/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DEVICEDRIVERSTATE_H
#define DEVICEDRIVERSTATE_H

#include <time.h>

#include <common/base/BaseTypes.h>
#include "DeviceContainer/DeviceDriver.h"

/**
 * Interface class.
 */
class DeviceDriverState {

public:
	/**
	 * Constructor
	 *
	 * @param the class which used the state
	 */
	DeviceDriverState(DeviceDriver* downer) :
		m_owner(downer) {
	};

	/**
	 * Destructor
	 */
	virtual ~DeviceDriverState() {
	};

	/**
	 * Init function.
	 */
	virtual void init() = 0;

protected:
	/**
	 * Link to the class which used the state
	 * Need to access properties and protocol
	 * @label owner
	 */
	DeviceDriver* m_owner;
};

template<typename DriverType>
class DriverState : public DeviceDriverState {

public:
	DriverState(DriverType * owner) : DeviceDriverState(owner) {
		m_typedOwner = owner;
	}

	inline DriverType * owner() {
		return m_typedOwner;
	}

private:
	DriverType * m_typedOwner;
};

#endif
