/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef HARDWARECOMMAND_H
#define HARDWARECOMMAND_H

//#include "Scheduler/Scheduler.h"
#include "Scheduler/Command.h"

#include <string>

class DeviceDriver;

/**
 * Concrete class implementing the command pattern for hardware.
 */
class HardwareCommand : public Command
{
public:
  HardwareCommand(DeviceDriver* aDeviceDriver = 0, const std::string& name = std::string("empty"));
  virtual ~HardwareCommand();

  virtual void execute();
  virtual void stop();
  virtual bool isBlocking(Command& aCommand);
  virtual bool reject(bool blocked);
  virtual AbstractController* getController() const;
  DeviceDriver* getDeviceDriver() const;
  virtual const std::string& getName() const;
  virtual bool equal(Command* c1);
  virtual std::string toString() const;

private:
  void checkDisconnected();
  DeviceDriver* m_deviceDriver;
  std::string m_name;
};

inline AbstractController* HardwareCommand::getController() const {
	return NULL;
}

std::ostream& operator<< (std::ostream& os, const HardwareCommand &c);

#endif
