/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef OBSERVER_H
#define OBSERVER_H

#include <common/base/BaseTypes.h>
#include <thread>
#include <string>
#include <vector>
#include <unistd.h>
#include <mutex>
#include <common/base/Date.h>

class DeviceDriverCounter;
class DeviceDriver;

class Observer {

public:
	static Observer* getInstance();
	static void resetInstance();

	void init();

	int32 getCycleDurationUs() const;

	void execute();

	std::string getPhase();

	void registerDeviceDriver(DeviceDriver* deviceDriver, int32 frequency);
	void unregisterDeviceDriver(DeviceDriver* deviceDriver);
	std::vector<DeviceDriverCounter*> getDeviceDriverCounters();
	//int32 getListDeviceDriverActivate();

	void printDurationSinceStart();

private:
	Observer();
	~Observer();

	void reset();

	static void loop();
	void loopPhases();

	void treatmentPhase();
	void writePhase();
	void readPhase();
	void controlCyclePhase();

	int32 _sleepTimeUs;
	std::string _phase;
	std::mutex _observerMutex;
	std::mutex _cmdObserverMutex;
	std::vector<DeviceDriverCounter*> _deviceDriverCounters;
	int32 _active;

	bool _running;
	std::thread* _thread;

	common::Date _startDate;

	static Observer* _instance;

	static const std::string OBSERVER_CYCLE_DURATION_US;
};

#endif //OBSERVER_H
