/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef SPY_H
#define SPY_H

#include <common/base/BaseTypes.h>
#include <thread>
#include <string>
#include <vector>
#include <unistd.h>
#include <mutex>

class DeviceDriver;
class DeviceDriverCounter;

class Spy {

public:
	static Spy* getInstance();
	static void resetInstance();

	void init();

	int32 getCycleDurationMs() const;

	void execute();
	void stop();

	std::string getPhase();

	void registerDeviceDriver(DeviceDriver* aDeviceDriver, int32 aFrequency);
	void unregisterDeviceDriver(DeviceDriver* aDeviceDriver);

	void activateDeviceDriver(DeviceDriver* aDeviceDriver);
	void deactivateDeviceDriver(DeviceDriver* aDeviceDriver);

	bool isInCharge() const;
	void inCharge();
	void notInCharge();

	std::vector<DeviceDriver*> getDeviceDriver();
	std::vector<DeviceDriver*> getListDeviceDriverActivate();
	std::vector<DeviceDriverCounter*> getDeviceDriverCounter();

private:
	Spy();
	~Spy();

	void reset();

	static void loop();
	void loopPhases();

	void treatmentPhase();
	void writePhase();
	void readPhase();
	void controlCyclePhase();

	int32 _sleepTimeMs;
	std::string _phase;
	std::mutex _spyMutex;
	std::mutex _cmdSpyMutex;

	std::vector<DeviceDriverCounter*> _deviceDriverCounters;
	std::vector<DeviceDriver*> _deactivationQueue;
	std::vector<DeviceDriver*> _activationQueue;
	bool _inCharge;

	bool _running;
	std::thread* _thread;

	static Spy* _instance;

	static const std::string SPY_CYCLE_DURATION_MS;
};

#endif
