/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef ERROR_H
#define ERROR_H

#include <vector>
#include <string>
#include <common/base/BaseTypes.h>

/**
 * @stereotype Abstract     throw(Exceptions::NotOk(UNKNOW_PROPERTY));

 */
class Error {
public:
	Error();
	~Error();
	void Lock();
	void Unlock();
	int32 getKeyId();
	void setKeyId(const int32 aKeyId);
	int32 getKeyFamily();
	void setKeyFamily(const int32 aKeyFamily);
	std::string getMasterMessage();
	void setMasterMessage(const std::string& aMasterMessage);
	int32 getStatus();
	void setStatus(const int32 aStatus);
	std::vector<std::string> getMessageLevel();
	void setMessageLevel(const std::vector<std::string>& aMessageLevel);
	std::vector<int32> getListChildrenMasked();
	void setListChildrenMasked(const std::vector<int32>& aListChildrenMasked);
	std::string getHelpLocation();
	void setHelpLocation(const std::string& aHelpLocation);
	std::string getAfterAction();
	void setAfterAction(const std::string& aAfterAction);
	void getDirective(int32 UserLevel);

	void print();

private:

	int32 m_KeyId;
	int32 m_KeyFamily;
	std::string m_MasterMessage;
	int32 m_Status;
	std::vector<std::string> m_MessageLevel;
	std::vector<int32> m_ListChildrenMasked;
	std::string m_HelpLocation;
	std::string m_AfterAction;
	bool m_ErrorLock;
};

inline int32 Error::getKeyId() {
	return m_KeyId;
}

inline void Error::setKeyId(const int32 aKeyId) {
	m_KeyId = aKeyId;
}
inline int32 Error::getKeyFamily() {
	return m_KeyFamily;
}

inline void Error::setKeyFamily(const int32 aKeyFamily) {
	m_KeyFamily = aKeyFamily;
}

inline std::string Error::getMasterMessage() {
	return m_MasterMessage;
}

inline void Error::setMasterMessage(const std::string& aMasterMessage) {
	m_MasterMessage = aMasterMessage;
}

inline int32 Error::getStatus() {
	return m_Status;
}

inline void Error::setStatus(const int32 aStatus) {
	m_Status = aStatus;
}

inline std::vector<std::string> Error::getMessageLevel() {
	return m_MessageLevel;
}

inline void Error::setMessageLevel(const std::vector<std::string>& aMessageLevel) {
	m_MessageLevel = aMessageLevel;
}

inline std::vector<int32> Error::getListChildrenMasked() {
	return m_ListChildrenMasked;
}

inline void Error::setListChildrenMasked(const std::vector<int32>& aListChildrenMasked) {
	m_ListChildrenMasked = aListChildrenMasked;
}

inline std::string Error::getHelpLocation() {
	return m_HelpLocation;
}
inline void Error::setHelpLocation(const std::string& aHelpLocation) {
	m_HelpLocation = aHelpLocation;

}

inline std::string Error::getAfterAction() {
	return m_AfterAction;
}
inline void Error::setAfterAction(const std::string& aAfterAction)

{
	m_AfterAction = aAfterAction;
}

inline void Error::Lock() {
	m_ErrorLock = true;
}

inline void Error::Unlock() {
	m_ErrorLock = false;
}

inline void Error::getDirective(int32 UserLevel) {
}

#endif //ERROR_H
