/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ErrorCatalogManager.h"
#include "Error.h"
#include "XMLErrorMessageCatalog.h"
#include "LoggingServices/Level.h"
#include "LoggingServices/Log.h"
#include "common/base/ServerProperties.h"
#include <string>

using namespace std;

ErrorCatalogManager* ErrorCatalogManager::_instance = 0;

void ErrorCatalogManager::initInstance() {
	if (_instance == 0) {
		_instance = new ErrorCatalogManager();
	}
}

ErrorCatalogManager::ErrorCatalogManager() {
	string home;
	home.append(common::ServerProperties::getInstance()->getConfigPath());

	string xmlFileError = home;
	xmlFileError.append("/.ErrorCatalog.xml");

	XMLErrorMessageCatalog myCatalog;

	myCatalog.getMapError(xmlFileError, _listError);
}

ErrorCatalogManager::~ErrorCatalogManager() {
	for(auto err : _listError) {
		delete err.second;
	}
}

ErrorCatalogManager* ErrorCatalogManager::getInstance() {
	return _instance;
}

void ErrorCatalogManager::reset() {
	delete _instance;
	_instance = 0;
}

int32 ErrorCatalogManager::addError(int32 ErrorID) {

	std::scoped_lock lock(_errorsMutex);

	map<int32, Error*>::iterator iter;

	if (!_listError.empty()) {
		for (iter = _listActiveError.begin(); iter != _listActiveError.end(); iter++) {
			for (size_t indx = 0; indx < iter->second->getListChildrenMasked().size(); indx++) {
				if ((iter->second->getListChildrenMasked())[indx] == ErrorID) {
					
					return 0;
				}
			}
		}

		if (_listError.find(ErrorID) != _listError.end()) {
			_listActiveError[ErrorID] = _listError[ErrorID];
			
		}
	}
	return 0;
}

int32 ErrorCatalogManager::deleteError(int32 ErrorID) {

	std::scoped_lock lock(_errorsMutex);

	map<int32, Error*>::iterator iter;

	if (_listActiveError.find(ErrorID) != _listActiveError.end()) {
		_listActiveError.erase(ErrorID);
		

	} else {
		for (iter = _listActiveError.begin(); iter != _listActiveError.end(); iter++) {
			for (size_t indx = 0; indx < iter->second->getListChildrenMasked().size(); indx++) {
				if ((iter->second->getListChildrenMasked())[indx] == ErrorID) {
					ErrorID = iter->first;
					return ErrorID;
				}
			}
		}

		_listActiveError.erase(ErrorID);
		
	}
	return 0;
}

Error*
ErrorCatalogManager::getError(int32 ErrorID) {

	std::scoped_lock lock(_errorsMutex);

	map<int32, Error*>::iterator iter;
	uint32 indx;

	if (_listActiveError.find(ErrorID) != _listActiveError.end()) {
		return _listActiveError[ErrorID];
	} else {
		for (iter = _listActiveError.begin(); iter != _listActiveError.end(); iter++) {
			for (indx = 0; indx < iter->second->getListChildrenMasked().size(); indx++) {
				if ((iter->second->getListChildrenMasked())[indx] == ErrorID) {
					
					return _listActiveError[iter->first];
				}
			}
		}
	}

	return _listError[1];
}
