/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef ERRORDEF_H
#define ERRORDEF_H

/**
 * General Errors 1 to 200
 */
static const uint32 UNKNOWN_ERROR = 1;
static const uint32 UNKNOWN_PROPERTY_ERROR = 2;
static const uint32 NOT_CONFIGURED_ERROR = 3;
static const uint32 NOTENOUGH_MEMORY_ERROR = 4;
static const uint32 ZERO_DIVISION_ERROR = 5;
static const uint32 UNKNOWN_COMMAND_ERROR = 6;
static const uint32 CONTROLLER_DISABLE_ERROR = 7;
static const uint32 DISCONNECTED_ERROR = 8;
static const uint32 COMMAND_REFUSED_ERROR = 9;
static const uint32 COMMAND_LINE_BAD_COMMAND = 10;
static const uint32 COMMAND_LINE_ILLEGAL_PARAM = 11;
static const uint32 COMMAND_LINE_DISACTIVED_PARAM = 12;

/**
 * Hardware Errors 201 to 400
 */

// Gate (StartStop)
static const uint32 BAD_GATE_MODE_ERROR = 201;
static const uint32 BAD_CLOCK_FREQUENCY_ERROR = 202;
static const uint32 LYNXQCL_FILE_ERROR = 203;
// Vme
static const uint32 VME_CRATEOFF_ERROR = 204;
static const uint32 VME_BUSERROR_ERROR = 205;
static const uint32 BAD_ADDRESSTYPE_ERROR = 206;
static const uint32 BAD_DATASIZETYPE_ERROR = 207;
static const uint32 VMEDEVICE_NOTLOADED_ERROR = 208;
static const uint32 DEVICEDRIVER_TYPE_ERROR = 209;

// Axis
static const uint32 AXIS_HIGH_HARD_STOP_ACTIVATED = 211;
static const uint32 AXIS_LOW_HARD_STOP_ACTIVATED = 212;
static const uint32 MANUAL_MODE_ACTIVATED = 213;
static const uint32 EMERGENCY_STOP_ACTIVATED = 214;

// RS232, RIO2ACQ, RIO3ACQ, RIOAXIS
static const uint32 TIME_OUT_ERROR = 215;
// MacDilution
static const uint32 MAC_DILUTION_ERROR = 216;
// Eurotherm 2804e , West
static const uint32 BAD_VALUE_ERROR = 217;
// Chopper Julich
static const uint32 CHOPPER_BAD_SPEED_ERROR = 218;
static const uint32 ABOVE_HIGH_LIMIT_ERROR = 219;
static const uint32 BELOW_LOW_LIMIT_ERROR = 220;
// GPIB
static const uint32 GPIB_BUSERROR_ERROR = 221;
static const uint32 GPIBDEVICE_NOTLOADED_ERROR = 222;
// Chopper Julich
static const uint32 CHOPPER_BAD_PHASE_ERROR = 223;

// RS232, GPIB
static const uint32 REMOTE_ERROR = 224;
// ALARM
static const uint32 ALARM_ERROR = 225;
// Figaro Langmuir mount directory "users/Nima" error
static const uint32 NIMA_PATH_ERROR = 226;
// Selector
static const uint32 SELECTOR_ERROR = 227;
// WARNING
static const uint32 WARNING_ERROR = 228;
// USER_FILES
static const uint32 USERFILES_ERROR = 229;
// Errors with values 225, 226, 227 and 228, 229 are defined in ControllerErrorDef (legacy).
static const uint32 SELECTOR_BAD_SPEED_ERROR = 230;
/**
 * Configuration Errors 401 to 500
 */
static const uint32 CHILD_BAD_TYPE_ERROR = 401;
static const uint32 CHILD_BAD_CHANNEL_ERROR = 402;
static const uint32 CHILD_BAD_CONF_ERROR = 403;
static const uint32 CHILD_IS_USED_ERROR = 404;
static const uint32 PARENT_BAD_TYPE_ERROR = 405;
static const uint32 PARENT_BAD_CONF_ERROR = 406;
static const uint32 PARENT_IS_USED_ERROR = 407;
static const uint32 NO_DEVICE_DRIVER_ERROR = 408;

static const uint32 BAD_TYPE_PROPERTY_ERROR = 411;
static const uint32 EXECUTION_ERROR = 412;

/**
 * Instrument abstraction errors 501 to 600
 */
static const uint32 SETTING_ERROR =503;
static const uint32 FIGARO_FOL_ERROR =504;
static const uint32 OPEN_FILE_ERROR =505;
static const uint32 FILE_SYNTAX_ERROR =506;
static const uint32 KINETIC_MAX_SLICES_ERROR =507;
static const uint32 KINETIC_SLICE_TIME_ERROR =508;
static const uint32 ACQ_TIME_OUT_ERROR =509;
// TAS
static const uint32 CANNOT_INVERSE_MATRIX_ERROR = 510;
static const uint32 TRIANGLECANNOTBECLOSED_ERROR = 511;
static const uint32 Q_NOT_INPLANE_ERROR = 512;
static const uint32 B_OUT_OF_RANGE_ERROR = 513;
static const uint32 BAD_GONIO_MODE_ERROR = 514;
static const uint32 UNREACHABLEK_ERROR = 515;
// NPP
static const uint32 CONFIG_FILE_ERROR = 516;
// Generic Scan
static const uint32 INCOMPATIBLE_NUMBER_OF_STEPS_ERROR = 517;
// LSS
static const uint32 STOP_DETECTOR_FORBIDDEN_ERROR = 518;
static const uint32 PAUSE_DETECTOR_FORBIDDEN_ERROR = 519;

/**
 * XML Errors 601 to 800
 */
static const uint32 XML_UNKNOWN_PROPERTIES_ERROR = 601;
static const uint32 XML_INIT_PROPERTIES_FILE_ERROR = 602;
static const uint32 XML_PARSING_PROPERTIES_FILE_ERROR = 603;
static const uint32 XML_OPEN_PROPERTIES_FILE_ERROR = 604;
static const uint32 XML_GET_PROPERTIES_ERROR = 605;

#endif //ERRORDEF_H
