/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ErrorsAlarmsManager/Exceptions.h"
#include "DataProvider/Publisher.h"
#include "DataProvider/ChangeAspectFactory.h"
#include "DataProvider/ChangeAspect.h"
#include "ErrorsAlarmsManager/ErrorCatalogManager.h"
#include "ErrorsAlarmsManager/Error.h"
#include "LoggingServices/Level.h"
#include "LoggingServices/Log.h"
#include <common/output/OutputStream.h>

using namespace common;
using namespace std;

Exceptions::Exceptions(int32  exceptionsID) :
	m_id(exceptionsID) {

	string MyMessage;
	if (exceptionsID != 0) {
		ErrorCatalogManager::getInstance()->addError(exceptionsID);
		string message = ErrorCatalogManager::getInstance()->getError(exceptionsID)->getMasterMessage();
	}
}

Exceptions::Exceptions(int32  exceptionsID, const std::string &i) :
	m_id(exceptionsID), info(i) {
	if (exceptionsID != 0) {
		ErrorCatalogManager::getInstance()->addError(exceptionsID);
	}
}

NotOk::NotOk(int32  ExceptionsID) :
	Exceptions(ExceptionsID) {

}

NotOk::NotOk(int32  ExceptionsID, const string &Info) :
	Exceptions(ExceptionsID, Info) {

}

NotOk::NotOk(const string& creatorType, const string& creatorName, Publisher* publisher, int32 exceptionsID) :
	Exceptions(exceptionsID) {

	ChangeAspectFactory changeAspectFactory;
	ChangeAspect* errorChangeAspect = changeAspectFactory.createErrorChangeAspect(creatorType, creatorName, exceptionsID);

	string message = ErrorCatalogManager::getInstance()-> getError(errorChangeAspect->getLongParam())->getMasterMessage();
	errorChangeAspect->setStringParam(message);
	publisher->notify(errorChangeAspect);
	if (exceptionsID != 0) {
		Log::getInstance()->log(creatorType, creatorName, Level::s_Error, string("Error: ") + message);
		err << message << eol;
	}
}

NotOk::NotOk(const string& creatorType, const string& creatorName, Publisher* publisher, int32 exceptionsID, const string& Info) :
	Exceptions(exceptionsID, Info) {

	ChangeAspectFactory changeAspectFactory;
	ChangeAspect* errorChangeAspect = changeAspectFactory.createErrorChangeAspect(creatorType, creatorName, exceptionsID);

	string message = ErrorCatalogManager::getInstance()-> getError(errorChangeAspect->getLongParam())->getMasterMessage();
	message.append(" -> ");
	message.append(Info);

	errorChangeAspect->setStringParam(message);

	publisher->notify(errorChangeAspect);
	if (exceptionsID != 0) {
		Log::getInstance()->log(creatorType, creatorName, Level::s_Error, string("Error: ") + creatorName + " " + message);
		err << creatorName << " -> " << message << eol;
	}

}

CannotSetValue::CannotSetValue() :
	NotOk(0) {
}

CannotSetValue::CannotSetValue(const std::string &reason) :
	NotOk(0, reason) {
}

