/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef EXCEPTIONS_H
#define EXCEPTIONS_H

#include <string>
#include <stdexcept>
#include "ErrorsAlarmsManager/DealError.h"
#include "ErrorsAlarmsManager/ErrorDef.h"

class Publisher;

class Exceptions: public std::exception {

public:
	Exceptions(int32 ExceptionsID);

	Exceptions(int32 ExceptionsID, const std::string &info);

	int32 getID() const {
		return m_id;
	}

	const char * what() const throw () {
		return info.c_str();
	}

	virtual ~Exceptions() throw () {
	}

private:

	int32 m_id;
	std::string info;

};

class NotOk: public Exceptions {

public:
	NotOk(int32 ExceptionsID);
	NotOk(int32 ExceptionsID, const std::string &info);

	NotOk(const std::string& creatorType, const std::string& creatorName, Publisher* publisher, int32 ExceptionsID);
	NotOk(const std::string& creatorType, const std::string& creatorName, Publisher* publisher, int32 ExceptionsID,
			const std::string& info);

};

class CannotSetValue: public NotOk {

public:
	CannotSetValue();
	CannotSetValue(const std::string &reason);
};

#endif //EXCEPTIONS_H
