/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*
 *
 * CASE Instrument Control Software
 *
 * Copyright (C) 2003  DPT/BI/SCI ILL
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA02111-1307, USA.
 *
 *  $Id: XMLErrorMessageCatalog.cpp,v 1.10 2007/07/18 05:31:01 ics Exp $
 */
#include "XMLErrorMessageCatalog.h"
#include <common/output/OutputStream.h>
#include <pugixml.hpp>

const std::string XMLErrorMessageCatalog::CATALOG_TAG = "Catalog";
const std::string XMLErrorMessageCatalog::FAMILY_TAG = "Family";
const std::string XMLErrorMessageCatalog::FAMILY_ID_ATTRIBUTE = "id";
const std::string XMLErrorMessageCatalog::ICSERROR_TAG = "IcsError";
const std::string XMLErrorMessageCatalog::ICSERROR_ID_ATTRIBUTE = "id";
const std::string XMLErrorMessageCatalog::MASTERMESSAGE_TAG = "MasterMessage";
const std::string XMLErrorMessageCatalog::STATUS_TAG = "Status";
const std::string XMLErrorMessageCatalog::MESSAGELEVEL_TAG = "MessageLevel";
const std::string XMLErrorMessageCatalog::LEVEL_TAG = "Level";
const std::string XMLErrorMessageCatalog::LISTCHILDRENMASKED_TAG = "ListChildrenMasked";
const std::string XMLErrorMessageCatalog::CHILD_TAG = "Child";
const std::string XMLErrorMessageCatalog::HELPLOCATION_TAG = "HelpLocation";
const std::string XMLErrorMessageCatalog::ACTION_TAG = "Action";
const std::string XMLErrorMessageCatalog::VARNAME_TAG = "VarName";

using namespace std;

void XMLErrorMessageCatalog::getMapError(const string &MyFileSource, map<int32, Error*> &aMapError) {

	pugi::xml_document doc;
	pugi::xml_parse_result result = doc.load_file(MyFileSource.c_str());
	if (result) {
		for (auto catalognode : doc.children()) {
			// CATALOG_TAG
			if (catalognode.name() == CATALOG_TAG) {
				for (auto familynode : catalognode.children()) {
					// FAMILY_TAG
					if (familynode.name() == FAMILY_TAG) {
						pugi::xml_attribute attr = familynode.attribute(FAMILY_ID_ATTRIBUTE.c_str());
						if (!attr) {
							missingAttributeError(MyFileSource, (uint32) familynode.offset_debug(), FAMILY_ID_ATTRIBUTE, FAMILY_TAG);
						}
						int32 familykey = stoi(attr.value());
						for (auto icserroralarmnode : familynode.children()) {
							// ICSERRORALARM_TAG
							if (icserroralarmnode.name() == ICSERROR_TAG) {
								Error* myErrorAlarm = new Error;
								myErrorAlarm->setKeyFamily(familykey);
								attr = icserroralarmnode.attribute(ICSERROR_ID_ATTRIBUTE.c_str());
								if (!attr) {
									missingAttributeError(MyFileSource, (uint32) familynode.offset_debug(), ICSERROR_ID_ATTRIBUTE, ICSERROR_TAG);
								}
								myErrorAlarm->setKeyId(stoi(attr.value()));
								for (auto children : icserroralarmnode.children()) {
									if (children.name() == MASTERMESSAGE_TAG) {
										myErrorAlarm->setMasterMessage(children.child_value());
									}
									else if (children.name() == STATUS_TAG) {
										myErrorAlarm->setStatus(stoi(children.child_value()));
									}
									else if (children.name() == MESSAGELEVEL_TAG) {
										vector<string> levels;
										for (auto messagelevel : children.children()) {
											if (messagelevel.name() == LEVEL_TAG) {
												levels.push_back(messagelevel.child_value());
											}
											else {
												unknownTagError(MyFileSource, (uint32) messagelevel.offset_debug(), messagelevel.name());
											}
										}
										myErrorAlarm->setMessageLevel(levels);
									}
									else if (children.name() == LISTCHILDRENMASKED_TAG) {
										vector<int32> childs;
										for (auto listchildmasked : children.children()) {
											if (listchildmasked.name() == CHILD_TAG) {
												childs.push_back(stoi(listchildmasked.child_value()));
											}
											else {
												unknownTagError(MyFileSource, (uint32) listchildmasked.offset_debug(), listchildmasked.name());
											}
										}
										myErrorAlarm->setListChildrenMasked(childs);
									}
									else if (children.name() == HELPLOCATION_TAG) {
										myErrorAlarm->setHelpLocation(children.child_value());
									}
									else if (children.name() == ACTION_TAG) {
										myErrorAlarm->setAfterAction(children.child_value());
									}
		//							else if (icserroralarmnode.name() == VARNAME_TAG) {
		//								myErrorAlarm->set(children.child_value());
		//							}
									else {
										unknownTagError(MyFileSource, (uint32) children.offset_debug(), children.name());
									}
								}
								aMapError[myErrorAlarm->getKeyId()] = myErrorAlarm;
							}
							else {
								unknownTagError(MyFileSource, (uint32) icserroralarmnode.offset_debug(), icserroralarmnode.name());
							}
						}
					}
					else {
						unknownTagError(MyFileSource, (uint32) familynode.offset_debug(), familynode.name());
					}
				}
			}
			else {
				unknownTagError(MyFileSource, (uint32) catalognode.offset_debug(), catalognode.name());
			}
		}
	}
	else {
		printError(MyFileSource, (uint32) result.offset, result.description());
	}
}
