/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "HistogrammingServices/HistogramManager.h"
#include "HistogrammingServices/HistogramConfigLoader.h"
#include <core/survey/Survey.h>
#include <core/survey/SurveyWriter.h>
#include <core/survey/NumorSurveyElement.h>
#include <core/dataaccess/DataAccess.h>
#include <boost/lexical_cast.hpp>
#include <iostream>

using namespace std;
using namespace boost;

HistogramManager* HistogramManager::_instance = 0;

HistogramManager* HistogramManager::getInstance() {
	if (_instance == 0) {
		_instance = new HistogramManager();
	}

	return _instance;
}

void HistogramManager::reset() {

}

void HistogramManager::init() {

	HistogramConfigLoader().loadConfig();
}

void HistogramManager::addHistogram(const std::string& controllerName, const std::string& propertyName, const std::optional<int>& activeWriteTime) {

	::core::Survey::getInstance()->addElement(controllerName, propertyName, activeWriteTime);
}

void HistogramManager::addHistogram(const std::string& controllerName, const std::string& propertyName, int32 index, const std::optional<int>& activeWriteTime) {

	::core::Survey::getInstance()->addElement(controllerName, propertyName, index, activeWriteTime);
}

void HistogramManager::numorStart(int32 numor) {
	::core::NumorSurveyElement numorElement(numor, true);
	::core::Survey::getInstance()->writeAndSend(&numorElement);
}

void HistogramManager::numorEnd(int32 numor) {
	::core::NumorSurveyElement numorElement(numor, false);
	::core::Survey::getInstance()->writeAndSend(&numorElement);
}

HistogramManager::HistogramManager() {

}

HistogramManager::~HistogramManager() {

}
