/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "IcsController.h"
#include "InstrumentAbstraction/ExperimentController.h"
#include "IcsControllerCommand.h"
#include "IcsControllerStopCommand.h"
#include "IcsControllerPauseCommand.h"
#include "IcsControllerRestartCommand.h"
#include <common/output/OutputStream.h>

using namespace std;
using namespace common;

namespace command {
namespace controller {

IcsController::IcsController(uint32 ID, ExperimentController * experimentController) :
	controller::Base(ID, experimentController->getName()),
	_experimentController(experimentController) {

	// loop over commands
	vector<string> commands = _experimentController->getCommandList();

	IcsControllerCommand* startCommand = 0;

	// searching for start command
	for (vector<string>::const_iterator c = commands.begin(); c != commands.end(); c++) {
		if (*c == IcsCommand::START) {
			startCommand = new IcsControllerCommand(IcsCommand::START, this, experimentController);
		}
	}

	// inserting commands
	for (vector<string>::const_iterator c = commands.begin(); c != commands.end(); c++) {

		if (*c == IcsCommand::START) {
			_addCommand(startCommand);

		} else if (*c == IcsCommand::STOP) {
			_addCommand(new IcsControllerStopCommand(this, experimentController));

		} else if (*c == IcsCommand::PAUSE) {
			_addCommand(new IcsControllerPauseCommand(this, experimentController));

		} else if (*c == IcsCommand::CONTINUE) {
			_addCommand(new IcsControllerRestartCommand(this, experimentController));

		} else {
			_addCommand(new IcsControllerCommand(*c, this, experimentController));
		}
	}

}

IcsController::~IcsController() {
}

const std::string& IcsController::_getType() const {
	return _experimentController->getType();
}

AbstractController * IcsController::getAbstractController() const {
	return _experimentController;
}

void IcsController::setValue(uint32 propertyID, const boost::any& value) {

	_experimentController->setValueSynchronized(_experimentController->getPropertyManager()->getPropertyName(propertyID), value);
}

}
}
