/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_ICSCONTROLLERCOMMAND_H_
#define COMMAND_ICSCONTROLLERCOMMAND_H_

#include "IcsCommand.h"
#include "InstrumentAbstraction/ClientCommand.h"

namespace command {

class IcsControllerCommand : public IcsCommand {

public:
	/**
	 * Constructor. Takes a Servant and address of the method providing the command
	 * functionality.
	 */
	IcsControllerCommand(const std::string& name, Servant* servant, AbstractController* controller);

	/**
	 * Virtual destructor.
	 */
	virtual ~IcsControllerCommand();

	/**
	 * Executes the command.
	 */
	virtual void execute(bool queue, bool skipIfWait);

	/**
	 * Returns true if the command has executed for longer than the desired timeout.
	 */
	virtual bool timeoutExceeded();

	/**
	 * Provides a stringified form of the Command.
	 */
	virtual std::string toString() const;

protected:
	ClientCommand* _command;
};

}

#endif