/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "IcsDriver.h"
#include "InstrumentAbstraction/TestController.h"
#include "IcsDriverCommand.h"
#include <common/output/OutputStream.h>

using namespace std;
using namespace common;

namespace command {
namespace driver {

IcsDriver::IcsDriver(uint32 ID, TestController * testController) :
	driver::Base(ID, testController->getName()),
	_testController(testController) {

	// loop over commands
	vector<string> commands = _testController->getDeviceDriver()->getCommandList();

	// searching for start command
	for (vector<string>::const_iterator c = commands.begin(); c != commands.end(); c++) {
		_addCommand(new IcsDriverCommand(*c, this, testController));
	}

}

IcsDriver::~IcsDriver() {
}

const std::string& IcsDriver::_getType() const {
	return _testController->getType();
}

AbstractController * IcsDriver::getAbstractController() const {
	return _testController;
}

void IcsDriver::setValue(uint32 propertyID, const boost::any& value) {
	_testController->setValueAndFormat(_testController->getPropertyManager()->getPropertyName(propertyID), value);
}

}
}
