/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "IcsDriverCommand.h"
#include "IcsController.h"
#include "InstrumentAbstraction/ExperimentController.h"
#include "Scheduler/Sender.h"
#include <common/output/OutputStream.h>

using namespace std;
using namespace common;

namespace command {

IcsDriverCommand::IcsDriverCommand(const std::string& name, Servant* servant, AbstractController* controller) :
	IcsCommand(name, servant),
	_command(new ClientCommand(controller, name, true, true)) {
}

IcsDriverCommand::~IcsDriverCommand() {
	delete _command;
}

void IcsDriverCommand::execute(bool queue, bool skipIfWait) {

	SetCommandActiveResult result = setActive(getServant()->_getType());

	if (result == EXECUTE) {
		// do it
	} else {
		err << "IcsDriverCommand : state " << (int32)result << " not managed for command " << _command->getController()->getName() << " " << _command->getName() << eol;
		return;
	}

	Sender<ClientCommand>::begin();
	Sender<ClientCommand>::post(*_command);
	Sender<ClientCommand>::endIf();

	setState(common::INACTIVE);
}

bool IcsDriverCommand::timeoutExceeded() {
	return false;
}

std::string IcsDriverCommand::toString() const {
	return string("\t") + getServant()->_getType() + " command " + getName() + " (" + to<string>(getID()) + ")\n";
}

}
