/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_ICS_SERVERMANAGER_H_
#define COMMAND_ICS_SERVERMANAGER_H_

#include <string>
#include <common/base/BaseTypes.h>
#include "DataProvider/Publisher.h"

namespace ics {

class ServerManager : public Publisher {

public:
	/**
	 * Initialises the instance.
	 */
	static void initInstance();

	/**
	 * Gets the instance.
	 */
	static ServerManager* getInstance();

	/**
	 * Resets the instance.
	 */
	static void resetInstance();

	/**
	 * Initialises the server.
	 */
	void init();

	/**
	 * Forces the reset.
	 */
	void forceReset();

	/**
	 * Notifies the start of the command zone.
	 */
	void onCommandZoneStopped();

	/**
	 * Notifies the start of the command zone.
	 */
	void onCommandZoneStarted();

	/**
	 * Notifies the start of the command zone.
	 */
	void onEstimatedTimeChanged(double estimatedTime);

	/**
	 * Notifies the start of the command zone.
	 */
	void onEstimatedTimeComputingStatus(long status);

	/**
	 * Unloads the modules that can be unloaded.
	 */
	void unloadUnusedModules();

	/**
	 * Notifies the start of the command zone.
	 */
	void onScanCommandBoxLaunch(const std::string filename);
	const std::string getScanCommandBoxFileName() const {
		return _ScanCommandBoxFileName;
	}
	/**
	 * Tests the controllers by calling the test method on each one.
	 * Returns false if does not succeed.
	 */
	bool test();

	/**
	 * Tests the controllers by calling the test method on each one.
	 * Returns false if does not succeed.
	 */
	bool testEstimatedTime();

	/**
	 * properties for property change aspect
	 */
	static const std::string TYPE;
	static const std::string NAME;
	static const std::string STATE_PROPERTY;
	static const std::string ESTIMATED_TIME_PROPERTY;
	static const std::string ESTIMATED_TIME_STATUS_PROPERTY;
	static const std::string SCAN_FILENAME_PROPERTY;

private:
	void reset();

	static ServerManager* _instance;
	static const std::string MODULES_PATH;

	std::string _ScanCommandBoxFileName;
};

}

#endif
