/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CLIENTCOMMAND_H
#define CLIENTCOMMAND_H

#include "Scheduler/Command.h"
#include "LoggingServices/LogStream.h"
#include <string>
#include <iostream>

class AbstractController;

/**
 * Concrete class implementing the command pattern for the client-side.
 */
class ClientCommand : public Command {

public:
	ClientCommand(AbstractController* aController = 0, const std::string& name = std::string("empty"), bool logging=false, bool blocking=false);
	virtual ~ClientCommand();

	virtual void execute();
	virtual void stop();
	virtual bool isBlocking(Command& aCommand);
	virtual bool reject(bool blocked);

	virtual AbstractController* getController() const;
	virtual const std::string& getName() const;
	virtual bool equal(Command* c1);
	virtual std::string toString() const;

private:
	void setCommandStateBeforeExecution();
	void setCommandStateAfterExecution();

	AbstractController* m_controller;
	std::string m_name;
	bool m_logging;
	bool m_blocking;
};

std::ostream& operator<<(std::ostream& os, const ClientCommand &c);

#endif
