/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Command.h"
#include "ExperimentController.h"
#include "Scheduler/Sender.h"
#include "InstrumentAbstraction/ClientCommand.h"

namespace controller {

void Command::initCommand(const std::string & commandName) {
	m_controller->initCommand(commandName);
	m_controller->addCommandFunction(commandName, this);
}

Command::~Command() {
	delete m_function;
}

void Command::execute() {
	m_function->execute();
}

void Command::command(bool logging, bool blocking) {
	Sender<ClientCommand>::begin();
	Sender<ClientCommand>::post(ClientCommand(m_controller, m_commandName, logging, blocking));
	Sender<ClientCommand>::endIf();
}

void Command::commandParallel(bool logging, bool blocking) {
	Sender<ClientCommand>::post(ClientCommand(m_controller, m_commandName, logging, blocking));
}

common::CommandState Command::getState(const std::string& commandName) const {
	return m_controller->getCommandManager()->getCommandState(commandName);
}

bool Command::hasRunningCommandState(common::CommandState state) const {
	return m_controller->getCommandManager()->hasRunningCommandState(state);
}

bool Command::hasOnlyInactiveCommands() const {
	return m_controller->getCommandManager()->hasOnlyInactiveCommands();
}

common::CommandState Command::getState() const {
	return getState(m_commandName);
}

}

