/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CONTROLLERCOMMAND_H
#define CONTROLLERCOMMAND_H

#include <common/base/Function.h>
#include <common/base/CommonTypes.h>
#include <string>
#include "CommandNames.h"

class ExperimentController;

namespace controller {

class Command {

public:
	template<class ServantType>
	Command(ExperimentController * controller, const std::string & commandName, void (ServantType::*f)(), ServantType * servant) :
		m_controller(controller),
		m_commandName(commandName),
		m_function(new common::Function<void, ServantType>(f, servant)) {
		initCommand(commandName);
	}

	Command() :
		m_controller(0),
		m_commandName(""),
		m_function(0) {
	}

	template<class ServantType>
	void init(ExperimentController * controller, const std::string & commandName, void (ServantType::*f)(), ServantType * servant) {
		m_controller = controller;
		m_commandName = commandName;
		m_function = new common::Function<void, ServantType>(f, servant);
		initCommand(commandName);
	}

	virtual ~Command();

	void execute();
	common::CommandState getState() const;

protected:
	/**
	 * First true (logging) should disappear.
	 * Second means blocks using AgentPolicy
	 */
	void command(bool logging, bool blocking);
	void commandParallel(bool logging, bool blocking);

	common::CommandState getState(const std::string& commandName) const;
	bool hasRunningCommandState(common::CommandState state) const;
	bool hasOnlyInactiveCommands() const;

private:
	void initCommand(const std::string & commandName);

	ExperimentController * m_controller;
	std::string m_commandName;
	common::BaseFunction<void> * m_function;
};

}

#endif

