/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "InstrumentAbstraction/ExperimentControllerCreator.h"
#include "InstrumentAbstraction/ExperimentController.h"
#include "Module/ModuleManager.h"

using namespace std;

/*
 * Base
 */
// A
// B

// C
//#include "InstrumentAbstraction/Base/ChopperController.h"

// D
// E

// H
// I
// K
// L



// N

// P

// R


// S
// T

//#include "InstrumentAbstraction/Base/TemperatureController.h"

//#include "InstrumentAbstraction/Base/TestCSController.h"
// V
//#include "InstrumentAbstraction/Base/VoltageController.h"
// W

/*
 * Cryopad
 */
// C


/*
 * Diffraction
 */
// C
// D
// T

/*
 * Lss
 */
// B

// C

// D
// F
// L
// P
// S
// T

/*
 * Pn
 */
// A
// G
//#include "InstrumentAbstraction/Pn/GamsCalculator.h"
//#include "InstrumentAbstraction/Pn/GamsCalibration.h"
// M
//#include "InstrumentAbstraction/Pn/Mpa3Environment.h"
// P
//#include "InstrumentAbstraction/Pn/PF2AcquisitionController.h"
//#include "InstrumentAbstraction/Pn/PF2PCIAcquisitionController.h"
//#include "InstrumentAbstraction/Pn/PF2ExperimentController.h"
//#include "InstrumentAbstraction/Pn/PF2PCIExperimentController.h"

/*
 * Son
 */

/*
 * Tas
 */
// A
// D
// F
// G
// H
// I
// P
// S
// T

/*
 * Tof
 */
// F
// I
// T


namespace ics {

const std::string TOOLS = "Tools";
const std::string EXECUTION_CONTROL = "Execution control";
const std::string ACQUISITION = "Acquisition";
const std::string COUNT = "Counts";
const std::string SCAN = "Scans";
const std::string SETTING = "Setting";
const std::string SAMPLE_ENVIRONMENT = "Sample environment";
const std::string VOLTAGE = "Voltages";
const std::string CURRENT = "Currents";
const std::string TEMPERATURE = "Temperatures";
const std::string PRESSURE = "Pressures";
const std::string BEAM_PARAMETERS = "Beam parameters";
const std::string CHOPPER = "Choppers";
const std::string COLLIMATION = "Collimations";
const std::string DIAPHRAGM = "Diaphragms";
const std::string POLARIZATIONS = "Polarizations";
const std::string ALIGNMENT = "Alignment";
const std::string AXIS = "Axis";
const std::string AUTOSAMPLER = "AutoSampler";
const std::string PRIMARY = "Primary";
const std::string SAMPLE = "Sample";
const std::string SECONDARY = "Secondary";

}


#define NEW_CONTROLLER(C) {if (create) {newExperimentController = new C(controllerName, ID);} else {m_typeClassMap[controllerType] = #C;}}

ExperimentControllerCreator* ExperimentControllerCreator::m_Instance = NULL;

ExperimentControllerCreator::
ExperimentControllerCreator()
	: m_ControllerCounter(0) {

	/*
	 * Base
	 */
	// A
	// B

	// C

    // D
    // E

	// G
    // H
	// K
	// L



	// N
	// O
	// P

	// R


	// S



	//m_ControllerTypes.push_back(TEST_CS_CONTROLLER);
	// V
//	m_ControllerTypes.push_back(VOLTAGE_EXPERIMENT_CONTROLLER);
	// W

	/*
	 * Cryopad
	 */
	// C

	/*
	 * Diffraction
	 */
	// D
	// T

	/*
	 * Lss
	 */
	// B
	// C
    // D
    // F
    // L
    // P
    // S

    /*
     * Pn
     */
    // A
    // G
//    m_ControllerTypes.push_back(GAMSCALCULATOR_EXPERIMENT_CONTROLLER);
//    m_ControllerTypes.push_back(GAMSCALIBRATION_EXPERIMENT_CONTROLLER);
//    m_ControllerTypes.push_back(GAMS_PEAK_FINDER);
    // M
//	m_ControllerTypes.push_back(MPA3_ENVIRONMENT_EXPERIMENT_CONTROLLER);
    // P

    /*
     * Scan
     */
    // C
	// P
//	m_ControllerTypes.push_back(PEAK_SCAN_EXPERIMENT_CONTROLLER);
	// S

    /*
     * Son
     */

    /*
     * Tas
     */
	// A
	// D
	// F
	// G
	// H
	// I
	// P
	// S
	// T

    /*
     * Tof
     */
    // F
	// I


   // T


	/*
	 * admin
	 */


}

ExperimentControllerCreator::~ExperimentControllerCreator() {
}

ExperimentControllerCreator* ExperimentControllerCreator::getInstance() {
	if (m_Instance == NULL)
		m_Instance = new ExperimentControllerCreator;

	return m_Instance;
}

void ExperimentControllerCreator::reset() {
	delete m_Instance;
	m_Instance = NULL;
}

void ExperimentControllerCreator::initFromModules() {

	vector<string> moduleControllerTypes = ModuleManager::getInstance()->getAllControllerTypes();

	for (vector<string>::const_iterator c = moduleControllerTypes.begin(); c != moduleControllerTypes.end(); ++c) {
		m_ControllerTypes.push_back(*c);
	}
}

ExperimentController* ExperimentControllerCreator::create(const string& controllerType, const string& controllerName, bool create) {

	ExperimentController* newExperimentController = NULL;

	/*
	 * Base
	 */
	// A
	// B

	// C

	// D
	// E

	// H
	// K
	// L


	// N

	// P

	// R


	// S

	// T

//	else if (controllerType == TEMPERATURE_EXPERIMENT_CONTROLLER) NEW_CONTROLLER(TemperatureController)

	//else if (controllerType == TEST_CS_CONTROLLER) NEW_CONTROLLER(TestCSController)
	// V
//    else if (controllerType == VOLTAGE_EXPERIMENT_CONTROLLER) NEW_CONTROLLER(VoltageController)
	// W

	/*
	 * Cryopad
	 */
	// C

	/*
	 * Diffraction
	 */
	// D
	// T

	/*
	 * Lss
	 */
	// B
	// C
	// D
 	// F
	// L
	// P
	// S

	// T

	/*
	 * Pn
	 */
	// A
	// G
//    else if (controllerType == GAMSCALCULATOR_EXPERIMENT_CONTROLLER) NEW_CONTROLLER(GamsCalculator)
//    else if (controllerType == GAMSCALIBRATION_EXPERIMENT_CONTROLLER) NEW_CONTROLLER(GamsCalibration)
	// M
//    else if (controllerType == MPA3_ENVIRONMENT_EXPERIMENT_CONTROLLER) NEW_CONTROLLER(Mpa3Environment)
    // P
//	else if (controllerType == PF2_ACQUISITION_EXPERIMENT_CONTROLLER) NEW_CONTROLLER(PF2AcquisitionController)
//	else if (controllerType == PF2_EXPERIMENT_EXPERIMENT_CONTROLLER) NEW_CONTROLLER(PF2ExperimentController)
//	else if (controllerType == PF2PCI_ACQUISITION_EXPERIMENT_CONTROLLER) NEW_CONTROLLER(PF2PCIAcquisitionController)
//	else if (controllerType == PF2PCI_EXPERIMENT_EXPERIMENT_CONTROLLER) NEW_CONTROLLER(PF2PCIExperimentController)

	/*
	 * Son
	 */

	/*
	 * Tas
	 */
	// A
	// D
	// F
	// G
	// G
	// I
	// P
	// S
	// T

	/*
	 * Tof
	 */
    // F
	// I
	// T


	// ------------------------------------

//	else {
		// calling the module manager
		newExperimentController = ModuleManager::getInstance()->makeController(controllerType, controllerName);
//	}

	return newExperimentController;
}

const vector<std::string>& ExperimentControllerCreator::getKnownTypes() const {

	return m_ControllerTypes;

}

bool ExperimentControllerCreator::verifyType(const string& controllerType) const {

	bool found = false;

	size_t size = m_ControllerTypes.size();
	for (size_t i = 0; i < size; i++) {
		if (m_ControllerTypes[i] == controllerType) {
			found = true;
		}
	}

	return found;
}

std::map<std::string, std::string> ExperimentControllerCreator::retrieveAllTypeClasses() {

	for (vector<string>::const_iterator t = m_ControllerTypes.begin(); t != m_ControllerTypes.end(); t++) {
		try {
			create(*t, "controller", false);
		} catch (...) {
		}
	}

	return m_typeClassMap;
}
