/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef EXPERIMENTCONTROLLERCREATOR_H
#define EXPERIMENTCONTROLLERCREATOR_H

#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <map>
#include <common/base/BaseTypes.h>

class ExperimentController;

class ExperimentControllerCreator {

public:

	/**
	 * Destructor
	 */
	~ExperimentControllerCreator();

	/**
	 * Instance accessor.
	 */
	static ExperimentControllerCreator* getInstance();

	/**
	 * Instance deletion.
	 */
	static void reset();

	/**
	 * Initialises the list of controllers loaded in modules.
	 */
	void initFromModules();

	/**
	 * Creation method - returns a new object of type ID (if it exists).
	 */
	ExperimentController* create(const std::string& controllerType, const std::string& controllerName, bool create = true);

	/**
	 * Returns a formatted string if available Types - this helps users determine
	 * the capabilities of the system. Each Type is separated by \n.
	 */
	const std::vector<std::string>& getKnownTypes() const;

	/**
	 * Returns true if a particular Type is available to the system.
	 */
	bool verifyType(const std::string& type) const;

	/**
	 * Creates all the type to get the full list of (type, class)
	 */
	std::map<std::string, std::string> retrieveAllTypeClasses();

private:

	/**
	 * Constructor.
	 */
	ExperimentControllerCreator();

	static ExperimentControllerCreator* m_Instance;

	std::vector<std::string> m_ControllerTypes;
	std::map<std::string, std::string> m_typeClassMap;

	int32 m_ControllerCounter;
};

#endif
