/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "InstrumentAbstraction/TestControllerCreator.h"

// DeviceContainer includes
#include "DeviceContainer/DeviceDriverCreator.h"

// TestController includes
#include "InstrumentAbstraction/TestController.h"

// Trace include
#include "LoggingServices/Level.h"

using namespace std;

class TestController;

TestControllerCreator::TestControllerCreator()  
  : m_ControllerCounter(0) {

  
}

TestControllerCreator::~TestControllerCreator() {

  
}

TestControllerCreator* TestControllerCreator::m_Instance = NULL;

TestControllerCreator* TestControllerCreator::getInstance() {
  
  if (m_Instance == NULL)
    m_Instance = new TestControllerCreator;

  return m_Instance;

}

void TestControllerCreator::reset() {

  delete m_Instance;

}

bool TestControllerCreator::verifyType(const string& driverType) const {

 return DeviceDriverCreator::getInstance()->verifyType(driverType);

}

string TestControllerCreator::getKnownTypes() const {

 return DeviceDriverCreator::getInstance()->getKnownIDs();
 
}

TestController* TestControllerCreator::create(const string& driverType,
					      const string& controllerName) {

  TestController* newTestController = NULL;

  newTestController = new TestController(driverType, controllerName);

  return newTestController;

}
