/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef TESTCONTROLLERCREATOR_H
#define TESTCONTROLLERCREATOR_H

#include <iostream>
#include <sstream>
#include <string>
#include <common/base/BaseTypes.h>
class DeviceDriver;
class TestController;
class TestControllerCreator;

class ElementaryController;

/**
 * This is a very simple factory class that creates a new instance of a 
 * particular TestController. It provides APIs to determine if the
 * ID is correct/exists. It is implemented using the singleton pattern.
 */
class TestControllerCreator {

public:

	/**
	 * Destructor
	 */
	~TestControllerCreator();

	/**
	 * Instance accessor.
	 */
	static TestControllerCreator* getInstance();

	/**
	 * Instance deletion.
	 */
	static void reset();

	/**
	 * Creation method - returns a new object of type ID (if it exists).
	 */
	TestController* create(const std::string& driverType, const std::string& controllerName);

	/**
	 * Returns a formatted string if available IDs - this helps users determine
	 * the capabilities of the system. Each ID is separated by \n.
	 */
	std::string getKnownTypes() const;

	/**
	 * Returns true if a particular ID is available to the system.
	 */
	bool verifyType(const std::string& driverType) const;

private:

	/**
	 * Constructor.
	 */
	TestControllerCreator();

	static TestControllerCreator* m_Instance;
	int32 m_ControllerCounter;

};

#endif //TESTCONTROLLERCREATOR_H
