/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef GENERICPROPERTYDESCRIPTION_H
#define GENERICPROPERTYDESCRIPTION_H

#include <string>

class ExperimentController;

/**
 * Contains all description of a property: name, type, default setpoint and default actual value. (In future maybe, minimum and maximum value, user permission on the property...) 
 */
class GenericPropertyDescription {

public:

	/**
	 * Constructor
	 */
	GenericPropertyDescription(const std::string& anAlias,
			ExperimentController* anEC, const std::string& aPropertyName) :
		m_Alias(anAlias), m_Controller(anEC), m_PropertyName(aPropertyName) {
	}

	GenericPropertyDescription(const GenericPropertyDescription& desc) {
		m_Alias = desc.getAlias();
		m_Controller = desc.getController();
		m_PropertyName = desc.getPropertyName();
	}

	/**
	 * Destructor
	 */
	~GenericPropertyDescription() {
	}

	const std::string& getAlias() const {
		return m_Alias;
	}
	ExperimentController* getController() const {
		return m_Controller;
	}
	const std::string& getPropertyName() const {
		return m_PropertyName;
	}

private:

	std::string m_Alias;
	ExperimentController* m_Controller;
	std::string m_PropertyName;
};

#endif //GENERICPROPERTYDESCRIPTION_H