/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef HARDWAREMANAGER_H
#define HARDWAREMANAGER_H

#include <sys/types.h>
#include <signal.h>
#include <string>
#include <map>
#include <common/base/Exception.h>


#include "InstrumentAbstraction/TestController.h"
#include "DeviceContainer/DeviceDriverCreator.h"
#include "DeviceContainer/DeviceDriver.h"

class DeviceDriver;
class TestControllerConfig;
class TestControllerConfigNode;


class HardwareManager {

public:
	class HardwareManagerError : public common::Exception {
	public:
		/**
		 * Constructor.
		 */
		HardwareManagerError(const std::string& message);
	};

	static void initInstance();
	static HardwareManager* getInstance();
	static void reset();

	void loadTestControllers();
	void initTestControllers();
	void loadReports();
	void loadTestControllers(const std::vector<TestControllerConfigNode>& configuration, bool checkParentChannel = true);
	void saveTestControllers(bool isBackup);
	void refreshTestControllers(const std::vector<TestControllerConfigNode>& configuration, bool checkParentChannel = true);

	TestController* getTestController(const std::string& testControllerName) ;
	DeviceDriver* getDriver(const std::string& testControllerName) ;
	bool controllerExists(const std::string& controllerName) const;
	bool controllerIDExists(const std::string& ID) const;

	void setPaused(bool value);

	std::string getTestControllersConfig();

	std::vector<std::string> getDriverTypesWithParentFunction(const std::string& parentFunction);
	void createDriver(const std::string& parentName, const std::string& type, const std::string& name, const std::string& mode, int32 channel);
	void removeDriver(const std::string& driverName);
	std::vector<std::string> getDriversWithType(const std::string& type);
	std::map<std::string, TestController*> getDriversPtrWithType(const std::string& type);
	std::map<std::string, TestController*> getAllInstalledDrivers() const;

private:
	HardwareManager();
	~HardwareManager();

	void init();
	void pause();
	void restart();

	TestController* loadTestController(const TestControllerConfig& conf) ;

	TestController* refreshTestController(const TestControllerConfig& conf) ;

	TestControllerConfig getTestControllerConfig(TestController* testController, bool all = false);
	void saveATestControllers(TestController *TC, std::vector<TestControllerConfigNode>& configs, bool all = false);

	void createFunctionDriverMap();

	static HardwareManager* _instance;
	struct FunctionParent {
		std::string function;
		std::string parentFunction;
	};
	std::map<std::string, FunctionParent> * _functionMap;

	std::map<std::string, TestController*> _installedTestControllers;
	std::vector<TestController*> _orderedInstalledTestControllers;
	std::map<std::string, bool> _installedTestControllersEnable;
};

#endif
