/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "InstrumentManager/NameManager.h"
#include "InstrumentManager/InstrumentManager.h"
#include "InstrumentManager/HardwareManager.h"
#include "InstrumentAbstraction/AbstractController.h"

using namespace std;

/*
 * Constructor
 */
NameManager::NameManager() {
}

/*
 * getInstance
 */
NameManager * NameManager::getInstance() {
	if (m_instance == 0) {
		m_instance = new NameManager();
	}
	return m_instance;
}

NameManager * NameManager::m_instance = 0;

/*
 * reset
 */
void NameManager::reset() {
	delete m_instance;
}

bool NameManager::controllerExists(const string& aName) const {
	/*
	 * Check also that it doesn't exist in the ScienceManager
	 */
	if (InstrumentManager::getInstance()->controllerExists(aName)) {
		return true;
	} else if (HardwareManager::getInstance()->controllerExists(aName)) {
		return true;
	}
	return false;
}

string NameManager::getControllerType(const string& aName) const {
	/*
	 * Check also that it doesn't exist in the ScienceManager
	 */
	if (InstrumentManager::getInstance()->controllerExists(aName)) {
		return EXPERIMENT_CONTROLLER_TYPE;
	} else if (HardwareManager::getInstance()->controllerExists(aName)) {
		return TEST_CONTROLLER_TYPE;
	}
	return "";
}

AbstractController* NameManager::getController(const string& aName) const
		 {
	AbstractController* controller = NULL;
	/*
	 * Check also that it doesn't exist in the ScienceManager
	 */
	if (InstrumentManager::getInstance()->controllerExists(aName)) {
		controller = InstrumentManager::getInstance()->getExperimentController(aName);
	} else if (HardwareManager::getInstance()->controllerExists(aName)) {
		controller = HardwareManager::getInstance()->getTestController(aName);
	} else {
		throw ControllerNotFound();
	}
	return controller;
}

