/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "LoggingServices/LogStream.h"
#include "InstrumentAbstraction/ExperimentController.h"
#include "LoggingServices/Log.h"

using namespace std;
using namespace common;

const uint32 LogStream::NAME_SIZE = 15;

LogStream::LogStream(std::string const & type, std::string const & name, Level const & level) :
	m_type(type), m_name(name), m_level(level) {
}

LogStream::~LogStream() {
}

LogStream & LogStream::operator<<(const string& s) {
	write(s);

	return *this;
}

LogStream & LogStream::operator<<(int64 v) {
	stringstream ss;
	ss << v;

	write(ss.str());

	return *this;
}

LogStream & LogStream::operator<<(int32 v) {
	stringstream ss;
	ss << v;

	write(ss.str());

	return *this;
}

LogStream & LogStream::operator<<(uint32 v) {
	stringstream ss;
	ss << v;

	write(ss.str());

	return *this;
}

LogStream & LogStream::operator<<(float64 v) {
	stringstream ss;
	ss << v;

	write(ss.str());

	return *this;
}

LogStream & LogStream::operator<<(float32 v) {
	stringstream ss;
	ss << v;

	write(ss.str());

	return *this;
}

LogStream & LogStream::operator<<(const PLog& pLog) {
	//if (pLog.getProperty() != 0) {
	m_lineElements.push_back(pLog.toPropertyLog());
	//}

	return *this;
}

LogStream & LogStream::operator<<(const LogStream& logStream) {
	//note: just copy the lineElements
	for (std::list<LogElement>::const_iterator it = logStream.getLineElements().begin(); it != logStream.getLineElements().end(); it++) {
		LogElement logElement = *it;
		m_lineElements.push_back(logElement);
	}

	// copy the image name only if local image name is empty
	if (m_imageName == "") {
		m_imageName = logStream.m_imageName;
	}

	return *this;
}

LogStream & LogStream::operator<<(End const &) {
	// Sending the message
	Log::getInstance()->log(m_type, m_name, m_level, m_lineElements, m_imageName);
	m_lineElements.clear();

	return *this;
}

LogStream & LogStream::operator<<(Image const & image) {
	m_imageName = image.name;

	return *this;
}

const std::list<common::LogElement>& LogStream::getLineElements() const {
	return m_lineElements;
}

void LogStream::write(const std::string& text) {
	m_lineElements.push_back(LogElement(false, false, "", "", "", "", text)); //ay ay ay!
}
