/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef LOGINPUT_H
#define LOGINPUT_H

#include "common/base/LogElement.h"
#include "LoggingServices/PLog.h"
#include "LoggingServices/Level.h"
#include "Servant.h"
#include "DataProvider/Property/TypedProperty/ValueProperty.h"
#include <common/output/OutputStream.h>
#include <string>
#include <sstream>
#include <iostream>

class LogStream {
public:

	struct End {
	};

	struct Image {
		std::string name;
	};

	LogStream(std::string const & type, std::string const & name, Level const & level);

	~LogStream();

	LogStream & operator<<(std::string const & s);
	LogStream & operator<<(int64 v);
	LogStream & operator<<(int32 v);
	LogStream & operator<<(uint32 v);
	LogStream & operator<<(float64 v);
	LogStream & operator<<(float32 v);
	LogStream & operator<<(const PLog& pLog);
	LogStream & operator<<(const LogStream& logStream);
	LogStream & operator<<(End const &);
	LogStream & operator<<(Image const &);

	const std::list<common::LogElement>& getLineElements() const;

	/**
	 * Logs a visible property.
	 */
	template<class Type>
	LogStream & operator<<(ValueProperty<Type> const & property) {

		if (property.getAlias().empty()) {
			// Return the value instead of the enriched property.
			common::warn << "The property '" << property.getServantName() << "." << property.getName() << "' has no alias, only its value is logged" << common::eol;
			std::ostringstream s;
			property.toString(s, property.get());
			s << property.getUnit();

			return this->operator<<(s.str());
		}

		if (m_name != property.getServantName()) {
			// Return the value with the unit instead of the enriched property.
			std::ostringstream s;
			property.toString(s, property.get());
			s << property.getUnit();

			return this->operator<<(s.str());
		}

		m_lineElements.push_back(property.toPropertyLog());

		return *this;
	}

	/**
	 * Logs a hidden property.
	 */
	template<class Type>
	LogStream & property(ValueProperty<Type> const & property) {

		// Do not log and display a warning.
		if (property.getAlias().empty()) {
			common::warn << "Cannot log " << property.getServantName() << "." << property.getName() << " in " << m_name << common::eol;
		} else {
			m_lineElements.push_back(property.toPropertyLog(true));
		}

		return *this;
	}

private:
	void reset();
	void write(std::string const & text);

	std::string m_type;
	std::string m_name;
	Level m_level;

	//For database log messages
	std::list<common::LogElement> m_lineElements;

	// for controllers that generate images
	std::string m_imageName;

	static const uint32 NAME_SIZE;
};

static const LogStream::End endlog = LogStream::End();

#endif
