/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */
#include <iostream>
#include "PLog.h"
#include "DataProvider/Property/AnyProperty.h"
#include "DataProvider/Property/PropertyInfos.h"

using namespace common;

PLog::PLog(AnyProperty* property, std::string const& controllerType, bool hidden) :
	m_hidden(hidden),
	m_fromValueProperty(false),
	m_controllerType(controllerType),
	m_anyProperty(property) {
}

PLog::PLog(const common::LogElement& valuePropertyLogElement, bool hidden) :
	m_hidden(hidden),
	m_fromValueProperty(true),
	m_anyProperty(0),
	m_valuePropertyLogElement(valuePropertyLogElement) {
}

PLog& PLog::value(std::string propertyValue) {
	m_value = propertyValue;

	return *this;
}

PLog& PLog::unit(std::string propertyUnit) {
	m_unit = propertyUnit;

	return *this;
}

common::LogElement PLog::toPropertyLog() const {

	if (m_fromValueProperty) {
		return m_valuePropertyLogElement;
	}

	//Create Propertylog with parameters: name, type, qualifier, unit, value
	//Get name, type and qualifier
	std::string qualifier;
	std::string name;
	std::string type;
	type = m_anyProperty->getType();

	//cout << PropertyInfos::getInstance() -> toString() << endl;

	//testing another way to check if property is scannable...through ScannableProperties class
	if (PropertyInfos::getInstance()->isScannable(m_controllerType, m_anyProperty->getName())) {
		qualifier = LogElement::ACTUAL;
		name = PropertyInfos::getInstance()->getItem(m_controllerType, m_anyProperty->getName()).shortPropertyName;

	} else {
		qualifier = LogElement::ACTUAL;

		if (!(m_anyProperty->getAlias()).empty()) {
			name = m_anyProperty->getAlias();
		} else {
			name = m_anyProperty->getName();
		}
	}

	return LogElement(true, m_hidden, name, type, qualifier, m_unit, m_value);
}

const AnyProperty* PLog::getProperty() const{
	return m_anyProperty;
}
