/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CONTROLLERFACTORY_H_
#define CONTROLLERFACTORY_H_

#include <string>
#include <ics/InstrumentAbstraction/ExperimentController.h>

class BaseControllerFactory {

public:
	virtual ~BaseControllerFactory();
	virtual ExperimentController * make(const std::string & name) = 0;
	virtual ExperimentController * copy(const ExperimentController * controller) = 0;
};

template<typename Type>
class ControllerFactory : public BaseControllerFactory {

public:
	virtual ExperimentController * make(const std::string & name) {
		ExperimentController * controller = new Type(name);
		controller->setType(Type::TYPE);

		return controller;
	}

	virtual ExperimentController * copy(const ExperimentController * controller) {
		const Type * typedController = dynamic_cast<const Type *>(controller);
		if (typedController != 0) {
			// copy constructor
			return new Type(*typedController);
		}

		return 0;
	}
};

#endif
