/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "LoadSequenceParser.h"
#include <common/output/OutputStream.h>
#include <iostream>

using namespace std;
using namespace common;

const std::string LoadSequenceParser::SEQUENCE_TAG = "sequence";
const std::string LoadSequenceParser::ID_ATTRIBUTE = "id";
const std::string LoadSequenceParser::MODULE_TAG = "module";
const std::string LoadSequenceParser::LIBRARY_ATTRIBUTE = "library";
const std::string LoadSequenceParser::DEPENDENCY_ATTRIBUTE = "dependency";

LoadSequenceParser::LoadSequenceParser() : m_id(0) {
}

LoadSequenceParser::~LoadSequenceParser() {
}

int LoadSequenceParser::getId() const {
	return m_id;
}

const std::vector<std::pair<std::string, std::string> >& LoadSequenceParser::getSequence() const {
	return m_sequence;
}

void LoadSequenceParser::startElement(const std::string& qName) {

	if (qName == SEQUENCE_TAG) {

		string id = getAttribute(ID_ATTRIBUTE);

		if (!id.empty()) {
			m_id = stoi(id);
		}

		return;

	} else if (qName == MODULE_TAG) {
		std::pair<std::string, std::string> info;
		info.first = getAttribute(LIBRARY_ATTRIBUTE);
		info.second = getAttribute(DEPENDENCY_ATTRIBUTE);

		m_sequence.push_back(info);
	}

}
