/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef ICS_LOADSEQUENCEPARSER_H_
#define ICS_LOADSEQUENCEPARSER_H_

#include <common/parser/GenericParser.h>
#include <vector>

/**
 * Class defining an XML parser for the load sequence file.
 */
class LoadSequenceParser : public common::GenericParser {

public:
	/**
	 * Constructor.
	 */
	LoadSequenceParser();
	
	/**
	 * Virtual destructor.
	 */
	virtual ~LoadSequenceParser();

	/**
	 * Gets the sequence id.
	 */
	int getId() const;

	/**
	 * Gets the sequence.
	 */
	const std::vector<std::pair<std::string, std::string> >& getSequence() const;
	
protected:
	virtual void startElement(const std::string& qName);

	int m_id;
	std::vector<std::pair<std::string, std::string> > m_sequence;
	
	const static std::string SEQUENCE_TAG;
	const static std::string ID_ATTRIBUTE;
	const static std::string MODULE_TAG;
	const static std::string LIBRARY_ATTRIBUTE;
	const static std::string DEPENDENCY_ATTRIBUTE;
};

#endif
