/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */


#ifndef FILEMANAGER_H_
#define FILEMANAGER_H_

#include <common/base/FileUtilities.h>
#include <iostream>
#include <boost/property_tree/xml_parser.hpp>
#include "DataProvider/Property/PropertyInfos.h"
#include <common/base/FileUtilities.h>
#include <common/base/ServerProperties.h>
#include <boost/filesystem/operations.hpp>
#include <boost/property_tree/xml_parser.hpp>
#include <boost/property_tree/exceptions.hpp>
#include <dlfcn.h>
#include <fstream>
#include <algorithm>
#include <stdio.h>
#include <dirent.h>
#include <boost/filesystem.hpp>

#include <rapidjson/writer.h>
#include <rapidjson/stringbuffer.h>
#include <rapidjson/prettywriter.h>


namespace pt = boost::property_tree;
namespace fs = boost::filesystem;

using namespace boost::filesystem;
using namespace common;
using namespace std;

class FileManager {

public:

	/**
	 * Returns the instance.
	 */
	static FileManager * getInstance();

	/**
	 * Initialises the instance.
	 */
	static void initInstance();

	/**
	 * Resets the instance.
	 */
	static void resetInstance();



	std::string getFileContent(const std::string& fileName);
	std::string getBinaryFileContent(const std::string& fileName);
	std::string getFileContentAbsolutePath(const std::string& fileName);
	void saveFile(const std::string& fileName, const std::string& content, const bool addServerHome);
	void saveBinaryFile(const std::string& fileName, const std::string& content, const bool addServerHome);
	std::string getTree(const std::string& rootDirectory, const std::string& extension, bool recurse = true);
	std::string getFileTree(const path currentPath, const std::string& extension, rapidjson::PrettyWriter<rapidjson::StringBuffer> &writer, bool recurse);
	void deleteFile(const std::string& fileName);
	bool atLeastOneFile(const std::string& path, const std::string& extension);
	std::vector<std::string> getFilesInDir(const std::string& path, const std::string& extension, const bool recursive);
	void createDirectory(const std::string& parent, const std::string& directoryName);
	void appendToFile(const std::string& fullFileName, const std::string& stringToAppend);
	bool fileExists(const std::string& fileName);


private:

	FileManager();
	~FileManager();

	const static std::string NAME;
	const static std::string ISDIRECTORY;
	const static std::string CHILDREN;

	static FileManager * m_instance;


	std::string _homePath;
};

#endif

