/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef ABSTRACTCONFIGURATION_H
#define ABSTRACTCONFIGURATION_H

#include <vector>
#include <string>
#include <common/base/Exception.h>

#include "PersistenceServices/Configuration/TestControllerConfig.h"
#include "PersistenceServices/Configuration/ExperimentControllerConfig.h"

/**
 * This class provide interface (proxy) for the configuration of controllers
 */
class AbstractConfiguration {

public:


	/*!
	 * \brief Configuration error exception
	 */
	/**
	 * Exception for initialisation failure
	 */
	class ConfigurationError : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		ConfigurationError(const std::string& message);
	};

	/**
	 * Destructor.
	 */
	virtual ~AbstractConfiguration() = 0;

	/**
	 * Loads the TestControllers
	 */
	virtual int32 loadTestControllers(std::vector<TestControllerConfig>& configs) = 0;

	/**
	 * Saves the TestControllers.
	 */
	virtual void saveTestControllers(const std::vector<TestControllerConfigNode>& configs, bool isBackup) = 0;

	/**
	 * Loads the ExperimentControllers
	 */
	virtual int32 loadExperimentControllers(std::vector<ExperimentControllerConfig>& configs) = 0;

	/**
	 * Saves the ExperimentControllers.
	 */
	virtual void saveExperimentControllers(const std::vector<ExperimentControllerConfig>& configs, bool isBackup) = 0;

};

#endif
