/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef ABSTRACTCONTROLLERCONFIG_H
#define ABSTRACTCONTROLLERCONFIG_H

#include<string>
#include<vector>

#include <ics/PersistenceServices/Configuration/PropertyConfig.h>
// include headers that implement a archive in simple text format
#include <boost/serialization/vector.hpp>

/**
 * This class gives interface for controller configuration
 */
class AbstractControllerConfig {

public:

	/**
	 * Destructor
	 */
	virtual ~AbstractControllerConfig();

	/*
	 * Return the enable state
	 */
	bool isEnabled() const;

	/**
	 * Set the enable state
	 */
	void setEnabled(bool enabled);

	/**
	 * Return the name
	 */
	std::string getName() const;

	/**
	 * To set the name
	 */
	void setName(const std::string& name);

	/**
	 * Return the type
	 */
	virtual const std::string& getType() const;

	/**
	 * To set type
	 */
	void setType(const std::string& type);

	/**
	 * add a property config in the list of properties
	 */
	void addProperty(const PropertyConfig& property);

	/**
	 * Return the list of properties config description
	 */
	const std::vector<PropertyConfig> getProperties() const;

protected:

	/**
	 * Constructor
	 */
	AbstractControllerConfig(const std::string& name, const std::string& type);

private:

	/**
	 * Name of controller
	 */
	std::string m_name;

	/**
	 * Type of controller
	 */
	std::string m_type;

	/**
	 * Using of controller
	 */
	bool m_enabled;

	/**
	 * List of properties config description
	 */
	std::vector<PropertyConfig> m_properties;

	friend class boost::serialization::access;
	// When the class Archive corresponds to an output archive, the
	// & operator is defined similar to <<.  Likewise, when the class Archive
	// is a type of input archive the & operator is defined similar to >>.
	template<class Archive>
	void serialize(Archive & ar, const uint32 version) {
		ar & m_name;
		ar & m_type;
		ar & m_enabled;
		ar & m_properties;
	}
};

/*
 * isEnabled 
 */
inline bool AbstractControllerConfig::isEnabled() const {
	return m_enabled;
}

/*
 * setEnabled 
 */
inline void AbstractControllerConfig::setEnabled(bool isEnabled) {
	m_enabled = isEnabled;
}

/*
 * getName
 */
inline std::string AbstractControllerConfig::getName() const {
	return m_name;
}

/*
 * setName
 */
inline void AbstractControllerConfig::setName(const std::string& name) {
	m_name = name;
}

/*
 * getType
 */
inline const std::string& AbstractControllerConfig::getType() const {
	return m_type;
}

/*
 * setType
 */
inline void AbstractControllerConfig::setType(const std::string& type) {
	m_type = type;
}

/*
 * addProperty
 */
inline void AbstractControllerConfig::addProperty(const PropertyConfig& property) {
	m_properties.push_back(property);
}

/*
 * getProperties
 */
inline const std::vector<PropertyConfig> AbstractControllerConfig::getProperties() const {
	return m_properties;
}
/*
 * addProperty
 */

#endif
