/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef ABSTRACTSETTING_H
#define ABSTRACTSETTING_H

#include <vector>
#include <string>
#include <common/base/Exception.h>

#include "PersistenceServices/Configuration/PropertyConfig.h"

/**
 * This class provide interface (proxy) for the settings
 */
class AbstractSetting {

public:

	/**
	 * Exception for initialisation failure
	 */
	class SettingError : public common::Exception {

	public:
		/**
		 * Constructor.
		 */
		SettingError(const std::string& message) : Exception("AbstractSetting::SettingError", message) {

		}
	};

	virtual ~AbstractSetting() {
	}

	/**
	 * load setting from a SettingFileName
	 *
	 */
	virtual std::vector<PropertyConfig> loadSettings(const std::string& SettingFileName, const std::string& id) = 0;

	/**
	 * get ids from the SettingFileName
	 *
	 */
	virtual std::string getIDs(const std::string& SettingFileName) = 0;

	/**
	 * Loads the description of a specific id
	 */
	virtual std::string getDescription(const std::string& SettingFileName, const std::string& Set_id) = 0;

	/*
	 * saveControllersSetting
	 */
	virtual void saveSettings(const std::string& SettingFileName, std::vector<PropertyConfig> Settings,
			const std::string& id) =0;

};

#endif
