/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PersistenceServices/Configuration/ExperimentControllerConfig.h"

using namespace std;

ExperimentControllerConfig::ExperimentControllerConfig() :
	AbstractControllerConfig("", ""),
	m_isFavourite(false) {
	// empty
}

ExperimentControllerConfig::ExperimentControllerConfig(const string& name, const string& type) :
	AbstractControllerConfig(name, type),
	m_isFavourite(false) {
	// Empty
}

ExperimentControllerConfig::~ExperimentControllerConfig() {
	//empty
}

void ExperimentControllerConfig::addRole(const RoleConfig& role) {
	m_roles.push_back(role);
}

const std::vector<RoleConfig>& ExperimentControllerConfig::getRoles() const {
	return m_roles;
}

void ExperimentControllerConfig::setFamily(const std::string& family) {
	m_family = family;
}

void ExperimentControllerConfig::setSubfamily(const std::string& subfamily) {
	m_subfamily = subfamily;
}

void ExperimentControllerConfig::setFavourite(bool value) {
	m_isFavourite = value;
}

const std::string& ExperimentControllerConfig::getFamily() const {
	return m_family;
}

const std::string& ExperimentControllerConfig::getSubfamily() const {
	return m_subfamily;
}

bool ExperimentControllerConfig::isFavourite() const {
	return m_isFavourite;
}

ostream& operator<<(ostream& s, const ExperimentControllerConfig& scConfig) {
	s << "\tExperimentControllerConfig" << endl << "\t\tname=" << scConfig.getName()
			<< endl << "\t\ttype=" << scConfig.getType() << endl
			<< "\t\tenabled=" << scConfig.isEnabled() << endl;
	const vector<RoleConfig> roles = scConfig.getRoles();
	vector<RoleConfig>::const_iterator rolesIterator;
	s << "\t\tRoles = " << endl;
	for (rolesIterator = roles.begin(); rolesIterator != roles.end(); rolesIterator++) {
		s << "\t\t\t" << *rolesIterator << endl;
	}
	const vector<PropertyConfig> properties = scConfig.getProperties();
	vector<PropertyConfig>::const_iterator propertiesIterator;
	s << "\t\tProperties = " << endl;
	for (propertiesIterator = properties.begin(); propertiesIterator
			!= properties.end(); propertiesIterator++) {
		s << "\t\t\t" << *propertiesIterator << endl;
	}

	s << "\t\tfavourite = " << scConfig.m_isFavourite << endl;
	s << "\t\tfamily = " << scConfig.m_family << endl;
	s << "\t\tsubfamily = " << scConfig.m_subfamily << endl;

	s << endl;
	return s;
}
