/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef EXPERIMENTCONTROLLERCONFIG_H
#define EXPERIMENTCONTROLLERCONFIG_H

#include <ics/PersistenceServices/Configuration/AbstractControllerConfig.h>
#include <ics/PersistenceServices/Configuration/RoleConfig.h>

#include <boost/serialization/base_object.hpp>

/**
 * this class contains information about configuration of an experiment
 * controller.
 */
class ExperimentControllerConfig: public AbstractControllerConfig {

public:

	/**
	 * Default Constructor
	 */
	ExperimentControllerConfig();

	/**
	 * Constructor
	 */
	ExperimentControllerConfig(const std::string& name, const std::string& type);

	/**
	 * Destructor
	 */
	virtual ~ExperimentControllerConfig();

	/**
	 * Add role in the roles list of controller
	 */
	void addRole(const RoleConfig& role);

	/**
	 * Return the roles list
	 */
	const std::vector<RoleConfig>& getRoles() const;

	/**
	 * Sets the family.
	 */
	void setFamily(const std::string& family);

	/**
	 * Sets the subfamily.
	 */
	void setSubfamily(const std::string& subfamily);

	/**
	 * Sets favourite.
	 */
	void setFavourite(bool value);

	/**
	 * Gets the family.
	 */
	const std::string& getFamily() const;

	/**
	 * Gets the subfamily.
	 */
	const std::string& getSubfamily() const;

	/**
	 * Returns true if favourite.
	 */
	bool isFavourite() const;

	/**
	 * << operator redefinition
	 */
	friend std::ostream& operator<<(std::ostream& s, const ExperimentControllerConfig& scConfig);

private:

	/**
	 * List of roles
	 */
	std::vector<RoleConfig> m_roles;

	bool m_isFavourite;
	std::string m_family;
	std::string m_subfamily;

	friend class boost::serialization::access;
	// When the class Archive corresponds to an output archive, the
	// & operator is defined similar to <<.  Likewise, when the class Archive
	// is a type of input archive the & operator is defined similar to >>.
	template<class Archive>
	void serialize(Archive & ar, const uint32 version) {
		ar & boost::serialization::base_object<AbstractControllerConfig>(*this);
		ar & m_roles;
		ar & m_isFavourite;
		ar & m_family;
		ar & m_subfamily;
	}

};

#endif
