/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef PROPERTYCONFIG_H
#define PROPERTYCONFIG_H

#include <iostream>
#include <string>
#include <vector>
#include <boost/serialization/access.hpp>
#include <common/base/BaseTypes.h>

/**
 * this class contains information about configuration of a static property.
 */
class PropertyConfig {

public:

	/**
	 * Constructor
	 */
	PropertyConfig();

	/**
	 * Constructor
	 */
	PropertyConfig(const std::string& name, const std::string& value);

	/**
	 * Constructor
	 */
	PropertyConfig(const std::string& name,
			const std::vector<std::string>& arrayValue, const int32 aNCol);

	/**
	 * Return the name of property
	 */
	std::string getName() const;

	/**
	 * Return the value of property
	 */
	std::string getValue() const;

	/**
	 * Return the array of array property
	 */
	std::vector<std::string> getArray() const;

	/**
	 * Return the name of property
	 */
	bool isArray() const;

	/**
	 * Return the name of property
	 */
	size_t getSize() const;

	/**
	 * Return the colums number of array property
	 */
	int32 getColsNb() const;

	/**
	 * To put change in a stream
	 */
	friend std::ostream& operator<<(std::ostream&s,
			const PropertyConfig& property);

private:

	/**
	 * Name of the property
	 */
	std::string m_name;

	/**
	 * Value of the property
	 */
	std::string m_value;

	bool m_IsArray;

	/**
	 * nb colums  of the array  property
	 */
	int32 m_arrayColsNb;

	/**
	 * array  of the property
	 */
	std::vector<std::string> m_arrayValue;

	friend class boost::serialization::access;
	// When the class Archive corresponds to an output archive, the
	// & operator is defined similar to <<.  Likewise, when the class Archive
	// is a type of input archive the & operator is defined similar to >>.
	template<class Archive>
	void serialize(Archive & ar, const uint32 version) {
		ar & m_name;
		ar & m_value;
		ar & m_IsArray;
		ar & m_arrayColsNb;
		ar & m_arrayValue;
	}
};

/*
 * getName 
 */
inline std::string PropertyConfig::getName() const {
	return m_name;
}

/*
 * getValue
 */
inline bool PropertyConfig::isArray() const {
	return m_IsArray;
}

/*
 * getValue
 */
inline std::string PropertyConfig::getValue() const {
	return m_value;
}

/*  
 * getSize 
 */
inline size_t PropertyConfig::getSize() const {
	return m_arrayValue.size();
}

/*
 * getColsNb
 */
inline int32 PropertyConfig::getColsNb() const {
	return m_arrayColsNb;
}

/*
 * getArray
 */
inline std::vector<std::string> PropertyConfig::getArray() const {
	return m_arrayValue;
}

#endif
