/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PersistenceServices/Configuration/ProxySettingManager.h"
#include "PersistenceServices/Configuration/XMLFileSetting.h"
#include "PersistenceServices/Configuration/PropertyConfig.h"

using namespace std;

/*
 * Constructor 
 */
ProxySettingManager::ProxySettingManager() :
	m_realSetting(0) {

	// Create real subject
	m_realSetting = new XMLFileSetting();
}

/*
 * Constructor
 */
ProxySettingManager::ProxySettingManager(const std::string& path) :
	m_realSetting(0) {

	// Create real subject
	m_realSetting = new XMLFileSetting(path);
}

/*
 * Destructor 
 */
ProxySettingManager::~ProxySettingManager() {
	// Remove real subject
	delete m_realSetting;
}

/*
 * getIDs
 */
string ProxySettingManager::getIDs(const string& SettingFileName) {
	string buf;
	if (m_realSetting != 0) {
		buf = m_realSetting->getIDs(SettingFileName);
	}
	return buf;
}

/*
 *  getDescription
 */
string ProxySettingManager::getDescription(const string& SettingFileName,
		const string& id) {
	string description;
	if (m_realSetting != 0) {
		description = m_realSetting->getDescription(SettingFileName, id);
	}
	return description;

}

/*
 * loadControllers settings
 */
vector<PropertyConfig> ProxySettingManager::loadSettings(
		const string& SettingFileName, const string& id) {
	vector<PropertyConfig> setting;
	if (m_realSetting != 0) {
		setting = m_realSetting->loadSettings(SettingFileName, id);
	}
	return setting;

}

/*
 * saveControllersSetting
 */
void ProxySettingManager::saveSettings(const string& SettingFileName, vector<
		PropertyConfig> Settings, const string& id) {
	if (m_realSetting != 0) {
		m_realSetting->saveSettings(SettingFileName, Settings, id);
	}

}
