/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef PROXYSETTINGMANAGER_H
#define PROXYSETTINGMANAGER_H

#include "PersistenceServices/Configuration/AbstractSetting.h"
#include "PersistenceServices/Configuration/PropertyConfig.h"
#include "PersistenceServices/Configuration/XMLFileSetting.h"

/**
 * This class provides a proxy for the setting manager, about
 * setting file access.
 */
class ProxySettingManager: public AbstractSetting {

public:
	/**
	 * Constructor
	 */
	ProxySettingManager();

	/**
	 * Constructor
	 */
	ProxySettingManager(const std::string& path);

	/**
	 * Destructor
	 */
	virtual ~ProxySettingManager();

	/**
	 * load setting from a SettingName
	 *
	 */
	virtual std::vector<PropertyConfig> loadSettings(const std::string& SettingFileName,
			const std::string& id);

	/**
	 * get ids from the SettingName
	 *
	 */
	virtual std::string getIDs(const std::string& SettingFileName);

	/**
	 * Loads the description of a specific id
	 */
	virtual std::string getDescription(const std::string& SettingFileName,
			const std::string& Set_id);

	/*
	 * saveControllersSetting
	 */
	virtual void saveSettings(const std::string& SettingFileName, std::vector<
			PropertyConfig> Settings, const std::string& id);

private:

	/**
	 * Real setting link
	 */
	AbstractSetting* m_realSetting;

};

#endif
