/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef ROLECONFIG_H
#define ROLECONFIG_H

#include<iostream>
#include<string>
#include <common/base/BaseTypes.h>

#include <boost/serialization/access.hpp>

/**
 * this class contains information about configuration of a controller role.
 */
class RoleConfig {

public:

	/**
	 * Constructor
	 */
	RoleConfig();

	/**
	 * Constructor
	 */
	RoleConfig(const std::string& roletype, const std::string& name, const std::string& who);

	/**
	 * Get the name of role
	 */
	const std::string& getRoleType() const;

	/**
	 * Get the name of role
	 */
	const std::string& getName() const;

	/**
	 * Get the controller in charge of that role
	 */
	const std::string& getWho() const;

	/**
	 * To put change in a stream
	 */
	friend std::ostream& operator<<(std::ostream&s, const RoleConfig& role);


private:

	std::string m_RoleType;

	/**
	 * Name of role
	 */
	std::string m_name;

	/**
	 * Controller name in charge of that role
	 */
	std::string m_who;

	friend class boost::serialization::access;
	// When the class Archive corresponds to an output archive, the
	// & operator is defined similar to <<.  Likewise, when the class Archive
	// is a type of input archive the & operator is defined similar to >>.
	template<class Archive>
	void serialize(Archive & ar, const uint32 version) {
		ar & m_RoleType;
		ar & m_name;
		ar & m_who;
	}
};

/*
 * getName
 */
inline const std::string& RoleConfig::getRoleType() const {
	return m_RoleType;
}

/*
 * getName
 */
inline const std::string& RoleConfig::getName() const {
	return m_name;
}

/*
 * getWho
 */
inline const std::string& RoleConfig::getWho() const {
	return m_who;
}

#endif
