/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef TESTCONTROLLERCONFIG_H
#define TESTCONTROLLERCONFIG_H

#include <ics/PersistenceServices/Configuration/AbstractControllerConfig.h>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/base_object.hpp>

/**
 * this class contains information about configuration of an test
 * controller.
 */
class TestControllerConfig: public AbstractControllerConfig {

public:

	/**
	 * Default Constructor
	 */
	TestControllerConfig();

	/**
	 * Constructor with parameters
	 */
	TestControllerConfig(const std::string& name,
			const std::string& type, const std::string& parentName = "");

	/**
	 * Destructor
	 */
	virtual ~TestControllerConfig();

	/**
	 * Set the parent name of controller
	 */
	void setParentName(const std::string& parentName);

	/**
	 * Get the parent name of controller
	 */
	const std::string& getParentName() const;

	/**
	 * Set the parent channel name of controller
	 */
	void setParentChannel(int32  channel);

	/**
	 * Get the parent channel name of controller
	 */
	int32 getParentChannel() const;

	/**
	 * << operator
	 */
	friend std::ostream
	& operator<<(std::ostream& s, const TestControllerConfig& tcConfig);

private:

	/*
	 * Parent name
	 */
	std::string m_parentName;

	/*
	 * Parent channel
	 */
	int32 m_parentChannel;

	friend class boost::serialization::access;
	// When the class Archive corresponds to an output archive, the
	// & operator is defined similar to <<.  Likewise, when the class Archive
	// is a type of input archive the & operator is defined similar to >>.
	template<class Archive>
	void serialize(Archive & ar, const uint32 version) {
		ar & boost::serialization::base_object<AbstractControllerConfig>(*this);
		ar & m_parentName;
		ar & m_parentChannel;
	}
};

/*
 * setParentName
 */
inline void TestControllerConfig::setParentName(const std::string& parentName) {
	m_parentName = parentName;
}

/*
 * getParentName
 */
inline const std::string& TestControllerConfig::getParentName() const {
	return m_parentName;
}

/*
 * setParentChannel
 */
inline void TestControllerConfig::setParentChannel(int32  channel) {
	m_parentChannel = channel;
}

/*
 * getParentChannel
 */
inline int32 TestControllerConfig::getParentChannel() const {
	return m_parentChannel;
}

/**
 * Structure used for saving the configuration file
 */
class TestControllerConfigNode {
public:
	TestControllerConfig config;
	std::vector<TestControllerConfigNode> children;

	friend class boost::serialization::access;
	// When the class Archive corresponds to an output archive, the
	// & operator is defined similar to <<.  Likewise, when the class Archive
	// is a type of input archive the & operator is defined similar to >>.
	template<class Archive>
	void serialize(Archive & ar, const uint32 version) {
		ar & config;
		ar & children;
	}
};

#endif
