/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PersistenceServices/Configuration/XMLFileSetting.h"
#include "PersistenceServices/Configuration/PropertyConfig.h"
#include "common/base/ServerProperties.h"

using namespace std;

/**
 * Init static members
 */
const string XMLFileSetting::ID_TAG = "Id";
const string XMLFileSetting::INSTRUMENTSETTINGS_TAG = "InstrumentSettings";
const string XMLFileSetting::SETTING_TAG = "Setting";
const string XMLFileSetting::DESCRIPTION_TAG = "Description";
const string XMLFileSetting::PROPERTY_TAG = "Property";
const string XMLFileSetting::PROPERTY_NAME_ATTRIBUTE = "name";
const string XMLFileSetting::PROPERTYARRAY_TAG = "PropertyArray";
const string XMLFileSetting::PROPERTYARRAY_CSV_TAG = "|";
const string XMLFileSetting::PROPERTYARRAY_COLS_ATTRIBUTE = "cols";
const string XMLFileSetting::ROW_TAG = "Row";

/*
 * Constructor 
 */
XMLFileSetting::XMLFileSetting(const std::string &path) {
	// Create Setting  files name
	if (path == "") {
		m_Filepath.append(common::ServerProperties::getInstance()->getConfigPath());
		m_Filepath.append("/");
		m_Filepath.append(".");
	} else {
		m_Filepath = path;
	}
}

/*
 * Destructor 
 */
XMLFileSetting::~XMLFileSetting() {
}

/*
 * removeID 
 */
void XMLFileSetting::removeID(const string &id) const {

}

/*
 * add
 */
string XMLFileSetting::getIDs(const string &SettingFileName) {
	string id;
	string filename = m_Filepath;
	filename.append(SettingFileName);
	filename.append("Setting.xml");

	pugi::xml_document doc;
	pugi::xml_parse_result result = doc.load_file(filename.c_str());
	if (result) {
		for (auto noderoot : doc.children()) {
			if (noderoot.name() == INSTRUMENTSETTINGS_TAG) {
				for (auto node : noderoot.children()) {
					if (node.name() == SETTING_TAG) {
						for (auto element : node.children()) {
							if (element.name() == ID_TAG) {
								id.append(element.child_value());
								id.append("|");
							}
						}
					} else {
						unknownTagError(filename, (uint32) node.offset_debug(), node.name());
					}
				}
			} else {
				unknownTagError(filename, (uint32) noderoot.offset_debug(), noderoot.name());
			}
			if (id.length() > 1) {
				id.erase(id.end() - 1);
			}
		}
	} else {
		printError(filename, (uint32) result.offset, result.description());
	}
	return id;
}

/*
 * checkSettingName
 */
bool XMLFileSetting::checkSettingName(const string &SettingFileName) const {
	return true;
}

/************************************************************************
 * 
 * Setting
 *
 ************************************************************************/

/*
 * LoadSettings
 */
vector<PropertyConfig> XMLFileSetting::loadSettings(const string &SettingFileName, const string &Set_id) {

	std::vector<PropertyConfig> settings;
	string filename = m_Filepath;
	filename.append(SettingFileName);
	filename.append("Setting.xml");

	pugi::xml_document doc;
	pugi::xml_parse_result result = doc.load_file(filename.c_str());
	if (result) {
		for (auto noderoot : doc.children()) {
			if (noderoot.name() == INSTRUMENTSETTINGS_TAG) {
				for (auto node : noderoot.children()) {
					if (node.name() == SETTING_TAG) {
						string id;
						string Description;
						for (auto element : node.children()) {
							if (element.name() == ID_TAG) {
								id.append(element.child_value());
							}
						}
						if (id == Set_id) {
							settings = createSetting(SettingFileName, node);
						}
					} else {
						unknownTagError(filename, (uint32) node.offset_debug(), node.name());
					}
				}
			} else {
				unknownTagError(filename, (uint32) noderoot.offset_debug(), noderoot.name());
			}
		}
	} else {
		printError(filename, (uint32) result.offset, result.description());
	}

	return settings;
}

/*
 * LoadSettings
 */
string XMLFileSetting::getDescription(const string &SettingFileName, const string &Set_id) {
	string description;

	string filename = m_Filepath;
	filename.append(SettingFileName);
	filename.append("Setting.xml");

	pugi::xml_document doc;
	pugi::xml_parse_result result = doc.load_file(filename.c_str());
	if (result) {
		for (auto noderoot : doc.children()) {
			if (noderoot.name() == INSTRUMENTSETTINGS_TAG) {
				for (auto node : noderoot.children()) {
					if (node.name() == SETTING_TAG) {
						string id;
						bool idok = false;
						for (auto element : node.children()) {
							if (element.name() == ID_TAG) {
								id.append(element.child_value());
							}
							if (id == Set_id) {
								idok = true;
							}
							if (idok) {
								if (element.name() == DESCRIPTION_TAG) {
									description = element.child_value();
									break;
								}
							}
						}
					} else {
						unknownTagError(filename, (uint32) node.offset_debug(), node.name());
					}
				}
			} else {
				unknownTagError(filename, (uint32) noderoot.offset_debug(), noderoot.name());
			}
		}
	} else {
		printError(filename, (uint32) result.offset, result.description());
	}

	return description;
}

/*
 * SaveSettings
 */
void XMLFileSetting::saveSettings(const string &SettingFileName, vector<PropertyConfig> Settings, const string &Set_id) {

}

/*
 * createSetting
 */
vector<PropertyConfig> XMLFileSetting::createSetting(const std::string &configFileName, const pugi::xml_node &element) {
	//  PropertyConfig config;
	vector<PropertyConfig> settingList;

	string id;
	string description;

	for (auto children : element.children()) {
		if (children.name() == ID_TAG) {
			id = children.child_value();
		} else if (children.name() == DESCRIPTION_TAG) {
			description = children.child_value();
		} else if (children.name() == PROPERTY_TAG) {
			string propertyName;
			string propertyValue;
			pugi::xml_attribute attr = children.attribute(PROPERTY_NAME_ATTRIBUTE.c_str());
			if (attr) {
				propertyName = attr.value();
			} else {
				missingAttributeError(configFileName, (uint32) children.offset_debug(), PROPERTY_NAME_ATTRIBUTE, PROPERTY_TAG);
			}
			propertyValue = children.child_value();
			settingList.push_back(PropertyConfig(propertyName, propertyValue));
		} else if (children.name() == PROPERTYARRAY_TAG) {
			pugi::xml_attribute attr = children.attribute(PROPERTY_NAME_ATTRIBUTE.c_str());
			string propertyName;
			if (attr) {
				propertyName = attr.value();
			} else {
				missingAttributeError(configFileName, (uint32) children.offset_debug(), PROPERTY_NAME_ATTRIBUTE, PROPERTY_TAG);
			}
			attr = children.attribute(PROPERTYARRAY_COLS_ATTRIBUTE.c_str());
			string propertyArrayCols;
			if (attr) {
				propertyArrayCols = attr.value();
			} else {
				missingAttributeError(configFileName, (uint32) children.offset_debug(), PROPERTYARRAY_COLS_ATTRIBUTE,
						PROPERTYARRAY_TAG);
			}
			int32 ncol = stoi(propertyArrayCols);
			vector<string> myString;
			for (auto row : children.children()) {
				if (row.name() == ROW_TAG) {
					string arrayValue = row.child_value();
					uint32 lastindex = 0;
					for (uint32 colId = 0; colId < (uint32) ncol; colId++) {
						uint32 myindex = arrayValue.find(PROPERTYARRAY_CSV_TAG, lastindex);
						myString.push_back(arrayValue.substr(lastindex, myindex - lastindex));
						lastindex = myindex + 1;
					}
				} else {
					unknownTagError(configFileName, (uint32) row.offset_debug(), row.name());
				}
			}
			settingList.push_back(PropertyConfig(propertyName, myString, ncol));
		} else {
			unknownTagError(configFileName, (uint32) children.offset_debug(), children.name());
		}
	}
	return settingList;
}
// Adding the TestController configuration to the list
// Copying the map into a vector
//m_SettingList.push_back(config);

/*
 * printError
 */
std::string XMLFileSetting::printError(const std::string &file, uint32 offset, const std::string &message) {
	throw SettingError(XMLErrors::printError(file, offset, message));
}
