/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef XMLFILESETTING_H
#define XMLFILESETTING_H

#include <string>
#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <pugixml.hpp>

#include "PersistenceServices/Configuration/AbstractSetting.h"
#include "PersistenceServices/XMLErrors.h"

class XMLFileSetting: public AbstractSetting, public XMLErrors {

public:

	/**
	 * Constructor
	 */
	XMLFileSetting(const std::string &path = "");

	/**
	 * Destructor
	 */
	virtual ~XMLFileSetting();

	/**
	 * Remove definively id into the setting
	 * not yet
	 */
	virtual void removeID(const std::string &id) const;

	/**
	 * get ids
	 *
	 */
	virtual std::string getIDs(const std::string &SettingFileName);

	/**
	 * Loads the description of a specific id
	 */
	virtual std::string getDescription(const std::string &SettingFileName, const std::string &Set_id);

	/**
	 * Check the configurationsetting  name, if it existed in the table
	 */
	virtual bool checkSettingName(const std::string &configFileName) const;

	/**
	 * Loads the Settings
	 */
	virtual std::vector<PropertyConfig> loadSettings(const std::string &SettingFileName, const std::string &Set_id);

	/**
	 * Saves the Settings.
	 */
	virtual void saveSettings(const std::string &SettingFileName, std::vector<PropertyConfig> Settings,
			const std::string &Set_id);

private:

	/**
	 * Creates settings.
	 */
	std::vector<PropertyConfig> createSetting(const std::string &configFileName, const pugi::xml_node &element);

	/*!
	 * \brief Print error in terminal using red bold characters
	 * \param[in] file XML file name
	 * \param[in] offset Character offset in the file
	 * \param[in] message Message to print
	 * \return the string error
	 */
	virtual std::string printError(const std::string &file, uint32 offset, const std::string &message);

	// XML tags
	static const std::string ID_TAG;
	static const std::string INSTRUMENTSETTINGS_TAG;
	static const std::string SETTING_TAG;
	static const std::string DESCRIPTION_TAG;
	static const std::string PROPERTY_TAG;
	static const std::string PROPERTY_NAME_ATTRIBUTE;
	static const std::string PROPERTYARRAY_TAG;
	static const std::string PROPERTYARRAY_CSV_TAG;
	static const std::string PROPERTYARRAY_COLS_ATTRIBUTE;
	static const std::string ROW_TAG;

	/**
	 * Complete name with path of setting files
	 */
	std::string m_Filepath;

//  std::vector<PropertyConfig> m_SettingList;
};

#endif
