/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "XMLErrors.h"
#include "sstream"
#include "fstream"
#include <common/output/OutputStream.h>

using namespace std;

/*
 * unknownTagError
 */
std::string XMLErrors::unknownTagError(const std::string &file, uint32 offset, const std::string &tagname) {
	ostringstream error;
	error << "Unknown tag " << tagname;
	printError(file, offset, error.str());
	return error.str();
}

/*
 * unknownAttributeError
 */
std::string XMLErrors::unknownAttributeError(const std::string &file, uint32 offset, const std::string &attributename) {
	ostringstream error;
	error << "Unknown attribute " << attributename;
	printError(file, offset, error.str());
	return error.str();
}

/*
 * missingAttributeError
 */
std::string XMLErrors::missingAttributeError(const std::string &file, uint32 offset, const std::string &attributename,
		const std::string &tagname) {
	ostringstream error;
	error << "Missing attribute " << attributename << " in a tag " << tagname;
	printError(file, offset, error.str());
	return error.str();
}

/*
 * missingTagNameError
 */
std::string XMLErrors::missingTagNameError(const std::string &file, uint32 offset, const std::string &parentname,
		const std::string &tagname) {
	ostringstream error;
	error << "Missing tag " << tagname << " in a parent " << parentname;
	printError(file, offset, error.str());
	return error.str();
}

/*
 * printError
 */
std::string XMLErrors::printError(const std::string &file, uint32 offset, const std::string &message) {
	ostringstream error;
	error << message;
	string buffer;
	uint32 line = 1;
	ifstream fbuf(file);
	if (fbuf.is_open()) {
		uint32 coffset = 0;
		while (fbuf.eof() == false) {
			getline(fbuf, buffer);
			if ((coffset + buffer.length() + 1) > offset) {
				break;
			} else {
				coffset += buffer.length() + 1;
				++line;
			}
		}
		buffer.erase(remove(buffer.begin(), buffer.end(), '\t'), buffer.end());
	}
	if (buffer.empty() == false) {
		error << " in file " << file << " at line " << line << " [..." << buffer << "]";
	}
	common::err << file << " " << error.str() << common::eol;
	return error.str();
}
