/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef XMLERRORS_H
#define XMLERRORS_H

#include <string>
#include <common/base/BaseTypes.h>

class XMLErrors {

public:

	XMLErrors() {

	}

	virtual ~XMLErrors() {

	}

	/*!
	 * \brief Print unknown tag error
	 * \param[in] file XML file name
	 * \param[in] offset Character offset in the file
	 * \param[in] tagname Unknown tag name string
	 * \return the string error
	 * \throws XmlConfigurationError Xml File Configuration Error
	 */
	virtual std::string unknownTagError(const std::string &file, uint32 offset, const std::string &tagname);

	/*!
	 * \brief Print missing attribute error
	 * \param[in] file XML file name
	 * \param[in] offset Character offset in the file
	 * \param[in] attributename Missing attribute name string
	 * \param[in] tagname tag name in which missing the attribute string
	 * \return the string error
	 * \throws XmlConfigurationError Xml File Configuration Error
	 */
	virtual std::string missingAttributeError(const std::string &file, uint32 offset, const std::string &attributename,
			const std::string &tagname);

	/*!
	 * \brief Print unknown tag error
	 * \param[in] file XML file name
	 * \param[in] offset Character offset in the file
	 * \param[in] tagname Unknown tag name string
	 * \return the string error
	 * \throws XmlConfigurationError Xml File Configuration Error
	 */
	virtual std::string unknownAttributeError(const std::string &file, uint32 offset, const std::string &attributename);

	/*!
	 * \brief Print Missing tag error
	 * \param[in] file XML file name
	 * \param[in] offset Character offset in the file
	 * \param[in] parentname Missing tag name parent name string
	 * \param[in] tagname Missing tag name string
	 * \return the string error
	 * \throws XmlConfigurationError Xml File Configuration Error
	 */
	virtual std::string missingTagNameError(const std::string &file, uint32 offset, const std::string &parentname,
			const std::string &tagname);

	/*!
	 * \brief Print error in terminal using red bold characters
	 * \param[in] file XML file name
	 * \param[in] offset Character offset in the file
	 * \param[in] message Message to print
	 * \return the string error
	 */
	virtual std::string printError(const std::string &file, uint32 offset, const std::string &message);

};

#endif // XMLERRORS_H
