/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cstdlib>

#include "LoggingServices/Level.h"

#include "AgentCollectorProxy.h"
#include "AgentXMLCollector.h"
#include "common/base/ServerProperties.h"

/**
 * Constructor
 */
AgentCollectorProxy::AgentCollectorProxy() {
	// Instanciate the real Agent Collector
	std::string filename;
	filename.append(common::ServerProperties::getInstance()->getConfigPath());
	filename.append("/XMLAgentTable.xml");
	m_realAgentCollector = new AgentXMLCollector(filename);
}

/**
 * Destructor
 */
AgentCollectorProxy::~AgentCollectorProxy() {
	// delete the real agent collector
	delete m_realAgentCollector;
	m_realAgentCollector = 0;
}

/**
 * loadTable
 */
void AgentCollectorProxy::loadTable(std::multimap<std::string, std::string>& compatibilityTable) {
	if (m_realAgentCollector != 0) {
		m_realAgentCollector->loadTable(compatibilityTable);
	}
}
