/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef AGENTPOLICY_H
#define AGENTPOLICY_H

#include <map>
#include <vector>
#include <string>

#include <boost/thread/mutex.hpp>

#include "InstrumentAbstraction/AbstractController.h"

//class AgentPolicyTest;
class AgentCollectorProxy;

class AgentPolicy {

public:
	/**
	 * Destructor
	 */
	~AgentPolicy();

	/**
	 * To get the unique instance of AgentPolicy class
	 */
	static AgentPolicy* getInstance();

	/**
	 * To warn the agentPolicy that an abstract controller has been activated
	 */
	void activateEC(AbstractController* anEC);

	/**
	 * To warn the agent policy that an abstract controller was deactivated
	 */
	void deactivateEC(AbstractController* anEC);

	/**
	 * To know if an abstact controller can be activated while
	 * other currently active abstact controllers
	 */
	bool canI(AbstractController* newEC);

	/**
	 * Check Agent table , verify name of table exist in the server
	 */
	void checkTable(const std::string& controllerList);

private:
	/**
	 * Constructor
	 */
	AgentPolicy();

	/**
	 *
	 */
	AgentCollectorProxy* m_agentCollector;
	std::vector<std::string> m_activeECs;
	std::multimap<std::string, std::string> m_compatibilityTable;

	/**
	 * The unique instance of this class
	 */
	static AgentPolicy* m_instance;

	/**
	 * Mutex used to protect class from concurent access
	 */
	std::recursive_mutex m_activeECsMutex;
	std::recursive_mutex m_compatibilityTableMutex;

};
#endif //AGENTPOLICY_H
