/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <iostream>
#include <vector>
#include "AgentXMLCollector.h"
#include <pugixml.hpp>
#include <common/base/BaseTypes.h>

using namespace std;

const std::string AgentXMLCollector::TABLE_TAG = "Table";
const std::string AgentXMLCollector::EC_TAG = "EC";
const std::string AgentXMLCollector::ECGROUP1_TAG = "ecGroup1";
const std::string AgentXMLCollector::ECGROUP2_TAG = "ecGroup2";
const std::string AgentXMLCollector::ECNAME_TAG = "ecName";
const std::string AgentXMLCollector::PARAl_TAG = "paral";

/*
 * Contructor
 */
AgentXMLCollector::AgentXMLCollector(const string& xmlFileName) :
	m_xmlFileName(xmlFileName) {
}

/*
 * Destructor
 */
AgentXMLCollector::~AgentXMLCollector() {
}

/*
 * loadTable
 */
void AgentXMLCollector::loadTable(
		std::multimap<std::string, std::string>& compatibilityTable) {

	pugi::xml_document pDoc;
	pugi::xml_parse_result result = pDoc.load_file(m_xmlFileName.c_str());
	if (result) {
		for (auto noderoot : pDoc.children()) {
			if (noderoot.name() == TABLE_TAG) {
				for (auto ec : noderoot.children()) {
					vector<string> group1;
					vector<string> group2;
					bool group1Par = true, group2Par = true;
					if (ec.name() == EC_TAG) {
						for (auto ecgroup : ec.children()) {
							if (ecgroup.name() == ECGROUP1_TAG) {
								pugi::xml_attribute attr = ecgroup.attribute(PARAl_TAG.c_str());
								string paral;
								if (attr) {
									paral = attr.value();
								}
								else {
									missingAttributeError(m_xmlFileName, (uint32) ecgroup.offset_debug(), PARAl_TAG, ECGROUP1_TAG);
								}
								group1Par = (paral == "true");
								for (auto ecname : ecgroup.children()) {
									if (ecname.name() == ECNAME_TAG) {
										group1.push_back(ecname.child_value());
									}
									else {
										unknownTagError(m_xmlFileName, (uint32) ecname.offset_debug(), ecname.name());
									}
								}
							}
							else if (ecgroup.name() == ECGROUP2_TAG) {
								pugi::xml_attribute attr = ecgroup.attribute(PARAl_TAG.c_str());
								string paral;
								if (attr) {
									paral = attr.value();
								}
								else {
									missingAttributeError(m_xmlFileName, (uint32) ecgroup.offset_debug(), PARAl_TAG, ECGROUP2_TAG);
								}
								group2Par = (paral == "true");
								for (auto ecname : ecgroup.children()) {
									if (ecname.name() == ECNAME_TAG) {
										group2.push_back(ecname.child_value());
									}
									else {
										unknownTagError(m_xmlFileName, (uint32) ecname.offset_debug(), ecname.name());
									}
								}
							}
							else {
								unknownTagError(m_xmlFileName, (uint32) ecgroup.offset_debug(), ecgroup.name());
							}
						}
					}
					else {
						unknownTagError(m_xmlFileName, (uint32) ec.offset_debug(), ec.name());
					}
					if (group1Par == false) {
						vector<string>::iterator iter1 = group1.begin();
						while (iter1 != group1.end()) {
							vector<string>::iterator iter2 = iter1;
							while (iter2 != group1.end()) {
								++iter2;
								if (iter2 != group1.end()) {
									compatibilityTable.insert(
											pair<string, string> (*iter1, *iter2));
								}
							}
							++iter1;
						}
					}
					if (group2Par == false) {
						vector<string>::iterator iter1 = group2.begin();
						while (iter1 != group2.end()) {
							vector<string>::iterator iter2 = iter1;
							while (iter2 != group2.end()) {
								++iter2;
								if (iter2 != group2.end()) {
									compatibilityTable.insert(
											pair<string, string> (*iter1, *iter2));
								}
							}
							++iter1;
						}
					}
					vector<string>::iterator group1Iter = group1.begin();
					while (group1Iter != group1.end()) {
						vector<string>::iterator group2Iter = group2.begin();
						while (group2Iter != group2.end()) {
							compatibilityTable.insert(pair<string, string> (
									*group1Iter, *group2Iter));
							++group2Iter;
						}
						++group1Iter;
					}
				}
			}
		}
	}
	else {
		XMLErrors::printError(m_xmlFileName, (uint32) result.offset, result.description());
	}
}
