/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COMMAND_H
#define COMMAND_H

#include <string>

class AbstractController;

/**
 * Interface class defining a command (Command pattern).
 */
class Command
{
public:
  virtual ~Command() {};
  // Body method
  virtual void execute() = 0;
  // Stop method. Used by the Scheduler when exiting.
  virtual void stop() = 0;
  // Return true if the command blocks the execution of aCommand
  virtual bool isBlocking(Command& aCommand) = 0;
  // Return true if the Command is rejected. Is called by the Scheduler if
  // the prior isBlocking has returned true.
  virtual bool reject(bool blocked) = 0;
  virtual AbstractController* getController() const = 0;

  virtual const std::string& getName() const = 0;
  virtual bool equal(Command* c1) = 0;

  virtual std::string toString() const = 0;
};

#endif
