/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Scheduler/Condition.h"
#include "Scheduler/Sender.h"

#include <boost/thread/mutex.hpp>
#include <iostream>

using namespace std;

Condition::Condition() :
	m_mutex(SenderShared::m_mutex.get()),
	m_condition(SenderShared::m_ready.get()),
	m_finished(false) {
}

void Condition::notify()
{
  //cout << pthread_self() << " - Condition notify " << m_mutex << endl;

  std::scoped_lock lock(*m_mutex);

  if (!isFinished())
  {
    m_condition->notify_one();
  }  
  
  //cout << pthread_self() << " - Condition notify " << m_mutex << " done" << endl;
}

void Condition::finalise()
{
}

void Condition::setFinished() 
{
  m_finished = true;
}

bool Condition::isFinished() const 
{
  return m_finished;
}

bool OrCondition::operator()()
{
  return (m_A() || m_B());
}

void OrCondition::finalise()
{
  m_A.finalise();
  m_B.finalise();
}

void OrCondition::setFinished() 
{
  Condition::setFinished();
  
  m_A.setFinished();
  m_B.setFinished();
}

bool AndCondition::operator()()
{
  return (m_A() && m_B());
}

void AndCondition::finalise()
{
  m_A.finalise();
  m_B.finalise();
}

void AndCondition::setFinished() 
{
  Condition::setFinished();
  
  m_A.setFinished();
  m_B.setFinished();
}

bool NotCondition::operator()()
{
  return (!m_A());
}

void NotCondition::finalise()
{
  m_A.finalise();
}

void NotCondition::setFinished() 
{
  Condition::setFinished();
  
  m_A.setFinished();
}

bool Eval::operator()()
{
  return (m_value);
}
