/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Scheduler/Terminated.h"
#include "DataProvider/Publisher.h"
#include "DataProvider/ChangeAspect.h"
#include "Scheduler/Sender.h"
#include "DeviceContainer/DeviceDriver.h"
#include <iostream>

using namespace std;

Terminated::Terminated(Publisher* aPublisher, int32 stop) :
	Condition(), Subscriber(),
	m_publisher(aPublisher),
	m_stop(stop),
	m_actionEnded(false),
	m_notified(false) {

	m_publisher->attach(this);
}

Terminated::Terminated(const Terminated& aTerminated) :
	m_publisher(aTerminated.m_publisher),
	m_stop(aTerminated.m_stop),
	m_actionEnded(aTerminated.m_actionEnded),
	m_notified(false) {
}

Terminated::~Terminated() {
}

bool Terminated::operator()() {
	return m_actionEnded;
}

void Terminated::finalise() {
	m_publisher->detach(this);
}

void Terminated::update(ChangeAspect* aChangeAspect) {

	std::scoped_lock lock(m_mutex);

	if (aChangeAspect->getType() == PROGRESS) {
		int32 progression = aChangeAspect->getLongParam();
		//cout << pthread_self() << " - updating Terminated " << static_cast<Subscriber*>(this) << " progression " << (int32 )progression << endl;

		if (progression >= m_stop) {
			// lock during the release. If the sleeper is being activated
			// at this point then we ensure that the m_actionEnded variable
			// is true before the sleepLock is locked.

			m_actionEnded = true;
		}
	}

	// variable m_notified used to be sure that only one notification is made if two updates arrive at the same time
	if (m_actionEnded && !m_notified) {
		//cout << pthread_self() << " - Terminated (" << static_cast<Subscriber*>(this) << ") notifies condition" << endl;
		//std::cout << aChangeAspect->getControllerName() << " Notify 3 - " << UniversalTime() << std::endl;
		Condition::notify();
		//cout << pthread_self() << " - Terminated (" << static_cast<Subscriber*>(this) << ") notified condition" << endl;
		m_notified = true;
	}
}
