/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "ServantConfiguration.h"
#include <ics/PersistenceServices/Configuration/XMLFileConfiguration2.h>

namespace ics {


ServantConfiguration::ServantConfiguration() {

}

ServantConfiguration::~ServantConfiguration() {
	delete _configuration2;
}

void ServantConfiguration::initInstance() {

	if (_instance == nullptr) {
		_instance = new ServantConfiguration();
		_instance->init();
	}
}

void ServantConfiguration::resetInstance() {

	if (_instance != nullptr) {
		delete _instance;
		_instance = nullptr;
	}
}

ServantConfiguration * ServantConfiguration::getInstance() {
	return _instance;
}

void ServantConfiguration::init() {

	_configuration2 = new XMLFileConfiguration2();
	_configuration2->loadTestControllers(_testControllerList);
	_configuration2->loadExperimentControllers(_experimentControllerList);
}

const std::vector<TestControllerConfig> & ServantConfiguration::getTestControllerList() const {
	return _testControllerList;
}

const std::vector<ExperimentControllerConfig> & ServantConfiguration::getExperimentControllerList() const {
	return _experimentControllerList;
}

void ServantConfiguration::saveTestControllerList(const std::vector<TestControllerConfigNode> & controllerList, bool isBackup) const {
	_configuration2->saveTestControllers(controllerList, isBackup);
}

void ServantConfiguration::saveExperimentControllerList(const std::vector<ExperimentControllerConfig> & controllerList, bool isBackup) const {
	_configuration2->saveExperimentControllers(controllerList, isBackup);
}

}
