/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef ICS_SERVANTCONFIGURATION_H_
#define ICS_SERVANTCONFIGURATION_H_

#include <string>
#include <vector>

#include <ics/PersistenceServices/Configuration/TestControllerConfig.h>
#include <ics/PersistenceServices/Configuration/ExperimentControllerConfig.h>

class XMLFileConfiguration2;

namespace ics {

/**
 * Base class for console.
 */
class ServantConfiguration {
	
public:
	/**
	 * Initialises the instance.
	 */
	static void initInstance();

	/**
	 * Resets the instance.
	 */
	static void resetInstance();

	/**
	 * Gets the instance. The instance must already be initialised by a call to initInstance().
	 */
	static ServantConfiguration * getInstance();

	/**
	 * Gets the test controller list.
	 */
	const std::vector<TestControllerConfig> & getTestControllerList() const;

	/**
	 * Gets the experiment controller list.
	 */
	const std::vector<ExperimentControllerConfig> & getExperimentControllerList() const;

	/**
	 * Saves the test controller list.
	 */
	void saveTestControllerList(const std::vector<TestControllerConfigNode> & controllerList, bool isBackup) const;

	/**
	 * Saves the experiment controller list.
	 */
	void saveExperimentControllerList(const std::vector<ExperimentControllerConfig> & controllerList, bool isBackup) const;

private:
	ServantConfiguration();
	~ServantConfiguration();

	void init();

	XMLFileConfiguration2* _configuration2 = nullptr;

	std::vector<TestControllerConfig> _testControllerList;
	std::vector<ExperimentControllerConfig> _experimentControllerList;

	static inline ServantConfiguration* _instance = nullptr;
};

}

#endif /*ICS_SERVANTCONFIGURATION_H_*/
