/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Utilities/Calculations.h"

#include <cmath>
#include <cstring>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <string>
#include <sstream>
#include <iostream>
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <boost/archive/iterators/base64_from_binary.hpp>
#include <boost/archive/iterators/binary_from_base64.hpp>
#include <boost/archive/iterators/transform_width.hpp>
#include <boost/archive/iterators/ostream_iterator.hpp>

int32 Calculations::round(float64 x) {
	float64 xabs = fabs(x);
	int32 sign = x<0?-1L:1L;
	int32 res;
	if(fmod(xabs,(float64)1.000000) >= 0.500000){
		res = sign*((int32 )floor(xabs) + 1);
	}else{
		res = sign*((int32 )floor(xabs));
	}
//	if (x < 0) {
//		if ((z - 0.5) < 0)
//			res = (int32 ) ceil(x);
//		else
//			res = (int32 ) (ceil(x) - 1);
//	}
//	else {
//		if ((z - 0.5) < 0)
//			res = (int32 ) floor(x);
//		else
//			res = (int32 ) (floor(x) + 1);
//	}
	return res;
}

uint32 Calculations::swapD32ByD8(uint32 val) {
#ifdef MACOSX
	return val;
#else
	uint32 i1, i2, i3, i4;
	i1 = val & 0x000000ff;
	i2 = (val >> 8) & 0x000000ff;
	i3 = (val >> 16) & 0x000000ff;
	i4 = (val >> 24) & 0x000000ff;

	return ((i1 << 24) + (i2 << 16) + (i3 << 8) + i4);
#endif
}

uint32 Calculations::swapD32ByD16(uint32 val) {
#ifdef MACOSX
	return val;
#else
	uint32 i1, i2;
	i1 = val & 0x0000ffff;
	i2 = val >> 16;
	return (i1 << 16) + i2;
#endif
}

int32 Calculations::swapD32ByD16(int32 val) {
#ifdef MACOSX
	return val;
#else
	int32 i1, i2;
	i1 = val << 16;
	i2 = val >> 16;
	i2 = i2 & 0x0000ffff;
	return (i1 | i2);
#endif
}

uint16 Calculations::swapD16ByD8(uint16 val) {
#ifdef MACOSX
	return val;
#else
	uint16 i1, i2;
	i1 = val & 0x00ff;
	i2 = val >> 8;
	return (i1 << 8) + i2;
#endif
}

int16 Calculations::swapD16ByD8(int16 val) {
#ifdef MACOSX
	return val;
#else
	int16 i1, i2;
	i1 = val << 8;
	i2 = val >> 8;
	i2 = i2 & 0x00ff;
	return (i1 | i2);
#endif
}

void Calculations::swapD16ByD8(char* val) {
#ifdef MACOSX
	return val;
#else
	char c1, c2;
	c1 = val[0];
	c2 = val[1];
	val[0] = c2;
	val[1] = c1;
#endif
}

//int32 Calculations::swapD32ByD16(int32  val) {
//#ifdef MACOSX
//	return val;
//#else
//	int32 i1, i2;
//	i1 = val << 16;
//	i2 = val >> 16;
//	i2 = i2 & 0x0000ffff;
//	return (i1 | i2);
//#endif
//}

float32 Calculations::swapD32ByD16(float32 in) {
#ifdef MACOSX
	return in;
#else
	float32 outbuf = in;
	uint16* tmp = (uint16*) &outbuf;
	uint16 buf;
	memcpy(&buf, &tmp[0], sizeof(uint16));
	memcpy(&tmp[0], &tmp[1], sizeof(uint16));
	memcpy(&tmp[1], &buf, sizeof(uint16));
	return (outbuf);
#endif
}

float64 Calculations::swapDouble64ByD32(float64 in) {
#ifdef MACOSX
	return in;
#else
	float64 outbuf = in;
	uint32* tmp = (uint32*) &outbuf;
	uint32 buf;
	memcpy(&buf, &tmp[0], sizeof(uint32));
	memcpy(&tmp[0], &tmp[1], sizeof(uint32));
	memcpy(&tmp[1], &buf, sizeof(uint32));
	return (outbuf);
#endif
}
/*
 * decToGray
 * Conversion Gray --> Binaire
 */
int32 Calculations::gray_dec(int32 gray)
{
	int32 i;
	int32 val,bit,res;
	uint32 mask;

	mask=0x80000000;							/* Masque un bit a 1 (MSB --> LSB)*/
	res=0;
	val=0;
	for(i=31;i+1;i--)							/* Codeur sur 32 bits*/
	{
		bit=(gray&mask)>>i;						/* extraction du bit de rang i*/
		res^=bit;							/* conversion: OUExclusif*/
		val+=res<<i;							/* ajout au rang i*/
		mask>>=1;							/* decalage du bit de masque*/
	}
	return (val);
}

/*------------------------------------------------------------------------------*/
/* Conversion code BCD --> Binaire						*/
int32 Calculations::bcd_bin(int32  bcd)
{
	int32 i;
	int32 bcd_digit, val;	 						/* 1 long=4 octets soit 8 fois 4 bits*/
	uint32 mask;							/* 4 bits codent un digit BCD*/

	val=0;
	mask=0xF0000000;							/* mask = %1111 0000 0000 0000 0000 0000 0000 0000*/
	for(i=8;i>0;i--)
	{
		bcd_digit=bcd&mask;
		bcd_digit>>=4*(i-1);						/* extraction de chaque digit BCD (poids fort->poids faible)*/
		if(bcd_digit>9)
		{								/* Erreur de codage: DIGIT>=10*/
			i=0;
			val=0x0;
		}
		else
		{								/* Bon codage*/
			val=val*10+bcd_digit;
			mask>>=4;						/* Decalage du masque au digit inferieur*/
		}
	}
	return (val);
}

/**
 * The code is taken from stackoverflow.
 * http://stackoverflow.com/questions/7053538/how-do-i-encode-a-string-to-base64-using-only-boost
 */
const std::string base64_padding[] = {"", "==","="};

std::string Calculations::encodeBase64(const std::string& data) {

	namespace bai = boost::archive::iterators;

	std::stringstream os;

	// Convert binary values to base64 characters
	typedef bai::base64_from_binary

	// Retrieve 6 bit integers from a sequence of 8 bit bytes
	<bai::transform_width<const char *, 6, 8> > base64_enc; // compose all the above operations in to a new iterator

	std::copy(base64_enc(data.c_str()), base64_enc(data.c_str() + data.size()), std::ostream_iterator<char>(os));

	os << base64_padding[data.size() % 3];

	return os.str();
}

std::string Calculations::decodeBase64(const std::string& data) {

	namespace bai = boost::archive::iterators;

	std::stringstream os;

	typedef bai::transform_width<bai::binary_from_base64<const char *>, 8, 6> base64_dec;

	unsigned int size = data.size();

	// Remove the padding characters, cf. https://svn.boost.org/trac/boost/ticket/5629
	if (size && data[size - 1] == '=') {
		--size;
		if (size && data[size - 1] == '=') --size;
	}
	if (size == 0) return std::string();

	std::copy(base64_dec(data.data()), base64_dec(data.data() + size), std::ostream_iterator<char>(os));

	return os.str();
}
