/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Utilities/Counter.h"

using namespace boost::posix_time;

/*
 * Counter
 */
Counter::Counter() :
	m_StartedTime(microsec_clock::local_time()) {
}

/*
 * getTime
 */
float64 Counter::getTime() const{
	time_period timediff(m_StartedTime, microsec_clock::local_time());
	return ((float64)timediff.length().total_seconds()
			+ (float64)timediff.length().fractional_seconds() / 1000000.);
}

/*
 * getTimeAndClear
 */
float64 Counter::getTimeAndClear() {
	float64 buf = getTime();
	clear();
	return buf;
}

/*
 * clear
 */
void Counter::clear() {
	m_StartedTime = microsec_clock::local_time();
}

/*
 * pause
 */
void Counter::pause() {
	 m_PauseTime = microsec_clock::local_time();
}

/*
 * resume
 */
void Counter::resume() {
	m_StartedTime += microsec_clock::local_time() - m_PauseTime;
}

/*
 * operator<<
 */
std::ostream& operator<<(std::ostream &os, const Counter& counter) {
	os << counter.getTime() << " s";
	return os;
}
