/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief This class propose a counter facilities
 * \author J. Locatelli
 * \date 01-10-2010
 */

#ifndef COUNTER_H
#define COUNTER_H

#include <boost/date_time/posix_time/posix_time.hpp>
#include <common/base/BaseTypes.h>

/*!
 * \class LangmuirDriver
 * \brief Main class for the Langmuir device driver
 *
 * \par
 * Class for Langmuir driver. Implements DeviceDriver methods.
 * \par
 * The Langmuir device is a windows computer which running the Nima software made with LabView.
 * It is connected to the nomad server using RS232 protocol.
 */
class Counter {

public:

	/*!
	 * \Constructor
	 */
	Counter();

	/*!
	 * \brief Getter of actual time of the counter is seconds
	 *
	 * \return actual time of the counter is seconds
	 */
	float64 getTime() const;

	/*!
	 * \brief Getter of actual time of the counter is seconds
	 *
	 * \return actual time of the counter is seconds
	 */
	float64 getTimeAndClear();

	/*!
	 * \brief Clear counter
	 */
	void clear();

	/*!
	 * \brief Pause counter
	 */
	void pause();

	/*!
	 * \brief Resume counter
	 */
	void resume();

private:

	boost::posix_time::ptime m_StartedTime; //! The started time
	boost::posix_time::ptime m_PauseTime; //! The paused time
};

std::ostream& operator<<(std::ostream &os, const Counter& counter);

#endif
