/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Utilities/SemaTools.h"
#include "Utilities/Simulated.h"
#include <sys/ipc.h>
#include <sys/sem.h>
#include <cstring>
#include <cerrno>
#include <iostream>
using namespace std;

const int32 SemaTools::CLE_FD = 61;

/*
 * Constructor
 */
SemaTools::SemaTools() {
	if ((Simulated::activated == false) && (Simulated::visa == false) && (Simulated::for3d == false)) {
		m_sem_id = SemMapE();
		if (m_sem_id == -1) {
			m_sem_id = SemCreateE();
		}
	}
}

/*
 * Desstructor
 */
SemaTools::~SemaTools() {
	if ((Simulated::activated == false) && (Simulated::visa == false) && (Simulated::for3d == false)) {
		SemDeleteE();
	}
}

/*
 * recuperation d'id d'un semaphore relie a cle
 */
int32 SemaTools::SemMapE() const {
	if ((Simulated::activated == false) && (Simulated::visa == false) && (Simulated::for3d == false)) {
		int32 sem_id; /* semaphore id */

		/* id du semaphore */
		if ((sem_id = semget(CLE_FD, 1, 0)) == -1) {
			cout << "PCI sem_tools function sem_map, IPC function semget : " << strerror(errno) << endl;
			return -1;
		}
		return sem_id;
	} else {
		return 0;
	}
}

/* creation d'un semaphore relie a cle de valeur initiale 1 */
int32 SemaTools::SemCreateE() const {
	if ((Simulated::activated == false) && (Simulated::visa == false) && (Simulated::for3d == false)) {
		int32 sem_id; /* semaphore id */
		union {
			int32 val;
			struct semid_ds *buf;
			ushort array[4];
		} arg;

		/* create semaphore */
		if (((sem_id = semget(CLE_FD, 1, (IPC_CREAT | IPC_EXCL | 0666))) == -1) && ((sem_id = semget(CLE_FD, 1, 0666)) == -1)) {
			cout << "PCI sem_tools function sem_create, IPC function semget : " << strerror(errno) << endl;
			return -1;
		}
		arg.val = 1;
		if (semctl(sem_id, 0, SETVAL, arg) == -1) {
			cout << "PCI sem_tools function sem_create, IPC function semctl : " << strerror(errno) << endl;
			return -1;
		}
		return sem_id;
	} else {
		return 0;
	}
}

/* suppression d'un semaphore relie a cle  */
void SemaTools::SemDeleteE() const {
	if ((Simulated::activated == false) && (Simulated::visa == false) && (Simulated::for3d == false)) {
		int32 sem_id; /* semaphore id */

		/* id du semaphore */
		if ((sem_id = semget(CLE_FD, 1, 0)) == -1) {
			cout << "PCI sem_tools function sem_delete, IPC function semget : " << strerror(errno) << endl;
		} else if (semctl(sem_id, 0, IPC_RMID, 0) == -1) {
			cout << "PCI sem_tools function sem_delete, IPC function semctl : " << strerror(errno) << endl;
		}
	}
}

void SemaTools::VE() {
	if ((Simulated::activated == false) && (Simulated::visa == false) && (Simulated::for3d == false)) {
		struct sembuf operation;
		operation.sem_num = 0;
		operation.sem_op = 1;
		operation.sem_flg = SEM_UNDO;

		if (semop(m_sem_id, &operation, 1) == -1) {
			cout << "PCI sem_tools function V, IPC function semop : " << strerror(errno) << endl;
			m_sem_id = SemCreateE();
		}
	}
}

void SemaTools::PE() {
	if ((Simulated::activated == false) && (Simulated::visa == false) && (Simulated::for3d == false)) {
		struct sembuf operation;
		operation.sem_num = 0;
		operation.sem_op = -1;
		operation.sem_flg = SEM_UNDO;
		if (semop(m_sem_id, &operation, 1) == -1) {
			cout << "PCI sem_tools function P, IPC function semop : " << strerror(errno) << endl;
			m_sem_id = SemCreateE();
		}
	}
}
