/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef STRINGUTILITIES_H
#define STRINGUTILITIES_H

#include <string>
#include <sstream>
#include <limits>
#include <common/base/BaseTypes.h>

class StringUtilities {

public:

	/*!
	 * \brief Method which compares two strings
	 *
	 * \param[in] s1 first string to compare
	 * \param[in] s2 second string to compare
	 * \param[in] nbChar fix minimum characters to compare
	 * \return if the two string are equal or not
	 */
	static bool cmp(const std::string& s1, const std::string& s2, uint32 nbChar = 0);

	/*!
	 * \brief concat a string with a number value separate by a point
	 *
	 * \param[in] name the string to concat
	 * \param[in] i The number to concat
	 * \return The concated string
	 */
	static std::string concat(const std::string& name, int32 i);

	/*!
	 * \brief Convert string to a <T> value type
	 *
	 * \param[in] t String to convert
	 * \return the converted value
	 */
	template<class out_type, class in_type> static out_type convert(in_type const& t) {
		std::stringstream stream;
		stream << t; // insert value to stream
		// store conversion’s result here
		out_type result;
		stream >> result; // write value to result
		// if there is a failure in conversion the stream will not be empty
		// this is checked by testing the eof bit
		if (!stream.eof()) // there can be overflow also
		{ // a max value in case of conversion error
			result = std::numeric_limits<out_type>::max();
		}
		return result;
	}

};

#endif
