/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "UnitTest.h"

using namespace std;

namespace utilities {

const float64 UnitTest::EPS = 1e-8;

/*
 * UnitTest
 */
UnitTest::UnitTest(ExperimentController* cont) : m_controller(cont) {
	string filename = getenv("HOME");
	filename += "/serverlog/unittest.log";
	m_file.open(filename.c_str(), ios::app);
}

/*
 * ~UnitTest
 */
UnitTest::~UnitTest() {
	if (m_file.is_open()) {
		m_file.close();
	}
}

/*
 * logFaillure
 */
void UnitTest::logFaillure(std::string caller, std::string file, int32 line, const std::string& message) {
	cout << m_controller->getName() << " : " << caller << "() : " << file << " : " << line <<  " : " << message << endl;
	m_file << m_controller->getName() << " : " << caller << "() : " << file << " : " << line <<  " : " << message << endl;
}

/*
 * checkTrueImpl
 */
bool UnitTest::checkTrueImpl(const std::string& caller,const std::string& file, int32 line, bool val) {
	if (val == false) {
		ostringstream buf;
		buf << "(" << val << ") is false";
		logFaillure(caller, file, line, buf.str());
	}
	return val;
}

/*
 * checkNotTrueImpl
 */
bool UnitTest::checkNotTrueImpl(std::string caller, std::string file, int32 line, bool val) {
	if (val == true) {
		ostringstream buf;
		buf << "(" << val << ") is true";
		logFaillure(caller, file, line, buf.str());
	}
	return !val;
}

/*
 * checkInt32EqualImpl
 */
bool UnitTest::checkInt32EqualImpl(std::string caller, std::string file, int32 line, int32 val1, int32 val2) {
	bool test = (val1 == val2);
	if (test == false) {
		ostringstream buf;
		buf << "(" << val1 << " == " << val2 << ") is false";
		logFaillure(caller, file, line, buf.str());
	}
	return test;
}

/*
 * checkUint32EqualImpl
 */
bool UnitTest::checkUint32EqualImpl(std::string caller, std::string file, int32 line, int32 val1, int32 val2) {
	bool test = (val1 == val2);
	if (test == false) {
		ostringstream buf;
		buf << "(" << val1 << " == " << val2 << ") is false";
		logFaillure(caller, file, line, buf.str());
	}
	return test;
}

/*
 * checkFloat64EqualImpl
 */
bool UnitTest::checkFloat64EqualImpl(std::string caller, std::string file, int32 line, float64 val1, float64 val2, float64 eps) {
	bool test = (fabs(val1 - val2) < eps);
	if (test == false) {
		ostringstream buf;
		buf << "(" << val1 << " == " << val2 << ") is false";
		logFaillure(caller, file, line, buf.str());
	}
	return test;
}

/*
 * checkInt32NotEqualImpl
 */
bool UnitTest::checkInt32NotEqualImpl(std::string caller, std::string file, int32 line, int32 val1, int32 val2) {
	bool test = (val1 != val2);
	if (test == false) {
		ostringstream buf;
		buf << "(" << val1 << " != " << val2 << ") is false";
		logFaillure(caller, file, line, buf.str());
	}
	return test;
}

/*
 * checkUint32NotEqualImpl
 */
bool UnitTest::checkUint32NotEqualImpl(std::string caller, std::string file, int32 line, int32 val1, int32 val2) {
	bool test = (val1 != val2);
	if (test == false) {
		ostringstream buf;
		buf << "(" << val1 << " != " << val2 << ") is false";
		logFaillure(caller, file, line, buf.str());
	}
	return test;
}

/*
 * checkFloat64NotEqualImpl
 */
bool UnitTest::checkFloat64NotEqualImpl(std::string caller, std::string file, int32 line, float64 val1, float64 val2, float64 eps) {
	bool test = (fabs(val1 - val2) > eps);
	if (test == false) {
		ostringstream buf;
		buf << "(" << val1 << " != " << val2 << ") is false";
		logFaillure(caller, file, line, buf.str());
	}
	return test;
}

/*
 * checkStringEqualImpl
 */
bool UnitTest::checkStringEqualImpl(std::string caller, std::string file, int32 line, const std::string& val1, const std::string& val2) {
	int32 test = val1.compare(val2);
	if (test != 0) {
		ostringstream buf;
		buf << "(" << val1 << " == " << val2 << ") is false";
		logFaillure(caller, file, line, buf.str());
	}
	return test;
}

/*
 * checkStringNotEqualImpl
 */
bool UnitTest::checkStringNotEqualImpl(std::string caller, std::string file, int32 line, const std::string& val1, const std::string& val2) {
	int32 test = val1.compare(val2);
	if (test == 0) {
		ostringstream buf;
		buf << "(" << val1 << " == " << val2 << ") is false";
		logFaillure(caller, file, line, buf.str());
	}
	return test;
}


}
