/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef WEBSPYDESCRIPTION_H
#define WEBSPYDESCRIPTION_H

#include <boost/serialization/vector.hpp>
#include <boost/serialization/map.hpp>
#include <boost/serialization/base_object.hpp>

#include <string>
#include <map>
#include <vector>

class WebSpyProperty {

public:

	WebSpyProperty() {

	}

	WebSpyProperty(const std::string& aPropertyName, const std::string& aPropertyValue) :
		m_PropertyName(aPropertyName), m_PropertyValue(aPropertyValue) {
	}

	WebSpyProperty(const WebSpyProperty& desc) {
		m_PropertyName = desc.m_PropertyName;
		m_PropertyValue = desc.m_PropertyValue;
	}

	const std::string& getPropertyName() const {
		return m_PropertyName;
	}

	const std::string& getPropertyValue() const {
		return m_PropertyValue;
	}

private:

	std::string m_PropertyName;
	std::string m_PropertyValue;

	friend class boost::serialization::access;

	// When the class Archive corresponds to an output archive, the
	// & operator is defined similar to <<.  Likewise, when the class Archive
	// is a type of input archive the & operator is defined similar to >>.
	template<class Archive>
	void serialize(Archive & ar, const uint32 version) {
		ar & m_PropertyName;
		ar & m_PropertyValue;
	}
};

class WebSpyName {

public:

	WebSpyName() {
	}

	void add(const std::string& name, const std::string& propertyName, const std::string& propertyValue) {
		if (m_Map.find(name) == m_Map.end()) {
			std::vector<WebSpyProperty> tmp;
			tmp.push_back(WebSpyProperty(propertyName, propertyValue));
			m_Map[name] = tmp;
		}
		else {
			std::vector<WebSpyProperty>& tmp = m_Map[name];
			tmp.push_back(WebSpyProperty(propertyName, propertyValue));
		}
	}

	WebSpyName(const WebSpyName& desc) {
		m_Map = desc.m_Map;
	}

	const std::map<std::string, std::vector<WebSpyProperty> >& getMap() const {
		return m_Map;
	}

private:

	std::map<std::string, std::vector<WebSpyProperty> > m_Map;

	friend class boost::serialization::access;

	// When the class Archive corresponds to an output archive, the
	// & operator is defined similar to <<.  Likewise, when the class Archive
	// is a type of input archive the & operator is defined similar to >>.
	template<class Archive>
	void serialize(Archive & ar, const uint32 version) {
		ar & m_Map;
	}
};

class WebSpyDescription {

public:

	WebSpyDescription() {

	}

	WebSpyDescription(const WebSpyDescription& desc) {
		m_Map = desc.m_Map;
	}

	void add(const std::string& type, const std::string& name, const std::string& propertyName,
			const std::string& propertyValue) {
		if (m_Map.find(type) == m_Map.end()) {
			WebSpyName tmp;
			tmp.add(name, propertyName, propertyValue);
			m_Map[type] = tmp;
		}
		else {
			WebSpyName &tmp = m_Map[type];
			tmp.add(name, propertyName, propertyValue);
		}
	}

	const std::map<std::string, WebSpyName>& getMap() const {
		return m_Map;
	}

private:

	std::map<std::string, WebSpyName> m_Map;

	friend class boost::serialization::access;

	// When the class Archive corresponds to an output archive, the
	// & operator is defined similar to <<.  Likewise, when the class Archive
	// is a type of input archive the & operator is defined similar to >>.
	template<class Archive>
	void serialize(Archive & ar, const uint32 version) {
		ar & m_Map;
	}
};

#endif //WEBSPYDESCRIPTION_H
