/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <common/base/Array.h>
#include <common/base/Convert.h>
#include <common/base/PropertyType.h>
#include <common/base/TypeToString.h>
#include <common/base/PortableByteArray.h>
#include <iostream>
#include <string>
#include <fstream>
#include <thread>
#include <memory>

std::thread t;
std::unique_ptr<int> u;

using namespace common;
using namespace std;

void writeFile(const std::string& encodedString, const std::string& type, int endianness) {

	ostringstream os;

	if (endianness == common::LITTLE_ENDIAN_ORDER) {
		os << "test-" << type << "-" << "little.bin";
	}
	else {
		os << "test-" << type << "-" << "big.bin";
	}

	ofstream testFile;
	testFile.open(os.str().c_str(), ios::binary);
	testFile << encodedString;
	testFile.close();
}

BOOST_AUTO_TEST_SUITE(suite)
;

BOOST_AUTO_TEST_CASE(testPrint1) {
	cout << "BaseTypeToString test" << endl;
	cout << BaseTypeToString::convert(PropertyType::INT8) << endl;
	cout << BaseTypeToString::convert(PropertyType::INT16) << endl;
	cout << BaseTypeToString::convert(PropertyType::INT32) << endl;
	cout << BaseTypeToString::convert(PropertyType::INT64) << endl;
	cout << BaseTypeToString::convert(PropertyType::FLOAT32) << endl;
	cout << BaseTypeToString::convert(PropertyType::FLOAT64) << endl;
	cout << BaseTypeToString::convert(PropertyType::BOOLEAN) << endl;
	cout << BaseTypeToString::convert(PropertyType::STRING) << endl;

	cout << "TypeToString test" << endl;
	cout << TypeToString<int8>::convert() << endl;
	cout << TypeToString<int16>::convert() << endl;
	cout << TypeToString<int32>::convert() << endl;
	cout << TypeToString<int64>::convert() << endl;
	cout << TypeToString<float32>::convert() << endl;
	cout << TypeToString<float64>::convert() << endl;
	cout << TypeToString<bool>::convert() << endl;
	cout << TypeToString<string>::convert() << endl;

	cout << TypeToString<Array<int8> >::convert() << endl;
	cout << TypeToString<Array<int16> >::convert() << endl;
	cout << TypeToString<Array<int32> >::convert() << endl;
	cout << TypeToString<Array<int64> >::convert() << endl;
	cout << TypeToString<Array<float32> >::convert() << endl;
	cout << TypeToString<Array<float64> >::convert() << endl;
	cout << TypeToString<Array<bool> >::convert() << endl;
	cout << TypeToString<Array<string> >::convert() << endl;

	cout << "Portable byte array" << endl;

	{
		int32 intArray[] = { 100, -21, 5665787, numeric_limits<int32>::min(), numeric_limits<int32>::max() };

		common::PortableByteArray<int32> portableByteArray;
		string encodedString;
		portableByteArray.encode(intArray, 5, encodedString, common::LITTLE_ENDIAN_ORDER);

		vector<int32> resultArray = portableByteArray.decode(encodedString);

		cout << "result size = " << resultArray.size() << endl;
		for (int i = 0; i < 5; ++i) {
			cout << "\t" << resultArray[i] << endl;
		}
		cout << endl;

		// Write the encoded string into a file.
		writeFile(encodedString, "int32", common::LITTLE_ENDIAN_ORDER);


		portableByteArray.encode(intArray, 5, encodedString, common::BIG_ENDIAN_ORDER);

		resultArray = portableByteArray.decode(encodedString);

		cout << "result size = " << resultArray.size() << endl;
		for (int i = 0; i < 5; ++i) {
			cout << "\t" << resultArray[i] << endl;
		}
		cout << endl;

		// Write the encoded string into a file.
		writeFile(encodedString, "int32", common::BIG_ENDIAN_ORDER);
	}

	{
		float64 floatArray[] = { 100.1, -21.2, 5665787.3, numeric_limits<float64>::min(), numeric_limits<float64>::max() };

		common::PortableByteArray<float64> portableByteArray;
		string encodedString;
		portableByteArray.encode(floatArray, 5, encodedString, common::LITTLE_ENDIAN_ORDER);

		vector<float64> resultArray = portableByteArray.decode(encodedString);

		cout << "result size = " << resultArray.size() << endl;
		for (int i = 0; i < 5; ++i) {
			cout << "\t" << resultArray[i] << endl;
		}
		cout << endl;

		// Write the encoded string into a file.
		writeFile(encodedString, "float64", common::LITTLE_ENDIAN_ORDER);


		portableByteArray.encode(floatArray, 5, encodedString, common::BIG_ENDIAN_ORDER);

		resultArray = portableByteArray.decode(encodedString);

		cout << "result size = " << resultArray.size() << endl;
		for (int i = 0; i < 5; ++i) {
			cout << "\t" << resultArray[i] << endl;
		}
		cout << endl;

		// Write the encoded string into a file.
		writeFile(encodedString, "float64", common::BIG_ENDIAN_ORDER);
	}
}

BOOST_AUTO_TEST_SUITE_END();
