/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <common/base/Exception.h>
#include <iostream>

using namespace common;
using namespace std;

BOOST_AUTO_TEST_SUITE(suite);

class AnException : public Exception {

public:
	AnException(string const & text)
	: Exception("AnException", text) {
	}
};

BOOST_AUTO_TEST_CASE(testPrint1)
{
	cout << "test common::Exception" << endl << endl;

	AnException e0("test0");
	AnException e1 = link(AnException("test1"), e0);

	cout << link(AnException("test1"), e0).printStack() << endl;
}

void foo() {

	throw AnException("exception!");
}

BOOST_AUTO_TEST_CASE(testThrow)
{
	cout << endl << "testThrow" << endl;

	try {
		for (int i = 0; i < 10; i++) {

			cout << "i = " << i << endl;

			foo();
		}

	} catch (Exception& e) {
		cout << e.printStack() << endl;
	}
}

BOOST_AUTO_TEST_CASE(testParallel)
{
	string result;
	string stack = "\tcontroller.test.com.Command.bar2() (70 : print(\"line b \", i);)\n\tcontroller.test.com.Command.bar1() (63 : bar2();)\n\tcontroller.test.com.Command.bar() (59 : bar1();)";

	int beginPosition = 0;
	int endPosition = 0;
	while (beginPosition < stack.size()) {
		endPosition = stack.find("\n", beginPosition);
		cout << "end =  " << endPosition << endl;
		if (endPosition != string::npos) {
			result += "\t" + stack.substr(beginPosition, endPosition - beginPosition + 1);
		} else {
			result += "\t" + stack.substr(beginPosition, stack.size() - beginPosition + 1);
			break;
		}
		cout << "continue" << endl;
		beginPosition = endPosition + 1;
	}

	cout << ">" << result << "<" << endl;
}

BOOST_AUTO_TEST_SUITE_END();