/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <common/connection/HttpConnection.h>
#include <common/connection/exp/HttpConnection.h>
#include <common/connection/exp/RestJsonHttpsConnection.h>

#include <common/connection/HttpsConnection.h>
#include <common/connection/RestJsonHttpConnection.h>
#include <common/connection/RestJsonHttpsConnection.h>
#include <iostream>

using namespace common;
using namespace std;

BOOST_AUTO_TEST_SUITE(suite);

BOOST_AUTO_TEST_CASE(test1)
{
	cout << "TEST 1" << endl;

	// Server node.js test-http-timeout should wait 5s before responding.
	try {
		HttpConnection con;
		con.openUrl("localhost", "3001");
		con.setMethod("GET");
		con.setPath("/");
		con.setProperty("Accept", "*/*");

		cout << con.send(true) << endl;

	} catch (const Exception& e) {
		cout << "error : " << e.getMessage() << endl;
	}
}

BOOST_AUTO_TEST_CASE(test1exp)
{
	cout << "TEST 1 exp" << endl;

	// Server node.js test-http-timeout should wait 5s before responding.
	try {
		exp::HttpConnection con;
		con.openUrl("localhost", "3001");
		con.setMethod("GET");
		con.setPath("/");
		con.setProperty("Accept", "*/*");

		cout << con.send(true) << endl;

	} catch (const Exception& e) {
		cout << "error : " << e.getMessage() << endl;
	}
}

BOOST_AUTO_TEST_CASE(test2)
{
	cout << "TEST 2" << endl;

	// Server node.js test-http-timeout should wait 5s before responding.
	try {
		HttpConnection con;
		con.openUrl("localhost", "3001");
		con.setMethod("GET");
		con.setTimeout(Duration(3000, Duration::MILLISECONDS));
		con.setPath("/");
		con.setProperty("Accept", "*/*");

		cout << con.send(true) << endl;

	} catch (const Exception& e) {
		cout << "error : " << e.getMessage() << endl;
	}
}

BOOST_AUTO_TEST_CASE(test2exp)
{
	cout << "TEST 2 exp" << endl;

	// Server node.js test-http-timeout should wait 5s before responding.
	try {
		exp::HttpConnection con;
		con.openUrl("localhost", "3001");
		con.setMethod("GET");
		con.setTimeout(Duration(3000, Duration::MILLISECONDS));
		con.setPath("/");
		con.setProperty("Accept", "*/*");

		cout << con.send(true) << endl;

	} catch (const Exception& e) {
		cout << "error : " << e.getMessage() << endl;
	}
}



BOOST_AUTO_TEST_CASE(test4)
{
	//curl  -XGET --noproxy "investigations.ill.fr" -H  "Content-Type: application/json" https://investigations.ill.fr/cycle

	cout << "TEST 4" << endl;

	try {
		RestJsonHttpsConnection con;
		con.setMethod("GET");
		con.setTimeout(Duration(1000, Duration::MILLISECONDS));
		con.openUrl("investigations.ill.fr");
		con.setPath("/cycle");

		boost::property_tree::ptree response = con.send(true);

	} catch (const Exception& e) {
		cout << "error : " << e.getMessage() << endl;
	}
}

BOOST_AUTO_TEST_CASE(test4exp)
{
	//curl  -XGET --noproxy "investigations.ill.fr" -H  "Content-Type: application/json" https://investigations.ill.fr/cycle

	cout << "TEST 4 exp" << endl;

	try {
		exp::HttpsConnection con;
		con.setTimeout(Duration(1000, Duration::MILLISECONDS));
		con.openUrl("investigations.ill.fr");
		con.setPath("/cycle");

		con.send(true);

	} catch (const Exception& e) {
		cout << "error : " << e.getMessage() << endl;
	}
}

BOOST_AUTO_TEST_CASE(test5)
{
	cout << "TEST 5" << endl;

	try {
		RestJsonHttpsConnection con;
		con.setMethod("GET");
		con.openUrl("scidata.ill.fr");
		con.setPath("/api/cycles/_current");

		con.send(true);

	} catch (const Exception& e) {
		cout << "error : " << e.getMessage() << endl;
	}
}

BOOST_AUTO_TEST_CASE(test5exp)
{
	cout << "TEST 5 exp" << endl;

	try {
		//exp::RestJsonHttpsConnection con;
		exp::HttpsConnection con;
		con.openUrl("scidata.ill.fr");
		con.setPath("/api/cycles/_current");

		con.send(true);

	} catch (const Exception& e) {
		cout << "error : " << e.getMessage() << endl;
	}
}

BOOST_AUTO_TEST_SUITE_END();
