/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <iostream>
#include <rapidjson/rapidjson.h>
#include <rapidjson/writer.h>
#include <rapidjson/stringbuffer.h>
#include <rapidjson/prettywriter.h>
#include <rapidjson/reader.h>
#include <rapidjson/document.h>
#include <iomanip>

using namespace std;
using namespace rapidjson;

BOOST_AUTO_TEST_SUITE(suite);

BOOST_AUTO_TEST_CASE(testJson)
{
	char c = '\"';
	cout << "\" = " << std::hex << (int)(unsigned char)c << std::dec << endl;

	string testString = "a string with special characters ≡ \" \\ \b \f \n \r \t / here";

	cout << "test string: " << testString << endl;

	for (int i = 0; i < testString.length(); ++i) {
		cout << "char '" << testString[i] << "'" << endl;
	}

	cout << "Testing Json" << endl;

	StringBuffer s;
	PrettyWriter<StringBuffer> writer(s);

	writer.StartObject();
	writer.Key("test");
	writer.String(testString.c_str());
	writer.EndObject();

	cout << "Encoded string is :" << s.GetString() << endl;


	Document document;
	document.Parse(s.GetString());

	if (document.HasParseError()) {
		cout << "Document error" << endl;
	}
	else {
		cout << "No error" << endl;
	}

	string errorString = "This is not a JSON string.";

	Document document2;
	document2.Parse(errorString.c_str());

	if (document2.HasParseError()) {
		cout << "Document error" << endl;
	}

}

BOOST_AUTO_TEST_SUITE_END();
