/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <common/utils/estimators/LinearEstimator.h>
#include <iostream>
#include <random>
#include <thread>
#include <chrono>

void test(std::mt19937& gen, std::uniform_real_distribution<>& dis, common::LinearEstimator& estimator, int N, int S) {

	std::cout << "*************************************" << std::endl;

	for (int n = 0; n < N; n++) {

		double duration = dis(gen);
		int interval = 1000 * duration / S;

		std::cout << "***************" << std::endl;
		std::cout << "duration " << duration << std::endl;

		double end = 15.0;

		estimator.start(0, end);

		std::cout << "estimated time " << estimator.getCurrentEstimatedTime() << std::endl;

		for (int i = 0; i < S; i++) {

			double progression = (i + 1) / static_cast<double>(S);

			std::this_thread::sleep_for(std::chrono::milliseconds(interval));

			std::cout << "progression " << progression << std::endl;
			std::cout << "elapsed time " << estimator.getElapsedTime() << std::endl;
			std::cout << "remaining time with progression " << estimator.getCurrentEstimatedRemainingTimeWithProgression(progression) << std::endl;
			std::cout << "remaining time without progression " << estimator.getCurrentEstimatedRemainingTimeWithoutProgression() << std::endl;
			std::cout << "remaining time " << estimator.getCurrentEstimatedRemainingTime(progression) << std::endl;
			std::cout << std::endl;
		}

		estimator.terminate();
		std::cout << "estimated factor " << estimator.getEstimatedTimeFactor() << std::endl;
		std::cout << "estimated time " << estimator.getEstimatedTime(0, end) << std::endl;

		std::cout << "***************" << std::endl;
	}
}

BOOST_AUTO_TEST_SUITE(suite);

BOOST_AUTO_TEST_CASE(testLinearEstimator)
{
	std::random_device rd;
	std::mt19937 gen(rd());
	std::uniform_real_distribution<> dis(1.5, 2.5);

	common::LinearEstimator estimator;

	int N = 3;
	int S = 4;

	std::cout << "*************************************" << std::endl;

	test(gen, dis, estimator, N, S);

	estimator.reset();

	test(gen, dis, estimator, N, S);

}

BOOST_AUTO_TEST_SUITE_END();
