/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <iostream>
#include <core/base/BooleanExpression.h>
#include <muParser.h>

using namespace std;

double * addVariable(const char *variableName, void *pUserData) {

	map<string, double> * variables = (map<string, double> *)pUserData;

	string name(variableName);
	(*variables)[name] = 10.0;

	cout << "added new variable " << name << endl;

	return &(*variables)[name];
}

double notOperator(double a) {
	if (a > 0.0) {
		return 0.0;
	}
	return 1.0;
}

double multiplyOperator(double a, double b) {
	return a * b;
}

double addOperator(double a, double b) {
	return a + b;
}

double divideOperator(double a, double b) {
	return a / b;
}

double lessOperator(double a, double b) {
	return a - b;
}

double lesserOperator(double a, double b) {
	return a < b;
}

double greaterOperator(double a, double b) {
	return a > b;
}

double equalsOperator(double a, double b) {
	return a == b;
}

double powerOperator(double a, double b) {
	return std::pow(a, b);
}

BOOST_AUTO_TEST_SUITE(suite);

BOOST_AUTO_TEST_CASE(testMuParser) {

	mu::Parser* parser = new mu::Parser();
	parser->DefineNameChars("0123456789_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ$.");
	parser->DefineOprtChars("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-*^/?<>=#!%&|~'_");
	parser->DefineInfixOprtChars("not+-*^/?<>=#!%&|~'_");

	parser->EnableBuiltInOprt(false);

	map<string, double> variables;

	parser->SetVarFactory(addVariable, &variables);

	parser->DefineInfixOprt("not", notOperator);
	parser->DefineOprt("and", multiplyOperator);
	parser->DefineOprt("or", addOperator);
	parser->DefineOprt("<", lesserOperator);
	parser->DefineOprt(">", greaterOperator);
	parser->DefineOprt("==", equalsOperator);
	parser->DefineOprt("*", multiplyOperator);
	parser->DefineOprt("/", divideOperator);
	parser->DefineOprt("+", addOperator);
	parser->DefineOprt("-", lessOperator);
	parser->DefineOprt("^", powerOperator);

	string exp = "10 or 5";
	//string expr2 = "(10 and 2)";
	parser->SetExpr(exp);
	double r = parser->Eval();

	cout << exp << " is " << r << endl;

	exp = "(10 < 5) or (2 > 5)";
	try {
		parser->SetExpr(exp);
		r = parser->Eval();
	}
	catch(mu::Parser::exception_type &e)
	{
	  cout << "Message:  " << e.GetMsg() << "\n";
	  cout << "Formula:  " << e.GetExpr() << "\n";
	  cout << "Token:    " << e.GetToken() << "\n";
	  cout << "Position: " << e.GetPos() << "\n";
	  cout << "Errc:     " << e.GetCode() << "\n";
	}

	cout << exp << " is " << r << endl;

	exp = "not (-10 == 5)";
	//string expr2 = "(10 and 2)";
	parser->SetExpr(exp);
	r = parser->Eval();

	cout << exp << " is " << r << endl;

	exp = "-x + y * 100 - z < 10";

	parser->SetExpr(exp);
	r = parser->Eval();

	cout << exp << " is " << r << endl;

	exp = "-A1.position + A2.position * 100 - z < 10";

	parser->SetExpr(exp);
	r = parser->Eval();

	cout << exp << " is " << r << endl;

	exp = "-A1.actual_position + A2.actual_position * 100 - z < 10";

	parser->SetExpr(exp);
	r = parser->Eval();

	cout << exp << " is " << r << endl;

	exp = "1 or 1 and 0";

	parser->SetExpr(exp);
	r = parser->Eval();

	cout << exp << " is " << r << endl;

	bool ce = (true || true && false);

	cout << "ce is " << ce << endl;

	exp = "2 ^ 3";

	parser->SetExpr(exp);
	r = parser->Eval();

	cout << exp << " is " << r << endl;

	exp = "-A1.actual_position + A2.actual_position * 100 - z < 10";

	mu::Parser* _parser(new mu::Parser());

	// initialise the parser
	_parser->DefineNameChars("0123456789_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ$.");
	_parser->DefineOprtChars("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-*^/?<>=#!%&|~'_");
	_parser->DefineInfixOprtChars("not+-*^/?<>=#!%&|~'_");
	_parser->EnableBuiltInOprt(false);
/*
	_parser->DefineInfixOprt("not", notOperator);
	_parser->DefineOprt("or", addOperator, 0);
	_parser->DefineOprt("and", multiplyOperator, 1);
	_parser->DefineOprt("<", lesserOperator, 2);
	_parser->DefineOprt("<=", lesserOrEqualsOperator, 2);
	_parser->DefineOprt(">", greaterOperator, 2);
	_parser->DefineOprt(">=", greaterOrEqualsOperator, 2);
	_parser->DefineOprt("==", equalsOperator, 2);
	_parser->DefineOprt("+", addOperator, 3);
	_parser->DefineOprt("-", lessOperator, 3);
	_parser->DefineOprt("/", divideOperator, 4);
	_parser->DefineOprt("*", multiplyOperator, 4);*/

	_parser->SetVarFactory(addVariable, &variables);

	_parser->DefineInfixOprt("not", notOperator);
	_parser->DefineOprt("and", multiplyOperator);
	_parser->DefineOprt("or", addOperator);
	_parser->DefineOprt("<", lesserOperator);
	_parser->DefineOprt(">", greaterOperator);
	_parser->DefineOprt("==", equalsOperator);
	_parser->DefineOprt("*", multiplyOperator);
	_parser->DefineOprt("/", divideOperator);
	_parser->DefineOprt("+", addOperator);
	_parser->DefineOprt("-", lessOperator);

	_parser->SetExpr(exp);

	_parser->Eval();
}

BOOST_AUTO_TEST_SUITE_END();
