/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <iostream>
#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/stats.hpp>
#include <boost/accumulators/statistics/mean.hpp>
#include <boost/accumulators/statistics/moment.hpp>
#include <boost/accumulators/statistics/variance.hpp>

namespace ba = boost::accumulators;

// mean
// u_n = u_n-1 + (x_n - u_n-1) / n
//
// sigma²
// s_n² = s_n-1² + ((x_n - u_n-1)² - s_n-1²) / n

struct Stats {

	void add(double value) {
		size++;
		sigma2 = sigma2 + (std::pow(value - mean, 2) - sigma2) / size;
		mean += (value - mean) / size;
	}

	int size = 0;
	double mean = 0;
	double sigma2 = 0;
};



BOOST_AUTO_TEST_SUITE(suite);

BOOST_AUTO_TEST_CASE(testStats)
{
    // Define an accumulator set for calculating the mean and variance
	ba::accumulator_set<double, ba::stats<ba::tag::mean, ba::tag::moment<2>, ba::tag::variance>> acc;

    Stats stats;

    std::vector<double> values = {7,5,4,9,12,45};

    for (double value : values) {
		acc(value);
		stats.add(value);
    }

    double s1 = std::sqrt(ba::variance(acc));
    double s2 = std::sqrt(ba::variance(acc) * values.size() / (values.size() - 1)); // The same value as Python statistics.stdev()
    double s2c = std::sqrt(ba::variance(acc) * ba::count(acc)/ (ba::count(acc) - 1)); // The same value as Python statistics.stdev()
    double s3 = std::sqrt(stats.sigma2);

    // Display the results
	std::cout << "Count:" << ba::count(acc) << std::endl;
    std::cout << "Mean:   " << ba::mean(acc) << "    " << stats.mean << std::endl;
    std::cout << "Variance: " << s1 << "    " << s2 << "    " << s2c << "    " << s3 << std::endl;

}

BOOST_AUTO_TEST_SUITE_END();
